Package: brainGraph
Type: Package
Version: 3.1.2
Date: 2026-04-29
Title: Graph Theory Analysis of Brain MRI Data
Authors@R: person(given="Christopher G.", family="Watson",
                  email="cgwatson@bu.edu", role=c("aut", "cre"),
                  comment=c(ORCID="0000-0002-7082-7631"))
Description: A set of tools for performing graph theory analysis of brain MRI
    data. It works with data from a Freesurfer analysis (cortical thickness,
    volumes, local gyrification index, surface area), diffusion tensor
    tractography data (e.g., from FSL) and resting-state fMRI data (e.g., from
    DPABI). It contains a graphical user interface for graph visualization and
    data exploration, along with several functions for generating useful
    figures.
URL: https://github.com/cwatson/brainGraph
BugReports: https://groups.google.com/forum/?hl=en#!forum/brainGraph-help
LazyData: true
Depends: R (>= 3.5.0), igraph (>= 1.2.4),
Imports: abind, data.table (>= 1.12.4), doParallel, foreach, grid,
        lattice, MASS, Matrix, methods, permute, parallel
Suggests: Hmisc, ade4, boot, car, expm, ggplot2, ggrepel, gridExtra,
        mediation, oro.nifti, scales
License: GPL-3
RoxygenNote: 7.3.3
Collate: 'glm_stats.R' 'brainGraph_GLM.R' 'glm_methods.R' 'NBS.R'
        'analysis_random_graphs.R' 'atlas.R' 'auc.R' 'boot_global.R'
        'brainGraph_mediate.R' 'centr_lev.R' 'communicability.R'
        'contract_brainGraph.R' 'corr_matrix.R' 'count_edges.R'
        'create_graphs.R' 'create_mats.R' 'data.R' 'data_tables.R'
        'distances.R' 'edge_asymmetry.R' 'get_resid.R' 'glm_design.R'
        'glm_fit.R' 'glm_randomise.R' 'graph_efficiency.R' 'hubs.R'
        'import.R' 'individ_contrib.R' 'list.R' 'method_helpers.R'
        'mtpc.R' 'methods.R' 'permute_group.R' 'plot_brainGraph.R'
        'plot_brainGraph_multi.R' 'plot_global.R' 'plot_group_means.R'
        'plot_rich_norm.R' 'plot_vertex_measures.R' 'random_graphs.R'
        'rich_club.R' 'robustness.R' 's_core.R'
        'set_brainGraph_attributes.R' 'small_world.R' 'spatial_dist.R'
        'utils.R' 'utils_matrix.R' 'vertex_roles.R' 'vulnerability.R'
        'write_brainnet.R' 'zzz.R'
NeedsCompilation: no
Packaged: 2026-04-30 06:34:42 UTC; cwatson
Author: Christopher G. Watson [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-7082-7631>)
Maintainer: Christopher G. Watson <cgwatson@bu.edu>
Repository: CRAN
Date/Publication: 2026-04-30 08:00:03 UTC
Built: R 4.6.0; ; 2026-05-01 21:06:12 UTC; unix

Package: brainGraph
Type: Package
Version: 3.1.2
Date: 2026-04-29
Title: Graph Theory Analysis of Brain MRI Data
Authors@R: person(given="Christopher G.", family="Watson",
                  email="cgwatson@bu.edu", role=c("aut", "cre"),
                  comment=c(ORCID="0000-0002-7082-7631"))
Description: A set of tools for performing graph theory analysis of brain MRI
    data. It works with data from a Freesurfer analysis (cortical thickness,
    volumes, local gyrification index, surface area), diffusion tensor
    tractography data (e.g., from FSL) and resting-state fMRI data (e.g., from
    DPABI). It contains a graphical user interface for graph visualization and
    data exploration, along with several functions for generating useful
    figures.
URL: https://github.com/cwatson/brainGraph
BugReports: https://groups.google.com/forum/?hl=en#!forum/brainGraph-help
LazyData: true
Depends: R (>= 3.5.0), igraph (>= 1.2.4),
Imports: abind, data.table (>= 1.12.4), doParallel, foreach, grid,
        lattice, MASS, Matrix, methods, permute, parallel
Suggests: Hmisc, ade4, boot, car, expm, ggplot2, ggrepel, gridExtra,
        mediation, oro.nifti, scales
License: GPL-3
RoxygenNote: 7.3.3
Collate: 'glm_stats.R' 'brainGraph_GLM.R' 'glm_methods.R' 'NBS.R'
        'analysis_random_graphs.R' 'atlas.R' 'auc.R' 'boot_global.R'
        'brainGraph_mediate.R' 'centr_lev.R' 'communicability.R'
        'contract_brainGraph.R' 'corr_matrix.R' 'count_edges.R'
        'create_graphs.R' 'create_mats.R' 'data.R' 'data_tables.R'
        'distances.R' 'edge_asymmetry.R' 'get_resid.R' 'glm_design.R'
        'glm_fit.R' 'glm_randomise.R' 'graph_efficiency.R' 'hubs.R'
        'import.R' 'individ_contrib.R' 'list.R' 'method_helpers.R'
        'mtpc.R' 'methods.R' 'permute_group.R' 'plot_brainGraph.R'
        'plot_brainGraph_multi.R' 'plot_global.R' 'plot_group_means.R'
        'plot_rich_norm.R' 'plot_vertex_measures.R' 'random_graphs.R'
        'rich_club.R' 'robustness.R' 's_core.R'
        'set_brainGraph_attributes.R' 'small_world.R' 'spatial_dist.R'
        'utils.R' 'utils_matrix.R' 'vertex_roles.R' 'vulnerability.R'
        'write_brainnet.R' 'zzz.R'
NeedsCompilation: no
Packaged: 2026-04-30 06:34:42 UTC; cwatson
Author: Christopher G. Watson [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-7082-7631>)
Maintainer: Christopher G. Watson <cgwatson@bu.edu>
Repository: CRAN
Date/Publication: 2026-04-30 08:00:03 UTC
Built: R 4.6.0; ; 2026-04-30 11:23:20 UTC; unix

S3method("[",bg_GLM)
S3method("[",brainGraphList)
S3method("[",brainGraph_resids)
S3method("[",corr_mats)
S3method(anova,bg_GLM)
S3method(case.names,NBS)
S3method(case.names,bg_GLM)
S3method(case.names,brainGraph_resids)
S3method(case.names,mtpc)
S3method(coef,bg_GLM)
S3method(coeff_var,array)
S3method(coeff_var,default)
S3method(coeff_var,matrix)
S3method(confint,bg_GLM)
S3method(cooks.distance,bg_GLM)
S3method(deviance,bg_GLM)
S3method(df.residual,NBS)
S3method(df.residual,bg_GLM)
S3method(df.residual,mtpc)
S3method(dfbeta,bg_GLM)
S3method(dfbetas,bg_GLM)
S3method(extractAIC,bg_GLM)
S3method(fitted,bg_GLM)
S3method(formula,NBS)
S3method(formula,bg_GLM)
S3method(formula,mtpc)
S3method(groups,brainGraphList)
S3method(groups,brainGraph_resids)
S3method(groups,corr_mats)
S3method(hatvalues,bg_GLM)
S3method(influence,bg_GLM)
S3method(inv,array)
S3method(inv,list)
S3method(inv,matrix)
S3method(inv,qr)
S3method(labels,NBS)
S3method(labels,bg_GLM)
S3method(labels,mtpc)
S3method(logLik,bg_GLM)
S3method(make_brainGraph,bg_mediate)
S3method(make_brainGraph,igraph)
S3method(make_brainGraph,matrix)
S3method(make_brainGraphList,NBS)
S3method(make_brainGraphList,array)
S3method(make_brainGraphList,bg_GLM)
S3method(make_brainGraphList,corr_mats)
S3method(make_brainGraphList,mtpc)
S3method(nobs,NBS)
S3method(nobs,bg_GLM)
S3method(nobs,brainGraphList)
S3method(nobs,brainGraph_resids)
S3method(nobs,mtpc)
S3method(nregions,NBS)
S3method(nregions,bg_GLM)
S3method(nregions,brainGraph_resids)
S3method(nregions,corr_mats)
S3method(nregions,mtpc)
S3method(plot,IC)
S3method(plot,bg_GLM)
S3method(plot,brainGraph)
S3method(plot,brainGraphList)
S3method(plot,brainGraph_GLM)
S3method(plot,brainGraph_NBS)
S3method(plot,brainGraph_boot)
S3method(plot,brainGraph_mediate)
S3method(plot,brainGraph_mtpc)
S3method(plot,brainGraph_permute)
S3method(plot,brainGraph_resids)
S3method(plot,corr_mats)
S3method(plot,mtpc)
S3method(print,bg_GLM)
S3method(print,brainGraphList)
S3method(print,infl.bg_GLM)
S3method(print,summary.IC)
S3method(print,summary.NBS)
S3method(print,summary.bg_GLM)
S3method(print,summary.bg_mediate)
S3method(print,summary.brainGraph)
S3method(print,summary.brainGraph_boot)
S3method(print,summary.brainGraph_permute)
S3method(print,summary.brainGraph_resids)
S3method(print,summary.mtpc)
S3method(qr,array)
S3method(region.names,bg_GLM)
S3method(region.names,brainGraph_resids)
S3method(region.names,corr_mats)
S3method(region.names,data.table)
S3method(region.names,mtpc)
S3method(residuals,bg_GLM)
S3method(rstandard,bg_GLM)
S3method(rstudent,bg_GLM)
S3method(sigma,bg_GLM)
S3method(summary,IC)
S3method(summary,NBS)
S3method(summary,bg_GLM)
S3method(summary,bg_mediate)
S3method(summary,brainGraph)
S3method(summary,brainGraph_boot)
S3method(summary,brainGraph_permute)
S3method(summary,brainGraph_resids)
S3method(summary,mtpc)
S3method(symmetrize,array)
S3method(symmetrize,matrix)
S3method(terms,NBS)
S3method(terms,bg_GLM)
S3method(terms,mtpc)
S3method(variable.names,NBS)
S3method(variable.names,bg_GLM)
S3method(variable.names,mtpc)
S3method(vcov,bg_GLM)
export(NBS)
export(analysis_random_graphs)
export(aop)
export(apply_thresholds)
export(as_atlas)
export(as_brainGraphList)
export(bg_to_mediate)
export(brainGraph_GLM)
export(brainGraph_GLM_design)
export(brainGraph_boot)
export(brainGraph_mediate)
export(brainGraph_permute)
export(centr_betw_comm)
export(centr_lev)
export(check_sID)
export(coeff_determ)
export(coeff_table)
export(coeff_var)
export(colMax)
export(colMaxAbs)
export(colMin)
export(communicability)
export(contract_brainGraph)
export(cor.diff.test)
export(corr.matrix)
export(count_homologous)
export(count_inter)
export(covratio.bg_GLM)
export(create_atlas)
export(create_mats)
export(dffits_bg_GLM)
export(diag_sq)
export(edge_asymmetry)
export(edge_spatial_dist)
export(efficiency)
export(fastLmBG)
export(fastLmBG_3d)
export(fastLmBG_3dY)
export(fastLmBG_3dY_1p)
export(fastLmBG_f)
export(fastLmBG_t)
export(gateway_coeff)
export(get.resid)
export(get_thresholds)
export(graph_attr_dt)
export(guess_atlas)
export(hubness)
export(import_scn)
export(inv)
export(is.brainGraph)
export(is.brainGraphList)
export(is_binary)
export(loo)
export(make_auc_brainGraph)
export(make_brainGraph)
export(make_brainGraphList)
export(make_ego_brainGraph)
export(make_empty_brainGraph)
export(make_intersection_brainGraph)
export(mean_distance_wt)
export(mtpc)
export(nregions)
export(pad_zeros)
export(part_coeff)
export(partition)
export(pinv)
export(plot_brainGraph_multi)
export(plot_global)
export(plot_rich_norm)
export(plot_vertex_measures)
export(plot_volumetric)
export(qr_Q2)
export(qr_R2)
export(randomise)
export(randomise_3d)
export(region.names)
export(rich_club_all)
export(rich_club_attrs)
export(rich_club_coeff)
export(rich_club_norm)
export(rich_core)
export(robustness)
export(s_core)
export(set_brainGraph_attr)
export(sim.rand.graph.clust)
export(sim.rand.graph.hqs)
export(sim.rand.graph.par)
export(slicer)
export(small.world)
export(symm_mean)
export(symmetrize)
export(vertex_attr_dt)
export(vertex_spatial_dist)
export(vif.bg_GLM)
export(vulnerability)
export(within_module_deg_z_score)
export(write_brainnet)
export(xfm.weights)

importFrom(MASS,Null,fractions,ginv)
importFrom(Matrix,colSums,rowSums)
importFrom(abind,abind,adrop)
importFrom(doParallel,registerDoParallel)
importFrom(grid,gpar,grid.draw,grid.newpage,textGrob)
importFrom(methods,hasArg)
importFrom(parallel, clusterEvalQ, makeCluster)
importFrom(permute,shuffleSet)
import(igraph)
import(data.table)
import(foreach)
import(lattice)
importFrom(graphics, abline, image, layout, legend, lines, mtext, panel.smooth, par, plot,
  points, polygon, strheight, strwidth, text, title)
importFrom(grDevices, adjustcolor, colorRampPalette, dev.copy, dev.cur,
  dev.off, dev.set, extendrange, nclass.FD, png, pdf, gray)
importFrom(stats, anova, as.dist, as.formula, case.names, coef, complete.cases, confint,
  cooks.distance, cov.wt, cov2cor, covratio, density, deviance, df.residual, dfbeta, dfbetas, dffits,
  extractAIC, fitted, formula, hatvalues, influence, lm, logLik, median, nobs, p.adjust, pf,
  pnorm, ppoints, printCoefmat, pt, qf, qnorm, qqline, qqnorm, qt, quantile, residuals, rnorm, rstandard,
  rstudent, runif, sd, setNames, sigma, t.test, terms, var, variable.names, vcov)
importFrom(utils, combn, data, file_test, flush.console, getTxtProgressBar, hasName, head, packageVersion,
  setTxtProgressBar, tail, txtProgressBar)

#' GLM non-parametric permutation testing
#'
#' \code{randomise} and \code{randomise_3d} perform non-parametric permutation
#' testing for analyses in which there is a single or multiple design matrix per
#' region, respectively. In the latter case, \code{X} should be a 3D array.
#'
#' @name randomise
#' @rdname randomise
NULL

#' Partition a design matrix into columns of interest and nuisance
#'
#' \code{partition} partitions a full design matrix into separate matrices of
#' covariates of interest and nuisance covariates based on a given contrast and
#' partition method.
#'
#' @section Model partitioning:
#' Consider the matrix formulation of the \emph{general linear model}:
#' \deqn{\mathbf{Y} = \mathbf{M} \psi + \in}
#' where \eqn{Y} is the vector of outcomes, \eqn{M} is the full design matrix
#' (including nuisance covariates), \eqn{\psi} is the vector of parameter
#' estimates, and \eqn{\in} is the vector of error terms. In a permutation
#' framework, algorithms are applied differently depending on the
#' presence/absence of nuisance covariates; thus the model is separated
#' depending on the contrast of interest:
#' \deqn{\mathbf{Y} = \mathbf{X}\beta + \mathbf{Z}\gamma + \in}
#' where \eqn{\mathbf{X}} contains covariates of interest, \eqn{\mathbf{Z}}
#' contains nuisance covariates, and \eqn{\beta} and \eqn{\gamma} are the
#' associated parameter estimates.
#'
#' The manner of partitioning depends on the method. For example, for the
#' \code{guttman} method, \code{X} is formed from the columns of \code{contrast}
#' that have non-zero entries.
#'
#' @param M Numeric matrix or array of the full design matrix(es)
#' @param contrast For \code{partition}, a numeric matrix with 1 or more rows
#'   (for T and F contrasts, respectively) representing a \emph{single
#'   contrast}.
#' @inheritParams GLM
#' @importFrom MASS Null ginv
#' @importFrom abind abind adrop
#' @export
#' @rdname randomise
#'
#' @return \code{partition} returns a list containing:
#'   \item{Mp}{Numeric array; the combined partitioned arrays}
#'   \item{X}{Numeric array for the covariates of interest}
#'   \item{Z}{Numeric array for the nuisance covariates}
#'   \item{eCm}{The \emph{effective contrast}, equivalent to the original, for
#'     the partitioned model \code{[X, Z]} and considering all covariates}
#'   \item{eCx}{Same as \code{eCm}, but considering only \code{X}}
#' @references Beckmann, C.F. and Jenkinson, M. and Smith, S.M. (2001) General
#'   multi-level linear modelling for group analysis in FMRI. Tech Rep.
#'   University of Oxford, Oxford.
#' @references Guttman, I. (1982) \emph{Linear Models: An Introduction}. Wiley,
#'   New York.
#' @references Ridgway, G.R. (2009) Statistical analysis for longitudinal MR
#'   imaging of dementia. PhD thesis.

partition <- function(M, contrast, part.method=c('beckmann', 'guttman', 'ridgway')) {
  # Convert single matrices to array, for simplicity. Will drop extra dim at the end
  dimM <- dim(M)
  if (length(dimM) == 2L) {
    M <- array(M, dim=c(dimM, 1L), dimnames=c(dimnames(M), NULL))
    dimM <- c(dimM, 1L)
  }

  part.method <- match.arg(part.method)
  if (part.method == 'guttman') {
    idx <- unique(which(contrast != 0, arr.ind=TRUE)[, 2L])
    X <- M[, idx, , drop=FALSE]
    Z <- M[, -idx, , drop=FALSE]
    k <- length(idx)
    eCm <- cbind(contrast[, idx, drop=FALSE], contrast[, -idx, drop=FALSE])

  } else {
    subjs <- dimnames(M)[[1L]]
    regions <- dimnames(M)[[3L]]
    k <- qr.default(contrast)$rank
    rZ <- qr.default(M[, , 1L])$rank - k
    X <- array(0, dim=c(dimM[1L], k, dimM[3L]), dimnames=list(subjs, dimnames(contrast)[[1L]], regions))
    Z <- array(0, dim=c(dimM[1L], rZ, dimM[3L]), dimnames=list(subjs, NULL, regions))
    if (part.method == 'beckmann') {
      Cu <- MASS::Null(t(contrast))

      for (r in seq_len(dimM[3L])) {
        D <- inv(M[, , r])
        cdc <- inv(contrast %*% D, contrast, transpose=TRUE)
        Pc <- crossprod(contrast, cdc) %*% contrast %*% D
        Cv <- Cu - Pc %*% Cu
        F3 <- inv(Cv, D %*% Cv)
        X[, , r] <- M[, , r] %*% tcrossprod(D, contrast) %*% cdc
        Z[, , r] <- M[, , r] %*% D %*% Cv %*% F3
      }

    } else if (part.method == 'ridgway') {
      pinvC <- MASS::ginv(contrast)
      C0 <- diag(dimM[2L]) - crossprod(contrast, t(pinvC))

      for (r in seq_len(dimM[3L])) {
        tmpX <- M[, , r] %*% pinvC
        tmpZ <- svd(M[, , r] %*% C0)$u
        Z[, , r] <- tmpZ[, 1L:rZ, drop=FALSE]
        X[, , r] <- tmpX - Z[, , r] %*% MASS::ginv(Z[, , r]) %*% tmpX
      }
    }
    eCm <- cbind(diag(k), matrix(0, nrow=k, ncol=rZ))
  }
  eCx <- eCm[, 1L:k, drop=FALSE]
  if (dimM[3L] == 1L) {
    X <- abind::adrop(X, drop=3L)
    Z <- abind::adrop(Z, drop=3L)
  }
  Mp <- abind::abind(X, Z, along=2L)
  return(list(Mp=Mp, X=X, Z=Z, eCm=eCm, eCx=eCx))
}

#' Helper function to setup for randomise
#'
#' \code{setup_randomise} is used to setup the data/objects for any function
#' that does permutations for GLM-based analysis.
#'
#' The tasks performed by this function are:
#' \enumerate{
#'   \item Separate the design matrix into the independent variables of interest
#'     and nuisance variables (which is contrast-dependent) using
#'     \code{\link{partition}}.
#'   \item Calculate the new contrast(s) based on this new design matrix (also
#'     through \code{\link{partition}}
#'   \item Calculate the inverse of the cross product of the full model
#'   \item Calculate the \dQuote{hat} and residual-forming matrices due to nuisance
#'     alone
#'   \item For F contrasts, return an inverse of the cross product between the
#'     contrast and the inverted design matrix, and the contrast's rank
#' }
#'
#' For F-contrasts, the \code{CMtM} matrix is
#' \deqn{(C (M^T M)^{-1} C^T)^{-1}}
#'
#' @inheritParams GLM
#' @inheritParams randomise
#' @return A list containing, for each of the following, a list with length
#'   equal to the number of contrasts:
#'   \item{Mp}{The full partitioned model, joined. For Still-White, this is
#'     actually just the \code{Xp} matrix.}
#'   \item{Rz}{The residual-forming matrix}
#'   \item{MtM}{The inverse of the cross product of the full model \code{Mp}}
#'   \item{eC}{The \emph{effective contrast}, equivalent to the original, for
#'     the partitioned model \code{[X, Z]} and considering all covariates}
#'   \item{CMtM}{The effective contrast \code{eC} multiplied by the cross
#'     product of the full model. For F-contrasts it is the inverse; for
#'     T-contrasts, the diagonal.}
#'   \item{rkC}{Rank of the effective contrast matrix.}
#'   \item{Xp}{(for Draper-Stoneman and Smith methods) List of matrices of
#'     covariates of interest}
#'   \item{Zp}{(for Draper-Stoneman and Smith methods) List of matrices of
#'     nuisance covariates}
#' @noRd

setup_randomise <- function(perm.method, part.method, X, contrasts, con.type, nC) {
  if (con.type == 't') contrasts <- matrix2list(contrasts)
  stopifnot(length(contrasts) == nC)
  Mp <- MtM <- Rz <- eC <- Xp <- Zp <- CMtM <- vector('list', length=nC)
  rkC <- rep.int(0L, nC)

  dimX <- dim(X)
  for (j in seq_len(nC)) {
    parts <- partition(X, contrasts[[j]], part.method)
    if (perm.method == 'stillWhite') {
      Mp[[j]] <- parts$X
      eC[[j]] <- parts$eCx
    } else {
      Mp[[j]] <- parts$Mp
      eC[[j]] <- parts$eCm
    }
    MtM[[j]] <- inv(Mp[[j]])
    Xp[[j]] <- parts$X
    Zp[[j]] <- parts$Z
    rkC[j] <- qr.default(eC[[j]])$rank

    Xpinv <- function(x) x %*% pinv(x)
    # Handle multiple design matrices
    if (length(dimX) == 3L) {
      In <- array(diag(dimX[1L]), dim=dimX[c(1L, 1L, 3L)])
      Hz <- switch(perm.method,
                   draperStoneman=, manly=array(0, dim=dim(In)),
                   terBraak=array(apply(Mp[[j]], 3L, Xpinv), dim=dim(In)),
                   array(apply(Zp[[j]], 3L, Xpinv), dim=dim(In)))  # All others
      CXtXfun <- if (part.method == 'guttman') cxtxfun_3d(con.type, transpose=FALSE)
        else cxtxfun_3d(con.type)
      CMtM[[j]] <- CXtXfun(eC[[j]], MtM[[j]], rkC[j], dimX[3L])
    } else {
      In <- diag(dimX[1L])
      # For Draper-Stoneman and Manly, Rz will be the identity matrix
      # NOTE: for ter Braak, this is really "Hm", and "Rz" is really "Rm"
      Hz <- switch(perm.method,
                   draperStoneman=, manly=matrix(0, dimX[1L], dimX[1L]),
                   freedmanLane=, smith=, stillWhite=Zp[[j]] %*% pinv(Zp[[j]]),
                   terBraak=Mp[[j]] %*% pinv(Mp[[j]]))

      CMtM[[j]] <- if (con.type == 'f') inv(eC[[j]] %*% MtM[[j]], eC[[j]], transpose=TRUE)
        else diag_sq(tcrossprod(eC[[j]] %*% MtM[[j]], eC[[j]]), rkC[j])
    }
    Rz[[j]] <- In - Hz
    if (perm.method %in% c('draperStoneman', 'smith')) {
      Xp[[j]] <- if (length(dimX) == 2L) Rz[[j]] %*% Xp[[j]]
        else array(vapply(seq_len(dimX[3L]), function(r) Rz[[j]][, , r] %*% Xp[[j]][, , r],
                          numeric(prod(dim(Xp[[j]])[1L:2L]))), dim=dim(Xp[[j]]))
      dimnames(Xp[[j]]) <- dimnames(parts$X)
    }
  }

  out <- list(Mp=Mp, Rz=Rz, MtM=MtM, eC=eC, CMtM=CMtM, rkC=rkC)
  if (perm.method %in% c('draperStoneman', 'smith', 'stillWhite')) out <- c(out, list(Xp=Xp, Zp=Zp))
  return(out)
}

#' @section Permutation methods:
#' The permutation methods can be split into 2 groups, depending on which part
#' of the model they permute. For full details, see \emph{Winkler et al., 2014}.
#' \describe{
#'   \item{Permute Y}{Freedman-Lane, Manly, and ter Braak}
#'   \item{Permute X}{Smith, Draper-Stoneman, and Still-White}
#' }
#'
#' Depending on the size of the data, it may be faster to use a method that
#' permutes \code{Y} instead of \code{X}. For example, in \code{\link{NBS}} with
#' dense matrices (more than 400-500 edges), it will be somewhat faster to use
#' the \dQuote{Smith} method compared to \dQuote{Freedman-Lane}. If using
#' \code{\link{brainGraph_GLM}}, the number of vertices follows the same
#' relationship.
#'
#' Furthermore, all methods except Still-White include the \code{Z} (nuisance
#' covariate) matrix when calculating the permuted statistics.
#'
#' @param X Numeric matrix or 3D array of the design matrix(es)
#' @param y Numeric matrix of outcome variables, with 1 column per region, or a
#'   single column if there is a different design matrix per region
#' @param ctype The contrast type
#' @param nC Integer; the number of contrasts
#' @param skip Integer vector indicating which (if any) contrasts to skip. Only
#'   used by \code{\link{NBS}}.
#' @param n,p,ny,dfR Integers for the number of observations, design matrix
#'   columns (its rank), number of regions/outcome variables, and residual
#'   degrees of freedom, respectively
#' @export
#' @inheritParams GLM
#' @importFrom foreach getDoParRegistered
#' @importFrom doParallel registerDoParallel
#' @return A numeric array with dimensions \eqn{n_y \times N \times n_c};
#'   the number of rows equals number of regions/outcome variables, number of
#'   columns equals \code{N}, and the 3rd dimension is the number of contrasts
#' @rdname randomise
#' @references Draper, N.R. and Stoneman, D.M. (1966) Testing for the inclusion
#'   of variables in linear regression by a randomisation technique.
#'   \emph{Technometrics}. \bold{8(4)}, 695--699.
#' @references Freedman, D. and Lane, D. (1983) A nonstochastic interpretation
#'   of reported significance levels. \emph{J Bus Econ Stat}, \bold{1(4)},
#'   292--298. \doi{10.1080/07350015.1983.10509354}
#' @references Manly B.F.J. (1986) Randomization and regression methods for
#'   testing for associations with geographical, environmental, and biological
#'   distances between populations. \emph{Res Popul Ecol}. \bold{28(2)},
#'   201--218.
#' @references Nichols, T.E. and Holmes, A.P. (2001) Nonparametric permutation
#'   tests for functional neuroimaging: A primer with examples. \emph{Human
#'   Brain Mapping}. \bold{15(1)}, 1--25.
#'   \doi{10.1002/hbm.1058}
#' @references Smith, S.M. and Jenkinson, M. and Beckmann, C. and Miller, K. and
#'   Woolrich, M. (2007) Meaningful design and contrast estimability in fMRI.
#'   \emph{NeuroImage}. \bold{34(1)}, 127--36.
#'   \doi{10.1016/j.neuroimage.2006.09.019}
#' @references Still, A.W. and White, A.P. (1981) The approximate randomization
#'   test as an alternative to the F test in analysis of variance. \emph{Br J
#'   Math Stat Psychol}. \bold{34(2)}, 243--252.
#' @references ter Braak, C.J.F. 1992. Permutation versus bootstrap significance
#'   tests in multiple regression and ANOVA. \emph{Bootstrapping and related
#'   techniques}. Springer, Berlin, Heidelberg. 79--85.
#' @references Winkler, A.M. and Ridgway, G.R. and Webster, M.A. and Smith, S.M.
#'   and Nichols, T.E. (2014) Permutation inference for the general linear
#'   model. \emph{NeuroImage}. \bold{92}, 381--397.
#'   \doi{10.1016/j.neuroimage.2014.01.060}

randomise <- function(perm.method, part.method, N, perms, X, y, contrasts,
                      ctype, nC, skip=NULL, n=dim(X)[1L], p=qr.default(X)$rank,
                      ny=dim(y)[2L], dfR=n-p) {
  i <- NULL
  randMats <- setup_randomise(perm.method, part.method, X, contrasts, ctype, nC)
  Mp <- randMats$Mp; eC <- randMats$eC; CMtM <- randMats$CMtM; rkC <- randMats$rkC

  statfun <- switch(ctype, t=fastLmBG_t_rand, f=fastLmBG_f_rand)
  cxtxfun <- if (ctype == 't') function(con, x, k) diag_sq(tcrossprod(con %*% x, con), k)
    else function(con, x, k) inv(con %*% x, con, transpose=TRUE)
  combfun <- cbind
  doReshape <- FALSE
  null.dist <- array(0, dim=c(ny, N, nC))

  # If X is the same for all contrasts (Guttman), get all stats at one time
  #-----------------------------------------------------------------------------
  if (nC > 1L && all_identical(Mp)) {
    if (ctype == 't') {
      eC[[1L]] <- do.call(rbind, eC)
      rkC <- qr.default(eC[[1L]])$rank
      combfun <- rbind
      CMtM[[1L]] <- cxtxfun(eC[[1L]], randMats$MtM[[1L]], rkC[1L])
    }
    null.dist <- array(0, dim=c(ny * N, nC, 1L))
    nCorig <- nC; nC <- 1L; skip <- NULL  # Redefine so we only use the 1 combined contrast
    doReshape <- TRUE
  }
  #-----------------------------------------------------------------------------
  # Loop through contrasts
  #-----------------------------------------------------------------------------
  if (!getDoParRegistered()) {
    cl <- makeCluster(getOption('bg.ncpus'))
    registerDoParallel(cl)
  }
  for (j in setdiff(seq_len(nC), skip)) {
    M <- Mp[[j]]; con <- eC[[j]]; rk <- rkC[j]

    if (perm.method %in% c('draperStoneman', 'smith', 'stillWhite')) {
      if (perm.method == 'stillWhite') {
        yz <- randMats$Rz[[j]] %*% y
        Mpfun <- function(M, porder, xcols) M[porder, , drop=FALSE]
        p <- rk
        df <- n - p
      } else {
        yz <- y; df <- dfR
        Mpfun <- function(M, porder, xcols) {
          M[, xcols] <- M[porder, xcols]
          M
        }
      }
      xcols <- seq_len(dim(randMats$Xp[[j]])[2L])
      I <- diag(1, n, p)  # Multiplier for "qr_Q2" (and "qr.qy")
      null.dist[, , j] <- foreach(i=seq_len(N), .combine=combfun) %dopar% {
        M <- Mpfun(M, perms[i, ], xcols)
        QR <- qr.default(M)
        Q <- qr_Q2(QR, I, n, p)
        R <- qr_R2(QR, p)
        MtM <- chol2inv(QR$qr, p)
        CMtM <- cxtxfun(con, MtM, rk)
        statfun(M, yz, con, QR, Q, R, n, p, ny, df, MtM, CMtM, rk)
      }

    # Freedman-Lane, Manly, & Ter Braak
    } else {
      Minv <- randMats$MtM[[j]]; CMinv <- CMtM[[j]]
      QR <- qr.default(M)
      Q <- qr_Q2(QR, n=n, p=p)
      R <- qr_R2(QR, p)
      yz <- randMats$Rz[[j]] %*% y
      if (ny == 1L) {
        yzMat <- matrix(0, n, N)
        for (i in seq_len(N)) yzMat[, i] <- yz[perms[i, ], ]
        null.dist[, , j] <- statfun(M, yzMat, con, QR, Q, R, n, p, N, dfR, Minv, CMinv, rk)
      } else {
        null.dist[, , j] <- foreach(i=seq_len(N), .combine=combfun) %dopar% {
          statfun(M, yz[perms[i, ], ], con, QR, Q, R, n, p, ny, dfR, Minv, CMinv, rk)
        }
      }
    }
  }
  if (isTRUE(doReshape)) null.dist <- array(null.dist, dim=c(ny, N, nCorig))
  return(null.dist)
}

#' @param runX Character vector of regions with non-singular designs
#' @export
#' @rdname randomise

randomise_3d <- function(perm.method, part.method, N, perms, X, y, contrasts,
                         ctype, nC, runX=dimnames(X)[[3L]], n=dim(X)[1L],
                         p=qr.default(X[, , 1L])$rank, ny=length(runX), dfR=n-p) {
  i <- NULL
  X <- X[, , runX, drop=FALSE]
  randMats <- setup_randomise(perm.method, part.method, X, contrasts, ctype, nC)
  Mp <- randMats$Mp; eC <- randMats$eC; CMtM <- randMats$CMtM; rkC <- randMats$rkC

  randfun <- switch(ctype, t=fastLmBG_t_rand_3d, f=fastLmBG_f_rand_3d)
  fitfun <- switch(perm.method, freedmanLane=, stillWhite=, terBraak=fastLmBG_3dY,
                   fastLmBG_3d)
  cxtxfun <- cxtxfun_3d(ctype)
  combfun <- cbind
  doReshape <- FALSE
  null.dist <- array(0, dim=c(ny, N, nC))

  # If X is the same for all contrasts (Guttman), get all stats at one time
  #-----------------------------------------------------------------------------
  if (nC > 1L && all_identical(Mp)) {
    if (ctype == 't') {
      eC[[1L]] <- do.call(rbind, eC); CMtM[[1L]] <- do.call(rbind, CMtM)
      rkC <- qr.default(eC[[1L]])$rank
      combfun <- rbind
      cxtxfun <- cxtxfun_3d(ctype, transpose=FALSE)
    }
    null.dist <- array(0, dim=c(ny * N, nC, 1L))
    nCorig <- nC; nC <- 1L  # Re-define so we don't loop past the 1 combined contrast
    doReshape <- TRUE
  }
  #-----------------------------------------------------------------------------
  # Loop through contrasts
  #-----------------------------------------------------------------------------
  if (!getDoParRegistered()) {
    cl <- makeCluster(getOption('bg.ncpus'))
    registerDoParallel(cl)
  }
  for (j in seq_len(nC)) {
    M <- Mp[[j]]; con <- eC[[j]]; rk <- rkC[j]

    if (perm.method %in% c('draperStoneman', 'smith', 'stillWhite')) {
      if (perm.method == 'stillWhite') {
        yz <- apply(randMats$Rz[[j]], 3L,  `%*%`, y)
        dimnames(yz)[[2L]] <- runX
        Mpfun <- function(M, porder, xcols) M[porder, , , drop=FALSE]
        p <- rk
        fitfun <- if (p == 1L) fastLmBG_3dY_1p else fastLmBG_3dY
        dfR <- n - p
      } else {
        yz <- y
        Mpfun <- function(M, porder, xcols) {
          M[, xcols, ] <- M[porder, xcols, ]
          M
        }
      }
      xcols <- seq_len(dim(randMats$Xp[[j]])[2L])
      vnames <- dimnames(Mp[[j]])[[2L]]
      I <- diag(1, n, p)  # Multiplier for "qr_Q2" (and "qr.qy")
      null.dist[, , j] <- foreach(i=seq_len(N), .combine=combfun) %dopar% {
        M <- Mpfun(M, perms[i, ], xcols)
        QR <- qr(M)
        Q <- lapply(QR, qr_Q2, I, n, p)
        R <- lapply(QR, qr_R2, p)
        MtM <- inv(QR, p, ny, vnames, runX)
        CMtM <- cxtxfun(con, MtM, rk, ny)
        randfun(fitfun, M, yz, con, runX, QR, Q, R, n, p, ny, dfR, MtM, CMtM, rk)
      }

    # Freedman-Lane, Manly, & Ter Braak
    } else {
      Minv <- randMats$MtM[[j]]; CMinv <- CMtM[[j]]
      QR <- qr(M)
      Q <- lapply(QR, qr_Q2, n=n, p=p)
      R <- lapply(QR, qr_R2, p)
      yz <- if (perm.method == 'manly') y else apply(randMats$Rz[[j]], 3L, `%*%`, y)
      if (dim(yz)[2L] > 1L) dimnames(yz)[[2L]] <- runX

      null.dist[, , j] <- foreach(i=seq_len(N), .combine=combfun) %dopar% {
        randfun(fitfun, M, yz[perms[i, ], ], con, runX, QR, Q, R, n, p, ny, dfR, Minv, CMinv, rk)
      }
    }
  }
  if (isTRUE(doReshape)) null.dist <- array(null.dist, dim=c(ny, N, nCorig))
  return(null.dist)
}

#' Perform an analysis with random graphs for brain MRI data
#'
#' \code{analysis_random_graphs} performs the steps needed for doing typical
#' graph theory analyses with brain MRI data if you need to generate equivalent
#' random graphs. This includes calculating \emph{small world} parameters and
#' normalized \emph{rich club} coefficients.
#'
#' \code{analysis_random_graphs} does the following:
#' \enumerate{
#'   \item Generate \code{N} random graphs for each graph and density/threshold
#'   \item Write graphs to disk in \code{savedir}. Read them back into \code{R}
#'     and combine into lists; then write these lists to disk. You can later
#'     delete the individual \code{.rds} files afterwards.
#'   \item Calculate \emph{small world} parameters, along with values for a few
#'     global graph measures that may be of interest.
#'   \item Calculate \emph{normalized rich club coefficients} and associated
#'     p-values.
#' }
#'
#' @param g.list List of \code{brainGraphList} objects; the length of this list
#'   should equal the number of thresholds/densities in the study
#' @param savedir Character string specifying the directory in which to save the
#'   generated graphs. Default: current working directory
#' @inheritParams Creating_Graphs
#' @export
#'
#' @return \code{analysis_random_graphs} returns a \emph{list} containing:
#' \item{rich}{A data table containing normalized rich-club coefficients and
#'   p-values}
#' \item{small}{A data table with small-world parameters}
#' \item{rand}{A data table with some global graph measures for all random
#'   graphs generated}
#'
#' @name Random Graphs
#' @rdname random_graphs
#' @family Random graph functions
#' @seealso \code{\link{small.world}}
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}
#' @examples
#' \dontrun{
#' rand_all <- analysis_random_graphs(g.norm, 1e2,
#'   savedir='/home/cwatson/dti/rand', clustering=F)
#' }

analysis_random_graphs <- function(g.list, level=g.list[[1L]]$level, N=100L, savedir='.', ...) {
  # Check if components are 'brainGraphList' objects
  matches <- vapply(g.list, is.brainGraphList, logical(1L))
  if (any(!matches)) stop("Input must be a list of 'brainGraphList' objects.")

  if (!dir.exists(savedir)) dir.create(savedir, recursive=TRUE)

  # Generate random graphs and calculate rich club coeff's for all thresholds
  fname_base <- if (level == 'subject') 'sub' else 'grp'

  kNumSubj <- vapply(g.list, nobs, integer(1L))
  phi.norm <- rand.dt <- small.dt <- vector('list', length(g.list))
  for (i in seq_along(g.list)) {
    phi.norm[[i]] <- vector('list', kNumSubj[i])

    print(paste0('Random graphs for threshold #', i, '; ', format(Sys.time(), '%H:%M:%S')))
    progbar <- txtProgressBar(min=0, max=kNumSubj[i], style=3)
    for (j in seq_along(g.list[[i]]$graphs)) {
      rand <- sim.rand.graph.par(g.list[[i]][j], level=level, N=N, atlas=g.list[[i]]$atlas, ...)
      saveRDS(rand, file=file.path(savedir, sprintf('rand%02i_%s%04i%s', i, fname_base, j, '.rds')))
      phi.norm[[i]][[j]] <- rich_club_norm(g.list[[i]][j], rand=rand)
      rm(list='rand')
      gc()
      setTxtProgressBar(progbar, j)
    }
  }
  close(progbar)

  # Get all small-worldness and random measures, all thresholds
  #-----------------------------------------------------------------------------
  for (i in seq_along(g.list)) {
    fnames <- list.files(savedir, sprintf('rand%02i_%s.*', i, fname_base), full.names=TRUE)
    rand.all <- lapply(fnames, readRDS)
    rand.all <- as_brainGraphList(rand.all, type='random', level=level)
    saveRDS(rand.all, file=file.path(savedir, sprintf('rand%02i%s', i, '_all.rds')))

    small.dt[[i]] <- small.world(g.list[[i]], rand.all)
    rand.dt[[i]] <- get_rand_attrs(rand.all, level)
    rm(list='rand.all')
    gc()
  }
  rand.dt <- rbindlist(rand.dt)
  small.dt <- rbindlist(small.dt)
  rich.dt <- rbindlist(lapply(phi.norm, rbindlist))

  sID <- getOption('bg.subject_id')
  gID <- getOption('bg.group')
  key1 <- if (level == 'subject') sID else gID
  setkeyv(small.dt, c(key1, 'density'))

  return(list(rich=rich.dt, small=small.dt, rand=rand.dt))
}

#' Convenience function to get attributes for lists of random graphs
#'
#' @param bg.list A \code{brainGraphList} object, created in
#'   \code{\link{analysis_random_graphs}}
#' @keywords internal
#' @return A \code{data.table}

get_rand_attrs <- function(bg.list, level) {
  threshold <- NULL
  rand <- bg.list$graphs

  grps <- groups(bg.list)
  N <- lengths(rand)

  mod <- unlist(lapply(rand, vapply, graph_attr, numeric(1L), USE.NAMES=FALSE, 'mod'))
  Cp <- unlist(lapply(rand, vapply, graph_attr, numeric(1L), USE.NAMES=FALSE, 'Cp'))
  Lp <- unlist(lapply(rand, vapply, graph_attr, numeric(1L), USE.NAMES=FALSE, 'Lp'))
  E.global <- unlist(lapply(rand, vapply, graph_attr, numeric(1L), USE.NAMES=FALSE, 'E.global'))

  if (level == 'subject') {
    ids <- names(rand)
    DT <- data.table(Study.ID=rep.int(ids, N), Group=rep.int(grps, N))
    setnames(DT, 'Study.ID', getOption('bg.subject_id'))
  } else if (level == 'group') {
    DT <- data.table(Group=rep.int(grps, N))
  }
  setnames(DT, 'Group', getOption('bg.group'))
  DT <- cbind(DT, data.table(mod=mod, Cp=Cp, Lp=Lp, E.global=E.global))
  if ('density' %in% graph_attr_names(rand[[1L]][[1L]])) {
    densities <- vapply(rand, function(x) graph_attr(x[[1L]], 'density'), numeric(1L))
    DT[, density := rep.int(densities, N)]
  }
  if ('threshold' %in% graph_attr_names(rand[[1L]][[1L]])) {
    threshes <- vapply(rand, function(x) graph_attr(x[[1L]], 'threshold'), numeric(1L))
    DT[, threshold := rep.int(threshes, N)]
  }
  return(DT)
}

################################################################################
# MAIN CREATION FUNCTIONS
################################################################################

#' Create a brainGraph object
#'
#' \code{make_brainGraph} is the main creation function for creating a
#' \code{brainGraph} graph object. This is simply an \code{igraph} graph
#' object with additional attributes (at all levels). Several of the graph-level
#' attributes serve the purpose of providing metadata on how the connectivity
#' matrices/networks were created.
#'
#' @section Graph-level attributes:
#' Graph-level attributes added are:
#' \describe{
#'   \item{version}{The R, \code{brainGraph}, and \code{igraph} package versions
#'     used to create the graph}
#'   \item{date}{The creation date, from \code{\link{as.POSIXct}}}
#'   \item{atlas}{Character string denoting the brain atlas used}
#'   \item{type}{Character string specifying whether this is an \emph{observed}
#'     or \emph{random} graph}
#'   \item{modality}{The imaging modality; you can choose anything you like,
#'     but the \code{summary.brainGraph} knows about \code{dti}, \code{fmri},
#'     \code{thickness}, \code{area}, and \code{volume}}
#'   \item{weighting}{What edge weights represent; you can choose anything you
#'     like, but \code{summary.brainGraph} knows about \code{fa}, \code{sld}
#'     (streamline density, tractography), \code{pearson}, \code{spearman},
#'     \code{kendall}, and \code{partial} (partial correlation coefficient)}
#'   \item{threshold}{Numeric indicating the threshold used to create the final
#'     connectivity matrix (if any)}
#'   \item{name}{Character string specifying the study ID or group/contrast
#'     name, depending on the \code{level} argument}
#'   \item{Group}{Character string specifying the experimental group that the
#'     given subject belongs to, or if it is a group-level graph}
#'   \item{subnet}{Integer vector, if \code{subnet} was specified in the call}
#' }
#'
#' @section Vertex attributes:
#' Vertex-level attributes added are:
#' \describe{
#'   \item{name}{The names of the brain regions in the network}
#'   \item{lobe}{The names of the major brain lobes for each vertex}
#'   \item{hemi}{The names of the hemisphere for each vertex (either \code{'L'},
#'     \code{'R'}, or \code{'B'})}
#'   \item{lobe.hemi}{The lobe-hemisphere combination (represented as an
#'     \emph{integer} vector)}
#'   \item{class}{The tissue class (if applicable)}
#'   \item{network}{The network (if the atlas is \code{dosenbach160})}
#'   \item{x,y,z}{The spatial coordinates of the (centers-of-mass) brain regions
#'     in MNI space}
#'   \item{x.mni,y.mni,z.mni}{Same as above}
#'   \item{color.lobe,color.class,color.network}{Colors for vertices of their
#'     respective membership}
#'   \item{circle.layout}{Integer vector indicating the order (going
#'     counter-clockwise from the top) for circular layouts}
#' }
#'
#' @section Edge attributes:
#' Edge-level attributes added are:
#' \describe{
#'   \item{color.lobe,color.class,color.network}{Correspond to the vertex
#'     attribute of the same name. Inter-group edges will be colored
#'     \emph{gray}}
#' }
#'
#' @section Specifying a subnetwork:
#' You can create a graph for a subset of an atlas's regions with the
#' \code{subnet} argument. This can either be a numeric or character vector. If
#' the input object (either a matrix or an \code{igraph} graph) has fewer
#' rows/columns or vertices, respectively, than the atlas then the \code{subnet}
#' graph attribute will also be added to the return object. This may occur if,
#' for example, you use \code{\link{make_auc_brainGraph}} on graphs that were
#' initially created from subnetworks.
#'
#' @param x An \code{igraph} graph object, numeric matrix, or \code{bg_mediate}
#'   object
#' @param atlas Character string specifying the brain atlas
#' @param type Character string indicating the type of graphs. Default:
#'   \code{observed}
#' @param level Character string indicating whether the graphs are subject-,
#'   group-, or contrast-specific. Default: \code{'subject'}
#' @param set.attrs Logical indicating whether to assign all graph-, vertex-,
#'   and edge-level attributes (via \code{\link{set_brainGraph_attr}}). Default:
#'   \code{TRUE}
#' @param modality Character string indicating imaging modality (e.g. 'dti').
#'   Default: \code{NULL}
#' @param weighting Character string indicating how the edges are weighted
#'   (e.g., 'fa', 'pearson', etc.). Default: \code{NULL}
#' @param threshold Integer or number indicating the threshold used when
#'   \dQuote{sparsifying} the connectivity matrix (if any). Default: \code{NULL}
#' @param ... Arguments passed to \code{\link{set_brainGraph_attr}}
#' @export
#'
#' @return A \code{brainGraph} graph object with additional graph-, vertex-, and
#'   edge-level attributes (see below).
#' @name Creating_Graphs
#' @rdname make_brainGraph
#' @family Graph creation functions

make_brainGraph <- function(x, atlas, type=c('observed', 'random'),
                            level=c('subject', 'group', 'contrast'), set.attrs=TRUE,
                            modality=NULL, weighting=NULL, threshold=NULL, ...) {
  UseMethod('make_brainGraph')
}

#' Create a brainGraph object from an igraph graph
#'
#' @param name Character string indicating subject ID or group/contrast name,
#'   depending on the \code{level}. Default: \code{NULL}
#' @param Group Character string indicating group membership. Default:
#'   \code{NULL}
#' @param subnet Integer or character vector indicating the vertices to keep, if
#'   you are interested in working with a subset of an atlas. By default, all
#'   vertices are used.
#' @export
#' @rdname make_brainGraph

make_brainGraph.igraph <- function(x, atlas, type=c('observed', 'random'),
                                   level=c('subject', 'group', 'contrast'),
                                   set.attrs=TRUE, modality=NULL, weighting=NULL,
                                   threshold=NULL, name=NULL, Group=NULL, subnet=NULL, ...) {
  Brodmann <- lobe <- hemi <- index <- x.mni <- y.mni <- z.mni <- NULL

  x <- get_metadata(x)
  x$atlas <- if (missing(atlas)) guess_atlas(x) else atlas
  DT <- get(x$atlas)

  if (is.null(subnet) && vcount(x) < dim(DT)[1L]) subnet <- V(x)$name
  if (!is.null(subnet)) {
    if (is.character(subnet)) subnet <- DT[name %in% subnet, which=TRUE]
    DT <- DT[subnet]
    if (vcount(x) != length(subnet)) x <- induced_subgraph(x, subnet)
    x$subnet <- subnet
  }
  if (!is_named(x)) {
    V(x)$name <- DT$name
  } else {
    nonmatches <- !V(x)$name %in% DT[, name]
    if (any(nonmatches)) {
      stop(paste('Check the following vertex names: ',
                 paste(V(x)$name[nonmatches], collapse=' ')))
    }
  }

  vorder <- match(V(x)$name, DT$name)
  V(x)$lobe <- DT[vorder, as.character(lobe)]
  V(x)$lobe.hemi <- as.numeric(DT[vorder, interaction(lobe, hemi)])
  V(x)$hemi <- DT[vorder, as.character(hemi)]

  cols <- c('class', 'network', 'area', 'gyrus', 'Yeo_7network', 'Yeo_17network')
  for (col in cols[hasName(DT, cols)]) {
    x <- set_vertex_attr(x, col, value=DT[vorder, as.character(get(col))])
  }
  if (hasName(DT, 'Brodmann')) V(x)$Brodmann <- DT[vorder, Brodmann]

  level <- match.arg(level)
  type <- match.arg(type)
  x$level <- level
  x$type <- type
  attrs <- c('modality', 'weighting', 'threshold', 'name', 'Group')
  attrs <- names(which(vapply(attrs, function(y) !is.null(get(y)), logical(1L))))
  for (a in attrs) x <- set_graph_attr(x, a, get(a))
  if (level == 'group' && !is.null(Group)) x$name <- x$Group

  if (type == 'observed') {
    l.cir <- vector('integer')
    lobes <- DT[, levels(lobe)]
    V(x)$x <- V(x)$x.mni <- DT[vorder, x.mni]
    V(x)$y <- V(x)$y.mni <- DT[vorder, y.mni]
    V(x)$z <- V(x)$z.mni <- DT[vorder, z.mni]
    x <- set_graph_colors(x, 'color.lobe', DT[vorder, as.numeric(lobe)])
    if (x$atlas == 'dosenbach160') l.cir <- c(l.cir, which(V(x)$hemi == 'B'))
    for (col in cols[hasName(DT, cols)]) {
      x <- set_graph_colors(x, paste0('color.', col), DT[vorder, as.numeric(get(col))])
    }

    lobeorder <- list('Frontal', c('Insula', 'Central'), c('Limbic', 'Cingulate'),
                      'SCGM', 'Temporal', 'Parietal', 'Occipital', 'Cerebellum', 'Brainstem')
    if (!'Brainstem' %in% lobes) lobeorder <- lobeorder[-9L]
    if (!'Cerebellum' %in% lobes) lobeorder <- lobeorder[-8L]
    if (!'SCGM' %in% lobes) lobeorder <- lobeorder[-4L]
    for (i in seq_along(lobeorder)) {
      l.cir <- c(l.cir, DT[lobe %in% lobeorder[[i]] & !hemi %in% c('B', 'R'),
                           .SD[order(-y.mni, x.mni), index]])
    }
    for (i in seq_along(lobeorder)) {
      l.cir <- c(l.cir, DT[lobe %in% rev(lobeorder)[[i]] & hemi == 'R',
                           .SD[order(y.mni, x.mni), index]])
    }
    V(x)$circle.layout <- l.cir
  }

  # Set a bunch of attributes
  if (isTRUE(set.attrs) && ecount(x) > 1) x <- set_brainGraph_attr(x, type, ...)

  class(x) <- c('brainGraph', class(x))
  return(x)
}

#' Create a brainGraph object from an adjacency matrix
#'
#' @param mode Character string defining how the matrix should be interpreted.
#'   Default: \code{'undirected'}
#' @param weighted Logical specifying whether to create a weighted network
#' @param diag Logical indicating whether to include the diagonal of the
#'   connectivity matrix. Default: \code{FALSE}
#' @export
#'
#' @rdname make_brainGraph
#' @examples
#' \dontrun{
#' bg <- make_brainGraph(A, 'dkt', modality='dti', weighting='fa',
#'   mode='undirected', diag=FALSE, weighted=TRUE)
#' }

make_brainGraph.matrix <- function(x, atlas, type=c('observed', 'random'),
                                   level=c('subject', 'group', 'contrast'),
                                   set.attrs=TRUE, modality=NULL, weighting=NULL,
                                   threshold=NULL, name=NULL, Group=NULL, subnet=NULL,
                                   mode='undirected', weighted=NULL, diag=FALSE, ...) {
  type <- match.arg(type)
  level <- match.arg(level)
  if (!is.null(subnet)) {
    if (is.character(subnet)) {
      DT <- get(atlas)
      subnet <- DT[name %in% subnet, which=TRUE]
    }
    x <- x[subnet, subnet, drop=FALSE]
  }
  g <- graph_from_adjacency_matrix(x, mode, weighted, diag)
  g <- make_brainGraph(g, atlas, type, level, set.attrs, modality, weighting,
                       threshold, name, Group, subnet, A=x, ...)
  return(g)
}

#' Create a graph with mediation-specific attributes
#'
#' \code{make_brainGraph.bg_mediate} creates a graph only for
#' \emph{vertex}-level analyses.
#'
#' @export
#' @rdname make_brainGraph
#' @return The method for \code{bg_mediate} returns a \code{brainGraph_mediate}
#'   object, which has extra attributes:
#'   \item{Graph}{\emph{mediator}, \emph{treat}, \emph{outcome}, \emph{nobs}}
#'   \item{Vertex}{\emph{b?.acme, p?.acme}, \emph{b?.ade, p?.ade},
#'     \emph{b?.prop, p?.prop}, \emph{b.tot, p.tot}}

make_brainGraph.bg_mediate <- function(x, atlas=x$atlas, type='observed',
                                       level='contrast', set.attrs=FALSE,
                                       modality=NULL, weighting=NULL,
                                       threshold=NULL, ...) {
  stopifnot(inherits(x, 'bg_mediate'), x$level == 'vertex')
  med.sum <- summary(x)$DT
  g.med <- make_empty_brainGraph(atlas, type, level, modality, weighting, threshold, ...)
  for (a in c('mediator', 'treat', 'outcome', 'nobs')) {
    g.med <- set_graph_attr(g.med, a, x[[a]])
  }
  for (a in c('b0.acme', 'b0.ade', 'b.tot', 'b0.prop')) {
    g.med <- set_vertex_attr(g.med, a, value=med.sum[[a]])
  }
  for (a in c('p0.acme', 'p0.ade', 'p.tot', 'p0.prop')) {
    g.med <- set_vertex_attr(g.med, a, value=1 - med.sum[[a]])
  }
  if (isTRUE(x$INT)) {
    for (a in c('b1.acme', 'b1.ade', 'b1.prop', 'b.avg.acme', 'b.avg.ade', 'b.avg.prop')) {
      g.med <- set_vertex_attr(g.med, a, value=med.sum[[a]])
    }
    for (a in c('p1.acme', 'p1.ade', 'p1.prop', 'p.avg.acme', 'p.avg.ade', 'p.avg.prop')) {
      g.med <- set_vertex_attr(g.med, a, value=1 - med.sum[[a]])
    }
  }
  class(g.med) <- c('brainGraph_mediate', class(g.med))
  return(g.med)
}

################################################################################
# OTHER METHODS
################################################################################

#' Determine whether the input is a brainGraph object
#'
#' @export
#' @rdname make_brainGraph
is.brainGraph <- function(x) inherits(x, 'brainGraph')

#' Print a summary of a brainGraph object
#'
#' @param object A \code{brainGraph} object
#' @param print.attrs Character string indicating whether or not to list the
#'   object's attributes (default: \code{all})
#' @export
#' @rdname make_brainGraph

summary.brainGraph <- function(object, print.attrs=c('all', 'graph', 'vertex', 'edge', 'none'), ...) {
  if (!is.brainGraph(object)) NextMethod(generic='summary', object=object)

  df <- print_bg_summary(object)
  if (object$level != 'subject') df <- df[-14L, ]

  print.attrs <- match.arg(print.attrs)
  attrtypes <- switch(print.attrs,
                      all=c('graph', 'vertex', 'edge'),
                      none=NULL,
                      print.attrs)
  attrs.l <- setNames(vector('list', length(attrtypes)), attrtypes)
  for (atype in attrtypes) {
    attrs <- switch(atype,
                    graph=graph_attr_names(object),
                    vertex=vertex_attr_names(object),
                    edge=edge_attr_names(object))
    if (length(attrs) > 0L) attrs.l[[atype]] <- print_text_vector(attrs, 3L)
  }
  out <- list(object=object, df=df, attrs=attrs.l, print.attrs=print.attrs)
  class(out) <- c('summary.brainGraph', class(out))
  out
}

#' @aliases summary.brainGraph
#' @method print summary.brainGraph
#' @export

print.summary.brainGraph <- function(x, ...) {
  print_title_summary('Summary for *', x$object$type, '* ', x$object$level, '-level graph: ', x$object$name)
  print(x$df, right=FALSE, row.names=FALSE)
  cat('\n')

  if (x$print.attrs != 'none') {
    for (atype in names(x$attrs)) {
      if (is.null(x$attrs[[atype]])) {
        cat('No', tolower(atype), 'attributes!')
      } else {
        title <- paste(simpleCap(atype), 'attributes')
        width <- getOption('width') - nchar(title) - 1L
        message(title, paste(rep.int('-', width / 2), collapse=''))
        print(x$attrs[[atype]], right=FALSE, row.names=FALSE)
      }
      cat('\n')
    }
  }
  invisible(x)
}

################################################################################
# OTHER CREATION FUNCTIONS
################################################################################

#' Create an empty graph with attributes for brainGraph
#'
#' \code{make_empty_brainGraph} creates an empty undirected \code{brainGraph}
#' object with vertex count equal to the atlas specified; i.e., it creates a
#' graph with 0 edges. Typically used to present results from an analysis in
#' which edges don't make sense (e.g., GLM comparing differences in a
#' vertex-level attribute).
#'
#' @export
#' @return \code{make_empty_brainGraph} -- An empty \code{brainGraph} graph
#'   object
#' @rdname make_brainGraph

make_empty_brainGraph <- function(atlas, type=c('observed', 'random'),
                                  level=c('subject', 'group', 'contrast'),
                                  modality=NULL, weighting=NULL, threshold=NULL,
                                  name=NULL, Group=NULL, ...) {
  n <- dim(get(atlas))[1L]
  A <- matrix(0, nrow=n, ncol=n)
  type <- match.arg(type)
  level <- match.arg(level)
  g <- make_brainGraph(A, atlas, type, level, set.attrs=FALSE, modality,
                       weighting, threshold, name, Group, ...)
  return(g)
}

#' Create a graph of the union of multiple vertex neighborhoods
#'
#' This function accepts multiple vertices, creates graphs of their
#' neighborhoods (of order 1), and returns the union of those graphs.
#'
#' @param g An \code{igraph} graph object
#' @param vs Either a character or integer vector (vertex names or indices,
#' respectively) for the vertices of interest
#' @export
#'
#' @return An \code{igraph} graph object containing the union of all edges and
#'   vertices in the neighborhoods of the input vertices; only the vertex
#'   attribute \emph{name} will be present
#' @family Graph creation functions
#' @seealso \code{\link[igraph]{ego}}
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}
#' @examples
#' \dontrun{
#' subg <- make_ego_brainGraph(g1[[N]], c(24, 58))
#' subg <- make_ego_brainGraph(g1[[N]], c('lPCUN', 'rPCUN'))
#' }

make_ego_brainGraph <- function(g, vs) {

  subgs <- make_ego_graph(g, order=1L, nodes=vs)
  if (is.character(vs)) vs <- which(V(g)$name %in% vs)

  for (i in seq_along(vs)) {
    subgs[[i]] <- delete_all_attr(subgs[[i]], keep.names=TRUE)
  }

  combine_graphs <- function(x, y) {
    n <- length(x)
    if (n < 2L) {
      res <- x[[1L]] %u% y
    } else {
      y <- x[[n]] %u% y
      x <- x[-n]
      res <- combine_graphs(x, y)
    }
    return(res)
  }

  inds <- unique(c(vs, unlist(lapply(vs, function(x) neighbors(g, x)))))
  subg.all <- combine_graphs(subgs, make_empty_graph(directed=FALSE) + vertices(V(g)$name[inds]))
  return(subg.all)
}

#' Create the intersection of graphs based on a logical condition
#'
#' Returns a graph object with vertices that meet certain criteria. By default,
#' only vertices that meet these criteria for \emph{all} input graphs will be
#' retained.
#'
#' If no vertices meet criteria for all input graphs, then an \code{igraph}
#' graph object with 0 vertices is returned. If \code{keep.all.vertices=TRUE},
#' this is essentially performing a \emph{union} of vertex sets that meet the
#' criteria. In any case, the return graph will have 0 edges.
#'
#' @param ... Graph objects or lists of graph objects
#' @param subgraph Character string specifying an equation (logical condition)
#'   for the vertices to subset
#' @param keep.all.vertices Logical indicating whether to keep all vertices that
#'   meet the criteria in at least 1 input graph. Default: \code{FALSE}
#' @export
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}
#' @return An \code{igraph} graph object
#' @examples
#' \dontrun{
#' res.mtpc <- mtpc(g, covars, ...)
#' g.mtpc <- make_glm_brainGraph(res.mtpc, atlas)
#'
#' ## All vertices with a significant MTPC result for all contrasts:
#' g.mtpc.int <- make_intersection_brainGraph(g.mtpc, subgraph='sig == 1')
#'
#' ## Return graphs with vertices with degree > 0 for each group separately
#' tapply(g.list, groups(g.list), make_intersection_brainGraph,
#'        subgraph='degree > 0')
#' }

make_intersection_brainGraph <- function(..., subgraph, keep.all.vertices=FALSE) {
  graphs <- args_as_list(...)
  stopifnot(all(vapply(graphs, is.brainGraph, logical(1L))))

  subs <- lapply(graphs, subset_graph, subgraph)
  inds.sub <- lapply(subs, `[[`, 'inds')

  combFun <- if (isTRUE(keep.all.vertices)) union else intersect
  inds.int <- Reduce(combFun, inds.sub)
  if (length(inds.int) == 0L) {
    g.int <- make_empty_graph(directed=FALSE)
  } else {
    names.int <- V(graphs[[1L]])[inds.int]$name
    g.int <- make_empty_brainGraph(graphs[[1L]]$atlas, graphs[[1L]]$type, graphs[[1L]]$level,
                                   graphs[[1L]]$modality, graphs[[1L]]$weighting, subnet=names.int)
  }
  return(g.int)
}

#' Calculate communicability
#'
#' \code{communicability} calculates the communicability of a network, a measure
#' which takes into account all possible paths (including non-shortest paths)
#' between vertex pairs.
#'
#' The communicability \eqn{G_{pq}} is a weighted sum of the number of walks
#' from vertex \emph{p} to \emph{q} and is calculated by taking the exponential
#' of the adjacency matrix \emph{A}:
#' \deqn{G_{pq} = \sum_{k=0}^{\infty} \frac{(\mathbf{A}^k)_{pq}}{k!} =
#' (e^{\mathbf{A}})_{pq}}
#' where \eqn{k} is \emph{walk} length.
#'
#' For weighted graphs with \eqn{D = diag(d_i)} a diagonal matrix of vertex
#' strength,
#' \deqn{G_{pq} = (e^{\mathbf{D}^{-1/2} \mathbf{A} \mathbf{D}^{-1/2}})_{pq}}
#'
#' @inheritParams efficiency
#' @export
#'
#' @return A numeric matrix of the communicability
#'
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}
#' @references Estrada, E. and Hatano, N. (2008) Communicability in complex
#'   networks. \emph{Physical Review E}. \bold{77}, 036111.
#'   \doi{10.1103/PhysRevE.77.036111}
#' @references Crofts, J.J. and Higham, D.J. (2009) A weighted communicability
#'   measure applied to complex brain networks. \emph{J. R. Soc. Interface}.
#'   \bold{6}, 411--414. \doi{10.1098/rsif.2008.0484}

communicability <- function(g, weights=NULL) {
  if (!requireNamespace('expm', quietly=TRUE)) stop('Must install the "expm" package.')
  stopifnot(is_igraph(g), is_connected(g))
  if (is.null(weights) && 'weight' %in% edge_attr_names(g)) {
    d <- check_strength(g)
    S <- diag(1 / sqrt(d))
    A <- as.matrix(as_adj(g, names=FALSE, attr='weight'))
  } else {
    S <- diag(vcount(g))
    A <- as_adj(g, names=FALSE, sparse=FALSE)
  }

  C <- expm::expm(S %*% A %*% S)
  return(C)
}

#' Calculate communicability betweenness centrality
#'
#' \code{centr_betw_comm} calculates the \emph{communicability betweenness} of
#' the vertices of a graph. The centrality for vertex \code{r} is
#' \deqn{\omega_r = \frac{1}{C} \sum_p \sum_q \frac{(e^{\mathbf{A}})_{pq} -
#' (e^{\mathbf{A} + \mathbf{E}(r)})_{pq}}{(e^{\mathbf{A}})_{pq}}}
#' where \eqn{C = (n - 1)^2 - (n - 1)} is a normalization factor.
#'
#' @inheritParams efficiency
#' @export
#'
#' @return A numeric vector of the centrality for each vertex
#'
#' @family Centrality functions
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}
#' @references Estrada, E. and Higham, D.J. and Hatano N. (2009) Communicability
#'   betweenness in complex networks. \emph{Physica A}, \bold{388}, 764--774.
#'   \doi{10.1016/j.physa.2008.11.011}

centr_betw_comm <- function(g, A=NULL) {
  stopifnot(is_igraph(g), is_connected(g))
  if (!requireNamespace('expm', quietly=TRUE)) stop('Must install the "expm" package.')
  if (is.null(A)) A <- as_adj(g, names=FALSE, sparse=FALSE)

  C <- expm::expm(A)
  N <- dim(A)[1L]
  n <- (N - 1)^2 - (N - 1)
  Wr <- rep.int(0, N)
  for (i in seq_len(N)) {
    Er <- A
    Er[i, ] <- Er[, i] <- 0
    G <- (C - expm::expm(Er)) / C
    Wr[i] <- (sum(G[-i, -i]) - sum(diag(G[-i, -i]))) / n
  }
  return(Wr)
}

#' Calculate graph vulnerability
#'
#' This function calculates the \emph{vulnerability} of the vertices of a graph.
#' Here, vulnerability is considered to be the proportional drop in global
#' efficiency when a given vertex is removed from the graph. The vulnerability
#' of the graph is considered the maximum across all vertices.
#'
#' @param g An \code{igraph} graph object
#' @param use.parallel Logical indicating whether or not to use \emph{foreach}
#'   (default: \code{TRUE})
#' @param weighted Logical indicating whether weighted efficiency should be
#'   calculated (default: \code{FALSE})
#' @export
#' @importFrom Matrix rowSums
#' @importFrom foreach getDoParRegistered
#' @importFrom doParallel registerDoParallel
#'
#' @return A numeric vector of length equal to the vertex count of \emph{g}
#'
#' @seealso \code{\link{efficiency}}
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}
#' @references Latora, V. and Marchiori, M. (2005) Variability and protection of
#'   infrastructure networks. \emph{Physical Review E}, \bold{71}, 015103.
#'   \doi{10.1103/physreve.71.015103}

vulnerability <- function(g, use.parallel=TRUE, weighted=FALSE) {
  i <- NULL
  stopifnot(is_igraph(g))
  if (isTRUE(weighted)) {
    weights <- NULL
    g.attr <- 'E.global.wt'
    e.attr <- 'weight'
  } else {
    weights <- NA
    g.attr <- 'E.global'
    e.attr <- NULL
    weighted <- NULL
  }
  A <- as_adj(g, sparse=FALSE, names=FALSE, attr=e.attr)
  calc <- g.attr %in% graph_attr_names(g)
  E.global <- if (calc) efficiency(g, 'global', weights) else graph_attr(g, g.attr)

  vuln <- rep.int(0, dim(A)[1L])
  verts <- which(rowSums((A > 0) + 0L) != 0L)
  `%d%` <- `%do%`
  if (isTRUE(use.parallel)) {
    if (!getDoParRegistered()) {
      cl <- makeCluster(getOption('bg.ncpus'))
      registerDoParallel(cl)
    }
    `%d%` <- `%dopar%`
  }
  vuln[verts] <- foreach(i=verts, .combine='c') %d% {
    g.tmp <- graph_from_adjacency_matrix(A[-i, -i, drop=FALSE], mode='undirected', weighted=weighted)
    E.global.tmp <- efficiency(g.tmp, 'global', weights)
    1 - (E.global.tmp / E.global)
  }
  return(vuln)
}

#' Mediation analysis with brain graph measures as mediator variables
#'
#' \code{brainGraph_mediate} performs simple mediation analyses in which a given
#' graph- or vertex-level measure (e.g., \emph{weighted global efficiency}) is
#' the mediator \emph{M}. The outcome (or dependent/response) variable \emph{Y}
#' can be a neuropsychological measure (e.g., \emph{IQ}) or can be a
#' disease-specific metric (e.g., recovery time).
#'
#' This code was adapted closely from \code{\link[mediation]{mediate}} in the
#' \code{mediation} package, and the procedure is exactly the same as theirs
#' (see the references listed below). If you use this function, please cite
#' their work.
#'
#' @note As of \code{brainGraph v2.0.0}, this function has been tested only for
#' a treatment (independent) variable \emph{X} being a \emph{factor} (e.g.,
#' disease group, old vs. young, etc.). If your treatment variable has more
#' than 2 levels, then you must explicitly specify the levels you would like to
#' compare; otherwise, the baseline and first levels are taken to be the
#' control and treatment values, respectively. Be aware that these are \emph{0}
#' indexed; that is, if you have 3 groups and you would like the treatment
#' group to be the 3rd, you should specify as either the group's character
#' string or as \code{treat.value=2}.
#'
#' @note Allowing for treatment-mediator interaction (setting \code{int=TRUE})
#' currently will only work properly if the mediator is a continuous variable;
#' since the mediator is always a graph metric, this should always be the case.
#'
#' @param covars A data table containing covariates of interest. It must include
#'   columns for \code{getOption('bg.subject_id')}, \code{treat},
#'   \code{outcome}, and \code{covar.names}.
#' @param mediator Character string; the name of the graph measure acting as
#'   the \emph{mediating} variable
#' @param treat Character string; the \emph{treatment} variable (e.g.,
#'   \emph{Group})
#' @param outcome Character string; the name of the outcome variable of interest
#' @param covar.names Character vector of the column name(s) in \code{covars} to
#'   include in the models as pre-treatment covariate(s).
#' @param control.value Value of \code{treat} to be used as the control
#'   condition. Default: \code{0}
#' @param treat.value Value of \code{treat} to be used as the treatment
#'   condition. Default: \code{1}
#' @param int Logical indicating whether or not to include an interaction of the
#'   mediator and treatment. Default: \code{FALSE}
#' @param boot Logical indicating whether or not to perform bootstrapping. This
#'   should always be done. Default: \code{TRUE}
#' @param boot.ci.type Character string; which type of CI's to calculate.
#'   Default: \code{perc}
#' @param N Integer; the number of bootstrap samples to run. Default:
#'   \code{1e3}
#' @param conf.level Numeric between 0 and 1; the level of the CI's to
#'   calculate. Default: \code{0.95} for the 2.5 and 97.5 percentiles)
#' @param long Logical indicating whether or not to return all bootstrap
#'   samples. Default: \code{FALSE}
#' @param ... Other arguments passed to \code{\link{brainGraph_GLM_design}}
#'   (e.g., \code{binarize}) (unused in the \code{summary} method)
#' @inheritParams GLM
#' @export
#' @importFrom foreach getDoParRegistered
#' @importFrom doParallel registerDoParallel
#'
#' @return An object of class \code{bg_mediate} with elements:
#'   \item{level}{Either \code{graph} or \code{vertex}.}
#'   \item{removed.subs}{A character vector of Study.ID's removed due to
#'     incomplete data}
#'   \item{X.m, X.y}{Design matrix and numeric array for the model with the
#'     mediator as the outcome variable (\code{X.m}) and for the model with the
#'     mediator as an additional predictor (\code{X.y}), respectively}
#'   \item{y.m, y.y}{Outcome variables for the associated design matrices above.
#'     \code{y.m} will be a matrix of size \emph{# subj. X # regions}}
#'   \item{res.obs}{A \code{data.table} of the observed values of the point
#'     estimates.}
#'   \item{res.ci}{A \code{data.table} of the confidence intervals for the
#'     effect estimates.}
#'   \item{res.p}{A \code{data.table} of the two-sided p-values for the effect
#'     estimates}
#'   \item{boot}{Logical, the \code{boot} argument.}
#'   \item{boot.ci.type}{Character string indicating which type of bootstrap
#'     confidence intervals were calculated.}
#'   \item{res.boot}{A \code{data.table} with \code{N} rows of the bootstrap
#'     results for all effects.}
#'   \item{treat}{Character string of the treatment variable.}
#'   \item{mediator}{Character string of the mediator variable.}
#'   \item{outcome}{Character string of the outcome variable.}
#'   \item{covariates}{Returns \code{NULL}; not used in this package.}
#'   \item{INT}{Logical indicating whether the models included an interaction
#'     between treatment and mediator.}
#'   \item{conf.level}{The confidence level.}
#'   \item{control.value}{The value of the treatment variable used as the
#'     control condition.}
#'   \item{treat.value}{The value of the treatment variable used as the
#'     treatment condition.}
#'   \item{nobs}{Integer; the number of observations in the models.}
#'   \item{sims}{Integer; the number of bootstrap replications.}
#'   \item{covar.names}{The pre-treatment covariate names.}
#'
#' @name Mediation
#' @rdname mediation
#' @family Group analysis functions
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}
#' @references Tingley, D. and Yamamoto, T. and Hirose, K. and Keele, L. and
#'   Imai, K. (2014) mediation: R package for causal mediation analysis.
#'   \emph{Journal of Statistical Software}, \bold{59(5)}, 1--38.
#'   \doi{10.18637/jss.v059.i05}
#' @references Imai, K. and Keele, L. and Yamamoto, T. (2010) Identification
#'   inference, and sensitivity analysis for causal mediation effects.
#'   \emph{Statistical Science}, \bold{25(1)}, 51--71.
#'   \doi{10.1214/10-STS321}
#' @references Imai, K. and Keele, L. and Tingley, D. (2010) A general approach
#'   to causal mediation analysis. \emph{Psychological Methods}, \bold{15(4)},
#'   309--334. \doi{10.1037/a0020761}
#' @references Imai, K. and Keele, L. and Tingley, D. and Yamamoto, T. (2011)
#'   Unpacking the black box of causality: learning about causal mechanisms from
#'   experimental and observational studies. \emph{American Political Science
#'   Review}, \bold{105(4)}, 765--789.
#'   \doi{10.1017/S0003055411000414}
#' @references Imai, K. and Yamamoto, T. (2013) Identification and sensitivity
#'   analysis for multiple causal mechanisms: revisiting evidence from framing
#'   experiments. \emph{Political Analysis}, \bold{21(2)}, 141--171.
#'   \doi{10.1093/pan/mps040}
#' @examples
#' \dontrun{
#' med.EglobWt.FSIQ <- brainGraph_mediate(g[[5]], covars.med, 'E.global.wt',
#'   'Group', 'FSIQ', covar.names=c('age', 'gender'), N=1e4)
#' med.strength.FSIQ <- brainGraph_mediate(g[[5]], covars.med, 'strength',
#'   'Group', 'FSIQ', covar.names=c('age', 'gender'), level='vertex')
#' }

brainGraph_mediate <- function(g.list, covars, mediator, treat,
                               outcome, covar.names, level=c('graph', 'vertex'),
                               control.value=0, treat.value=1, int=FALSE,
                               boot=TRUE, boot.ci.type=c('perc', 'bca'), N=1e3,
                               conf.level=0.95, long=FALSE, ...) {
  region <- NULL
  if (!is.brainGraphList(g.list)) try(g.list <- as_brainGraphList(g.list))
  g.list <- g.list[]

  stopifnot(all(hasName(covars, c(treat, outcome, covar.names))))
  sID <- getOption('bg.subject_id')
  if (!hasName(covars, sID)) covars[, eval(sID) := seq_len(dim(covars)[1L])]
  covars[, eval(sID) := check_sID(get(sID))]
  covars <- droplevels(covars[, c(sID, treat, covar.names, outcome), with=FALSE])
  incomp <- covars[!complete.cases(covars), which=TRUE]
  names(incomp) <- covars[incomp, get(sID)]

  level <- match.arg(level)
  dt.graph <- glm_data_table(g.list, level, mediator)
  DT <- covars[dt.graph, on=sID]
  if (length(incomp) > 0L) DT <- DT[-incomp]

  DT[, eval(treat) := as.factor(get(treat))]
  t.levels <- DT[, levels(get(treat))]
  if (all(c(treat.value, control.value) %in% t.levels)) {
    cat.0 <- control.value
    cat.1 <- treat.value
  } else {
    stopifnot(is.numeric(c(control.value, treat.value)))
    cat.0 <- t.levels[control.value + 1L]
    cat.1 <- t.levels[treat.value + 1L]
  }

  cols <- c(sID, treat, covar.names)
  X.m <- brainGraph_GLM_design(DT[, cols, with=FALSE], ...)
  y.y <- DT[, get(outcome)]
  treatstr <- paste0(treat, cat.1)

  # Different across regions
  DT.m <- melt(DT, id.vars=names(covars), variable.name='region', value.name=mediator)
  regions <- names(dt.graph)[-1L]
  y.m <- as.matrix(DT[, c(sID, regions), with=FALSE], rownames=sID)
  des_args <- list(...)
  if (isTRUE(int)) des_args <- c(des_args, list(int=c(treat, mediator)))
  cols <- append(cols, mediator, after=1L)
  X.y <- lapply(regions, function(r)
                do.call(brainGraph_GLM_design,
                        c(list(covars=DT.m[region == r, cols, with=FALSE]), des_args)))
  names(X.y) <- regions
  attrs <- attributes(X.y[[1L]])[-c(1L, 2L)]
  X.y <- abind::abind(X.y, along=3L)
  attributes(X.y) <- c(attributes(X.y), attrs)

  # Calculate the resampled statistics for each region
  if (!getDoParRegistered()) {
    cl <- makeCluster(getOption('bg.ncpus'))
    registerDoParallel(cl)
  }
  res_boot <- boot_mediate(X.m, y.m, X.y, y.y, mediator, treat, treatstr, int, N)

  res_p <- as.data.table(pvalArray(res_boot, N, dim(y.m)[2L]), keep.rownames='region')
  res_boot <- rbindlist(apply(res_boot, 3L, as.data.table), idcol='region')
  res_obs <- res_boot[, .SD[.N], by=region]
  res_boot <- res_boot[, .SD[-.N], by=region]

  conf.limits <- (1 + c(-1, 1) * conf.level) / 2
  if (isTRUE(boot)) {
    boot.ci.type <- match.arg(boot.ci.type)
    bootFun <- switch(boot.ci.type, perc=fastquant, bca=BC.CI2)
    if (boot.ci.type == 'bca') conf.limits <- qnorm(conf.limits)
    res_ci <- res_boot[, lapply(.SD, bootFun, conf.limits, N), by=region]
  }

  if (isFALSE(long)) res_boot <- NULL
  out <- list(level=level, removed.subs=incomp, X.m=X.m, X.y=X.y, y.m=y.m, y.y=y.y,
              res.obs=res_obs, res.ci=res_ci, res.p=res_p,
              boot=boot, boot.ci.type=boot.ci.type, res.boot=res_boot,
              treat=treat, mediator=mediator, outcome=outcome, covariates=NULL, INT=int,
              conf.level=conf.level, control.value=cat.0, treat.value=cat.1,
              nobs=dim(X.m)[1L], sims=N, covar.names=covar.names)
  out$atlas <- guess_atlas(g.list[[1L]])
  class(out) <- c('bg_mediate', class(out))
  return(out)
}

#' Calculate bootstrapped estimates of direct, indirect, and total effects
#'
#' @param X.m,X.y Design matrices for the mediation variable as outcome
#'   (\code{X.m}) and the mediation as a covariate predicting the specified
#'   outcome variable (\code{X.y})
#' @param y.m,y.y Numeric matrix and vector, respectively, containing the
#'   outcome variable data of the above designs
#' @param treatstr Character string denoting the column of the design matrix for
#'   the treatment variable
#' @inheritParams brainGraph_mediate
#' @noRd

boot_mediate <- function(X.m, y.m, X.y, y.y, mediator, treat, treatstr, int, N) {
  b <- NULL

  regions <- dimnames(y.m)[[2L]]
  dimXm <- dim(X.m)
  n <- dimXm[1L]
  pm <- dimXm[2L]
  dfRm <- n - pm
  py <- dim(X.y)[2L]
  ny <- dim(y.m)[2L]

  # Randomization/resampling matrix
  index <- t(replicate(N, sample.int(n, replace=TRUE)))
  index <- rbind(index, seq_len(n))

  # Don't use "drop" when it isn't needed (i.e., vertex-level). It is much slower.
  if (ny == 1L) {
    Xyperm <- function(X, porder) X[porder, , , drop=FALSE]
    Xyfun <- f_beta_3d_g
  } else {
    Xyperm <- function(X, porder) X[porder, , ]
    Xyfun <- f_beta_3d
  }
  # For faster calculation of the "Q" matrix for both "fits.m" and "beta.y"
  diagIndsY <- diag(1, n, py)
  diagIndsM <- diag(1, n, pm)

  # These are used in the loop calculating "effects.tmp"
  # The last 2 columns are (1 - tt[3]) and (1 - tt[4])
  ttMat <- matrix(c(1, 1, 1, 0, 0, 1,
                    0, 0, 1, 0, 0, 1,
                    1, 0, 1, 1, 0, 0,
                    1, 0, 0, 0, 1, 1),
                   nrow=6L)
  effects.tmp <- array(NA, dim=c(n, 4L, ny))

  vnames <- dimnames(X.y)[[2L]]
  treatstrOther <- setdiff(grep(treat, dimnames(X.m)[[2L]], value=TRUE), treatstr)
  Xcols <- c(mediator, treatstr)
  if (isTRUE(int)) {
    treatintstr <- paste0(treatstr, ':', mediator)
    Xcols <- c(Xcols, treatintstr)
    ecols <- 1L:4L
  } else {
    ecols <- c(1L, 3L)  # "d1" == "d0", and "z1" == "z0", so don't recalculate
  }
  bcols <- which(vnames %in% Xcols)

  # When "int=FALSE", reduces to the "Baron & Kenny" approach (for "d1", at least)
  fun_effects <- function(Xy.diff, beta.y, Xcols, bcols, regions) {
    X <- Xy.diff[, Xcols, ]
    b <- beta.y[bcols, ]
    vapply(regions, function(r) X[, , r] %*% b[, r], numeric(n))
  }

  # Loop through the resamples
  #-----------------------------------------------------------------------------
  res <- foreach(b=seq_len(N + 1L), .combine=rbind) %dopar% {
    neworder <- index[b, ]

    # Mediator predictions
    fits.m <- f_beta_m(X.m[neworder, ], y.m[neworder, ], diagIndsM, n, pm, ny, dfRm)
    error <- vapply(fits.m$sigma, function(r) rnorm(n, mean=0, sd=r), numeric(n))

    X.m.t <- X.m.c <- X.m
    X.m.t[, treatstr] <- 1
    X.m.c[, treatstr] <- 0
    X.m.t[, treatstrOther] <- X.m.c[, treatstrOther] <- 0
    PredictM1 <- X.m.t %*% fits.m$coefficients + error
    PredictM0 <- X.m.c %*% fits.m$coefficients + error

    # Outcome predictions
    beta.y <- Xyfun(Xyperm(X.y, neworder), y.y[neworder], regions, diagIndsY, n, py, ny)

    # e1: Mediation(1); e2: Mediation(0); e3: Direct(1); e4: Direct(0)
    # AKA: d1, d0, z1, z0 (respectively) for each "sim")
    for (e in ecols) {
      X.y.t <- X.y.c <- X.y
      X.y.t[, treatstr, ] <- ttMat[1L, e]
      X.y.c[, treatstr, ] <- ttMat[2L, e]
      X.y.t[, mediator, ] <- PredictM1 * ttMat[3L, e] + PredictM0 * ttMat[5L, e]  #PredictMt
      X.y.c[, mediator, ] <- PredictM1 * ttMat[4L, e] + PredictM0 * ttMat[6L, e]  #PredictMc
      if (isTRUE(int)) {
        #X.y.t[, treatstrOther, ] <- X.y.c[, treatstrOther, ] <- 0
        X.y.t[, treatintstr, ] <- X.y.t[, treatstr, ] * X.y.t[, mediator, ]
        X.y.c[, treatintstr, ] <- X.y.c[, treatstr, ] * X.y.c[, mediator, ]
      }
      Xy.diff <- X.y.t - X.y.c
      effects.tmp[, e, ] <- fun_effects(Xy.diff, beta.y, Xcols, bcols, regions)
    }

    return(colMeans(effects.tmp))
  }
  res <- array(res, dim=c(4L, N + 1L, ny))
  if (isFALSE(int)) res[c(2L, 4L), , ] <- res[ecols, , ]
  res2 <- array(dim=c(10L, N + 1L, ny))
  res2[1L:4L, , ] <- res
  res2 <- aperm(res2, c(2L, 1L, 3L))
  dimnames(res2)[2L:3L] <- list(c('d1', 'd0', 'z1', 'z0', 'tau', 'n0', 'n1', 'd.avg', 'z.avg', 'n.avg'), regions)
  res2[, 'd.avg', ] <- (res2[, 'd1', ] + res2[, 'd0', ]) / 2
  res2[, 'z.avg', ] <- (res2[, 'z1', ] + res2[, 'z0', ]) / 2
  res2[, 'tau', ] <- res2[, 'd.avg', ] + res2[, 'z.avg', ]    # i.e., (d1 + d0 + z1 + z0) / 2
  res2[, 'n0', ] <- res2[, 'd0', ] / res2[, 'tau', ]
  res2[, 'n1', ] <- res2[, 'd1', ] / res2[, 'tau', ]
  res2[, 'n.avg', ] <- (res2[, 'n1', ] + res2[, 'n0', ]) / 2
  return(res2)
}

# Only calculate coefficients and sigma for "y.m ~ X.m"
f_beta_m <- function(X, Y, diagMat, n, p, ny, dfR) {
  QR <- qr.default(X, LAPACK=TRUE)
  Q <- qr_Q2(QR, diagMat, n, p)
  R <- qr_R2(QR, p)
  beta <- backsolve(R, crossprod(Q, Y), p)
  beta[QR$pivot, ] <- beta
  ehat <- Y - X %*% beta
  s <- if (ny == 1L) sum(ehat^2) else .colSums(ehat^2, n, ny)
  list(coefficients=beta, sigma=sqrt(s / dfR))
}

# Functions to calculate coefficients for "y.y ~ X.y"; i.e., multiple designs, 1 outcome
f_beta_3d <- function(X, Y, regions, diagMat, n, p, ny) {
  QR <- qr(X, LAPACK=TRUE)
  Q <- lapply(QR, qr_Q2, diagMat, n, p)
  R <- lapply(QR, qr_R2, p)
  beta <- matrix(NaN, p, ny, dimnames=list(NULL, regions))
  for (r in regions) {
    beta[QR[[r]]$pivot, r] <- backsolve(R[[r]], crossprod(Q[[r]], Y), p)
  }
  beta
}

# If "level='graph'", this avoids using "drop=FALSE" above
f_beta_3d_g <- function(X, Y, regions, diagMat, n, p, ny=1L) {
  QR <- qr.default(X[, , 1L], LAPACK=TRUE)
  Q <- qr_Q2(QR, diagMat, n, p)
  R <- qr_R2(QR, p)
  beta <- backsolve(R, crossprod(Q, Y), p)
  beta[QR$pivot, ] <- beta
  dimnames(beta) <- list(NULL, 'graph')
  beta
}

#' Calculate P-values in an array for mediation analysis
#'
#' @param res_boot Numeric array with \eqn{(N+1) \times 10 \times N_y} dimensions,
#'   where \eqn{N} is the number of resamples and \eqn{N_y} is the number of regions
#' @noRd

pvalArray <- function(res_boot, N=dim(res_boot)[1L] - 1L, ny=dim(res_boot)[3L]) {
  seqN <- seq_len(N)
  gt0 <- colSums(res_boot[seqN, , ] > 0)
  lt0 <- N - gt0  # Only differs if there are "x == 0" exactly
  #lt0 <- colSums(res_boot[seqN, , ] < 0)
  pMat <- 2 * pmin.int(gt0, lt0) / N
  zeros <- which(res_boot[N + 1L, , ] == 0)
  if (length(zeros) > 0L) pMat[zeros] <- 1
  dim(pMat) <- c(10L, ny)
  dimnames(pMat) <- dimnames(res_boot)[2L:3L]
  t(pMat)
}

#' Bias-corrected and accelerated confidence intervals
#'
#' @param theta Numeric vector with the bootstrap-resampled statistics
#' @param quants Numeric vector with 2 elements: the confidence limits; i.e.,
#'   \code{qnorm(c((1 - alpha) / 2, (1 + alpha) / 2))}
#' @param N Integer; the number of resamples
#' @noRd

BC.CI2 <- function(theta, quants, N) {
  avg <- sum(theta) / N
  z.inv <- sum(theta < avg) / N
  z <- qnorm(z.inv)
  U <- (N - 1L) * (avg - theta)
  U2 <- U^2
  top <- sum(U * U2)
  under <- 6 * (sum(U2))^1.5
  a <- top / under
  lower.upper <-  pnorm(z + (z + quants) / (1 - a * (z + quants)))
  fastquant(theta, lower.upper, N)
}

#' A pared down "quantile" function with no argument checking
#'
#' This function uses the default \code{type=7} from
#' \code{\link[stats]{quantile}}.
#'
#' @param x Numeric vector of bootstrap-resampled statistics
#' @param probs Numeric vector of probabilities
#' @param N Integer; the number of resamples
#' @noRd

fastquant <- function(x, probs, N) {
  index <- 1 + (N - 1) * probs
  lo <- floor(index)
  hi <- ceiling(index)
  x <- sort(x, partial=unique(c(lo, hi)))
  qs <- x[lo]
  i <- which(index > lo & x[hi] != qs)
  h <- (index - lo)[i]
  qs[i] <- (1 - h) * qs[i] + h * x[hi[i]]
  qs
}

################################################################################
# S3 METHODS FOR "bg_mediate"
################################################################################

#' Print a summary from a brainGraph mediation analysis
#'
#' @param object A \code{bg_mediate} object
#' @param mediate Logical indicating whether or not to use the \code{summary}
#'   method from \code{\link[mediation]{mediate}} (default: \code{FALSE}). If
#'   \code{TRUE}, only a single region can be printed.
#' @param region Character string specifying which region's results to
#'   summarize; only relevant if \code{level='vertex'} (default: \code{NULL})
#' @export
#' @rdname mediation

summary.bg_mediate <- function(object, mediate=FALSE, region=NULL, digits=max(3L, getOption('digits') - 2L), ...) {
  stopifnot(inherits(object, 'bg_mediate'))
  Mediator <- treat <- Outcome <- NULL

  DT.obs <- copy(object$res.obs)
  DT.ci <- copy(object$res.ci)
  DT.p <- copy(object$res.p)
  setnames(DT.ci, c('region', paste0(names(DT.ci)[-1L], '.ci')))
  setnames(DT.p, c('region', paste0(names(DT.p)[-1L], '.p')))
  DT.all <- merge(merge(DT.obs, DT.p, by='region'), DT.ci, by='region')
  DT.all[, c('Mediator', 'treat', 'Outcome') := with(object, mediator, treat, outcome)]

  change <- matrix(c('d0', 'd0.p', 'z0', 'z0.p', 'tau', 'tau.p', 'n0', 'n0.p',
                     'b0.acme', 'p0.acme', 'b0.ade', 'p0.ade', 'b.tot', 'p.tot', 'b0.prop', 'p0.prop'),
                   ncol=2L)
  setnames(DT.all, change[, 1L], change[, 2L])
  change_ci <- change[seq.int(1L, 7L, 2L), ]
  change_ci <- cbind(paste0(change_ci[, 1L], '.ci'),
                     sub('[bdnz]([01]?)\\.', 'ci.low\\1.', change_ci[, 2L]))
  change_ci <- cbind(change_ci, sub('low', 'high', change_ci[, 2L]))
  DT.all[, eval(change_ci[, 2L]) := lapply(.SD, function(x) x[1L]), by=region, .SDcols=change_ci[, 1L]]
  DT.all[, eval(change_ci[, 3L]) := lapply(.SD, function(x) x[2L]), by=region, .SDcols=change_ci[, 1L]]

  mainnames <- c('Mediator', 'treat', 'Outcome', 'region')
  acme <- c('b0.acme', 'ci.low0.acme', 'ci.high0.acme', 'p0.acme')
  total <- sub('0.acme', '.tot', acme)
  # Different behavior if mediator-treatment interaction
  if (isTRUE(object$INT)) {
    change1 <- sub('0', '1', change)[-c(5L, 6L), ]
    change1 <- rbind(change1, sub('1', '.avg', change1))
    setnames(DT.all, change1[, 1L], change1[, 2L])
    change_ci1 <- sub('0', '1', change_ci)[-3L, ]
    change_ci1 <- rbind(change_ci1, sub('1', '.avg', change_ci1))
    DT.all[, eval(change_ci1[, 2L]) := lapply(.SD, function(x) x[1L]), by=region, .SDcols=change_ci1[, 1L]]
    DT.all[, eval(change_ci1[, 3L]) := lapply(.SD, function(x) x[2L]), by=region, .SDcols=change_ci1[, 1L]]
    acme <- c(acme, sub('0', '1', acme), sub('0', '.avg', acme))

  } else {
    DT.all[, grep('1|avg', names(DT.all)) := NULL]
  }
  DT.all[, grep('.*.ci', names(DT.all)) := NULL]
  setcolorder(DT.all, c(mainnames, acme, sub('acme', 'ade', acme), total, sub('acme', 'prop', acme)))
  DT.all <- DT.all[, .SD[1L], keyby=region]
  DT.all[, region := as.factor(region)]

  object <- c(object, list(DT.sum=DT.all, region=region, digits=digits, mediate=mediate))
  class(object) <- c('summary.bg_mediate', class(object))
  return(object)
}

#' @aliases summary.bg_mediate
#' @method print summary.bg_mediate
#' @export

print.summary.bg_mediate <- function(x, ...) {
  region <- NULL
  width <- getOption('width') / 4
  dashes <- rep.int('-', width)
  print_title_summary(simpleCap(x$level), '-level mediation results')
  cat('# of observations: ', x$nobs, '\n')

  # Print a table of the model variables
  message('\n', 'Variables', '\n', dashes)
  df <- data.frame(A=c('  Mediator:', ' Treatment:', '    Control condition:',
                       '  Treatment condition:', '   Outcome:'),
                   B=c(x$mediator, x$treat, x$control.value, x$treat.value, x$outcome))
  cov.df <- data.frame(A=c('', 'Covariates:', rep.int('', length(x$covar.names) - 1L)),
                       B=c('', x$covar.names))
  df <- rbind(df, cov.df)
  dimnames(df)[[2L]] <- rep.int('', 2L)
  print(df, right=FALSE, row.names=FALSE)
  cat('\nTreatment-mediator interaction? ', x$INT, '\n\n')

  print_subs_summary(x)

  if (isTRUE(x$boot)) {
    low <- (1 - x$conf.level) / 2
    high <- 1 - low
    message('\n', 'Bootstrapping', '\n', dashes)
    ci <- switch(x$boot.ci.type,
                 perc='Percentile bootstrap',
                 bca='Bias-corrected accelerated')
    cat('Bootstrap CI type: ', ci, '\n')
    cat('# of bootstrap replicates: ', prettyNum(x$sims, ','), '\n')
    cat('Bootstrap CI level: ',
        sprintf('[%s]', paste(paste0(100 * c(low, high), '%'), collapse=' ')), '\n\n')
  }

  if (isTRUE(x$mediate)) {
    if (!requireNamespace('mediation', quietly=TRUE)) stop('Must install the "mediation" package.')
    region <- if (is.null(x$region)) x$DT.sum[, levels(region)[1L]] else x$region
    message('Mediation summary for: ', region, '\n', dashes)
    print(summary(bg_to_mediate(x, region)))
  } else {
    if (is.null(x$region)) {
      regions <- x$DT.sum[, levels(region)]
    } else {
      regions <- x$region
    }
    message('Mediation statistics', '\n', dashes)
    print(x$DT.sum[region %in% regions])
  }
  invisible(x)
}

#' Convert brainGraph results to mediate object
#'
#' \code{\link{bg_to_mediate}} converts the results into an object of class
#' \code{\link[mediation]{mediate}}. In \code{brainGraph}, it is only used for
#' the \code{\link[mediation]{summary.mediate}} method, but you can similarly
#' use its output for the \code{\link[mediation]{plot.mediate}} method.
#'
#' @param x Object output from \code{\link{brainGraph_mediate}}
#' @export
#'
#' @return \code{bg_to_mediate} returns an object of class \code{mediate}
#' @seealso \code{\link[mediation]{mediate}}
#' @rdname mediation

bg_to_mediate <- function(x, region=NULL) {
  if (!inherits(x, c('bg_mediate', 'summary.bg_mediate'))) {
    stop('Use only with \'bg_mediate\' objects!')
  }
  if (x$level == 'graph') {
    res.obs <- x$res.obs
    res.ci <- x$res.ci
    res.p <- x$res.p
  } else {
    regions <- x$res.obs[, unique(region)]
    if (is.null(region)) {
      i <- 1L
    } else {
      i <- which(region == regions)
    }
    res.obs <- x$res.obs[region == regions[i]]
    res.ci <- x$res.ci[region == regions[i]]
    res.p <- x$res.p[region == regions[i]]
  }
  # My object is missing: call=cl, model.y=model.y, model.m=model.m)
  out <- list(d0=res.obs$d0, d1=res.obs$d1, d0.ci=res.ci$d0, d1.ci=res.ci$d1, d0.p=res.p$d0, d1.p=res.p$d1,
              z0=res.obs$z0, z1=res.obs$z1, z0.ci=res.ci$z0, z1.ci=res.ci$z1, z0.p=res.p$z0, z1.p=res.p$z1,
              n0=res.obs$n0, n1=res.obs$n1, n0.ci=res.ci$n0, n1.ci=res.ci$n1, n0.p=res.p$n0, n1.p=res.p$n1,
              tau.coef=res.obs$tau, tau.ci=res.ci$tau, tau.p=res.p$tau,
              d.avg=res.obs$d.avg, d.avg.ci=res.ci$d.avg, d.avg.p=res.p$d.avg,
              z.avg=res.obs$z.avg, z.avg.ci=res.ci$z.avg, z.avg.p=res.p$z.avg,
              n.avg=res.obs$n.avg, n.avg.ci=res.ci$n.avg, n.avg.p=res.p$n.avg,
              boot=x$boot, boot.ci.type=x$boot.ci.type, treat=x$treat, mediator=x$mediator, covariates=x$covariates,
              INT=x$INT, control.value=x$control.value, treat.value=x$treat.value, nobs=x$nobs, sims=x$sims,
              robustSE=FALSE, cluster=NULL)
  class(out) <- 'mediate'
  return(out)
}

#' Count number of edges of a brain graph
#'
#' \code{count_homologous} counts the number of edges between homologous regions
#' in a brain graph (e.g. between L and R superior frontal).
#'
#' @param g A \code{brainGraph} graph object
#' @export
#' @return \code{count_homologous} - a named vector of the edge ID's connecting
#'   homologous regions
#'
#' @name Count Edges
#' @rdname count_edges
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}

count_homologous <- function(g) {
  V1 <- V2 <- NULL
  stopifnot(is.brainGraph(g))
  if (any(grepl('(\\.|_)L[0-9]*', V(g)$name)) && !grepl('destrieux', g$atlas)) {
    lh <- '(\\.|_)L[0-9]*'
    rh <- '(\\.|_)R[0-9]*'
  } else {
    lh <- '^l'
    rh <- '^r'
  }

  dt <- as.data.table(as_edgelist(g))
  eids <- dt[, which(gsub(lh, '', V1) == gsub(rh, '', V2))]
  names(eids) <- dt[eids, V1]
  return(eids)
}

#' Count number of inter-group edges
#'
#' \code{count_inter} counts the number of edges between and within all vertices
#' in one group (e.g. \emph{lobe}, \emph{hemi}, \emph{network}, etc.).
#'
#' @param group Character string specifying which grouping to calculate edge
#'   counts for. Default: \code{'lobe'}
#' @export
#' @return \code{count_inter} - a \code{data.table} of total, intra-, and
#'   inter-group edge counts
#'
#' @rdname count_edges
#' @examples
#' \dontrun{
#' g1.lobecounts <- count_inter(g[[1]][[N]], 'lobe')
#' }

count_inter <- function(g, group=c('lobe', 'hemi', 'network', 'class',
                                   'gyrus', 'Yeo_7network', 'Yeo_17network',
                                   'area', 'Brodmann')) {
  total <- intra <- inter <- NULL
  group <- match.arg(group)
  stopifnot(is.brainGraph(g), group %in% vertex_attr_names(g))

  group.names <- get(g$atlas)[, levels(get(group))]
  A <- as_adj(g, names=FALSE, sparse=FALSE)
  Nm <- length(group.names)
  mat <- matrix(0, Nm, Nm)
  vattrs <- vertex_attr(g, group)
  matches <- lapply(group.names, function(x) which(vattrs == x))
  for (i in seq_len(Nm)) {
    for (j in seq.int(i, Nm)) {
      mat[i, j] <- sum(A[matches[[i]], matches[[j]]])
    }
  }
  mat[lower.tri(mat)] <- t(mat)[lower.tri(mat)]
  intra <- diag(mat) <- diag(mat) / 2
  rownames(mat) <- colnames(mat) <- group.names
  DT <- data.table(group=group.names, intra=intra, inter=rowSums(mat)-intra)
  DT[, total := intra + inter]
  setnames(DT, 'group', group)
  return(list(mat=mat, DT=DT))
}

#' Set graph, vertex, and edge attributes common in MRI analyses
#'
#' \code{set_brainGraph_attr} is a convenience function that sets a number of
#' graph, vertex, and edge attributes for a given graph object. Specifically, it
#' calculates measures that are common in MRI analyses of brain networks.
#'
#' Including \code{type='random'} in the function call will reduce the number of
#' attributes calculated. It will only add graph-level attributes for:
#' clustering coefficient, characteristic path length, rich club coefficient,
#' global efficiency, and modularity.
#'
#' @section Negative edge weights:
#' If there are any negative edge weights in the graph, several of the
#' distance-based metrics will \emph{not} be calculated, because they can throw
#' errors which is undesirable when processing a large dataset. The metrics are:
#' local and nodal efficiency, diameter, characteristic path length, and
#' hubness.
#'
#' @section Transforming edge weights:
#' For distance-based measures, it is important to transform the edge weights so
#' that the \emph{strongest} connections are re-mapped to having the
#' \emph{lowest} weights. Then you may calculate e.g., the \emph{shortest path
#' length} which will include the strongest connections.
#'
#' \code{xfm.type} allows you to choose from 5 options for transforming edge
#' weights when calculating distance-based metrics (e.g., shortest paths). There
#' is no \dQuote{best-practice} for choosing one over the other, but the reciprocal is
#' probably most common.
#' \describe{
#'   \item{\code{1/w}}{reciprocal (default)}
#'   \item{\code{-log(w)}}{the negative (natural) logarithm}
#'   \item{\code{1-w}}{subtract weights from 1}
#'   \item{\code{-log10(w/max(w))}}{negative (base-10) log of normalized
#'   weights}
#'   \item{\code{-log10(w/max(w)+1)}}{same as above, but add 1 before taking
#'     the log}
#' }
#'
#' To transform the weights back to original values, specify \code{invert=TRUE}.
#'
#' @section Community detection:
#' \code{clust.method} allows you to choose from any of the clustering
#' (community detection) functions available in \code{igraph}. These functions
#' begin with \code{cluster_}; the function argument should not include this
#' leading character string. There are a few possibilities, depending on the
#' value and the type of input graph:
#' \enumerate{
#'   \item By default, \code{louvain} is used, calling
#'     \code{\link[igraph]{cluster_louvain}}
#'   \item Uses \code{spinglass} if there are any negative edges and/or the
#'     selected method is \code{spinglass}
#'   \item Uses \code{walktrap} if there are any negative edge weights and any
#'     other method (besides \code{spinglass}) is selected
#'   \item Automatically transforms the edge weights if \code{edge_betweenness}
#'     is selected and the graph is weighted, because the algorithm considers
#'     edges as \emph{distances}
#' }
#'
#' @param g A graph object
#' @param use.parallel Logical indicating whether to use \emph{foreach}.
#'   Default: \code{TRUE}
#' @param A Numeric matrix; the (weighted) adjacency matrix, which can be used
#'   for faster calculation of local efficiency. Default: \code{NULL}
#' @param clust.method Character string indicating which method to use for
#'   community detection. Default: \code{'louvain'}
#' @inheritParams Creating_Graphs
#' @export
#'
#' @return A graph object with the following attributes:
#'   \item{Graph-level}{Density, connected component sizes, diameter, # of
#'     triangles, transitivity, average path length, assortativity, global &
#'     local efficiency, modularity, vulnerability, hub score, rich-club
#'     coefficient, # of hubs, edge asymmetry}
#'   \item{Vertex-level}{Degree, strength; betweenness, eigenvector, and
#'     leverage centralities; hubs; transitivity (local); k-core, s-core; local
#'     & nodal efficiency; color (community, lobe, component); membership
#'     (community, lobe, component); gateway and participation coefficients,
#'     within-module degree z-score; vulnerability; and coordinates (x, y, and
#'     z)}
#'   \item{Edge-level}{Color (community, lobe, component), edge betweenness,
#'     Euclidean distance (in mm), weight (if weighted)}
#'
#' @seealso \code{\link[igraph]{components}}, \code{\link[igraph]{diameter}},
#' \code{\link[igraph]{centr_betw}}, \code{\link[igraph]{betweenness}},
#' \code{\link[igraph]{centr_eigen}}, \code{\link[igraph]{transitivity}},
#' \code{\link[igraph]{distances}}, \code{\link[igraph]{assortativity}},
#' \code{\link[igraph]{coreness}}, \code{\link[igraph]{communities}},
#' \code{\link[igraph]{knn}}
#'
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}
#' @name Attributes
#' @rdname attributes

set_brainGraph_attr <- function(g, type=c('observed', 'random'),
                                use.parallel=TRUE, A=NULL,
                                xfm.type=c('1/w', '-log(w)', '1-w',
                                           '-log10(w/max(w))',
                                           '-log10(w/max(w)+1)'),
                                clust.method='louvain') {
  if (!is_igraph(g) && !is.brainGraph(g)) {
    stop('Input graph must have class either "brainGraph" or "igraph"')
  }
  V(g)$degree <- degree(g)
  g$Cp <- transitivity(g, type='localaverage')
  g$Lp <- mean_distance(g)
  g$rich <- rich_club_all(g)
  g$E.global <- efficiency(g, 'global', weights=NA)

  # Handle different cases for different community detection methods
  clust.funs <- ls('package:igraph', pattern='^cluster_')
  clust.funs <- substr(clust.funs, 9L, nchar(clust.funs))
  if (!clust.method %in% clust.funs) {
    stop('Invalid clustering method! You must choose from the following:\n',
         paste(clust.funs, collapse='\n'))
  } else if (clust.method == 'spinglass' && !is_connected(g)) {
    warning('Invalid clustering method for an unconnected graph; using "louvain".')
    clust.method <- 'louvain'
  }
  if (clust.method %in% c('edge_betweenness', 'fast_greedy', 'walktrap')) {
    comm <- eval(parse(text=paste0('cluster_', clust.method, '(g, weights=NULL)')))
  } else if (clust.method == 'infomap') {
    comm <- cluster_infomap(g, e.weights=NA)
  } else {
    comm <- eval(parse(text=paste0('cluster_', clust.method, '(g, weights=NA)')))
  }
  g$clust.method <- clust.method
  g$mod <- max(comm$modularity)

  type <- match.arg(type)
  if (type == 'observed') {
    n <- vcount(g)
    # Graph-level attributes
    #---------------------------------------------------------------------------
    g$density <- graph.density(g)

    # Connected components et al.
    clusts <- components(g)
    comps <- rev(table(clusts$csize))
    x <- clusts$membership
    V(g)$comp <- match(x, order(table(x), decreasing=TRUE))
    if (length(unique(x)) < n) g <- set_graph_colors(g, 'color.comp', V(g)$comp)
    g$conn.comp <- data.frame(size=as.integer(names(comps)), number=as.integer(comps))
    g$max.comp <- g$conn.comp[1L, 1L]
    g$num.tri <- sum(count_triangles(g)) / 3
    g$diameter <- diameter(g, weights=NA)
    g$transitivity <- transitivity(g)
    g$assort <- assortativity_degree(g)

    if (is_weighted(g)) {
      V(g)$strength <- strength(g)
      g$strength <- mean(V(g)$strength)
      V(g)$knn.wt <- knn(g)$knn
      V(g)$s.core <- s_core(g, A)
      g$rich.wt <- rich_club_all(g, weighted=TRUE, A=A)
      V(g)$transitivity.wt <- transitivity(g, type='weighted')

      # Avoid errors when there are negative weights
      if (any(E(g)$weight < 0)) {
        warning('Negative weights present. Skipping distance-based functions.')
        if (!clust.method %in% c('spinglass', 'walktrap')) {
          warning('Invalid clustering method for negative edge weights; using "walktrap".')
          clust.method <- 'walktrap'
        }
      } else {
        # Need to convert weights for distance measures
        xfm.type <- match.arg(xfm.type)
        g <- xfm.weights(g, xfm.type)
        Adist <- as_adj(g, names=FALSE, sparse=FALSE, attr='weight')
        D <- distances(g)
        V(g)$E.local.wt <- efficiency(g, 'local', use.parallel=use.parallel, A=Adist)
        g$E.local.wt <- mean(V(g)$E.local.wt)
        V(g)$E.nodal.wt <- efficiency(g, 'nodal', D=D)
        g$E.global.wt <- mean(V(g)$E.nodal.wt)
        g$diameter.wt <- diameter(g)
        V(g)$Lp.wt <- mean_distance_wt(g, 'vertex', D=D)
        g$Lp.wt <- mean_distance_wt(g, 'graph', D=D)
        if (clust.method == 'edge_betweenness') comm.wt <- cluster_edge_betweenness(g)

        # Convert back to connection strength
        g <- xfm.weights(g, xfm.type, invert=TRUE)
        V(g)$hubs.wt <- hubness(g)
        g$num.hubs.wt <- sum(V(g)$hubs.wt >= 2)
      }

      g$clust.method.wt <- clust.method
      if (clust.method != 'edge_betweenness') {
        comm.wt <- eval(parse(text=paste0('cluster_', clust.method, '(g)')))
      }
      g$mod.wt <- max(comm.wt$modularity)
      x <- comm.wt$membership
      V(g)$comm.wt <- match(x, order(table(x), decreasing=TRUE))
      if (length(unique(x)) < n) g <- set_graph_colors(g, 'color.comm.wt', V(g)$comm.wt)
      V(g)$GC.wt <- gateway_coeff(g, V(g)$comm.wt, A=A, weighted=TRUE)
      V(g)$PC.wt <- part_coeff(g, V(g)$comm.wt, A=A, weighted=TRUE)
      V(g)$z.score.wt <- within_module_deg_z_score(g, V(g)$comm.wt, A=A, weighted=TRUE)
    }

    if (is_directed(g)) {
      hubs <- hub_score(g)
      g$hub.score <- hubs$value
      authorities <- authority_score(g)
      g$authority.score <- authorities$value
      V(g)$hub.score <- hubs$vector
      V(g)$authority.score <- authorities$vector
    }

    # Attributes requiring an atlas
    #-----------------------------------------------------------------------------
    if (!is.null(A) && !is_binary(A)) A[A != 0] <- 1
    if (!is.null(g$atlas)) {
      g$assort.lobe <- assortativity_nominal(g, as.integer(factor(V(g)$lobe)))
      g$assort.lobe.hemi <- assortativity_nominal(g, V(g)$lobe.hemi)

      g$asymm <- edge_asymmetry(g, A=A)$asymm
      V(g)$asymm <- edge_asymmetry(g, 'vertex', A=A)$asymm

      E(g)$dist <- edge_spatial_dist(g)
      g$spatial.dist <- mean(E(g)$dist)
      V(g)$dist <- vertex_spatial_dist(g)
      V(g)$dist.strength <- V(g)$dist * V(g)$degree

      vattrs <- vertex_attr_names(g)
      cols <- c('class', 'network', 'area', 'gyrus', 'Yeo_7network', 'Yeo_17network')
      doAssort <- cols[cols %in% vattrs]
      for (col in doAssort) {
        g <- set_graph_attr(g, paste0('assort.', col),
                            assortativity_nominal(g, factor(vertex_attr(g, col))))
      }
    }

    D <- distances(g, weights=NA)
    V(g)$knn <- knn(g, weights=NA)$knn
    V(g)$Lp <- mean_distance_wt(g, 'vertex', weights=NA, D=D)

    E(g)$btwn <- edge.betweenness(g)
    V(g)$btwn.cent <- centr_betw(g)$res
    V(g)$hubs <- hubness(g, weights=NA)
    g$num.hubs <- sum(V(g)$hubs >= 2)
    V(g)$ev.cent <- centr_eigen(g)$vector
    V(g)$lev.cent <- centr_lev(g, A=A)
    V(g)$k.core <- coreness(g)
    V(g)$transitivity <- transitivity(g, type='local', isolates='zero')
    V(g)$E.local <- efficiency(g, type='local', weights=NA, use.parallel=use.parallel, A=A)
    V(g)$E.nodal <- efficiency(g, type='nodal', weights=NA, D=D)
    g$E.local <- mean(V(g)$E.local)
    V(g)$vulnerability <- vulnerability(g, use.parallel=use.parallel)
    g$vulnerability <- max(V(g)$vulnerability)
    V(g)$eccentricity <- eccentricity(g)

    # Community stuff
    x <- comm$membership
    V(g)$comm <- match(x, order(table(x), decreasing=TRUE))
    if (length(unique(x)) < n) g <- set_graph_colors(g, 'color.comm', V(g)$comm)
    V(g)$circle.layout.comm <- order(V(g)$comm, V(g)$degree)
    V(g)$GC <- gateway_coeff(g, V(g)$comm, A=A)
    V(g)$PC <- part_coeff(g, V(g)$comm, A=A)
    V(g)$z.score <- within_module_deg_z_score(g, V(g)$comm, A=A)
  }

  return(g)
}

#' Delete all attributes of a graph
#'
#' Deletes all graph-, vertex-, and edge-level attributes of an \code{igraph}
#' graph object.
#'
#' @param g An \code{igraph} graph object
#' @param keep.names Logical indicating whether to keep the \code{name} vertex
#'   attribute (default: \code{FALSE})
#'
#' @keywords internal
#' @return An \code{igraph} graph object
#' @seealso \code{\link[igraph]{delete_graph_attr},
#'   \link[igraph]{delete_vertex_attr}, \link[igraph]{delete_edge_attr}}
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}

delete_all_attr <- function(g, keep.names=FALSE) {
  for (att in graph_attr_names(g)) g <- delete_graph_attr(g, att)
  for (att in edge_attr_names(g)) g <- delete_edge_attr(g, att)
  vattrs <- vertex_attr_names(g)
  if (isTRUE(keep.names)) vattrs <- setdiff(vattrs, 'name')
  for (att in vattrs) g <- delete_vertex_attr(g, att)

  return(g)
}

#' Color graph vertices and edges
#'
#' \code{set_graph_colors} takes an integer vector representing membership of
#' some grouping (e.g., a community or connected component) and creates a
#' character vector of colors for each grouping. Isolated vertices will be
#' colored \emph{gray}. Edges are assigned the same color if connected to
#' vertices in the same group, and assigned \emph{gray} otherwise.
#'
#' @param g An \code{igraph} graph object
#' @param name Character string of the name of the attribute to add
#' @param memb An integer vector representing membership of e.g. a community
#'
#' @return The same graph with additional vertex and edge attributes
#' @keywords internal

set_graph_colors <- function(g, name, memb) {
  stopifnot(length(memb) == vcount(g))
  colname <- strsplit(name, '.', fixed=TRUE)[[1L]][2L]
  if (colname %in% c('lobe', 'class', 'network', 'area', 'gyrus', 'Yeo_7network', 'Yeo_17network')) {
    atlas.dt <- get(g$atlas)
    memb <- atlas.dt[name %in% V(g)$name, as.integer(get(colname))] # Preserve order
  } else {
    memb <- as.integer(factor(memb))
  }
  big.groups <- which(table(memb) > 1L)

  # Vertex colors
  group.cols.memb <- rep_len('gray', max(memb))
  if (length(big.groups) > 0L) group.cols.memb[big.groups] <- group.cols[big.groups]
  g <- set_vertex_attr(g, name, value=group.cols.memb[memb])

  # Edge colors
  m <- ecount(g)
  if (m > 0) {
    newcols <- rep.int('gray50', m)
    if (length(big.groups) > 0L) {
      tmp <- vector('list', length=max(big.groups))
      A <- as_adj(g, names=FALSE, sparse=FALSE, edges=TRUE)
      for (i in big.groups) {
        x <- which(memb == i)
        tmp[[i]] <- unique(as.integer(A[x, x]))[-1L]
        if (!is.null(tmp[[i]])) newcols[tmp[[i]]] <- group.cols[i]
      }
    }

    g <- set_edge_attr(g, name, value=newcols)
  }
  return(g)
}

#' Transform edge weights
#'
#' @param xfm.type Character string specifying how to transform the weights.
#'   Default: \code{1/w}
#' @param invert Logical indicating whether or not to invert the transformation.
#'   Default: \code{FALSE}
#' @export
#' @return \code{xfm.weights} returns the same graph object, with transformed
#'   edge weights plus a graph attribute (\code{xfm.type}) recording the method
#'   of transformation
#' @rdname attributes

xfm.weights <- function(g, xfm.type=c('1/w', '-log(w)', '1-w',
                                      '-log10(w/max(w))', '-log10(w/max(w)+1)'),
                        invert=FALSE) {
  stopifnot(is_igraph(g), is_weighted(g))
  xfm.type <- match.arg(xfm.type)
  if (xfm.type == '1/w') {
    E(g)$weight <- 1 / E(g)$weight

  } else if (xfm.type == '-log(w)') {
    if (isTRUE(invert)) {
      E(g)$weight <- exp(-E(g)$weight)
    } else {
      E(g)$weight <- -log(E(g)$weight)
    }

  } else if (xfm.type == '1-w') {
    E(g)$weight <- 1 - E(g)$weight

  } else if (xfm.type == '-log10(w/max(w))') {
    if (isTRUE(invert)) {
      E(g)$weight <- g$max.weight / 10^(E(g)$weight)
    } else {
      g$max.weight <- max(E(g)$weight)
      E(g)$weight <- -log10(E(g)$weight / g$max.weight)
    }

  } else if (xfm.type == '-log10(w/max(w)+1)') {
    if (isTRUE(invert)) {
      E(g)$weight <- g$max.weight * (1 / 10^(E(g)$weight) - 1)
    } else {
      g$max.weight <- max(E(g)$weight)
      E(g)$weight <- -log10(E(g)$weight / g$max.weight + 1)
    }
  }

  g$xfm.type <- xfm.type
  return(g)
}

#' Calculate graph small-worldness
#'
#' \code{small.world} calculates the normalized characteristic path length and
#' clustering coefficient based on observed and random graphs, used to calculate
#' the small-world coefficient \eqn{\sigma}.
#'
#' @param g.list A \code{brainGraphList} object or list of graphs
#' @param rand List of (lists of) equivalent random graphs (output from
#' \code{\link{sim.rand.graph.par}})
#' @export
#'
#' @return A \code{data.table} with the following components:
#' \item{density}{The range of density thresholds used.}
#' \item{N}{The number of random graphs that were generated.}
#' \item{Lp,Lp.rand,Lp.norm}{The observed, average random, and normalized
#'   characteristic path length.}
#' \item{Cp,Cp.rand,Cp.norm}{The observed, average random, and normalized
#'   clustering coefficient.}
#' \item{sigma}{The small-world measure of the graph.}
#'
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}
#' @references Watts, D.J. and Strogatz S.H. (1998) Collective dynamics of
#'   "small-world" networks. \emph{Nature}, \bold{393}, 440--442.
#'   \doi{10.1038/30918}

small.world <- function(g.list, rand) {
  level <- NULL
  sID <- getOption('bg.subject_id')
  gID <- getOption('bg.group')

  if (is_igraph(g.list) || is.brainGraph(g.list)) {
    g.list <- list(g.list)
  } else if (is.brainGraphList(g.list)) {
    level <- g.list$level
    g.list <- g.list$graphs
    rand <- rand$graphs
  } else if (!is.list(g.list)) {
    stop('Input should be a list, igraph, brainGraph, or brainGraphList object.')
  }

  Lp <- vapply(g.list, graph_attr, numeric(1L), 'Lp')
  Cp <- vapply(g.list, graph_attr, numeric(1L), 'Cp')
  densities <- vapply(g.list, graph_attr, numeric(1L), 'density')

  N <- lengths(rand)
  Lp.rand <- colMeans(vapply(rand, vapply, numeric(N[1L]), graph_attr, numeric(1L), 'Lp'))
  Cp.rand <- colMeans(vapply(rand, vapply, numeric(N[1L]), graph_attr, numeric(1L), 'Cp'))

  Cp.norm <- Cp / Cp.rand
  Lp.norm <- Lp / Lp.rand
  sigma <- Cp.norm / Lp.norm
  DT <- data.table(density=densities, N, Lp, Cp, Lp.rand, Cp.rand,
                   Lp.norm, Cp.norm, sigma)
  if (!is.null(names(g.list))) {
    DT[, eval(sID) := names(g.list)]
  }
  attrs <- c('Group', 'threshold')
  for (x in attrs) {
    if (x %in% graph_attr_names(g.list[[1L]])) {
      DT[, eval(x) := sapply(g.list, graph_attr, x)]
    }
  }
  if (hasName(DT, 'Group')) setnames(DT, 'Group', gID)
  if (level == 'group' && DT[, all(get(sID) == get(gID))]) DT[, eval(sID) := NULL]
  return(DT)
}

#' Write files to be used for visualization with BrainNet Viewer
#'
#' Write the \code{.node} and \code{.edge} files necessary for visualization
#' with the BrainNet Viewer software.
#'
#' @details For the \code{.node} file, there are 6 columns:
#' \itemize{
#'   \item \emph{Columns 1-3}: Vertex x-, y-, and z-coordinates
#'   \item \emph{Column 4}: Vertex color
#'   \item \emph{Column 5}: Vertex size
#'   \item \emph{Column 6}: Vertex label
#' }
#' The \code{.edge} file is the graph's associated adjacency matrix; a weighted
#' adjacency matrix can be returned by using the \code{edge.wt} argument.
#'
#' @param g The \code{igraph} graph object of interest
#' @param vcolor Character string indicating how to color the vertices (default:
#'   \code{'none'})
#' @param vsize Character string indicating what size the vertices should be;
#'   can be any vertex-level attribute (default: \code{'constant'})
#' @param edge.wt Character string indicating the edge attribute to use to
#'   return a weighted adjacency matrix (default: \code{NULL})
#' @param file.prefix Character string for the basename of the \code{.node} and
#'   \code{.edge} files that are written
#' @export
#'
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}
#' @references Xia, M. and Wang, J. and He, Y. (2013). BrainNet Viewer: a
#'   network visualization tool for human brain connectomics. \emph{PLoS One},
#'   \bold{8(7)}, e68910. \doi{10.1371/journal.pone.0068910}
#' @examples
#' \dontrun{
#' write_brainnet(g, vcolor='community', vsize='degree', edge.wt='t.stat')
#' }

write_brainnet <- function(g, vcolor='none', vsize='constant', edge.wt=NULL, file.prefix='') {
  x.mni <- y.mni <- z.mni <- NULL
  stopifnot(is.brainGraph(g))

  atlas.dt <- get(g$atlas)
  coords <- round(atlas.dt[, cbind(x.mni, y.mni, z.mni)])

  vnames <- vertex_attr_names(g)
  if (vcolor == 'none') {
    cols <- rep.int(1L, vcount(g))
  } else {
    stopifnot(vcolor %in% vnames)
    cols <- as.integer(factor(vertex_attr(g, vcolor)))
  }

  if (vsize == 'constant') {
    size <- 5L
  } else {
    stopifnot(vsize %in% vnames)
    size <- vertex_attr(g, vsize)
  }

  if (file.prefix == '') {
    nodefile <- paste0(quote(g), '_', vsize, '_', vcolor, '.node')
    edgefile <- paste0(quote(g), '.edge')
  } else {
    nodefile <- paste0(file.prefix, '.node')
    edgefile <- paste0(file.prefix, '.edge')
  }
  fwrite(cbind(coords, cols, size, V(g)$name),
         file=nodefile, quote=FALSE, sep='\t', col.names=FALSE)

  fwrite(as_adj(g, sparse=FALSE, attr=edge.wt),
         file=edgefile, quote=FALSE, sep='\t', col.names=FALSE)
}

#' Default options for brainGraph
#'
#' brainGraph is a package for performing \emph{graph theory analysis} of brain
#' MRI data.
#'
#' @section Package options:
#'
#' brainGraph uses the following \code{\link{options}} to configure behavior:
#' \itemize{
#'   \item \code{bg.subject_id}: character string specifying the name your
#'     project/study uses as a subject identifier. All imported data (e.g.,
#'     covariates tables) \emph{MUST} have a column matching this. One possible
#'     alternative is \code{'participant_id'}, recommended by BIDS. Default:
#'     \code{'Study.ID'}
#'   \item \code{bg.group}: character string specifying the name your
#'     project/study uses as a group identifier. All imported data (e.g.,
#'     covariates tables) \emph{MUST} have a column matching this. One possible
#'     alternative is \code{'group'}, recommended by BIDS. Default:
#'     \code{'Group'}
#'   \item \code{bg.session}: character string specifying the name your
#'     project/study uses as a \dQuote{time} or session identifier, in the case
#'     of longitudinal studies. All imported data (e.g., covariates tables)
#'     \emph{MUST} have a column matching this. One possible alternative is
#'     \code{'session_id'}, recommended by BIDS. Default: \code{'Time'}
#'   \item \code{bg.progress}: logical indicating whether to show progress bars
#'     for functions that provide the option. Default: \code{TRUE}
#'   \item \code{bg.ncpus}: integer indicating the number of cores to use for
#'     parallel operations. Only used if you have not already registered a
#'     parallel backend (see Chapter 5 of the User Guide or
#'     \url{https://github.com/cwatson/brainGraph/blob/master/README.md} for
#'     examples). Default: \code{2L}
#' }
#' @docType package
#' @name brainGraph
#' @aliases brainGraph-options
NULL

# Default values for options
bg.options <- list(
  bg.subject_id='Study.ID',
  bg.group='Group',
  bg.session='Time',
  bg.progress=TRUE,
  bg.ncpus=2L
)

# Hack to avoid WARNING or NOTE w/ Rcheck
# See https://github.com/Rdatatable/data.table/issues/4171
patterns <- function(...) NULL

.onLoad <- function(libname, pkgname) {
  op <- options()
  toset <- !(hasName(op, names(bg.options)))
  if (any(toset)) options(bg.options[toset])

  invisible()
}

#' Calculate the s-core of a network
#'
#' Calculates the \emph{s-core} decomposition of a network. This is analogous to
#' the \emph{k-core} decomposition, but takes into account the \emph{strength}
#' of vertices (i.e., in weighted networks). If an unweighted network is
#' supplied, then the output of the function \code{\link[igraph]{coreness}} is
#' returned.
#'
#' The \emph{s-core} consists of all vertices \eqn{i} with \eqn{s_i > s}, where
#' \eqn{s} is some threshold value. The \eqn{s_0} core is the entire network,
#' and the threshold value of the \eqn{s_{n}} core is
#' \deqn{s_{n-1} = min_i s_i}
#' for all vertices \eqn{i} in the \eqn{s_{n-1}} core.
#'
#' Note that in networks with a wide distribution of vertex strengths, in which
#' there are almost as many unique values as there are vertices, then several
#' separate cores will have a single vertex. See the reference provided below.
#'
#' @param W Numeric matrix of edge weights (default: \code{NULL})
#' @inheritParams efficiency
#' @export
#'
#' @return Integer vector of the vertices' \emph{s-core} membership
#'
#' @seealso \code{\link[igraph]{coreness}}
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}
#' @references Eidsaa, M and Almaas, E. (2013) s-core network decomposition: a
#'   generalization of k-core analysis to weighted networks. \emph{Physical
#'   Review E}, \bold{88}, 062819.
#'   \doi{10.1103/PhysRevE.88.062819}

s_core <- function(g, W=NULL) {
  stopifnot(is_igraph(g))
  if (!is_weighted(g)) return(coreness(g))

  if (is.null(W)) W <- as_adj(g, names=FALSE, sparse=FALSE, attr='weight')
  ct <- 1L
  s.core <- vector('integer', length=dim(W)[1L])
  repeat {
    str.tmp <- colSums(W)
    s.thr <- min(str.tmp[which(str.tmp > 0)])
    v.remove <- which(str.tmp <= s.thr & str.tmp > 0)
    if (length(v.remove) > 0L) {
      s.core[v.remove] <- ct
      W[v.remove, ] <- W[, v.remove] <- 0
      ct <- ct + 1L
    }
    if (sum(colSums(W) > 0) == 0L) break
  }
  return(s.core)
}

#' Plot global graph measures across densities
#'
#' Create a faceted line plot of global graph measures across a range of graph
#' densities, calculated from a list of \code{brainGraphList} objects. This
#' requires that the variables of interest are graph-level attributes of the
#' input graphs.
#'
#' You can choose to insert a dashed vertical line at a specific
#' density/threshold of interest, rename the variable levels (which become the
#' facet titles), exclude variables, and include a \code{brainGraph_permute}
#' object of permutation data to add asterisks indicating significant group
#' differences.
#'
#' @inheritParams Random Graphs
#' @param xvar A character string indicating whether the variable of
#' interest is \dQuote{density} or \dQuote{threshold} (e.g. with DTI data)
#' @param vline Numeric of length 1 specifying the x-intercept if you would like
#'   to plot a vertical dashed line (e.g., if there is a particular density of
#'   interest). Default: \code{NULL}
#' @param level.names Character vector of variable names, which are displayed as
#'   facet labels. If you do not want to change them, specify \code{NULL}. By
#'   default, they are changed to pre-set values.
#' @param exclude Character vector of variables to exclude. Default: \code{NULL}
#' @param perms A \code{\link[data.table]{data.table}} of permutation group differences
#' @param alt Character vector of alternative hypotheses; required if
#'   \emph{perms} is provided, but defaults to \dQuote{two.sided} for all
#'   variables
#' @export
#'
#' @return Either a \code{trellis} or \code{ggplot} object
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}

plot_global <- function(g.list, xvar=c('density', 'threshold'), vline=NULL,
                        level.names='default', exclude=NULL, perms=NULL, alt='two.sided') {
  sig <- trend <- yloc <- value <- variable <- threshold <- panel.num <- NULL

  # Check if components are 'brainGraphList' objects
  matches <- vapply(g.list, is.brainGraphList, logical(1L))
  if (any(!matches)) stop("Input must be a list of 'brainGraphList' objects.")

  sID <- getOption('bg.subject_id')
  gID <- getOption('bg.group')
  DT <- rbindlist(lapply(g.list, graph_attr_dt), use.names=TRUE)
  idvars <- c('atlas', 'modality', 'weighting', sID, gID, 'threshold', 'density')
  idvars <- idvars[which(hasName(DT, idvars))]
  DT.m <- melt(DT, id.vars=idvars)
  DT.m <- droplevels(DT.m[!variable %in% exclude])

  # Add asterisks if a data.table of permutation values is provided
  if (!is.null(perms)) {
    DT.m[, c('sig', 'trend') := '']
    DT.m[, yloc := extendrange(value)[1L], by=variable]
    vars <- setdiff(names(perms$DT), 'densities')
    nvars <- length(vars)
    if (length(alt) < nvars) alt <- rep_len(alt, nvars)
    for (i in seq_along(vars)) {
      dt <- plot(perms, measure=vars[i], alternative=alt[i])[[1L]]$data[variable == 'obs.diff']
      DT.m[variable == vars[i], sig := dt$sig]
      DT.m[variable == vars[i], trend := dt$trend]
      DT.m[variable == vars[i], yloc := dt$yloc]
    }
  }

  if (!is.null(level.names)) {
    if (level.names == 'default') level.names <- rename_levels(DT.m)
    nvars <- DT.m[, nlevels(variable)]
    if (length(level.names) < nvars) level.names <- rep_len(level.names, nvars)
    levels(DT.m$variable) <- level.names
  }

  xvar <- match.arg(xvar)
  # 'base' plotting
  if (!requireNamespace('ggplot2', quietly=TRUE)) {
    grps <- DT.m[, levels(as.factor(get(gID)))]
    DT.m[, panel.num := as.numeric(variable)]
    if (!is.null(vline)) {
      if (is.null(perms)) {
        panelfun <- function(x, y, groups, ...) {
          panel.abline(v=vline, lty=2, col='grey60')
          panel.xyplot(x, y, groups, ...)
        }
      } else {
        panelfun <- function(x, y, groups, ...) {
          panel.num <- NULL
          panel.abline(v=vline, lty=2, col='grey60')
          panel.xyplot(x, y, groups, ...)
          DT.m[panel.num == panel.number() & sig == '*',
               panel.text(density, yloc, labels='*', col=plot.cols[1L])]
          DT.m[panel.num == panel.number() & trend == '*',
               panel.text(density, yloc, labels='*', col=plot.cols[2L])]
        }
      }
    } else {
      if (is.null(perms)) {
        panelfun <- function(x, y, groups, ...) {
          panel.xyplot(x, y, groups, ...)
        }
      } else {
        panelfun <- function(x, y, groups, ...) {
          panel.num <- NULL
          panel.xyplot(x, y, groups, ...)
          DT.m[panel.num == panel.number() & sig == '*',
               panel.text(density, yloc, labels='*', col=plot.cols[1L])]
          DT.m[panel.num == panel.number() & trend == '*',
               panel.text(density, yloc, labels='*', col=plot.cols[2L])]
        }
      }
    }
    p <- xyplot(value ~ get(xvar) | variable, data=DT.m, groups=get(gID), type='l',
                xlab=xvar, panel=panelfun, scales=list(y=list(relation='free')),
                auto.key=list(space='bottom', title=gID, cex.title=1, columns=length(grps),
                              lines=TRUE, points=FALSE))

  # 'ggplot2' plotting
  } else {
    p <- switch(xvar,
                density=ggplot2::ggplot(DT.m, ggplot2::aes(x=density, y=value, col=get(gID))),
                threshold=ggplot2::ggplot(DT.m, ggplot2::aes(x=threshold, y=value, col=get(gID))) + ggplot2::scale_x_reverse())

    if (hasName(DT.m, sID)) {
      p <- p + ggplot2::stat_smooth(method='gam', formula=y~s(x))
    } else {
      p <- p + ggplot2::geom_line()
    }
    p <- p +
      ggplot2::facet_wrap(~ variable, scales='free_y') +
      ggplot2::theme(legend.position='bottom', axis.text.x=ggplot2::element_text(angle=45, hjust=1))

    if (!is.null(vline)) p <- p + ggplot2::geom_vline(xintercept=vline, lty=2, col='grey60')
    if (!is.null(perms)) {
      p <- p +
        ggplot2::geom_text(ggplot2::aes(y=yloc, label=sig), col='red', size=3) +
        ggplot2::geom_text(ggplot2::aes(y=yloc, label=trend), col='blue', size=3)
    }
  }
  return(p)
}

#' Rename the levels of global metrics in a data.table
#'
#' @param DT A \code{data.table}
#' @return The levels of the \code{variable} column after being changed
#' @keywords internal
rename_levels <- function(DT) {
  orig <- c('Lp', 'E.global', 'E.local', 'mod', 'diameter', 'transitivity', 'num.hubs')
  orig <- c(orig, paste0(orig, '.wt'))
  orig <- c(orig, 'Cp', 'max.comp', 'num.tri', 'asymm', 'spatial.dist', 'vulnerability',
            'strength', paste0('assort', c('', '.lobe.hemi', '.lobe', '.class', '.network')))
  abbr <- c('Char. path length', 'Global eff.', 'Local eff.', 'Modularity',
            'Diameter', 'Transitivity', '# of hubs')
  abbr <- c(abbr, paste(abbr, '(wt)'))
  abbr <- c(abbr, 'Clustering coeff.', 'Max. conn. comp.', '# of triangles', 'Asymmetry index',
            'Edge distance', 'Vulnerability', 'Strength',
            paste(c('Degree', 'Lobe & hemi', 'Lobe', 'Class', 'Network'), 'assort.'))
  inds <- match(levels(DT$variable), orig)
  orig <- orig[inds]
  abbr <- abbr[inds]
  orig <- paste0('^', orig, '$')
  out <- diag(mapply(sub, orig, abbr, MoreArgs=list(x=levels(DT$variable))))
  return(out)
}

#' Plot group distributions of volumetric measures for a given brain region
#'
#' This function takes a \dQuote{tidied} dataset of cortical volumetric measures
#' (thickness, volume, LGI, etc.) and plots a histogram or violin plot for 1 or
#' more groups, and of 1 or more brain regions.
#'
#' @param dat A data table of volumetric data; needs columns for 'Group',
#' 'region', and 'value'
#' @param regions A vector of character strings or integers of the brain
#' region(s) to plot; if integer, the region(s) is/are chosen from the input
#' data table based on the index
#' @param type A character string indicating the plot type; either 'histogram'
#' or 'violin'
#' @param all.vals A logical indicating whether or not to plot horizontal lines
#' for all observations (only valid for 'violin' plots) (default: TRUE)
#' @param modality A character string indicating the type of volumetric measure
#' ('thickness', 'volume', 'lgi', or 'area')
#' @export
#'
#' @return A \code{trellis} or \code{ggplot} object
#' @family Structural covariance network functions
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}

plot_volumetric <- function(dat, regions, type=c('violin', 'histogram'),
                            all.vals=TRUE,
                            modality=c('thickness', 'volume', 'lgi', 'area')) {
  region <- value <- Group <- ..density.. <- avg <- group.mean <- bwidth <-
    breaks <- x <- width <- NULL
  if (!is.character(regions)) regions <- dat[, levels(region)][regions]
  stopifnot(all(regions %in% dat[, levels(region)]))

  modality <- match.arg(modality)
  ax.lab <- switch(modality,
                   thickness='Thickness (mm)',
                   volume=expression(paste('Volume (', mm^{3}, ')')),
                   lgi='Local Gyrification Index',
                   area=expression(paste('Surface area (', mm^{2}, ')')))

  subDT <- dat[region %in% regions]
  type <- match.arg(type)
  if (type == 'histogram') {
    if (!all(vapply(c('scales', 'ggplot2'), requireNamespace, logical(1L), quietly=TRUE))) {
      stop('Please install the "scales" and "ggplot2" packages to plot with histograms.')
    }
    # Allow for variable bin widths
    groups <- subDT[, levels(Group)]
    setkey(subDT, region, Group)
    breaksdt <- subDT[, list(breaks=pretty(range(value), n=nclass.FD(value))),
                      by=list(Group, region)]
    breaksdt[, bwidth := .SD[1L:2L, diff(breaks)], by=list(Group, region)]
    subDT[, bwidth := rep(breaksdt[, min(bwidth), by=region]$V1,
                          times=subDT[, .N, by=region]$N)]
    # A partial recreation of Hadley's ggplot2:::bin function
    create_bins <- function(x, binwidth) {
      breaks <- sort(scales::fullseq(range(x), binwidth, pad=TRUE))
      bins <- cut(x, breaks, include.lowest=TRUE, right=FALSE)
      left <- breaks[-length(breaks)]
      right <- breaks[-1L]
      x <- (left + right) / 2
      width <- diff(breaks)

      out <- data.frame(count=as.numeric(tapply(rep(1, length(bins)), bins, sum,
                                                na.rm=TRUE)),
                        x=x,
                        width=width)
      out$count[is.na(out$count)] <- 0
      out$density <- out$count / out$width / sum(abs(out$count), na.rm=TRUE)
      out$ndensity <- out$density / max(abs(out$density), na.rm=TRUE)
      out$ncount <- out$count / max(abs(out$count), na.rm=TRUE)
      return(out)
    }
    my.df <- subDT[, create_bins(value, unique(bwidth)), by=list(Group, region)]

    meandt <- subDT[, list(avg=mean(value)), by=list(Group, region)]
    vol.plot <- ggplot2::ggplot(my.df) +
      ggplot2::geom_histogram(ggplot2::aes(x, y=density, width=width, fill=Group),
                     alpha=0.6, position='dodge', stat='identity') +
      ggplot2::geom_vline(data=meandt, ggplot2::aes(xintercept=avg, col=Group), lty=2, size=0.5) +
      ggplot2::geom_density(data=subDT, ggplot2::aes(x=value, col=Group), size=0.8) +
      ggplot2::facet_wrap(~ region, scales='free') +
      ggplot2::theme(legend.position=c(1, 1), legend.justification=c(1, 1),
            legend.background=ggplot2::element_rect(size=0.5)) +
      ggplot2::xlab(ax.lab)

  } else if (type == 'violin') {
    # 'base' plotting
    if (!requireNamespace('ggplot2', quietly=TRUE)) {
      gID <- getOption('bg.group')
      vol.plot <- bwplot(value ~ get(gID) | region, data=subDT, panel=panel.violin, xlab=gID, ylab=ax.lab)

    # 'ggplot2' plotting
    } else {
      vol.plot <- ggplot2::ggplot(subDT, ggplot2::aes(x=Group, y=value, fill=Group)) +
        ggplot2::geom_violin(trim=FALSE) +
        ggplot2::facet_wrap(~ region, scales='free_y') +
        ggplot2::theme(legend.position='none') +
        ggplot2::ylab(ax.lab)
      if (isTRUE(all.vals)) {
        subDT[, group.mean := mean(value), by=list(Group, region)]
        vol.plot <- vol.plot +
          ggplot2::geom_segment(ggplot2::aes(x=as.numeric(Group)-0.1, xend=as.numeric(Group)+0.1,
                                             y=value, yend=value), col='black') +
          ggplot2::geom_segment(ggplot2::aes(x=as.numeric(Group)-0.3, xend=as.numeric(Group)+0.3,
                                             y=group.mean, yend=group.mean), col='black')
      } else {
        vol.plot <- vol.plot + ggplot2::geom_boxplot(width=0.1)
      }
    }
  }
  return(vol.plot)
}

#' Analysis of network robustness
#'
#' This function performs a \dQuote{targeted attack} of a graph or a
#' \dQuote{random failure} analysis, calculating the size of the largest
#' component after edge or vertex removal.
#'
#' In a targeted attack, it will sort the vertices by either degree or
#' betweenness centrality (or sort edges by betweenness), and successively
#' remove the top vertices/edges. Then it calculates the size of the largest
#' component.
#'
#' In a random failure analysis, vertices/edges are removed in a random order.
#'
#' @param g An \code{igraph} graph object
#' @param type Character string; either \code{'vertex'} or \code{'edge'}
#'   removals. Default: \code{vertex}
#' @param measure Character string; sort by either \code{'btwn.cent'} or
#'   \code{'degree'}, or choose \code{'random'}. Default: \code{'btwn.cent'}
#' @param N Integer; the number of iterations if \code{'random'} is chosen.
#'   Default: \code{1e3}
#' @export
#' @importFrom foreach getDoParRegistered
#' @importFrom doParallel registerDoParallel
#'
#' @return Data table with elements:
#'   \item{type}{Character string describing the type of analysis performed}
#'   \item{measure}{The input argument \code{measure}}
#'   \item{comp.size}{The size of the largest component after edge/vertex
#'     removal}
#'   \item{comp.pct}{Numeric vector of the ratio of maximal component size after
#'     each removal to the observed graph's maximal component size}
#'   \item{removed.pct}{Numeric vector of the ratio of vertices/edges removed}
#'   \item{Group}{Character string indicating the subject group, if applicable}
#'
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}
#' @references Albert, R. and Jeong, H. and Barabasi, A. (2000) Error and attack
#'   tolerance of complex networks. \emph{Nature}, \bold{406}, 378--381.
#'   \doi{10.1038/35019019}

robustness <- function(g, type=c('vertex', 'edge'),
                       measure=c('btwn.cent', 'degree', 'random'), N=1e3) {
  i <- NULL
  stopifnot(is_igraph(g))
  type <- match.arg(type)
  measure <- match.arg(measure)
  orig_max <- max(components(g)$csize)
  n <- switch(type, vertex=vcount(g), edge=ecount(g))
  removed.pct <- seq.int(0, 1, length.out=n+1L)
  if (measure == 'random') {
    otype <- paste('Random', type, 'removal')
    rand <- matrix(rep.int(seq_len(n), N), nrow=n, ncol=N)
    index <- apply(rand, 2L, sample)
  } else {
    otype <- paste('Targeted', type, 'attack')
    max.comp.removed <- rep.int(orig_max, n)
  }
  if (!getDoParRegistered()) {
    cl <- makeCluster(getOption('bg.ncpus'))
    registerDoParallel(cl)
  }
  if (type == 'vertex') {
    if (measure == 'random') {
      max.comp <- foreach(i=seq_len(N), .combine='cbind') %dopar% {
        ord <- V(g)$name[index[, i]]
        tmp <- rep.int(orig_max, n)
        for (j in seq_len(n - 1L)) {
          g <- delete_vertices(g, ord[j])
          tmp[j + 1L] <- max(components(g)$csize)
        }
        tmp
      }
      max.comp.removed <- rowMeans(max.comp)

    } else {
      val <- if (measure == 'btwn.cent') centr_betw(g)$res else check_degree(g)
      ord <- V(g)$name[order(val, decreasing=TRUE)]
      for (j in seq_len(n - 1L)) {
        g <- delete_vertices(g, ord[j])
        max.comp.removed[j + 1L] <- max(components(g)$csize)
      }
    }

  } else {
    if (measure == 'degree') {
      stop('For edge attacks, must choose "btwn.cent" or "random"!')
    } else if (measure == 'random') {
      max.comp <- foreach(i=seq_len(N), .combine='cbind') %dopar% {
        el <- as_edgelist(g, names=FALSE)[index[, i], ]
        tmp <- rep.int(orig_max, n)
        for (j in seq_len(n - 1L)) {
          g.rand <- graph_from_edgelist(el[-seq_len(j), , drop=FALSE], directed=FALSE)
          tmp[j + 1L] <- max(components(g.rand)$csize)
        }
        tmp
      }
      max.comp.removed <- rowMeans(max.comp)

    } else {
      ord <- order(E(g)$btwn, decreasing=TRUE)
      el <- as_edgelist(g, names=FALSE)[ord, ]
      for (j in seq_len(n - 1L)) {
        g <- graph_from_edgelist(el[-seq_len(j), , drop=FALSE], directed=FALSE)
        max.comp.removed[j + 1L] <- max(components(g)$csize)
      }
    }

  }
  max.comp.removed <- c(max.comp.removed, 0)
  comp.pct <- max.comp.removed / orig_max
  out <- data.table(type=otype, measure=measure, comp.size=max.comp.removed,
                    comp.pct=comp.pct, removed.pct=removed.pct)
  if ('name' %in% graph_attr_names(g)) out[, eval(getOption('bg.group')) := g$name]
  return(out)
}

#' Rich club calculations
#'
#' \code{rich_club_coeff} calculates the \emph{rich club} of a graph, returning
#' the rich-club coefficient, \eqn{\phi}, and the subgraph of rich club
#' vertices.
#'
#' @param k Integer; the minimum degree for including a vertex. Default: 1
#' @param weighted Logical indicating whether or not edge weights should be
#'   used. Default: \code{FALSE}
#' @inheritParams efficiency
#' @export
#'
#' @return \code{\link{rich_club_coeff}} - a list with components:
#'   \item{phi}{The rich club coefficient, \eqn{\phi}.}
#'   \item{graph}{A subgraph containing only the rich club vertices.}
#'   \item{Nk,Ek}{The number of vertices/edges in the rich club graph.}
#'
#' @name Rich Club
#' @rdname rich_club
#' @family Rich-club functions
#'
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}
#' @references Zhou, S. and Mondragon, R.J. (2004) The rich-club phenomenon
#'   in the internet topology. \emph{IEEE Comm Lett}, \bold{8}, 180--182.
#'   \doi{10.4018/978-1-59140-993-9.ch066}
#' @references Opsahl, T. and Colizza, V. and Panzarasa, P. and Ramasco, J.J.
#'   (2008) Prominence and control: the weighted rich-club effect. \emph{Physical
#'   Review Letters}, \bold{101.16}, 168702.
#'   \doi{10.1103/PhysRevLett.101.168702}

rich_club_coeff <- function(g, k=1, weighted=FALSE, A=NULL) {
  stopifnot(is_igraph(g))
  degs <- check_degree(g)
  Nv <- vcount(g)
  Nk <- sum(degs > k)
  if (Nk == 0L) return(list(phi=NaN, graph=make_empty_graph(), Nk=0L, Ek=0))

  eattr <- wts <- NULL
  if (isTRUE(weighted)) {
    eattr <- 'weight'
    wts <- TRUE
  }
  if (is.null(A)) A <- as_adj(g, names=FALSE, sparse=FALSE, attr=eattr)

  vids <- order(degs)[(Nv - Nk + 1L):Nv]
  g.rich <- graph_from_adjacency_matrix(A[vids, vids, drop=FALSE], mode='undirected',
                                        diag=FALSE, weighted=wts)
  Ek <- ecount(g.rich)

  if (isTRUE(weighted)) {
    Wr <- sum(E(g.rich)$weight)
    weights <- sort(E(g)$weight, decreasing=TRUE)[seq_len(Ek)]
    phi <- Wr / sum(weights)
  } else {
    phi <- graph.density(g.rich)
  }

  return(list(phi=phi, graph=g.rich, Nk=Nk, Ek=Ek))
}

#' Rich club coefficient for all degrees in a graph
#'
#' \code{rich_club_all} is a wrapper for \code{\link{rich_club_coeff}} that
#' calculates the rich-club coefficient for all degrees present in the graph. It
#' returns a \code{data.table} with the coefficients and vertex and edge counts
#' for each successive rich club.
#' @export
#' @return \code{\link{rich_club_all}} - a \code{data.table} with components:
#'   \item{k}{A vector of all vertex degrees present in the original graph}
#'   \item{phi}{The rich-club coefficient}
#'   \item{Nk,Ek}{The number of vertices/edges in the rich club for each
#'     successive \emph{k}}
#'
#' @rdname rich_club

rich_club_all <- function(g, weighted=FALSE, A=NULL) {
  stopifnot(is_igraph(g))
  k <- check_degree(g)
  deg_range <- seq_len(max(k))
  R <- lapply(deg_range, function(x) rich_club_coeff(g, x, weighted, A=A))
  phi <- vapply(R, with, numeric(1L), phi)
  Nk <- vapply(R, with, numeric(1L), Nk)
  Ek <- vapply(R, with, numeric(1L), Ek)
  dt.rich <- data.table(k=deg_range, phi=phi, Nk=Nk, Ek=Ek)
  return(dt.rich)
}

#' Calculate the normalized rich club coefficient
#'
#' \code{rich_club_norm} will (optionally) generate a number of random graphs,
#' calculate their rich club coefficients (\eqn{\phi}), and return
#' \eqn{\phi_{norm}} of the graph of interest, which is the observed rich-club
#' coefficient divided by the mean across the random graphs.
#'
#' If random graphs have already been generated, you can supply a list as an
#' argument.
#'
#' @param N Integer; the number of random graphs to generate. Default: 100
#' @param rand A list of \code{igraph} graph objects, if random graphs have
#'   already been generated. Default: \code{NULL}
#' @param ... Other parameters (passed to \code{\link{sim.rand.graph.par}})
#' @export
#'
#' @return \code{\link{rich_club_norm}} - a data table with columns:
#'   \item{k}{Sequence of degrees}
#'   \item{rand}{Rich-club coefficients for the random graphs}
#'   \item{orig}{Rich-club coefficients for the original graph.}
#'   \item{norm}{Normalized rich-club coefficients.}
#'   \item{p}{P-values based on the distribution of \code{rand}}
#'   \item{p.fdr}{The FDR-adjusted P-values}
#'   \item{density}{The observed graph's density}
#'   \item{threshold,Group,name}{}
#'
#' @rdname rich_club
#' @family Random graph functions
#'
#' @references Colizza, V. and Flammini, A. and Serrano, M.A. and Vespignani, A.
#'   (2006) Detecting rich-club ordering in complex networks. \emph{Nature
#'   Physics}, \bold{2}, 110--115. \doi{10.1038/nphys209}

rich_club_norm <- function(g, N=1e2, rand=NULL, ...) {
  k <- orig <- p <- p.fdr <- NULL
  sID <- getOption('bg.subject_id')
  gID <- getOption('bg.group')

  stopifnot(is_igraph(g))
  degs <- check_degree(g)
  gnames <- graph_attr_names(g)
  if (!'rich' %in% gnames) g$rich <- rich_club_all(g)
  if (is.null(rand)) {
    rand <- sim.rand.graph.par(g, N, ...)
  } else {
    if (!all(vapply(rand, is_igraph, logical(1L)))) {
      stop('Argument "rand" must be a list of igraph graph objects!')
    }
    N <- length(rand)
  }

  phi.rand <- t(sapply(rand, function(x) x$rich$phi))
  max.deg <- max(degs)
  DT <- data.table(k=rep(seq_len(max.deg), each=N),
                   rand=c(phi.rand),
                   orig=rep(g$rich$phi, each=N))
  DT[, norm := unique(orig) / mean(rand), by=k]
  DT[, p := (sum(rand >= unique(orig)) + 1L) / (N + 1L), by=k]
  dt.phi <- DT[, .SD[1L], by=k]
  dt.phi[, p.fdr := p.adjust(p, 'fdr')]
  dt.phi[, rand := NULL]
  dt.phi$rand <- DT[, mean(rand), by=k]$V1
  dt.phi$density <- if ('density' %in% gnames) g$density else graph.density(g)
  if ('threshold' %in% gnames) dt.phi$threshold <- g$threshold
  if ('Group' %in% gnames) dt.phi$Group <- g$Group
  if (!is.null(g$name)) {
    dt.phi[, eval(sID) := g$name]
  }
  if (dt.phi[, all(get(sID) == get(gID))]) dt.phi[, eval(sID) := NULL]
  return(dt.phi)
}

#' Assign graph attributes based on rich-club analysis
#'
#' Assigns vertex- and edge-level attributes based on the results of a
#' \emph{rich-club} analysis, based on a range of vertex degrees in which the
#' rich-club coefficient was determined to be significantly greater than that of
#' a set of random graphs (see \code{\link{rich_club_norm}}).
#'
#' Vertices which are in the rich club will be assigned an attribute
#' \code{rich}, taking on a binary value. Their colors (attribute
#' \code{color.rich}) will be either \emph{red} or \emph{gray}. Their sizes
#' (attribute \code{size.rich}) will either be 10 or will be proportional to
#' their degree.
#'
#' Edge attribute \code{type.rich} takes on three values: \emph{rich-club} (if
#' it connects two rich-club vertices), \emph{feeder} (if it connects a rich- to
#' a non-rich-club vertex), and \emph{local} (if it connects two non-rich-club
#' vertices). The \code{color.rich} attribute is \emph{red}, \emph{orange}, or
#' \emph{green}. Edge sizes (\code{size.rich}) will be largest for
#' \emph{rich-club} connections, then smaller for \emph{feeder}, and smallest
#' for \emph{local}.
#'
#' @param g An \code{igraph} graph object
#' @param deg.range Numeric vector of the range of degrees indicating
#'   inclusion in the rich-club; if the default \emph{NULL}, it will be from 1
#'   to the maximum degree in the graph
#' @param adj.vsize Logical indicating whether to adjust vertex size
#'   proportional to degree. Default: \code{FALSE}
#' @export
#'
#' @return An \code{igraph} graph object with additional attributes:
#'   \item{rich}{Binary indicating membership in the rich-club}
#'   \item{type.rich}{Edge attribute indicating the type of connection}
#'   \item{color.rich}{Edge and vertex attributes}
#'   \item{size.rich}{Edge and vertex attributes}
#'
#' @family Rich-club functions
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}
#' @examples
#' \dontrun{
#' g <- rich_club_attrs(g, rich.dt[density == densities[N] & p.fdr < .01,
#'                                 range(k)])
#' }

rich_club_attrs <- function(g, deg.range=NULL, adj.vsize=FALSE) {
  stopifnot(is_igraph(g))
  degs <- check_degree(g)
  if (is.null(deg.range)) deg.range <- c(1, max(degs))
  V(g)$rich <- 0
  rich <- which(degs >= deg.range[1L] & degs <= deg.range[2L])
  notrich <- setdiff(seq_len(vcount(g)), rich)
  V(g)$rich[rich] <- 1
  E(g)[rich %--% rich]$type.rich <- 'rich-club'
  E(g)[rich %--% notrich]$type.rich <- 'feeder'
  E(g)[notrich %--% notrich]$type.rich <- 'local'

  V(g)$color.rich[rich] <- 'red'
  V(g)[notrich]$color.rich <- 'gray'
  E(g)[E(g)$type.rich == 'rich-club']$color.rich <- 'red'
  E(g)[E(g)$type.rich == 'feeder']$color.rich <- 'orange'
  E(g)[E(g)$type.rich == 'local']$color.rich <- 'green'

  V(g)$size.rich[rich] <- if (isTRUE(adj.vsize)) vec.transform(degs[rich], 3, 15) else 10
  V(g)[notrich]$size.rich <- 0
  E(g)[E(g)$type.rich == 'rich-club']$size.rich <- 3.5
  E(g)[E(g)$type.rich == 'feeder']$size.rich <- 1.5
  E(g)[E(g)$type.rich == 'local']$size.rich <- 0.5

  return(g)
}

#' Calculate the rich core of a graph
#'
#' \code{rich_core} finds the boundary of the rich core of a graph, based on the
#' decreasing order of vertex degree. It also calculates the degree that
#' corresponds to that rank, and the core size relative to the total number of
#' vertices in the graph.
#'
#' For weighted graphs, the degree is substituted by a normalized weight:
#' \deqn{ceiling(A / w_{min})}
#' where \eqn{w_{min}} is the minimum weight (that is greater than 0), and
#' \eqn{ceiling()} is the \emph{ceiling} function that rounds up to the nearest
#' integer.
#'
#' @export
#' @return \code{\link{rich_core}} - a data table with columns:
#'   \item{density}{The density of the graph.}
#'   \item{rank}{The rank of the boundary for the rich core.}
#'   \item{k.r}{The degree/strength of the vertex at the boundary.}
#'   \item{core.size}{The size of the core relative to the graph size.}
#'   \item{weighted}{Whether or not weights were used}
#'
#' @rdname rich_club
#'
#' @references Ma, A and Mondragon, R.J. (2015) Rich-cores in networks.
#'   \emph{PLoS One}, \bold{10(3)}, e0119678.
#'   \doi{10.1371/journal.pone.0119678}

rich_core <- function(g, weighted=FALSE, A=NULL) {
  stopifnot(is_igraph(g))
  if (isTRUE(weighted)) {
    if (is.null(A)) A <- as_adj(g, names=FALSE, sparse=FALSE, attr='weight')
    w_min <- min(A[A != 0])
    A <- ceiling(A / w_min)
  } else {
    if (is.null(A)) A <- as_adj(g, names=FALSE, sparse=FALSE)
  }
  degs <- colSums(A)
  vorder <- order(degs, decreasing=TRUE)
  kplus <- vapply(vorder, function(x) sum(A[x, which(degs > degs[x])]), double(1L))

  dens <- if ('density' %in% graph_attr_names(g)) g$density else graph.density(g)
  r <- max(which(kplus == max(kplus)))
  k.r <- as.integer(degs[vorder][r])
  core.size <- r / dim(A)[2L]
  return(data.table(density=dens, rank=r, k.r=k.r, core.size=core.size, weighted=weighted))
}

#' Approaches to estimate individual network contribution
#'
#' \code{loo} calculates the individual contribution to group network data for
#' each subject in each group using a \dQuote{leave-one-out} approach. The
#' residuals of a single subject are excluded, and a correlation matrix is
#' created. This is compared to the original correlation matrix using the Mantel
#' test.
#'
#' @param resids An object of class \code{brainGraph_resids} (the output from
#'   \code{\link{get.resid}})
#' @param corrs List of lists of correlation matrices (as output by
#'   \code{\link{corr.matrix}}).
#' @param level Character string; the level at which you want to calculate
#'   contributions (either \code{global} or \code{regional})
#' @export
#' @importFrom foreach getDoParRegistered
#' @importFrom doParallel registerDoParallel
#'
#' @return A \code{data.table} with columns for
#'   \item{Study.ID}{Subject identifier}
#'   \item{Group}{Group membership}
#'   \item{region}{If \code{level='regional'}}
#'   \item{IC,RC}{The value of the individual/regional contributions}
#'
#' @family Structural covariance network functions
#' @name IndividualContributions
#' @rdname individ_contrib
#' @examples
#' \dontrun{
#' IC <- loo(resids.all, corrs)
#' RC <- loo(resids.all, corrs, level='regional')
#' }
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}
#' @references Saggar, M. and Hosseini, S.M.H. and Buno, J.L. and Quintin, E.
#'   and Raman, M.M. and Kesler, S.R. and Reiss, A.L. (2015) Estimating
#'   individual contributions from group-based structural correlations networks.
#'   \emph{NeuroImage}, \bold{120}, 274--284.
#'   \doi{10.1016/j.neuroimage.2015.07.006}

loo <- function(resids, corrs, level=c('global', 'regional')) {
  ..sID <- ..gID <- NULL
  sID <- getOption('bg.subject_id')
  gID <- getOption('bg.group')
  i <- NULL
  stopifnot(inherits(resids, 'brainGraph_resids'), inherits(corrs, 'corr_mats'))
  level <- match.arg(level)
  group.vec <- groups(resids)
  n <- nobs(resids)
  if (level == 'global') {
    if (!requireNamespace('ade4', quietly=TRUE)) stop('Must install the "ade4" package.')
    combFun <- c
    diffFun <- function(a, b) 1 - ade4::mantel.rtest(as.dist(a), as.dist(b), nrepet=1e3)$obs
  } else if (level == 'regional') {
    combFun <- rbind
    diffFun <- function(a, b) colSums(abs(a - b))
  }
  if (!getDoParRegistered()) {
    cl <- makeCluster(getOption('bg.ncpus'))
    registerDoParallel(cl)
  }
  IC <- foreach(i=seq_len(n), .combine=combFun) %dopar% {
    resids.excl <- resids[-i]
    new.corrs <- corr.matrix(resids.excl[group.vec[i]], densities=0.1)
    diffFun(corrs$R[, , group.vec[i]], new.corrs$R[, , 1L])
  }

  DT <- cbind(resids$resids.all[, c(..sID, ..gID)], IC)
  if (level == 'regional') {
    DT <- melt(DT, id.vars=c(sID, gID), variable.name='region', value.name='RC')
  }
  out <- list(method='Leave one out', level=level, DT=DT)
  class(out) <- c('IC', class(out))
  return(out)
}

#' Add-one-patient approach to estimate individual network contribution
#'
#' \code{aop} calculates the individual contribution using an
#' \dQuote{add-one-patient} approach. The residuals of a single patient are
#' added to those of a control group, and a correlation matrix is created. This
#' is repeated for all individual patients and each patient group.
#'
#' @note For \code{aop}, it is assumed by default that the control group is the
#'   first group.
#'
#' @param control.value Integer or character string specifying the control group
#'   (default: 1L)
#' @export
#' @importFrom foreach getDoParRegistered
#' @importFrom doParallel registerDoParallel
#'
#' @rdname individ_contrib
#' @examples
#' \dontrun{
#' IC <- aop(resids.all, corrs)
#' RC <- aop(resids.all, corrs, level='regional')
#' }

aop <- function(resids, corrs, level=c('global', 'regional'), control.value=1L) {
  ..sID <- ..gID <- NULL
  sID <- getOption('bg.subject_id')
  gID <- getOption('bg.group')
  i <- NULL
  stopifnot(inherits(resids, 'brainGraph_resids'), inherits(corrs, 'corr_mats'))

  corr.mat <- corrs[, control.value]$R[, , 1L]
  grps <- groups(resids)
  kNumSubj <- table(grps)
  grps <- unique(grps)
  if (is.numeric(control.value)) control.value <- grps[control.value]
  patient.str <- setdiff(grps, control.value)

  control.inds <- resids$resids.all[get(gID) == control.value, which=TRUE]
  level <- match.arg(level)
  if (level == 'global') {
    if (!requireNamespace('ade4', quietly=TRUE)) stop('Must install the "ade4" package.')
    combFun <- c
    diffFun <- function(a, b) 1 - ade4::mantel.rtest(as.dist(a), as.dist(b), nrepet=1e3)$obs
  } else if (level == 'regional') {
    combFun <- rbind
    diffFun <- function(a, b) data.table(t(colSums(abs(a - b))))
  }

  if (!getDoParRegistered()) {
    cl <- makeCluster(getOption('bg.ncpus'))
    registerDoParallel(cl)
  }
  IC <- setNames(vector('list', length(patient.str)), patient.str)
  for (j in patient.str) {
    pat.inds <- resids$resids.all[get(gID) == j, which=TRUE]
    IC[[j]] <- foreach(i=seq_len(kNumSubj[j]), .combine=combFun) %dopar% {
      resids.aop <- resids[c(control.inds, pat.inds[i])]
      resids.aop$resids.all[, eval(gID) := control.value]
      resids.aop$resids.all <- droplevels(resids.aop$resids.all)
      resids.aop$Group <- resids.aop$resids.all[, levels(get(gID))]
      setkeyv(resids.aop$resids.all, gID)
      new.corr <- corr.matrix(resids.aop, densities=0.1)$R[, , 1L]
      diffFun(corr.mat, new.corr)
    }
    IC[[j]] <- cbind(resids$resids.all[j, c(..sID, ..gID)], IC[[j]])
  }
  DT <- rbindlist(IC)
  if (level == 'global') {
    setnames(DT, 'V2', 'IC')
  } else if (level == 'regional') {
    DT <- melt(DT, id.vars=c(sID, gID), variable.name='region', value.name='RC')
  }
  out <- list(method='Add one patient', level=level, DT=DT)
  class(out) <- c('IC', class(out))
  return(out)
}

#' Print a summary of individual contribution estimates
#'
#' The \code{summary} method prints the group/region-wise means and standard
#' deviations.
#'
#' @param object,x A \code{IC} object
#' @param region Character vector specifying which regions' IC's to print. Only
#'   relevant if \code{method='Leave one out'}
#' @param ... Unused
#' @inheritParams summary.bg_GLM
#' @export
#' @rdname individ_contrib

summary.IC <- function(object, region=NULL, digits=max(3L, getOption('digits') - 2L), ...) {
  avg <- diff_from_mean <- IC <- Max <- med <- Min <- RC <- se <- stdev <- NULL
  gID <- getOption('bg.group')
  object$digits <- digits
  DT.sum <- copy(object$DT)
  if (object$level == 'regional') {
    regions <- if (is.null(region)) region.names(DT.sum) else region
    object$regions <- regions
    DT.sum <- droplevels(DT.sum[region %in% regions])
    DT.sum[, Min := min(RC), by=list(get(gID), region)]
    DT.sum[, med := median(RC), by=list(get(gID), region)]
    DT.sum[, avg := mean(RC), by=list(get(gID), region)]
    DT.sum[, Max := max(RC), by=list(get(gID), region)]
    DT.sum[, stdev := sd(RC), by=list(get(gID), region)]
    DT.sum[, se := stdev / sqrt(.N), by=list(get(gID), region)]
    outliers <- DT.sum[, .SD[RC > avg + 2 * stdev], by=list(get(gID), region)]
    setnames(outliers, 'get', gID)
    outliers.reg <- outliers[, .N, by=region]
    outliers.reg.vec <- with(outliers.reg, structure(N, names=as.character(region)))
    object$outliers <- list(DT=outliers, region=outliers.reg.vec)
  } else if (object$level == 'global') {
    DT.sum[, avg := mean(IC), by=gID]
    outliers <- DT.sum[, .SD[IC > avg + 2 * sd(IC)], by=gID]
    outliers[, diff_from_mean := IC - avg]
    outliers[, avg := NULL]
    DT.sum[, avg := NULL]
    object$outliers$DT <- outliers
  }
  object$DT.sum <- DT.sum

  # Calculate some group descriptive statistics
  if (object$level == 'global') {
    grps <- object$DT.sum[, levels(factor(get(gID)))]
    sums <- setNames(vector('list', length(grps)), grps)
    for (g in grps) {
      sums[[g]] <- object$DT.sum[get(gID) == g, quantile(IC, c(0, .1, .25, .5, .75, .9, 1))]
      sums[[g]] <- append(sums[[g]], object$DT.sum[get(gID) == g, mean(IC)], after=4L)
      names(sums[[g]]) <- c('Min.', '10%', '1st Qu.', 'Median', 'Mean', '3rd Qu.', '90%', 'Max.')
    }
    sums <- t(abind::abind(sums, along=2L))
    object$sums <- sums
  }
  class(object) <- c('summary.IC', class(object))
  return(object)
}

#' @aliases summary.IC
#' @method print summary.IC
#' @export

print.summary.IC <- function(x, ...) {
  sID <- getOption('bg.subject_id')
  gID <- getOption('bg.group')
  region <- IC <- NULL
  print_title_summary('Individual contributions')
  cat('Method: ', x$method, '\n')
  cat('Level: ', x$level, '\n\n')

  if (x$level == 'regional') {
    print(x$DT.sum[region == levels(region)[1L], table(get(gID))])
  } else {
    print(x$DT.sum[, table(get(gID))])
  }
  cat('\n')

  width <- getOption('width')
  dashes <- rep.int('-', width / 4)
  if (x$level == 'global') {
    message('Group summaries\n', dashes)
    print(x$sums, digits=x$digits)
    cat('\n')
  }

  if (x$level == 'regional') {
    message('# of outliers per region: (sorted in descending order)\n', dashes)
    print(sort(x$outliers$region, decreasing=TRUE))
    cat('\n')
    DT <- x$DT.sum[, .SD[1L, !c(sID, 'RC'), with=FALSE], by=list(get(gID), region)]
    setnames(DT, c('get', 'med', 'avg', 'stdev', 'se'), c(gID, 'Median', 'Mean', 'Std. Dev', 'Std. Err'))
    message('Region summaries\n', dashes)
    print(DT, digits=x$digits)
  } else {
    message('Outliers\n', dashes)
    print(x$outliers$DT[order(get(gID), -IC)], digits=x$digits)
  }
  invisible(x)
}


#' Plot regional contributions estiamtes
#'
#' The \code{plot} method is only valid for \emph{regional} contribution
#' estimates, and plots the average regional contribution for each
#' vertex/region.
#'
#' @param plot.type Character string indicating the type of plot; the default is
#'   to plot the mean (along with standard errors)
#' @param ids Logical indicating whether to plot Study ID's for outliers.
#'   Otherwise plots the integer index
#' @export
#' @rdname individ_contrib

plot.IC <- function(x, plot.type=c('mean', 'smooth', 'boxplot'), region=NULL, ids=TRUE, ...) {
  sID <- getOption('bg.subject_id')
  gID <- getOption('bg.group')
  RC <- avg <- se <- ind <- mark <- IC <- NULL
  DT <- summary(x)$DT.sum
  kNumGroups <- DT[, nlevels(factor(get(gID)))]
  grps <- DT[, levels(factor(get(gID)))]
  leg.pos <- if (kNumGroups == 1L) 'none' else 'bottom'

  if (x$level == 'regional') {
    xlabel <- 'Region'
    ylabel <- 'Regional contribution'
    regions <- if (is.null(region)) region.names(DT) else region
    txtsize <- if (length(regions) > 50) 6 else 9

    # 'base' plotting
    if (!requireNamespace('ggplot2', quietly=TRUE)) {
      DT2 <- DT[region %in% regions, .SD[1L], by=c(gID, 'region')]
      ymin <- DT2[, min(avg - se)]
      ymax <- DT2[, max(avg + se)]
      if (kNumGroups > 1L) par(mar=c(8.6, 4.1, 2.1, 0.4), xpd=TRUE)
      plot(0, type='n', xlim=c(0, length(regions)), ylim=extendrange(c(ymin, ymax)),
           xlab=xlabel, ylab=ylabel, xaxt='n')
      for (i in seq_len(kNumGroups)) {
        DT2[get(gID) == grps[i], lines(avg, col=plot.cols[i])]
        DT2[get(gID) == grps[i],
            polygon(x=c(region, rev(region)), y=c(avg + se, rev(avg - se)),
                    col=adjustcolor(plot.cols[i], alpha.f=0.4), border=plot.cols[i])]
      }
      text(x=seq_along(regions), par('usr')[3L], labels=DT2$region,
           srt=45, pos=1, offset=0.8, cex=0.5, xpd=TRUE)
      if (kNumGroups > 1L) {
        legend('bottom', title=gID, grps, fill=plot.cols[1L:kNumGroups],
               inset=c(0, -0.35), horiz=TRUE)
      }

    # 'ggplot2' plotting
    } else {
      plot.type <- match.arg(plot.type)
      if (plot.type == 'boxplot') {
        p <- ggplot2::ggplot(DT[region %in% regions], ggplot2::aes(x=region, y=RC)) +
          ggplot2::geom_boxplot(ggplot2::aes(fill=get(gID), group=interaction(get(gID), region)))

      } else {
        p <- ggplot2::ggplot(DT[region %in% regions], ggplot2::aes(x=region, col=get(gID), group=get(gID)))
        if (plot.type == 'smooth') {
          p <- p + ggplot2::stat_smooth(method='loess', ggplot2::aes(y=RC))

        } else if (plot.type == 'mean') {
          DT[, avg := mean(RC), by=list(get(gID), region)]
          DT[, se := sd(RC) / sqrt(.N), by=list(get(gID), region)]
          p <- p + ggplot2::geom_line(ggplot2::aes(y=avg)) +
            ggplot2::geom_ribbon(ggplot2::aes(ymin=avg-se, ymax=avg+se, fill=get(gID)), alpha=0.5)
        }
      }
    }

  } else {
    xlabel <- 'Subject'
    ylabel <- 'Individual contribution'
    n <- dim(DT)[1L]
    txtsize <- if (n > 50) 9 else 12
    if (isTRUE(ids)) {
      DT[, ind := get(sID)]
    } else {
      spec <- paste0('%0', floor(log10(n) + 1), 'i')
      DT[, ind := sprintf(spec, .I)]
    }
    DT[, mark := ifelse(IC > mean(IC) + 2*sd(IC), 1, 0)]
    DT[mark == 0, ind := '']
    DT[, mark := as.factor(mark)]

    # 'base' plotting
    if (!requireNamespace('ggplot2', quietly=TRUE)) {
      if (kNumGroups > 1L) par(mar=c(8.6, 4.1, 2.1, 0.4), xpd=TRUE, xaxt='n')
      plot(0, type='n', xlim=c(0, n), ylim=extendrange(DT$IC),
           xlab=xlabel, ylab=ylabel, xaxt='n')
      DT[, plot(IC, type='p', pch=19, col=plot.cols[get(gID)])]
      text(x=seq_len(n), par('usr')[3L], labels=DT[, get(sID)],
           srt=45, pos=1, offset=0.8, cex=0.5, xpd=TRUE)
      if (kNumGroups > 1L) {
        legend('bottom', title=gID, grps, fill=plot.cols[1L:kNumGroups],
               inset=c(0, -0.35), horiz=TRUE)
      }

    # 'ggplot2' plotting
    } else {
      textfun <- if (!requireNamespace('ggrepel', quietly=TRUE)) ggplot2::geom_text
        else ggrepel::geom_text_repel
      p <- ggplot2::ggplot(DT, ggplot2::aes(x=get(sID), y=IC, col=get(gID))) +
        textfun(ggplot2::aes(label=ind), size=3) +
        ggplot2::geom_point(ggplot2::aes(shape=mark, size=mark)) +
        ggplot2::scale_color_manual(name=gID, labels=grps, values=plot.cols[1L:kNumGroups]) +
        ggplot2::scale_shape_manual(name=gID, labels=grps, values=c(20, 17)) +
        ggplot2::scale_size_manual(name=gID, labels=grps, values=c(2, 3))
    }
  }

  ptitle <- paste0(ylabel, 's, ', tolower(x$method), ' method')
  if (!requireNamespace('ggplot2', quietly=TRUE)) {
    title(ptitle)
  } else {
    p <- p +
      ggplot2::labs(x=xlabel, y=ylabel, title=ptitle, fill=gID, col=gID) +
      ggplot2::theme(legend.position=leg.pos,
                     panel.grid.major=ggplot2::element_blank(), panel.grid.minor=ggplot2::element_blank(),
                     axis.text.x=ggplot2::element_text(size=txtsize, angle=45, vjust=0.5),
                     plot.title=ggplot2::element_text(hjust=0.5, face='bold'))
    return(p)
  }
}

#' Import data for structural connectivity analysis
#'
#' Given a directory, atlas name, and imaging modality/structural metric, this
#' function imports data for structural connectivity analysis. It expects files
#' containing a table of region-wise structural MRI measures (e.g., mean
#' cortical thickness), with one file for each hemisphere. The first column of
#' all files should contain the \emph{subject ID}; the column name will be
#' changed to the value of \code{getOption('bg.subject_id')}.
#'
#' The files should have specific names; the second in the following list is
#' only required for atlases/parcellations that include \emph{subcortical gray
#' matter} (e.g., \code{dk.scgm}).
#' \itemize{
#'   \item \code{${parcellation}_${hemi}_${modality}.csv} for cortical volume,
#'     thickness, surface area, or local gyrification index (LGI). Here,
#'     \code{${parcellation}} can be \code{aparc}, \code{aparc.DKTatlas40},
#'     or \code{aparc.a2009s}. For example, for cortical thickness with the
#'     \emph{Desikan-Killiany} atlas, the filename should be
#'     \code{aparc_lh_thickness.csv}. If you are using a custom atlas, see the
#'     \emph{Note} below. The \code{${hemi}} variable is either \code{lh} or
#'     \code{rh}. Finally, \code{${modality}} should be either \code{volume},
#'     \code{thickness}, \code{area}, or \code{lgi}.
#'   \item \code{asegstats.csv} for SCGM volume
#' }
#'
#' @note When using a custom atlas, the name of the atlas's data.table should
#'   match the \code{${parcellation}} portion of the filename (specification
#'   shown above). Furthermore, it must conform to the output of Freesurfer's
#'   \code{aparcstats2table} (and \code{asegstats2table}, if applicable).
#'   Otherwise, please contact me for inclusion of a different data type.
#'
#' @note The subject ID column will be zero-padded (to the left) to avoid issues
#'   when the variable is numeric; this ensures that all ID's will have the same
#'   number of characters and sorting will be done properly.
#'
#' @param datadir The path name of the directory containing the data files
#' @param atlas Character string specifying the atlas in use. For a custom
#'   atlas, please specify \code{'custom'}, and provide the name to the
#'   \code{custom.atlas} argument
#' @param modality The structural imaging measure (default: \code{'thickness'})
#' @param exclude.subs Vector indicating the subjects to exclude, if any
#'   (default: \code{NULL})
#' @param custom.atlas Character string specifying the name of the R object for
#'   the atlas in use, if \code{atlas='custom'} was also supplied (default:
#'   \code{NULL})
#' @export
#'
#' @return A list containing:
#'   \item{atlas}{Character string}
#'   \item{modality}{Character string}
#'   \item{lhrh}{A \code{data.table} of structural MRI measures for both
#'     hemispheres}
#'   \item{aseg}{A \code{data.table} of structural MRI measures for subcortical
#'     gray matter, if applicable}
#'   \item{subs.excluded}{Vector of subject ID's that were excluded}
#'   \item{subs.missing}{Vector of subject ID's that are not present in \emph{both}
#'     the cortical and subcortical tables (if applicable)}
#'
#' @family Structural covariance network functions
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}
#' @examples
#' \dontrun{
#'   raw_data <- import_scn('/home/cwatson/data', atlas='dkt',
#'                          exclude.subs=c('con07', 'con23', 'pat15'))
#' }

import_scn <- function(datadir, atlas, modality='thickness', exclude.subs=NULL, custom.atlas=NULL) {
  subs.missing <- scgm <- NULL

  atl <- c('dk', 'dkt', 'destrieux')
  atl <- c(atl, paste0(atl, '.scgm'))
  atlas <- match.arg(atlas, choices=c(atl, 'custom'))
  if (atlas == 'custom') {
    stopifnot(!is.null(custom.atlas), exists(custom.atlas))
    atlas <- parc <- custom.atlas
  } else {
    parc <- switch(atlas,
                   dk=, dk.scgm='aparc',
                   dkt=, dkt.scgm='aparc.DKTatlas40',
                   destrieux=, destrieux.scgm='aparc.a2009s')
  }

  # Cortical measures, both hemispheres
  files <- file.path(datadir, paste0(parc, c('_lh_', '_rh_'), modality, '.csv'))
  lh <- update_fs_names(files[1L], modality, parc, 'lh', exclude.subs)
  rh <- update_fs_names(files[2L], modality, parc, 'rh', exclude.subs)
  lhrh <- merge(lh, rh)

  # Subcortical measures
  if (grepl('scgm', atlas)) {
    asegfile <- file.path(datadir, 'asegstats.csv')
    scgm <- update_fs_names(asegfile, 'aseg', parcellation=NULL, hemi=NULL, exclude.subs=exclude.subs)
    sID <- getOption('bg.subject_id')
    subs.missing <- union(setdiff(scgm[[sID]], lhrh[[sID]]), setdiff(lhrh[[sID]], scgm[[sID]]))
  }

  return(list(atlas=atlas, modality=modality, lhrh=lhrh, aseg=scgm,
              subs.excluded=exclude.subs, subs.missing=subs.missing))
}

#' Update column names in a Freesurfer table
#'
#' Updates column names by abbreviating to match with the names in the
#' \code{brainGraph} atlases.
#' @keywords internal

update_fs_names <- function(filename, modality, parcellation, hemi, exclude.subs) {
  sID <- getOption('bg.subject_id')
  stopifnot(file.exists(filename))
  DT <- fread(filename, colClasses=list(character=1L))
  setnames(DT, 1L, sID)

  if (modality == 'aseg') {
    DT <- DT[, 1L:15L, with=FALSE]
    full <- c('Left-', 'Right-', 'Thalamus.Proper', 'Putamen', 'Pallidum',
              'Caudate', 'Hippocampus', 'Amygdala', 'Accumbens.area')
    abbrev <- c('l', 'r', 'THAL', 'PUT', 'PALL', 'CAUD', 'HIPP', 'AMYG', 'ACCU')
    for (i in seq_along(full)) names(DT) <- gsub(full[i], abbrev[i], names(DT))
  } else {
    # Remove the "mean" column, if present
    if (any(grepl('mean', ignore.case=TRUE, names(DT)))) {
      DT <- DT[, -grep('mean', ignore.case=TRUE, names(DT)), with=FALSE]
    }
    names(DT) <- gsub(paste0(hemi, '[_|\\.]'), substr(hemi, 1L, 1L), names(DT))
    names(DT) <- gsub(paste0('_', modality), '', names(DT))

    if (parcellation == 'aparc.a2009s') {
      names(DT) <- gsub('-', '.', names(DT))
    } else {
      full <- c('bankssts', 'caudalanteriorcingulate', 'caudalmiddlefrontal', 'precuneus', 'cuneus',
                'entorhinal', 'fusiform', 'inferiorparietal', 'inferiortemporal', 'isthmuscingulate',
                'lateraloccipital', 'lateralorbitofrontal', 'lingual', 'medialorbitofrontal',
                'middletemporal', 'parahippocampal', 'paracentral', 'parsopercularis',
                'parsorbitalis', 'parstriangularis', 'pericalcarine', 'postcentral',
                'posteriorcingulate', 'precentral', 'rostralanteriorcingulate',
                'rostralmiddlefrontal', 'superiorfrontal', 'superiorparietal', 'superiortemporal',
                'supramarginal', 'frontalpole', 'temporalpole', 'transversetemporal', 'insula')
      abbrev <- c('BSTS', 'cACC', 'cMFG', 'PCUN', 'CUN', 'ENT', 'FUS', 'IPL', 'ITG', 'iCC', 'LOG',
                  'LOF', 'LING', 'MOF', 'MTG', 'PARH', 'paraC', 'pOPER', 'pORB', 'pTRI', 'periCAL',
                  'postC', 'PCC', 'preC', 'rACC', 'rMFG', 'SFG', 'SPL', 'STG', 'SMAR', 'FP', 'TP',
                  'TT', 'INS')
      for (i in seq_along(full)) names(DT) <- gsub(full[i], abbrev[i], names(DT))
    }
  }

  if (!is.null(exclude.subs)) DT <- DT[!get(sID) %in% exclude.subs]
  DT[, eval(sID) := check_sID(get(sID))]
  setkeyv(DT, sID)
  return(DT)
}

#' Fit General Linear Models at each vertex of a graph
#'
#' \code{brainGraph_GLM} specifies and fits a General Linear Model (GLM) at each
#' vertex for a given vertex measure (e.g. \emph{degree}) or at the graph-level
#' (e.g., \emph{global efficiency}). Given a contrast matrix or list of
#' contrast(s), and contrast type (for t- or F-contrast(s), respectively) it
#' will calculate the associated statistic(s) for the given contrast(s).
#'
#' The \code{measure} argument will be the graph- or vertex-level measure of
#' interest. Often, this will serve as the model's \emph{outcome} (or dependent,
#' or response) variable; i.e., the variable typically denoted by \emph{y} in
#' GLMs. In other cases, you may wish to choose some other variable as the
#' outcome; e.g., IQ, age, etc. Then you could test for a direct association
#' between the network measure and outcome of interest, or test for another
#' association while adjusting for the network metric. For these applications,
#' you must provide the variable name via the \code{outcome} argument. This is
#' analogous to \code{-evperdat} in FSL's PALM and to \code{--pvr} in
#' FreeSurfer.
#'
#' @section Design matrix:
#' The GLM's \emph{design matrix} will often be identical to the \emph{model
#' matrix} associated with \code{lm} objects (if \dQuote{dummy} coding, the
#' default, is used) and is created from the input \code{data.table} and
#' arguments passed to \code{\link{brainGraph_GLM_design}}. The first column
#' should have the name of \code{getOption('bg.subject_id')} and its values
#' must match the \emph{name} graph-level attribute of the input graphs. The
#' covariates table must be supplied even if you provide your own design matrix
#' \code{X}.  If \code{level='vertex'} and \code{outcome == measure}, there will
#' be a single design for all regions but a separate model for each region
#' (since the graph measure varies by region). If \code{level='vertex'} and
#' \code{outcome != measure}, there will be a separate design (and, therefore, a
#' separate model) for each region even though the outcome is the same in all
#' models.
#'
#' @section Contrasts and statistics:
#' Either t- or F-contrasts can be calculated (specified by \code{con.type}).
#' Multiple t-contrasts can be specified by passing a multi-row \emph{matrix} to
#' \code{contrasts}. Multiple F-contrasts can be specified by passing a
#' \emph{list} of matrices; all matrices must have the same number of columns.
#' All F-contrasts are necessarily \emph{two-sided}; t-contrasts can be any
#' direction, but only one can be chosen per function call.
#' If you choose \code{con.type="f"}, the calculated effect size is represented
#' by the \code{ESS} (\dQuote{extra sum of squares}), the additional variance
#' explained for by the model parameters of interest (as determined by the
#' contrast matrix). The standard error for F-contrasts is the sum of squared
#' errors of the \emph{full model}.
#'
#' @section Non-parametric permutation tests:
#' You can calculate permutations of the data to build a null distribution of
#' the maximum statistic which corrects for multiple testing. To account for
#' complex designs, the design matrix must be \emph{partitioned} into covariates
#' of interest and nuisance; the default method is the \emph{Beckmann} method.
#' The default permutation strategy is that of Freedman & Lane (1983), and is
#' the same as that in FSL's \emph{randomise}. See \code{\link{randomise}}.
#'
#' @param g.list A \code{brainGraphList} object
#' @param covars A \code{data.table} of covariates
#' @param measure Character string of the graph measure of interest
#' @param contrasts Numeric matrix (for T statistics) or list of matrices (for F
#'   statistics) specifying the contrast(s) of interest; if only one contrast is
#'   desired, you can supply a vector (for T statistics)
#' @param con.type Character string; either \code{'t'} or \code{'f'} (for t or
#'   F-statistics). Default: \code{'t'}
#' @param outcome Character string specifying the name of the outcome variable,
#'   if it differs from the graph metric (\code{measure})
#' @param X Numeric matrix, if you wish to supply your own design matrix.
#'   Ignored if \code{outcome != measure}.
#' @param con.name Character vector of the contrast name(s); if \code{contrasts}
#'   has row/list names, those will be used for reporting results
#' @param alternative Character string, whether to do a two- or one-sided test.
#'   Default: \code{'two.sided'}
#' @param alpha Numeric; the significance level. Default: 0.05
#' @param level Character string; either \code{vertex} (default) or
#'   \code{graph}
#' @param permute Logical indicating whether or not to permute group labels.
#'   Default: \code{FALSE}
#' @param perm.method Character string indicating the permutation method.
#'   Default: \code{'freedmanLane'}
#' @param part.method Character string; the method of partitioning the design
#'   matrix into covariates of interest and nuisance. Default: \code{'beckmann'}
#' @param N Integer; number of permutations to create. Default: \code{5e3}
#' @param perms Matrix of permutations, if you would like to provide your own.
#'   Default: \code{NULL}
#' @param long Logical indicating whether or not to return all permutation
#'   results. Default: \code{FALSE}
#' @param ... Arguments passed to \code{\link{brainGraph_GLM_design}}
#' @export
#' @importFrom permute shuffleSet
#'
#' @return An object of class \code{bg_GLM} containing some input-specific
#'   variables (\code{level}, \code{outcome}, \code{measure}, \code{con.type},
#'   \code{contrasts}, \code{con.name}, \code{alt}, \code{alpha},
#'   \code{permute}, \code{perm.method}, \code{part.method}, \code{N}) in
#'   addition to:
#'   \item{DT.Xy}{A data table from which the design matrices are created and
#'     the outcome variable, for all regions.}
#'   \item{X}{A named numeric matrix or a 3D array of the design matrix.
#'     Rownames are subject IDs, column names are predictor variables, and
#'     dimnames along the 3rd dimension are region names (if applicable). This
#'     is a 3D array only if \code{outcome != measure} and \code{level ==
#'     'vertex'}.}
#'   \item{y}{A named numeric matrix of the outcome variable. Rownames are Study
#'     IDs and column names are regions. There will be multiple columns only if
#'     \code{outcome == measure} and \code{level == 'vertex'}.}
#'   \item{DT}{A data table with an entry for each vertex (region) containing
#'     statistics of interest}
#'   \item{removed.subs}{A named integer vector in which the names are subject
#'     ID's of those removed due to incomplete data (if any). The integers
#'     correspond to the row number in the input \code{covars} table.}
#'   \item{runX}{If \code{outcome != measure} and \code{level == 'vertex'}, this
#'     will be a character vector of the regions for which the design matrix is
#'     invertible. Otherwise, it is \code{NULL}.}
#'   \item{runY}{Character vector of the regions for which the outcome variable
#'     has 0 variability. For example, if \code{level='vertex'} and
#'     \code{measure='degree'}, some regions may be disconnected or have the
#'     same degree for all subjects.}
#'   \item{atlas}{Character string of the atlas used (guessed based on the
#'     vertex count).}
#'   \item{perm}{A list containing: \emph{null.dist} (the null distribution of
#'     maximum statistics), \emph{thresh} (the statistic value corresponding
#'     to the \eqn{100 \times (1 - \alpha)}th\% percentile of the null
#'     distribution)}
#'
#' @name GLM
#' @rdname glm
#' @family GLM functions
#' @family Group analysis functions
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}
#'
#' @examples
#' \dontrun{
#' conmat <- matrix(c(0, 0, 0, 1), nrow=1)
#' rownames(conmat) <- 'Control > Patient'
#'
#' res.lm <- brainGraph_GLM(g[[6]], covars=covars.all[tract == 1],
#'   measure='strength', contrasts=conmat, alt='greater', permute=TRUE, long=TRUE)
#' }
brainGraph_GLM <- function(g.list, covars, measure, contrasts, con.type=c('t', 'f'),
                           outcome=NULL, X=NULL, con.name=NULL,
                           alternative=c('two.sided', 'less', 'greater'),
                           alpha=0.05, level=c('vertex', 'graph'),
                           permute=FALSE,
                           perm.method=c('freedmanLane', 'terBraak', 'smith',
                                         'draperStoneman', 'manly', 'stillWhite'),
                           part.method=c('beckmann', 'guttman', 'ridgway'),
                           N=5e3, perms=NULL, long=FALSE, ...) {
  level <- match.arg(level)
  ctype <- match.arg(con.type)
  alt <- if (ctype == 'f') 'two.sided' else match.arg(alternative)

  # Initial GLM setup
  glmSetup <- setup_glm(g.list, level, covars, X, contrasts, ctype, con.name, measure, outcome, ...)
  X <- glmSetup$X; y <- glmSetup$y; DT.Xy <- glmSetup$DT.Xy; outcome <- glmSetup$outcome
  contrasts <- glmSetup$contrasts; nC <- glmSetup$nC

  #-------------------------------------
  # Do the model fitting/estimation
  #-------------------------------------
  dimX <- dim(X)
  namesX <- dimnames(X)
  # Handle the case when there is a different design matrix for each region
  if (length(dimX) == 3L) {
    QR <- qr(X)
    runX <- check_if_singular(QR)  # Don't fit a model for regions w/ a rank-deficient design
    runY <- namesX[[3L]]
    if (length(runX) == 0L) {
      print(head(X[, , 1L]), digits=3L)
      stop('Design matrices are rank-deficient for all regions; please check your data')
    }

    fits <- fastLmBG_3d(X, y, runX, QR[runX])
    names(fits$sigma) <- colnames(fits$coefficients) <- colnames(fits$fitted.values) <- runX
    rownames(fits$fitted.values) <- rownames(fits$residuals) <- namesX[[1L]]
    XtX <- aperm(fits$cov.unscaled, c(3L, 1L, 2L))
    fits$var.covar <- aperm(fits$sigma^2 * XtX, c(2L, 3L, 1L))
    fits$se <- sqrt(apply(fits$var.covar, 3L, diag_sq, fits$rank))
    rownames(fits$coefficients) <- rownames(fits$se) <- namesX[[2L]]

    # If any regions were skipped, add NaN/NA into the correct spot in the stats
    if (length(runX) < dimX[3L]) fits <- add_nans(fits, dimX, namesX, runX)

  # Single design matrix for all regions
  } else {
    runY <- names(which(apply(y, 2L, function(r) diff(range(r))) != 0))
    runX <- dimnames(y)[[2L]]
    fits <- fastLmBG(X, y)
    names(fits$sigma) <- colnames(fits$coefficients) <- colnames(fits$fitted.values) <- runX
    fits$var.covar <- aperm(outer(fits$sigma^2, fits$cov.unscaled), c(2L, 3L, 1L))
    fits$se <- sqrt(apply(fits$var.covar, 3L, diag_sq, fits$rank))
    rownames(fits$coefficients) <- rownames(fits$se) <- namesX[[2L]]
  }
  obs_stats <- if (ctype == 't') fastLmBG_t(fits, contrasts, alt, alpha)
    else fastLmBG_f(fits, contrasts)

  DT.lm <- rbindlist(apply(obs_stats, 3L, as.data.table, keep.rownames='region'), idcol='Contrast')

  out <- list(level=level, X=X, y=y, outcome=outcome, measure=measure, DT.Xy=DT.Xy,
              con.type=ctype, contrasts=contrasts, con.name=glmSetup$con.name, alt=alt, alpha=alpha,
              DT=DT.lm, removed.subs=glmSetup$incomp, permute=permute, runX=runX, runY=runY)
  out$atlas <- guess_atlas(g.list$graphs[[1L]])
  out <- c(out, fits)
  class(out) <- c('bg_GLM', class(out))
  if (isFALSE(permute)) return(out)

  #-------------------------------------
  # Permutation testing
  #-------------------------------------
  perm.method <- match.arg(perm.method)
  part.method <- match.arg(part.method)
  if (is.null(perms) || dim(perms)[2L] != dimX[1L]) perms <- shuffleSet(n=dimX[1L], nset=N)

  n <- dimX[1L]; p <- fits$rank; dfR <- n - p; ny <- length(runX)
  null.dist <- if (length(dimX) == 3L) randomise_3d(perm.method, part.method, N, perms, X, y,
                                                    contrasts, ctype, nC, runX, n, p, ny, dfR)
    else randomise(perm.method, part.method, N, perms, X, y,
                   contrasts, ctype, nC, n=n, p=p, ny=ny, dfR=dfR)

  myMax <- switch(alt, two.sided=colMaxAbs, less=colMin, greater=colMax)
  null.max <- apply(null.dist, 3L, myMax, ny)
  mySort <- sortfun(alt)
  index <- floor((1 - alpha) * N) + 1L
  null.thresh <- apply(null.max, 2L, mySort, index)
  names(null.thresh) <- glmSetup$con.name
  compfun <- switch(alt,
                    two.sided=function(x, y) (rowSums(abs(x) >= abs(y), na.rm=TRUE) + 1L),
                    less=function(x, y) (rowSums(x <= y, na.rm=TRUE) + 1L),
                    greater=function(x, y) (rowSums(x >= y, na.rm=TRUE) + 1L))
  p.perm <- vapply(seq_len(nC), function(x) compfun(t(matrix(null.max[, x], N, ny)), obs_stats[runX, 3L, x]),
                   numeric(ny)) / (N + 1L)
  dim(p.perm) <- c(ny, nC)
  dimnames(p.perm) <- list(runX, glmSetup$con.name)
  DT.lm[!is.nan(p), p.perm := c(p.perm[intersect(runX, runY), ])]

  perm <- list(thresh=null.thresh)
  if (isTRUE(long)) perm <- c(perm, list(null.dist=null.dist, null.max=null.max))
  out <- c(out, list(perm.order=perms, perm.method=perm.method, part.method=part.method, N=N, perm=perm))
  class(out) <- c('bg_GLM', class(out))
  return(out)
}

################################################################################
# HELPER FUNCTIONS
################################################################################

#' Helper functions to set-up for GLM analyses
#'
#' \code{setup_glm} is used to setup the data/objects for any function that uses
#' the main GLM functionality in \code{brainGraph}.
#'
#' This function: removes unused levels from \code{covars} and \code{DT.y.m},
#' removes subjects with incomplete data, creates a design matrix (if not
#' supplied), and supplies names to the contrast matrix.
#'
#' @inheritParams GLM
#' @keywords internal
#' @name GLM helpers
#' @rdname glm_helpers

setup_glm <- function(g.list, level, covars, X, contrasts, con.type, con.name, measure, outcome, ...) {
  region <- NULL
  sID <- getOption('bg.subject_id')

  # Get the outcome variable(s) into a data.table
  if (!is.brainGraphList(g.list)) try(g.list <- as_brainGraphList(g.list))
  g.list <- g.list[]

  DT.y <- glm_data_table(g.list, level, measure)
  DT.y.m <- melt(DT.y, id.vars=sID, variable.name='region', value.name=measure)

  covars <- droplevels(copy(covars))
  if (!hasName(covars, sID)) covars[, eval(sID) := seq_len(dim(covars)[1L])]
  covars[, eval(sID) := check_sID(get(sID))]
  incomp <- covars[!complete.cases(covars), which=TRUE]
  names(incomp) <- covars[incomp, get(sID)]
  DT.Xy <- DT.y.m[covars, on=sID]
  if (length(incomp) > 0L) DT.Xy <- DT.Xy[!get(sID) %in% names(incomp)]

  regions <- DT.y.m[, levels(region)]
  # Swap the outcome and measure variables, if outcome is not a network metric
  if (!is.null(outcome)) {
    setcolorder(DT.Xy, c(sID, 'region', setdiff(names(covars), c(sID, outcome)), measure, outcome))

    # Get all design matrices into a 3-D array
    X <- setNames(vector('list', length(regions)), regions)
    for (rgn in regions) {
      X[[rgn]] <- brainGraph_GLM_design(DT.Xy[region == rgn, !c('region', outcome), with=FALSE], ...)
    }
    attrs <- attributes(X[[rgn]])[-c(1L, 2L)]  # Don't include "dim" and "dimnames"
    X <- drop(abind::abind(X, along=3L))
    attributes(X) <- c(attributes(X), attrs)
  } else {
    if (is.null(X)) X <- brainGraph_GLM_design(DT.Xy[region == regions[1L], !c('region', measure), with=FALSE], ...)
    outcome <- measure
  }

  # Create a matrix of outcome variables
  y <- as.matrix(dcast(DT.Xy, paste(sID, '~ region'), value.var=outcome), rownames=sID)
  y <- y[dimnames(X)[[1L]], , drop=FALSE]  # Since dcast changes the row order if input is out of order
  if (level == 'graph' || outcome != measure) {
    y <- y[, 1L, drop=FALSE]
    if (outcome != measure) dimnames(y)[[2L]] <- outcome
  }

  tmp <- contrast_names(contrasts, con.type, con.name, X)
  out <- list(incomp=incomp, X=X, y=y, contrasts=tmp$contrasts, con.name=tmp$con.name,
              nC=tmp$nC, DT.Xy=DT.Xy, outcome=outcome)

  return(out)
}

#' Set contrast and column names for GLM analysis
#'
#' \code{contrast_names} checks the dimensions of contrasts, generates contrast
#' names, and sets column names for GLM functions. For F-contrasts, if a
#' \code{matrix} is given, it converts it to a \code{list} to simplify processing
#' later.
#' @return \code{contrast_names} -- list containing the contrasts (matrix or
#'   list), contrast names, and number of contrasts
#' @keywords internal
#' @rdname glm_helpers

contrast_names <- function(contrasts, con.type, con.name, X) {
  if (!is.list(contrasts)) {
    if (is.vector(contrasts)) contrasts <- t(contrasts)
    contrasts <- if (con.type == 't') matrix2list(contrasts) else list(contrasts)
  } else {
    stopifnot(con.type == 'f')
  }

  ncols <- vapply(contrasts, ncol, integer(1L))
  if (any(ncols != dim(X)[2L])) {
    df <- data.frame(con.number=seq_along(contrasts), ncols=ncols)
    print(df, row.names=FALSE)
    stop('Design matrix has ', dim(X)[2L], ' columns but contrasts do not match (see above)')
  }
  nC <- length(contrasts)

  if (is.null(con.name)) {
    if (is.null(names(contrasts))) names(contrasts) <- paste('Contrast', seq_len(nC))
    con.name <- names(contrasts)
  } else {
    if (length(con.name) < nC) {
      con.name <- c(con.name, paste('Contrast', seq_len(nC)[-(seq_along(con.name))]))
    }
    names(contrasts) <- con.name
  }
  for (i in seq_len(nC)) dimnames(contrasts[[i]])[[2L]] <- dimnames(X)[[2L]]

  if (con.type == 't') contrasts <- abind::abind(contrasts, along=1L)

  return(list(contrasts=contrasts, con.name=con.name, nC=nC))
}

#' \code{check_if_singular} returns the names of the regions in which the design
#' matrix is of full rank (i.e., non-singular) and therefore is invertible.
#'
#' @param QR List of QR decompositions for each design matrix
#' @keywords internal
#' @rdname glm_helpers

check_if_singular <- function(QR) {
  stopifnot(is.list(QR), all(lapply(QR, is.qr)))
  p <- vapply(QR, function(x) dim(x$qr)[2L], integer(1L))
  rk <- vapply(QR, function(x) x$rank, integer(1L))
  names(which(p == rk))
}

################################################################################
# S3 METHODS FOR "bg_GLM"
################################################################################

#' @export
#' @rdname glm

print.bg_GLM <- function(x, ...) {
  cat('\nA', paste0(x$level, '-level'), '"bg_GLM" object\n')
  print_model_summary(x)
  print_contrast_type_summary(x)
  invisible(x)
}

#' Print a summary from brainGraph_GLM analysis
#'
#' The \code{summary} method prints the results, only for which
#' \eqn{p < \alpha}, where \code{alpha} comes from the \code{bg_GLM} object.
#' \dQuote{Simple} P-values are used by default, but you may change this to the
#' FDR-adjusted or permutation P-values via the function argument \code{p.sig}.
#' You may also choose to subset by \emph{contrast}.
#'
#' @param object,x A \code{bg_GLM} object
#' @param p.sig Character string specifying which P-value to use for displaying
#'   significant results (default: \code{p})
#' @param contrast Integer specifying the contrast to plot/summarize; defaults
#'   to showing results for all contrasts
#' @param digits Integer specifying the number of digits to display for P-values
#' @param print.head Logical indicating whether or not to print only the first
#'   and last 5 rows of the statistics tables (default: \code{TRUE})
#' @export
#' @rdname glm

summary.bg_GLM <- function(object, p.sig=c('p', 'p.fdr', 'p.perm'), contrast=NULL, alpha=object$alpha,
                           digits=max(3L, getOption('digits') - 2L), print.head=TRUE, ...) {
  stopifnot(inherits(object, 'bg_GLM'))
  threshold <- NULL
  object$p.sig <- match.arg(p.sig)
  object$printCon <- contrast
  object$digits <- digits
  object$print.head <- print.head
  DT.sum <- object$DT[get(object$p.sig) < alpha]
  if (hasName(DT.sum, 'threshold')) DT.sum[, threshold := NULL]

  # Change column order and names for `DT.sum`
  newcols <- c('Contrast', 'region', 'gamma', 'ci.low', 'ci.high',
               'se', 'stat', 'p', 'p.fdr', 'p.perm')
  oldnames <- c('region', 'gamma', 'se', 'stat', 'p', 'p.fdr', 'p.perm', 'ci.low', 'ci.high')
  newnames <- c('Region', 'Estimate', 'Std. error', 't value', 'p-value', 'p-value (FDR)', 'p-value (perm.)')
  if (isFALSE(object$permute)) {
    oldnames <- oldnames[-7L]
    newnames <- newnames[-7L]
    newcols <- newcols[-10L]
  }
  if (object$con.type == 't') {
    clp <- 100 * (1 - object$alpha)
    newnames <- c(newnames, paste0(clp, '% CI ', c('low', 'high')))
  } else if (object$con.type == 'f') {
    newcols[3L] <- oldnames[2L] <- 'ESS'
    newnames[c(2L, 4L)] <- c('Extra Sum Sq.', 'F value')
    newcols <- newcols[-c(4L, 5L)]
    oldnames <- oldnames[-grep('ci.', oldnames)]
  }
  setcolorder(DT.sum, newcols)
  setnames(DT.sum, oldnames, newnames)

  object$DT.sum <- DT.sum
  class(object) <- c('summary.bg_GLM', class(object))
  return(object)
}

#' @aliases summary.bg_GLM
#' @method print summary.bg_GLM
#' @export

print.summary.bg_GLM <- function(x, ...) {
  print_title_summary('brainGraph GLM results (', x$level, '-level)')

  print_model_summary(x)
  print_contrast_type_summary(x)
  print_subs_summary(x)

  if (isTRUE(x$permute)) {
    print_permutation_summary(x)
    cat(paste0(toupper(x$con.type), '-statistic threshold (based on the null distribution):\n'))
    print(x$perm$thresh)
  }

  # Print results for each contrast
  message('\n', 'Statistics', '\n', rep.int('-', getOption('width') / 4))
  print_contrast_stats_summary(x)

  invisible(x)
}

#' Plot GLM diagnostics for a brain network
#'
#' The \code{plot} method plots the GLM diagnostics (similar to that of
#' \code{\link[stats]{plot.lm}}). There are a total of 6 possible plots,
#' specified by the \code{which} argument; the behavior is the same as in
#' \code{\link[stats]{plot.lm}}. Please see the help for that function.
#'
#' @param region Character string specifying which region's results to
#'   plot; only relevant if \code{level='vertex'}. Default: \code{NULL}
#' @param which Integer vector indicating which of the 6 plots to print to the
#'   plot device. Default: \code{c(1:3, 5)}
#' @param ids Logical indicating whether to plot subject ID's for outliers.
#'   Otherwise plots the integer index
#' @export
#' @importFrom grid gpar grid.draw grid.newpage textGrob
#' @rdname glm
#'
#' @return The \code{plot} method returns a \emph{list} of \code{ggplot} objects
#'   (if installed) or writes the plots to a PDF in the current directory named
#'   \code{bg_GLM_diagnostics.pdf}
#' @seealso \code{\link[stats]{plot.lm}}
#'
#' @examples
#' \dontrun{
#' ## Save objects and then to multipage PDF
#' lmPlots <- plot(x)
#' ggsave('lmPlots.pdf', lmPlots)
#'
#' ## Save all the GLM sub-objects from MTPC analysis
#' res.mtpc <- mtpc(...)
#' glmPlots <- lapply(res.mtpc$res.glm, plot, which=1:6)
#' ml <- marrangeGrob(glmPlots, nrow=1, ncol=1)
#' ggsave('glmPlots.pdf', ml, width=8.5, height=11)
#' }

plot.bg_GLM <- function(x, region=NULL, which=c(1L:3L, 5L), ids=TRUE, ...) {
  cl.h <- ind <- level <- mark <- resid <- NULL
  stopifnot(inherits(x, 'bg_GLM'))
  if (!is.numeric(which) || any(which < 1L) || any(which > 6L)) stop("'which' must be in 1:6")
  show <- rep.int(FALSE, 6L)
  show[which] <- TRUE
  if (!requireNamespace('ggplot2', quietly=TRUE)) {
    mfrow <- switch(length(which),
                    c(1, 1), c(1, 2), c(1, 3),
                    c(2, 2), c(2, 3), c(2, 3))
    pdf('bg_GLM_diagnostics.pdf')
    par(mfrow=mfrow)
  } else {
    if (!requireNamespace('gridExtra', quietly=TRUE)) stop('Must install the "gridExtra" package.')
    textfun <- if (!requireNamespace('ggrepel', quietly=TRUE)) ggplot2::geom_text else ggrepel::geom_text_repel
    prows <- 1L + (length(which) > 1L)
    mytheme <- ggplot2::theme(plot.title=ggplot2::element_text(hjust=0.5),
                              legend.position='none',
                              axis.text.y=ggplot2::element_text(hjust=0.5, angle=90))
    model_formula <- split_string(formula(x))
  }

  xlabs <- c('Fitted values', 'Theoretical Quantiles', 'Fitted values', 'Obs. number',
             'Leverage', expression(Leverage~h[ii]))
  ylabs <- c('Residuals', 'Sample Quantiles', expression(sqrt(Standardized~residuals)),
             'Cook\'s distance', 'Standardized residuals', 'Cook\'s distance')
  mains <- c('Residuals vs Fitted', 'Normal Q-Q', 'Scale-Location', 'Cook\'s distance',
             'Residuals vs Leverage', expression("Cook's dist vs Leverage "*h[ii] / (1 - h[ii])))

  # Local function to plot for a single region ('ggplot2')
  #-----------------------------------------------------------------------------
  plot_single <- function(dat, region) {
    leverage <- resid.std <- cook <- ind <- mark <- fit <- leverage.tr <- NULL

    diagPlots <- vector('list', length=6L)
    # 1. Resids vs fitted
    if (show[1L]) {
      diagPlots[[1L]] <- ggplot2::ggplot(dat, ggplot2::aes(x=fit, y=resid)) +
        ggplot2::geom_point(ggplot2::aes(shape=mark)) +
        textfun(ggplot2::aes(x=fit, y=resid, label=ind), size=3) +
        ggplot2::stat_smooth(method='loess', se=FALSE, span=1, col='red') +
        ggplot2::geom_hline(yintercept=0, lty=3) +
        mytheme + ggplot2::labs(title=mains[1L], xlab=xlabs[1L], ylab=ylabs[1L])
    }

    # 2. QQ-plot
    if (show[2L]) {
      dat[order(resid.std), x := qnorm(ppoints(resid.std))]
      diagPlots[[2L]] <- ggplot2::ggplot(dat, ggplot2::aes(x=x, y=resid.std)) +
        textfun(ggplot2::aes(x=x, y=resid.std, label=ind), size=3) +
        ggplot2::geom_line(ggplot2::aes(x=x, y=x), col='gray50', lty=3) +
        ggplot2::geom_point(ggplot2::aes(shape=mark)) +
        mytheme + ggplot2::labs(title=mains[2L], x=xlabs[2L], y=ylabs[2L])
    }

    # 3. Scale-Location plot
    if (show[3L]) {
      diagPlots[[3L]] <- ggplot2::ggplot(dat, ggplot2::aes(x=fit, y=sqrt(abs(resid.std)))) +
        ggplot2::geom_point(ggplot2::aes(shape=mark)) +
        textfun(ggplot2::aes(x=fit, y=sqrt(abs(resid.std)), label=ind), size=3) +
        ggplot2::stat_smooth(method='loess', se=FALSE, col='red') +
        ggplot2::ylim(c(0, NA)) +
        mytheme + ggplot2::labs(title=mains[3L], x=xlabs[3L], y=ylabs[3L])
    }

    # 4. Cook's distance
    if (show[4L]) {
      diagPlots[[4L]] <- ggplot2::ggplot(dat, ggplot2::aes(x=seq_len(dimX[1L]), y=cook)) +
        ggplot2::geom_bar(stat='identity', position='identity') +
        ggplot2::geom_text(ggplot2::aes(y=cook, label=ind), size=3, vjust='outward') +
        mytheme + ggplot2::labs(title=mains[4L], x=xlabs[4L], y=ylabs[4L])
    }

    # 5. Residual vs Leverage plot
    if (show[5L]) {
      r.hat <- dat[, range(leverage, na.rm=TRUE)]
      hh <- seq.int(min(r.hat[1L], r.hat[2L] / 100), 1L, length.out=101L)
      dt.cook <- data.table(hh=rep.int(hh, 2L), level=rep(c(0.5, 1.0), each=101L))
      dt.cook[, cl.h := sqrt(level * p * (1 - hh) / hh), by=level]

      diagPlots[[5L]] <- ggplot2::ggplot(dat, ggplot2::aes(x=leverage, y=resid.std)) +
        ggplot2::geom_point(ggplot2::aes(shape=mark)) +
        textfun(ggplot2::aes(x=leverage, y=resid.std, label=ind), size=3) +
        ggplot2::stat_smooth(method='loess', se=FALSE, col='red') +
        ggplot2::geom_vline(xintercept=0, lty=3, col='gray50') +
        ggplot2::geom_hline(yintercept=0, lty=3, col='gray50') +
        ggplot2::geom_line(data=dt.cook[level == 0.5], ggplot2::aes(x=hh, y=cl.h), col='red', lty=2, na.rm=TRUE) +
        ggplot2::geom_line(data=dt.cook[level == 1.0], ggplot2::aes(x=hh, y=cl.h), col='red', lty=2, na.rm=TRUE) +
        ggplot2::geom_line(data=dt.cook[level == 0.5], ggplot2::aes(x=hh, y=-cl.h), col='red', lty=2, na.rm=TRUE) +
        ggplot2::geom_line(data=dt.cook[level == 1.0], ggplot2::aes(x=hh, y=-cl.h), col='red', lty=2, na.rm=TRUE) +
        ggplot2::xlim(extendrange(c(0, r.hat[2L]))) +
        ggplot2::ylim(dat[, extendrange(range(resid.std), f=0.09)]) +
        mytheme + ggplot2::labs(title=mains[5L], x=xlabs[5L], y=ylabs[5L])
    }

    # 6. Cook's dist vs leverage
    if (show[6L]) {
      mytheme$plot.title$size <- 9
      diagPlots[[6L]] <- ggplot2::ggplot(dat, ggplot2::aes(x=leverage.tr, y=cook)) +
        ggplot2::geom_point(ggplot2::aes(shape=mark)) +
        textfun(ggplot2::aes(x=leverage.tr, y=cook, label=ind), size=3) +
        ggplot2::geom_abline(slope=seq.int(0, 3, 0.5), lty=2, col='gray50') +
        ggplot2::xlim(c(0, dat[, max(leverage.tr)])) +
        ggplot2::ylim(c(0, dat[, max(cook) * 1.025])) +
        mytheme +
        ggplot2::labs(title=mains[6L], x=xlabs[6L], y=ylabs[6L])
    }

    top_title <- textGrob(paste0('Outcome: ', outcome, '    Region: ', region),
                          gp=gpar(fontface='bold', cex=1.0))
    p.all <- gridExtra::arrangeGrob(grobs=diagPlots[which], nrow=prows, top=top_title, bottom=model_formula)
    return(p.all)
  }

  # Local function to plot for a single region ('base')
  #-----------------------------------------------------------------------------
  plot_single_base <- function(dat, region) {
    leverage <- fit <- leverage.tr <- NULL
    label.pos <- c(4, 2)  # Default value in 'plot.lm'
    text.id <- function(x, y, label, adj.x=TRUE) {
      labpos <- if (adj.x) label.pos[1 + as.numeric(x > mean(range(x)))] else 3
      text(x, y, label, cex=0.75, xpd=TRUE, pos=labpos, offset=0.25)
    }

    # 1. Resids vs. fitted
    if (show[1L]) {
      ylim <- dat[, extendrange(resid, f=0.08)]
      dat[mark == 0, plot(fit, resid, pch=19, ylim=ylim,
                          main=mains[1L], xlab=xlabs[1L], ylab=ylabs[1L])]
      dat[mark == 1, points(fit, resid, pch=17)]
      dat[, panel.smooth(fit, resid, iter=3, lwd=2, pch=NA)]
      dat[, abline(h=0, lty=3)]
      dat[mark == 1, text.id(fit, resid, ind)]
    }

    # 2. QQ-plot
    if (show[2L]) {
      ylim <- dat[, range(resid.std, na.rm=TRUE)]
      ylim[2L] <- extendrange(ylim, f=0.075)[2L]
      #dat[order(resid.std), x := qnorm(ppoints(resid.std))]
      qq <- dat[, qqnorm(resid.std, main=mains[2L], ylab=ylabs[2L], ylim=ylim)]
      dat[, qqline(resid.std, lty=3, col='gray50')]
      dat[mark == 1, text.id(x, resid.std, ind)]
    }

    # 3. Scale-Location plot
    if (show[3L]) {
      ylim <- dat[, c(0, max(sqrt(abs(resid.std)), na.rm=TRUE))]
      dat[mark == 0, plot(fit, sqrt(abs(resid.std)), pch=19, ylim=ylim,
                          main=mains[3L], xlab=xlabs[3L], ylab=ylabs[3L])]
      dat[mark == 1, points(fit, sqrt(abs(resid.std)), pch=17)]
      dat[, panel.smooth(fit, sqrt(abs(resid.std)), iter=3, lwd=2, pch=NA)]
      dat[mark == 1, text.id(fit, sqrt(abs(resid.std)), ind)]
    }

    # 4. Cook's distance
    if (show[4L]) {
      ylim <- dat[, c(0, max(cook, na.rm=TRUE) * 1.075)]
      dat[, plot(cook, type='h', ylim=ylim, main=mains[4L], xlab=xlabs[4L], ylab=ylabs[4L])]
      xpos <- dat[mark == 1, which=TRUE]
      dat[mark == 1, text.id(xpos, cook, ind, adj.x=FALSE)]
    }

    # 5. Residual vs Leverage plot
    if (show[5L]) {
      xlim <- dat[, c(0, max(leverage, na.rm=TRUE))]
      ylim <- dat[, extendrange(resid.std, f=0.08)]
      r.hat <- dat[, range(leverage, na.rm=TRUE)]
      hh <- seq.int(min(r.hat[1L], r.hat[2L] / 100), 1L, length.out=101L)
      dt.cook <- data.table(hh=rep.int(hh, 2L), level=rep(c(0.5, 1.0), each=101L))
      dt.cook[, cl.h := sqrt(level * p * (1 - hh) / hh), by=level]

      dat[mark == 0, plot(leverage, resid.std, xlim=xlim, ylim=ylim, main=mains[5L],
                          xlab=xlabs[5L], ylab=ylabs[5L], pch=19)]
      dat[mark == 1, points(leverage, resid.std, pch=17)]
      dat[, panel.smooth(leverage, resid.std, iter=3, lwd=2, pch=NA)]
      abline(h=0, v=0, lty=3, col='gray')
      dt.cook[, lines(hh, cl.h, lty=2, col=2), by=level]
      dt.cook[, lines(hh, -cl.h, lty=2, col=2), by=level]
      dat[mark == 1, text.id(leverage, resid.std, ind)]
    }

    # 6. Cook's dist vs leverage
    if (show[6L]) {
      xlim <- dat[, c(0, max(leverage.tr, na.rm=TRUE))]
      ylim <- dat[, c(0, max(cook, na.rm=TRUE) * 1.025)]
      dat[mark == 0, plot(leverage.tr, cook, xlim=xlim, ylim=ylim, main=mains[6L],
                          xlab=xlabs[6L], ylab=ylabs[6L], pch=19)]
      dat[mark == 1, points(leverage.tr, cook, pch=17)]
      dat[, panel.smooth(leverage.tr, cook, iter=3, lwd=2, pch=NA)]
      dat[mark == 1, text.id(leverage.tr, cook, ind)]
      bval <- dat[, pretty(sqrt(p * cook / leverage.tr), 5)]
      bi2 <- bval^2
      usr <- par("usr")
      xmax <- usr[2L]
      ymax <- usr[4L]
      for (i in seq_along(bval)) {
        if (p * ymax > bi2[i] * xmax) {
          xi <- xmax + strwidth(" ") / 3
          yi <- bi2[i] * xi / p
          abline(0, bi2[i], lty=2)
          text(xi, yi, paste(bval[i]), adj=0, xpd=TRUE)
        } else {
          yi <- ymax - 1.5 * strheight(" ")
          xi <- p * yi / bi2[i]
          lines(c(0, xi), c(0, yi), lty=2)
          text(xi, ymax - 0.8 * strheight(" "), paste(bval[i]), adj=0.5, xpd=TRUE)
        }
      }
    }

    title(paste0('Outcome: ', outcome, '    Region: ', region), line=-1, outer=TRUE)
  }

  dimX <- dim(x$X)
  p <- if (length(dimX) == 3L) qr.default(x$X[, , 1L])$rank else qr.default(x$X)$rank
  rnames <- if (isTRUE(ids)) case.names(x) else as.character(seq_len(dimX[1L]))
  outcome <- x$outcome

  # Get all GLM diagnostics
  fits <- fitted(x)
  resids <- residuals(x)
  hat <- hatvalues(x)
  hat.tr <- hat / (1 - hat)
  resid.std <- rstandard(x)
  cook <- cooks.distance(x)
  regions <- region.names(x)
  DT <- setNames(vector('list', length(regions)), regions)
  for (i in regions) {
    DT[[i]] <- data.table(fit=fits[, i], resid=resids[, i], leverage=hat[, i],
                          resid.std=resid.std[, i], cook=cook[, i], leverage.tr=hat.tr[, i])
    DT[[i]][, ind := rnames]
    DT[[i]][, mark := ifelse(abs(resid) < mean(resid) + 2 * sd(resid), 0, 1)]
    DT[[i]][mark == 0, ind := '']
    DT[[i]][, mark := as.factor(mark)]
  }

  if (x$level == 'graph') {
    if (!requireNamespace('ggplot2', quietly=TRUE)) {
      plot_single_base(DT[['graph']], region='graph-level')
      return(invisible(x))
    } else {
      p.all <- plot_single(DT[['graph']], region='graph-level')
      grid.newpage()
      grid.draw(p.all)
    }

  } else if (x$level == 'vertex') {
    if (length(x$runY) == 0) return(NULL)
    run <- if (is.null(region)) run <- intersect(intersect(x$runX, x$runY), regions) else region

    # 'base' plotting
    if (!requireNamespace('ggplot2', quietly=TRUE)) {
      for (z in run) plot_single_base(DT[[z]], z)
      return(invisible(x))

    # 'ggplot2' plotting
    } else {
      p.all <- setNames(vector('list', length(run)), run)
      for (z in run) p.all[[z]] <- plot_single(DT[[z]], z)
      p.all[vapply(p.all, is.null, logical(1L))] <- NULL
    }
  }
  return(p.all)
}

#' Bootstrapping for global graph measures
#'
#' Perform bootstrapping to obtain groupwise standard error estimates of a
#' global graph measure.
#'
#' The confidence intervals are calculated using the \emph{normal approximation}
#' at the \eqn{100 \times conf}\% level (by default, 95\%).
#'
#' For getting estimates of \emph{weighted global efficiency}, a method for
#' transforming edge weights must be provided. The default is to invert them.
#' See \code{\link{xfm.weights}}.
#'
#' @param densities Numeric vector of graph densities to loop through
#' @param resids An object of class \code{brainGraph_resids} (the output from
#'   \code{\link{get.resid}})
#' @param R Integer; the number of bootstrap replicates. Default: \code{1e3}
#' @param measure Character string of the measure to test. Default: \code{mod}
#' @param conf Numeric; the level for calculating confidence intervals. Default:
#'   \code{0.95}
#' @param .progress Logical indicating whether or not to show a progress bar.
#'   Default: \code{getOption('bg.progress')}
#' @inheritParams xfm.weights
#' @export
#'
#' @return \code{brainGraph_boot} -- an object of class \code{brainGraph_boot}
#'   containing some input variables, in addition to a list of
#'   \code{\link[boot]{boot}} objects (one for each group).
#'
#' @name Bootstrapping
#' @family Group analysis functions
#' @family Structural covariance network functions
#' @seealso \code{\link[boot]{boot}}, \code{\link[boot]{boot.ci}}
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}
#' @examples
#' \dontrun{
#' boot.E.global <- brainGraph_boot(densities, resids.all, 1e3, 'E.global')
#' }

brainGraph_boot <- function(densities, resids, R=1e3,
                            measure=c('mod', 'E.global', 'Cp', 'Lp', 'assortativity',
                                      'strength', 'mod.wt', 'E.global.wt'),
                            conf=0.95, .progress=getOption('bg.progress'),
                            xfm.type=c('1/w', '-log(w)', '1-w', '-log10(w/max(w))', '-log10(w/max(w)+1)')) {
  stopifnot(inherits(resids, 'brainGraph_resids'))
  if (!requireNamespace('boot', quietly=TRUE)) stop('Must install the "boot" package.')

  # 'statistic' function for the bootstrapping process
  statfun <- function(x, i, measure, res.obj, xfm.type) {
    corrs <- corr.matrix(res.obj[i], densities=densities, rand=TRUE)
    if (measure %in% c('strength', 'mod.wt', 'E.global.wt')) {
      g.boot <- apply(corrs[[1L]]$r.thresh, 3L, function(y)
                      graph_from_adjacency_matrix(corrs[[1L]]$R * y, mode='undirected',
                                                  diag=FALSE, weighted=TRUE))
    } else {
      g.boot <- apply(corrs[[1L]]$r.thresh, 3L, graph_from_adjacency_matrix,
                      mode='undirected', diag=FALSE)
    }
    res <- switch(measure,
        mod=, mod.wt=vapply(g.boot, function(y) max(cluster_louvain(y)$modularity), numeric(1L)),
        E.global=vapply(g.boot, efficiency, numeric(1L), 'global'),
        E.global.wt=vapply(g.boot, function(g) efficiency(xfm.weights(g, xfm.type), 'global'), numeric(1L)),
        Cp=vapply(g.boot, transitivity, numeric(1L), type='localaverage'),
        Lp=vapply(g.boot, mean_distance, numeric(1L)),
        assortativity=vapply(g.boot, assortativity_degree, numeric(1L)),
        strength=vapply(g.boot, function(y) mean(strength(y)), numeric(1L)))
    return(res)
  }

  # Show a progress bar so you aren't left in the dark
  intfun <- statfun
  if (isTRUE(.progress)) {
    intfun <- function(data, indices, measure, res.obj, xfm.type) {
      curVal <- get('counter', envir=env) + ncpus
      assign('counter', curVal, envir=env)
      setTxtProgressBar(get('progbar', envir=env), curVal)
      flush.console()
      statfun(data, indices, measure, res.obj, xfm.type)
    }
  }

  ncpus <- getOption('bg.ncpus')
  if (.Platform$OS.type == 'windows') {
    my.parallel <- 'snow'
    cl <- makeCluster(ncpus, type='SOCK')
    clusterEvalQ(cl, library(brainGraph))
  } else {
    my.parallel <- 'multicore'
    cl <- NULL
  }

  measure <- match.arg(measure)
  xfm.type <- match.arg(xfm.type)
  env <- environment()
  grps <- unique(groups(resids))
  my.boot <- setNames(vector('list', length(grps)), grps)
  for (g in grps) {
    counter <- 0
    res.dt <- resids$resids.all[g]
    if (isTRUE(.progress)) progbar <- txtProgressBar(min=0, max=R, style=3)
    my.boot[[g]] <- boot::boot(res.dt, intfun, measure=measure, res.obj=resids[g], xfm.type=xfm.type, R=R,
                         parallel=my.parallel, ncpus=ncpus, cl=cl)
    if (isTRUE(.progress)) close(progbar)
  }

  out <- list(measure=measure, densities=densities, Group=grps, conf=conf, boot=my.boot)
  class(out) <- c('brainGraph_boot', class(out))
  return(out)
}

#' Print a summary from a bootstrap analysis
#'
#' @param object,x A \code{brainGraph_boot} object
#' @export
#' @rdname Bootstrapping

summary.brainGraph_boot <- function(object, ...) {
  if (!requireNamespace('boot', quietly=TRUE)) stop('Must install the "boot" package.')
  kNumDensities <- length(object$densities)
  # Get everything into a data.table
  boot.dt <- with(object,
                  data.table(Group=rep(Group, each=kNumDensities),
                             density=rep.int(densities, length(Group)),
                             Observed=c(vapply(boot, with, numeric(kNumDensities), t0)),
                             se=c(vapply(boot, function(x) apply(x$t, 2L, sd), numeric(kNumDensities)))))
  setnames(boot.dt, 'Group', getOption('bg.group'))
  ci <- with(object,
             vapply(seq_along(densities), function(x)
                    vapply(boot, function(y)
                           boot::boot.ci(y, type='norm', index=x, conf=conf)$normal[2L:3L],
                           numeric(2L)),
                    numeric(2L * length(Group))))
  boot.dt$ci.low <- c(t(ci[2L * (seq_along(object$Group) - 1L) + 1L, ]))
  boot.dt$ci.high <- c(t(ci[2L * (seq_along(object$Group) - 1L) + 2L, ]))

  meas.full <- switch(object$measure,
                      mod='Modularity', mod.wt='Modularity (weighted)',
                      E.global='Global efficiency', E.global.wt='Global efficiency (weighted)',
                      Cp='Clustering coefficient',
                      Lp='Average shortest path length',
                      assortativity='Degree assortativity',
                      strength='Average strength')
  boot.sum <- list(meas.full=meas.full, DT.sum=boot.dt, conf=object$conf, R=object$boot[[1L]]$R)
  class(boot.sum) <- c('summary.brainGraph_boot', class(boot.sum))
  boot.sum
}

#' @aliases summary.brainGraph_boot
#' @method print summary.brainGraph_boot
#' @export

print.summary.brainGraph_boot <- function(x, ...) {
  print_title_summary('Bootstrap analysis')
  cat('Graph metric: ', x$meas.full, '\n')
  cat('Number of bootstrap samples generated: ', x$R, '\n')
  conf.pct <- 100 * x$conf
  cat(conf.pct, '% confidence intervals\n\n')
  print(x$DT.sum)
  invisible(x)
}

#' Plot bootstrap output of global graph measures across densities
#'
#' The \code{plot} method returns two \code{ggplot} objects: one with shaded
#' regions based on the standard error, and the other based on confidence
#' intervals (calculated using the normal approximation).
#'
#' @param ... Unused
#' @param alpha A numeric indicating the opacity for the confidence bands
#' @export
#' @rdname Bootstrapping
#'
#' @return \code{plot} -- \emph{list} with the following elements:
#'   \item{se}{A ggplot object with ribbon representing standard error}
#'   \item{ci}{A ggplot object with ribbon representing confidence intervals}

plot.brainGraph_boot <- function(x, ..., alpha=0.4) {
  Observed <- se <- ci.low <- ci.high <- NULL
  gID <- getOption('bg.group')

  boot.sum <- summary(x)
  boot.dt <- boot.sum$DT.sum

  # 'base' plotting
  if (!requireNamespace('ggplot2', quietly=TRUE)) {
    grps <- boot.dt[, unique(get(gID))]

    par(mfrow=c(2L, 1L))
    # 1. SE plot
    ymin <- boot.dt[, min(Observed - se)]
    ymax <- boot.dt[, max(Observed + se)]
    boot.dt[get(gID) == grps[1L],
            plot(density, Observed, type='l', col=plot.cols[1L],
                 ylim=c(ymin, ymax), ylab=boot.sum$meas.full)]
    boot.dt[get(gID) == grps[2L],
            lines(density, Observed, col=plot.cols[2L])]
    for (i in 1L:2L) {
      boot.dt[get(gID) == grps[i],
              polygon(x=c(density, rev(density)),
                      y=c(Observed + se, rev(Observed - se)),
                      col=adjustcolor(plot.cols[i], alpha.f=alpha),
                      border=plot.cols[i])]
    }
    legend('topright', title=gID, grps, fill=plot.cols[1L:2L])

    # 2. CI plot
    ymin <- boot.dt[, min(ci.low)]
    ymax <- boot.dt[, max(ci.high)]
    boot.dt[get(gID) == grps[1L],
            plot(density, Observed, type='l', col=plot.cols[1L],
                 ylim=c(ymin, ymax), ylab=boot.sum$meas.full)]
    boot.dt[get(gID) == grps[2L],
            lines(density, Observed, col=plot.cols[2L])]
    for (i in 1L:2L) {
      boot.dt[get(gID) == grps[i],
              polygon(x=c(density, rev(density)),
                      y=c(ci.high, rev(ci.low)),
                      col=adjustcolor(plot.cols[i], alpha.f=alpha),
                      border=plot.cols[i])]
    }
    legend('topright', title=gID, grps, fill=plot.cols[1L:2L])
    return(invisible(x))

  # 'ggplot2' plotting
  } else {
    b <- ggplot2::ggplot(boot.dt, ggplot2::aes(x=density, y=Observed, col=get(gID))) +
      ggplot2::geom_line() +
      ggplot2::labs(y=boot.sum$meas.full)
    se <- b + ggplot2::geom_ribbon(ggplot2::aes(ymin=Observed-se, ymax=Observed+se, fill=get(gID)), alpha=alpha)
    ci <- b + ggplot2::geom_ribbon(ggplot2::aes(ymin=ci.low, ymax=ci.high, fill=get(gID)), alpha=alpha)
    return(list(se=se, ci=ci))
  }
}

#' Calculate weighted shortest path lengths
#'
#' Calculate graph or vertex average shortest path lengths. For vertices, this
#' is just the row means of the distance matrix. For the graph-level, it is the
#' overall mean of the distance matrix.
#'
#' By default, edge weights are not transformed (e.g., inverted). However, if
#' set to \code{TRUE}, then the input graph must have a graph-level attribute
#' called \code{'xfm.type'} or you must supply a value in the function call. If
#' you supply a distance matrix (the \code{D} argument), it is not necessary to
#' transform edge weights, as it is assumed the the distance matrix was
#' calculated from a graph with transformed edge weights already.
#'
#' @param level Character string indicating whether to calculate vertex- or
#'   graph-level shortest path length. Default: \code{'graph'}
#' @inheritParams efficiency
#' @inheritParams xfm.weights
#' @export
#' @return Numeric vector (if \code{level='vertex'}) of each vertex's shortest
#'   path length, or a single number for the graph-level average

mean_distance_wt <- function(g, level=c('graph', 'vertex'), weights=NULL,
                             xfm=FALSE, xfm.type=NULL, D=NULL) {
  if (isTRUE(xfm)) {
    if (is.null(xfm.type)) {
      xfm.type <- if ('xfm.type' %in% graph_attr_names(g)) g$xfm.type else '1/w'
    }
    g <- xfm.weights(g, xfm.type)
  }
  weights <- check_weights(g, weights)
  level <- match.arg(level)
  if (is.null(D)) D <- distances(g, weights=weights)
  D[is.infinite(D)] <- diag(D) <- NA
  Lp <- switch(level,
               vertex=rowMeans(D, na.rm=TRUE),
               graph=mean(D, na.rm=TRUE))
  Lp[is.nan(Lp)] <- 0
  return(Lp)
}

#' Calculate graph global, local, or nodal efficiency
#'
#' This function calculates the global efficiency of a graph or the local or
#' nodal efficiency of each vertex of a graph.
#'
#' Local efficiency for vertex \emph{i} is:
#' \deqn{E_{local}(i) = \frac{1}{N} \sum_{i \in G} E_{global}(G_i)}
#' where \eqn{G_i} is the subgraph of neighbors of \emph{i}, and \emph{N} is the
#' number of vertices in that subgraph.
#'
#' Nodal efficiency for vertex \emph{i} is:
#' \deqn{E_{nodal}(i) = \frac{1}{N-1} \sum_{j \in G} \frac{1}{d_{ij}}}
#'
#' Global efficiency for graph \emph{G} with \emph{N} vertices is:
#' \deqn{E_{global}(G) = \frac{1}{N(N-1)} \sum_{i \ne j \in G} \frac{1}{d_{ij}}}
#' where \eqn{d_{ij}} is the shortest path length between vertices \emph{i} and
#' \emph{j}. Alternatively, global efficiency is equal to the mean of all nodal
#' efficiencies.
#'
#' @param g An \code{igraph} graph object
#' @param type Character string; either \code{local}, \code{nodal}, or
#'   \code{global}. Default: \code{local}
#' @param weights Numeric vector of edge weights; if \code{NULL} (the default),
#'   and if the graph has edge attribute \code{weight}, then that will be used.
#'   To avoid using weights, this should be \code{NA}.
#' @param xfm Logical indicating whether to transform the edge weights. Default:
#' \code{FALSE}
#' @param use.parallel Logical indicating whether or not to use \code{foreach}.
#'   Default: \code{TRUE}
#' @param A Numeric matrix; the adjacency matrix of the input graph. Default:
#'   \code{NULL}
#' @param D Numeric matrix; the graph's \dQuote{distance matrix}
#' @inheritParams xfm.weights
#' @export
#' @importFrom Matrix rowSums
#' @importFrom foreach getDoParRegistered
#' @importFrom doParallel registerDoParallel
#'
#' @return A numeric vector of the efficiencies for each vertex of the graph
#'   (if \emph{type} is \code{local|nodal}) or a single number (if \emph{type}
#'   is \code{global}).
#'
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}
#' @references Latora, V. and Marchiori, M. (2001) Efficient behavior of
#'   small-world networks. \emph{Phys Rev Lett}, \bold{87.19}, 198701.
#'   \doi{10.1103/PhysRevLett.87.198701}
#' @references Latora, V. and Marchiori, M. (2003) Economic small-world
#'   behavior in weighted networks. \emph{Eur Phys J B}, \bold{32}, 249--263.
#'   \doi{10.1140/epjb/e2003-00095-5}

efficiency <- function(g, type=c('local', 'nodal', 'global'), weights=NULL,
                       xfm=FALSE, xfm.type=NULL, use.parallel=TRUE, A=NULL, D=NULL) {
  stopifnot(is_igraph(g))
  i <- NULL
  if (isTRUE(xfm)) {
    if (is.null(xfm.type)) {
      xfm.type <- if ('xfm.type' %in% graph_attr_names(g)) g$xfm.type else '1/w'
    }
    g <- xfm.weights(g, xfm.type)
  }
  weights <- check_weights(g, weights)

  type <- match.arg(type)
  if (type == 'local') {
    e.attr <- weighted <- NULL
    if (is.null(weights)) {
      e.attr <- 'weight'
      weighted <- TRUE
    }
    if (is.null(A)) A <- as_adj(g, names=FALSE, sparse=FALSE, attr=e.attr)
    eff <- rep.int(0, dim(A)[1L])
    verts <- which(rowSums((A > 0) + 0) > 1)
    X <- apply(A, 1L, function(x) which(x > 0))
    if (is.matrix(X)) X <- as.list(data.frame(X))   # If the graph is complete

    if (length(verts) > 0L) {
      `%d%` <- `%do%`
      if (isTRUE(use.parallel)) {
        if (!getDoParRegistered()) {
          cl <- makeCluster(getOption('bg.ncpus'))
          registerDoParallel(cl)
        }
        `%d%` <- `%dopar%`
      }
      eff[verts] <- foreach(i=verts, .combine='c') %d% {
        g.sub <- graph_from_adjacency_matrix(A[X[[i]], X[[i]]], mode='undirected', weighted=weighted)
        efficiency(g.sub, 'global', weights=weights)
      }
    }
  } else {
    Nv <- vcount(g)
    if (is.null(D)) {
      if (Nv > 650) {
        D <- foreach(i=seq_len(Nv), .combine=rbind) %dopar% {
          distances(g, v=i, weights=weights)
        }
      } else {
        D <- distances(g, weights=weights)
      }
    }
    Dinv <- 1 / D
    eff <- colSums(Dinv * is.finite(Dinv), na.rm=TRUE) / (Nv - 1L)
    if (type == 'global') eff <- sum(eff) / length(eff)
  }
  return(eff)
}

#' Check if all elements in a list are identical
#'
#' @param l A list
#' @noRd

all_identical <- function(l) all(mapply(identical, head(l, 1L), tail(l, -1L)))

#' Convert arguments into a single list
#'
#' \code{args_as_list} converts its arguments into a single list. If the
#' argument is already a single list, it returns the same object. If multiple
#' objects are given, it wraps them in a list.
#'
#' @param ... Graphs or a list of graphs
#' @noRd

args_as_list <- function(...) {
  ll <- lapply(list(...), function(l) if (is_igraph(l)) list(l) else l)
  unlist(ll, recursive=FALSE)
}

#' Check for vertex or edge attributes
#'
#' \code{check_weights} is a helper function for dealing with edge weights that
#' get passed to different \code{igraph} functions.
#'
#' @return \code{check_weights} - If \code{weights=NULL} and the graph has a
#'   \code{'weight'} attribute, then \code{NULL} will be returned. If
#'   \code{weights=NA}, then \code{NA} is returned.
#' @keywords internal
#' @rdname check_attributes

check_weights <- function(g, weights) {
  if (is.null(weights) && 'weight' %in% edge_attr_names(g)) {
    weights <- NULL
  } else {
    if (!is.null(weights) && any(!is.na(weights))) {
      weights <- as.numeric(weights)
    } else {
      weights <- NA
    }
  }
  weights
}

#' Check for presence of a degree attribute
#'
#' \code{check_degree} is a helper function to check if \code{degree} is a
#' vertex attribute of the input graph. Returns a numeric vector of the degree
#' values.
#' @keywords internal
#' @rdname check_attributes

check_degree <- function(g) {
  if ('degree' %in% vertex_attr_names(g)) V(g)$degree else degree(g)
}

#' Check for presence of a strength attribute
#'
#' \code{check_strength} is a helper function to check if \code{strength} is a
#' vertex attribute of the input graph. Returns a numeric vector of the strength
#' values.
#' @keywords internal
#' @rdname check_attributes

check_strength <- function(g) {
  if ('strength' %in% vertex_attr_names(g)) V(g)$strength else strength(g)
}

#' Test if an object is a character vector of numbers
#'
#' \code{check_sID} is a convenience function to test if a vector (typically the
#' \emph{subject ID} column in a \code{data.table}) is a character vector of
#' numbers, a factor vector of numbers, or a numeric vector. If so, it will
#' zero-pad the variable to have equal width.
#'
#' This function is meant to avoid issues that arise when sorting a vector of
#' numbers that have been converted to \code{character}. For example,
#' \code{\link{import_scn}} automatically reads in the first column (with
#' \emph{FreeSurfer} outputs this is the column of subject IDs) as a
#' \code{character} variable. If the subject IDs had been all numbers/integers,
#' then sorting (i.e., setting the \code{key} in a \code{data.table}) would be
#' incorrect: e.g., it might be \code{'1', '10', '2', ...}.
#'
#' @return \code{check_sID} returns either the input vector or a character
#'   vector padded with \code{0}
#' @export
#' @rdname pad_zeros

check_sID <- function(x) {
  cls <- class(x)
  if (cls == 'factor') {
    test <- suppressWarnings(as.numeric(as.character(x)) == x)
  } else {
    test <- suppressWarnings(as.character(as.numeric(x)) == x)
  }
  if (isTRUE(all(test))) x <- pad_zeros(x)
  return(x)
}

#' Calculate the p-value for differences in correlation coefficients
#'
#' Given two sets of correlation coefficients and sample sizes, this function
#' calculates and returns the \emph{z-scores} and \emph{p-values} associated
#' with the difference between correlation coefficients.
#'
#' @param r1,r2 Numeric (vector or matrix) of correlation coefficients for both
#'   groups
#' @param n Integer vector; number of observations for both groups
#' @inheritParams GLM
#' @export
#'
#' @return A list with elements \code{p} and \code{z}, the p-values
#'   and z-scores for the difference in correlations.
#'
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}
#' @examples
#' \dontrun{
#' kNumSubjs <- summary(covars$Group)
#' corr.diffs <- cor.diff.test(corrs$R[, , 1], corrs$R[, , 2], kNumSubjs)
#' edge.diffs <- t(sapply(which(corr.diffs$p < .05), function(x)
#'                        mapply('[[',
#'                               dimnames(corr.diffs$p),
#'                               arrayInd(x, dim(corr.diffs$p)))
#'                               ))
#' }
cor.diff.test <- function(r1, r2, n, alternative=c('two.sided', 'less', 'greater')) {
  stopifnot(length(n) == 2L)

  z1 <- 0.5 * log((1 + r1) / (1 - r1))
  z2 <- 0.5 * log((1 + r2) / (1 - r2))

  n <- n - 3L
  SEdiff <- sqrt((1 / n[1L]) + (1 / n[2L]))
  diff.z <- (z1 - z2) / SEdiff

  alt <- match.arg(alternative)
  p <- switch(alt,
              less=pnorm(diff.z),
              greater=pnorm(diff.z, lower.tail=FALSE),
              two.sided=2 * pnorm(abs(diff.z), lower.tail=FALSE))

  return(list(p=p, z=diff.z))
}

#' Utility functions
#'
#' \code{get_metadata} adds metadata to a list-like object.
#'
#' If the object is a graph, graph-level attributes will be added. The
#' elements added are:
#' \describe{
#'   \item{version}{A list with R, brainGraph, and igraph versions}
#'   \item{sys}{Character vector of system information}
#'   \item{date}{The date and time of creation}
#' }
#'
#' @param object A list-like object
#' @keywords internal
#' @return \code{get_metadata} - the same object with version, system, and date
#'   information added
#' @name Utility functions
#' @rdname utils

get_metadata <- function(object) {
  object$version <- list(r=R.version.string,
                         bg=packageVersion('brainGraph'),
                         ig=packageVersion('igraph'))
  object$sys <- Sys.info()[c(1L:3L, 5L)]
  object$date <- format(Sys.time(), '%Y-%m-%dT%H:%M:%OS0')
  return(object)
}

#' Create a data.table with a single graph metric
#'
#' \code{glm_data_table} is used in \code{brainGraph_GLM} and
#' \code{brainGraph_mediate} to create a \code{data.table} with the
#' \emph{subject IDs} and column(s) for the graph- or vertex-level metric of
#' interest.
#'
#' @inheritParams GLM
#' @return \code{glm_data_table} - A \code{data.table} with one column
#'   containing the subject ID's and 1 or more columns with the graph- or
#'   vertex-level measure of interest.
#' @keywords internal
#' @rdname glm_helpers

glm_data_table <- function(g.list, level, measure) {
  sID <- getOption('bg.subject_id')
  if (level == 'vertex') {
    y <- t(vapply(g.list, vertex_attr, numeric(vcount(g.list[[1L]])), measure))
    colnames(y) <- V(g.list[[1L]])$name
    DT.y <- as.data.table(y, keep.rownames=sID)
  } else if (level == 'graph') {
    DT.y <- as.data.table(vapply(g.list, graph_attr, numeric(1L), measure), keep.rownames=TRUE)
    setnames(DT.y, names(DT.y), c(sID, 'graph'))
  }
  return(DT.y)
}

#' Convert a matrix to a list of rows
#'
#' \code{matrix2list} makes working with different contrast types (i.e., t or F)
#' a little simpler.
#'
#' @param mat Numeric matrix in which each row is a single contrast vector
#' @return \code{matrix2list} -- A list with length equal to the number of rows
#'   of \code{C}
#' @keywords internal
#' @rdname glm_helpers

matrix2list <- function(mat) {
  l <- lapply(seq_len(dim(mat)[1L]), function(i) mat[i, , drop=FALSE])
  nam <- dimnames(mat)[[1L]]
  if (!is.null(nam)) names(l) <- nam
  return(l)
}

#' Helper function to calculate a max or min
#'
#' @keywords internal
#' @rdname glm_helpers
maxfun <- function(alternative) {
  switch(alternative,
         two.sided=function(x) max(abs(x), na.rm=TRUE),
         less=function(x) min(x, na.rm=TRUE),
         greater=function(x) max(x, na.rm=TRUE))
}

#' Helper function to sort values
#'
#' @keywords internal
#' @rdname glm_helpers
sortfun <- function(alternative) {
  switch(alternative,
         two.sided=function(x, ind) sort(abs(x), partial=ind)[ind],
         less=function(x, ind) sort(x, decreasing=TRUE)[ind],
         greater=function(x, ind) sort(x, partial=ind)[ind])
}

#' Faster version of outer for 2 vectors
#'
#' \code{outer_vec} simply performs the cross-product, specifically \code{x %*%
#' t(y)}, and assigns dimnames to the resulting matrix.
#' @keywords internal
#' @rdname utils

outer_vec <- function(x, y) {
  robj <- tcrossprod(x, y)
  dimnames(robj) <- list(names(x), names(y))
  robj
}

#' Pad the front of a numeric or character vector with zeros
#'
#' \code{pad_zeros} pads a vector with zeros to avoid issues with ordering a
#' column of integers or integers converted to \code{character}.
#'
#' If \dQuote{x} is a numeric vector, then the resultant string width will be
#' determined by \code{max(x)} or \code{x} itself if the input is a single
#' integer. For example, if \code{x=10}, it will return \code{'01', '02', ...,
#' '10'}. If \dQuote{x} is a character vector, then the output's string width
#' will be \code{max(nchar(x))}. For example, if \code{x} includes both
#' \code{'1'} and \code{'1000'}, it will return \code{'0001'}, etc.
#'
#' @param x \code{pad_zeros} accepts either a vector (numeric or character) or a
#'   single integer. \code{check_sID} accepts a character, numeric, or factor
#'   vector
#' @return A character vector with zero-padded values
#' @export
#' @examples
#' pad_zeros(10)  # '01' '02' ... '10'
#' x <- c(1, 10, 100)
#' pad_zeros(x)   # '001' '010' '100'
#' x <- as.character(x)
#' pad_zeros(x)   # '001' '010' '100'

pad_zeros <- function(x) {
  if (is.numeric(x)) {
    if (length(x) == 1L) x <- seq_len(x)
    n <- max(x)
    x <- formatC(x, width=floor(log10(n) + 1L), flag='0')
  } else if (is.character(x)) {
    nc <- nchar(x)
    if (length(unique(nc)) == 1L) return(x)
    spec <- paste0('%0', max(nc), 's')
    x <- gsub(' ', '0', sprintf(spec, x))
  }
  return(x)
}

#' Apply a rotation matrix to a set of points
#'
#' This function takes a set of points and applies a rotation matrix (e.g. will
#' rotate points 90 deg. if given \dQuote{pi/2} as input)
#'
#' @param x A matrix with 2 columns of the points to rotate
#' @param theta The angle to apply
#' @return A matrix with 2 columns of the points' new locations
#' @noRd

rotation <- function(x, theta) {
  R <- matrix(c(cos(theta), sin(theta), -sin(theta), cos(theta)), nrow=2L)
  x %*% R
}

#' Capitalize the first letter of a character string
#'
#' @noRd
simpleCap <- function(x) paste0(toupper(substring(x, 1L, 1L)), substring(x, 2L))

#' Add newlines to a character string for printing
#'
#' \code{split_string} inserts separator characters in a character string to
#' truncate strings for printing.
#'
#' The \code{delim} argument determines \emph{where} to insert the separator.
#' For example, given the default, it will attempt to insert newline characters
#' only following a plus sign.
#'
#' @param x A character string
#' @param max_len Integer; the max length of one line. Default: 80
#' @param delim Character specifying where to end a line if it is longer than
#'   \code{max_len}. Default: \code{'+'}
#' @param sep Character specifying what to split by. Default: \code{'\n'} (a
#'   newline character)
#' @keywords internal
#' @rdname utils

split_string <- function(x, max_len=80L, delim='\\+', sep='\n') {
  str_len <- nchar(x)
  if (str_len > max_len) {
    nlines <- (str_len %/% max_len) + (str_len %% max_len > 0L)
    delims <- gregexpr(delim, x)[[1L]]
    endpts <- rep.int(str_len, nlines)
    for (i in seq_len(nlines - 1L)) endpts[i] <- max(delims[delims < max_len*i])
    startpts <- c(1L, endpts[-nlines] + 1L)
    lines <- paste(Map(function(a, b) substr(x, a, b), startpts, endpts), '\n')
    x <- trimws(paste(lines, collapse=''), 'right')
  }
  return(x)
}

#' Subset graphs based on a given logical condition
#'
#' \code{subset_graph} will subset a given graph based on the given logical
#' condition(s). This can be a \dQuote{simple} logical equation, or can include
#' combinations of \emph{AND} and \emph{OR} (i.e., \code{&} and \code{|}).
#'
#' @param g A graph object
#' @param condition Character string specifying an equation for which vertices to
#'   keep
#' @keywords internal
#' @return A graph object

subset_graph <- function(g, condition) {
  stopifnot(nzchar(condition))

  # Function for creating the condition string to later subset the graph
  get_cond_string <- function(orig) {
    spec <- '\\s\\&\\s|\\s\\|\\s'  # Splits are either " & " or " | "
    conditions <- strsplit(orig, split=spec)[[1L]]
    if (length(conditions) > 1L) {  # Multiple conditions
      if (isFALSE(grepl(spec, orig))) {
        stop('Logical operators must be surrounded by spaces!')
      }
      nchars <- cumsum(nchar(conditions))
      endpts <- nchars + seq.int(from=2L, by=3L, length.out=length(nchars))
      splits <- vapply(endpts, function(x) substr(orig, start=x, stop=x), character(1L))
      conditions <- trimws(conditions) # Remove unnecessary whitespace

      cond.string <- paste(vapply(seq_along(conditions), function(x)
                                  paste0('V(g)$', conditions[x], splits[x]), character(1L)),
                           collapse='')
    } else {
      cond.string <- paste0('V(g)$', conditions)
    }
    return(cond.string)
  }

  # Handle when logical expressions are separated by parentheses
  if (isTRUE(grepl('\\(.*\\&.*\\)', condition)) || isTRUE(grepl('\\(.*\\|.*\\)', condition))) {
    subs <- strsplit(condition, split='\\) & \\(')[[1L]]
    subs <- as.list(trimws(subs, whitespace='[\\(\\) ]'))
    cond.strings <- vapply(subs, get_cond_string, character(1L))
    cond.string <- paste0('(', paste(cond.strings, collapse=') & ('), ')')
  } else {
    cond.string <- get_cond_string(condition)
  }

  cond <- eval(parse(text=cond.string))
  if (sum(cond, na.rm=TRUE) == 0L) {
    warning('No vertices meet criteria! No graph created')
    g <- inds <- NULL
  } else {
    inds <- which(cond)
    cond <- setdiff(seq_len(vcount(g)), inds)
    orig.class <- class(g)
    g <- delete.vertices(g, cond)
    class(g) <- orig.class
  }
  list(g=g, inds=inds)
}

#' Transform a vector to have a different range
#'
#' \code{vec.transform} takes a vector and transforms it to have a new range,
#' given the input, or the default values of [0, 1].
#'
#' @param min.val the minimum value of the new range
#' @param max.val the maximum value of the new range
#'
#' @keywords internal
#' @return A vector of the transformed input.
#' @rdname utils

vec.transform <- function(x, min.val=0, max.val=1) {
  diffrange <- diff(range(x, na.rm=TRUE))
  if (diffrange == 0) {
    out <- rep_len(max.val, length(x))
  } else {
    out <- ((x - min(x, na.rm=TRUE)) * (max.val - min.val) / diffrange) + min.val
  }
  return(out)
}

#' Subset observations of a bg_GLM object
#'
#' The \code{[} method allows you to select observations (i.e., rows of \code{X}
#' and \code{y}) and independent variables (i.e., columns of \code{X}) from a
#' \code{bg_GLM} object.
#'
#' @note The \code{[} method is used when calculating \emph{studentized
#' residuals} and other \dQuote{leave-one-out} diagnostics, and typically should
#' not be called directly by the user.
#'
#' @param i Integer/character vector; the observation number(s) or row names to
#'   select or remove
#' @param j Integer/character vector; the design matrix column number(s) or
#'   names to select or remove
#' @export
#' @return A \code{bg_GLM} object with the specified row(s) selected or removed
#'   from both \code{X} and \code{y}, and column(s) selected/removed from
#'   \code{X}
#' @include brainGraph_GLM.R
#' @rdname glm

`[.bg_GLM` <- function(x, i, j) {
  dimX <- dim(x$X)
  if (missing(i)) i <- seq_len(dimX[1L])
  if (missing(j)) j <- seq_len(dimX[2L])
  x$X <- if (length(dimX) == 3L) x$X[i, j, , drop=FALSE] else x$X[i, j, drop=FALSE]
  x$y <- x$y[i, , drop=FALSE]
  return(x)
}

#' Extract basic information from a bg_GLM object
#'
#' These functions return the \code{terms}, \emph{term labels}, \emph{model
#' formula}, \dQuote{case names}, \dQuote{variable names}, \emph{region names},
#' and number of observations for a \code{bg_GLM} object. The term labels are
#' used for ANOVA tables.
#'
#' @note \code{formula} returns a character string, not a \code{formula}
#' object.
#'
#' @param x,object A \code{bg_GLM} object
#' @param ... Unused
#' @export
#' @return \code{terms} returns a named integer list in which the names are the
#'   term labels and the list elements are the column(s) of the design matrix
#'   for each term. \code{nobs} returns an integer. The other functions return
#'   character vectors.
#' @name GLM basic info
#' @rdname glm_info

nobs.bg_GLM <- function(object, ...) dim(object$X)[1L]

#' @export
#' @rdname glm_info

terms.bg_GLM <- function(x, ...) {
  cvnames <- setdiff(names(x$DT.Xy), c(getOption('bg.subject_id'), 'region', x$outcome))
  vnames <- variable.names(x)
  cols <- cols_orig <- sapply(cvnames, function(y) list(grep(y, vnames)))

  # Determine if there are any interaction terms
  rmatches <- gregexpr(':', vnames)
  numMatches <- vapply(rmatches, function(y) length(y[y > 0L]), integer(1L))
  if (any(numMatches > 0L)) {
    combos <- combn(seq_along(cvnames), 2L, simplify=FALSE)
    ints <- lapply(combos, function(y) intersect(cols[[y[1L]]], cols[[y[2L]]]))
    matches <- lapply(combos[which(lengths(ints) > 0L)], as.matrix)
    int_terms <- vapply(matches, function(y) cvnames[y], character(2L))
    twoWays <- apply(int_terms, 2L, paste, collapse=':')
    nonzero <- Filter(length, ints)
    for (i in seq_along(twoWays)) {
      cols[[twoWays[i]]] <- nonzero[[i]]
      cols[int_terms[, i]] <- lapply(cols[int_terms[, i]], setdiff, nonzero[[i]])
    }
    cols <- Filter(length, cols)
    if (any(numMatches == 2L)) {
      combos <- combn(seq_along(cvnames), 3L)
      ints <- apply(combos, 2L, function(y) intersect(cols_orig[[y[1L]]], intersect(cols_orig[[y[2L]]], cols_orig[[y[3L]]])))
      matches <- as.matrix(combos[, which(lengths(ints) > 0L)])
      int_terms <- cvnames[matches]
      dim(int_terms) <- dim(matches)
      int_name <- apply(int_terms, 2L, paste, collapse=':')
      nonzero <- Filter(length, ints)
      for (i in seq_along(int_name)) {
        cols[[int_name[i]]] <- nonzero[[i]]
        cols[int_terms[, i]] <- lapply(cols[int_terms[, i]], setdiff, nonzero[[i]])
        cols[twoWays] <- lapply(cols[twoWays], setdiff, nonzero[[i]])
      }
    }
  }
  if (any(grepl('Intercept', vnames))) {
    cols <- c(list(Intercept=grep('Intercept', vnames)), cols)
  }
  cols <- cols[order(unlist(lapply(cols, `[[`, 1L)))]
  return(cols)
}

#' @export
#' @rdname glm_info

formula.bg_GLM <- function(x, ...) {
  tlabels <- labels(x)
  if (any(grepl('Intercept', tlabels))) tlabels <- tlabels[-grep('Intercept', tlabels)]
  rmatches <- gregexpr(':', tlabels)
  intOrder <- vapply(rmatches, function(y) length(y[y > 0L]), integer(1L)) + 1L
  splits <- strsplit(tlabels, ':')
  if (any(intOrder == 3L)) {
    for (i in which(intOrder == 3L)) {
      inds.remove <- vapply(splits[[i]], function(y) which(y == tlabels), integer(1L))
      inds.remove <- c(inds.remove, i)
      twoWays <- combn(splits[[i]], 2L, function(y) paste(y, collapse=':'))
      inds.remove <- c(inds.remove, vapply(twoWays, function(y) which(y == tlabels), integer(1L)))
      tlabels <- tlabels[-inds.remove]
      tlabels <- c(tlabels, paste(splits[[i]], collapse=' * '))
    }
  } else {
    for (i in which(intOrder > 1L)) {
      if (all(splits[[i]] %in% tlabels)) {
        inds.remove <- vapply(splits[[i]], function(y) which(y == tlabels), integer(1L))
        inds.remove <- c(inds.remove, i)
        tlabels <- tlabels[-inds.remove]
        tlabels <- c(tlabels, paste(splits[[i]], collapse=' * '))
      }
    }
  }
  form <- paste(x$outcome, '~', paste(tlabels, collapse=' + '))
  return(form)
}

#' @export
#' @rdname glm_info
labels.bg_GLM <- function(object, ...) labels(terms(object))

#' @export
#' @method case.names bg_GLM
#' @rdname glm_info
case.names.bg_GLM <- function(object, ...) dimnames(object$X)[[1L]]

#' @export
#' @method variable.names bg_GLM
#' @rdname glm_info
variable.names.bg_GLM <- function(object, ...) dimnames(object$X)[[2L]]

#' @export
#' @method region.names bg_GLM
#' @rdname glm_info

region.names.bg_GLM <- function(object) {
  if (object$level == 'graph') {
    rgn <- 'graph'
  } else {
    rgn <- if (length(dim(object$X)) == 3L) dimnames(object$X)[[3L]] else dimnames(object$y)[[2L]]
  }
  rgn
}

#' @export
#' @rdname glm_info

nregions.bg_GLM <- function(object) {
  if (object$level == 'graph') {
    n <- 1L
  } else {
    n <- if (length(dim(object$X)) == 3L) dim(object$X)[3L] else dim(object$y)[2L]
  }
  n
}

#' brainGraph generic methods
#'
#' These functions are S3 \emph{generics} for various \code{brainGraph}-defined
#' objects.
#'
#' @name brainGraph-methods
NULL

#' Return group names
#'
#' \code{groups} returns the \dQuote{Group} graph attribute for each graph or
#' observation in the object.
#'
#' @param x,object An object
#' @export
#' @method groups brainGraphList
#' @rdname brainGraph-methods

groups.brainGraphList <- function(x) {
  out <- NULL
  if (x$type == 'observed') {
    if ('Group' %in% graph_attr_names(x[1L])) {
      out <- vapply(x[], graph_attr, character(1L), 'Group')
    }
  } else if (x$type == 'random') {
    subs <- x$graphs
    if ('Group' %in% graph_attr_names(subs[[1L]][[1L]])) {
      out <- vapply(subs, function(g) graph_attr(g[[1L]], 'Group'), character(1L))
    }
  }
  return(out)
}

#' @export
#' @method groups brainGraph_resids
#' @rdname residuals
groups.brainGraph_resids <- function(x) {
  sID <- getOption('bg.subject_id')
  gID <- getOption('bg.group')
  structure(x$resids.all[, as.character(get(gID))], names=x$resids.all[, get(sID)])
}

#' @export
#' @method groups corr_mats
#' @rdname brainGraph-methods
groups.corr_mats <- function(x) names(x$r.thresh)

#' Extract region names from brainGraph objects
#'
#' \code{region.names} is a generic method for extracting region names from
#' various \code{brainGraph} objects. These are generally convenience functions.
#'
#' For a \code{data.table}, \code{region.names} assumes that it contains a
#' \emph{factor} column named \code{region}.
#'
#' @export
#' @rdname brainGraph-methods

region.names <- function(object) {
  UseMethod('region.names')
}

#' @export
#' @method region.names data.table
#' @rdname brainGraph-methods
region.names.data.table <- function(object) {
  colname <- if (hasName(object, 'Region')) 'Region' else 'region'
  object[, levels(get(colname))]
}

#' @export
#' @method region.names brainGraph_resids
#' @rdname residuals
region.names.brainGraph_resids <- function(object) names(object$resids.all)[-c(1L, 2L)]

#' @export
#' @method region.names corr_mats
#' @rdname correlation_matrices
region.names.corr_mats <- function(object) dimnames(object$R)[[1L]]

#' @export
#' @method region.names mtpc
#' @include mtpc.R
#' @rdname mtpc
region.names.mtpc <- region.names.bg_GLM

#' Extract the number of regions in a brainGraph object
#'
#' \code{nregions} is a generic method for extracting the number of regions from
#' various \code{brainGraph} objects.
#'
#' @export
#' @rdname brainGraph-methods

nregions <- function(object) {
  UseMethod('nregions')
}

#' @export
#' @rdname residuals
nregions.brainGraph_resids <- function(object) dim(object$resids.all)[2L] - 2L

#' @export
#' @rdname correlation_matrices
nregions.corr_mats <- function(object) dim(object$R)[1L]

#' @export
#' @include mtpc.R
#' @rdname mtpc
nregions.mtpc <- nregions.bg_GLM

#' @export
#' @rdname NBS
nregions.NBS <- function(object) dim(object$T.mat)[1L]

#' Calculate coefficient of variation
#'
#' \code{coeff_var} is a S3 generic that calculates the \emph{coefficient of
#' variation}, defined as
#' \deqn{CV(x) = \frac{sd(x)}{mean(x)}}
#'
#' If \code{x} is a matrix, it will calculate the CV for each \emph{column}. If
#' \code{x} is a 3D array, it will calculate the coefficient of variation for
#' each \emph{row-column} combination. If the input dimensions are \eqn{n \times
#' n \times r}, a matrix with size \eqn{n \times n} will be returned.
#'
#' @param x Numeric vector, matrix, or array
#' @param na.rm Logical indicating whether \code{NA} values should be stripped
#'   when calculating sums. Default: \code{FALSE}
#' @param ... Unused
#' @return A numeric vector or matrix
#' @export

coeff_var <- function(x, na.rm=FALSE, ...) {
  UseMethod('coeff_var')
}

#' @export
#' @rdname coeff_var
coeff_var.default <- function(x, na.rm=FALSE, ...) {
  N <- sum(!is.na(x))
  mu <- sum(x, na.rm=na.rm) / N
  sqrt(1 / (N - 1L) * (sum((x - mu)^2, na.rm=na.rm))) / mu
}

#' @export
coeff_var.matrix <- function(x, na.rm=FALSE, ...) {
  apply(x, 2L, coeff_var)
}

#' @export
coeff_var.array <- function(x, na.rm=FALSE, ...) {
  dimX <- dim(x)
  N <- if (anyNA(x)) apply(x, 1L:2L, function(z) sum(!is.na(z))) else dimX[3L]
  mu <- rowSums(x, na.rm=na.rm, dims=2L) / N
  sqrt(1 / (N - 1L) * (rowSums((x - array(mu, dim=dimX))^2, na.rm=na.rm, dims=2L))) / mu
}

#' Calculate the inverse of the cross product of a design matrix
#'
#' \code{inv} is a \code{S3} generic that calculates the inverse of the cross
#' product of a design matrix, also referred to as the \dQuote{unscaled
#' covariance matrix}.
#'
#' If \code{x} is a matrix, the Cholesky decomposition of the cross product is
#' calculated (or using \code{\link{tcrossprod}} if \code{transpose=TRUE}), and
#' the inverse is calculated from that result. That is,
#' \deqn{inv(X) = (X^T X)^{-1}}
#' \deqn{inv(X, transpose=TRUE) = (X X^T)^{-1}}
#' \deqn{inv(X, y) = (X^T y)^{-1}}
#'
#' If \code{x} is a 3-dimensional array, then the inverse will be calculated for
#' each matrix along the 3rd dimension, with the same input arguments for each.
#'
#' Finally, there is a method for objects with class \code{qr}, and lists of QR
#' decomposition objects.
#'
#' @note These methods should only be used on \emph{full-rank} matrices, as
#' there is no error checking being performed.
#'
#' @param x A numeric matrix or array, a \code{qr} object, or a list of
#'   \code{qr} objects
#' @param ... Unused
#' @export
#' @return A numeric matrix or array
#' @name Inverse
#' @rdname inverse

inv <- function(x, ...) {
  UseMethod('inv')
}

#' @param y A numeric matrix or vector (for the \code{matrix} and \code{array}
#'   methods). If supplied, this will be multiplied by \code{x} before the
#'   inverse is calculated. Default: \code{NULL}
#' @param transpose Logical. If \code{FALSE} (the default), take the cross
#'   product of the arguments. If \code{TRUE}, use \code{\link{tcrossprod}}
#' @export
#' @rdname inverse

inv.matrix <- function(x, y=NULL, transpose=FALSE, ...) {
  xty <- if (isTRUE(transpose)) tcrossprod(x, y) else crossprod(x, y)
  xinv <- chol2inv(chol.default(xty))
  dimnames(xinv) <- if (isTRUE(transpose)) dimnames(x)[c(1L, 1L)] else dimnames(x)[c(2L, 2L)]
  xinv
}

#' @export
#' @rdname inverse

inv.array <- function(x, y=NULL, transpose=FALSE, ...) {
  namesX <- dimnames(x)
  dimX <- dim(x)
  dim_ind <- if (isTRUE(transpose)) 1L else 2L
  if (is.null(y)) {
    dims <- dimX[c(dim_ind, dim_ind)]
    nms <- namesX[c(dim_ind, dim_ind)]
  } else {
    dims <- c(dimX[dim_ind], dim(y)[dim_ind])
    nms <- c(namesX[dim_ind], dimnames(y)[dim_ind])
  }
  xinv <- array(0, dim=c(dims, dimX[3L]), dimnames=c(nms, namesX[3L]))
  for (k in seq_len(dimX[3L])) {
    xinv[, , k] <- inv(abind::adrop(x[, , k, drop=FALSE], drop=3L), y, transpose)
  }
  xinv
}

#' @export
#' @rdname inverse

inv.qr <- function(x, p=x$rank, ...) {
  xinv <- chol2inv(x$qr, p)
  dimnames(xinv) <- dimnames(x$qr)[c(2L, 2L)]
  xinv
}

#' @param p The rank of the original matrix
#' @param r The number of design matrices; i.e., the length of the input list
#' @param vnames Character vector of the design matrix's variable names
#' @param nms The region names; i.e., the names of the input list
#' @export
#' @rdname inverse

inv.list <- function(x, p=x[[1L]]$rank, r=length(x), vnames=dimnames(x[[1L]]$qr)[[2L]],
                     nms=names(x), ...) {
  array(vapply(x, inv, numeric(p * p), p), dim=c(p, p, r), dimnames=list(vnames, vnames, nms))
}

#' Calculate vertex hubness
#'
#' \code{hubness} calculates the \dQuote{hubness} (see reference) of the
#' vertices in a graph. These are vertices which meet at least two of the
#' following four criteria:
#' \enumerate{
#'   \item Have high degree/strength
#'   \item Have high betweenness centrality
#'   \item Have low clustering coefficient
#'   \item Have low average path length
#' }
#' For each criterion, \dQuote{high} or \dQuote{low} means \dQuote{in the top
#' 20\%} across all vertices. Vertices meeting any of the criteria get a value
#' of 1 for that metric; these are summed to yield the hubness score which
#' ranges from 0-4. As in the reference article, vertices with a score of 2 or
#' higher are to be considered hubs, although that determination isn't made in
#' this function.
#'
#' @param prop.keep Numeric (between 0 and 1) indicating the proportion of
#'   vertices to consider as having a high score. Default: 0.2 (20\%)
#' @inheritParams efficiency
#' @inheritParams xfm.weights
#' @export
#'
#' @return A numeric vector with the vertices' hubness score
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}
#' @references van den Heuvel, M.P. and Mandl, R.C.W. and Stam, C.J. and Kahn,
#'   R.S. and Pol, H.E.H. (2010) Aberrant frontal and temporal complex network
#'   structure in schizophrenia: a graph theoretical analysis. \emph{The Journal
#'   of Neuroscience}, \bold{30(47)}, 15915--15926.
#'   \doi{10.1523/JNEUROSCI.2874-10.2010}

hubness <- function(g, xfm.type=g$xfm.type, weights=NULL, prop.keep=0.2) {
  stopifnot(is_igraph(g), prop.keep <= 1, prop.keep >= 0)
  Nv <- vcount(g)
  cutoff <- round(prop.keep * Nv)
  weights <- check_weights(g, weights)

  S <- strength(g, weights=weights)
  Cp <- transitivity(g, type='weighted', isolates='zero', weights=weights)
  btwn <- centr_betw(g)$res

  vattr <- 'Lp'
  if (is_weighted(g)) {
    g <- xfm.weights(g, xfm.type)
    vattr <- 'Lp.wt'
  }
  if (vattr %in% vertex_attr_names(g)) {
    Lp <- vertex_attr(g, vattr)
  } else {
    Lp <- mean_distance_wt(g, 'vertex', weights=weights)
  }

  M <- matrix(c(-S, -btwn, Cp, Lp), nrow=Nv, ncol=4L)
  H <- matrix(0L, nrow=Nv, ncol=4L)
  for (i in 1L:4L) H[order(M[, i])[1L:cutoff], i] <- 1L
  hubs <- rowSums(H)
  return(hubs)
}

#' Atlas helper functions
#'
#' \code{guess_atlas} tries to determine which atlas is being used based on the
#' data; i.e., the number of vertices/regions.
#'
#' @section Guessing the atlas from an object:
#' There are several valid inputs to \code{guess_atlas}:
#' \describe{
#'   \item{data.table}{The atlas will be guessed based on the number of columns
#'     (subtracting by 1 if a \dQuote{Study ID} column is present). This is the
#'     same behavior as for \code{data.frame} objects, as well.}
#'   \item{igraph}{The vertex count}
#'   \item{brainGraph}{If there is a \code{atlas} graph-level attribute, it will
#'     return that. Otherwise, the vertex count.}
#'   \item{matrix,array}{The number of rows, which should equal the number of
#'     columns if the input is a connectivity matrix.}
#' }
#'
#' Note that this will only work properly for atlases that are currently in the
#' package. If you are using a custom atlas and you receive errors, please open
#' an issue on \emph{GitHub}.
#'
#' @param x,object An object to test or convert to an atlas data.table
#' @export
#' @return \code{guess_atlas} - Character string; either the matched atlas or
#'   \code{NA}
#' @name Atlas Helpers
#' @rdname atlas_helpers

guess_atlas <- function(x) {
  bgAtlases <- data(package='brainGraph')$results[, 3L]
  Nv <- vapply(bgAtlases, function(x) dim(get(x))[1L], integer(1L))

  n <- switch(class(x)[1L],
              data.frame=, data.table=dim(x)[2L] - (hasName(x, getOption('bg.subject_id'))),
              igraph=vcount(x),
              brainGraph=if (is.null(x$atlas)) vcount(x) else x$atlas,
              matrix=, array=dim(x)[1L],
              x)

  if (is.character(n)) {
    atlas <- n
  } else {
    matched <- which(Nv == n)
    atlas <- if (length(matched) > 0L) names(matched) else NA_character_
  }
  return(atlas)
}

#' Check if an object is a valid atlas, and convert if it isn't
#'
#' \code{as_atlas} and \code{create_atlas} converts/coerces an object to a
#' a \code{data.table}, or creates one, that is compatible with
#' \code{brainGraph}.
#'
#' @section Coercing to an atlas:
#' There are several things \code{as_atlas} tries to do to make it work without
#' error:
#' \itemize{
#'   \item Coerce the object to \code{data.table}
#'   \item Add a column of integers named \code{index}
#'   \item Change columns named \code{'x'}, \code{'y'}, or \code{'z'} to have
#'     \code{.mni} at the end
#'   \item Convert the \code{lobe} and \code{hemi} columns to be \emph{factors}
#' }
#'
#' @export
#' @return \code{as_atlas} and \code{create_atlas} return a \code{data.table}
#'   that conforms to other atlases in the package, or exits with an error.
#' @rdname atlas_helpers
#' @examples
#' my_atlas <- data.frame(name=paste('Region', 1:10), x.mni=rnorm(10),
#'   y.mni=rnorm(10), z.mni=rnorm(10),
#'   lobe=rep(c('Frontal', 'Parietal', 'Temporal', 'Occipital', 'Limbic'), 2),
#'   hemi=c(rep('L', 5), rep('R', 5)))
#' my_atlas2 <- as_atlas(my_atlas)
#' str(my_atlas)
#' str(my_atlas2)

as_atlas <- function(object) {
  index <- NULL
  # First, coerce to data.table
  if (!is.data.table(object)) {
    warning('Atlas is being coerced to "data.table"')
    object <- as.data.table(object)
  }

  # Check column names; add "index" if not present
  xnames <- names(object)
  if (!'index' %in% xnames) object[, index := seq_len(.N)]
  if ('x' %in% xnames) setnames(object, 'x', 'x.mni')
  if ('y' %in% xnames) setnames(object, 'y', 'y.mni')
  if ('z' %in% xnames) setnames(object, 'z', 'z.mni')
  atlas_cols <- c('name', 'x.mni', 'y.mni', 'z.mni', 'lobe', 'hemi')
  if (!all(hasName(object, atlas_cols))) {
    stop(paste('Atlas is invalid. Required columns are:\n', paste(atlas_cols, collapse=' ')))
  }

  for (col in c('lobe', 'hemi')) {
    if (!is.factor(object[[col]])) {
      warning('Coercing "', col, '" column to factor')
      object[, eval(col) := as.factor(get(col))]
    }
  }
  other_names <- setdiff(atlas_cols, names(object))
  setcolorder(object, c(atlas_cols, other_names))
  return(object)
}

#' @param regions Character vector of region names
#' @param coords Numeric matrix of spatial coordinates; must have 3 columns
#' @param lobes Character or factor vector of lobe membership
#' @param hemis Character or factor vector of hemisphere membership. There
#'   should probably not be more than 3 unique elements (for left, right, and
#'   bi-hemispheric regions)
#' @param other A \emph{named list} of vectors with other data. The names of the
#'   list will become column names in the return object.
#' @export
#' @rdname atlas_helpers
#' @examples
#' regions <- paste('Region', 1:10)
#' xyz <- matrix(rnorm(30), nrow=10, ncol=3)
#' lobe <- rep(c('Frontal', 'Parietal', 'Temporal', 'Occipital', 'Limbic'), 2)
#' hemi <- c(rep('L', 5), rep('R', 5))
#' other <- list(network=rep(c('Default mode', 'Task positive'), 5))
#' my_atlas <- create_atlas(regions, xyz, lobe, hemi, other)
#' str(my_atlas)

create_atlas <- function(regions, coords, lobes, hemis, other=NULL) {
  if (is.list(coords)) {
    warning("'coords' is a list; using only the first element.")
    coords <- coords[[1L]]
  }
  if (!is.numeric(coords)) {
    warning('Coercing coordinates to numeric.')
    try(coords <- as.numeric(coords))
  }
  if (!is.matrix(coords)) {
    warning('Converting coordinates to a 3-column matrix.')
    try(coords <- matrix(coords, ncol=3L))
  }
  dims <- dim(coords)
  if (dims[2L] != 3L) stop('Coordinates must be a matrix with 3 columns.')

  kNumRegions <- length(regions)
  kNumCoords <- dims[1L]
  kNumLobes <- length(lobes)
  kNumHemis <- length(hemis)
  if (length(unique(c(kNumRegions, kNumCoords, kNumLobes, kNumHemis))) != 1L) {
    stop('All input data must have the same length.')
  }

  out <- data.table(name=as.character(regions),
                    x.mni=coords[, 1L],
                    y.mni=coords[, 2L],
                    z.mni=coords[, 3L],
                    lobe=as.factor(lobes),
                    hemi=as.factor(hemis),
                    index=seq_len(kNumCoords))

  # Add extra columns, if given
  if (!is.null(other)) {
    nms <- names(other)
    stopifnot(!is.null(nms))
    stopifnot(all(lengths(other) == kNumRegions))
    for (x in nms) out[, eval(x) := other[[x]]]
  }
  return(out)
}

#' Calculate Euclidean distance of edges and vertices
#'
#' \code{edge_spatial_dist} calculates the Euclidean distance of an
#' \code{igraph} graph object's edges. The distances are in \emph{mm} and based
#' on MNI space. These distances are \emph{NOT} along the cortical surface, so
#' can only be considered approximations, particularly concerning
#' inter-hemispheric connections. The input graph must have \emph{atlas} as a
#' graph-level attribute.
#'
#' @param g An \code{igraph} graph object
#' @export
#'
#' @return \code{edge_spatial_dist} - a numeric vector with length equal to the
#'   edge count of the input graph, consisting of the Euclidean distance (in
#'   \emph{mm}) of each edge
#'
#' @name Graph Distances
#' @rdname spatial_dist
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}

edge_spatial_dist <- function(g) {
  stopifnot(is_igraph(g), 'atlas' %in% graph_attr_names(g))

  name <- x.mni <- y.mni <- z.mni <- NULL
  coords <- get(g$atlas)[, list(name, x.mni, y.mni, z.mni)]
  setkey(coords, name)
  es <- as_edgelist(g)
  dists <- sqrt(rowSums((coords[es[, 2L], list(x.mni, y.mni, z.mni)] -
                         coords[es[, 1L], list(x.mni, y.mni, z.mni)])^2))
}

#' Calculate average Euclidean distance for each vertex
#'
#' \code{vertex_spatial_dist} calculates, for each vertex of a graph, the
#' average Euclidean distance across all of that vertex's connections.
#'
#' @inheritParams edge_spatial_dist
#' @export
#' @importFrom Matrix colSums
#'
#' @return \code{vertex_spatial_dist} - a named numeric vector with length equal
#'   to the number of vertices, consisting of the average distance (in
#'   \emph{mm}) for each vertex
#'
#' @rdname spatial_dist
#' @references Alexander-Bloch, A.F. and Vertes, P.E. and Stidd, R. et al.
#'   (2013) The anatomical distance of functional connections predicts brain
#'   network topology in health and schizophrenia. \emph{Cerebral Cortex},
#'   \bold{23}, 127--138. \doi{10.1093/cercor/bhr388}

vertex_spatial_dist <- function(g) {
  stopifnot(is_igraph(g), 'dist' %in% edge_attr_names(g))

  A <- as_adj(g, names=FALSE, attr='dist')
  dists <- colSums(A) / colSums(A != 0)
  return(dists)
}

#' Plot vertex-level graph measures at a single density or threshold
#'
#' Creates boxplots of a single vertex-level graph measure at a single density
#' or threshold, grouped by the variable specified by \code{group.by} and
#' optionally faceted by another variable (e.g., \emph{lobe} or \emph{network}).
#'
#' @param measure A character string of the graph measure to plot
#' @param facet.by Character string indicating the variable to facet by (if
#'   any). Default: \code{NULL}
#' @param group.by Character string indicating which variable to group the data
#'   by. Default: \code{getOption('bg.group')}
#' @param type Character string indicating the plot type. Default:
#'   \code{'violin'}
#' @param show.points Logical indicating whether or not to show individual data
#'   points (default: FALSE)
#' @param ylabel A character string for the y-axis label
#' @param ... Arguments passed to \code{geom_boxplot} or \code{geom_violin}
#' @inheritParams plot_brainGraph_multi
#' @export
#'
#' @return A \code{trellis} or \code{ggplot} object
#'
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}
#' @examples
#' \dontrun{
#' p.deg <- plot_vertex_measures(g[[1]], facet.by='network', measure='degree')
#' }

plot_vertex_measures <- function(g.list, measure, facet.by=NULL, group.by=getOption('bg.group'),
                                 type=c('violin', 'boxplot'),
                                 show.points=FALSE, ylabel=measure, ...) {
  variable <- value <- NULL
  gID <- getOption('bg.group')

  if (!is.brainGraphList(g.list)) try(g.list <- as_brainGraphList(g.list))
  DT <- vertex_attr_dt(g.list)
  stopifnot(all(hasName(DT, c(measure, group.by))))
  idvars <- c('atlas', 'modality', 'weighting', getOption('bg.subject_id'), gID, 'threshold',
              'density', 'region', 'lobe', 'hemi', 'class', 'network')
  idvars <- idvars[which(hasName(DT, idvars))]
  DT.m <- melt(DT, id.vars=idvars)
  setnames(DT.m, group.by, 'group.by')

  type <- match.arg(type)

  # 'base' plotting
  if (!requireNamespace('ggplot2', quietly=TRUE)) {
    panelfun <- if (type == 'violin') panel.violin else panel.bwplot
    p <- bwplot(value ~ group.by | get(facet.by), data=DT.m[variable == measure],
                panel=panelfun,
                xlab=group.by, ylab=ylabel)

  # 'ggplot2' plotting
  } else {
    p <- ggplot2::ggplot(DT.m[variable == measure], ggplot2::aes(x=group.by, y=value, fill=group.by))
    p <- if (type == 'violin') p + ggplot2::geom_violin(...) else p + ggplot2::geom_boxplot(...)

    if (!is.null(facet.by)) {
      stopifnot(hasName(DT, facet.by))
      p <- p + ggplot2::facet_wrap(as.formula(paste('~', facet.by)), scales='free_y')
    }
    if (isTRUE(show.points)) {
      p <- p + ggplot2::geom_jitter(position=ggplot2::position_jitter(width=0.1, height=0))
    }
    p <- p + ggplot2::labs(x=group.by, y=ylabel) + ggplot2::theme(legend.position='none')
  }

  return(p)
}

#' Simulate N random graphs w/ same clustering and degree sequence as the input
#'
#' \code{sim.rand.graph.par} simulates \code{N} simple random graphs with the
#' same clustering (optional) and degree sequence as the input. Essentially a
#' wrapper for \code{\link[igraph]{sample_degseq}} (or, if you want to match by
#' clustering, \code{\link{sim.rand.graph.clust}}) and
#' \code{\link{make_brainGraph}}. It uses \code{\link[foreach]{foreach}} for
#' parallel processing.
#'
#' If you do not want to match by clustering, then simple rewiring of the input
#' graph is performed (the number of rewires equaling the larger of \code{1e4}
#' and \eqn{10 \times m}, where \eqn{m} is the graph's edge count).
#'
#' @param g A graph object
#' @param N Integer; the number of random graphs to simulate. Default: 100
#' @param clustering Logical; whether or not to control for clustering. Default:
#'   \code{FALSE}
#' @param rewire.iters Integer; number of rewiring iterations for the initial
#'   graph randomization. Default: 1e4
#' @param cl The clustering measure. Default: \emph{transitivity}
#' @param max.iters The maximum number of iterations to perform; choosing a
#'   lower number may result in clustering that is further away from the
#'   observed graph's. Default: 100
#' @param ... Other arguments passed to \code{\link{make_brainGraph}}
#' @inheritParams Creating_Graphs
#' @export
#' @importFrom foreach getDoParRegistered
#' @importFrom doParallel registerDoParallel
#'
#' @return \code{sim.rand.graph.par} - a \emph{list} of \emph{N} random graphs
#'   with some additional vertex and graph attributes
#'
#' @rdname random_graphs
#' @seealso \code{\link[igraph]{rewire}, \link[igraph]{sample_degseq},
#'   \link[igraph]{keeping_degseq}}
#' @examples
#' \dontrun{
#' rand1 <- sim.rand.graph.par(g[[1]][[N]], N=1e3)
#' rand1.cl <- sim.rand.graph.par(g[[1]][[N]], N=1e2,
#'   clustering=T, max.iters=1e3)
#' }

sim.rand.graph.par <- function(g, level=c('subject', 'group'), N=100L,
                               clustering=FALSE, rewire.iters=max(10*ecount(g), 1e4L),
                               cl=g$transitivity, max.iters=100L, ...) {
  stopifnot(is_igraph(g))
  level <- match.arg(level)
  if (!getDoParRegistered()) {
    clust <- makeCluster(getOption('bg.ncpus'))
    registerDoParallel(clust)
  }
  if (isTRUE(clustering)) {
    r <- foreach(i=seq_len(N), .packages=c('igraph', 'brainGraph'), .export=g$atlas) %dopar% {
      tmp <- sim.rand.graph.clust(g, rewire.iters, cl, max.iters)
      tmp$atlas <- g$atlas
      tmp <- make_brainGraph(tmp, type='random', level=level, set.attrs=TRUE, ...)
      tmp
    }
  } else {
    if (is_connected(g)) {
      V(g)$degree <- degree(g)
      r <- foreach(i=seq_len(N)) %dopar% {
        tmp <- sample_degseq(V(g)$degree, method='vl')
        tmp$Group <- g$Group
        tmp$name <- g$name
        tmp$threshold <- g$threshold
        tmp$density <- graph.density(tmp)
        tmp <- make_brainGraph(tmp, type='random', level=level, set.attrs=TRUE, ...)
        tmp
      }
    } else {
      r <- foreach(i=seq_len(N)) %dopar% {
        tmp <- rewire(g, keeping_degseq(loops=FALSE, rewire.iters))
        tmp <- make_brainGraph(tmp, type='random', level=level, set.attrs=TRUE, ...)
        tmp
      }
    }
  }
}

#' Simulate a random graph with given degree sequence and clustering
#'
#' \code{sim.rand.graph.clust} simulates a random graph with a given degree
#' sequence \emph{and} clustering coefficient. Increasing the \code{max.iters}
#' value will result in a closer match of clustering with the observed graph.
#'
#' @export
#' @return \code{sim.rand.graph.clust} - A single \code{igraph} graph object
#'
#' @rdname random_graphs
#' @seealso \code{\link[igraph]{transitivity}}
#' @references Bansal, S. and Khandelwal, S. and Meyers, L.A. (2009) Exploring
#'   biological network structure with clustered random networks. \emph{BMC
#'   Bioinformatics}, \bold{10}, 405--421.
#'   \doi{10.1186/1471-2105-10-405}

sim.rand.graph.clust <- function(g, rewire.iters=1e4, cl=g$transitivity, max.iters=100) {
  g <- rewire(g, keeping_degseq(loops=FALSE, rewire.iters))
  g.cand <- g
  A <- as_adj(g.cand, sparse=FALSE, names=FALSE)
  degs <- colSums(A)
  degs.large <- which(degs > 1)

  cur.iter <- 0
  while ((transitivity(g) < cl) & (cur.iter < max.iters)) {
    repeat {
      g.cand <- g
      A <- as_adj(g.cand, sparse=FALSE, names=FALSE)

      # If E(y1, y2) and E(z1, z2) don't exist, rewire 2 edges
      repeat {
        e <- choose.edges(A, degs.large)
        if ((A[e$y1, e$y2] == 0) && (A[e$z1, e$z2] == 0)) break
      }

      A[e$y1, e$z1] <- A[e$z1, e$y1] <- A[e$y2, e$z2] <- A[e$z2, e$y2] <- 0
      A[e$y1, e$y2] <- A[e$y2, e$y1] <- A[e$z1, e$z2] <- A[e$z2, e$z1] <- 1
      g.cand <- graph_from_adjacency_matrix(A, mode='undirected')

      if (transitivity(g.cand) > transitivity(g)) break
    }

    cur.iter <- cur.iter + 1
    g <- g.cand
  }

  return(g)
}

#' Select edges for re-wiring
#'
#' \code{choose.edges} selects edges to be re-wired when simulating random
#' graphs while controlling for \emph{clustering}. It is based on the algorithm
#' by Bansal et al. (2009), BMC Bioinformatics.
#'
#' @param A Numeric (adjacency) matrix
#' @param degs.large Integer vector of vertex numbers with degree greater than
#'   one
#'
#' @return A data frame with four elements; two edges \code{(y1, z1)} and
#'   \code{(y2, z2)} will be removed, and two edges \code{(y1, y2)} and
#'   \code{(z1, z2)} will be added between the four vertices.
#' @keywords internal
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}

choose.edges <- function(A, degs.large) {
  # Uniformly select a random vertex with degree > 1 (x),
  # and 2 of its neighbors (y1 & y2)
  #-----------------------------------------------------------------------------
  get_random_v <- function(A, degs.large) {
    repeat {
      x <- degs.large[sample.int(length(degs.large), 1L)]
      neighb <- intersect(which(A[x, ] == 1), degs.large)
      if (length(neighb) >= 2L) return(list(x, neighb[sample.int(length(neighb), 2L)]))
    }
  }
  #-----------------------------------------------------------------------------
  repeat {
    tmp <- get_random_v(A, degs.large)
    x <- tmp[[1L]]
    y <- tmp[[2L]]

    # Try to select random neighbors (z1 & z2) of y1 & y2 s.t. z1 != z2 != x
    y1.neighb <- which(A[y[1L], ] == 1)
    choices1 <- setdiff(y1.neighb, c(x, y[2L]))
    if (length(choices1) == 0L) {
      next
    } else {
      z1 <- choices1[sample.int(length(choices1), 1L)]
    }

    y2.neighb <- which(A[y[2L], ] == 1)
    choices2 <- setdiff(y2.neighb, c(x, y[1L], z1))
    if (length(choices2) > 0L) break
  }

  z2 <- choices2[sample.int(length(choices2), 1L)]
  return(data.frame(y1=y[1L], y2=y[2L], z1, z2))
}

#' Hirschberger-Qi-Steuer null covariance matrix generation
#'
#' \code{sim.rand.graph.hqs} generates a number of random covariance matrices
#' using the Hirschberger-Qi-Steuer (HQS) algorithm, and create graphs from
#' those matrices.
#'
#' \code{sim.rand.graph.hqs} - The first step is to create the observed
#' covariance of residuals (or whatever matrix/data.table is provided). Then
#' random covariance matrices are created with the same distributional
#' properties as the observed matrix, they are converted to correlation
#' matrices, and finally graphs from these matrices. By default, weighted graphs
#' will be created in which the edge weights represent correlation values. If
#' you want binary matrices, you must provide a correlation threshold.
#'
#' @param resids A \code{brainGraph_resids} object, a \code{data.table} of
#'   residuals, or a numeric matrix
#' @param weighted Logical indicating whether to create weighted graphs. If
#'   true, a threshold must be provided.
#' @param r.thresh Numeric value for the correlation threshold, if
#'   \code{weighted=FALSE}.
#' @export
#' @inheritParams Creating_Graphs
#' @importFrom foreach getDoParRegistered
#' @importFrom doParallel registerDoParallel
#' @return \code{sim.rand.graph.hqs} - A list of random graphs from the null
#'   covariance matrices
#'
#' @rdname random_graphs
#' @references Hirschberger M., Qi Y., Steuer R.E. (2007) Randomly generating
#'   portfolio-selection covariance matrices with specified distributional
#'   characteristics. \emph{European Journal of Operational Research}.
#'   \bold{177}, 1610--1625. \doi{10.1016/j.ejor.2005.10.014}

sim.rand.graph.hqs <- function(resids, level=c('subject', 'group'), N=100L,
                               weighted=TRUE, r.thresh=NULL, ...) {
  level <- match.arg(level)
  if (inherits(resids, 'brainGraph_resids')) {
    sID <- getOption('bg.subject_id')
    gID <- getOption('bg.group')
    resids <- resids$resids.all[, !c(sID, gID), with=FALSE]
  }
  A <- cov.wt(resids)$cov
  a <- 1 / sqrt(diag(A))

  e <- mean(c(A[upper.tri(A)], A[lower.tri(A)]), na.rm=TRUE)
  v <- var(c(A[upper.tri(A)], A[lower.tri(A)]), na.rm=TRUE)
  ebar <- mean(diag(A), na.rm=TRUE)
  n <- dim(A)[1L]

  m <- max(2, floor((ebar^2 - e^2) / v))
  ehat <- sqrt(e / m)
  vhat <- sqrt(ehat^4 + v / m) - ehat^2

  # Returns a "n X n" matrix
  hqs <- function(ehat, vhat, n, m) {
    u <- matrix(0, nrow=n, ncol=m)
    for (i in seq_len(m)) {
      u[, i] <- runif(n)
    }
    Q <- qnorm(u)
    f <- ehat + sqrt(vhat)*Q
    sig <- tcrossprod(f)
    return(sig)
  }

  if (!getDoParRegistered()) {
    cl <- makeCluster(getOption('bg.ncpus'))
    registerDoParallel(cl)
  }
  if (isTRUE(weighted)) {
    g <- foreach(i=seq_len(N)) %dopar% {
      S <- hqs(ehat, vhat, n, m)
      R <- outer(a, a) * S  # Convert to a correlation matrix
      make_brainGraph(R, type='random', level=level, set.attrs=TRUE, weighted=TRUE, ...)
    }
  } else {
    if (is.null(r.thresh)) stop('Please provide a value for thresholding')
    g <- foreach(i=seq_len(N)) %dopar% {
      S <- hqs(ehat, vhat, n, m)
      R <- outer(a, a) * S  # Convert to a correlation matrix
      R.thresh <- ifelse(R > r.thresh, 1, 0)
      make_brainGraph(R.thresh, type='random', level=level, set.attrs=TRUE, ...)
    }
  }
  return(g)
}

#' Create a design matrix for linear model analysis
#'
#' \code{brainGraph_GLM_design} takes a \code{data.table} of covariates and
#' returns a \emph{design matrix} to be used in linear model analysis.
#'
#' There are three different ways to code factors: \emph{dummy}, \emph{effects},
#' or \emph{cell-means} (chosen by the argument \code{coding}). \emph{Effects}
#' coding is sometimes referred to as \emph{deviation} coding. \emph{Dummy}
#' coding is the default when calling \code{lm}. To understand the difference
#' between these, see Chapter 8 of the User Guide.
#'
#' @section Character variables:
#' The default behavior is to convert all character columns (excluding the Study
#' ID column and any that you list in the \code{binarize} argument) to factor
#' variables. To change this, set \code{factorize=FALSE}. So, if your covariates
#' include multiple character columns, but you want to convert \emph{Scanner} to
#' binary instead of a factor, you may still specify \code{binarize='Scanner'}
#' and get the expected result. \code{binarize} will convert the given factor
#' variable(s) into numeric variable(s), which is performed \emph{before}
#' centering (if applicable).
#'
#' @section Centering:
#' The argument \code{mean.center} will mean-center (i.e., subtract the mean of
#' from each variable) any non-factor variables (including any dummy/indicator
#' covariates). This is done \emph{after} \dQuote{factorizing} and
#' \dQuote{binarizing}. If \code{center.how='all'}, then the \dQuote{grand mean}
#' will be used; otherwise, the groupwise means will be used. The grouping
#' variable is determined by \code{center.by} and is by default \code{'Group'}.
#'
#' @section Interactions:
#' \code{int} specifies which variables should interact with one another. This
#' argument accepts both numeric/continuous (e.g., \emph{Age}) and factor
#' variables (e.g., \emph{Sex}). All interaction combinations will be generated:
#' if you supply 3 variables, all two-way and the single three-way interaction
#' will be generated. This variable \emph{must} have at least two elements; it
#' is otherwise ignored. It is generally recommended that centering be performed
#' when including interaction terms.
#'
#' @param covars A \code{data.table} of covariates
#' @param coding Character string indicating how factor variables will be coded.
#'   Default: \code{'dummy'}
#' @param factorize Logical indicating whether to convert \emph{character}
#'   columns into \emph{factor}. Default: \code{TRUE}
#' @param binarize Character vector specifying the column name(s) of the
#'   covariate(s) to be converted from type \code{factor} to \code{numeric}.
#'   Default: \code{NULL}
#' @param int Character vector specifying the column name(s) of the
#'   covariate(s) to test for an interaction. Default: \code{NULL}
#' @param mean.center Logical indicating whether to mean center non-factor
#'   variables. Default: \code{FALSE}
#' @param center.how Character string indicating whether to use the grand mean
#'   or groupwise means. Default: \code{'all'}
#' @param center.by Character string indicating which grouping variable to use
#'   for calculating means (if applicable). Default: \code{'Group'}
#' @export
#'
#' @return A numeric matrix. Rownames are subject ID's and column names are the
#'   variable names. There will be additional attributes recording the
#'   \code{coding}, \code{factorize}, and \code{mean.center} function arguments.
#'   There will also be attributes for \code{binarize} and \code{int} if they
#'   are not \code{NULL}, and \code{center.how} and \code{center.by} if
#'   \code{mean.center=TRUE}.
#'
#' @name GLM design
#' @rdname glm_design
#' @family GLM functions
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}
#' @examples
#' \dontrun{
#' # Recreate design matrix when "outcome == measure"
#' DT <- res.glm$DT.Xy[region == levels(region)[1L],
#'                     !c('region', res.glm$outcome),
#'                     with=FALSE]
#' X <- do.call(brainGraph_GLM_design, c(list(covars=DT),
#'                                       attributes(res.glm$X)[-c(1L, 2L)]))
#' all.equal(X, res.glm$X)
#' }

brainGraph_GLM_design <- function(covars, coding=c('dummy', 'effects', 'cell.means'),
                                  factorize=TRUE, binarize=NULL, int=NULL,
                                  mean.center=FALSE, center.how=c('all', 'within-groups'),
                                  center.by=getOption('bg.group')) {
  sID <- getOption('bg.subject_id')
  covars <- copy(covars)
  if (!hasName(covars, sID)) covars[, eval(sID) := seq_len(dim(covars)[1L])]
  covars[, eval(sID) := check_sID(get(sID))]
  X <- matrix(1, nrow=dim(covars)[1L], ncol=1L)
  dimnames(X) <- list(covars[, get(sID)], 'Intercept')

  attrs <- list(factorize=factorize, mean.center=mean.center)
  if (isTRUE(factorize)) {
    cols <- names(which(vapply(covars, is.character, logical(1L))))
    cols <- cols[!is.element(cols, sID)]
    cols <- setdiff(cols, binarize)
    if (length(cols) > 0L) covars[, (cols) := lapply(.SD, as.factor), .SDcols=cols]
  }
  if (!is.null(binarize)) {
    stopifnot(all(hasName(covars, binarize)))
    covars[, (binarize) := lapply(.SD, function(x) as.numeric(x) - 1), .SDcols=binarize]
    attrs <- c(attrs, list(binarize=binarize))
  }

  nums <- which(vapply(covars, is.numeric, logical(1L)))
  if (isTRUE(mean.center)) {
    center.how <- match.arg(center.how)
    covars[, (nums) := lapply(.SD, as.numeric), .SDcols=nums]
    if (center.how == 'all') {
      covars[, (nums) := lapply(.SD, function(x) x - mean(x)), .SDcols=nums]
    } else if (center.how == 'within-groups') {
      covars[, (nums) := lapply(.SD, function(x) x - mean(x)), .SDcols=nums, by=center.by]
    }
    attrs <- c(attrs, list(center.how=center.how, center.by=center.by))
  }
  if (length(nums) > 0L) X <- cbind(X, as.matrix(covars[, nums, with=FALSE]))

  factors <- names(which(vapply(covars, class, character(1L)) == 'factor'))
  coding <- match.arg(coding)
  attrs <- c(list(coding=coding), attrs)
  base_val <- switch(coding, effects=-1L, dummy=, cell.means=0L)
  starting <- switch(coding, cell.means=1, dummy=, effects=2)
  colRemove <- switch(coding, cell.means=999L, dummy=, effects=1L)  # Hack to keep all names w/ cell means
  if (coding == 'cell.means' && all(X[, 1L] == 1)) X <- X[, -1L, drop=FALSE]  # Remove intercept term
  for (f in factors) X <- create_dummy_vars(covars, X, f, base_val, starting, colRemove)

  if (!is.null(int) && length(int) > 1L) {
    stopifnot(all(hasName(covars, int)))
    intcomb <- combn(int, 2L, simplify=FALSE)
    if (length(int) == 3L) intcomb <- c(intcomb, combn(int, 3L, simplify=FALSE))
    for (x in intcomb) X <- get_int(X, coding, factors, x)
    attrs <- c(attrs, list(int=int))
  }

  attributes(X) <- c(attributes(X), attrs)
  return(X)
}

#' Create dummy variables from a factor
#'
#' Create dummy variables from a factor variable in a data.table.
#'
#' @param fact Character string specifying the column name of the factor
#' @param base_val Integer value for the \dQuote{base} group or factor level.
#'   For dummy and cell-means coding, this is 0. For effects coding, it is -1.
#' @param starting Numeric; the starting value to iterate over factor levels.
#'   For dummy and effects coding, this is 2. For cell-means, it is 1.
#' @param colRemove Numeric specifying which factor level to exclude when
#'   assigning names. For dummy and effects coding, this is 1 (i.e., there is
#'   no column for the base factor level in the matrix). For cell-means, this is
#'   999 (a hack to make it work without knowing the number of levels in
#'   advance)
#' @return A numeric matrix with new columns
#' @noRd

create_dummy_vars <- function(covars, X, fact, base_val, starting, colRemove) {
  cov.levels <- covars[, levels(get(fact))]
  cov.vec <- covars[, as.numeric(get(fact))]
  for (i in starting:max(cov.vec)) {
    cov.vec.sub <- ifelse(cov.vec == i, 1L, ifelse(cov.vec == 1, base_val, 0))
    X <- cbind(X, cov.vec.sub)
  }
  p <- dim(X)[2L]
  colnames(X)[(p - max(cov.vec) + starting):p] <- paste0(fact, cov.levels[-colRemove])
  return(X)
}

get_int <- function(X, coding, factors, int) {
  get_colnames <- function(string, X) {
    cnames <- grep(string, colnames(X), value=TRUE)
    if (any(grepl(':', cnames))) cnames <- cnames[-grep(':', cnames)]
    return(cnames)
  }

  p <- dim(X)[2L]
  intnames <- lapply(int, get_colnames, X)

  if (length(int) == 3L) {
    X <- cbind(X, X[, intnames[[1L]]] * X[, intnames[[2L]]] * X[, intnames[[3L]]])
    colnames(X)[(p + 1L):dim(X)[2L]] <- paste(intnames[[1L]], intnames[[2L]], intnames[[3L]], sep=':')
  } else {
    if (all(int[1L:2L] %in% factors) && coding == 'cell.means') {
      for (j in seq_along(intnames[[1L]])) {
        X <- cbind(X, X[, intnames[[1L]][j]] * X[, intnames[[2L]]])
        colnames(X)[(p + 1L):dim(X)[2L]] <- paste(intnames[[1L]][j], intnames[[2L]], sep=':')
        p <- dim(X)[2L]
      }
      X <- X[, -which(colnames(X) %in% intnames[[1L]])]
      X <- X[, -which(colnames(X) %in% intnames[[2L]])]
    } else {  # One of the int. terms is numeric
      X <- cbind(X, X[, intnames[[1L]]] * X[, intnames[[2L]]])
      colnames(X)[(p + 1L):dim(X)[2L]] <- paste(intnames[[1L]], intnames[[2L]], sep=':')
      if (coding == 'cell.means') {
        if (!int[1L] %in% factors) {
          X <- X[, -which(colnames(X) %in% intnames[[1L]])]
        } else if (!hasName(factors, int[2L])) {
          X <- X[, -which(colnames(X) %in% intnames[[2L]])]
        }
      }
    }
  }
  return(X)
}

#' Contract graph vertices based on brain lobe and hemisphere
#'
#' Create a new graph after merging vertices within specified groups.
#' By default, groups are brain \emph{lobe} and \emph{hemisphere} membership.
#'
#' The \code{size} vertex-level attribute of the resultant graph is equal to the
#' number of vertices in each group. The x-, y-, and z-coordinates of the new
#' graph are equal to the mean coordinates of the vertices per group.
#' The new edge weights are equal to the number of inter-group connections of
#' the original graph.
#'
#' @param g A \code{brainGraph} graph object
#' @param vgroup Character string; the name of the vertex attribute to use when
#'   contracting the graph. Default: \code{'lobe.hemi'}
#' @export
#'
#' @return A new \code{brainGraph} graph object with vertex-level attributes
#'   representing the mean spatial coordinates, and vertex- and edge-level
#'   attributes of color names
#'
#' @seealso \code{\link[igraph]{contract}}
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}

contract_brainGraph <- function(g, vgroup='lobe.hemi') {
  stopifnot(is.brainGraph(g), vgroup %in% vertex_attr_names(g))

  vattrs <- vertex_attr(g, vgroup)
  if (!is.numeric(vattrs)) vattrs <- as.numeric(factor(vattrs))
  g.sub <- contract(g, vattrs, vertex.attr.comb='concat')
  E(g.sub)$count <- 1
  g.sub <- simplify(g.sub, edge.attr.comb=list(count='sum', 'concat'))

  # Simplify coordinate- and color-related vertex and edge attributes
  for (nam in c('x', 'y', 'z')) {
    val <- vapply(vertex_attr(g.sub, nam), mean, numeric(1L))
    g.sub <- delete_vertex_attr(g.sub, nam)
    g.sub <- set_vertex_attr(g.sub, nam, value=val)
  }
  for (nam in grep('color', vertex_attr_names(g.sub), value=TRUE)) {
    vvals <- lapply(vertex_attr(g.sub, nam), unique)
    if (all(lengths(vvals) == 1L)) {
      g.sub <- delete_vertex_attr(g.sub, nam)
      g.sub <- set_vertex_attr(g.sub, nam, value=unlist(vvals))
    }
    evals <- lapply(edge_attr(g.sub, nam), unique)
    if (all(lengths(evals) == 1L)) {
      g.sub <- delete_edge_attr(g.sub, nam)
      g.sub <- set_edge_attr(g.sub, nam, value=unlist(evals))
    }
  }

  V(g.sub)$size <- vapply(V(g.sub), function(v) sum(E(g.sub)[v %--% V(g.sub)]$count), numeric(1L))

  class(g.sub) <- c('brainGraph', class(g.sub))
  return(g.sub)
}

#' Plot a brain graph with a specific spatial layout
#'
#' \code{plot.brainGraph} plots a graph in which the spatial layout of the nodes
#' is important. The network itself is plotted over a brain MRI slice from the
#' MNI152 template by default (when \code{mni=TRUE}).
#'
#' @section Selecting specific vertices to display:
#' With the argument \code{subgraph}, you can supply a simple logical expression
#' specifying which vertices to show. For example, \emph{'degree > 10'} will
#' plot only vertices with a \emph{degree} greater than 10. Combinations of
#' \emph{AND} (i.e., \code{&}) and \emph{OR} (i.e., \code{|}) are allowed. This
#' requires that any vertex attribute in the expression must be present in the
#' graph; e.g., \code{V(g)$degree} must exist.
#'
#' @section Title, subtitle, and label:
#' By default, a \emph{title} (i.e., text displayed at the top of the figure) is
#' not included. You can include one by passing a character string to
#' \code{main}, and control the size with \code{cex.main}. A \emph{subtitle}
#' (i.e., text at the bottom), is included by default and displays the number of
#' vertices and edges along with the graph density. To exclude this, specify
#' \code{subtitle=NULL}. A \dQuote{label} can be included in one corner of the
#' figure (for publications). For example, you can choose \code{label='A.'} or
#' \code{label='a)'}. Arguments controlling the location and appearance can be
#' changed, but the default values are optimal for bottom-left placement. See
#' \code{\link[graphics]{mtext}} for more details. The label-specific arguments
#' are:
#' \describe{
#'   \item{side}{The location. \code{1} is for bottom placement.}
#'   \item{line}{If \code{side=1} (bottom), a negative number places the text
#'     \emph{above} the bottom of the figure; a higher number could result in
#'     the bottom part of the text to be missing. This can differ if
#'     \code{plane='circular'}, in which case you may want to specify a positive
#'     number.}
#'   \item{adj}{Seems to be the percentage away from the margin. So, for
#'     example, \code{adj=0.1} would place the text closer to the center than
#'     the default value, and \code{adj=0.5} places it in the center.}
#'   \item{cex}{The degree of \dQuote{character expansion}. A value of 1 would
#'     not increase the text size.}
#'   \item{col}{The text color.}
#'   \item{font}{The font type. The default \code{font=2} is bold face. See
#'     \code{\link[graphics]{par}} for details.}
#' }
#'
#' @param x A \code{brainGraph} graph object
#' @param plane Character string indicating which orientation to plot.
#'   Default: \code{'axial'}
#' @param hemi Character string indicating which hemisphere to plot. Default:
#'   \code{'both'}
#' @param mni Logical indicating whether or not to plot over a slice of the
#'   brain. Default: \code{TRUE}
#' @param subgraph Character string specifying a logical condition for vertices
#'   to plot. Default: \code{NULL}
#' @param main Character string; the main title. Default: \code{NULL}
#' @param subtitle Character string; the subtitle. Default: \code{'default'}
#' @param label Character string specifying text to display in one corner of the
#'   plot (e.g., \code{'A.'}). Default: NULL
#' @param side Label placement. Default: \code{1} (bottom)
#' @param line Which margin line to place the text.
#' @param adj If \code{side=1}, a value closer to 0 places the text closer to
#'   the left margin. Default: \code{0.025}
#' @param cex Amount of character expansion of the label text. Default:
#'   \code{2.5}
#' @param col Label font color. Default: \code{'white'}
#' @param font Integer specifying the font type. Default: \code{2} (bold face)
#' @param show.legend Logical indicating whether or not to show a legend.
#'   Default: \code{FALSE}
#' @param rescale Logical, whether to rescale the coordinates. Default:
#'   \code{FALSE}
#' @param asp Numeric constant; the aspect ratio. Default: 0
#' @param ... Other parameters (passed to \code{\link[igraph]{plot.igraph}}).
#'   See \code{\link[igraph]{plot.common}} for details.
#' @export
#' @importFrom graphics legend mtext par title
#' @importFrom methods hasArg
#'
#' @family Plotting functions
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}
#' @examples
#' \dontrun{
#' plot(g[[1]], hemi='R')
#' plot(g[[1]], subgraph='degree > 10 | btwn.cent > 50')
#'
#' ## Place label in upper-left
#' plot(g.ex, label='A)', side=3, line=-2.5)
#' }

plot.brainGraph <- function(x, plane=c('axial', 'sagittal', 'circular'),
                            hemi=c('both', 'L', 'R'), mni=TRUE, subgraph=NULL,
                            main=NULL, subtitle='default', label=NULL, side=1,
                            line=-2, adj=0.025, cex=2.5, col='white', font=2,
                            show.legend=FALSE, rescale=FALSE, asp=0, ...) {
  stopifnot(is.brainGraph(x))

  plane <- match.arg(plane)
  hemi <- match.arg(hemi)
  if (plane == 'sagittal' && hemi == 'both') hemi <- 'L'
  if (isTRUE(mni) && plane != 'circular') plot_mni(plane, hemi)

  mult <- 100
  adjust <- 0L
  atlas <- graph_attr(x, 'atlas')
  Nv <- vcount(x)
  keep <- seq_len(Nv)
  if (plane == 'circular') {
    layout.circ <- rotation(layout.circle(x, order=V(x)$circle.layout), -pi/2 - pi/Nv)
  }
  #---------------------------------------------------------
  # Set figure window limits and handle plotting of individual hemispheres
  #---------------------------------------------------------
  if (hemi != 'both') subgraph <- paste0(subgraph, " & hemi == '", hemi, "'")
  if (!is.null(subgraph)) {
    if (substr(subgraph, 1L, 3L) == ' & ') subgraph <- substr(subgraph, 4L, nchar(subgraph))
    sg <- subset_graph(x, subgraph)
    x <- sg$g
    keep <- sg$inds
    Nv <- vcount(x)
  }

  if (plane == 'sagittal') {
    adjust <- 1L
    V(x)$x <- -V(x)$y.mni
    V(x)$y <- V(x)$z.mni
    xlim.g <- switch(atlas,
                     destrieux=, destrieux.scgm=, dosenbach160=c(-85, 120),
                     hcp_mmp1.0=c(-90, 125), hoa112=c(-85, 125), lpba40=c(-75, 120),
                     brainnetome=, gordon333=, power264=c(-90, 120),
                     c(-85, 110))
    ylim.g <- switch(atlas,
                     destrieux=, destrieux.scgm=, dosenbach160=c(-85, 120),
                     hoa112=c(-85, 130), lpba40=c(-80, 100), c(-85, 125))

    if (hemi == 'R') {
      V(x)$x <- -V(x)$x
      xlim.g <- -rev(xlim.g)
    }
  } else if (plane == 'axial') {
    xlim.g <- switch(atlas,
                     dk=, dk.scgm=, dkt=, dkt.scgm=c(-105, 105),
                     brainnetome=, hcp_mmp1.0=, hoa112=, lpba40=, power264=c(-100, 100),
                     dosenbach160=c(-95, 95), c(-92, 95))
    ylim.g <- switch(atlas,
                     aal90=, aal116=, aal2.94=, aal2.120=c(-110, 70),
                     brainsuite=, dosenbach160=c(-115, 85),
                     destrieux=, destrieux.scgm=, hcp_mmp1.0=c(-120, 83),
                     hoa112=c(-122, 77), lpba40=c(-115, 70), c(-125, 85))
  } else if (plane == 'circular') {
    par(new=TRUE, bg='black', mar=c(5, 0, 2, 0)+0.1)
    mult <- 1
    V(x)$x <- layout.circ[keep, 1L]
    V(x)$y <- layout.circ[keep, 2L]
    xlim.g <- c(-1.5, 1.5)
    ylim.g <- c(-1.00, 1.5)
  }

  #---------------------------------------------------------
  # Handle extra arguments in case a subgraph was created
  #---------------------------------------------------------
  # Default values
  #-------------------------------------
  medial <- which(abs(V(x)$x.mni) < 20)
  vcols <- 'lightblue'
  ecols <- 'red'
  vsize <- mult * 10
  ewidth <- 1.5
  vlabel <- V(x)$name
  fargs <- list(...)

  # Vertex and edge colors
  #-------------------------------------
  if (hasArg('vertex.color')) {
    vcols <- fargs$vertex.color
    if (is.character(vcols) && length(vcols) == 1L) {
      stopifnot(vcols %in% vertex_attr_names(x))
      vcols <- vertex_attr(x, vcols)
      if (adjust == 1L) vcols[medial] <- adjustcolor(vcols[medial], alpha.f=0.4)
    }
  }
  if (hasArg('edge.color')) {
    ecols <- fargs$edge.color
    if (is.character(ecols) && length(ecols) == 1L) {
      stopifnot(ecols %in% edge_attr_names(x))
      ecols <- edge_attr(x, ecols)
      if (adjust == 1L) {
        medial.es <- as.numeric(E(x)[medial %--% medial])
        ecols[medial.es] <- adjustcolor(ecols[medial.es], alpha.f=0.4)
      }
    }
  }

  # Vertex sizes & edge widths
  #-------------------------------------
  if (hasArg('vertex.size')) {
    if (is.character(fargs$vertex.size)) {
      stopifnot(fargs$vertex.size %in% vertex_attr_names(x))
      vsize <- mult * vertex_attr(x, fargs$vertex.size)
      if (any(!is.finite(vsize))) x <- delete_vertices(x, which(!is.finite(vsize)))
      vsize <- mult * vec.transform(vsize, as.numeric(min(vsize) > 0), 15)
    } else {
      vsize <- mult * fargs$vertex.size
    }
  }
  if (hasArg('edge.width')) {
    ewidth <- fargs$edge.width
    if (is.character(ewidth)) {
      stopifnot(ewidth %in% edge_attr_names(x))
      ewidth <- edge_attr(x, ewidth)
    }
  }
  # Vertex labels
  #-------------------------------------
  vlabel.dist <- ifelse(vsize >= mult * 10, 0, 5)
  vlabel.col <- ifelse(vcols %in% c('red', 'blue', 'magenta'), 'white', 'blue')
  vlabel.font <- 2
  if (hasArg('vertex.label')) {
    vlabel <- if (!is.na(fargs$vertex.label)) vertex_attr(x, fargs$vertex.label) else NA
  }
  vlabel.cex <- if (hasArg('vertex.label.cex')) fargs$vertex.label.cex else 1

  NextMethod(generic='plot', object=x, asp=asp, rescale=rescale, xlim=xlim.g, ylim=ylim.g,
       vertex.color=vcols, edge.color=ecols, vertex.size=vsize,
       edge.width=ewidth, vertex.label=vlabel, vertex.label.cex=vlabel.cex,
       vertex.label.dist=vlabel.dist, vertex.label.font=vlabel.font,
       vertex.label.color=vlabel.col, ...)

  if (!is.null(label)) mtext(label, side=side, line=line, adj=adj, cex=cex, col=col, font=font)

  if (!is.null(subtitle) && subtitle == 'default') {
    Ne <- ecount(x)
    g.density <- round(graph.density(x), digits=3L)
    subtitle <- paste('# vertices: ', Nv, '\t# edges: ', Ne, '\n', 'Density: ', g.density)
  }
  par(new=TRUE, mar=c(5, 0, 2, 0)+0.1)
  cex.main <- if (hasArg('cex.main')) fargs$cex.main else 2.5
  title(main=main, sub=subtitle, col.main='white', col.sub='white', cex.main=cex.main)

  if (isTRUE(show.legend) && hasArg('vertex.color')) {
    atlas.dt <- get(atlas)
    vattr <- strsplit(fargs$vertex.color, '.', fixed=TRUE)[[1L]][2L]
    if (vattr %in% c('lobe', 'network', 'class', 'area', 'Yeo_7network', 'Yeo_17network')) {
      vattr_all <- vertex_attr(x, vattr)
      vattr_graph <- unique(vattr_all)
      atlas_vals <- atlas.dt[, get(vattr)]
      cnames <- intersect(levels(atlas_vals), vattr_graph)
      totals <- unname(table(atlas_vals)[cnames])
      if (vattr != 'class') {
        cnames <- paste0(cnames, ': ', table(vattr_all)[cnames], ' / ', totals)
      }
      group.nums <- which(levels(atlas_vals) %in% vattr_graph)
    } else if (vattr == 'rich') {
      group.nums <- 1L:3L
      cnames <- c('Rich-club', 'Feeder', 'Local')
    } else {
      group.nums <- as.integer(which(table(vertex_attr(x, vattr)) > 2))
      cnames <- paste0(simpleCap(vattr), '. ', group.nums)
    }
    nGroups <- length(group.nums)
    cex <- ifelse(nGroups > 10L, 0.75, ifelse(nGroups > 5L, 0.85, 1))
    cols <- group.cols[group.nums]
    l.inset <- c(-0.01, 0)  # Increasing 1st moves it toward the center (L-R), 2nd toward center (U-D)
    loc <- 'topleft'
    if (!is.null(label) && side == 3) loc <- 'topright'
    legend(loc, cnames, fill=cols, bg='black', text.col='white', cex=cex, inset=l.inset)
  }
}

#' Plot a slice of the MNI152 template
#'
#' Helper function that plots a single slice from the MNI152 template.
#' @inheritParams plot.brainGraph
#' @noRd

plot_mni <- function(plane, hemi) {
  if (plane == 'axial') {
    slice <- 46
    X <- mni152_mat
  } else {
    if (hemi == 'L') {
      X <- mni152_L
      slice <- 62
    } else if (hemi == 'R') {
      X <- mni152_R
      slice <- 30
    }
  }
  dims <- dim(X)
  x <- seq_len(dims[1L])
  y <- seq_len(dims[2L])
  zlim <- c(3500, max(X[, , slice]))
  imcol <- gray(0:64/64)
  breaks <- seq.int(min(zlim), max(zlim), length.out=length(imcol) + 1L)

  par(mai=c(0, 0, 0, 0), mar=c(0, 0, 0, 0), bg='black', bty='o')
  graphics::image(x, y, X[, , slice], col=imcol, breaks=breaks, asp=1)
  par(new=TRUE)
}

#' Plot a graph with results from GLM-based analyses
#'
#' These methods are convenience functions for plotting a graph based on results
#' from GLM-based analyses (i.e., \code{\link{brainGraph_GLM},
#' \link{brainGraph_mediate}, \link{mtpc}, \link{NBS}}). There are several
#' default arguments which differ depending on the input object.
#'
#' The default arguments are specified so that the user only needs to type
#' \code{plot(x)} at the console, if desired. For all methods, the plot's
#' \emph{subtitle} will be omitted.
#'
#' @section NBS:
#' By default, a subgraph will be plotted consisting of only those vertices
#' which are part of a significant connected component. Vertex/edge colors will
#' correspond to connected component membership. Vertex names will be omitted.
#' Finally, the plot title will contain the contrast name.
#'
#' @section brainGraph_GLM:
#' By default, a subgraph will be plotted consisting of only those vertices for
#' which \eqn{p < \alpha}. It will also include a plot title with the outcome
#' measure and contrast name.
#'
#' @section mtpc:
#' By default, a subgraph will be plotted consisting of only those vertices for
#' which \eqn{A_{mtpc} > A_{crit}}. It will also include a plot title with the
#' outcome measure and contrast name.
#'
#' @section brainGraph_mediate:
#' By default, a subgraph will be plotted consisting of only those vertices for
#' which \eqn{P_{acme} < \alpha}. It will also include a plot title with the
#' treatment, mediator, and outcome variable names.
#'
#' @param x A \code{brainGraph_GLM}, \code{brainGraph_mtpc},
#'   \code{brainGraph_mediate}, or \code{brainGraph_NBS} object
#' @param alpha Numeric; the significance level. Default: \code{0.05}
#' @param subgraph Character string specifying the condition for subsetting the
#'   graph.
#' @param vertex.label Character vector of the vertex labels to be displayed.
#' @param vertex.color Character string specifying the vertex attribute to color
#'   the vertices by.
#' @param edge.color Character string specifying the edge attribute to color
#'   the edges by.
#' @param cex.main Numeric; the scaling factor for text size; see
#'   \code{\link[graphics]{par}}
#' @param ... Other arguments passed to \code{\link{plot.brainGraph}}
#' @inheritParams plot.brainGraph
#' @export
#' @name Plotting GLM graphs
#' @rdname glm_graph_plots
#' @family Plotting functions

plot.brainGraph_NBS <- function(x, alpha=0.05, subgraph=paste('p.nbs >', 1 - alpha),
                                vertex.label=NA,
                                vertex.color='color.comp', edge.color='color.comp', subtitle=NULL,
                                main=paste0('NBS: ', x$name), cex.main=2, ...) {
  NextMethod(generic='plot', object=x, subgraph=subgraph,
             vertex.label=vertex.label, vertex.color=vertex.color, edge.color=edge.color,
             main=main, subtitle=subtitle, cex.main=cex.main, ...)
}

#' @param p.sig Character string indicating which p-value to use for determining
#'   significance (default: \code{p})
#' @export
#' @rdname glm_graph_plots

plot.brainGraph_GLM <- function(x, p.sig=c('p', 'p.fdr', 'p.perm'),
                                subgraph=NULL,
                                main=paste0(x$outcome, ': ', x$name),
                                subtitle=NULL, cex.main=2, ...) {
  p.sig <- match.arg(p.sig)
  if (is.null(subgraph)) subgraph <- paste0(p.sig, ' > 1 -', x$alpha)
  NextMethod(generic='plot', object=x, subgraph=subgraph,
             main=main, subtitle=subtitle, cex.main=cex.main, ...)
}

#' @export
#' @rdname glm_graph_plots

plot.brainGraph_mtpc <- function(x, subgraph='sig == 1',
                                 main=paste0(x$outcome, ': ', x$name),
                                 subtitle=NULL, cex.main=2, ...) {
  NextMethod(generic='plot', object=x, subgraph=subgraph,
             main=main, subtitle=subtitle, cex.main=cex.main, ...)
}

#' @export
#' @rdname glm_graph_plots

plot.brainGraph_mediate <- function(x, subgraph='p.acme > 0.95',
          main=sprintf('Effect of "%s" on\n"%s"\nmediated by "%s"', x$treat, x$outcome, x$mediator),
          subtitle=NULL, cex.main=1, ...) {
  NextMethod(generic='plot', object=x, subgraph=subgraph,
             main=main, subtitle=subtitle, cex.main=cex.main, ...)
}

#' Calculate an asymmetry index based on edge counts
#'
#' Calculate an \emph{asymmetry index}, a ratio of intra-hemispheric edges in
#' the left to right hemisphere of a graph for brain MRI data.
#'
#' The equation is:
#' \deqn{A = \frac{E_{lh} - E_{rh}}{0.5 \times (E_{lh} + E_{rh})}}
#' where \emph{lh} and \emph{rh} are left and right hemispheres, respectively.
#' The range of this measure is \eqn{[-2, 2]} (although the limits will only be
#' reached if all edges are in one hemisphere), with negative numbers
#' indicating more edges in the right hemisphere, and a value of 0 indicating
#' equal number of edges in each hemisphere.
#'
#' The \code{level} argument specifies whether to calculate asymmetry for each
#' vertex, or for the whole hemisphere.
#'
#' @inheritParams efficiency
#' @param level Character string indicating whether to calculate asymmetry for
#'   each region, or the hemisphere as a whole (default: \code{'hemi'})
#' @export
#'
#' @return A data table with edge counts for both hemispheres and the asymmetry
#'   index; if \code{level} is \emph{vertex}, the data table will have
#'   \code{vcount(g)} rows.
#'
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}

edge_asymmetry <- function(g, level=c('hemi', 'vertex'), A=NULL) {
  stopifnot(is_igraph(g), 'hemi' %in% vertex_attr_names(g))

  level <- match.arg(level)
  if (is.null(A)) A <- as_adj(g, sparse=FALSE, names=FALSE)
  L <- which(V(g)$hemi == 'L')
  R <- which(V(g)$hemi == 'R')
  if (level == 'hemi') {
    lh <- sum(A[L, L]) / 2
    rh <- sum(A[R, R]) / 2
    regions <- 'all'

  } else if (level == 'vertex') {
    lh <- rowSums(A[, L, drop=FALSE])
    rh <- rowSums(A[, R, drop=FALSE])
    regions <- V(g)$name
  }
  asymm <- data.table(region=regions, lh=lh, rh=rh)
  asymm[, asymm := 2 * (lh - rh) / (lh + rh)]
  return(asymm)
}

# Get info from brainGraph and brainGraphList objects
#-------------------------------------------------------------------------------
print_bg_summary <- function(object) {
  name <- name_str <- dens.pct <- Group <- modality <- weighting <- thresh <- clustmethod <- 'N/A'

  ver <- vapply(object$version, as.character, character(1L))
  date_created <- sub('T', ' ', object$date)
  type <- simpleCap(object$type)
  atlasfull <-
    switch(object$atlas,
           aal116='AAL-116', aal2.120=, aal2.94='AAL2', aal90='AAL-90',
           brainsuite='Brainsuite', craddock200='Craddock-200',
           destrieux='Destrieux', destrieux.scgm='Destrieux + SCGM',
           dk='Desikan-Killiany', dk.scgm='Desikan-Killiany + SCGM',
           dkt='Desikan-Killiany-Tourville', dkt.scgm='Desikan-Killiany-Tourville + SCGM',
           dosenbach160='Dosenbach-160', hoa112='Harvard-Oxford cortical and subcortical',
           lpba40='LONI probabilistic brain atlas', hcp_mmp1.0='HCP MMP',
           brainnetome='Brainnetome', power264='Power-264', gordon333='Gordon-333', object$atlas)
  if (!is.null(object$modality)) {
    modality <-
      switch(object$modality, dti='DTI', fmri='fMRI', thickness='Cortical thickness',
             area='Cortical surface area', volume='Cortical/subcortical volume', object$modality)
  }
  if (!is.null(object$weighting)) {
    weighting <-
        switch(object$weighting, fa='FA (fractional anisotropy)',
               sld='Streamline density', pearson='Pearson correlation',
               spearman='Spearman\'s rank correlation',
               kendall='Kendall\'s rank correlation', partial='Partial correlation',
               object$weighting)
  }
  thresh_str <- 'Threshold:'
  if (!is.null(object$threshold)) {
    thresh <- prettyNum(object$threshold, ',', digits=4L)
    if (length(thresh) > 1L) {
      thresh_str <- sub(':', 's:', thresh_str)
      if (length(thresh) > 6L) thresh <- thresh[1L:6L]
      thresh <- paste(thresh, collapse='; ')
    }
  }
  if (!is.null(object$clust.method)) {
    clustmethod <-
      switch(object$clust.method, edge_betweenness='Edge betweenness',
             fast_greedy='Greedy optimization (hierarchical agglomeration)',
             infomap='Infomap', label_prop='Label propagation',
             leading_eigen='Leading eigenvector',
             louvain='Louvain (multi-level modularity optimization)', optimal='Optimal',
             spinglass='Potts spin glass model', walktrap='Walktrap algorithm',
             object$clust.method)
  }
  if (is.brainGraph(object)) {
    if (!is_weighted(object)) weighting <- 'Unweighted'
    dens.pct <- sprintf('%1.2f%s', 100 * graph.density(object), '%')
    if (!is.null(object$name)) name <- object$name
    if (!is.null(object$Group)) Group <- object$Group
    name_str <- switch(object$level,
                       group='Group:',
                       contrast='Contrast:',
                       'Subject ID:')
  } else if (is.brainGraphList(object)) {
    gg <- if (object$type == 'random') object$graphs[[1L]][[1L]] else object$graphs[[1L]]
    if (!is_weighted(gg)) weighting <- 'Unweighted'
  }

  df <- data.frame(
          A=c('Software versions',
              '       R release:', '      brainGraph:', '          igraph:',
              'Date created:', 'Observed or random?', 'Brain atlas used:',
              'Imaging modality:', 'Edge weighting:', 'Clustering method:',
              'Graph density:', thresh_str, name_str, 'Group:'),
          B=c('', ver, date_created, type, atlasfull, modality, weighting,
              clustmethod, dens.pct, thresh, name, Group))
  dimnames(df)[[2L]] <- rep.int('', 2L)
  return(df)
}

# Print a character vector "x" as a data.frame with "numCols" columns
#-------------------------------------------------------------------------------
print_text_vector <- function(x, numCols) {
  div <- length(x) %/% numCols
  splits <- split(x, ceiling(seq_along(x) / div))
  lens <- lengths(splits)
  nsplits <- length(splits)
  splits[[nsplits]] <- c(splits[[nsplits]], rep.int('', div - lens[nsplits]))
  attrs.df <- as.data.frame(splits, stringsAsFactors=FALSE)
  dimnames(attrs.df)[[2L]] <- rep.int('', nsplits)
  return(attrs.df)
}

# Print a simple title
#-------------------------------------------------------------------------------
print_title_summary <- function(...) {
  title <- paste0(..., collapse='')
  width <- max(getOption('width') / 2L, nchar(title))
  message('\n', rep.int('=', width))
  message(title)
  message(rep.int('=', width))
}

# Print info about the model
#-------------------------------------------------------------------------------
print_model_summary <- function(x) {
  cat('GLM formula:\n\t', formula(x), '\n')
  cat('based on', nobs(x), 'observations, with', df.residual(x), 'degrees of freedom.')
  cat('\n\n')
  invisible(x)
}

# Print details about the contrast matrix
#-------------------------------------------------------------------------------
print_contrast_type_summary <- function(x) {
  cat('Contrast type: ', paste(toupper(x$con.type), 'contrast'), '\n')
  symb <- switch(x$alt, two.sided='!=', greater='>', less='<')
  alt <- sprintf('C %s 0', symb)
  cat('Alternative hypothesis: ', alt, '\n')
  cat('Contrasts: ', '\n')

  con <- x$contrasts
  if (is.matrix(con)) con <- list(con)
  for (i in seq_along(con)) {
    con[[i]] <- as.character(MASS::fractions(con[[i]]))
    con[[i]] <- sub('^0$', '.', con[[i]])
    if (is.list(x$contrasts)) message(x$con.name[i])
    print(format(con[[i]], justify='right'), quote=FALSE)
    cat('\n')
  }

  invisible(x)
}

# Print the subjects removed due to incomplete data
#-------------------------------------------------------------------------------
print_subs_summary <- function(x) {
  n <- length(x$removed.subs)
  if (n > 0L) {
    message(n, ' subjects removed due to incomplete data:')
    cat('  ', paste(names(x$removed.subs), collapse=', '), '\n')
  }
  invisible(x)
}

# Print per-contrast statistics
#-------------------------------------------------------------------------------
print_contrast_stats_summary <- function(x, ...) {
  Contrast <- csize <- p.perm <- `p-value` <- NULL
  printCon <- if (is.null(x$printCon)) x$DT[, unique(Contrast)] else x$printCon

  cls <- class(x)
  if ('summary.NBS' %in% cls) {
    DT <- x$DT.sum[csize > 1]
    setnames(DT, c('csize', 'ecount'), c('# vertices', '# edges'))
    DT[, `p-value` := signif(p.perm)]
    DT[, c('alt', 'N', 'p.perm') := NULL]
  } else {
    DT <- x$DT.sum[, !c(intersect(names(x$DT.sum), 'Outcome')), with=FALSE]
    oldnames <- grep('p-value', names(x$DT.sum), value=TRUE)
    newname <- if (x$con.type == 'f') 'Pr(>F)' else 'Pr(>|t|)'
    newnames <- sub('p-value', newname, oldnames)
    setnames(DT, oldnames, newnames)
  }

  for (i in printCon) {
    message(i, ': ')
    if (dim(DT[Contrast == i])[1L] == 0L) {
      message('\tNo signficant results!\n')
    } else {
      if ('summary.NBS' %in% cls) {
        printCoefmat(DT[Contrast == i, !'Contrast'], tst.ind=2L, P.values=TRUE, has.Pvalue=TRUE, digits=x$digits, ...)
      } else {
        if (isTRUE(x$print.head)) {
          print(DT[Contrast == i, !'Contrast'], topn=5L, nrows=10L, digits=x$digits)
        } else {
          print(DT[Contrast == i, !'Contrast'], digits=x$digits)
        }
      }
      cat('\n')
    }
  }
  invisible(x)
}

# Print details regarding permutation analyses
#-------------------------------------------------------------------------------
print_permutation_summary <- function(x) {
  message('\n', 'Permutation analysis', '\n', rep.int('-', getOption('width') / 4L))
  perm.method <- switch(x$perm.method,
                        freedmanLane='Freedman-Lane',
                        terBraak='ter Braak',
                        smith='Smith')
  cat('Permutation method: ', perm.method, '\n')
  cat('Partition method: ', simpleCap(x$part.method), '\n')
  cat('# of permutations: ', prettyNum(x$N, ','), '\n')

  invisible(x)
}

# Print an abbreviated graph metric in full
#-------------------------------------------------------------------------------
print_full_metric <- function(x) {
  full <- switch(x,
      coreness=, degree=, eccentricity=, diameter=, strength=, vulnerability=simpleCap(x),
      knn='K-nearest neighbor degree', mod='Modularity', mod.wt='Modularity (weighted)',
      E.global='Global efficiency', E.global.wt='Global efficiency (weighted)',
      E.local='Local efficiency', E.local.wt='Local efficiency (weighted)',
      E.nodal='Nodal efficiency', E.nodal.wt='Nodal efficiency (weighted)',
      Cp='Clustering coefficient', Lp='Characteristic path length',
      assort=, assortativity='Degree assortativity', assort.lobe='Lobe assortativity',
      btwn.cent='Betweenness centrality', ev.cent='Eigenvector centrality',
      clo.cent='Closeness centrality', lev.cent='Leverage centrality', pagerank='PageRank',
      subg.cent='Subgraph centrality', communicability='Communicability betweenness centrality',
      asymm='Edge asymmetry', max.comp='Largest connected component', num.hubs='# of hubs',
      num.tri='# of triangles', spatial.dist='Mean Euclidean distance', transitivity='Local transitivity')
  return(full)
}

#' Gateway coefficient, participation coefficient, and within-mod degree z-score
#'
#' \code{gateway_coeff} calculates the gateway coefficient of each vertex,
#' based on community membership.
#'
#' The gateway coefficient \eqn{G_i} of vertex \emph{i} is:
#' \deqn{G_i = 1 - \sum_{S=1}^{N_M} \left ( \frac{\kappa_{iS}}{\kappa_i} \right
#' )^2 (g_{iS})^2}
#' where \eqn{\kappa_{iS}} is the number of edges from vertex \emph{i} to
#' vertices in module \emph{S}, and \eqn{\kappa_i} is the degree of vertex
#' \emph{i}. \eqn{N_M} equals the number of modules. \eqn{g_{ii}} is a weight,
#' defined as:
#' \deqn{g_{iS} = 1 - \bar{\kappa_{iS}} \bar{c_{iS}}}
#' where
#' \deqn{\bar{\kappa_{iS}} = \frac{\kappa_{iS}}{\sum_j \kappa_{jS}}}
#' for all nodes \eqn{j} in node \eqn{i}'s module, and
#' \deqn{\bar{c_{iS}} = c_{iS} / max(c_n)}
#'
#' @inheritParams efficiency
#' @param memb A numeric vector of membership indices of each vertex
#' @param centr Character string; the type of centrality to use in calculating
#'   GC. Default: \code{btwn.cent}
#' @param weighted Logical indicating whether to calculate metrics using edge
#' weights. Default: \code{FALSE}
#' @export
#' @return A vector of the participation coefficients, within-module degree
#'   z-scores, or gateway coefficients for each vertex of the graph.
#'
#' @name Vertex Roles
#' @rdname vertex_roles
#'
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}
#' @references Vargas, E.R. and Wahl, L.M. (2014) The gateway coefficient: a
#'   novel metric for identifying critical connections in modular networks.
#'   \emph{Eur Phys J B}, \bold{87}, 161--170.
#'   \doi{10.1140/epjb/e2014-40800-7}

gateway_coeff <- function(g, memb, centr=c('btwn.cent', 'degree', 'strength'),
                          A=NULL, weighted=FALSE) {
  stopifnot(is_igraph(g))
  if (is.null(A)) {
    if (isTRUE(weighted)) {
      A <- as_adj(g, sparse=FALSE, names=FALSE, attr='weight')
    } else {
      A <- as_adj(g, sparse=FALSE, names=FALSE)
    }
  }
  Ki <- colSums(A)
  centr <- match.arg(centr)
  if (centr == 'btwn.cent') {
    if ('btwn.cent' %in% vertex_attr_names(g)) {
      cent <- V(g)$btwn.cent
    } else {
      cent <- centr_betw(g)$res
    }
  } else {
    cent <- Ki
  }
  N <- max(memb)
  if (N == 1L) return(rep.int(0, length(memb)))
  Cn <- max(vapply(seq_len(N), function(x) sum(cent[memb == x]), numeric(1L)))

  Kis <- t(rowsum(A, memb))
  Kjs <- rowsum(Kis, memb)
  barKis <- Cis <- matrix(0, nrow=length(Ki), ncol=N)

  for (i in which(Ki > 0)) {
    barKis[i, ] <- Kis[i, ] / Kjs[, memb[i]]
    Vi <- which(A[, i] != 0)
    Cis[i, ] <- vapply(seq_len(N), function(x) sum(cent[Vi[memb[Vi] == x]]), numeric(1L))
  }

  barCis <- Cis / Cn
  gis <- 1 - barKis * barCis
  1 - ((1 / Ki^2) * rowSums(Kis^2 * gis^2, na.rm=TRUE))
}

#' Participation coefficient
#'
#' \code{part_coeff} calculates the participation coefficient of each vertex,
#' based on community membership.
#'
#' The participation coefficient \eqn{P_i} of vertex \emph{i} is:
#' \deqn{P_i = 1 - \sum_{s=1}^{N_M} \left ( \frac{\kappa_{is}}{\kappa_i} \right )^2}
#' where \eqn{\kappa_{is}} is the number of edges from vertex \emph{i} to
#' vertices in module \emph{s}, and \eqn{\kappa_s} is the degree of vertex
#' \emph{i}. \eqn{N_M} equals the number of modules.
#'
#' As discussed in Guimera et al., \eqn{P_i = 0} if vertex \emph{i} is connected
#' only to vertices in the same module, and \eqn{P_i = 1} if vertex \emph{i} is
#' equally connected to all other modules.
#'
#' @export
#' @rdname vertex_roles
#'
#' @references Guimera, R. and Amaral, L.A.N. (2005) Cartography of complex
#'   networks: modules and universal roles. \emph{Journal of Statistical
#'   Mechanics: Theory and Experiment}, \bold{02}, P02001.
#'   \doi{10.1088/1742-5468/2005/02/P02001}

part_coeff <- function(g, memb, A=NULL, weighted=FALSE) {
  stopifnot(is_igraph(g))
  if (is.null(A)) {
    if (isTRUE(weighted)) {
      A <- as_adj(g, sparse=FALSE, names=FALSE, attr='weight')
    } else {
      A <- as_adj(g, sparse=FALSE, names=FALSE)
    }
  }
  Ki <- colSums(A)
  N <- max(memb)
  Kis <- t(rowsum(A, memb))
  1 - ((1 / Ki^2) * rowSums(Kis^2))
}

#' Calculate vertex within-module degree z-score
#'
#' \code{within_module_deg_z_score} is a measure of the connectivity from a
#' given vertex to other vertices in its module/community.
#'
#' The within-module degree z-score is:
#' \deqn{z_i = \frac{\kappa_i - \bar{\kappa}_{s_i}}{\sigma_{\kappa_{s_i}}}}
#' where \eqn{\kappa_i} is the number of edges from vertex \emph{i} to vertices
#' in the same module \eqn{s_i}, \eqn{\bar{\kappa}_{s_i}} is the average of
#' \eqn{\kappa} over all vertices in \eqn{s_i}, and \eqn{\sigma_{\kappa_{s_i}}}
#' is the standard deviation.
#'
#' @export
#' @rdname vertex_roles

within_module_deg_z_score <- function(g, memb, A=NULL, weighted=FALSE) {
  stopifnot(is_igraph(g))
  if (is.null(A)) {
    if (isTRUE(weighted)) {
      A <- as_adj(g, sparse=FALSE, names=FALSE, attr='weight')
    } else {
      A <- as_adj(g, sparse=FALSE, names=FALSE)
    }
  }
  N <- max(memb)
  nS <- tabulate(memb)
  z <- Ki <- rep.int(0, dim(A)[1L])
  Ksi <- sigKsi <- rep.int(0, N)

  for (S in seq_len(N)) {
    x <- rowSums(A[memb == S, memb == S, drop=FALSE])
    Ki[memb == S] <- x
    Ksi[S] <- sum(x) / nS[S]
    sigKsi[S] <- sqrt(sum((x - Ksi[S])^2) / (nS[S] - 1L))
  }
  z <- (Ki - Ksi[memb]) / sigKsi[memb]
  z[is.infinite(z)] <- 0
  return(z)
}

#' Calculate the AUC across densities of given attributes
#'
#' Given a list of \code{brainGraphList} objects, this function will calculate
#' the area under the curve (AUC) across all thresholds/densities for each
#' subject or group.
#'
#' If the elements of the input list do not have a \code{threshold} element (or
#' if it is \code{NULL}) then the AUC will be calculated on the interval
#' \eqn{[0, 1]} (inclusive). This has the same effect as specifying
#' \code{norm=TRUE} in the function call.
#'
#' @param g.list A list of \code{brainGraphList} objects
#' @param g.attr A character vector of graph attribute name(s). Default:
#'   \code{NULL}
#' @param v.attr A character vector of vertex attribute name(s). Default:
#'   \code{NULL}
#' @param norm Logical indicating whether to normalize threshold values to be
#'   between 0 and 1 (inclusive). Default: \code{FALSE}
#' @export
#' @return A \code{brainGraphList} object with one graph for each subject
#' @importFrom doParallel registerDoParallel
#' @importFrom foreach getDoParRegistered
#'
#' @examples
#' \dontrun{
#' g.auc <- make_auc_brainGraph(g.fa, g.attr='E.global.wt')
#' }

make_auc_brainGraph <- function(g.list, g.attr=NULL, v.attr=NULL, norm=FALSE) {
  i <- NULL
  # Check if components are 'brainGraphList' objects
  matches <- vapply(g.list, is.brainGraphList, logical(1L))
  if (any(!matches)) stop("Input must be a list of 'brainGraphList' objects.")

  # Get the meta variables first
  attrs <- c('atlas', 'type', 'level', 'modality', 'weighting')
  out <- setNames(vector('list', length(attrs)), attrs)
  for (a in attrs) out[[a]] <- g.list[[1L]][[a]]

  if (!is.null(g.list[[1L]]$threshold)) {
    thresholds <- vapply(g.list, `[[`, numeric(1L), 'threshold')
    if (isTRUE(norm)) thresholds <- vec.transform(thresholds)
  } else {
    thresholds <- seq.int(from=0, to=1, length.out=length(g.list))
  }
  subjects <- names(g.list[[1L]]$graphs)
  grps <- groups(g.list[[1L]])

  if (!getDoParRegistered()) {
    cl <- makeCluster(getOption('bg.ncpus'))
    registerDoParallel(cl)
  }
  g.auc <- foreach(i=seq_along(subjects)) %dopar% {
    g.subj <- lapply(g.list, `[`, i)
    g.tmp <- with(out,
        make_empty_brainGraph(atlas, type, level, modality, weighting,
                              name=subjects[i], Group=grps[i], subnet=g.subj[[1L]]$subnet))
    if (!is.null(g.attr)) {
      for (k in g.attr) {
        y <- sapply(g.subj, graph_attr, k)
        g.tmp <- set_graph_attr(g.tmp, k, abs(auc_diff(thresholds, y)))
      }
    }
    if (!is.null(v.attr)) {
      for (k in v.attr) {
        y <- t(sapply(g.subj, vertex_attr, k))
        g.tmp <- set_vertex_attr(g.tmp, k, value=abs(auc_diff(thresholds, y)))
      }
    }
    g.tmp
  }
  out <- get_metadata(out)
  out$graphs <- g.auc
  names(out$graphs) <- subjects
  class(out) <- c('brainGraphList', 'list')
  return(out)
}

#' Difference in the area-under-the-curve of two vectors
#'
#' This function takes two vectors, calculates the area-under-the-curve (AUC),
#' and calculates the difference between the two (if applicable).
#'
#' There are 4 different behaviors for this function:
#' \enumerate{
#'   \item If \code{x} is a single numeric value, then \code{y} should be a
#'     vector of 2 values and the difference is returned. This generally should
#'     not occur and may be removed in the future.
#'   \item If \code{y} has 1 column (or is a vector), then the AUC of \code{y}
#'     is returned.
#'   \item If \code{y} has exactly 2 columns, then each column should contain
#'     the values of interest for each subject group, and the difference in
#'     AUC's for each group is returned.
#'   \item If \code{y} has multiple columns (e.g., equal to the number of
#'     vertices of a graph), it will calculate the AUC for each column.
#' }
#'
#' @param x Numeric vector of the x-values
#' @param y A numeric vector or matrix
#'
#' @keywords internal
#' @return A numeric value of the difference between two groups, or a numeric
#'   vector of the AUC across vertices

auc_diff <- function(x, y) {
  if (length(x) > 1L) {
    if (NCOL(y) == 1L) {  # A single vector, for MTPC
      return(sum(-diff(x) * (head(y, -1L) + tail(y, -1L))) / 2)
    } else if (ncol(y) > 2L) {
      return(apply(y, 2L, function(z) auc_diff(x, z)))
    } else {
      return(auc_diff_perm(x, y))
    }
  } else {
    return(y[1L] - y[2L])
  }
}

#' @param x Numeric vector
#' @param y Numeric matrix with 2 columns and number of rows equal to the length
#'   of \code{x}
#' @return A single numeric value of the between-group AUC difference
#' @noRd

auc_diff_perm <- function(x, y) {
  -diff(apply(y, 2L, function(z) sum(diff(x) * (head(z, -1L) + tail(z, -1L))) / 2))
}

#' Calculate correlation matrix and threshold
#'
#' \code{corr.matrix} calculates the correlation between all column pairs of a
#' given data frame, and thresholds the resultant correlation matrix based on a
#' given density (e.g., \code{0.1} if you want to keep only the 10\% strongest
#' correlations). If you want to threshold by a specific correlation coefficient
#' (via the \code{thresholds} argument), then the \code{densities} argument is
#' ignored.
#'
#' If you wish to exclude regions from your analysis, you can give the indices
#' of their columns with the \code{exclude.reg} argument.
#'
#' By default, the Pearson correlation coefficients are calculated, but you can
#' return Spearman by changing the \code{type} argument.
#'
#' @param resids An object of class \code{brainGraph_resids} (the output from
#'   \code{\link{get.resid}})
#' @param densities Numeric vector indicating the resultant network
#'   densities; keeps the top \emph{X}\% of correlations
#' @param thresholds Numeric; absolute correlation value to threshold by
#'   (default: \code{NULL})
#' @param what Character string indicating whether to correlate the residuals or
#'   the raw structural MRI values (default: \code{'resids'})
#' @param exclude.reg Character vector of regions to exclude (default:
#'   \code{NULL})
#' @param type Character string indicating which type of correlation coefficient
#'   to calculate (default: \code{'pearson'})
#' @param rand Logical indicating whether the function is being called for
#'   permutation testing; not intended for general use (default: \code{FALSE})
#' @export
#'
#' @return A \code{corr_mats} object containing the following components:
#'   \item{R,P}{Numeric arrays of correlation coefficients and P-values. The
#'     length of the 3rd dimension equals the number of groups}
#'   \item{r.thresh}{A list of 3-d binary arrays indicating correlations that
#'     are above a certain threshold. The length of the list equals the number
#'     of groups, and the length of the 3rd dimension equals the number of
#'     thresholds/densities.}
#'   \item{thresholds}{Numeric matrix of the thresholds supplied. The number of
#'     columns equals the number of groups.}
#'   \item{what}{Residuals or raw values}
#'   \item{exclude.reg}{Excluded regions (if any)}
#'   \item{type}{Pearson or Spearman}
#'   \item{atlas}{The brain atlas used}
#'   \item{densities}{Numeric vector; the densities of the resulting graphs, if
#'     you chose to threshold each group to have equal densities.}
#'
#' @rdname correlation_matrices
#' @family Structural covariance network functions
#' @seealso \code{\link[Hmisc]{rcorr}}
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}
#' @examples
#' \dontrun{
#' myResids <- get.resid(lhrh, covars)
#' corrs <- corr.matrix(myResids, densities=densities)))
#' }

corr.matrix <- function(resids, densities, thresholds=NULL, what=c('resids', 'raw'),
                        exclude.reg=NULL, type=c('pearson', 'spearman'), rand=FALSE) {
  if (!requireNamespace('Hmisc', quietly=TRUE)) stop('You must install "Hmisc" to use this function.')
  stopifnot(inherits(resids, 'brainGraph_resids'))
  gID <- getOption('bg.group')
  N <- nregions(resids)
  regions <- region.names(resids)

  sID <- getOption('bg.subject_id')
  # Different behavior if called for permutation testing
  if (isTRUE(rand)) {
    res.all <- as.matrix(resids$resids.all[, !c(sID, gID), with=FALSE])
    corrs <- Hmisc::rcorr(res.all)
    r <- corrs$r
    emax <- N  * (N - 1) / 2
    thresholds <- get_thresholds(r, densities, emax)
    r.thresh <- array(0, dim=c(N, N, length(thresholds)), dimnames=list(regions, regions, NULL))
    for (i in seq_along(thresholds)) r.thresh[, , i] <- ifelse(r > thresholds[i], 1, 0)
    return(list(list(R=r, r.thresh=r.thresh)))
  }

  what <- match.arg(what)
  type <- match.arg(type)
  if (what == 'resids') {
    res.all <- resids$resids.all[, !sID, with=FALSE]
  } else if (what == 'raw') {
    res.all <- resids$data[, c(sID, gID, regions), with=FALSE]
    setkeyv(res.all, c(gID, sID))
    res.all <- res.all[, !sID, with=FALSE]
  }
  if (!is.null(exclude.reg)) {
    res.all <- res.all[, -exclude.reg, with=FALSE]
    regions <- setdiff(regions, exclude.reg)
    N <- N - length(exclude.reg)
  }

  # Loop through the groups
  grps <- unique(groups(resids))
  kNumGroups <- length(grps)
  r <- p <- array(0, dim=c(N, N, kNumGroups), dimnames=list(regions, regions, grps))
  r.thresh <- setNames(vector('list', kNumGroups), grps)
  if (!is.null(thresholds)) {
    kNumThresh <- length(thresholds)
    thresh.mat <- matrix(rep.int(thresholds, kNumGroups),
                         nrow=kNumThresh, ncol=kNumGroups,
                         dimnames=list(NULL, grps))
  } else {
    kNumThresh <- length(densities)
    thresh.mat <- matrix(0, nrow=kNumThresh, ncol=kNumGroups,
                         dimnames=list(NULL, grps))
  }
  for (g in grps) {
    corrs <- Hmisc::rcorr(as.matrix(res.all[g, !gID, with=FALSE]), type=type)
    r[, , g] <- corrs$r
    p[, , g] <- corrs$P

    # Calculate a threshold so that "density"% of possible connections are present
    if (hasArg('densities')) {
      tmp <- corrs$r
      emax <- N  * (N - 1) / 2
      if (any(densities == 1)) {
        i <- which(densities == 1)
        thresh.mat[i, g] <- min(tmp, na.rm=TRUE)
        thresh.mat[-i, g] <- get_thresholds(tmp, densities[-i], emax)
      } else {
        thresh.mat[, g] <- get_thresholds(tmp, densities, emax)
      }
    }
    r.thresh[[g]] <- array(0, dim=c(N, N, kNumThresh), dimnames=list(regions, regions))
    for (i in seq_along(thresh.mat[, g])) {
      r.thresh[[g]][, , i] <- ifelse(r[, , g] > thresh.mat[i, g], 1, 0)
    }
  }
  out <- list(R=r, P=p, r.thresh=r.thresh, thresholds=thresh.mat, what=what,
              exclude.reg=exclude.reg, type=type, atlas=resids$atlas)
  if (hasArg('densities')) out <- c(out, list(densities=densities))
  class(out) <- c('corr_mats', class(out))
  return(out)
}

#' Subset correlation matrix objects
#'
#' @param x,object A \code{corr_mats} object
#' @param i Integer for subsetting by density/threshold
#' @param g Integer, character, or logical for subsetting by group
#' @export
#'
#' @name Extract.corr_mats
#' @rdname correlation_matrices

`[.corr_mats` <- function(x, i, g=NULL) {
  if (!is.null(g)) {
    grps <- groups(x)
    kNumGroups <- length(grps)
    if (is.logical(g) && length(g) != kNumGroups) {
      stop('Logical indexing vector must be of the same length as the number of groups.')
    }
    if (length(g) > kNumGroups) {
      warning('Indexing vector cannot have length greater than the number of groups.')
      g <- g[seq_len(kNumGroups)]
    }
    g <- switch(class(g),
                integer=, numeric=, logical=grps[g],
                character=which(grps %in% g))
    x$R <- x$R[, , g, drop=FALSE]
    x$P <- x$P[, , g, drop=FALSE]
    x$r.thresh <- x$r.thresh[g]
    x$thresholds <- x$thresholds[, g, drop=FALSE]
  }
  if (missing(i)) i <- seq_len(dim(x$thresholds)[1L])
  for (g in groups(x)) x$r.thresh[[g]] <- x$r.thresh[[g]][, , i, drop=FALSE]
  x$thresholds <- x$thresholds[i, , drop=FALSE]
  if (hasName(x, 'densities')) x$densities <- x$densities[i]
  return(x)
}

#' Plot raw or thresholded correlation matrices
#'
#' The \code{plot} method will plot \dQuote{heat maps} of the correlation
#' matrices.
#'
#' @section Plotting correlation matrices:
#' There are several ways to control the plot appearance. First, you may plot
#' the \dQuote{raw} correlations, or only those of the thresholded (binarized)
#' matrices. Second, you may order the vertices by a given vertex attribute; by
#' default, they will be ordered by \emph{lobe}, but you may also choose to
#' order by, e.g., \emph{network} (for the \code{dosenbach160} atlas) or by
#' \emph{community membership}. In the latter case, you need to pass a
#' \code{brainGraphList} object to the \code{graphs} argument; each graph in the
#' object must have a vertex attribute specified in \code{order.by}. Finally,
#' you can control the legend text with \code{grp.names}.
#'
#' @param mat.type Character string indicating whether to plot raw or thresholded
#'   (binarized) matrices. Default: \code{'raw'}
#' @param thresh.num Integer specifying which threshold to plot (if
#'   \code{mat.type='thresholded'}). Default: \code{1L}
#' @param ordered Logical indicating whether to order the vertices by some
#'   grouping. Default: \code{TRUE}
#' @param order.by Character string indicating how to group vertices. Default:
#'   \code{'lobe'}
#' @param graphs A \code{brainGraphList} object containing graphs with the
#' vertex-level attribute of interest. Default: \code{NULL}
#' @param grp.names Character vector specifying the names of each group of
#'   vertices. Default: \code{NULL}
#' @param legend.title Character string for the legend title. Default is to
#'   leave blank
#' @param ... Unused
#' @export
#' @rdname correlation_matrices
#' @examples
#' \dontrun{
#' corrs <- corr.matrix(myResids, densities)
#' plot(corrs, order.by='comm', graphs=g.list, grp.names='Community')
#' }

plot.corr_mats <- function(x, mat.type=c('thresholded', 'raw'), thresh.num=1L,
                           ordered=TRUE, order.by='lobe', graphs=NULL,
                           grp.names=NULL, legend.title=NULL, ...) {
  Var1 <- Var2 <- Group <- value <- memb1 <- memb2 <- memb <- col.text <- NULL
  grps <- groups(x)
  regions <- region.names(x)
  kNumVertices <- length(regions)
  base_size <- if (kNumVertices > 90L) 7.5 else 9
  if (is.null(legend.title) && isTRUE(ordered)) {
    legend.title <- switch(order.by,
      comp='Connected\nComponent', hemi='Hemisphere', comm=, comm.wt='Community (#)',
      class='Tissue class', simpleCap(order.by))
  }

  type <- match.arg(mat.type)
  # 'ggplot2'
  if (requireNamespace('ggplot2', quietly=TRUE)) {
    legend.pos <- if (type == 'raw' || isTRUE(ordered)) 'right' else 'none'
    mytheme <- ggplot2::theme(legend.position=legend.pos,
      axis.text.x=ggplot2::element_text(size=0.7*base_size, angle=45),
      axis.text.y=ggplot2::element_text(size=0.7*base_size),
      axis.ticks=ggplot2::element_blank(), axis.title.x=ggplot2::element_blank(),
      axis.title.y=ggplot2::element_blank(), plot.title=ggplot2::element_text(hjust=0.5, face='bold'))
  }
  matplots <- setNames(vector('list', length(grps)), grps)
  if (type == 'raw') {
    legend.title <- paste0('Corr. coeff.\n(', simpleCap(x$type), ')')
    mats <- as.data.table(x$R, sorted=FALSE)
    setnames(mats, c('Var1', 'Var2', 'Group', 'value'))
    mats[, Var1 := factor(Var1, levels=regions)]
    mats[, Var2 := factor(Var2, levels=regions)]
    for (g in grps) {
      if (!requireNamespace('ggplot2', quietly=TRUE)) {
        matplots[[g]] <- levelplot(t(x$R[rev(seq_along(regions)), , g]), main=g, xlab=NULL, ylab=NULL,
                                   col.regions=colorRampPalette(c('white', 'red')))
      } else {
        mats.m <- mats[Group == g]
        matplots[[g]] <- ggplot2::ggplot(mats.m, ggplot2::aes(Var1, Var2, fill=value)) +
          ggplot2::geom_tile() +
          ggplot2::scale_fill_gradient2(low='white', high='red') + mytheme +
          ggplot2::labs(title=g, fill=legend.title) + ggplot2::ylim(rev(levels(mats.m$Var2)))
      }
    }
    return(matplots)
  } else {
    mats <- sapply(x$r.thresh, function(m) m[, , thresh.num], simplify='array')
  }

  if (isTRUE(ordered)) {
    if (!hasName(get(x$atlas), order.by)) {
      if (is.brainGraphList(graphs)) {
        g.list <- graphs[]
      } else if (is.brainGraphList(graphs[[thresh.num]])) {
        g.list <- graphs[[thresh.num]][]
      } else {
        stop('Please provide a (list of) "brainGraphList" object(s).')
      }
      order.mat <- vapply(g.list, vertex_attr, numeric(kNumVertices), order.by)
      if (is.null(grp.names)) grp.names <- ''
      if (length(grp.names) == 1L) grp.names <- paste(grp.names, seq_len(max(order.mat)))
    } else {
      tmp <- get(x$atlas)[, get(order.by)]
      order.mat <- matrix(rep.int(as.integer(tmp), 2L), ncol=2L)
      grp.names <- levels(tmp)
    }

    dimnames(order.mat) <- dimnames(new.order) <- list(regions, grps)
    kNumCols <- apply(order.mat, 2L, max)
    cols.new <- lapply(kNumCols, function(y) c(group.cols[seq_len(y)], 'gray50', 'white'))
    if (order.by == 'comp') cols.new <- lapply(cols.new, function(y) setdiff(y, 'gray50'))
    grp.names <- c(grp.names, 'Inter', '')
    new.order <- apply(order.mat, 2L, order)

  } else {
    new.order <- order.mat <- matrix(rep.int(seq_len(kNumVertices), 2L), ncol=2L)
    dimnames(order.mat) <- dimnames(new.order) <- list(regions, grps)
    cols.new <- lapply(grps, function(y) c('white', 'gray50'))
  }
  names(cols.new) <- grps

  mats.ord <- lapply(grps, function(y) array(mats[new.order[, y], new.order[, y], y],
                                             dim=c(kNumVertices, kNumVertices, 1L)))
  names(mats.ord) <- grps
  for (g in grps) {
    dimnames(mats.ord[[g]]) <- list(regions[new.order[, g]], regions[new.order[, g]], g)
    mats.m <- as.data.table(mats.ord[[g]], sorted=FALSE)
    setnames(mats.m, c('Var1', 'Var2', 'Var3', 'value'))
    mats.m[, Var1 := factor(Var1, levels=regions[new.order[, g]])]
    mats.m[, Var2 := factor(Var2, levels=regions[new.order[, g]])]
    if (isTRUE(ordered)) {
      mats.m[, memb := '']
      mats.m[, memb1 := grp.names[order.mat[as.character(Var1), g]]]
      mats.m[, memb2 := grp.names[order.mat[as.character(Var2), g]]]
      mats.m[value == 1, memb := ifelse(memb1 == memb2, memb1, 'Inter')]
      mats.m[, memb := factor(memb, levels=grp.names)]
      mats.m[, memb1 := factor(memb1, levels=grp.names)]
      mats.m[, col.text := cols.new[[g]][as.integer(memb1)]]
    } else {
      mats.m[, memb := factor(value)]
      mats.m[, col.text := 'black']
    }

    # 'base' plotting
    if (!requireNamespace('ggplot2', quietly=TRUE)) {
      mats.m[, memb := as.numeric(memb)]
      mat <- as.matrix(dcast(mats.m, Var1 ~ Var2, value.var='memb'), rownames='Var1')
      matplots[[g]] <- levelplot(t(mat[rev(seq_along(regions)), ]), col.regions=cols.new[[g]],
                                 main=g, xlab=NULL, ylab=NULL,
                                 colorkey=list(at=seq_len(kNumCols[g] + 2L),
                                               labels=list(at=seq_len(kNumCols[g] + 2L), labels=grp.names)))

    # 'ggplot2' plotting
    } else {
      mytheme$axis.text.x <- ggplot2::element_text(size=0.7*base_size, angle=45,
                                          color=mats.m[seq_len(kNumVertices), col.text])
      mytheme$axis.text.y <- ggplot2::element_text(size=0.7*base_size,
                                          color=mats.m[rev(seq_len(kNumVertices)), col.text])
      matplots[[g]] <- ggplot2::ggplot(mats.m, ggplot2::aes(Var1, Var2, fill=memb)) +
        ggplot2::geom_tile() +
        ggplot2::scale_fill_manual(values=cols.new[[g]]) + mytheme +
        ggplot2::labs(title=g, fill=legend.title) + ggplot2::ylim(rev(levels(mats.m$Var2)))
    }
  }
  return(matplots)
}

################################################################################
# MAIN CREATION FUNCTIONS
################################################################################

#' Create a list of brainGraph graphs
#'
#' \code{make_brainGraphList} creates a \code{brainGraphList} object, a list
#' containing a set of graphs for all subjects (or group-average graphs) in a
#' study at a specific threshold (or density), in addition to some graph-level
#' attributes common to those graphs.
#'
#' In addition to creating the initial \code{igraph} graphs from the
#' connectivity matrices, then attributes will be calculated and assigned for
#' each graph via \code{\link{set_brainGraph_attr}} if \code{set.attrs=TRUE}.
#' Other arguments can be passed to that function. You may display a progress
#' bar by setting \code{.progress=TRUE}.
#'
#' This object can be considered comparable to a 4-D \emph{NIfTI} file,
#' particularly that returned by FSL's \emph{TBSS} \dQuote{prestats} step since
#' that file contains the FA volumes for all study subjects.
#'
#' @note If the input is a \code{corr_mats} object, and the extent of the 3-D
#' array is greater than 1, then only the first will be converted to a graph.
#'
#' @param x 3-D numeric array of all subjects' connectivity matrices (for a
#'   single threshold) or a \code{corr_mats} object
#' @param gnames Character vector of graph names (e.g., study IDs if
#'   \code{level='subject'}). Default: \code{NULL}
#' @param ... Other arguments passed to \code{\link{set_brainGraph_attr}}
#' @inheritParams Creating_Graphs
#' @export
#'
#' @return \code{make_brainGraphList} returns an object of class
#'   \code{brainGraphList} with elements:
#'   \item{threshold}{The specified threshold/density}
#'   \item{version}{The versions of \code{R}, \code{igraph}, and
#'     \code{brainGraph} used when creating the graphs}
#'   \item{atlas}{The atlas common to all the graphs}
#'   \item{modality}{The imaging modality (if supplied)}
#'   \item{weighting}{A string indicating what edge weights represent (if
#'     applicable)}
#'   \item{graphs}{A \emph{named list} of \code{brainGraph} graphs; the names
#'     correspond to the individual graphs' Study IDs}
#'
#' @name brainGraphList
#' @rdname brainGraphList
#' @family Graph creation functions
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}
#' @examples
#' \dontrun{
#' # Create a list, one for each threshold
#' g <- vector('list', length(thresholds))
#' for (i in seq_along(thresholds)) {
#'   g[[i]] <- make_brainGraphList(A.norm.sub[[i]], thresholds[i], atlas,
#'       covars.dti$Study.ID, covars.dti$Group, modality='dti', weighting='fa')
#' }
#' }

make_brainGraphList <- function(x, atlas, type=c('observed', 'random'),
                                level=c('subject', 'group', 'contrast'),
                                set.attrs=TRUE, modality=NULL, weighting=NULL,
                                threshold=NULL, gnames=NULL, ...) {
  UseMethod('make_brainGraphList')
}

#' @param grpNames Character (or factor) vector of group names. If \code{level ==
#'   'group'}, then you do not need to include this argument (the group names
#'   will be the same as \code{gnames}). Default: \code{NULL})
#' @param .progress Logical indicating whether to print a progress bar. Default:
#'   \code{getOption('bg.progress')}
#' @export
#' @importFrom doParallel registerDoParallel
#' @importFrom foreach getDoParRegistered getDoParWorkers
#' @rdname brainGraphList

make_brainGraphList.array <- function(x, atlas, type=c('observed', 'random'),
                                      level=c('subject', 'group', 'contrast'),
                                      set.attrs=TRUE, modality=NULL,
                                      weighting=NULL, threshold=NULL,
                                      gnames=NULL, grpNames=NULL, subnet=NULL,
                                      mode='undirected', weighted=NULL, diag=FALSE,
                                      .progress=getOption('bg.progress'), ...) {
  i <- NULL
  level <- match.arg(level)
  kNumGraphs <- dim(x)[3L]
  if (is.null(gnames)) {
    gnames <- pad_zeros(kNumGraphs)
  } else {
    stopifnot(length(gnames) == kNumGraphs)
    gnames <- check_sID(gnames)
  }
  if (is.null(grpNames)) {
    grpNames <- if (level == 'group') gnames else rep.int('1', kNumGraphs)
  } else {
    stopifnot(length(grpNames) == kNumGraphs)
    grpNames <- switch(class(grpNames),
                       factor=as.character(grpNames),
                       integer=, numeric=pad_zeros(grpNames),
                       grpNames)
  }

  type <- match.arg(type)
  if (missing(atlas)) atlas <- guess_atlas(x)
  attrs <- c('atlas', 'type', 'level', 'modality', 'weighting', 'threshold')
  out <- setNames(vector('list', length(attrs)), attrs)
  for (a in attrs) {
    if (!is.null(get(a))) out[[a]] <- get(a)
  }
  out <- get_metadata(out)
  if (!is.null(threshold)) threshold <- rep.int(threshold, kNumGraphs)

  # Show a progress bar so you aren't left in the dark
  #---------------------------------------------------------
  if (!getDoParRegistered()) {
    cl <- makeCluster(getOption('bg.ncpus'))
    registerDoParallel(cl)
  }
  loopfun <- make_brainGraph
  if (isTRUE(.progress)) {
    ncpu <- getDoParWorkers()
    print(paste('Start time:', format(as.POSIXct(Sys.time()), '%Y-%m-%d %H:%M:%OS0')))
    progbar <- txtProgressBar(min=0, max=kNumGraphs, style=3)
    loopfun <- function(...) {
      curVal <- get('counter', envir=env) + ncpu
      assign('counter', curVal, envir=env)
      setTxtProgressBar(get('progbar', envir=env), curVal)
      flush.console()
      make_brainGraph(...)
    }
  }

  env <- environment()
  counter <- 0L
  g <- foreach(i=seq_len(kNumGraphs)) %dopar% {
    res <- loopfun(x[, , i], atlas, type, level, set.attrs, modality, weighting, threshold[i],
                   name=gnames[i], Group=grpNames[i], subnet=subnet, mode=mode,
                   diag=diag, weighted=weighted, use.parallel=FALSE, ...)
  }
  if (isTRUE(.progress)) close(progbar)

  out$graphs <- g
  names(out$graphs) <- gnames
  class(out) <- c('brainGraphList', 'list')
  return(out)
}

#' @export
#' @rdname brainGraphList

make_brainGraphList.corr_mats <- function(x, atlas=x$atlas, type='observed',
                                          level='group',
                                          set.attrs=TRUE, modality=NULL,
                                          weighting=NULL, threshold=x$densities,
                                          gnames=names(x$r.thresh), grpNames=gnames,
                                          mode='undirected', weighted=NULL,
                                          diag=FALSE, .progress=getOption('bg.progress'), ...) {
  dims <- vapply(x$r.thresh, dim, numeric(3L))[3L, ]
  if (any(dims > 1L)) x <- x[1L]
  A <- abind::abind(x$r.thresh)
  if (isTRUE(weighted)) A <- x$R * A
  out <- make_brainGraphList(A, atlas=atlas, type=type, level=level,
                             set.attrs=set.attrs, modality=modality, weighting=weighting,
                             threshold=threshold, mode=mode, weighted=weighted, diag=diag,
                             gnames=gnames, grpNames=grpNames, .progress=.progress, ...)
  return(out)
}

#' Create a graph list with GLM-specific attributes
#'
#' These methods create a \code{brainGraphList} with attributes specific to the
#' results of \code{\link{brainGraph_GLM}}, \code{\link{mtpc}}, or
#' \code{\link{NBS}}. The \code{graphs} element of the returned object will
#' contain one graph for each contrast.
#'
#' @note Only valid for \emph{vertex}-level and \emph{NBS} analyses.
#'
#' @param x A \code{bg_GLM}, \code{mtpc}, or \code{NBS} object
#' @param atlas Character string specifying the brain atlas to use
#' @inheritParams brainGraphList
#' @export
#'
#' @return A \code{brainGraphList} object, with a graph object for each contrast
#'   with additional attributes:
#'   \item{Graph}{\emph{name} (contrast name), \emph{outcome} (the outcome
#'     variable), \emph{alpha} (the significance level); for MTPC:
#'     \emph{tau.mtpc}, \emph{S.mtpc}, \emph{S.crit}, \emph{A.crit}}
#'   \item{Vertex}{\emph{size2} (t-statistic); \emph{size} (the t-stat
#'     transformed for visualization purposes); \emph{p} (equal to \eqn{1-p});
#'     \emph{p.fdr} (equal to \eqn{1-p_{FDR}}, the FDR-adjusted p-value);
#'     \emph{effect.size} (the contrast of parameter estimates for t-contrasts;
#'     the extra sum of squares for F-contrasts); \emph{se} (the
#'     standard error of \emph{gamma}); \emph{A.mtpc}, \emph{sig} (binary
#'     indicating whether \code{A.mtpc > A.crit}) (for MTPC)}
#' @name Creating_Graphs_GLM
#' @rdname glm_brainGraphList
#' @family Graph creation functions
#' @seealso \code{\link{brainGraph_GLM}, \link{mtpc}, \link{NBS}}

make_brainGraphList.bg_GLM <- function(x, atlas=x$atlas, type='observed',
                                       level='contrast', set.attrs=FALSE,
                                       modality=NULL, weighting=NULL,
                                       threshold=NULL, gnames=x$con.name, ...) {
  Contrast <- contrast <- p <- p.fdr <- p.perm <- se <- stat <- region <- ESS <- NULL
  if (x$level == 'graph') stop('Not valid for graph-level results.')

  attrs <- c('atlas', 'type', 'level', 'modality', 'weighting', 'threshold')
  out <- setNames(vector('list', length(attrs)), attrs)
  for (a in attrs) {
    if (!is.null(get(a))) out[[a]] <- get(a)
  }
  out <- get_metadata(out)

  g.diffs <- vector('list', length(gnames))
  for (i in seq_along(g.diffs)) {
    g.diffs[[i]] <- make_empty_brainGraph(atlas, type, level, modality, weighting,
                                          threshold, name=gnames[i], ...)
    for (a in c('con.type', 'outcome', 'alt', 'alpha')) {
      g.diffs[[i]] <- set_graph_attr(g.diffs[[i]], a, x[[a]])
    }

    DT <- x$DT[Contrast == gnames[i]]
    V(g.diffs[[i]])$p <- 1 - DT$p
    V(g.diffs[[i]])$p.fdr <- 1 - DT$p.fdr
    if (x$con.type == 't') {
      V(g.diffs[[i]])$effect.size <- DT$gamma
    } else {
      V(g.diffs[[i]])$effect.size <- DT$ESS
    }
    V(g.diffs[[i]])$se <- DT$se
    V(g.diffs[[i]])$size2 <- DT$stat
    V(g.diffs[[i]])$size <- vec.transform(V(g.diffs[[i]])$size2, 0, 20)
    if (isTRUE(x$permute)) V(g.diffs[[i]])$p.perm <- 1 - DT$p.perm
    class(g.diffs[[i]]) <- c('brainGraph_GLM', class(g.diffs[[i]]))
  }
  out$graphs <- g.diffs
  names(out$graphs) <- gnames
  class(out) <- c('brainGraphList', 'list')
  return(out)
}

#' Create graph list for MTPC results
#'
#' @export
#' @rdname glm_brainGraphList

make_brainGraphList.mtpc <- function(x, atlas=x$atlas, type='observed', level='contrast',
                                     set.attrs=FALSE, modality=NULL, weighting=NULL,
                                     threshold=NULL, gnames=x$con.name, ...) {
  contrast <- region <- A.mtpc <- A.crit <- S.crit <- S.mtpc <- tau.mtpc <- NULL
  if (x$level == 'graph') stop('Not valid for graph-level results.')

  attrs <- c('atlas', 'type', 'level', 'modality', 'weighting', 'threshold')
  out <- setNames(vector('list', length(attrs)), attrs)
  for (a in attrs) {
    if (!is.null(get(a))) out[[a]] <- get(a)
  }
  out <- get_metadata(out)

  g.diffs <- vector('list', length(gnames))
  for (i in seq_along(g.diffs)) {
    g.diffs[[i]] <- make_empty_brainGraph(atlas, type, level, modality, weighting,
                                          threshold, name=gnames[i], ...)
    for (a in c('con.type', 'outcome', 'alt', 'alpha')) {
      g.diffs[[i]] <- set_graph_attr(g.diffs[[i]], a, x[[a]])
    }
    for (a in c('tau.mtpc', 'S.mtpc', 'S.crit', 'A.crit')) {
      g.diffs[[i]] <- set_graph_attr(g.diffs[[i]], a, x$stats[contrast == i, get(a)])
    }
    V(g.diffs[[i]])$A.mtpc <- x$DT[contrast == i, unique(A.mtpc), by=region]$V1
    V(g.diffs[[i]])$sig <- 0L
    sig.regions <- as.integer(x$DT[contrast == i & A.mtpc > A.crit, unique(region)])
    V(g.diffs[[i]])[sig.regions]$sig <- 1L
    class(g.diffs[[i]]) <- c('brainGraph_mtpc', class(g.diffs[[i]]))
  }
  out$graphs <- g.diffs
  names(out$graphs) <- gnames
  class(out) <- c('brainGraphList', 'list')
  return(out)
}

#' Create a graph list for NBS results
#'
#' @export
#' @return \code{make_brainGraphList.NBS} returns graphs with additional
#'   attributes:
#'   \item{Vertex}{\emph{comp} (integer vector indicating connected component
#'     membership), \emph{p.nbs} (P-value for each component)}
#'   \item{Edge}{\emph{stat} (the test statistic for each connection), \emph{p}
#'     (the P-value)}
#' @rdname glm_brainGraphList

make_brainGraphList.NBS <- function(x, atlas, type='observed', level='contrast',
                                    set.attrs=TRUE, modality=NULL, weighting=NULL,
                                    threshold=NULL, gnames=x$con.name,
                                    mode='undirected', weighted=TRUE, diag=FALSE,
                                    ...) {
  contrast <- p.perm <- csize <- NULL

  bgList <- make_brainGraphList(x$T.mat, atlas, type, level, set.attrs=set.attrs,
      modality=modality, weighting=weighting, threshold=threshold, mode=mode,
      weighted=weighted, diag=diag, gnames=gnames, ...)
  pList <- make_brainGraphList(x$p.mat, atlas, level=level, set.attrs=FALSE, weighted=TRUE, ...)

  for (i in seq_along(bgList[])) {
    class(bgList$graphs[[i]]) <- c('brainGraph_NBS', class(bgList[i]))
    bgList$graphs[[i]]$con.type <- x$con.type
    bgList$graphs[[i]]$alt <- x$alt

    if (ecount(bgList[i]) == 0) next
    E(bgList$graphs[[i]])$stat <- E(bgList[i])$weight
    E(bgList$graphs[[i]])$p <- 1 - E(pList[i])$weight
    if (any(E(bgList[i])$weight < 0)) bgList$graphs[[i]] <- delete_edge_attr(bgList[i], 'weight')

    clusts <- components(bgList[i])
    comps <- sort(unique(clusts$csize), decreasing=TRUE)
    memb <- clusts$membership
    x.tab <- table(memb)
    x.tab.st <- sort(x.tab, decreasing=TRUE)
    V(bgList$graphs[[i]])$comp <- match(memb, order(x.tab, decreasing=TRUE))
    V(bgList$graphs[[i]])$p.nbs <- 0
    xdt <- copy(x$components$observed)
    for (j in seq_along(comps)) {
      inds <- which(xdt[contrast == i, csize[j]] == x.tab.st)
      V(bgList$graphs[[i]])[V(bgList[i])$comp %in% inds]$p.nbs <- 1 - xdt[contrast == i, p.perm[j]]
    }
  }
  return(bgList)
}

################################################################################
# OTHER METHODS
################################################################################

#' Index subjects and/or groups in a brainGraphList
#'
#' The \code{[} method will let you subset/slice the graphs for individual
#' subjects and/or \emph{groups}.
#'
#' @section Subsetting/extracting:
#' The first index is for subsetting the individual graphs. The second index is
#' for subsetting by group membership and requires that the graphs have a
#' \code{Group} graph attribute. When both are included, the first index cannot
#' have length or numeric value greater than the number of \emph{remaining}
#' subjects \emph{after} subsetting by group.
#'
#' If the indexing vector(s) is (are) \code{character}, the vector(s) must
#' contain one (or more) of the subject or group names. If \code{logical}, its
#' length must equal the number of subjects or groups.
#'
#' @param i Integer, character, or logical vector for subsetting by subject, or
#' by group (if \code{x$level='group'})
#' @param g Integer, character, or logical vector for subsetting by group (if
#'   \code{x$level='subject'})
#' @param drop If \code{TRUE} (the default), then return only the list of
#'   graphs; otherwise, subset the graphs and return the entire object
#' @export
#'
#' @return \code{[} -- A \code{brainGraphList} object (if \code{drop=FALSE}) or
#'   a list of graphs
#' @name Extract.brainGraphList
#' @rdname brainGraphList
#' @examples
#' \dontrun{
#' # Subset the first 10 subjects, irrespective of group
#' my.bgl[1:10]
#'
#' # Return object for only 'Control' subjects
#' my.bgl[, 'Control']
#'
#' # Return object with graphs from groups 1 and 3
#' my.bgl[g=c(1, 3), drop=FALSE]
#'
#' # Subset the first 10 subjects of group 2
#' my.bgl[1:10, 2]
#'}

`[.brainGraphList` <- function(x, i, g=NULL, drop=TRUE) {
  stop_msg <- 'Logical indexing vector must be of the same length as the number of groups.'
  warn_msg <- 'Indexing vector cannot have length greater than the number of groups.'
  # Subset groups; doesn't make sense for contrast lists
  if (!is.null(g) && x$level != 'contrast') {
    group.vec <- groups(x)
    grpNames <- unique(group.vec)
    kNumGroups <- length(grpNames)
    if (is.logical(g) && length(g) != kNumGroups) stop(stop_msg)
    if (length(g) > kNumGroups) {
      warning(warn_msg)
      g <- g[seq_len(kNumGroups)]
    }
    g <- switch(class(g),
                numeric=, integer=, logical=which(group.vec %in% grpNames[g]),
                factor=which(group.vec %in% as.character(g)),
                character=which(group.vec %in% g))
    x$graphs <- x$graphs[g]
  }

  # Subset subjects
  kNumSubs <- length(x$graphs)
  if (missing(i)) {
    if (isTRUE(drop)) return(x$graphs)
    i <- seq_len(kNumSubs)
  }
  if (is.logical(i) && length(i) != kNumSubs) stop(sub('group', 'subject', stop_msg))
  if (length(i) > kNumSubs) {
    warning(sub('group', 'subject', warn_msg))
    i <- i[seq_len(kNumSubs)]
  }

  if (is.character(i)) i <- which(names(x$graphs) %in% i)
  x$graphs <- if (length(i) == 1L && isTRUE(drop)) x$graphs[[i]] else x$graphs[i]
  out <- if (isTRUE(drop)) x$graphs else x
  return(out)
}

#' @export
#' @rdname brainGraphList
print.brainGraphList <- function(x, ...) {
  kNumGraphs <- length(x$graphs)
  gnames <- names(x$graphs)
  print_title_summary('A "brainGraphList" object of *', x$type,
                      '* graphs containing ', kNumGraphs, ' ', x$level, 's.')

  df <- print_bg_summary(x)
  df <- df[-c(6L, 10L, 11L, 13L, 14L), ]
  print(df, right=FALSE, row.names=FALSE)
  cat('\n')

  # Print subject/group names
  msg <- switch(x$level, subject='IDs:', 'names:')
  message(paste(simpleCap(x$level), msg))
  if (kNumGraphs < 10L) {
    print(gnames)
    cat('\n')
  } else {
    attrs.df <- print_text_vector(gnames, 6L)
    print(attrs.df, row.names=FALSE)
    cat('\n')
  }
  if (x$level == 'subject' && 'Group' %in% graph_attr_names(x$graphs[[1L]])) {
    message('Group membership:')
    print(table(groups(x)))
    cat('\n')
  }

  if (x$type == 'random') {
    message('# of random graphs:')
    print(lengths(x$graphs))
  }
  invisible(x)
}

#' @export
#' @rdname brainGraphList
is.brainGraphList <- function(x) inherits(x, 'brainGraphList')

#' @param object A \code{brainGraphList} object
#' @export
#' @rdname brainGraphList
nobs.brainGraphList <- function(object, ...) length(object$graphs)

#' Coerce list of graphs to a brainGraphList object
#'
#' \code{as_brainGraphList} coerces a list of graphs to a \code{brainGraphList}
#' object. It is assumed that certain metadata attributes -- threshold, package
#' version, atlas, imaging modality, edge weighting, and whether they are
#' random graphs -- are identical for all graphs in the list.
#'
#' To convert an object with 3 \dQuote{levels} (i.e., subject-level lists from
#' an older \code{brainGraph} version), see the code in the Examples below.
#'
#' @param g.list List of graph objects
#' @export
#'
#' @rdname brainGraphList
#' @examples
#' \dontrun{
#' ## Convert old version single-subject graph lists
#' ## g[[1]] is group 1, g[[1]][[1]] is threshold 1, g[[1]][[1]][[1]] is subj. 1
#' kNumThresholds <- length(g[[1]])
#' g.l <- vector('list', kNumThresholds)
#' for (i in seq_len(kNumThresholds)) {
#'   g.l[[i]] <- as_brainGraphList(do.call(Map, c(c, g))[[i]])
#' }
#' }

as_brainGraphList <- function(g.list, type=c('observed', 'random'),
                              level=c('subject', 'group', 'contrast')) {
  if (!inherits(g.list, 'list')) g.list <- list(g.list)

  type <- match.arg(type)
  level <- match.arg(level)
  if (type == 'random') {
    stopifnot(all(vapply(g.list, function(x) is_igraph(x[[1L]]), logical(1L))))
    g1 <- g.list[[1L]][[1L]]
    ids <- vapply(g.list, function(x) graph_attr(x[[1L]], 'name'), character(1L))
  } else {
    stopifnot(all(vapply(g.list, is.brainGraph, logical(1L))))
    g1 <- g.list[[1L]]
    ids <- vapply(g.list, graph_attr, character(1L), 'name')
  }

  attrnames <- graph_attr_names(g1)
  attrs <- c('atlas', 'modality', 'weighting', 'threshold', 'version', 'sys', 'date')
  out <- setNames(vector('list', length=9L), c('type', 'level', attrs))
  out$type <- type
  out$level <- level
  for (x in attrs) {
    if (x %in% attrnames) out[[x]] <- graph_attr(g1, x)
  }
  if (length(out$version) < 3L) out$version <- list(r='', bg=out$version, ig='')
  for (x in c('sys', 'date')) if (is.null(out[[x]])) out[[x]] <- ''
  out$graphs <- g.list
  names(out$graphs) <- ids
  class(out) <- c('brainGraphList', 'list')
  return(out)
}

#' Plot a brainGraphList and write to PDF
#'
#' The \code{plot} method will write a PDF file containing plots for all graphs
#' in the given object.
#'
#' You can choose to highlight edge differences between subsequent list
#' elements; in this case, new/different edges are colored pink. This is useful
#' mostly for a list of group-level graphs.
#'
#' @param x A \code{brainGraphList} object
#' @param filename.base Character string specifying the base of the filename
#' @param diffs Logical, indicating whether edge differences should be
#'   highlighted. Default: \code{FALSE}
#' @param ... Other parameters (passed to \code{\link{plot.brainGraph}})
#' @inheritParams plot.brainGraph
#' @export
#' @importFrom methods hasArg
#' @importFrom grDevices dev.off pdf
#'
#' @family Plotting functions
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}

plot.brainGraphList <- function(x, plane, hemi, filename.base, diffs=FALSE, ...) {
  pdf(file=sprintf('%s.pdf', filename.base), onefile=TRUE)

  kNumGraphs <- length(x$graphs)
  plot(x[1L], plane=plane, hemi=hemi, main=x[1L]$name, ...)
  for (i in seq.int(from=2L, to=kNumGraphs)) {
    plot(x[i], plane=plane, hemi=hemi, main=x[i]$name, ...)
    if (isTRUE(diffs)) {
      g.diff <- graph.difference(x$graphs[[i]], x$graphs[[i - 1L]])
      class(g.diff) <- c('brainGraph', class(g.diff))
      ecols <- rep.int('deeppink', ecount(g.diff))
      if (hasArg('edge.color')) ecols <- list(...)$edge.color
      plot(g.diff, add=TRUE, vertex.label=NA, vertex.shape='none',
           edge.width=5, edge.color=ecols, mni=FALSE, subtitle=NULL)
    }
  }
  dev.off()
}

#' Network-based statistic for brain MRI data
#'
#' Calculates the \emph{network-based statistic (NBS)}, which allows for
#' family-wise error (FWE) control over network data, introduced for brain MRI
#' data by Zalesky et al. Requires a three-dimensional array of all subjects'
#' connectivity matrices and a \code{data.table} of covariates, in addition to a
#' contrast matrix or list. A null distribution of the largest connected
#' component size is created by fitting a GLM to permuted data. For details, see
#' \code{\link{GLM}}.
#'
#' When printing a \code{summary}, you can include arguments to
#' \code{\link[stats]{printCoefmat}}.
#'
#' @note It is assumed that the order of the subjects in \code{covars} matches
#' that of the input array \code{A}. You will need to ensure that this is the
#' case. Prior to \code{v3.0.0}, the \code{covars} table was sorted by
#' \code{Study.ID} before creating the design matrix.
#'
#' @param A Three-dimensional array of all subjects' connectivity matrices
#' @param p.init Numeric; the initial p-value threshold (default: \code{0.001})
#' @param symm.by Character string; how to create symmetric off-diagonal
#'   elements. Default: \code{max}
#' @inheritParams GLM
#' @export
#' @importFrom permute shuffleSet
#'
#' @return An object of class \code{NBS} with some input arguments in addition
#'   to:
#'   \item{X}{The design matrix}
#'   \item{removed.subs}{Character vector of subject ID's removed due to
#'     incomplete data (if any)}
#'   \item{T.mat}{3-d array of (symmetric) numeric matrices containing the
#'     statistics for each edge}
#'   \item{p.mat}{3-d array of (symmetric) numeric matrices containing the
#'     P-values}
#'   \item{components}{List containing data tables of the observed and permuted
#'     connected component sizes and P-values}
#'   \item{rank,df.residual,qr,cov.unscaled}{The rank, residual degrees of
#'     freedom, QR decomposition, and unscaled covariance matrix of the design
#'     matrix}
#'
#' @family Group analysis functions
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}
#' @references Zalesky, A. and Fornito,  A. and Bullmore, E.T. (2010)
#'   Network-based statistic: identifying differences in brain networks.
#'   \emph{NeuroImage}, \bold{53(4)}, 1197--1207.
#'   \doi{10.1016/j.neuroimage.2010.06.041}
#' @examples
#' \dontrun{
#' max.comp.nbs <- NBS(A.norm.sub[[1]], covars.dti, N=5e3)
#' }

NBS <- function(A, covars, contrasts, con.type=c('t', 'f'), X=NULL, con.name=NULL,
                p.init=0.001, perm.method=c('freedmanLane', 'terBraak', 'smith',
                                            'draperStoneman', 'manly', 'stillWhite'),
                part.method=c('beckmann', 'guttman', 'ridgway'), N=1e3,
                perms=NULL, symm.by=c('max', 'min', 'avg'),
                alternative=c('two.sided', 'less', 'greater'), long=FALSE, ...) {
  Contrast <- contrast <- p.perm <- csize <- NULL
  dimA <- dim(A)
  stopifnot(dimA[3L] == dim(covars)[1L])

  # Initial GLM setup
  ctype <- match.arg(con.type)
  alt <- if (ctype == 'f') 'two.sided' else match.arg(alternative)

  # Remove subjects w/ incomplete data, and create design matrix
  sID <- getOption('bg.subject_id')
  covars <- droplevels(copy(covars))
  if (!hasName(covars, sID)) covars[, eval(sID) := seq_len(dim(covars)[1L])]
  covars[, eval(sID) := check_sID(get(sID))]
  incomp <- covars[!complete.cases(covars), which=TRUE]
  names(incomp) <- covars[incomp, get(sID)]
  if (length(incomp) > 0L) {
    A <- A[, , -incomp, drop=FALSE]
    dimA <- dim(A)
    covars <- covars[-incomp]
  }
  if (is.null(X)) X <- brainGraph_GLM_design(covars, ...)
  tmp <- contrast_names(contrasts, ctype, con.name, X)
  contrasts <- tmp$contrasts; nC <- tmp$nC

  dimX <- dim(X)
  n <- dimX[1L]; p <- dimX[2L]; dfR <- n - p; nV <- dimA[1L]

  # Symmetrize the array and use only the upper triangle's data
  A <- symmetrize(A, symm.by)
  inds.high <- which(upper.tri(A[, , 1L]), arr.ind=TRUE)
  ny <- dim(inds.high)[1L]
  yMat <- matrix(0, n, ny)
  for (r in seq_len(ny)) yMat[, r] <- A[inds.high[r, 1L], inds.high[r, 2L], ]

  # Remove region pairs where the value is 0 for all subjects
  sums <- .colSums(yMat, n, ny)
  if (any(sums == 0)) {
    v <- which(sums == 0)
    inds.high <- inds.high[-v, , drop=FALSE]
    yMat <- yMat[, -v, drop=FALSE]
    ny <- dim(yMat)[2L]
  }

  # Do the model fitting/estimation and filter based on "p.init"
  fits <- fastLmBG(X, yMat)
  obs_stats <- if (ctype == 't') fastLmBG_t(fits, contrasts, alt) else fastLmBG_f(fits, contrasts)
  sig_inds <- which(obs_stats[, 'p', , drop=FALSE] < p.init, arr.ind=TRUE)[, -2L, drop=FALSE]

  # Create stat and p-val matrices, and get observed components
  comps.obs <- vector('list', nC)
  T.max <- p.mat <- array(0, dim=c(nV, nV, nC))
  nrows <- rep.int(0L, nC)
  if (length(sig_inds) > 0L) {
    nrows <- vapply(seq_len(nC), function(x) sum(sig_inds[, 2L] == x), integer(1L))
  }
  skip <- which(nrows == 0L)
  noSkip <- setdiff(seq_len(nC), skip)
  if (length(skip) > 0L) for (k in skip) comps.obs[[k]] <- data.table(csize=0)
  for (j in noSkip) {
    rowInds <- sig_inds[sig_inds[, 2L] == j, 1L]
    T.max[, , j][inds.high[rowInds, , drop=FALSE]] <- obs_stats[rowInds, 'stat', j]
    p.mat[, , j][inds.high[rowInds, , drop=FALSE]] <- obs_stats[rowInds, 'p', j]

    clusts <- components(graph_from_adjacency_matrix(T.max[, , j], diag=FALSE,
                                                     mode='undirected', weighted=TRUE))
    comps.obs[[j]] <- data.table(csize=with(clusts, sort(csize[csize > 1L], decreasing=TRUE)))
  }
  comps.obs <- rbindlist(comps.obs, idcol='contrast')

  out <- list(covars=covars, X=X, con.type=ctype, contrasts=contrasts, con.name=tmp$con.name,
              alt=alt, p.init=p.init, removed.subs=incomp, T.mat=T.max, p.mat=p.mat, N=N)
  out <- c(out, fits[c('rank', 'df.residual', 'qr', 'cov.unscaled')])
  if (length(skip) == nC) {
    out <- c(out, list(components=list(observed=comps.obs, permuted=NULL)))
    class(out) <- c('NBS', class(out))
    return(out)
  }

  # Create a null distribution of maximum component sizes
  #---------------------------------------------------------
  part.method <- match.arg(part.method); perm.method <- match.arg(perm.method)
  if (is.null(perms) || dim(perms)[2L] != n) perms <- shuffleSet(n=n, nset=N)
  null.dist <- randomise(perm.method, part.method, N, perms, X, yMat,
                         contrasts, ctype, nC, skip=skip, n=n, p=p, ny=ny, dfR=dfR)

  if (ctype == 't') {
    thresh <- switch(alt, two.sided=qt(p.init / 2, dfR, lower.tail=FALSE),
                     less=qt(p.init, dfR), greater=qt(p.init, dfR, lower.tail=FALSE))
    statfun <- switch(alt, two.sided=function(stat) abs(stat) > thresh, less=function(stat) stat < thresh,
                      greater=function(stat) stat > thresh)
    sig_inds_null <- which(statfun(null.dist), arr.ind=TRUE)
  } else {
    rkC <- vapply(contrasts, function(x) qr.default(x)$rank, integer(1L))
    thresh <- qf(p.init / 2, rkC, dfR, lower.tail=FALSE)
    sig_inds_null <- vector('list', nC)
    for (j in noSkip) {
      sig_inds_null[[j]] <- which(null.dist[, , j] > thresh[j], arr.ind=TRUE)
      sig_inds_null[[j]] <- cbind(sig_inds_null[[j]], dim3=j)
    }
    sig_inds_null <- do.call(rbind, sig_inds_null)
  }

  # Get the maximum component for each contrast & permutation
  comps.perm <- matrix(0, N, nC)
  compsJ <- rep.int(0, N)
  comps.perm[, noSkip] <- foreach(j=noSkip, .combine=cbind) %dopar% {
    sigJ <- sig_inds_null[sig_inds_null[, 3L] == j, 1L:2L]
    kNumEdges <- tabulate(sigJ[, 2L])

    # If there are 1-2 edges, the component size can only be 2 or 3
    singleEdge <- which(kNumEdges == 1L)
    doubleEdge <- which(kNumEdges == 2L)
    sigJ2 <- sigJ[sigJ[, 2L] %in% doubleEdge, ]
    compsDouble <- vapply(doubleEdge, function(x)
                          sum(tabulate(inds.high[sigJ2[sigJ2[, 2L] == x, 1L], ]) != 0),
                          numeric(1L))
    compsDouble[compsDouble == 4] <- 2
    compsJ[singleEdge] <- 2
    compsJ[doubleEdge] <- compsDouble

    doPerms <- which(kNumEdges > 2L)
    sigJ <- sigJ[sigJ[, 2L] %in% doPerms, ]
    for (i in doPerms) {
      rowInds <- sigJ[sigJ[, 2L] == i, 1L]
      compsJ[i] <- max(components(graph(t(inds.high[rowInds, , drop=FALSE]), directed=FALSE))$csize)
    }
    compsJ
  }

  p.perm <- unlist(lapply(noSkip, function(j)
                          vapply(comps.obs[contrast == j, csize], function(x)
                                 sum(comps.perm[, j] >= x) + 1L, numeric(1L))))
  p.perm <- p.perm / (N + 1L)
  comps.obs[contrast %in% noSkip, p.perm := p.perm]
  comps.obs[, Contrast := factor(contrast, labels=tmp$con.name)]

  comps.out <- list(observed=comps.obs)
  if (isTRUE(long)) comps.out$permuted <- comps.perm
  out <- c(out, list(part.method=part.method, perm.method=perm.method, components=comps.out))
  class(out) <- c('NBS', class(out))
  return(out)
}

################################################################################
# METHODS
################################################################################

#' Print a summary of NBS analysis
#'
#' @param object,x A \code{NBS} object
#' @inheritParams summary.bg_GLM
#' @export
#' @rdname NBS

summary.NBS <- function(object, contrast=NULL, digits=max(3L, getOption('digits') - 2L), ...) {

  # Observed component sizes
  #--------------------------------------
  nbs.dt <- with(object, data.table(alt=alt, N=N, components$observed, ecount=0))
  g.nbs <- make_brainGraphList(object, guess_atlas(object$T.mat), set.attrs=FALSE, .progress=FALSE)
  for (j in seq_along(g.nbs[])) {
    nbs.dt[contrast == j, ecount := vapply(seq_len(.N), function(x)
                                           ecount(subset_graph(g.nbs[j], paste('comp ==', x))$g),
                                           numeric(1L))]
  }

  nbs.sum <- c(object, list(printCon=contrast, DT.sum=nbs.dt, digits=digits), list(...))
  class(nbs.sum) <- c('summary.NBS', class(object))
  return(nbs.sum)
}

#' @aliases summary.NBS
#' @method print summary.NBS
#' @export

print.summary.NBS <- function(x, ...) {
  print_title_summary('Network-based statistic results')
  print_model_summary(x)
  print_permutation_summary(x)

  cat('Initial p-value: ', x$p.init, '\n\n')

  print_contrast_type_summary(x)
  print_subs_summary(x)

  # Print results for each contrast
  message('\n', 'Statistics', '\n', rep.int('-', getOption('width') / 4))
  print_contrast_stats_summary(x, ...)

  invisible(x)
}

#' @export
#' @rdname NBS
#' @include glm_methods.R
nobs.NBS <- nobs.bg_GLM

#' @export
#' @rdname NBS
terms.NBS <- function(x, ...) {
  x$outcome <- x$measure <- 'weight'
  x$DT.Xy <- x$covars
  terms(structure(x, class='bg_GLM'))
}

#' @export
#' @rdname NBS
formula.NBS <- function(x, ...) {
  x$outcome <- x$measure <- 'weight'
  x$DT.Xy <- x$covars
  formula(structure(x, class='bg_GLM'))
}

#' @export
#' @rdname NBS
#' @include glm_methods.R
labels.NBS <- labels.bg_GLM

#' @method case.names NBS
#' @export
#' @rdname NBS
#' @include glm_methods.R
case.names.NBS <- case.names.bg_GLM

#' @method variable.names NBS
#' @export
#' @rdname NBS
#' @include glm_methods.R
variable.names.NBS <- variable.names.bg_GLM

#' @method df.residual NBS
#' @export
#' @rdname NBS
#' @include glm_stats.R
df.residual.NBS <- df.residual.bg_GLM

#' Coordinates for data from brain atlases
#'
#' Datasets containing spatial coordinates for: the original AAL atlases, the
#' newer AAL2 atlases, Freesurfer atlases, Brainsuite, Craddock200,
#' Dosenbach160, Harvard-Oxford, and LONI probabilistic brain atlas. In addition
#' to coordinates, there are indices for the major lobes and hemispheres of the
#' brain, the \emph{class} variable (for Destrieux atlases), functional networks
#' (for Dosenbach, Power, and Gordon atlases; plus the Yeo network labels for
#' the Brainnetome atlas).
#'
#' @format A data frame with 90 or 116 (for the original AAL atlases), 94 or 120
#' (for the newer AAL2 atlases), 148 or 162 (for Destrieux), 68 or 82 (for DK),
#' 62 or 76 (for DKT), 74 (Brainsuite), 200 (Craddock), 160 (Dosenbach), 112
#' (Harvard-Oxford), 40 (LONI), 246 (Brainnetome), 360 (HCP), 264 (Power), or
#' 333 (Gordon) observations on (some of) the following 19 variables:
#' \describe{
#'   \item{\code{name}}{a character vector of region names}
#'   \item{\code{x.mni}}{a numeric vector of x-coordinates (in MNI space)}
#'   \item{\code{y.mni}}{a numeric vector of y-coordinates (in MNI space)}
#'   \item{\code{z.mni}}{a numeric vector of z-coordinates (in MNI space)}
#'   \item{\code{lobe}}{a factor with some of levels \code{Frontal}
#'     \code{Parietal} \code{Temporal} \code{Occipital} \code{Insula}
#'     \code{Limbic} \code{Cingulate} \code{SCGM} \code{Cerebellum} (for
#'     \code{aal116} and \code{aal2.120}) and \code{Brainstem} (for
#'     \code{craddock200})}
#'   \item{\code{hemi}}{a factor with levels \code{L} \code{R} and \code{B} (for
#'     \code{dosenbach160})}
#'   \item{\code{index}}{a numeric vector}
#'   \item{\code{name.full}}{a character vector of full region names, for the DK
#'     and DKT atlases}
#'   \item{\code{class}}{a factor with levels \code{G} \code{G_and_S} \code{S},
#'     for the Destrieux atlases}
#'   \item{\code{network}}{(dosenbach160) a factor with levels \code{default}
#'     \code{fronto-parietal} \code{cingulo-opercular} \code{sensorimotor}
#'     \code{cerebellum} \code{occipital}}
#'   \item{\code{gyrus}}{(brainnetome) Abbreviated names of gyri/regions
#'     (including subcortical), with 24 unique values}
#'   \item{\code{gyrus.full}}{(brainnetome) Full names of \code{gyrus}}
#'   \item{\code{subregion}}{(brainnetome) Abbreviated names of subregions
#'     (including subdivisions of subcortical gray matter)}
#'   \item{\code{subregion.full}}{(brainnetome) Full names of \code{subregion}}
#'   \item{\code{Yeo_7network}}{(brainnetome) Factor with 8 levels consisting of
#'     SCGM plus the 7 networks from Yeo et al.}
#'   \item{\code{Yeo_17network}}{(brainnetome) Factor with 18 levels consisting
#'     of SCGM plus the 17 networks from Yeo et al.}
#'   \item{\code{area}}{(HCP) a factor with 23 cortical areas}
#'   \item{\code{Anatomy}}{(power264) Full region/gyrus names for the Power
#'     atlas; contains 53 unique regions}
#'   \item{\code{Brodmann}}{(power264) Integer values for Brodmann areas}
#' }
#'
#' @name Brain Atlases
#' @aliases aal116
#' @rdname brain_atlases
#' @references Tzourio-Mazoyer, N. and Landeau, B. and Papathanassiou, D. and
#'   Crivello, F. and Etard, O. and Delcroix, N. and Mazoyer, B. and Joliot, M.
#'   (2002) Automated anatomical labeling of activations in SPM using a
#'   macroscopic anatomical parcellation of the MNI MRI single-subject brain.
#'   \emph{NeuroImage}, \bold{15(1)}, 273--289.
#'   \doi{10.1006/nimg.2001.0978}
#' @references Rolls, E.T. and Joliot, M. and Tzourio-Mazoyer, N. (2015)
#'   Implementation of a new parcellation of the orbitofrontal cortex in the
#'   automated anatomical labelling atlas. \emph{NeuroImage}, \bold{122}, 1--5.
#'   \doi{10.1016/j.neuroimage.2015.07.075}
"aal116"

#' @aliases aal90
#' @rdname brain_atlases
"aal90"

#' @aliases aal2.120
#' @rdname brain_atlases
"aal2.120"

#' @aliases aal2.94
#' @rdname brain_atlases
"aal2.94"

#' Coordinates for data from Freesurfer atlases
#'
#' @aliases destrieux destrieux.scgm dk dk.scgm dkt dkt.scgm
#' @rdname brain_atlases
#' @references Destrieux, C. and Fischl, B. and Dale, A. and Halgren E. (2010)
#'   Automatic parcellation of human cortical gyri and sulci using standard
#'   anatomic nomenclature. \emph{NeuroImage}, \bold{53(1)}, 1--15.
#'   \doi{10.1016/j.neuroimage.2010.06.010}
#' @references Desikan, R.S. and Segonne, F. and Fischl, B. et al. (2006) An
#'   automated labeling system for subdividing the human cerebral cortex on MRI
#'   scans into gyral based regions of interest. \emph{NeuroImage}, \bold{31},
#'   968--980. \doi{10.1016/j.neuroimage.2006.01.021}
#' @references Klein, A. and Tourville, J. (2012) 101 labeled brain images
#'   and a consistent human cortical labeling protocol. \emph{Front Neurosci},
#'   6. \doi{10.3389/fnins.2012.00171}
"destrieux"

#' @rdname brain_atlases
"destrieux.scgm"

#' @rdname brain_atlases
"dk"

#' @rdname brain_atlases
"dk.scgm"

#' @rdname brain_atlases
"dkt"

#' @rdname brain_atlases
"dkt.scgm"

#' Coordinates for data from the Brainsuite atlas
#'
#' @aliases brainsuite
#' @rdname brain_atlases
#' @references Shattuck, D.W. and Leahy, R.M. (2002) BrainSuite: an automated
#'   cortical surface identification tool. \emph{Medical Image Analysis},
#'   \bold{8(2)}, 129--142.
#' @references Pantazis, D. and Joshi, A.A. and Jintao, J. and Shattuck, D.W.
#'   and Bernstein, L.E. and Damasio, H. and Leahy, R.M. (2009) Comparison of
#'   landmark-based and automatic methods for cortical surface registration.
#'   \emph{NeuroImage}, \bold{49(3)}, 2479--2493.
"brainsuite"

#' Coordinates for data from the Craddock200 atlas
#'
#' @aliases craddock200
#' @rdname brain_atlases
#' @references Craddock, R.C. and James, G.A. and Holtzheimer, P.E. and Hu, X.P.
#'   and Mayberg, H.S. (2012) A whole brain fMRI atlas generated via spatially
#'   constrained spectral clustering. \emph{Human Brain Mapping}, \bold{33},
#'   1914--1928. \doi{10.1002/hbm.21333}
"craddock200"

#' Coordinates for data from the Dosenbach160 atlas
#'
#' @aliases dosenbach160
#' @rdname brain_atlases
#' @references Dosenbach, N.U. and Nardos, B. and Cohen, A.L. and Fair, D.A. and
#'   Power, J.D. and Church, J.A. and Nelson, S.M. and Wig, G.S. and Vogel, A.C.
#'   and Lessov-Schlaggar, C.N. and Barnes, K.A. (2010) Prediction of individual
#'   brain maturity using fMRI. \emph{Science}, \bold{329(5997)}, 1358--1361.
"dosenbach160"

#' Coordinates for data from the Harvard-Oxford atlas
#'
#' @aliases hoa112
#' @rdname brain_atlases
#' @references Makris, N. and Goldstein, J.M. and Kennedy, D. et al. (2006)
#'   Decreased volume of left and total anterior insular lobule in
#'   schizophrenia. \emph{Schizophr Res}, \bold{83(2-3)}, 155--171.
"hoa112"

#' Coordinates for data from the LONI probabilistic brain atlas
#'
#' @aliases lpba40
#' @rdname brain_atlases
#' @references Shattuck, D.W. and Mirza, M. and Adisetiyo, V. and Hojatkashani,
#'   C. and Salamon, G. and Narr, K.L. and Poldrack, R.A. and Bilder, R.M. and
#'   Toga, A.W. (2008) Construction of a 3D probabilistic atlas of human
#'   cortical structures. \emph{NeuroImage}, \bold{39(3)}, 1064--1080.
#'   \doi{10.1016/j.neuroimage.2007.09.031}
"lpba40"

#' Coordinates for data from HCP-MMP1.0 atlas
#'
#' @note Use of the HCP parcellation is subject to the terms at
#'   \url{https://balsa.wustl.edu/WN56}. In particular: "I will acknowledge the
#'   use of WU-Minn HCP data and data derived from WU-Minn HCP data when
#'   publicly presenting any results or algorithms that benefitted from their
#'   use."
#'
#' @aliases hcp_mmp1.0
#' @rdname brain_atlases
#' @source \url{https://neuroimaging-core-docs.readthedocs.io/en/latest/pages/atlases.html}
#' @references Glasser, M.F. and Coalson, T.S. and Robinson, E.C. and Hacker,
#'   C.D. and Harwell, J. and Yacoub, E. and Ugurbil, K. and Andersson, J. and
#'   Beckmann, C.F. and Jenkinson, M. and Smith, S.M. and van Essen, D.C.
#'   (2016) A multi-modal parcellation of human cerebral cortex. \emph{Nature},
#'   \bold{536}, 171--178. \doi{10.1038/nature18933}. PMID:
#'   27437579.
"hcp_mmp1.0"

#' Coordinates for data from the Power atlas
#'
#' @aliases power264
#' @rdname brain_atlases
#' @references Power, J.D. and Cohen, A.L. and Nelson, S.M. and Wig, G.S. and
#'   Barnes, K.A. and Church, J.A. and Vogel, A.C. and Laumann, T.O. and Miezin,
#'   F.M. and Schlaggar, B.L. and Petersen, S.E. (2011) Functional network
#'   organization of the human brain. \emph{Neuron}, \bold{72(4)}, 665--678.
#'   \doi{10.1016/j.neuron.2011.09.006}
"power264"

#' Coordinates for data from the Brainnetome atlas
#'
#' @aliases brainnetome
#' @rdname brain_atlases
#' @references Fan, L. and Li, H. and Zhuo, J. and Zhang, Y. and Wang, J. and
#'   Chen, L. and Yang, Z. and Chu, C. and Xie, S. and Laird, A.R. and Fox,
#'   P.T. and Eickhoff, S.B. and Yu, C. and Jiang, T (2016) The Human
#'   Brainnetome Atlas: A New Brain Atlas Based on Connectional Architecture.
#'   \emph{Cerebral Cortex}, \bold{26(8)}, 3508--3526.
#'   \doi{10.1093/cercor/bhw157}

"brainnetome"

#' Coordinates for data from the Gordon atlas
#'
#' @note Region names in the \code{gordon333} atlas were chosen to match those
#'   of the \code{hcp_mmp1.0} atlas. Many were determined from the coordinates
#'   (using FSL's \code{atlasquery}), while the rest were entered manually by
#'   me. The \code{lobe} values were matched to the HCP atlas, as well.
#' @aliases gordon333
#' @rdname brain_atlases
#' @references Gordon, E.M. and Laumann, T.O. and Adeyemo, B. and Huckins, J.F.
#'   and Kelley, W.M. and Petersen, S.E. (2014) Generation and Evaluation of a
#'   Cortical Area Parcellation from Resting-State Correlations. \emph{Cerebral
#'   Cortex}, \bold{26(1)}, 288--303.
#'   \doi{10.1093/cercor/bhu239}
"gordon333"

#' Extract model fit statistics from a bg_GLM object
#'
#' These functions extract or calculate model fit statistics of a
#' \code{bg_GLM} object. These can be found in the output from
#' \code{\link[stats]{summary.lm}}.
#'
#' These mimic the same functions that operate on \code{\link{lm}} objects, and
#' include:
#' \describe{
#'   \item{coef}{Regression coefficients (parameter estimates)}
#'   \item{confint}{Confidence intervals (by default, 95\%) for parameter
#'     estimates}
#'   \item{fitted}{Fitted (mean) values; i.e., the design matrix multiplied by
#'     the parameter estimates, \eqn{X \hat{\beta}}}
#'   \item{residuals}{Model residuals; i.e., the response/outcome variable minus
#'     the \emph{fitted} values. Partial residuals can also be calculated}
#'   \item{deviance}{Model deviance, or the \emph{residual sum of squares}}
#'   \item{coeff_determ}{Calculate the \emph{coefficient of determination} (or
#'     \eqn{R^2}), adjusted or unadjusted}
#'   \item{df.residual}{Residual degrees of freedom}
#'   \item{sigma}{Residual standard deviation, sometimes called the \emph{root
#'     mean squared error (RMSE)}}
#'   \item{vcov}{Variance-covariance matrix of the model parameters}
#' }
#'
#' \code{coeff_table} returns model coefficients, standard errors, T-statistics,
#' and P-values for all model terms and regions in a \code{bg_GLM} object. This
#' is the same as running \code{summary(x)$coefficients} for a \code{lm} object.
#'
#' @note \code{sigma} -- The denominator is \emph{not} the number of
#'   observations, but rather the model's \emph{residual degrees of freedom}.
#' @note When calculating \emph{partial residuals}, the parameter estimates are
#'   \emph{not} re-calculated after removing one of the model terms.
#'
#' @param object A \code{bg_GLM} object
#' @param ... Unused
#' @export
#' @return A named numeric vector, matrix, or array, depending on the function:
#'   \item{coef}{Matrix in which rownames are parameter names and column names
#'     are regions}
#'   \item{fitted,residuals}{Matrix in which rownames are Study ID's and column
#'     names are regions. If \code{type='partial'}, an array is returned in
#'     which columns are \emph{terms} and the 3rd dimension are regions}
#'   \item{deviance,coeff_determ,sigma}{Numeric vector with elements for each
#'     region}
#'   \item{df.residual}{Single integer; the degrees of freedom}
#'   \item{confint,vcov,coeff_table}{Numeric array; the extent of the third
#'     dimension equals the number of regions}
#'
#' @name GLM statistics
#' @rdname glm_stats
#' @seealso \code{\link{GLM}}, \code{\link[car]{Anova}}
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}

coef.bg_GLM <- function(object, ...) {
  if (!is.null(object$coefficients)) return(object$coefficients)

  X <- object$X
  if (is.null(p <- object$rank)) p <- dim(X)[2L]
  if (length(dim(X)) == 3L) {
    dimX <- dim(X)
    runX <- object$runX
    if (is.null(QR <- object$qr)) QR <- qr(X[, , runX])
    Q <- lapply(QR, qr_Q2, n=dimX[1L], p=p)
    R <- lapply(QR, qr_R2, p=p)
    coeffs <- matrix(NaN, dimX[2L], dimX[3L], dimnames=dimnames(X)[c(2L, 3L)])
    coeffs[, runX] <- vapply(runX, function(r) backsolve(R[[r]], crossprod(Q[[r]], object$y), p),
                             numeric(p))
  } else {
    if (is.null(QR <- object$qr)) QR <- qr.default(X)
    Q <- qr_Q2(QR, p=p)
    R <- qr_R2(QR, p)
    coeffs <- backsolve(R, crossprod(Q, object$y), p)
  }

  return(coeffs)
}

# Helper function to calculate standard errors of coefficients
coef_se <- function(object, pnames=variable.names(object)) {
  if (!is.null(object$se)) return(object$se)
  dimX <- dim(object$X)
  if (length(dimX) == 3L) {
    std_err <- apply(vcov(object), 3L, function(x) sqrt(diag(x)))[pnames, , drop=FALSE]
  } else {
    XtX <- object$cov.unscaled
    if (is.null(XtX)) XtX <- inv(object$X)[pnames, pnames, drop=FALSE]
    sig <- sigma(object)^2
    std_err <- sqrt(outer_vec(diag(XtX), sig))
  }
  std_err
}

#' @param parm Vector of parameters to calculate confidence intervals for.
#'   Default is to use all parameters
#' @param level The confidence level. Default: \code{0.95}
#' @export
#' @rdname glm_stats

confint.bg_GLM <- function(object, parm, level=0.95, ...) {
  pnames <- variable.names(object)
  dfR <- object$df.residual <- df.residual(object)
  object$coefficients <- coef(object)
  dimC <- dim(object$coefficients)
  std_err <- coef_se(object, pnames)
  if (missing(parm)) {
    parm <- pnames
  } else if (is.numeric(parm)) {
    parm <- pnames[parm]
  }
  a <- (1 - level) / 2
  a <- c(a, 1 - a)
  t.crit <- qt(a, dfR)
  ci <- array(object$coefficients, dim=c(dimC, 2L)) + outer(std_err, t.crit)
  ci <- aperm(ci, c(1L, 3L, 2L))
  ci <- ci[parm, , , drop=FALSE]
  dimnames(ci)[[2L]] <- paste(format(100 * a, trim=TRUE, scientific=FALSE, digits=3L), '%')
  return(ci)
}

#' @export
#' @rdname glm_stats

fitted.bg_GLM <- function(object, ...) {
  if (!is.null(object$fitted.values)) return(object$fitted.values)
  coeffs <- coef(object)

  # outcome != measure && level == 'vertex' (multiple design mats)
  dimX <- dim(object$X)
  if (length(dimX) == 3L) {
    runX <- object$runX
    fits <- matrix(NaN, dimX[1L], dimX[3L], dimnames=dimnames(object$X)[c(1L, 3L)])
    fits[, runX] <- vapply(runX, function(x) object$X[, , x] %*% coeffs[, x],
                           numeric(dimX[1L]))
  } else {
    fits <- object$X %*% coeffs
  }
  return(fits)
}

#' @param type Character string specifying the type of residuals to return.
#'   Default: \code{'response'}
#' @export
#' @rdname glm_stats

residuals.bg_GLM <- function(object, type=c('response', 'partial'), ...) {
  cf <- object$coefficients <- coef(object)
  type <- match.arg(type)
  if (type == 'partial') {
    object$residuals <- object$fitted.values <- NULL
    regions <- region.names(object)
    tm <- terms(object)
    if (any(grepl('Intercept', names(tm)))) tm <- tm[-grep('Intercept', names(tm))]
    resids <- array(0, dim=c(nobs(object), length(regions), length(tm)))
    for (i in seq_along(tm)) {
      object$coefficients <- cf[-tm[[i]], , drop=FALSE]
      resids[, , i] <- residuals(object[, -tm[[i]]])
    }
    resids <- aperm(resids, c(1L, 3L, 2L))
    dimnames(resids) <- list(case.names(object), names(tm), regions)

  } else {
    if (!is.null(object$residuals)) return(object$residuals)
    fits <- fitted(object)
    y <- if (dim(object$y)[2L] == 1L) c(object$y) else object$y
    resids <- y - fits
  }
  return(resids)
}

#' @export
#' @rdname glm_stats
deviance.bg_GLM <- function(object, ...) colSums(residuals(object)^2)

#' @param adjusted Logical indicating whether to calculate the adjusted
#'   R-squared. Default: \code{FALSE}
#' @export
#' @rdname glm_stats

coeff_determ <- function(object, adjusted=FALSE) {
  stopifnot(inherits(object, 'bg_GLM'))
  SSR <- deviance(object)
  if (dim(object$y)[2L] > 1L) {
    SStot <- diag(tcrossprod(t(object$y) - colMeans(object$y)))
  } else {
    SStot <- c(crossprod(object$y - mean(object$y)))
  }
  numer <- if (isTRUE(adjusted)) SSR / df.residual(object) else SSR
  denom <- if (isTRUE(adjusted)) SStot / (nobs(object) - 1L) else SStot
  1 - numer / denom
}

#' @export
#' @method df.residual bg_GLM
#' @rdname glm_stats

df.residual.bg_GLM <- function(object, ...) {
  if (!is.null(object$df.residual)) return(object$df.residual)
  dimX <- dim(object$X)
  dimX[1L] - dimX[2L]
}

#' @export
#' @rdname glm_stats

sigma.bg_GLM <- function(object, ...) {
  if (!is.null(object$sigma)) return(object$sigma)
  sqrt(deviance(object) / df.residual(object))
}

#' @export
#' @rdname glm_stats

vcov.bg_GLM <- function(object, ...) {
  if (!is.null(object$var.covar)) return(object$var.covar)
  sig <- sigma(object)^2
  X <- object$X
  dimX <- dim(X)

  XtX <- object$cov.unscaled
  if (is.null(XtX)) {
    if (length(dimX) == 3L) {
      namesX <- dimnames(X)
      dimV <- dimX[c(2L, 2L, 3L)]
      namesV <- namesX[c(2L, 2L, 3L)]
      runX <- object$runX
      XtX <- array(NaN, dim=dimV, dimnames=namesV)
      XtX[, , runX] <- inv(X[, , runX, drop=FALSE])
    } else {
      XtX <- inv(X)
    }
  }
  if (length(dimX) == 3L) {
    XtX <- aperm(XtX, c(3L, 1L, 2L))
    vcv <- sig * XtX
  } else {
    vcv <- outer(sig, XtX)
  }
  aperm(vcv, c(2L, 3L, 1L))
}

#' @param CI Logical indicating whether to include confidence intervals of
#'   parameter estimates in the coefficient summary table. Default: \code{FALSE}
#' @export
#' @importFrom abind abind
#' @rdname glm_stats

coeff_table <- function(object, CI=FALSE, level=0.95) {
  stopifnot(inherits(object, 'bg_GLM'))
  dfR <- object$df.residual <- df.residual(object)
  cf <- object$coefficients <- coef(object)
  se <- object$se <- coef_se(object)
  tab <- abind(cf, se, along=1.5)
  cnames <- c('Estimate', 'Std. Error', 't value', 'Pr(>|t|)')
  if (isTRUE(CI)) {
    ci <- confint(object, level=level)
    tab <- abind(tab, ci, along=2L)
    cnames <- append(cnames, dimnames(ci)[[2L]], after=2L)
  }
  t.stat <- tab[, 1L, , drop=FALSE] / tab[, 2L, , drop=FALSE]
  tab <- abind(tab, t.stat, along=2L)
  tab <- abind(tab, 2 * pt(abs(t.stat), dfR, lower.tail=FALSE), along=2L)
  dimnames(tab)[2L:3L] <- list(cnames, region.names(object))
  return(tab)
}

# Convenience function to calculate leave-one-out resid SD
sigma_loo <- function(model) {
  if (!is.null(model$sigma.loo)) return(model$sigma.loo)
  dfR <- df.residual(model)
  resids2 <- residuals(model)^2
  var.hat <- t(colSums(resids2) - t(resids2))
  sqrt(var.hat / (dfR - 1L))
}

#' Influence measures for a bg_GLM object
#'
#' These functions compute common (leave-one-out) diagnostics for the models in
#' a \code{bg_GLM} object.
#'
#' The \code{influence} method calculates all diagnostics present in
#' \code{\link[stats]{lm.influence}} and
#' \code{\link[stats]{influence.measures}}, consisting of the following
#' functions:
#' \describe{
#'   \item{rstandard}{Standardized residuals. Choosing \code{type='predictive'}
#'     returns leave-one-out cross validation residuals. The \dQuote{PRESS}
#'     statistic can be calculated as \code{colSums(resids.p^2)}}
#'   \item{rstudent}{Studentized residuals}
#'   \item{hatvalues}{The \emph{leverage}, or the diagonal of the
#'     \emph{hat/projection matrix}}
#'   \item{cooks.distance}{Cook's distance}
#'   \item{dffits_bg_GLM}{The change in fitted values when deleting
#'     observations}
#'   \item{dfbeta}{The change in parameter estimates (coefficients) when
#'     deleting observations}
#'   \item{dfbetas}{The \emph{scaled} change in parameter estimates}
#'   \item{covratio.bg_GLM}{The covariance ratios, or the change in the
#'     determinant of the covariance matrix of parameter estimates when deleting
#'     observations}
#' }
#'
#' @section Outlier values:
#' Each variable has a different criterion for determining outliers. In the
#' following: \code{x} is the influence variable (for \code{DFBETA}, the
#' criterion applies to all DFBETAs); \code{k} is the number of columns of the
#' design matrix; \code{dfR} is the residual degrees of freedom; and \code{n} is
#' the number of observations.
#' \describe{
#'   \item{DFBETAs}{If \eqn{|x| > 1}}
#'   \item{DFFITs}{If \eqn{|x| > 3 \sqrt{k / dfR}}}
#'   \item{covratio}{If \eqn{|1 - x| > (3k / dfR)}}
#'   \item{cook}{If \eqn{F_{k, dfR}(x) > 0.5}}
#'   \item{hat}{If \eqn{x > 3k / n}}
#' }
#' The return object of \code{influence} has a \code{print} method which will
#' list the subjects/variables/regions for which an outlier was detected.
#'
#' @param model A \code{bg_GLM} object
#' @param type The type of standardized residuals. Default: \code{'sd.1'}
#' @param ... Unused
#' @export
#' @return Most influence functions return a numeric matrix in which rownames
#'   are Study ID's and column names are regions. \code{dfbeta} and
#'   \code{dfbetas} return a numeric array in which each column is a parameter
#'   estimate and the 3rd dimension is for each region. \code{influence} returns
#'   a list with class \code{infl.bg_GLM} and elements:
#'   \item{infmat}{Numeric array (like \code{dfbeta}) with DFBETAs, DFFITs,
#'     covratios, Cook's distance, and hat values}
#'   \item{is.inf}{Logical array of the same data as \code{infmat}; values of
#'     \code{TRUE} indicate the subject-variable-region combination is an
#'     outlier value}
#'   \item{f}{The model \emph{formula}}
#'   \item{sigma}{The leave-one-out residual standard deviation}
#'   \item{wt.res}{Model residuals}
#'
#' @name GLM influence measures
#' @rdname glm_influence
#' @seealso \code{\link{GLM}}
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}

rstandard.bg_GLM <- function(model, type=c('sd.1', 'predictive'), ...) {
  model$residuals <- residuals(model)
  hat <- hatvalues(model)
  type <- match.arg(type)
  denom <- if (type == 'sd.1') t(sigma(model) * sqrt(1 - t(hat))) else 1 - hat
  model$residuals / denom
}

#' @export
#' @rdname glm_influence

rstudent.bg_GLM <- function(model, ...) {
  res <- model$residuals <- residuals(model)
  hat <- hatvalues(model)
  var.hat <- sigma_loo(model)
  res / (var.hat * sqrt(1 - hat))
}

#' @export
#' @rdname glm_influence

hatvalues.bg_GLM <- function(model, ...) {
  if (!is.null(model$hatvalues)) return(model$hatvalues)
  X <- model$X
  dimX <- dim(X)
  namesX <- dimnames(X)
  if (!is.null(model$cov.unscaled)) {
    XtX <- model$cov.unscaled
    if (length(dimX) == 3L) {
      hv <- matrix(NaN, dimX[1L], dimX[3L], dimnames=namesX[c(1L, 3L)])
      for (r in model$runX) {
        hv[, r] <- colSums(tcrossprod(XtX[, , r], X[, , r]) * t(X[, , r]))
      }
    } else {
      hv <- colSums(tcrossprod(XtX, X) * t(X))
      rgn <- region.names(model)
      hv <- matrix(hv, dimX[1L], length(rgn), dimnames=list(namesX[[1L]], rgn))
    }
    return(hv)
  }
  if (length(dimX) == 3L) {
    runX <- model$runX
    A <- U <- array(NaN, dim=dimX[c(2L, 2L, 3L)], dimnames=namesX[c(2L, 2L, 3L)])
    A[, , runX] <- apply(X[, , runX, drop=FALSE], 3L, crossprod)
    U[, , runX] <- apply(A[, , runX, drop=FALSE], 3L, chol.default)
    Z <- array(0, dim=dimX[c(2L, 1L, 3L)])
    for (i in seq_len(dimX[3L])) {
      Z[, , i] <- forwardsolve(t(U[, , i]), t(X[, , i]))
    }
    res <- colSums(Z^2)
    dimnames(res) <- c(namesX[1L], namesX[3L])
  } else {
    A <- crossprod(X)
    U <- chol.default(A)
    Z <- forwardsolve(t(U), t(X))
    res <- colSums(Z^2)
    rgn <- region.names(model)
    res <- matrix(res, dimX[1L], length(rgn), dimnames=list(namesX[[1L]], rgn))
  }
  return(res)
}

#' @export
#' @rdname glm_influence

cooks.distance.bg_GLM <- function(model, ...) {
  hat <- hatvalues(model)
  res <- model$residuals <- residuals(model)
  sd <- sigma(model)
  hat * (res / t(sd * (1 - t(hat))))^2 / model$rank
}

#' @export
#' @rdname glm_influence

dffits_bg_GLM <- function(model) {
  res <- model$residuals <- residuals(model)
  hat <- hatvalues(model)
  sig <- sigma_loo(model)
  res * sqrt(hat) / (sig * (1 - hat))
}

#' @export
#' @rdname glm_influence

dfbeta.bg_GLM <- function(model, ...) {
  X <- model$X
  dimX <- dim(X)
  namesX <- dimnames(X)
  regions <- region.names(model)
  Nv <- length(regions)

  resids <- residuals(model)
  hat <- hatvalues(model)
  mult <- resids / (1 - hat)
  multArr <- array(mult, dim=c(dimX[1L], Nv, dimX[2L]))
  multArr <- aperm(multArr, c(1L, 3L, 2L))

  if (length(dimX) == 3L) {
    runX <- model$runX
    numer <- array(NaN, dim=dimX, dimnames=namesX)
    XtX <- model$cov.unscaled
    if (is.null(XtX)) {
      XtX <- array(NaN, dim=dimX[c(2L, 2L, 3L)], dimnames=namesX[c(2L, 2L, 3L)])
      XtX[, , runX] <- inv(X[, , runX, drop=FALSE])
    }
    numer[, , runX] <- vapply(runX, function(r) t(tcrossprod(XtX[, , r], X[, , r])),
                              numeric(dimX[1L] * dimX[2L]))
  } else {
    XtX <- model$cov.unscaled
    if (is.null(XtX)) XtX <- inv(X)
    numer <- t(tcrossprod(XtX, X))
    numer <- array(numer, dim=c(dimX, Nv), dimnames=c(namesX, list(regions)))
  }
  numer * multArr
}

#' @export
#' @rdname glm_influence

dfbetas.bg_GLM <- function(model, ...) {
  resids <- model$residuals <- residuals(model)
  sig <- sigma_loo(model)

  X <- model$X
  dimX <- dim(X)
  XtX <- model$cov.unscaled
  if (length(dimX) == 3L) {
    namesX <- dimnames(X)
    runX <- model$runX
    if (is.null(XtX)) {
      XtX <- array(NaN, dim=dimX[c(2L, 2L, 3L)], dimnames=namesX[c(2L, 2L, 3L)])
      XtX[, , runX] <- inv(model$X[, , runX, drop=FALSE])
      model$cov.unscaled <- XtX
    }
    xxi <- apply(XtX, 3L, function(x) sqrt(diag(x)))
    dfb <- if (is.null(model$dfbeta)) dfbeta(model) else model$dfbeta

    dfbs <- array(NaN, dim=dimX, dimnames=dimnames(dfb))
    for (i in runX) {
      dfbs[, , i] <- dfb[, , i] / tcrossprod(sig[, i], xxi[, i])
    }
  } else {
    if (is.null(XtX)) XtX <- model$cov.unscaled <- inv(X)
    xxi <- sqrt(diag(XtX))
    dfb <- if (is.null(model$dfbeta)) dfbeta(model) else model$dfbeta
    dfbs <- dfb / aperm(outer(sig, xxi), c(1L, 3L, 2L))
  }
  return(dfbs)
}

#' @export
#' @rdname glm_influence

covratio.bg_GLM <- function(model) {
  dfR <- model$df.residual <- df.residual(model)
  p <- model$rank
  if (is.null(p)) p <- dim(model$X)[2L]
  omh <- 1 - hatvalues(model)
  res <- model$residuals <- residuals(model)
  sig <- sigma_loo(model)
  e.star <- res / (sig * sqrt(omh))
  1 / (omh * ((e.star^2 + dfR - 1) / dfR)^p)
}

#' @param do.coef Logical indicating whether to calculate \code{dfbeta}
#' @param region Character string of the region(s) to return results for.
#'   Default is to calculate for all regions
#' @export
#' @rdname glm_influence
#' @importFrom abind abind

influence.bg_GLM <- function(model, do.coef=TRUE, region=NULL, ...) {
  n <- nobs(model)
  hat <- model$hatvalues <- hatvalues(model)
  model$residuals <- residuals(model)
  sig <- model$sigma.loo <- sigma_loo(model)
  dff <- dffits_bg_GLM(model)
  covr <- covratio.bg_GLM(model)
  cook <- cooks.distance(model)

  # Create an array and determine which observations are influential based on any metric
  vnames <- variable.names(model)
  k <- length(vnames)
  dfR <- n - k

  cnames <- c('dffit', 'cov.r', 'cook.d', 'hat')
  infmat <- abind(dff, covr, cook, hat, along=1.5)
  infl <- abind(abs(dff) > 3 * sqrt(k / dfR),
                abs(1 - covr) > (3 * k) / dfR,
                pf(cook, k, dfR) > 0.5,
                hat > (3 * k) / n,
                along=1.5)
  if (isTRUE(do.coef)) {
    coeffs <- dfbetas(model)
    cnames <- c(paste0('dfb.', vnames), cnames)
    infmat <- abind(coeffs, infmat, along=2L)
    infl <- abind(abs(coeffs) > 1, infl, along=2L)
  }
  dimnames(infmat)[[2L]] <- dimnames(infl)[[2L]] <- cnames
  if (!is.null(region)) {
    infmat <- infmat[, , region, drop=FALSE]
    infl <- infl[, , region, drop=FALSE]
    sig <- sig[, region, drop=FALSE]
    model$residuals <- model$residuals[, region, drop=FALSE]
  }
  ans <- list(infmat=infmat, is.inf=infl, f=formula(model), sigma=sig, wt.res=model$residuals)
  class(ans) <- c('infl.bg_GLM', class(ans))
  return(ans)
}

#' @method print infl.bg_GLM
#' @export

print.infl.bg_GLM <- function(x, region=NULL, ...) {
  sID <- getOption('bg.subject_id')
  Region <- total <- value <- NULL
  message('\nInfluence measures and outliers for a bg_GLM model with formula:')
  cat('  ', x$f, '\n\n')
  DT <- as.data.table(x$is.inf, key='V1')
  setnames(DT, c(sID, 'variable', 'Region', 'value'))
  if (!is.null(region)) DT <- DT[Region %in% region]
  DT <- DT[value == TRUE]
  DT.split <- split(DT, by='variable', keep.by=FALSE)

  if (dim(x$infmat)[3L] > 1L) {
    DT.split.wide <- lapply(DT.split, dcast, paste(sID, '~ Region'))
    for (nm in names(DT.split.wide)) {
      DT.split.wide[[nm]][, total := DT.split[[nm]][, .N, by=sID]$N]
      message('Variable: ', nm)
      print(DT.split.wide[[nm]])
      cat('\n')
    }
  } else {  # single region
    for (nm in names(DT.split)) {
      message('Variable: ', nm)
      cat('  ', paste(DT.split[[nm]][, as.character(get(sID))], collapse=', '), '\n')
      cat('\n')
    }
  }
  invisible(x)
}

#' Variance inflation factors for \code{bg_GLM} objects
#'
#' @param mod A \code{bg_GLM} object
#' @param ... Unused
#' @export
#' @return A named array of VIFs; names of the 3rd dimension are regions

vif.bg_GLM <- function(mod, ...) {
  stopifnot(inherits(mod, 'bg_GLM'))
  X <- mod$X
  dimX <- dim(X)
  cols <- terms(mod)
  tlabels <- names(cols)
  if ((hasInt <- any(grepl('Intercept', tlabels)))) {
    int <- grep('Intercept', tlabels)
    cols <- lapply(cols[-int], `-`, 1L)
  } else {
    warning('No intercept; VIFs may not be sensible.')
  }

  regions <- region.names(mod)
  kNumRegions <- length(regions)
  n.terms <- length(cols)
  if (n.terms < 2L) stop('Model contains fewer than 2 terms!')
  df <- lengths(cols, use.names=FALSE)
  terms_single <- which(df == 1L)
  terms_mult <- which(df > 1L)
  v <- mod$cov.unscaled
  if (length(dimX) == 3L) {
    runX <- mod$runX
    #TODO: to remove
    if (is.null(v)) {
      v <- array(NaN, dim=dimX[c(2L, 2L, 3L)], dimnames=dimnames(X)[c(2L, 2L, 3L)])
      v[, , runX] <- inv(X[, , runX, drop=FALSE])
    }
    R <- array(NaN, dim=dim(v), dimnames=dimnames(v))
    if (isTRUE(hasInt)) {
      v <- v[-int, -int, , drop=FALSE]
      R <- R[-int, -int, , drop=FALSE]
    }
    R[, , runX] <- apply(v[, , runX, drop=FALSE], 3L, cov2cor)
    detR <- setNames(rep(NaN, kNumRegions), regions)
    detR[runX] <- apply(R[, , runX, drop=FALSE], 3L, det)
    res <- array(NaN, dim=c(n.terms, 3L, kNumRegions), dimnames=list(NULL, NULL, regions))

    if (length(terms_single) > 0L) {
      for (rgn in runX) {
        res[terms_single, 1L, rgn] <- vapply(cols[terms_single], function(j)
                                             det(as.matrix(R[-j, -j, rgn])),
                                             numeric(1L))
      }
    }
    if (length(terms_mult) > 0L) {
      for (rgn in runX) {
        res[terms_mult, 1L, rgn] <- vapply(cols[terms_mult], function(j)
                                           det(R[j, j, rgn]) * det(as.matrix(R[-j, -j, rgn])),
                                           numeric(1L))
      }
    }
    res[, 1L, ] <- t(t(res[, 1L, ]) / detR)
  } else {
    if (is.null(v)) v <- inv(X)
    if (hasInt) v <- v[-int, -int, drop=FALSE]
    R <- cov2cor(v)
    detR <- det(R)
    res <- matrix(0, nrow=n.terms, ncol=3L)
    if (length(terms_single) > 0L) {
      res[terms_single, 1L] <- vapply(cols[terms_single], function(j) det(R[-j, -j, drop=FALSE]), numeric(1L))
    }
    if (length(terms_mult) > 0L) {
      res[terms_mult, 1L] <- vapply(cols[terms_mult], function(j)
                                    det(R[j, j, drop=FALSE]) * det(R[-j, -j, drop=FALSE]),
                                    numeric(1L))
    }
    res[, 1L] <- res[, 1L] / detR
    res <- array(res, dim=c(dim(res), kNumRegions))
  }
  dimnames(res) <- list(names(cols), c('GVIF', 'Df', 'GVIF^(1/(2*Df))'), regions)
  res[, 2L, ] <- df
  if (any(df != 1L)) res[, 3L, ] <- res[, 1L, ]^(1 / (2L * df))
  return(res)
}

#' @section ANOVA tables:
#' The \code{anova} method calculates the so-called \emph{Type III} test
#' statistics for a \code{bg_GLM} object. These standard ANOVA statistics
#' include: sum of squares, mean squares, degrees of freedom, F statistics, and
#' P-values. Additional statistics calculated are: \eqn{\eta^2}, partial
#' \eqn{\eta^2}, \eqn{\omega^2}, and partial \eqn{\omega^2} as measures of
#' \emph{effect size}.
#'
#' @param region Character vector indicating the region(s) to calculate ANOVA
#'   statistics for. Default: \code{NULL} (use all regions)
#' @export
#' @return \code{anova} returns a \emph{list} of tables of class \code{anova}
#' @rdname glm_stats

anova.bg_GLM <- function(object, region=NULL, ...) {
  regions <- if (is.null(region)) region.names(object) else region
  kNumRegions <- length(regions)
  RSS <- deviance(object)[regions]

  cols <- terms(object)
  object$coefficients <- object$fitted.values <- object$residuals <- object$rank <- object$qr <- NULL
  SSt <- vapply(cols, function(x) deviance(object[, -x])[regions], numeric(kNumRegions))
  ss <- cbind(SSt - RSS, Residuals=RSS)
  df <- c(lengths(cols), df.residual(object))
  ms <- t(t(ss) / df)

  nrows <- length(df)      # Number of terms + 1 (residuals)
  SSTot <- .rowSums(ss, kNumRegions, nrows)
  eta2 <- ss / SSTot
  eta2.part <- ss / (ss + RSS)
  MSE <- RSS / df[nrows]
  Ndf <- nobs(object) - df
  MS <- tcrossprod(MSE, df)
  omega2 <- (ss - MS) / (SSTot + MSE)
  omega2.part <- (ss - MS) / (ss + tcrossprod(MSE, Ndf))
  f <- ms / MSE
  p <- matrix(nrow=kNumRegions, ncol=nrows, dimnames=dimnames(ss))
  p[, -nrows] <- t(pf(t(f[, -nrows]), df[-nrows], df[nrows], lower.tail=FALSE))

  dfMat <- matrix(df, kNumRegions, nrows, byrow=TRUE)
  arr <- abind(ss, ms, dfMat, f, eta2, eta2.part, omega2, omega2.part, p, along=1.5)
  arr <- aperm(arr, c(3L, 2L, 1L))
  dimnames(arr)[[2L]] <- c('Sum Sq', 'Mean Sq', 'Df', 'F value', 'eta^2', 'Partial eta^2',
                           'omega^2', 'Partial omega^2', 'Pr(>F)')
  arr[nrows, 4L:8L, ] <- NA
  tab <- apply(arr, 3L, as.data.frame)
  for (i in regions) {
    attr(tab[[i]], 'heading') <- c('Anova Table (Type III tests)\n',
                                   paste('Response:', object$outcome))
    class(tab[[i]]) <- c('anova', class(tab[[i]]))
  }
  return(tab)
}

#' Model selection for bg_GLM objects
#'
#' These functions compute the log-likelihood and Akaike's \emph{An Information
#' Criterion (AIC)} of a \code{bg_GLM} object. See
#' \code{\link[stats]{logLik.lm}} and \code{\link[stats]{extractAIC}} for
#' details.
#'
#' The functions \code{\link[stats]{AIC}} and \code{\link[stats]{BIC}} will also
#' work for \code{bg_GLM} objects because they each call \code{logLik}.
#'
#' @param object,fit A \code{bg_GLM} object
#' @param REML Logical indicating whether to return the \emph{restricted}
#'   log-likelihood. Default: \code{FALSE}
#' @param ... Unused
#' @export
#' @return \code{logLik} returns an object of class \code{logLik} with several
#'   attributes. \code{extractAIC} returns a numeric vector in which the first
#'   element is the \emph{equivalent degrees of freedom} and the remaining are
#'   the AIC's for each region
#'
#' @name GLM model selection
#' @rdname glm_model_select

logLik.bg_GLM <- function(object, REML=FALSE, ...) {
  X <- object$X
  p <- object$rank
  N0 <- N <- nobs(object)

  if (isTRUE(REML)) {
    dimX <- dim(X)
    N <- N - p
    if (length(dimX) == 3L) {
      runX <- object$runX
      QR <- setNames(rep(NaN, dimX[3L]), dimnames(X)[[3L]])
      QR[runX] <- colSums(vapply(object$qr[runX], function(x)
                                 log(abs(diag(x$qr))), numeric(dimX[2L])))
    } else {
      QR <- sum(log(abs(diag(object$qr$qr))))
    }
  }
  val <- -0.5 * N * (log(2 * pi) + 1 - log(N) + log(deviance(object)))
  if (isTRUE(REML)) val <- val - QR
  structure(val, nall=N0, nobs=N, df=p+1L, class='logLik')
}

#' @param scale Should be left at its default
#' @param k Numeric; the weight of the equivalent degrees of freedom
#' @export
#' @rdname glm_model_select

extractAIC.bg_GLM <- function(fit, scale=0, k=2, ...) {
  n <- nobs(fit)
  edf <- n - df.residual(fit)
  RSS <- deviance(fit)
  dev <- if (scale > 0) RSS / scale - n else n * log(RSS / n)
  c(edf, dev + k * edf)
}

#' Save PNG of one or three views for all graphs in a brainGraphList
#'
#' \code{plot_brainGraph_multi} writes a PNG file to disk containing three views
#' (columns) of 1 or more \code{brainGraph} objects (from left-to-right): left
#' sagittal, axial, and right sagittal. The number of rows in the figure will
#' equal the number of graphs to plot.
#'
#' Whether the first input is a \code{brainGraphList} object or a \code{list} of
#' \code{brainGraph} objects, \emph{all} graphs in the object will be displayed
#' in the figure. For \code{plot_brainGraph_multi}, this may be undesirable if
#' you have more than 4 or 5 graphs in one object. You can choose fewer by using
#' simple subsetting operations (see Examples below).
#'
#' @section Using subgraphs, titles, and labels:
#' There are three arguments that can differ for each graph to be displayed.
#' Each follows the same \dQuote{rules}. If you would like the same value
#' applied to all graphs, you can specify a \code{character} string. If you
#' would like a different value for each group, you must supply a \code{vector}
#' or \code{list} with length equal to the number of graphs. If its length is
#' less than the number of graphs, values will be recycled. To \dQuote{skip}
#' applying a value to one (or more) graph(s), you can use the \code{NULL} value
#' only within a list (see the Examples below).
#' \describe{
#'   \item{subgraph}{Can be used to apply one or more conditions for subsetting
#'     the graph(s).}
#'   \item{main}{Controls the main plot title, which appears in the \emph{axial}
#'     view along with each graph's \code{name} attribute. Depending on the
#'     \code{level} of the \code{brainGraphList}, this will either be a Study
#'     ID, Group name, or contrast name.}
#'   \item{label}{Can be used to print a text label in a corner for each
#'     group/graph. For example, you can print a letter if you will refer to,
#'     e.g., \dQuote{Figure 1A}, \dQuote{Figure 1B}, etc.}
#' }
#'
#' @note All other arguments (passed to \code{\link{plot.brainGraph}}) will be
#' applied to \emph{all} graphs. For example, if you include
#' \code{vertex.label=NA} in the function call, vertex labels will be omitted
#' for all graphs.
#'
#' @param g.list A \code{brainGraphList} or a list of \code{brainGraph} objects
#' @param filename Character string of the filename of the PNG to be written.
#' @param subgraph A vector or list of character strings to (optionally) subset
#'   the graph(s), possibly by multiple conditions
#' @param main A vector or list of character strings to be placed in the main
#'   title of the center (axial) plot for each graph
#' @param label A vector or list of character strings to be placed in one of the
#'   corners of the left plot (sagittal) in each row
#' @param cex.main Numeric specifying the level of character expansion for the
#'   plot titles. Default: \code{1} (no expansion)
#' @param ... Other arguments passed to \code{\link{plot.brainGraph}}
#' @export
#' @importFrom graphics layout par
#' @importFrom grDevices dev.off png
#'
#' @family Plotting functions
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}
#' @examples
#' \dontrun{
#' ## "g.hubs" contains 2 groups; apply same subset to both
#' plot_brainGraph_multi(g.hubs, filename='Figure01_hubs.png',
#'   subgraph='N > 0', vertex.color='color.lobe', vertex.size=15,
#'   show.legend=TRUE, vertex.label.cex=1.5)
#'
#' ## Single group, different subgraphs for both plots
#' ## "g" is a "brainGraphList" object
#' gg <- g[rep(1, 3), drop=FALSE]
#' plot_brainGraph_multi(gg, filename='group1_5-6-7core.png',
#'   vertex.color='color.lobe', edge.color='color.lobe', vertex.label=NA,
#'   subgraph=as.list(paste('coreness >', 5:7)),
#'   main=as.list(paste('k-core', 5:7)))
#'
#' ## Apply different subset for groups 1 & 3; no subset for group 2
#' plot_brainGraph_multi(g, groups=1:3, vertex.label=NA,
#'   subgraph=list('degree > 5', NULL, 'degree > 4'))
#' }

plot_brainGraph_multi <- function(g.list, filename='orthoview.png',
                                  subgraph=NULL, main=NULL, label=NULL,
                                  cex.main=1, ...) {
  if (!is.brainGraphList(g.list)) try(g.list <- as_brainGraphList(g.list))
  g.list <- g.list[]
  kNumGraphs <- length(g.list)

  width <- 671 * 3
  height <- 673
  png(filename=filename, width=width, height=height*kNumGraphs, units='px', res=300)
  layout(matrix(1L:(3L*kNumGraphs), kNumGraphs, 3L, byrow=TRUE),
         widths=c(4/3, 1, 4/3), heights=rep.int(1L, kNumGraphs))

  # See if there are multiple "subgraph", "main", or "label" arguments
  if (!is.null(subgraph)) subgraph <- rep_len(as.list(subgraph), kNumGraphs)
  main.title <- lapply(g.list, graph_attr, 'name')
  if (!is.null(main)) {
    main <- rep_len(as.list(main), kNumGraphs)
    main.title <- Map(paste0, main.title, ': ', main)
  }
  if (!is.null(label)) label <- rep_len(as.list(label), kNumGraphs)

  for (i in seq_len(kNumGraphs)) {
    plot(g.list[[i]], plane='sagittal', hemi='L', subgraph=subgraph[[i]], main='\n\n\nLH',
         label=label[[i]], cex.main=cex.main, ...)
    plot(g.list[[i]], subgraph=subgraph[[i]], main=main.title[[i]], cex.main=cex.main, ...)
    plot(g.list[[i]], plane='sagittal', hemi='R', subgraph=subgraph[[i]], main='\n\n\nRH',
         cex.main=cex.main, ...)
  }
  dev.off()
}

#' Save PNG of a single view for all graphs in a brainGraphList
#'
#' \code{slicer} writes a PNG file to disk containing a single view (i.e.,
#' either sagittal, axial, or circular) of all \code{brainGraph} objects in the
#' input list/\code{brainGraphList}.
#'
#' @param nrows Integer; the number of rows in the figure
#' @param ncols Integer; the number of columns in the figure
#' @inheritParams plot.brainGraph
#' @export
#' @rdname plot_brainGraph_multi

slicer <- function(g.list, nrows, ncols, plane='axial', hemi='both', filename='all.png',
                   main=NULL, cex.main=1, ...) {
  if (!is.brainGraphList(g.list)) try(g.list <- as_brainGraphList(g.list))
  g.list <- g.list[]
  kNumGraphs <- length(g.list)
  if (missing(nrows) && missing(ncols)) {
    nrows <- 1L
    ncols <- kNumGraphs
  } else {
    if (missing(ncols)) {
      ncols <- ceiling(kNumGraphs / nrows)
    } else if (missing(nrows)) {
      nrows <- ceiling(kNumGraphs / ncols)
    }
  }
  kNumPlots <- nrows * ncols
  if (kNumPlots < kNumGraphs) {
    warning('Specified # of rows and columns less than the # of graphs; adjusting nrows.')
    rem <- (kNumGraphs / nrows) %% ncols
    nrows <- (kNumGraphs %/% ncols) + (rem > 0L)
    kNumPlots <- nrows * ncols
  }

  width <- 671
  height <- if (plane == 'axial') 768 else 640
  png(filename=filename, width=width*ncols, height=height*nrows, units='px', res=300)
  layout(matrix(seq_len(kNumPlots), nrows, ncols, byrow=TRUE),
         widths=rep.int(1L, ncols), heights=rep.int(1L, nrows))

  main.title <- lapply(g.list, graph_attr, 'name')
  if (!is.null(main)) {
    main <- rep_len(as.list(main), kNumGraphs)
    main.title <- Map(paste0, main.title, ': ', main)
  }
  for (i in seq_len(kNumGraphs)) {
    plot(g.list[[i]], plane=plane, hemi=hemi, main=main.title[[i]], cex.main=cex.main, ...)
  }
  if (kNumPlots > kNumGraphs) {
    for (i in seq_len(kNumPlots - kNumGraphs)) par(new=TRUE)
  }
  dev.off()
}

#' Plot normalized rich club coefficients against degree threshold
#'
#' Returns a line plot of the normalized rich club coefficient. Optionally, can
#' include a shaded region demarcating the \code{\link{rich_core}} cutoff (if
#' you supply a list of graph objects to the \code{g} argument).
#'
#' @param rich.dt A \code{data.table} with rich-club coefficients
#' @param facet.by A character string indicating whether the variable of
#' interest is \dQuote{density} or \dQuote{threshold} (e.g. with DTI data)
#' @param densities A numeric vector of the densities to plot
#' @param alpha The significance level. Default: \code{0.05}
#' @param fdr A logical, indicating whether or not to use the FDR-adjusted
#'   p-value for determining significance. Default: \code{TRUE}
#' @param g.list A list \code{brainGraphList} objects; required if you want to
#'   plot a shaded region demarcating the \code{\link{rich_core}}
#' @param smooth Logical indicating whether or not to plot a smooth curve when
#'   data from multiple subjects (per group) are present. Default: \code{TRUE}.
#'   Ignored for group-level data.
#' @export
#'
#' @return A \code{trellis} or \code{ggplot} object
#'
#' @family Rich-club functions
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}
#' @examples
#' \dontrun{
#' plot_rich_norm(rich.dt, facet.by='density', densities[N:(N+1)], g=g)
#' }

plot_rich_norm <- function(rich.dt, facet.by=c('density', 'threshold'),
                           densities, alpha=0.05, fdr=TRUE, g.list=NULL, smooth=TRUE) {
  yloc <- norm <- xstart <- xend <- panel.num <- NULL
  gID <- getOption('bg.group')

  facet.by <- match.arg(facet.by)
  subDT <- rich.dt[round(get(facet.by), 2L) %in% round(densities, 2L)]
  pvar <- if (isTRUE(fdr)) 'p.fdr' else 'p'
  subDT[, star := ifelse(get(pvar) < alpha, '*', '')]
  subDT[, yloc := extendrange(norm)[1L], by=facet.by]
  subDT[, eval(gID) := as.factor(get(gID))]
  grps <- subDT[, levels(get(gID))]
  if (length(grps) > 1L) {
    for (i in 2L:length(grps)) {
      subDT[get(gID) == grps[i], yloc := yloc - i * 0.05 * diff(range(norm, na.rm=TRUE)), by=facet.by]
    }
  }
  setkeyv(subDT, c(facet.by, gID))

  rects <- data.table(density=subDT[, unique(get(facet.by))], xstart=0L, xend=0L,
                      Group=rep(subDT[, unique(get(gID))],
                                each=length(densities)))
  setnames(rects, 'Group', gID)
  if (!is.null(g.list)) {
    # Check if components are 'brainGraphList' objects
    matches <- vapply(g.list, is.brainGraphList, logical(1L))
    if (any(!matches)) stop("Input must be a list of 'brainGraphList' objects.")

    densities.g <- round(vapply(g.list, function(x) graph_attr(x[1L], 'density'), numeric(1L)),  2L)
    densities.g <- which(densities.g %in% round(densities, 2L))
    g.list <- g.list[densities.g]
    k <- vapply(g.list, function(y) vapply(y[], function(x) rich_core(x)$k.r, integer(1L)), integer(nobs(g.list[[1L]])))
    max.k <- apply(as.matrix(k), 1L, max)

    rects[, xstart := rep(max.k, each=length(densities))]
    rects[, xend := rep(subDT[, max(k), by=facet.by]$V1, each=length(densities))]
  }
  setnames(rects, 'density', facet.by)
  setkeyv(rects, key(subDT))
  rects[, eval(facet.by) := factor(get(facet.by), labels=densities)]
  subDT[, eval(facet.by) := factor(get(facet.by), labels=densities)]
  setkeyv(subDT, c(facet.by, gID))
  setkeyv(rects, c(facet.by, gID))
  rects <- subDT[rects]

  sID <- getOption('bg.subject_id')
  smooth <- FALSE
  if (hasName(subDT, sID) && subDT[, !all.equal(as.character(get(gID)), get(sID))]) {
    smooth <- TRUE
  }

  # 'base' plotting
  #-----------------------------------------------------------------------------
  if (!requireNamespace('ggplot2', quietly=TRUE)) {
    ymin <- rects[, min(yloc)]
    ymax <- rects[, max(norm)]
    ylimit <- extendrange(c(ymin, ymax))
    ylabel <- expression(phi[norm])
    rects[, panel.num := as.numeric(get(facet.by))]
    if (isTRUE(smooth)) {
      plot.type <- 'smooth'
      panelfun <- function(x, y, groups, xleft, xright, ...) {
        panel.num <- NULL
        panel.abline(h=1, lty=2)
        panel.xyplot(x, y, groups, ...)
        rects[panel.num == panel.number() & get(gID) == grps[1L] & star == '*',
              panel.text(k, yloc, labels='*', col=plot.cols[1L])]
        rects[panel.num == panel.number() & get(gID) == grps[2L] & star == '*',
              panel.text(k, yloc, labels='*', col=plot.cols[2L])]
      }
      p <- xyplot(norm ~ k | get(facet.by), groups=get(gID), data=rects,
                  ylim=ylimit, xlab='Degree (k)', ylab=ylabel,
                  type=plot.type, panel=panelfun)
    } else {
      plot.type <- 'l'
      panelfun <- function(x, y, groups, xleft, xright, ...) {
        panel.num <- NULL
        panel.rect(xleft, ybottom=-999, xright, ytop=999, col='lightgrey', border=NULL)
        panel.abline(h=1, lty=2)
        panel.xyplot(x, y, groups, ...)
        rects[panel.num == panel.number() & get(gID) == grps[1L] & star == '*',
              panel.text(k, yloc, labels='*', col=plot.cols[1L])]
        rects[panel.num == panel.number() & get(gID) == grps[2L] & star == '*',
              panel.text(k, yloc, labels='*', col=plot.cols[2L])]
      }
      p <- xyplot(norm ~ k | get(facet.by), groups=get(gID), data=rects,
                  ylim=ylimit, xlab=list(label='Degree (k)', cex=0.8), ylab=ylabel,
                  xleft=rects$xstart, xright=rects$xend, type=plot.type, panel=panelfun,
                  auto.key=list(space='bottom', title=gID, cex.title=1, columns=length(grps),
                                lines=TRUE, points=FALSE))
    }

  # 'ggplot2' plotting
  #-----------------------------------------------------------------------------
  } else {
    if (isTRUE(smooth)) {
      p <- ggplot2::ggplot(data=rects, ggplot2::aes(x=k, y=norm, group=get(sID))) +
        ggplot2::stat_smooth(ggplot2::aes(col=get(gID), group=get(gID)))
    } else {
      p <- ggplot2::ggplot(data=rects, ggplot2::aes(x=k, y=norm)) +
        ggplot2::geom_line(ggplot2::aes(col=get(gID)), size=1.25) +
        ggplot2::geom_rect(data=rects,
                  ggplot2::aes(x=NULL, y=NULL, xmin=xstart, xmax=xend, ymin=-Inf, ymax=Inf),
                  alpha=0.02, fill='lightpink')
    }
    p <- p +
      ggplot2::geom_hline(yintercept=1, size=0.5, lty=2) +
      ggplot2::geom_text(ggplot2::aes(y=yloc, col=get(gID), label=star), size=6, show.legend=FALSE) +
      ggplot2::facet_wrap(as.formula(paste('~', facet.by)), scales='free') +
      ggplot2::labs(x='Degree (k)', y=expression(phi[norm])) +
      ggplot2::theme(legend.position='bottom')
  }

  return(p)
}

#' Create connection matrices for tractography or fMRI data
#'
#' \code{create_mats} creates arrays from connection matrices (e.g.,
#' \emph{fdt_network_matrix} from FSL or \emph{ROICorrelation.txt} from DPABI).
#' You may choose to normalize these matrices by the \emph{waytotal} or
#' \emph{region size} (tractography), or not at all.
#'
#' @section Connection matrix files:
#' The \code{A.files} argument is mandatory and may be specified in a few ways:
#' \enumerate{
#'   \item A character vector of the filenames (preferably with full path).
#'   \item A single character string specifying the \emph{directory} in which
#'     all connectivity matrices are located. This will load \emph{all} files in
#'     the directory.
#'   \item A \emph{named list} in which the names match the arguments to
#'     \code{\link{list.files}}. This will load \emph{all} files in \code{path}
#'     that match the \code{pattern} argument, if present, and will load
#'     \emph{all} files in child directories if \code{recursive=TRUE}. See
#'     examples below.
#' }
#' The same options apply to \code{div.files} as well.
#'
#' @section Thresholding methods:
#' The argument \code{threshold.by} has 5 options:
#' \enumerate{
#'   \item \code{consensus} Threshold based on the raw (normalized, if selected)
#'     values in the matrices. If this is selected, it uses the
#'     \code{sub.thresh} value to perform \dQuote{consensus} thresholding.
#'   \item \code{density} Threshold the matrices to yield a specific graph
#'     density (given by the \code{mat.thresh} argument).
#'   \item \code{mean} Keep only connections for which the cross-subject mean is
#'     at least 2 standard deviations higher than the threshold (specified by
#'     \code{mat.thresh})
#'   \item \code{consistency} Threshold based on the coefficient of variation to
#'     yield a graph with a specific density (given by \code{mat.thresh}). The
#'     edge weights will still represent those of the input matrices. See
#'     Roberts et al. (2017) for more on \dQuote{consistency-based}
#'     thresholding.
#'   \item \code{raw} Threshold each subject's matrix \emph{individually},
#'     irrespective of group membership. Ignores \code{sub.thresh}.
#' }
#'
#' The argument \code{mat.thresh} allows you to choose a numeric threshold,
#' below which the connections will be replaced with 0; this argument will also
#' accept a numeric vector. The argument \code{sub.thresh} will keep only those
#' connections for which at least \emph{X}\% of subjects have a positive entry
#' (the default is 0.5, or 50\%).
#'
#' @param A.files Character vector of the filenames with connection matrices
#' @param modality Character string indicating data modality (default:
#'   \code{dti})
#' @param divisor Character string indicating how to normalize the connection
#'   matrices; either 'none' (default), 'waytotal', 'size', or 'rowSums'
#'   (ignored if \code{modality} equals \code{fmri})
#' @param div.files Character vector of the filenames with the data to
#'   normalize by (e.g. a list of \emph{waytotal} files) (default: \code{NULL})
#' @param threshold.by Character string indicating how to threshold the data;
#'   choose \code{density}, \code{mean}, or \code{consistency} if you want all
#'   resulting matrices to have the same densities (default: \code{consensus})
#' @param mat.thresh Numeric (vector) for thresholding connection matrices
#'   (default: 0)
#' @param sub.thresh Numeric (between 0 and 1) for thresholding by subject
#'   numbers (default: 0.5)
#' @param inds List (length equal to number of groups) of integers; each list
#'   element should be a vector of length equal to the group sizes
#' @param algo Character string of the tractography algorithm used (default:
#'   \code{'probabilistic'}). Ignored if \emph{modality} is \code{fmri}.
#' @param P Integer; number of samples per seed voxel (default: 5000)
#' @param ... Arguments passed to \code{\link{symmetrize}}
#' @export
#' @importFrom abind abind
#'
#' @return A list containing:
#' \item{A}{A 3-d array of the raw connection matrices}
#' \item{A.norm}{A 3-d array of the normalized connection matrices}
#' \item{A.bin}{A list of 3-d arrays of binarized connection matrices, one array
#'   for each threshold}
#' \item{A.bin.sums}{A list of 3-d arrays of connection matrices, with each
#'   entry signifying the number of subjects with a connection present; the
#'   number of list elements equals the length of \code{mat.thresh}, and the
#'   extent of the arrays equals the number of groups}
#' \item{A.inds}{A list of arrays of binarized connection matrices, containing 1
#'   if that entry is to be included}
#' \item{A.norm.sub}{List of 3-d arrays of the normalized connection matrices
#'   for all given thresholds}
#' \item{A.norm.mean}{List of 3-d arrays of connection matrices averaged for
#'   each group}
#'
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}
#' @references Roberts, JA and Perry, A and Roberts, G and Mitchell, PB and
#'   Breakspear, M (2017) Consistency-based thresholding of the human
#'   connectome. \emph{NeuroImage}. \bold{145}, 118--129.
#'   \doi{10.1016/j.neuroimage.2016.09.053}
#' @examples
#' \dontrun{
#' thresholds <- seq(from=0.001, to=0.01, by=0.001)
#' fmri.mats <- create_mats(f.A, modality='fmri', threshold.by='consensus',
#'   mat.thresh=thresholds, sub.thresh=0.5, inds=inds)
#' dti.mats <- create_mats(f.A, divisor='waytotal', div.files=f.way,
#'   mat.thresh=thresholds, sub.thresh=0.5, inds=inds)
#'
#' # Specify a directory and filename pattern
#' conn_files <- list(path='~/data', pattern='.*fdt_network_matrix')
#' dti.mats <- create_mats(conn_files, ...)
#' }

create_mats <- function(A.files, modality=c('dti', 'fmri'),
                        divisor=c('none', 'waytotal', 'size', 'rowSums'),
                        div.files=NULL,
                        threshold.by=c('consensus', 'density', 'mean', 'consistency', 'raw'),
                        mat.thresh=0, sub.thresh=0.5, inds=list(seq_along(A.files)),
                        algo=c('probabilistic', 'deterministic'), P=5e3, ...) {

  # Argument checking
  #-----------------------------------------------------------------------------
  kNumSubjs <- lengths(inds)
  nz <- sum(kNumSubjs)
  A.files <- dir2files(A.files)
  stopifnot(all(file.exists(A.files)),
            length(A.files) == nz,
            sub.thresh >= 0 && sub.thresh <= 1)
  if (!is.null(div.files)) {
    div.files <- dir2files(div.files)
    stopifnot(length(div.files) == length(A.files))
  }

  A <- read.array(A.files)
  Nv <- dim(A)[1L]
  emax <- Nv * (Nv - 1) / 2
  A[is.nan(A)] <- 0
  A.norm <- A

  modality <- match.arg(modality)
  algo <- match.arg(algo)
  divisor <- match.arg(divisor)

  # Normalize DTI matrices
  #-------------------------------------
  if (modality == 'dti' && algo == 'probabilistic') {
    A.norm <- normalize_mats(A.norm, divisor, div.files, P)
    A.norm[is.nan(A.norm)] <- 0
  }

  # Calculate the thresholded matrices
  #-------------------------------------
  threshold.by <- match.arg(threshold.by)
  mats <- switch(threshold.by,
                 raw=threshold_raw(A.norm, mat.thresh, ...),
                 mean=threshold_mean_sd(A.norm, mat.thresh, ...),
                 density=threshold_density(A.norm, mat.thresh, ...),
                 consistency=threshold_consistency(A.norm, mat.thresh, ...),
                 consensus=threshold_consensus(A.norm, mat.thresh, sub.thresh, inds,
                                               modality, algo, divisor, div.files, ...))

  A.norm.mean <- lapply(seq_along(mat.thresh), function(x)
                        abind(lapply(inds, function(y)
                                     rowMeans(mats$A.norm.sub[[x]][, , y], dims=2L)), along=3L))
  if (threshold.by == 'density') {
    A.norm.mean <- lapply(seq_along(mat.thresh), function(x)
                          array(apply(A.norm.mean[[x]], 3L, function(y)
                                      y * (y > get_thresholds(y, mat.thresh[x], emax))),
                                dim=dim(A.norm.mean[[x]])))
  }

  c(list(A=A), mats, list(A.norm.mean=A.norm.mean))
}

#-------------------------------------------------------------------------------
# Thresholding helper functions
#-------------------------------------------------------------------------------

#' Raw thresholding of a 3D array
#'
#' \code{threshold_raw} applies the input thresholds directly to the array.
#'
#' @return A list with elements
#'   \item{A.inds}{List of (binary) matrices with entries equal to 1 if the
#'     values in the input array (at that row-column) should be kept}
#'   \item{A.norm.sub}{List of 3D arrays of the input array with entries
#'     \dQuote{zeroed out} if they exceed the given threshold values. The list
#'     length will equal the length of \code{mat.thresh}}
#' @noRd

threshold_raw <- function(A, mat.thresh, ...) {
  A.norm.sub <- vector('list', length(mat.thresh))
  for (x in seq_along(mat.thresh)) {
    A.norm.sub[[x]] <- A * (A > mat.thresh[x])
    A.norm.sub[[x]] <- symmetrize(A.norm.sub[[x]], ...)
  }
  list(A.norm=A, A.inds=NULL, A.norm.sub=A.norm.sub)
}

#' Threshold by the mean + 2SD value (across all subjects)
#'
#' This calculates the mean (across subjects) and adds 2 standard deviations of
#' a 3D array to be used for thresholding/filtering of connectivity matrices.
#'
#' @noRd

threshold_mean_sd <- function(A, mat.thresh, ...) {
  A.norm.sub <- vector('list', length(mat.thresh))
  # Threshold: mean + 2SD > mat.thresh
  dimA <- dim(A)
  all.mean <- rowMeans(A, dims=2L)
  all.means <- array(all.mean, dim=dimA)
  all.sums <- rowSums((A - all.means)^2, dims=2L)
  all.sd <- sqrt(all.sums / (dimA[3L] - 1L))
  all.thresh <- array(all.mean + (2 * all.sd), dim=dimA)
  for (x in seq_along(mat.thresh)) {
    A.norm.sub[[x]] <- A * (all.thresh > mat.thresh[x])
    A.norm.sub[[x]] <- symmetrize(A.norm.sub[[x]], ...)
  }
  list(A.norm=A, A.inds=NULL, A.norm.sub=A.norm.sub)
}

#' Apply thresholds that will result in the given densities for all subjects
#'
#' @noRd

threshold_density <- function(A, mat.thresh, ...) {
  stopifnot(all(mat.thresh >= 0), all(mat.thresh <= 1))
  A.norm.sub <- vector('list', length(mat.thresh))
  nz <- dim(A)[3L]
  Asym <- symmetrize(A, ...)
  thrMat <- vapply(mat.thresh, function(x) apply(Asym, 3L, get_thresholds, x), numeric(nz))
  Aperm <- aperm(Asym, c(3L, 1L, 2L))
  for (x in seq_along(mat.thresh)) {
    A.norm.sub[[x]] <- Aperm * (Aperm > thrMat[, x])
    A.norm.sub[[x]] <- aperm(A.norm.sub[[x]], c(2L, 3L, 1L))
  }
  list(A.norm=A, A.inds=NULL, A.norm.sub=A.norm.sub)
}

#' Helper function to threshold based on consistency
#'
#' Creates thresholded 3D arrays using the \emph{coefficient of variation} and
#' user-defined thresholds.
#'
#' @noRd

threshold_consistency <- function(A, mat.thresh, ...) {
  stopifnot(all(mat.thresh >= 0), all(mat.thresh <= 1))
  A.norm.sub <- vector('list', length(mat.thresh))
  dimA <- dim(A)

  all.cv <- coeff_var(A)
  all.cv <- symmetrize(all.cv, 'min')
  threshes <- get_thresholds(all.cv, mat.thresh, decreasing=TRUE)
  A.inds <- lapply(threshes, function(x) ifelse(all.cv < x, 1L, 0L))
  for (z in seq_along(A.inds)) {
    tmp <- array(A.inds[[z]], dim=dimA)
    A.norm.sub[[z]] <- A * (tmp == 1L)
    A.norm.sub[[z]] <- symmetrize(A.norm.sub[[z]], ...)
  }
  list(A.norm=A, A.inds=A.inds, A.norm.sub=A.norm.sub)
}

#' Threshold based on group consensus
#'
#' Threshold data based on group \dQuote{consensus}; i.e., given a threshold for
#' the number/percent of subjects (\code{sub.thresh}) who exceed the
#' \code{mat.thresh} values, then keep those entries and zero out the rest.
#' @noRd

threshold_consensus <- function(A, mat.thresh, sub.thresh, inds, modality,
                                algo, divisor, div.files, ...) {
  A.bin <- A.bin.sums <- A.inds <- A.norm.sub <- vector('list', length(mat.thresh))
  # Binarize the array, then keep entries w/ >= "sub.thresh"% for each group
  for (x in seq_along(mat.thresh)) {
    A.bin[[x]] <- (A > mat.thresh[x]) + 0L
    A.bin.sums[[x]] <- lapply(inds, function(z)
                              rowSums(A.bin[[x]][, , z], dims=2L))
    A.bin.sums[[x]] <- abind(A.bin.sums[[x]], along=3L)
  }

  # For deterministic, threshold by size *after* binarizing
  if (modality == 'dti' && algo == 'deterministic' && divisor == 'size') {
    A <- normalize_mats(A, divisor, div.files, P=1)
  }

  # This is a list (# mat.thresh) of Nv x Nv x Ng arrays
  kNumSubjs <- lengths(inds)
  if (sub.thresh == 0) {
    A.inds <- lapply(seq_along(mat.thresh), function(x)
                     ifelse(A.bin.sums[[x]] > 0, 1L, 0L))
  } else {
    thresh <- sub.thresh * kNumSubjs
    for (x in seq_along(mat.thresh)) {
      tmp <- aperm(A.bin.sums[[x]], c(3L, 1L, 2L))
      A.inds[[x]] <- array(0L, dim=dim(tmp))
      A.inds[[x]][tmp >= thresh] <- 1L
      A.inds[[x]] <- aperm(A.inds[[x]], c(2L, 3L, 1L))
    }
  }

  # Back to a list of arrays for all subjects
  dimA <- dim(A); Nv <- dimA[1L]
  for (x in seq_along(mat.thresh)) {
    A.norm.sub[[x]] <- array(0, dim=dimA)
    keep <- array(FALSE, dim=dimA)
    for (y in seq_along(inds)) {
      tmp <- A.inds[[x]][, , y] == 1L
      keep[, , inds[[y]]] <- array(tmp, dim=c(Nv, Nv, kNumSubjs[y]))
    }
    A.norm.sub[[x]][keep] <- A[keep]
    A.norm.sub[[x]] <- symmetrize(A.norm.sub[[x]], ...)
  }
  list(A.norm=A, A.bin=A.bin, A.bin.sums=A.bin.sums, A.inds=A.inds, A.norm.sub=A.norm.sub)
}

#' Read in a list of text files as a numeric array
#'
#' \code{read.array} reads in a list of text files containing numeric matrices
#' and returns a numeric array. The order along the 3rd dimension will be the
#' same as the order of \code{infiles}.
#'
#' @param infiles Character vector of filenames
#' @param ncols Integer specifying the number of columns. By default, it is
#'   assumed the files contain \emph{square} matrices, but this behavior can be
#'   overridden through this argument
#' @noRd

read.array <- function(infiles, ncols=NULL) {
  Nv <- length(readLines(infiles[1L]))
  if (is.null(ncols)) ncols <- Nv
  array(sapply(infiles, function(x)
               matrix(scan(x, what=numeric(), n=Nv*ncols, quiet=TRUE),
                      Nv, ncols, byrow=TRUE)),
        dim=c(Nv, ncols, length(infiles)))
}

#' Normalize matrices in an array based on values in other matrices/vectors
#'
#' \code{normalize_mats} will normalize the matrices in \code{A} based on values
#' in separate matrices (found in the files specified by \code{div.files}) or
#' based on the matrix's row sums (if \code{divisor='rowSums'}).
#'
#' @param A Numeric array
#' @param divisor Character string. Either \code{rowSums}, \code{waytotal}, or
#'   \code{size}. If \code{none}, then it returns \code{A}.
#' @param div.files Character vector of filenames; required unless
#'   \code{divisor='rowSums'} or \code{divisor='none'}
#' @param P Integer representing the number of samples (in probabilistic
#'   tractography). Only used if \code{divisor='size'}.
#' @noRd

normalize_mats <- function(A, divisor, div.files, P) {
  dimA <- dim(A)
  if (divisor == 'none') {
    A.norm <- A
  } else if (divisor == 'rowSums') {
    A.norm <- array(apply(A, 3L, function(x) x / rowSums(x)), dim=dimA)

  } else {
    stopifnot(!is.null(div.files))
    div <- read.array(div.files, ncols=1L)

    if (divisor == 'waytotal') {
      # Control for streamline count by waytotal
      W <- array(apply(div, 3L, function(x) x[, rep.int(1L, dimA[1L])]), dim=dimA)
      A.norm <- A / W

    } else if (divisor == 'size') {
      # Control for the size (# voxels) of both regions 'x' and 'y'
      R <- array(apply(div, 3L, function(x) outer(x, x, FUN='+')), dim=dimA)
      A.norm <- 2 * A / (P * R)
    }
  }
  return(A.norm)
}

#' Threshold additional set of matrices
#'
#' \code{apply_thresholds} thresholds an additional set of matrices (e.g.,
#' FA-weighted matrices for DTI tractography) based on the matrices that have
#' been returned from \code{\link{create_mats}}. This ensures that the same
#' connections are present in both sets of matrices.
#'
#' The argument \code{W.files} accepts the same formats as \code{A.files}; see
#' \code{\link{create_mats}} for details.
#'
#' @param sub.mats List (length equal to number of thresholds) of numeric arrays
#'   (3-dim) for all subjects
#' @param group.mats List (length equal to number of thresholds) of numeric
#'   arrays (3-dim) for group-level data
#' @param W.files Character vector of the filenames of the files with
#'   connectivity matrices
#' @inheritParams create_mats
#' @export
#'
#' @return List containing:
#' \item{W}{A 3-d array of the raw connection matrices}
#' \item{W.norm.sub}{List of 3-d arrays of the normalized connection matrices
#'   for all given thresholds}
#' \item{W.norm.mean}{List of 3-d arrays of the normalized connection matrices
#'   averaged for each group}
#'
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}
#' @examples
#' \dontrun{
#'   W.mats <- apply_thresholds(A.norm.sub, A.norm.mean, f.W, inds)
#' }

apply_thresholds <- function(sub.mats, group.mats, W.files, inds) {
  if (is.list(W.files) || length(W.files) == 1L) W.files <- dir2files(W.files)
  dimA <- dim(sub.mats[[1L]])
  stopifnot(length(W.files) == dimA[3L])
  W <- read.array(W.files)
  W.norm.sub <- W.norm.mean <- vector('list', length(sub.mats))
  for (x in seq_along(sub.mats)) {
    W.norm.sub[[x]] <- array(0, dim=dimA)
    keep <- sub.mats[[x]] > 0
    W.norm.sub[[x]][keep] <- W[keep]
  }
  ng <- dim(group.mats[[1L]])[3L]
  W.norm.mean <- lapply(seq_along(group.mats), function(x)
                        lapply(seq_len(ng), function(g)
                               ifelse(group.mats[[x]][, , g] > 0,
                                      rowMeans(W.norm.sub[[x]][, , inds[[g]]], dims=2L),
                                      0)))
  W.norm.mean <- lapply(W.norm.mean, function(x) abind(x, along=3L))
  return(list(W=W, W.norm.sub=W.norm.sub, W.norm.mean=W.norm.mean))
}

#' Return a vector of filenames based on a directory name or options list
#'
#' @param x Either a character string specifying a directory or a list of
#'   arguments to be passed to \code{\link{list.files}}
#' @keywords internal

dir2files <- function(x) {
  if (length(x) > 1L && !is.list(x)) return(x)  # Vector of filenames
  fargs <- formals(list.files)
  if (is.list(x)) {
    if (is.null(attr(x, 'names'))) names(x) <- names(fargs)[seq_along(x)]
    if (!hasName(x, 'full.names')) x <- c(x, full.names=TRUE)
  } else {
    x <- list(path=x, full.names=TRUE)
  }
  x <- c(x, fargs[setdiff(names(fargs), names(x))])
  stopifnot(file_test('-d', x$path))
  do.call(list.files, x)
}

#' Calculate a vertex's leverage centrality
#'
#' Calculates the leverage centrality of each vertex in a graph.
#'
#' The leverage centrality relates a vertex's degree with the degree of its
#' neighbors. The equation is:
#' \deqn{l_i = \frac{1}{k_i} \sum_{j \in N_i} \frac{k_i - k_j}{k_i + k_j}}
#' where \eqn{k_i} is the degree of the \eqn{i^{th}} vertex and \eqn{N_i} is the
#' set of neighbors of \emph{i}. This function replaces \emph{NaN} with
#' \emph{NA} (for functions that have the argument \emph{na.rm}).
#'
#' @inheritParams efficiency
#' @export
#'
#' @return A vector of the leverage centrality for all vertices.
#'
#' @family Centrality functions
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}
#' @references Joyce, K.E. and Laurienti P.J. and Burdette J.H. and Hayasaka S.
#' (2010) A new measure of centrality for brain networks. \emph{PLoS One},
#' \bold{5(8)}, e12200. \doi{10.1371/journal.pone.0012200}

centr_lev <- function(g, A=NULL) {
  stopifnot(is_igraph(g))

  if (is.null(A)) A <- as_adj(g, sparse=FALSE, names=FALSE)
  k <- colSums(A)
  lev.cent <- rep.int(NA, dim(A)[1L])
  # This is a tiny bit slower for larger graphs
#  for (i in which(k > 0)) {
#    lev.cent[i] <- sum((k[i] - k[A[i, ] == 1]) / (k[i] + k[A[i, ] == 1])) / k[i]
#  }
  dd <- lapply(seq_along(k), function(x) k[A[x, ] == 1])
  lev.cent[which(k > 0)] <- vapply(which(k > 0), function(x) sum((k[x] - dd[[x]]) / (k[x] + dd[[x]])) / k[x], numeric(1L))

  lev.cent[is.nan(lev.cent)] <- NA
  return(lev.cent)
}

#' Multi-threshold permutation correction
#'
#' Applies the \emph{multi-threshold permutation correction (MTPC)} method to
#' perform inference in graph theory analyses of brain MRI data.
#'
#' This is a multi-step procedure: (steps 3-4 are the time-consuming steps)
#' \enumerate{
#'   \item Apply thresholds \eqn{\tau} to the networks, and compute network
#'     metrics for all networks and thresholds. (already done beforehand)
#'   \item Compute test statistics \eqn{S_{obs}} for each threshold. (done by
#'     \code{\link{brainGraph_GLM}})
#'   \item Permute group assignments and compute test statistics for each
#'     permutation and threshold. (done by \code{\link{brainGraph_GLM}})
#'   \item Build a null distribution of the maximum statistic across thresholds
#'     (and across brain regions) for each permutation. (done by
#'     \code{\link{brainGraph_GLM}})
#'   \item Determine the critical value, \eqn{S_{crit}} from the null
#'     distribution of maximum statistics.
#'   \item Identify clusters where \eqn{S_{obs} > S_{crit}} and compute the AUC
#'     for these clusters (denoted \eqn{A_{MTPC}}).
#'   \item Compute a critical AUC (\eqn{A_{crit}}) from the mean of the
#'     supra-critical AUC's for the permuted tests.
#'   \item Reject \eqn{H_0} if \eqn{A_{MTPC} > A_{crit}}.
#' }
#'
#' @param g.list A list of \code{brainGraphList} objects for all thresholds
#' @param thresholds Numeric vector of the thresholds applied to the raw
#'   connectivity matrices.
#' @param clust.size Integer indicating the size of \dQuote{clusters} (i.e.,
#'   consecutive thresholds for which the observed statistic exceeds the null)
#'   (default: \code{3L})
#' @param res.glm A list of \code{bg_GLM} objects, as output by a previous run
#'   of \code{mtpc}. Useful if you want to change the cluster size without
#'   re-running all of the GLM's and permutations (default: \code{NULL})
#' @param ... Other arguments passed to \code{\link{brainGraph_GLM}} and/or
#'   \code{\link{brainGraph_GLM_design}}
#' @inheritParams GLM
#' @export
#' @importFrom permute shuffleSet
#'
#' @return An object of class \code{mtpc} with some input arguments plus the
#'   following elements:
#'   \item{X,qr,cov.unscaled}{Design matrix, QR decomposition, and unscaled
#'     covariance matrix, if the design is the same across thresholds}
#'   \item{contrasts}{The contrast matrix or list of matrices}
#'   \item{con.name}{Contrast names}
#'   \item{removed.subs}{Named integer vector of subjects with incomplete data}
#'   \item{atlas}{The atlas of the input graphs}
#'   \item{rank,df.residual}{The model rank and residual degrees of freedom}
#'   \item{res.glm}{List with length equal to the number of thresholds; each
#'     list element is the output from \code{\link{brainGraph_GLM}}}
#'   \item{DT}{A \code{data.table} for all thresholds, combined from the outputs
#'     of \code{\link{brainGraph_GLM}}}
#'   \item{stats}{A data.table containing \code{S.mtpc} (the max. observed
#'     statistic), \code{tau.mtpc} (the threshold of the max. observed
#'     statistic), \code{S.crit} (the critical statistic value), and
#'     \code{A.crit} (the critical AUC)}
#'   \item{null.dist}{Numeric array with \code{N} columns and number of rows
#'     equal to the number of thresholds. The 3rd dimension is for each
#'     contrast. Each element of the array is the maximum statistic
#'     for that permutation, threshold, and contrast combination.}
#'   \item{perm.order}{Numeric matrix; the permutation set applied for all
#'     thresholds (each row is a separate permutation)}
#'
#' @family Group analysis functions
#' @family GLM functions
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}
#' @references Drakesmith, M. and Caeyenberghs, K. and Dutt, A. and Lewis, G. and
#'   David, A.S. and Jones, D.K. (2015) Overcoming the effects of false
#'   positives and threshold bias in graph theoretical analyses of neuroimaging
#'   data. \emph{NeuroImage}, \bold{118}, 313--333.
#'   \doi{10.1016/j.neuroimage.2015.05.011}
#' @examples
#' \dontrun{
#' diffs.mtpc <- mtpc(g.list=g.norm, thresholds=thresholds, N=N,
#'      covars=covars.dti, measure='E.nodal.wt', coding='effects',
#'      contrasts=c(0, 0, 0, 0, -2), alt='greater',
#'      binarize=c('Sex', 'Scanner'), con.name='Group 1 > Group 2')
#' sig.regions <- diffs.mtpc$DT[A.mtpc > A.crit]
#' }

mtpc <- function(g.list, thresholds, covars, measure, contrasts, con.type=c('t', 'f'),
                 outcome=NULL, con.name=NULL, level=c('vertex', 'graph'),
                 clust.size=3L, perm.method=c('freedmanLane', 'terBraak', 'smith',
                                              'draperStoneman', 'manly', 'stillWhite'),
                 part.method=c('beckmann', 'guttman', 'ridgway'), N=500L, perms=NULL,
                 alpha=0.05, res.glm=NULL, long=TRUE, ...) {
  S.mtpc <- tau.mtpc <- A.crit <- A.mtpc <- Contrast <- S.crit <- region <- stat <-
    threshold <- values <- null.out <- NULL

  # Check if components are 'brainGraphList' objects
  matches <- vapply(g.list, is.brainGraphList, logical(1L))
  if (any(!matches)) stop("Input must be a list of 'brainGraphList' objects.")
  stopifnot(length(g.list) == length(thresholds))

  if (is.null(res.glm)) {
    # If 'outcome != measure' make sure the same perm order is used throughout
    if (is.null(perms)) perms <- shuffleSet(n=length(g.list[[1L]]$graphs), nset=N)
    if (!is.null(outcome) && (outcome != measure)) {
      n <- sum(complete.cases(covars))
      if (dim(perms)[1L] != n) perms <- shuffleSet(n=n, nset=N)
    }
    res.glm <- lapply(g.list, function(z)
      brainGraph_GLM(z, covars, measure, contrasts, con.type, outcome, con.name=con.name, N=N,
                     level=level, permute=TRUE, perm.method=perm.method, part.method=part.method,
                     perms=perms, alpha=alpha, long=TRUE, ...))
  }
  alt <- res.glm[[1L]]$alt
  for (i in seq_along(thresholds)) res.glm[[i]]$DT[, threshold := thresholds[i]]
  mtpc.all <- rbindlist(lapply(res.glm, function(x) x$DT))
  setkey(mtpc.all, Contrast, region)
  mtpc.all[, c('S.crit', 'A.mtpc', 'A.crit') := 0]

  conNames <- res.glm[[1L]]$con.name
  myMax <- switch(alt, two.sided=colMaxAbs, less=colMin, greater=colMax)
  mySort <- sortfun(alt)
  index <- floor((1 - alpha) * N) + 1L
  # Dimensions will be "thresholds X Nperms X contrasts"
  null.max.all <- aperm(sapply(res.glm, function(x) x$perm$null.max, simplify='array'),
                        c(3L, 1L, 2L))
  dimnames(null.max.all) <- list(NULL, NULL, conNames)
  null.max.perms <- apply(null.max.all, 3L, myMax)
  Scrit <- structure(apply(null.max.perms, 2L, mySort, index), names=conNames)
  for (x in conNames) mtpc.all[Contrast == x, S.crit := Scrit[x]]

  rlefun <- rle_comp(alt)
  rle_obs <- mtpc.all[, rlefun(stat, S.crit), by=key(mtpc.all)]
  crit.regions <- rle_obs[(values)][lengths >= clust.size, list(region=unique(region)), by=Contrast]
  setkeyv(crit.regions, key(mtpc.all))
  mtpc.all[crit.regions,
           A.mtpc := auc_rle(stat, S.crit, thresholds, alt, clust.size),
           by=key(mtpc.all)]

  null.crit <- setNames(vector('list', length(conNames)), conNames)
  Acrit <- setNames(rep.int(0, length(conNames)), conNames)
  for (x in conNames) {
    null.crit[[x]] <- which(apply(null.max.all[, , x], 2L, function(y)
                                  any(with(rlefun(y, Scrit[x]), values == TRUE & lengths >= clust.size))))
    if (length(null.crit) > 0L) {
      null.crits <- abind::adrop(null.max.all[, null.crit[[x]], x, drop=FALSE], drop=3L)
      Acrit[x] <- mean(apply(null.crits, 2L, auc_rle, Scrit[x], thresholds, alt, clust.size))
      mtpc.all[Contrast == x, A.crit := Acrit[x]]
    }
  }

  maxobsfun <- switch(alt,
                      two.sided=function(x) max(is.finite(abs(x)) * abs(x), na.rm=TRUE),
                      less=function(x) min(is.finite(x) * x, na.rm=TRUE),
                      greater=function(x) max(is.finite(x) * x, na.rm=TRUE))
  mtpc.stats <- mtpc.all[, list(S.mtpc=unique(maxobsfun(stat))), by=Contrast]
  mtpc.stats[, tau.mtpc := mtpc.all[mtpc.stats, .SD[S.mtpc == stat, threshold]]]
  mtpc.stats[, S.crit := Scrit]
  mtpc.stats[, A.crit := Acrit]
  setcolorder(mtpc.stats, c('Contrast', 'tau.mtpc', 'S.mtpc', 'S.crit', 'A.crit'))

  glm.attr <- res.glm[[1L]]
  #TODO: may have to remove also 'var.covar'
  #TODO: is it possible for 'removed.subs' and 'df.residual' to be different?
  glm.attr[c('y', 'DT.Xy', 'DT', 'permute', 'runX', 'runY', 'coefficients', 'residuals',
             'sigma', 'fitted.values', 'se', 'perm', 'perm.order')] <- NULL
  if (glm.attr$outcome != glm.attr$measure) glm.attr[c('X', 'qr', 'cov.unscaled')] <- NULL
  for (i in seq_along(thresholds)) {
    res.glm[[i]]$perm[c('null.dist', 'null.max')] <- NULL
    res.glm[[i]]$perm.order <- NULL
  }

  if (isTRUE(long)) null.out <- null.max.all
  out <- c(glm.attr, list(res.glm=res.glm, clust.size=clust.size, DT=mtpc.all, stats=mtpc.stats,
                          null.dist=null.out, perm.order=perms))
  class(out) <- c('mtpc', class(out))
  return(out)
}

#' Run length encoding of the comparison of 2 vectors
#'
#' Returns a function that will compare 2 vectors and return the run length
#' encoding (RLE) of the logical vector from that comparison.
#' @noRd

rle_comp <- function(alt) {
  switch(alt,
         two.sided=function(x, y) rle(abs(x) > abs(y)),
         less=function(x, y) rle(x < y),
         greater=function(x, y) rle(x > y))
}

#' Calculate the AUC for the observed T- or F-statistic
#'
#' @noRd

auc_rle <- function(t.stat, S.crit, thresholds, alt, clust.size) {
  inds <- get_rle_inds(clust.size, alt, t.stat, S.crit, thresholds)
  x <- Map(function(a, b) thresholds[a:b], inds$starts, inds$ends)
  y <- Map(function(a, b) t.stat[a:b], inds$starts, inds$ends)
  auc <- mapply(auc_diff, x, y)
  return(sum(abs(auc) * is.finite(auc), na.rm=TRUE))
}

#' Get the starting and ending points for significant runs
#'
#' Given a vector of observed statistics and a critical value, calculate the
#' \dQuote{runs} (using \code{\link{rle}}) based on comparing these values. Then
#' return the starting and ending points of the runs.
#' @noRd

get_rle_inds <- function(clust.size, alt, t.stat, S.crit, thresholds) {
  compfun <- rle_comp(alt)
  runs <- compfun(t.stat, S.crit)
  sigruns <- which(runs$values == TRUE & runs$lengths >= clust.size)
  ends <- cumsum(runs$lengths)
  starts <- c(1L, head(ends, -1L) + 1L)
#  ends <- runs.len.cumsum[myruns]
#  newindex <- ifelse(sigruns > 1L, sigruns - 1L, 0L)
#  if (0L %in% newindex) starts <- c(1L, starts)
  return(list(starts=starts[sigruns], ends=ends[sigruns]))
}

################################################################################
# S3 METHODS
################################################################################

#' Print a summary of MTPC results
#'
#' @param object,x A \code{mtpc} object
#' @inheritParams summary.bg_GLM
#' @export
#' @rdname mtpc

summary.mtpc <- function(object, contrast=NULL, digits=max(3L, getOption('digits') - 2L), print.head=TRUE, ...) {
  Contrast <- A.mtpc <- A.crit <- stat <- region <- S.mtpc <- NULL
  object$printCon <- contrast
  object$digits <- digits
  object$print.head <- print.head

  # Summary table
  whichmaxfun <- switch(object$alt, two.sided=function(x) which.max(abs(x)), less=which.min, greater=which.max)
  DT.sum <- object$DT[A.mtpc > A.crit, .SD[whichmaxfun(is.finite(stat) * stat)], by=list(Contrast, region)]
  DT.sum <- DT.sum[, c('region', 'stat', 'Contrast', 'threshold', 'S.crit', 'A.mtpc', 'A.crit'), with=FALSE]
  setcolorder(DT.sum, c('Contrast', 'region', 'threshold', 'stat', 'S.crit', 'A.mtpc', 'A.crit'))
  setnames(DT.sum, c('region', 'stat', 'threshold'), c('Region', 'S.mtpc', 'tau.mtpc'))
  setorder(DT.sum, Contrast, -S.mtpc)
  object$DT.sum <- DT.sum

  class(object) <- c('summary.mtpc', class(object))
  return(object)
}

#' @aliases summary.mtpc
#' @method print summary.mtpc
#' @export

print.summary.mtpc <- function(x, ...) {
  print_title_summary('MTPC results (', x$level, '-level)')

  print_model_summary(x)
  print_contrast_type_summary(x)
  print_subs_summary(x)

  cat('# of thresholds: ', length(x$res.glm), '\n')
  cat('Cluster size (across thresholds): ', x$clust.size, '\n')

  print_permutation_summary(x)
  message('\n', 'Statistics', '\n', rep.int('-', getOption('width') / 4))
  clp <- 100 * (1 - x$alpha)
  cat('tau.mtpc: threshold of the maximum observed statistic\n')
  cat('S.mtpc: maximum observed statistic\n')
  cat('S.crit: the critical', paste0('(', clp, 'th percentile)'), 'value of the null max. statistic\n')
  cat('A.crit: critical AUC from the supra-critical null AUCs\n\n')
  print(x$stats)
  cat('\n')

  # Print results for each contrast
  print_contrast_stats_summary(x)

  invisible(x)
}

#' Plot statistics from an MTPC analysis
#'
#' Plot the statistics from an MTPC analysis, along with the maximum permuted
#' statistics. The output is similar to Figure 11 in Drakesmith et al. (2015).
#'
#' @param only.sig.regions Logical indicating whether to plot only significant
#'   regions (default: \code{TRUE})
#' @param show.null Logical indicating whether to plot points of the maximum
#'   null statistics (per permutation)
#' @param caption.stats Logical indicating whether to print the MTPC statistics
#'   in the caption of the plot. Default: \code{FALSE}
#' @inheritParams plot.bg_GLM
#' @export
#' @rdname mtpc
#'
#' @return The \code{plot} method returns a \code{trellis} object or a list of
#'   \code{ggplot} objects
#' @examples
#' \dontrun{
#' mtpcPlots <- plot(mtpc.diffs)
#'
#' ## Arrange plots into 3x3 grids
#' ml <- marrangeGrob(mtpcPlots, nrow=3, ncol=3)
#' ggsave('mtpc.pdf', ml)
#' }

plot.mtpc <- function(x, contrast=1L, region=NULL, only.sig.regions=TRUE,
                      show.null=TRUE, caption.stats=FALSE, ...) {
  Contrast <- stat_ribbon <- stat <- threshold <- S.crit <- A.mtpc <- A.crit <- panel.num <-
    xstart <- starts <- xend <- NULL

  stopifnot(inherits(x, 'mtpc'))
  conNames <- x$con.name
  mycontrast <- conNames[contrast]
  DT <- x$DT[Contrast == mycontrast, !'p.fdr']
  DT[, stat_ribbon := stat]  # For filling in supra-threshold areas
  thresholds <- DT[, unique(threshold)]
  DT$nullthresh <- x$stats[Contrast == mycontrast, unique(S.crit)]
  whichmaxfun <- switch(x$alt, two.sided=function(y) which.max(abs(y)), less=which.min, greater=which.max)
  myMax <- switch(x$alt, two.sided=colMaxAbs, less=colMin, greater=colMax)
  thr <- apply(x$null.dist[, , mycontrast], 2L, whichmaxfun)
  thr.y <- myMax(x$null.dist[, , mycontrast])
  nullcoords <- data.table(threshold=thresholds[thr], y=thr.y)

  # 'ggplot2'; Local function to plot for a single region
  plot_single <- function(x, DT, nullcoords, show.null) {
    stat <- S.crit <- threshold <- stat_ribbon <- nullthresh <- y <- A.mtpc <- A.crit <- NULL

    myMax <- switch(x$alt, two.sided=function(y) max(abs(y)), less=min, greater=max)
    inds <- DT[, get_rle_inds(x$clust.size, x$alt, stat, S.crit, threshold)]
    n <- dim(inds)[1L]
    if (n == 0L) {
      DT[, stat_ribbon := NA]
    } else {
      if (n == 1L) {
        all.inds <- c(apply(inds, 1L, function(z) z[1L]:z[2L]))
      } else if (n > 1L) {
        all.inds <- c(unlist(apply(inds, 1L, function(z) z[1L]:z[2L])))
      } else {
        all.inds <- 1L:dim(DT)[1L]
      }
      DT[-all.inds, stat_ribbon := NA]
    }

    lineplot <- ggplot2::ggplot(data=DT, mapping=ggplot2::aes(x=threshold)) +
      ggplot2::geom_line(ggplot2::aes(y=stat), col='red4', size=1.25, na.rm=TRUE) +
      ggplot2::geom_hline(ggplot2::aes(yintercept=nullthresh), lty=2)
    if (n > 0L) {
      if (x$alt == 'less') {
        lineplot <- lineplot +
          ggplot2::geom_ribbon(ggplot2::aes(ymax=stat_ribbon, ymin=nullthresh), fill='red4', alpha=0.3)
      } else {
        lineplot <- lineplot +
          ggplot2::geom_ribbon(ggplot2::aes(ymin=stat_ribbon, ymax=nullthresh), fill='red4', alpha=0.3)
      }
      lineplot <- lineplot + ggplot2::geom_point(ggplot2::aes(y=stat_ribbon), col='red4', size=2, na.rm=TRUE)
    }

    if (isTRUE(all(diff(thresholds) < 0))) lineplot <- lineplot + ggplot2::scale_x_reverse()
    p.region <- if (x$level == 'graph') 'graph-level' else DT[, as.character(unique(region))]
    p.title <- paste0('Region: ', p.region)
    p.subtitle <- paste0('Outcome: ', x$outcome)

    if (isTRUE(show.null)) {
      lineplot <- lineplot + ggplot2::geom_point(data=nullcoords, ggplot2::aes(y=y), col='darkgreen', alpha=0.4, na.rm=TRUE)
    }
    if (isTRUE(caption.stats)) {
      Smtpc <- bquote('S'['mtpc']*' = '~ .(DT[, format(myMax(stat))]))
      Scrit <- bquote('S'['crit']*' = '~ .(DT[, format(unique(S.crit))]))
      Amtpc <- bquote('A'['mtpc']*' = '~ .(DT[, format(max(A.mtpc))]))
      Acrit <- bquote('A'['crit']*' = '~ .(DT[, format(unique(A.crit))]))
      statslabel <- bquote(atop(.(Scrit)~ ';'~ .(paste('\t'))~ .(Acrit),
                                .(Smtpc)~ ';'~ .(paste('\t'))~ .(Amtpc)))

      if (DT[, unique(A.mtpc) > unique(A.crit)]) statslabel <- bquote(.(statslabel)~ .(paste0('\t(p < ', x$alpha, ')')))
      lineplot <- lineplot +
        ggplot2::labs(caption=statslabel) +
        ggplot2::theme(plot.caption=ggplot2::element_text(hjust=0))
    }
    lineplot <- lineplot +
      ggplot2::labs(title=p.title, subtitle=p.subtitle, x='Threshold', y=paste0(toupper(x$con.type), '-statistic')) +
      ggplot2::theme(plot.title=ggplot2::element_text(hjust=0.5, face='bold'),
            plot.subtitle=ggplot2::element_text(hjust=0.5))

    return(lineplot)
  }

  # 'base' plotting
  #-------------------------------------------------------
  if (!requireNamespace('ggplot2', quietly=TRUE)) {
    if (x$level == 'vertex') {
      if (is.null(region)) {
        if (isFALSE(only.sig.regions)) {
          myregions <- DT[, levels(region)]
        } else {
          myregions <- droplevels(DT[A.mtpc > A.crit])[, levels(region)]
        }
      } else {
        myregions <- region
      }
    }

    DT <- droplevels(DT[region %in% myregions])
    DT[, panel.num := as.numeric(region)]
    inds <- droplevels(DT[, get_rle_inds(x$clust.size, x$alt, stat, S.crit, threshold), by=region])
    inds[, panel.num := as.numeric(region)]
    inds[, xstart := thresholds[starts]]
    inds[, xend := thresholds[ends]]
    if (isTRUE(show.null)) {
      panelfun <- function(x, y, ...) {
        panel.num <- starts <- nullthresh <- NULL
        panel.points(-nullcoords$threshold, nullcoords$y, pch=19, fill='darkgreen')
        for (i in seq_len(inds[panel.num == panel.number(), .N])) {
          xcoords <- -thresholds[inds[panel.num == panel.number()][i, seq(starts, ends)]]
          ycoords <- DT[panel.num == panel.number() & threshold %in% -xcoords, c(stat, nullthresh)]
          panel.polygon(x=c(xcoords, rev(xcoords)), y=ycoords, col='lightgrey', border=NULL)
        }
        panel.abline(h=DT[, nullthresh], lty=2)
        panel.xyplot(x, y, ..., col='red', lwd=2)
      }
    } else {
      panelfun <- function(x, y, ...) {
        panel.num <- starts <- nullthresh <- NULL
        for (i in seq_len(inds[panel.num == panel.number(), .N])) {
          xcoords <- thresholds[inds[panel.num == panel.number()][i, seq(starts, ends)]]
          ycoords <- DT[panel.num == panel.number() & threshold %in% xcoords, c(stat, nullthresh)]
          panel.polygon(x=c(xcoords, rev(xcoords)), y=ycoords, col='lightgrey', border=NULL)
        }
        panel.abline(h=DT[, nullthresh], lty=2)
        panel.xyplot(x, y, ..., col='red', lwd=2)
      }
    }

    lineplot <- xyplot(stat ~ -threshold | region, data=DT, type=c('l', 'p'), pch=19,
                       xlab='Threshold', ylab=paste0(toupper(x$con.type), '-statistic'),
                       main=paste0('Outcome: ', x$outcome),
                       scales=list(y=list(relation='free')),
                       panel=panelfun)
    return(lineplot)

  # 'ggplot2' plotting
  #-------------------------------------------------------
  } else {
    if (x$level == 'graph') {
      lineplot <- plot_single(x, DT, nullcoords, show.null)
      return(lineplot)
    } else if (x$level == 'vertex') {
      if (is.null(region)) {
        if (isFALSE(only.sig.regions)) {
          region <- DT[, levels(region)]
        } else {
          region <- droplevels(DT[A.mtpc > A.crit])[, levels(region)]
        }
      }
      lineplots <- setNames(vector('list', length(region)), region)
      for (z in region) {
        lineplots[[z]] <- plot_single(x, DT[region == z], nullcoords, show.null)
      }
      return(lineplots)
    }
  }
}

#' @export
#' @rdname mtpc
nobs.mtpc <- function(object, ...) nobs(object$res.glm[[1L]])

#' @export
#' @rdname mtpc
terms.mtpc <- function(x, ...) terms(x$res.glm[[1]])

#' @export
#' @rdname mtpc
formula.mtpc <- formula.bg_GLM

#' @export
#' @rdname mtpc
labels.mtpc <- labels.bg_GLM

#' @method case.names mtpc
#' @export
#' @rdname mtpc
case.names.mtpc <- function(object, ...) case.names(object$res.glm[[1]])

#' @method variable.names mtpc
#' @export
#' @rdname mtpc
variable.names.mtpc <- function(object, ...) variable.names(object$res.glm[[1]])

#' @method df.residual mtpc
#' @export
#' @rdname mtpc
df.residual.mtpc <- df.residual.bg_GLM

#' Create a data table with graph global and vertex measures
#'
#' \code{graph_attr_dt} is a helper function that takes a \code{brainGraphList}
#' or a list of graphs and creates a \code{data.table} of global measures for
#' each graph. Each row will be for a different graph.
#'
#' @param bg.list A \code{brainGraphList} object, or a list of graph objects
#' @export
#' @return A \code{data.table}
#'
#' @name Graph Data Tables
#' @rdname data_tables
#'
#' @seealso \code{\link[igraph]{graph_attr}, \link[igraph]{graph_attr_names}}

graph_attr_dt <- function(bg.list) {
  name <- NULL
  if (is.brainGraphList(bg.list)) {
    level <- bg.list$level
    bg.list <- bg.list$graphs
  } else {
    if (!inherits(bg.list, 'list')) bg.list <- list(bg.list)
    level <- 'subject'
  }
  N <- length(bg.list)
  inds <- which(vapply(graph_attr(bg.list[[1L]]), is.numeric, logical(1L)))
  g.attr.num <- names(inds)
  if ('subnet' %in% g.attr.num) g.attr.num <- setdiff(g.attr.num, 'subnet')
  dt.G <- as.data.table(vapply(g.attr.num, function(x)
                               vapply(bg.list, graph_attr, numeric(1L), x), numeric(N)))

  if (N == 1L) {
    dt.G <- as.data.table(t(dt.G))
    setnames(dt.G, g.attr.num)
  }

  g.attrs <- graph_attr_names(bg.list[[1L]])
  g.attrs.char <- c('name', 'atlas', 'modality', 'weighting', 'Group')
  for (x in g.attrs.char) {
    if (x %in% g.attrs) dt.G[, eval(x) := vapply(bg.list, graph_attr, character(1L), x)]
  }
  if (level == 'subject') {
    setnames(dt.G, 'name', getOption('bg.subject_id'))
  } else if (level == 'group') {
    dt.G[, name := NULL]
  }
  setnames(dt.G, 'Group', getOption('bg.group'))

  return(dt.G)
}

#' Create a data table with graph vertex measures
#'
#' \code{vertex_attr_dt} is a helper function that creates a \code{data.table}
#' in which each row is a vertex and each column is a different network measure
#' (degree, centrality, etc.).
#'
#' @inheritParams graph_attr_dt
#' @export
#'
#' @rdname data_tables
#'
#' @seealso \code{\link[igraph]{vertex_attr}, \link[igraph]{vertex_attr_names},
#' \link[igraph]{graph_from_data_frame}}

vertex_attr_dt <- function(bg.list) {
  name <- NULL
  if (is.brainGraphList(bg.list)) {
    level <- bg.list$level
    atlas <- bg.list$atlas
    bg.list <- bg.list$graphs
  } else {
    if (!inherits(bg.list, 'list')) bg.list <- list(bg.list)
    atlas <- bg.list[[1L]]$atlas
    level <- 'subject'
  }

  dt.V <- rbindlist(lapply(bg.list, as_data_frame, what='vertices'))
  cols.char <- names(which(vapply(vertex_attr(bg.list[[1L]]), is.character, logical(1L))))
  cols.keep <- c('name', 'lobe', 'hemi', 'class', 'network', 'area',
                 'Anatomy', 'gyrus', 'subregion', 'Yeo_7network', 'Yeo_17network')
  cols.rem <- setdiff(cols.char, cols.keep)
  cols.rem <- c(cols.rem, 'x', 'y', 'z', 'x.mni', 'y.mni', 'z.mni',
                'lobe.hemi', 'circle.layout', 'circle.layout.comm')
  dt.V[, eval(cols.rem) := NULL]
  setnames(dt.V, 'name', 'region')

  # Add some important graph attributes, as well
  Nv <- vcount(bg.list[[1L]])
  g.attrs <- graph_attr_names(bg.list[[1L]])
  g.attrs.char <- c('name', 'atlas', 'modality', 'weighting', 'Group', 'density', 'threshold')
  for (x in g.attrs.char) {
    if (x %in% g.attrs) dt.V[, eval(x) := rep(sapply(bg.list, graph_attr, x), each=Nv)]
  }
  if (level == 'subject') {
    setnames(dt.V, 'name', getOption('bg.subject_id'))
  } else if (level == 'group') {
    dt.V[, name := NULL]
  }
  setcolorder(dt.V,
              c('density', 'region', 'lobe', 'hemi',
                names(dt.V[, !c('density', 'region', 'lobe', 'hemi'), with=FALSE])))
  setnames(dt.V, 'Group', getOption('bg.group'))

  return(dt.V)
}

#' Matrix/array utility functions
#'
#' These functions are utility/helper functions when working with matrices or
#' arrays.
#'
#' @param x Numeric matrix or array (the latter, for \code{qr.array} and
#'   \code{symmetrize.array})
#' @param n,p Integer; the number of rows or rank (respectively) of
#'   the input matrix or QR decomposition
#' @param ... Arguments passed to either \code{\link{sort}} (for
#'   \code{get_thresholds}) or \code{\link{qr.default}} (for \code{qr.array}).
#'   For the former, this will typically only be \code{decreasing=TRUE}, if that
#'   is the desired behavior
#' @name Matrix utilities
#' @rdname matrix_utils
NULL

#' @export
#' @rdname matrix_utils

colMax <- function(x, n=dim(x)[1L]) {
  do.call(pmax.int, c(lapply(seq_len(n), function(i) x[i, ]), na.rm=TRUE))
}

#' @export
#' @rdname matrix_utils

colMaxAbs <- function(x, n=dim(x)[1L]) colMax(abs(x), n)

#' @export
#' @rdname matrix_utils

colMin <- function(x, n=dim(x)[1L]) {
  do.call(pmin.int, c(lapply(seq_len(n), function(i) x[i, ]), na.rm=TRUE))
}

#' Return the diagonal of a square matrix
#'
#' \code{diag_sq} is a pared-down version of \code{\link{diag}} for square
#' matrices. It does not return any dimnames, does not check if \code{x} is a
#' square matrix, and it cannot be used to \emph{create} a matrix with a given
#' value along the diagonal. Meant to be used in code that is called repeatedly
#' (thousands of times).
#'
#' @param inds Vector-based indices of the diagonal
#' @export
#' @return \code{diag_sq} returns an unnamed numeric vector with the values
#'   along the diagonal of the input matrix
#' @rdname matrix_utils

diag_sq <- function(x, n=dim(x)[1L], inds=1L + 0L:(n - 1L) * (n + 1L)) {
  x[inds]
}

#' Calculate thresholds
#'
#' \code{get_thresholds} calculates the threshold values that would result in a
#' specific graph density. These depend, necessarily on the values in the matrix
#' themselves.
#'
#' Given a vector of densities, \code{get_thresholds} returns the numeric values
#' that will result in graphs of the given densities after thresholding by those
#' values. In the \emph{Examples} section, the thresholds should result in
#' graphs with densities of \eqn{5, 15, \dots, 55} percent.
#'
#' @param densities Numeric vector of densities
#' @param emax Integer; the maximum number of edges
#' @return \code{get_thresholds} returns a numeric vector of the thresholds
#' @export
#' @rdname matrix_utils
#' @examples
#' x <- matrix(runif(25 * 25), 25, 25)
#' x <- symmetrize(x)
#' diag(x) <- 0
#' densities <- seq(0.05, 0.55, by=0.1)
#' threshes <- get_thresholds(x, densities)
#' ## Verify that the densities are correct
#' graphs <- lapply(threshes, function(th) {
#'   graph_from_adjacency_matrix(x * (x > th), mode='undirected',
#'                               diag=FALSE, weighted=TRUE)
#'   })
#' sapply(graphs, graph.density)

get_thresholds <- function(x, densities, emax=dim(x)[1L] * (dim(x)[1L] - 1L) / 2, ...) {
  stopifnot(all(densities >= 0) && all(densities <= 1))
  sort(x[lower.tri(x)], ...)[emax - densities * emax]
}

#' @return \code{is_binary} returns a logical of length 1
#' @export
#' @rdname matrix_utils

is_binary <- function(x) identical(sum(abs(x)) - sum(x == 1), 0)

#' Calculate Moore-Penrose pseudoinverse of a full rank matrix
#'
#' \code{pinv} calculates \eqn{M^{+} = (M^T M)^{-1} M^T} for full (column) rank
#' matrices. However, it does not verify the matrix's rank.
#'
#' @export
#' @return \code{pinv} returns the input matrix's pseudoinverse
#' @rdname inverse

pinv <- function(x) tcrossprod(inv(x), x)

#' Calculate the QR decomposition for each matrix in an array
#'
#' \code{qr.array} will calculate the QR decomposition for each matrix in a 3D
#' array.
#'
#' @export
#' @return \code{qr.array} returns a \emph{list} in which each element is the QR
#'   decomposition of each matrix along \code{x}'s 3rd dimension
#' @rdname matrix_utils

qr.array <- function(x, ...) apply(x, 3L, qr.default, ...)

#' Slightly faster calculation of the Q and R matrices
#'
#' \code{qr_Q2} and \code{qr_R2} are simplified versions of \code{\link{qr.Q}}
#' and \code{\link{qr.R}}.
#'
#' @param QR A \code{qr} object
#' @param y A numeric matrix with \code{1} along the diagonal, of the same size
#'   as the input matrix (i.e., \code{QR$qr})
#' @export
#' @rdname matrix_utils

qr_Q2 <- function(QR, y=diag(1, n, p), n=dim(QR$qr)[1L], p=QR$rank) {
  qr.qy(QR, y)
}

#' @export
#' @rdname matrix_utils

qr_R2 <- function(QR, p=QR$rank) {
  R <- QR$qr[seq.int(p), , drop=FALSE]
  R[row(R) > col(R)] <- 0
  R
}

#' Symmetrize a matrix with the mean of off-diagonal elements
#'
#' \code{symm_mean} returns a symmetric matrix in which the off-diagonal
#' elements \eqn{A[i, j]} and \eqn{A[j, i]} are set to the mean of the values
#' in the input matrix.
#'
#' @export
#' @rdname matrix_utils

symm_mean <- function(x) {
  0.5 * (x + t(x))
}

#' Symmetrize matrices and arrays
#'
#' \code{symmetrize} will symmetrize a numeric matrix (or each matrix in an
#' array) by assigning to the off-diagonal elements either the \code{max}
#' (default), \code{min}, or \code{average} of \eqn{\{A(i, j), A(j, i)\}}.
#'
#' @param symm.by Character string; how to create symmetric off-diagonal
#'   elements. Default: \code{max}
#' @export
#' @rdname matrix_utils

symmetrize <- function(x, ...) {
  UseMethod('symmetrize')
}

#' @export
#' @rdname matrix_utils
symmetrize.matrix <- function(x, symm.by=c('max', 'min', 'avg'), ...) {
  dims <- dim(x)
  stopifnot(dims[1L] == dims[2L])

  symm.by <- match.arg(symm.by)
  switch(symm.by,
         avg=symm_mean(x),
         max=pmax(x, t(x)),
         min=pmin(x, t(x)))
}

#' @export
#' @rdname matrix_utils
symmetrize.array <- function(x, symm.by=c('max', 'min', 'avg'), ...) {
  dims <- dim(x)
  stopifnot(dims[1L] == dims[2L])
  symm.by <- match.arg(symm.by)
  array(apply(x, 3L, symmetrize, symm.by), dim=dims, dimnames=dimnames(x))
}

#' Linear model residuals in structural covariance networks
#'
#' \code{get.resid} runs linear models across brain regions listed in a
#' \code{data.table} (e.g., cortical thickness), adjusting for variables in
#' \code{covars} (e.g. age, sex, etc.), and calculates the
#' \emph{externally Studentized} (or \emph{leave-one-out}) residuals.
#'
#' You can choose to run models for each of your subject groups separately or
#' combined (the default) via the \code{method} argument. You may also choose
#' whether to include the mean, per-hemisphere structural measure in the
#' models. Finally, you can specify variables that are present in \code{covars}
#' which you would like to exclude from the models. Optional arguments can be
#' provided that get passed to \code{\link{brainGraph_GLM_design}}.
#'
#' If you do not explicitly specify the atlas name, then it will be guessed from
#' the size of your data. This could cause problems if you are using a custom
#' atlas, with or without the same number of regions as a dataset in the
#' package.
#'
#' @note It is assumed that \code{dt.vol} was created using
#' \code{\link{import_scn}}. In older versions, there were issues when the Study
#' ID was specified as an integer and was not \dQuote{zero-padded}. This is done
#' automatically by \code{\link{import_scn}}, so if you are using an external
#' program, please be sure that the Study ID column is matched in both
#' \code{dt.vol} and \code{covars}.
#'
#' @param dt.vol A \code{data.table} containing all the volumetric measure of
#'   interest (i.e., the object \code{lhrh} as output by
#'   \code{\link{import_scn}})
#' @param covars A \code{data.table} of the covariates of interest
#' @param method Character string indicating whether to test models for subject
#'   groups separately or combined. Default: \code{comb.groups}
#' @param use.mean Logical should we control for the mean hemispheric brain
#'   value (e.g. mean LH/RH cortical thickness). Default: \code{FALSE}
#' @param exclude.cov Character vector of covariates to exclude. Default:
#'   \code{NULL}
#' @param atlas Character string indicating the brain atlas
#' @param ... Arguments passed to \code{\link{brainGraph_GLM_design}} (optional)
#' @export
#'
#' @return \code{get.resid} - an object of class \code{brainGraph_resids} with
#'   elements:
#'   \item{data}{A data.table with the input volume/thickness/etc. data as well
#'     as the covariates used in creating the design matrix.}
#'   \item{X}{The \emph{design matrix}, if using default arguments. If
#'   \code{use.mean=TRUE} then it will be a \emph{named list} with a separate
#'   matrix for the left and right hemispheres. If \code{method='sep.groups'}, a
#'   nested named list for each group and hemisphere.}
#'   \item{method}{The input argument \code{method}}
#'   \item{use.mean}{The input argument \code{use.mean}}
#'   \item{resids.all}{The \dQuote{wide} \code{data.table} of residuals}
#'   \item{Group}{Group names}
#'   \item{atlas}{The atlas name}
#' @name Residuals
#' @rdname residuals
#' @seealso \code{\link[stats]{influence.measures}}, \code{\link[stats]{qqnorm}}
#' @family Structural covariance network functions
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}

get.resid <- function(dt.vol, covars, method=c('comb.groups', 'sep.groups'),
                      use.mean=FALSE, exclude.cov=NULL, atlas=NULL, ...) {
  mean.lh <- mean.rh <- NULL
  gID <- getOption('bg.group')
  stopifnot(hasName(covars, gID))
  grps <- covars[, levels(as.factor(get(gID)))]
  sID <- getOption('bg.subject_id')
  if (!hasName(covars, sID)) covars[, eval(sID) := seq_len(dim(covars)[1L])]
  covars[, eval(sID) := check_sID(get(sID))]
  DT.cov <- merge(covars, dt.vol, by=sID)
  idvars <- names(covars)

  method <- match.arg(method)
  xcols <- setdiff(names(covars), exclude.cov)
  if (method == 'sep.groups') {
    xcols <- setdiff(xcols, gID)
    X <- resids <- setNames(vector('list', length(grps)), grps)
  }
  if (isTRUE(use.mean)) {
    idvars <- c(idvars, 'mean.lh', 'mean.rh')
    if (any(grepl('(\\.|_)L[0-9]*', names(dt.vol)))) {
      if (!dim(dt.vol) %in% c(148L, 162L)) {  # Hack to exclude "destrieux" atlases
        lh <- '(\\.|_)L[0-9]*'
        rh <- '(\\.|_)R[0-9]*'
      } else {
        lh <- '^l'
        rh <- '^r'
      }
    } else {
      lh <- '^l'
      rh <- '^r'
    }
    DT.cov[, mean.lh := rowMeans(.SD), .SDcols=patterns(lh)]
    DT.cov[, mean.rh := rowMeans(.SD), .SDcols=patterns(rh)]

    if (method == 'comb.groups') {
      res <- by_hemi(DT.cov, lh, rh, xcols, ...)
      X <- res$X; resids <- res$resids
    } else {
      for (g in grps) {
        res <- by_hemi(DT.cov[get(gID) == g], lh, rh, xcols, ...)
        X[[g]] <- res$X; resids[[g]] <- res$resids
      }
      resids <- do.call(rbind, resids)
    }

  } else {
    ycols <- setdiff(names(dt.vol), sID)
    if (method == 'comb.groups') {
      Y <- as.matrix(DT.cov[, ycols, with=FALSE])
      dimnames(Y)[[1L]] <- DT.cov[, get(sID)]
      X <- brainGraph_GLM_design(DT.cov[, xcols, with=FALSE], ...)
      resids <- rstudent_mat(X, Y)
    } else {
      for (g in grps) {
        Y <- as.matrix(DT.cov[get(gID) == g, ycols, with=FALSE])
        dimnames(Y)[[1L]] <- DT.cov[get(gID) == g, get(sID)]
        X[[g]] <- brainGraph_GLM_design(DT.cov[get(gID) == g, xcols, with=FALSE], ...)
        resids[[g]] <- rstudent_mat(X[[g]], Y)
      }
      resids <- do.call(rbind, resids)
    }
  }

  # Return data to "wide" format with just the residuals
  resids.all <- as.data.table(resids, keep.rownames=sID)
  resids.all <- merge(DT.cov[, c(sID, gID), with=FALSE], resids.all, by=sID)
  setkeyv(resids.all, c(gID, sID))

  out <- list(data=DT.cov, X=X, method=method, use.mean=use.mean,
              resids.all=resids.all, Group=grps, atlas=NULL)
  out$atlas <- if (is.null(atlas)) guess_atlas(dt.vol) else atlas

  class(out) <- c('brainGraph_resids', class(out))
  return(out)
}

# Do the work when including "mean.{lh,rh}" in the models
by_hemi <- function(DT, lh, rh, xcols, ...) {
  X <- resids <- setNames(vector('list', 2L), c('lh', 'rh'))
  for (hemi in names(X)) {
    Y <- as.matrix(DT[, .SD, .SDcols=patterns(get(hemi))])
    dimnames(Y)[[1L]] <- DT[, get(getOption('bg.subject_id'))]
    X[[hemi]] <- brainGraph_GLM_design(DT[, c(xcols, paste0('mean.', hemi)), with=FALSE], ...)
    resids[[hemi]] <- rstudent_mat(X[[hemi]], Y)
  }
  resids <- do.call(cbind, resids)
  list(X=X, resids=resids)
}

#' Calculate studentized residuals with matrix input
#'
#' @inheritParams fastLmBG
#' @noRd

rstudent_mat <- function(X, Y) {
  fits <- fastLmBG(X, Y)
  res <- fits$residuals

  res2 <- res^2
  var.hat <- t(colSums(res2) - t(res2)) / (fits$df.residual - 1L)
  leverage <- colSums(tcrossprod(fits$cov.unscaled, X) * t(X))
  res / sqrt(var.hat * (1 - leverage))
}

#' Indexing for structural covariance residuals
#'
#' The \code{[} method reorders or subsets residuals based on a given
#' numeric vector. However, this is used in bootstrap and permutation analysis
#' and should generally not be called directly by the user.
#'
#' @param x,object A \code{brainGraph_resids} object
#' @param i Numeric vector of the indices
#' @param g Character string indicating the group. Default: \code{NULL}
#' @export
#'
#' @name Extract.brainGraph_resids
#' @rdname residuals

`[.brainGraph_resids` <- function(x, i, g=NULL) {
  stop_msg <- 'Logical indexing vector must be of the same length as the number of groups.'
  warn_msg <- 'Indexing vector cannot have length greater than the number of groups.'
  sID <- getOption('bg.subject_id')
  gID <- getOption('bg.group')
  if (!is.null(g)) {
    group.vec <- groups(x)
    grpNames <- unique(group.vec)
    kNumGroups <- length(grpNames)
    if (is.logical(g) && length(g) != kNumGroups) stop(stop_msg)
    if (length(g) > kNumGroups) {
      warning(warn_msg)
      g <- g[seq_len(kNumGroups)]
    }
    g <- switch(class(g),
                numeric=, integer=, logical=which(group.vec %in% grpNames[g]),
                factor=which(group.vec %in% as.character(g)),
                character=which(group.vec %in% g))
    x$resids.all <- droplevels(x$resids.all[g])
  }
  if (missing(i)) i <- seq_len(dim(x$resids.all)[1L])
  x$resids.all <- x$resids.all[i]
  x$Group <- x$resids.all[, levels(as.factor(get(gID)))]
  setkeyv(x$resids.all, c(gID, sID))
  x$data <- x$data[get(sID) %in% case.names(x)]
  return(x)
}

#' Print a summary of residuals for structural covariance data
#'
#' The \code{summary} method prints the number of outliers per region, and the
#' number of times a given subject was an outlier (i.e., across regions).
#'
#' @param region Character vector of region(s) to focus on; default behavior is
#'   to show summary for all regions
#' @param outlier.thresh Number indicating how many standard deviations
#'   above/below the mean indicate an outlier. Default: \code{2}
#' @export
#' @return \code{\link{summary.brainGraph_resids}} returns a list with two
#'   data tables, one of the residuals, and one of only the outlier regions
#' @rdname residuals

summary.brainGraph_resids <- function(object, region=NULL, outlier.thresh=2, ...) {
  Region <- resids <- mark <- NULL
  sID <- getOption('bg.subject_id')
  gID <- getOption('bg.group')
  if (is.null(region)) region <- region.names(object)
  DT <- melt(object$resids.all[, c(sID, gID, region), with=FALSE], id.vars=c(sID, gID),
             variable.name='Region', value.name='resids')
  setkey(DT, Region, resids)
  DT[, mark := 0]
  DT[abs(resids) > abs(mean(resids) + outlier.thresh * sd(resids)), mark := 1, by=Region]
  outliers <- DT[mark == 1, !'mark']
  outliers.reg <- outliers[, .N, by=Region]
  outliers.reg.vec <- structure(outliers.reg$N, names=as.character(outliers.reg$Region))

  outliers.sub <- outliers[, .N, by=sID]
  outliers.sub.vec <- structure(outliers.sub$N, names=as.character(outliers.sub[, get(sID)]))

  out <- list(Region=region, DT.sum=DT,
              outliers=list(DT=outliers, Region=outliers.reg.vec, subject=outliers.sub.vec))
  class(out) <- c('summary.brainGraph_resids', class(out))
  return(out)
}

#' @aliases summary.brainGraph_resids
#' @method print summary.brainGraph_resids
#' @export

print.summary.brainGraph_resids <- function(x, ...) {
  resids <- NULL
  print_title_summary('Structural covariance residuals')
  oldwidth <- getOption('width')
  dashes <- rep_len('-', oldwidth / 4)
  options(width=max(oldwidth / 2, 80L))

  if (length(x$Region) == 1L) {
    message('# of outliers for region ', x$Region, ': ', appendLF=FALSE)
    cat(unname(x$outliers$Region), '\n')
  } else {
    message('# of outliers per region: (sorted in descending order)\n', dashes)
    print(sort(x$outliers$Region, decreasing=TRUE))
  }
  cat('\n')

  if (length(x$Region) == 1L) {
    message('Subjects that are outliers:\n', dashes)
    print(sort(names(x$outliers$subject)))
  } else {
    message('Number of times each subject (top 10%) was an outlier: (sorted in descending order)\n', dashes)
    n <- length(x$outliers$subject)
    print(sort(x$outliers$subject, decreasing=TRUE)[seq_len(floor(n / 10L))])
  }
  cat('\n')

  message('Outliers\n', dashes)
  print(x$outliers$DT[order(resids)])
  options(width=oldwidth)
  invisible(x)
}

#' Plot model residuals for each brain region
#'
#' The \code{plot} method lets you check the model residuals for each brain
#' region in a structural covariance analysis. It shows a \emph{qqplot} of the
#' studentized residuals, as output from \code{\link{get.resid}}.
#'
#' @param cols Logical indicating whether to color by group. Default:
#'   \code{FALSE}
#' @inheritParams plot.bg_GLM
#' @export
#'
#' @return The \code{plot} method returns a \code{trellis} object or a list of
#'   \code{ggplot} objects
#'
#' @rdname residuals
#' @examples
#' \dontrun{
#' myresids <- get.resids(lhrh, covars)
#' residPlots <- plot(myresids, cols=TRUE)
#'
#' ## Save as a multi-page PDF
#' ml <- marrangeGrob(residPlots, nrow=3, ncol=3)
#' ggsave('residuals.pdf', ml)
#' }

plot.brainGraph_resids <- function(x, region=NULL, outlier.thresh=2, cols=FALSE, ids=TRUE, ...) {
  Region <- ind <- mark <- resids <- pch <- cex <- labpos <- NULL
  sID <- getOption('bg.subject_id')
  gID <- getOption('bg.group')
  DT <- summary(x, region=region, outlier.thresh=outlier.thresh)$DT.sum
  regions <- DT[, levels(Region)]
  setkeyv(DT, c(gID, sID))

  rnames <- if (isTRUE(ids)) case.names(x) else as.character(seq_len(nobs(x)))
  DT[, ind := rnames, by=Region]
  setkey(DT, Region, resids)
  DT[, x := qnorm(ppoints(resids)), by=Region]
  DT[mark == 0, ind := '']

  # 'base' plotting
  if (!requireNamespace('ggplot2', quietly=TRUE)) {
    xylabs <- paste(c('Theoretical', 'Sample'), 'Quantiles')

    # Set plotting parameters
    DT[, pch := 1]
    DT[, cex := 0.5]
    DT[mark == 1, pch := 17]
    DT[mark == 1, cex := 1]
    DT[, labpos := 4]
    DT[resids > 0, labpos := 2]

    if (isTRUE(cols)) {
      #TODO: save as object and control # of rows for panels
      DT[Region %in% regions,
         qqmath(~ resids | Region, xlab=xylabs[1L], ylab=xylabs[2L],
                as.table=TRUE, pch=pch, cex=cex, col=plot.cols[factor(get(gID))],
                panel=function(x, ..., subscripts) {
                  panel.qqmathline(x, col='black')
                  funArgs <- list(...)
                  pch <- if (hasArg('pch')) funArgs$pch[subscripts] else 16
                  cex <- if (hasArg('cex')) funArgs$cex[subscripts] else 0.5
                  col <- if (hasArg('col')) funArgs$col[subscripts] else 'black'
                  panel.qqmath(x, pch=pch, cex=cex, col=col, grid=TRUE)
                  panel.text(DT$x[subscripts], x, label=ind[subscripts], col='black', cex=0.7, pos=labpos[subscripts], offset=0.75)
                },
                par.settings=list(layout.heights=list(xlab.key.padding=1)),
                scales=list(x=list(tck=c(1, 0), alternating=FALSE)),
                key=list(title='Group', cex.title=1, space='bottom', columns=length(unique(get(gID))),
                         text=list(levels(factor(get(gID))), cex=0.9),
                         points=list(pch=16, col=plot.cols[1L:nlevels(factor(get(gID)))]))
                )]
    } else {
      DT[Region %in% regions,
         qqmath(~ resids | Region, xlab=xylabs[1L], ylab=xylabs[2L],
                panel=function(x, ...) {
                  panel.qqmathline(x, ...)
                  panel.qqmath(x, ...)
                })]
    }

  # 'ggplot2' plotting
  } else {
    DT[, mark := as.factor(mark)]

    textfun <- if (!requireNamespace('ggrepel', quietly=TRUE)) ggplot2::geom_text else ggrepel::geom_text_repel
    # Local function to plot for a single region
    plot_single <- function(DT.resids, cols) {
      Region <- resids <- NULL
      p <- ggplot2::ggplot(DT.resids, ggplot2::aes(x=x, y=resids, col=get(gID))) +
        textfun(ggplot2::aes(label=ind), size=3) +
        ggplot2::geom_point(ggplot2::aes(shape=mark, size=mark)) +
        ggplot2::geom_line(ggplot2::aes(x=x, y=x), col='gray50') +
        ggplot2::scale_shape_manual(values=c(20, 17)) +
        ggplot2::scale_size_manual(values=c(2, 3)) +
        ggplot2::theme(plot.title=ggplot2::element_text(hjust=0.5, size=10, face='bold'),
              legend.position='none',
              axis.text.y=ggplot2::element_text(hjust=0.5, angle=90)) +
        ggplot2::labs(title=paste0('Normal Q-Q: ', DT.resids[, unique(Region)]),
             x='Theoretical Quantiles', y='Sample Quantiles')
      if (isFALSE(cols)) {
        p <- p + ggplot2::scale_color_manual(values=rep.int('black', DT.resids[, length(unique(get(gID)))]))
      }
      return(p)
    }

    p.all <- setNames(vector('list', length(regions)), regions)
    for (z in regions) {
      p.all[[z]] <- plot_single(DT[Region == z], cols)
    }
    return(p.all)
  }
}

#' @export
#' @rdname residuals
nobs.brainGraph_resids <- function(object, ...) dim(object$resids.all)[1L]

#' @export
#' @method case.names brainGraph_resids
#' @rdname residuals
case.names.brainGraph_resids <- function(object, ...) {
  sID <- getOption('bg.subject_id')
  object$resids.all[, as.character(get(sID))]
}

# There is a lot of re-used code across these functions to avoid excessive
# branching when being repeatedly run in a loop (i.e., avoiding code such as
# "if (perm.method == ...) {FOO} else if (...) "). This is because, for a
# given function call, many of the inputs (in some cases, all) do not change and
# should not be checked in every permutation.

#===============================================================================
# Model fitting functions
#===============================================================================

#' Fit design matrices to one or multiple outcomes
#'
#' These are the \dQuote{base} model-fitting functions that solve the
#' \emph{least squares problem} to estimate model coefficients, residuals, etc.
#' for brain network data.
#'
#' @section Parameter estimation:
#' These functions use the \emph{QR} decomposition to calculate the least
#' squares solution which is the same as the base \code{\link[stats]{lm}}
#' function. If we substitute \eqn{X = QR} in the standard normal equations, the
#' equation to be solved reduces to
#' \deqn{X^T X \hat{\beta} = X^T y \Rightarrow R \hat{\beta} = Q^T y}
#'
#' Since \code{R} is an \emph{upper-triangular} matrix, we can use the
#' \code{\link{backsolve}} function which is a bit faster than
#' \code{\link{solve}}. In some cases, the \code{fastLmBG*} functions are about
#' as fast or faster (particularly when \code{X} is not permuted) as one in
#' which the normal equations are solved directly; additionally, using the
#' \emph{QR} method affords greater numerical stability.
#'
#' @section Different scenarios:
#' There are a few different scenarios for fitting models of the data, with a
#' separate function for each:
#' \describe{
#'   \item{fastLmBG}{The main function for when there is a single design matrix
#'     \eqn{X} and any number of outcome variables \eqn{Y}.}
#'   \item{fastLmBG_3d}{Fits models when there is a different design matrix
#'     \eqn{X} for each region and a single outcome variable \eqn{Y}, which in
#'     this case will be a column matrix.}
#'   \item{fastLmBG_3dY}{Fits models when there is both a different design
#'     matrix \eqn{X} and outcome variable \eqn{Y} for each region. Occurs under
#'     permutation for the Freedman-Lane, ter Braak, and Still-White methods.}
#'   \item{fastLmBG_3dY_1p}{Fits models when there is both a different design
#'     and outcome variable for each region, and also when \eqn{X} is a rank-1
#'     matrix (i.e., it has 1 column). Only occurs under permutation with the
#'     Still-White method if there is a single regressor of interest.}
#' }
#'
#' In the last case above, model coefficients are calculated by simple (i.e.,
#' non-matrix) algebra.
#'
#' @section Improving speed/efficiency:
#' Speed/efficiency gains will be vast for analyses in which there is a single
#' design matrix \eqn{X} for all regions, there are multiple outcome variables
#' (i.e., vertex-level analysis), and the permutation method chosen does
#' not permute \eqn{X}. Specifically, these are \emph{Freedman-Lane}, \emph{ter
#' Braak}, and \emph{Manly} methods. Therefore, the QR decomposition, the
#' \eqn{Q} and \eqn{R} matrices, and the \dQuote{unscaled covariance matrix}
#' (which is \eqn{(X^T X)^{-1}}) only need to be calculated once for the entire
#' analysis. Other functions (e.g., \code{lm.fit}) would recalculate these for
#' each permutation.
#'
#' Furthermore, this (and the other model fitting functions in the package) will
#' likely only work in models with full rank. I sacrifice proper error checking
#' in favor of speed, but hopefully any issues with the model will be identified
#' prior to the permutation step. Finally, the number of observations, model
#' rank, number of outcome variables, and degrees of freedom will not change and
#' therefore do not need to be recalculated (although these probably amount to a
#' negligible speed boost).
#'
#' In case there are multiple design matrices, or the permutation method
#' permutes the design, then the QR decomposition will need to be calculated
#' each time anyway. For these cases, I use more simplified functions
#' \code{qr_Q2} and \code{qr_R2} to calculate the \eqn{Q} and \eqn{R} matrices,
#' and then the fitted values, residuals, and residual standard deviation are
#' calculated at the same time (whereas \code{lm.fit} and others would calculate
#' these each time).
#'
#' @param X Design matrix or 3D array of design matrices
#' @param Y Numeric matrix; there should be 1 column for each outcome variable
#'   (so that in a graph-level analysis, this is a column matrix)
#' @param QR,Q,R The QR decomposition(s) and Q and R matrix(es) of the design
#'   matrix(es). If \code{X} is a 3D array, these should be \emph{lists}
#' @param n,p,ny,dfR Integers; the number of observations, model \emph{rank},
#'   number of regions/outcome variables, and residual degrees of freedom
#' @param XtXinv Numeric matrix or array; the inverse of the cross-product of
#'   the design matrix(es)
#' @export
#'
#' @return A list with elements
#'   \item{coefficients}{Parameter estimates}
#'   \item{rank}{Model rank}
#'   \item{df.residual}{Residual degrees of freedom}
#'   \item{residuals}{Model residuals}
#'   \item{sigma}{The residual standard deviation, or \emph{root mean square
#'     error (RMSE)}}
#'   \item{fitted.values}{Model fitted values}
#'   \item{qr}{The design matrix QR decomposition(s)}
#'   \item{cov.unscaled}{The \dQuote{unscaled covariance matrix}}
#' @name GLM fits
#' @rdname glm_fit
#' @family GLM functions
#' @seealso randomise
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}

fastLmBG <- function(X, Y, QR=qr.default(X), Q=qr_Q2(QR, n=n, p=p), R=qr_R2(QR, p),
                     n=dim(X)[1L], p=QR$rank, ny=dim(Y)[2L], dfR=n-p, XtXinv=inv(QR)) {

  beta <- backsolve(R, crossprod(Q, Y), p)
  yhat <- X %*% beta
  ehat <- Y - yhat
  s <- if (ny == 1L) sum(ehat^2) else .colSums(ehat^2, n, ny)

  list(coefficients=beta, rank=p, df.residual=dfR, residuals=ehat,
       sigma=sqrt(s / dfR), fitted.values=yhat, qr=QR, cov.unscaled=XtXinv)
}

#' @param runX Character vector of the regions for which the design matrix is
#'   not singular
#' @export
#' @rdname glm_fit

fastLmBG_3d <- function(X, Y, runX, QR=qr(X[, , runX, drop=FALSE]),
                        Q=lapply(QR, qr_Q2, n=n, p=p), R=lapply(QR, qr_R2, p),
                        n=dim(X)[1L], p=QR[[1L]]$rank, ny=length(runX), dfR=n-p,
                        XtXinv=inv(QR)) {

  beta <- vapply(runX, function(r) backsolve(R[[r]], crossprod(Q[[r]], Y), p), numeric(p))
  yhat <- vapply(runX, function(r) X[, , r] %*% beta[, r], numeric(n))
  ehat <- c(Y) - yhat
  s <- .colSums(ehat^2, n, ny) / dfR

  list(coefficients=beta, rank=p, df.residual=dfR, residuals=ehat,
       sigma=sqrt(s), fitted.values=yhat, qr=QR, cov.unscaled=XtXinv)
}

#' @export
#' @rdname glm_fit

fastLmBG_3dY <- function(X, Y, runX, QR=qr(X[, , runX, drop=FALSE]),
                         Q=lapply(QR, qr_Q2, n=n, p=p), R=lapply(QR, qr_R2, p),
                         n=dim(X)[1L], p=QR[[1L]]$rank, ny=length(runX), dfR=n-p,
                         XtXinv=inv(QR)) {

  beta <- vapply(runX, function(r) backsolve(R[[r]], crossprod(Q[[r]], Y[, r]), p), numeric(p))
  yhat <- vapply(runX, function(r) X[, , r] %*% beta[, r], numeric(n))
  ehat <- Y - yhat
  s <- .colSums(ehat^2, n, ny) / dfR

  list(coefficients=beta, rank=p, df.residual=dfR, residuals=ehat,
       sigma=sqrt(s), fitted.values=yhat, qr=QR, cov.unscaled=XtXinv)
}

#' @export
#' @rdname glm_fit

fastLmBG_3dY_1p <- function(X, Y, runX, QR=qr(X[, , runX, drop=FALSE]),
                            Q=lapply(QR, qr_Q2, diag(1L, n, 1L), n, 1L),
                            R=lapply(QR, function(r) r$qr[1L]), n=dim(X)[1L], p=1L,
                            ny=length(runX), dfR=n-1L, XtXinv=inv(QR)) {

  Qarr <- sapply(Q, function(x) x, simplify='array')
  beta <- colSums(Qarr[, 1L, ] * Y) / unlist(R, recursive=FALSE, use.names=FALSE)
  yhat <- X[, 1L, ] %*% diag(beta)
  ehat <- Y - yhat
  dim(beta) <- c(1L, ny)
  dimnames(beta)[[2L]] <- dimnames(yhat)[[2L]] <- dimnames(ehat)[[2L]] <- runX
  s <- .colSums(ehat^2, n, ny) / dfR

  list(coefficients=beta, rank=p, df.residual=dfR, residuals=ehat,
       sigma=sqrt(s), fitted.values=yhat, qr=QR, cov.unscaled=XtXinv)
}

#===============================================================================
# Contrast-based statistics
#===============================================================================

#' Calculate GLM statistics for T and F contrasts
#'
#' \code{fastLmBG_t} and \code{fastLmBG_f} calculate contrast-based statistics
#' for T or F contrasts, respectively. It accepts any number of \emph{contrasts}
#' (i.e., a multi-row contrast matrix).
#'
#' @section Contrast-based statistics:
#' The \emph{contrast of parameter estimates}, \eqn{\gamma}, for T contrasts is
#' \deqn{\gamma = C \hat{\beta}}
#' where \eqn{C} is the contrast matrix with size \eqn{k \times p} (where
#' \eqn{k} is the number of contrasts) and \eqn{\hat{\beta}} is the matrix of
#' parameter estimates with size \eqn{p \times r} (where \eqn{r} is the number
#' of regions). For F contrasts, the effect size is the \emph{extra sum of
#' squares} and is calculated as
#' \deqn{\gamma (C (X^T X)^{-1} C^T)^{-1} \gamma^T}
#' The \emph{standard error} of a T contrast is
#' \deqn{\sqrt{\hat{\sigma} (X^T X)^{-1}}}
#' where \eqn{\hat{\sigma}} is the \emph{residual standard deviation} of the
#' model and the second term is the unscaled covariance matrix. The standard
#' error for F contrasts is simply the \emph{residual sum of squares}. P-values
#' and FDR-adjusted P-values (across regions) are also calculated. Finally, if
#' \eqn{\alpha} is provided for T contrasts, confidence limits are calculated.
#'
#' @param fits List object output by one of the model fitting functions (e.g.,
#'   \code{fastLmBG})
#' @inheritParams GLM
#' @export
#' @return \code{fastLmBG_t} -- A multidimensional array with the third
#'   dimension equaling the number of contrasts; each matrix contains the
#'   contrast of parameter estimates, standard error of the contrast,
#'   T-statistics, P-values, FDR-adjusted P-values, and confidence intervals (if
#'   \code{alpha} is given)
#' @rdname glm_fit

fastLmBG_t <- function(fits, contrasts, alternative=c('two.sided', 'less', 'greater'), alpha=NULL) {
  is3d <- inherits(fits$qr, 'list')
  dfR <- fits$df.residual
  XtX <- fits$cov.unscaled

  gamma <- t(contrasts %*% fits$coefficients)

  if (is3d) {
    CXtX <- cxtxfun_3d('t', transpose=FALSE)(contrasts, xtx=XtX, rkC=qr.default(contrasts)$rank)
    se <- sqrt(fits$sigma^2 * t(CXtX))
  } else {
    CXtX <- contrasts %*% tcrossprod(XtX, contrasts)
    se <- sqrt(outer_vec(fits$sigma^2, diag(CXtX)))
  }

  stat <- gamma / se
  alt <- match.arg(alternative)
  pfun <- switch(alt,
                 two.sided=function(stat, df) 2 * pt(abs(stat), df, lower.tail=FALSE),
                 less=function(stat, df) pt(stat, df),
                 greater=function(stat, df) pt(stat, df, lower.tail=FALSE))
  p <- pfun(stat, dfR)
  pfdr <- apply(p, 2L, p.adjust, 'fdr')
  dim(pfdr) <- dim(p)

  res <- abind::abind(gamma=gamma, se=se, stat=stat, p=p, p.fdr=pfdr, along=1.5)
  if (!is.null(alpha)) {
    t.limits <- c(1, -1) * qt(alpha / 2, dfR)
    cilow <- gamma + t.limits[1L] * se
    cihigh <- gamma + t.limits[2L] * se
    res <- abind::abind(res, ci.low=cilow, ci.high=cihigh, along=2L)
  }
  res
}

#' @param rkC,nC Integers; the rank of the contrast matrix and number of
#'   contrasts, respectively (for F contrasts)
#' @export
#' @return \code{fastLmBG_f} -- A numeric matrix with columns for the effect
#'   size, standard error, F statistic, P-values, and FDR-adjusted P-values
#' @rdname glm_fit

fastLmBG_f <- function(fits, contrasts, rkC=NULL, nC=length(contrasts)) {
  is3d <- inherits(fits$qr, 'list')
  dfR <- fits$df.residual
  XtX <- fits$cov.unscaled
  if (is.null(rkC)) rkC <- vapply(contrasts, function(x) qr.default(x)$rank, integer(1L))

  # This will be a list of "nC" matrices w/ dims "nregions X rkC"
  gamma <- lapply(contrasts, function(x) t(x %*% fits$coefficients))

  if (is3d) {
    cxtxfun <- cxtxfun_3d('f')
    rgns <- dimnames(XtX)[[3L]]
    CXtX <- vector('list', nC)
    for (x in seq_len(nC)) {
      CXtX[[x]] <- cxtxfun(contrasts[[x]], XtX, rkC[x], dim(XtX)[3L])
      dimnames(CXtX[[x]])[[3L]] <- rgns
    }
    ESS <- mapply(function(x, y)
                  vapply(rgns, function(r) rowSums(x[r, ] %*% y[, , r] * x[r, ]), numeric(1L)),
                  gamma, CXtX)
  } else {
    CXtX <- lapply(contrasts, function(x) inv(x %*% XtX, x, transpose=TRUE))
    ESS <- mapply(function(x, y) rowSums(x %*% y * x), gamma, CXtX)
  }

  SSEF <- matrix(fits$sigma^2 * dfR, dim(ESS)[1L], nC)
  stat <- t(t(ESS) / rkC) / (SSEF / dfR)
  p <- t(pf(t(stat), rkC, dfR, lower.tail=FALSE))
  pfdr <- apply(p, 2L, p.adjust, 'fdr')
  dim(pfdr) <- dim(p)
  abind::abind(ESS=ESS, se=SSEF, stat=stat, p=p, p.fdr=pfdr, along=1.5)
}

#===============================================================================
# Statistics for "randomise" and "randomise_3d"
#===============================================================================
#-----------------------------------------------------------
# T statistic functions
#-----------------------------------------------------------

#' Calculate T and F statistics when called from randomise
#'
#' These functions only call \code{fastLmBG} (the main model-fitting functions)
#' and calculates the T- or F-statistics.
#'
#' @noRd

fastLmBG_t_rand <- function(X, Y, contrasts, QR, Q, R, n, p, ny, dfR, XtXinv, CXtX, rkC) {
  all_fits <- fastLmBG(X, Y, QR, Q, R, n, p, ny, dfR, XtXinv)
  gamma <- t(contrasts %*% all_fits$coefficients)
  se <- sqrt(tcrossprod(all_fits$sigma^2, CXtX))  # Only works if CXtX is the  diagonal
  gamma / se
}

#' @param fitfun Function that will calculate model coefficients, residuals,
#'   etc. Can be one of: \code{fastLmBG_3d}, \code{fastLmBG_3dY},
#'   \code{fastLmBG_3dY_1p}
#' @noRd

fastLmBG_t_rand_3d <- function(fitfun, X, Y, contrasts, runX, QR, Q, R, n, p, ny, dfR, XtXinv, CXtX, rkC) {
  all_fits <- fitfun(X, Y, runX, QR, Q, R, n, p, ny, dfR, XtXinv)
  gamma <- t(contrasts %*% all_fits$coefficients)
  se <- sqrt(all_fits$sigma^2 * t(CXtX))
  gamma / se
}

#-----------------------------------------------------------
# F statistic functions
#-----------------------------------------------------------

#' @noRd
fastLmBG_f_rand <- function(X, Y, contrast, QR, Q, R, n, p, ny, dfR, XtXinv, CXtX, rkC) {
  all_fits <- fastLmBG(X, Y, QR, Q, R, n, p, ny, dfR, XtXinv)
  gamma <- t(contrast %*% all_fits$coefficients)
  ESS <- rowSums(gamma %*% CXtX * gamma)
  (ESS / rkC) / all_fits$sigma^2
}

#' @noRd
fastLmBG_f_rand_3d <- function(fitfun, X, Y, contrasts, runX, QR, Q, R, n, p, ny, dfR, XtXinv, CXtX, rkC) {
  all_fits <- fitfun(X, Y, runX, QR, Q, R, n, p, ny, dfR, XtXinv)
  gamma <- t(contrasts %*% all_fits$coefficients)
  ESS <- vapply(seq_len(ny), function(r) rowSums(gamma[r, ] %*% CXtX[, , r] * gamma[r, ]), numeric(1L))
  (ESS / rkC) / all_fits$sigma^2
}

#-----------------------------------------------------------
# Helper functions
#-----------------------------------------------------------

#' Add rows/columns for regions not fitted
#'
#' \code{add_nans} adds rows/columns (or higher dimensions) to model fit data
#' for regions which were skipped (due to having a singular design matrix,
#' usually).
#'
#' @param fits List object output by one of the model fitting functions (e.g.,
#'   \code{\link{fastLmBG}})
#' @param dimX Integer vector containing the dimensions of the original design
#'   matrix/array (including singular designs)
#' @param namesX List of character vectors containing the dimension names from
#'   the original design matrix/array
#' @param runX Character vector of regions for which models were fit
#'
#' @return \code{add_nans} -- the original \code{fits} object with \code{NaN} or
#'   \code{NA} inserted
#' @keywords internal
#' @rdname glm_helpers

add_nans <- function(fits, dimX, namesX, runX=names(fits$qr)) {
  # Initialize the full matrices/arrays
  coeffs <- se <- matrix(NaN, fits$rank, dimX[3L], dimnames=namesX[c(2L, 3L)])
  resids <- fvals <- matrix(NaN, dimX[1L], dimX[3L], dimnames=namesX[c(1L, 3L)])
  sig <- setNames(rep_len(NaN, dimX[3L]), namesX[[3L]])
  QRx <- setNames(vector('list', dimX[3L]), namesX[[3L]])
  XtX <- vcv <- array(NaN, dim=dimX[c(2L, 2L, 3L)], dimnames=namesX[c(2L, 2L, 3L)])

  # Fill in and re-assign
  coeffs[, runX] <- fits$coefficients
  fits$coefficients <- coeffs
  resids[, runX] <- fits$residuals
  fits$residuals <- resids
  fvals[, runX] <- fits$fitted.values
  fits$fitted.values <- fvals
  names(fits$sigma) <- runX
  sig[runX] <- fits$sigma
  fits$sigma <- sig
  QRx[runX] <- fits$qr
  fits$qr <- QRx
  XtX[, , runX] <- fits$cov.unscaled
  fits$cov.unscaled <- XtX
  if (!is.null(fits$var.covar)) {
    vcv[, , runX] <- fits$var.covar
    fits$var.covar <- vcv
  }
  se[, runX] <- fits$se
  fits$se <- se

  fits
}

#' Return a function to calculate the CXtX matrix/array for multiple designs
#'
#' \code{cxtxfun_3d} returns a function that calculates the \dQuote{CXtX}
#' matrix/array, used to calculate the standard error of a contrast. The
#' function signature will be \code{f(contrast, xtx, rkC, ny)}.
#'
#' The \dQuote{CXtX} matrix/array for T-contrasts is the diagonal of
#' \deqn{C (X^T X)^{-1} C^T}
#' For F-contrasts, it is the \emph{inverse} of the matrix:
#' \deqn{(C (X^T X)^{-1} C^T)^{-1}}
#' where in both cases, \eqn{C} is the contrast matrix and \eqn{X} is the design
#' matrix/array.
#'
#' For T-contrasts, the function will return a numeric vector/matrix with
#' dimensions \eqn{k \times r} where \eqn{k} is the \emph{number} of contrasts
#' (i.e., the number of rows in the contrast matrix) and \eqn{r} is the number
#' of regions. For F-contrasts, the function would return a numeric array with
#' dimensions \eqn{k \times k \times r}, where \eqn{k} is the rank of the
#' contrast matrix and \eqn{r} is the number of regions. If there is a single
#' design for all regions, it will be a \eqn{k \times k} matrix.
#'
#' @param con.type Either \code{'t'} or \code{'f'}
#' @param transpose Logical indicating whether to transpose the output of the
#'   selected function. Ignored for F-contrasts. Should be \code{FALSE} for
#'   T-contrasts when the \dQuote{Guttman} partition method is used. Default:
#'   \code{TRUE}
#' @return \code{cxtxfun_3d} -- A function with arguments for the contrast
#'   (numeric matrix), the \dQuote{unscaled covariance} array, the (row) rank of
#'   the contrast, and the number of regions in the analysis (only used for
#'   F-contrasts)
#' @keywords internal
#' @rdname glm_helpers

cxtxfun_3d <- function(con.type, transpose=TRUE) {
    if (con.type == 't') {
      if (isTRUE(transpose)) {
        fun <- function(contrast, xtx, rkC, ny) {
          t(apply(xtx, 3L, function(r) diag_sq(tcrossprod(contrast %*% r, contrast), rkC)))
        }
      } else {
        fun <- function(contrast, xtx, rkC, ny) {
          apply(xtx, 3L, function(r) diag_sq(tcrossprod(contrast %*% r, contrast), rkC))
        }
      }

    # The F-contrast function uses the last argument (ny)
    } else if (con.type == 'f') {
      fun <- function(contrast, xtx, rkC, ny) {
        res <- apply(xtx, 3L, function(r) inv(contrast %*% r, contrast, transpose=TRUE))
        dim(res) <- c(rkC, rkC, ny)
        res
      }
    }

  fun
}

#' Permutation test for group difference of graph measures
#'
#' \code{brainGraph_permute} draws permutations from linear model residuals to
#' determine the significance of between-group differences of a global or
#' vertex-wise graph measure. It is intended for structural covariance networks
#' (in which there is only one graph per group), but can be extended to other
#' types of data.
#'
#' If you would like to calculate differences in the area-under-the-curve (AUC)
#' across densities, then specify \code{auc=TRUE}.
#'
#' There are three possible \dQuote{levels}:
#' \enumerate{
#'   \item \emph{graph} Calculate modularity (Louvain algorithm), clustering
#'   coefficient, characteristic path length, degree assortativity, and global
#'   efficiency.
#'   \item \emph{vertex} Choose one of: centrality metrics (betweenness,
#'   closeness, communicability, eigenvector, leverage, pagerank, subgraph);
#'   k-core; degree; eccentricity; nodal or local efficiency; k-nearest neighbor
#'   degree; shortest path length; transitivity; or vulnerability.
#'   \item \emph{other} Supply your own function. This is useful if you want to
#'   calculate something that I haven't hard-coded. It must take as its own
#'   arguments: \code{g} (a list of lists of \code{igraph} graph objects); and
#'   \code{densities} (numeric vector).
#' }
#'
#' @param densities Numeric vector of graph densities
#' @param resids An object of class \code{brainGraph_resids} (the output from
#'   \code{\link{get.resid}})
#' @param N Integer; the number of permutations (default: 5e3)
#' @param perms Numeric matrix of permutations, if you would like to provide
#'   your own (default: \code{NULL})
#' @param auc Logical indicating whether or not to calculate differences in the
#'   area-under-the-curve of metrics (default: \code{FALSE})
#' @param level A character string for the attribute \dQuote{level} to calculate
#'   differences (default: \code{graph})
#' @param measure A character string specifying the vertex-level metric to
#'   calculate, only used if \code{level='vertex'} (default: \code{btwn.cent}).
#'   For the \code{summary} method, this is to focus on a single
#'   \emph{graph-level} measure (since multiple are calculated at once).
#' @param .function A custom function you can pass if \code{level='other'}
#' @export
#' @importFrom permute shuffleSet
#' @importFrom foreach getDoParRegistered
#' @importFrom doParallel registerDoParallel
#'
#' @return An object of class \code{brainGraph_permute} with input arguments in
#'   addition to:
#'   \item{DT}{A data table with permutation statistics}
#'   \item{obs.diff}{A data table of the observed group differences}
#'   \item{Group}{Group names}
#'
#' @family Group analysis functions
#' @family Structural covariance network functions
#'
#' @author Christopher G. Watson, \email{cgwatson@@bu.edu}
#' @examples
#' \dontrun{
#' myResids <- get.resid(lhrh, covars)
#' myPerms <- shuffleSet(n=nrow(myResids$resids.all), nset=1e3)
#' out <- brainGraph_permute(densities, m, perms=myPerms)
#' out <- brainGraph_permute(densities, m, perms=myPerms, level='vertex')
#' out <- brainGraph_permute(densities, m, perms=myPerms,
#'   level='other', .function=myFun)
#' }

brainGraph_permute <- function(densities, resids, N=5e3, perms=NULL, auc=FALSE,
                               level=c('graph', 'vertex', 'other'),
                               measure=c('btwn.cent', 'coreness', 'degree', 'eccentricity',
                                         'clo.cent', 'communicability', 'ev.cent', 'lev.cent',
                                         'pagerank', 'subg.cent', 'E.local', 'E.nodal',
                                         'knn', 'Lp', 'transitivity', 'vulnerability'),
                               .function=NULL) {
  stopifnot(inherits(resids, 'brainGraph_resids'))
  gID <- getOption('bg.group')
  measure <- match.arg(measure)
  level <- match.arg(level)
  if (level == 'other') {
    if (!is.function(.function)) stop('".function" must be a function!')
    measure <- 'other'
  } else if (level == 'graph') {
    measure <- NULL
  }

  if (is.null(perms)) perms <- shuffleSet(n=nobs(resids), nset=N)
  dims <- dim(perms)
  N <- dims[1L]
  perms <- rbind(perms, seq_len(dims[2L]))  # last row is observed metrics
  grps <- as.numeric(resids$resids.all[, get(gID)])

  if (!getDoParRegistered()) {
    cl <- makeCluster(getOption('bg.ncpus'))
    registerDoParallel(cl)
  }
  # Should return a "data.table" (graph-level) or numeric matrix (vertex-level)
  # When "auc=FALSE", should have a "densities" column
  res.perm <- switch(level,
               vertex=permute_vertex_foreach(perms, densities, resids, grps, auc, measure),
               other=permute_other_foreach(perms, densities, resids, grps, .function),
               graph=permute_graph_foreach(perms, densities, resids, grps, auc))

  if (length(densities) == 1L) res.perm <- cbind(densities=densities, res.perm)
  if (level == 'vertex') {
    res.perm <- as.data.table(res.perm)
    start <- if (isTRUE(auc)) 1L else 2L
    setnames(res.perm, seq.int(start, dim(res.perm)[2L]), region.names(resids))
  }

  obs.ind <- if (isTRUE(auc)) N + 1L else (N + 1L) * seq_along(densities)
  obs.diff <- res.perm[obs.ind]
  res.perm <- res.perm[-obs.ind]
  out <- list(atlas=resids$atlas, auc=auc, N=N, level=level, measure=measure, densities=densities,
              resids=resids, DT=res.perm, obs.diff=obs.diff, Group=resids$Group)
  class(out) <- c('brainGraph_permute', class(out))
  return(out)
}

#==============================================================================
# Helper functions
#==============================================================================
make_graphs_perm <- function(densities, resids, inds, grps) {
  corrs <- lapply(unique(grps), function(x)
                  corr.matrix(resids[which(grps[inds] == x)],
                              densities=densities, rand=TRUE))
  sapply(corrs, lapply, function(x)
         apply(x$r.thresh, 3L, graph_from_adjacency_matrix, mode='undirected', diag=FALSE))
}

# Graph level
graph_attr_perm <- function(g, densities) {
  mod <- sapply(g, sapply, function(x) modularity(cluster_louvain(x)))
  Cp <- sapply(g, sapply, function(x) transitivity(x, type='localaverage'))
  Lp <- sapply(g, sapply, mean_distance)
  assort <- sapply(g, sapply, assortativity_degree)
  E.global <- sapply(g, sapply, efficiency, 'global')
  list(mod=mod, Cp=Cp, Lp=Lp, assort=assort, E.global=E.global)
}

permute_graph_foreach <- function(perms, densities, resids, grps, auc) {
  i <- NULL
  N <- dim(perms)[1L]
  if (isTRUE(auc)) {
    res.perm <- foreach(i=seq_len(N), .combine='rbind') %dopar% {
      g <- make_graphs_perm(densities, resids, perms[i, ], grps)
      meas.list <- graph_attr_perm(g, densities)
      t(vapply(meas.list, function(y) auc_diff_perm(densities, y), numeric(1L)))
    }
    res.perm <- as.data.table(res.perm)
  } else {
    # Returns a list of (Ndensities * Nperms) X 2 matrices for all metrics
    res.perm <- foreach(i=seq_len(N), .combine=function(a, b) Map(rbind, a, b)) %dopar% {
      g <- make_graphs_perm(densities, resids, perms[i, ], grps)
      meas.list <- graph_attr_perm(g, densities)
    }
    res.perm <- data.table(densities=rep.int(densities, N),
                           sapply(res.perm, function(x) as.numeric(-diff(t(x)))))
    setkey(res.perm, densities)
  }
  return(res.perm)
}

# Vertex-level
vertex_attr_funs <- function(measure) {  # Move the "switch" outside the "foreach" loop
  switch(measure,
         coreness=coreness,
         degree=degree,
         eccentricity=eccentricity,
         clo.cent=function(x) centr_clo(x)$res,
         communicability=centr_betw_comm,
         ev.cent=function(x) centr_eigen(x)$vector,
         lev.cent=function(x) centr_lev(x)$res,
         pagerank=function(x) page_rank(x, weights=NA)$vector,
         subg.cent=subgraph_centrality,
         E.local=function(x) efficiency(x, type='local', weights=NA, use.parallel=FALSE),
         E.nodal=function(x) efficiency(x, type='nodal', weights=NA),
         knn=function(x) knn(x)$knn,
         Lp=function(x) mean_distance_wt(x, level='vertex', weights=NA),
         transitivity=function(x) transitivity(x, type='local', isolates='zero'),
         vulnerability=function(x) vulnerability(x, use.parallel=FALSE),
         function(x) centr_betw(x)$res)
}

permute_vertex_foreach <- function(perms, densities, resids, grps, auc, measure) {
  i <- NULL
  if (isTRUE(auc)) {
    diffFun <- function(densities, meas.list) {
      sapply(seq_len(dim(meas.list[[1L]])[2L]), function(x)
             auc_diff(densities, cbind(meas.list[[1L]][, x], meas.list[[2L]][, x])))
    }
  } else {
    diffFun <- function(densities, meas.list) cbind(densities, meas.list[[1L]] - meas.list[[2L]])
  }
  fun <- vertex_attr_funs(measure)
  res.perm <- foreach(i=seq_len(dim(perms)[1L]), .combine='rbind') %dopar% {
    g <- make_graphs_perm(densities, resids, perms[i, ], grps)
    meas.list <- lapply(g, function(x) t(sapply(x, fun)))
    diffFun(densities, meas.list)
  }
}

# Other-level
permute_other_foreach <- function(perms, densities, resids, grps, .function) {
  i <- NULL
  res.perm <- foreach(i=seq_len(dim(perms)[1L]), .combine='rbind') %dopar% {
    g <- make_graphs_perm(densities, resids, perms[i, ], grps)
    .function(g, densities)
  }
}

#==============================================================================
# Methods
#==============================================================================

#' Print a summary from a permutation analysis
#'
#' @param object,x A \code{brainGraph_permute} object (output by
#'   \code{\link{brainGraph_permute}}).
#' @param p.sig Character string specifying which p-value to use for displaying
#'   significant results (default: \code{p})
#' @param ... Unused
#' @inheritParams GLM
#' @export
#' @rdname brainGraph_permute

summary.brainGraph_permute <- function(object, measure=object$measure,
                                       alternative=c('two.sided', 'less', 'greater'),
                                       alpha=0.05, p.sig=c('p', 'p.fdr'), ...) {
  perm.diff <- p <- p.fdr <- region <- obs.diff <- ..measure <- NULL
  gID <- getOption('bg.group')
  if (object$level == 'other') {  # Hack to figure out which level it is when level="other"
    object$level <- if (dim(object$DT)[2L] > 6L) 'vertex' else 'graph'
  }

  if (object$level == 'graph' && is.null(measure)) measure <- 'mod'
  group_str <- paste0(measure, '.', object$Group)
  permDT <- copy(object$DT)
  g <- with(object, make_graphs_perm(densities, resids, seq_len(nobs(resids)),
                                     resids$resids.all[, as.numeric(get(gID))]))

  densities <- object$densities
  # VERTEX-LEVEL
  #-------------------------------------
  if (object$level == 'vertex') {
    obsDT <- copy(object$obs.diff)
    fun <- vertex_attr_funs(measure)
    obs <- lapply(g, function(x) t(sapply(x, fun)))

    if (isTRUE(object$auc)) {
      obs <- lapply(obs, apply, 2L, function(y) -auc_diff(densities, y))
      permDT[, densities := 1]
      obsDT[, densities := 1]
      densities <- 1
    }
    sum.dt <- data.table(densities=densities,
                         region=rep(V(g[[1L]][[1L]])$name, each=length(densities)),
                         g1=c(obs[[1L]]), g2=c(obs[[2L]]),
                         key=c('densities', 'region'))
    setnames(sum.dt, c('g1', 'g2'), group_str)
    obs.m <- melt(obsDT, id.vars='densities', variable.name='region', value.name='obs.diff')
    sum.dt <- merge(sum.dt, obs.m)
    permDT <- melt(permDT, id.vars='densities', variable.name='region', value.name=measure)
    setkeyv(permDT, key(sum.dt))
    permdiff <- permDT[, list(perm.diff=mean(get(measure))), by=key(sum.dt)]
    sum.dt <- merge(sum.dt, permdiff, by=key(sum.dt))

  # GRAPH-LEVEL
  #-------------------------------------
  } else if (object$level == 'graph') {
    stopifnot(hasName(permDT, measure))
    permDT[, region := 'graph']
    obs <- graph_attr_perm(g, densities)[[measure]]

    if (isTRUE(object$auc)) {
      obs <- t(apply(obs, 2L, function(y) -auc_diff(densities, y)))
      permDT[, densities := 1]
      densities <- 1
    }

    sum.dt <- data.table(densities=densities, region='graph', obs)
    setnames(sum.dt, c('V1', 'V2'), group_str)
    sum.dt[, obs.diff := object$obs.diff[[measure]]]
    sum.dt[, perm.diff := permDT[, mean(get(measure)), by=densities]$V1]
  }
  result.dt <- merge(permDT[, c('densities', 'region', ..measure)],
                     sum.dt[, c('densities', 'region', 'obs.diff')],
                     by=c('densities', 'region'))

  alt <- match.arg(alternative)
  compfun <- switch(alt,
                    two.sided=function(perm, obs) sum(abs(perm) >= abs(unique(obs))),
                    less=function(perm, obs) sum(perm <= unique(obs)),
                    greater=function(perm, obs) sum(perm >= unique(obs)))
  result.dt[, p := (compfun(get(measure), obs.diff) + 1L) / (.N + 1L), by=key(result.dt)]
  CI <- switch(alt, two.sided=c(alpha / 2, 1 - (alpha / 2)), less=c(alpha, 1), greater=c(1 / object$N, 1 - alpha))
  result.dt[, c('ci.low', 'ci.high') := as.list(sort(get(measure))[ceiling(.N * CI)]), by=key(result.dt)]
  result.dt <- result.dt[, .SD[1L], by=key(result.dt)]
  sum.dt <- merge(sum.dt, result.dt[, !c(measure, 'obs.diff'), with=FALSE], by=key(result.dt))
  setcolorder(sum.dt, c('densities', 'region', group_str, 'obs.diff', 'perm.diff', 'ci.low', 'ci.high', 'p'))
  if (isFALSE(object$auc)) {
    groupby <- if (object$level == 'graph') NULL else 'densities'
    sum.dt[, p.fdr := p.adjust(p, 'fdr'), by=groupby]
  }

  meas.full <- print_full_metric(measure)
  p.sig <- match.arg(p.sig)
  perm.sum <- c(object, list(DT.sum=sum.dt, meas.full=meas.full, alt=alt, alpha=alpha, p.sig=p.sig))
  perm.sum$measure <- measure
  class(perm.sum) <- c('summary.brainGraph_permute', class(perm.sum))
  perm.sum
}

#' @aliases summary.brainGraph_permute
#' @method print summary.brainGraph_permute
#' @export

print.summary.brainGraph_permute <- function(x, ...) {
  print_title_summary('Permutation analysis')
  cat('# of permutations:', prettyNum(x$N, ','), '\n')
  cat('Level: ', x$level, '\n')
  cat('Graph metric: ', x$meas.full, '\n')
  if (isTRUE(x$auc)) cat('Area-under-the-curve (AUC) calculated across', length(x$densities), 'densities:\n', x$densities, '\n')

  symb <- switch(x$alt, two.sided='!=', greater='>', less='<')
  alt <- sprintf('%s - %s %s 0', x$Group[1L], x$Group[2L], symb)
  cat('Alternative hypothesis: ', '\t', alt, '\n')
  cat('Alpha: ', x$alpha, '\n\n')
  if (with(x, dim(DT.sum[get(p.sig) < alpha])[1L]) == 0L) {
    cat('No significant results!\n')
  } else {
    clp <- 100 * (1 - x$alpha)
    setnames(x$DT.sum, c('ci.low', 'ci.high'), paste0(clp, '% CI ', c('low', 'high')))
    with(x, print(DT.sum[get(p.sig) < alpha]))
  }
  invisible(x)
}

#' Plot results from permutation testing
#'
#' @param ptitle Character string specifying a title for the plot (default:
#'   \code{NULL})
#' @export
#' @return The \code{plot} method returns a \emph{list} of \code{ggplot} objects
#' @rdname brainGraph_permute

plot.brainGraph_permute <- function(x, measure=x$measure,
                                    alternative=c('two.sided', 'less', 'greater'),
                                    alpha=0.05, p.sig=c('p', 'p.fdr'), ptitle=NULL, ...) {
  densities <- Group <- sig <- trend <- yloc <- obs <- mylty <- ci.low <- ci.high <-
    variable <- value <- reg.num <- region <- perm.diff <- obs.diff <- plot2 <- reg.num2 <- NULL
  p.sig <- match.arg(p.sig)
  perm.sum <- summary(x, measure=measure, alternative=alternative, alpha=alpha)
  measure <- perm.sum$measure
  sum.dt <- perm.sum$DT.sum
  if (is.null(ptitle)) ptitle <- perm.sum$meas.full
  ylabel2 <- sprintf('Observed and permutation difference (%s - %s)', x$Group[1L], x$Group[2L])

  # GRAPH LEVEL
  #-------------------------------------
  if (x$level == 'graph') {
    plot.dt <- melt(sum.dt, id.vars=setdiff(names(sum.dt), paste0(measure, '.', x$Group)),
                    variable.name='Group', value.name='obs')
    plot.dt[, Group := factor(Group, labels=x$Group)]
    idvars <- c('densities', 'region', 'p', 'Group', 'obs')
    if (isFALSE(x$auc)) idvars <- c(idvars, 'p.fdr')
    plot.dt <- melt(plot.dt, id.vars=idvars)
    plot.dt[, c('sig', 'trend') := '']
    plot.dt[get(p.sig) < alpha, sig := '*']
    plot.dt[get(p.sig) >= alpha & get(p.sig) < 2 * alpha, trend := '*']
    plot.dt[, yloc := extendrange(obs, f=0.07)[1L]]
    plot.dt[, mylty := 0]
    plot.dt[variable == 'obs.diff', mylty := 1]
    plot.dt[variable %in% c('ci.low', 'ci.high'), mylty := 2]
    plot.dt[variable == 'perm.diff', mylty := 3]

    # 'base' plotting
    if (!requireNamespace('ggplot2', quietly=TRUE)) {
      # 1. Line plot of observed group values
      gID <- getOption('bg.group')
      grps <- paste(perm.sum$measure, x$Group, sep='.')
      ymin <- plot.dt[1L, yloc]
      ymax <- plot.dt[, max(obs)]

      par(mar=c(8.6, 4.1, 4.1, 2.1), xpd=TRUE)
      plot.dt[variable == 'obs.diff' & get(gID) == grps[1L],
              plot(densities, obs, type='l', col=plot.cols[1L],
                   main=perm.sum$meas.full, xlab='Density', ylab='Observed values')]
      plot.dt[variable == 'obs.diff' & get(gID) == grps[2L],
              lines(densities, obs, col=plot.cols[2L])]
      plot.dt[variable == 'obs.diff' & sig == '*',
              text(densities, yloc, sig, cex=1.75, col='red')]
      plot.dt[variable == 'obs.diff' & trend == '*',
              text(densities, yloc, trend, cex=1.75, col='blue')]
      legend('bottom', title=gID, sub(paste0(perm.sum$measure, '.'), '', grps),
             fill=plot.cols[1L:2L], inset=c(0, -0.35), horiz=TRUE)

      # 2. Line plot of observed difference and CI's
      ymin <- plot.dt[variable == 'ci.low', min(value)]
      ymax <- plot.dt[variable == 'ci.high', max(value)]

      par(mar=c(9.1, 4.1, 4.1, 2.1), xpd=TRUE)
      plot.dt[variable == 'obs.diff' & get(gID) == grps[1L],
              plot(densities, value, type='l', col='red', ylim=c(ymin, ymax),
                   main=perm.sum$meas.full, xlab='Density', ylab=ylabel2)]
      plot.dt[variable == 'obs.diff' & get(gID) == grps[1L],
              points(densities, value, col='red', pch=19)]
      plot.dt[variable == 'ci.high' & get(gID) == grps[1L],
              lines(densities, value, lty=2)]
      plot.dt[variable == 'ci.low' & get(gID) == grps[1L],
              lines(densities, value, lty=2)]
      plot.dt[variable == 'perm.diff' & get(gID) == grps[1L],
              lines(densities, value, lty=3)]
      legend('bottom', c('Observed diff.', '95% CI', 'Mean permutation diff.'),
             lty=1L:3L, inset=c(0, -0.4))
      return(invisible(x))

    # 'ggplot2' plotting
    } else {
      # 1. Line plot of observed group values
      p <- ggplot2::ggplot(plot.dt, ggplot2::aes(x=densities))
      plot1 <- p +
        ggplot2::geom_line(ggplot2::aes(y=obs, col=Group)) +
        ggplot2::geom_text(ggplot2::aes(y=yloc, label=sig), col='red', size=8) +
        ggplot2::geom_text(ggplot2::aes(y=yloc, label=trend), col='blue', size=8) +
        ggplot2::theme(legend.position='bottom')

      # 2. Line plot of observed difference and CI's
      plot2 <- p +
        ggplot2::geom_line(data=plot.dt[variable =='obs.diff'], ggplot2::aes(y=value, lty=factor(mylty)), col='red') +
        ggplot2::geom_point(data=plot.dt[variable == 'obs.diff'], ggplot2::aes(y=value), col='red', size=3) +
        ggplot2::geom_line(data=plot.dt[variable == 'perm.diff'], ggplot2::aes(y=value, lty=factor(mylty))) +
        ggplot2::geom_line(data=plot.dt[variable == 'ci.low'], ggplot2::aes(y=value, lty=factor(mylty))) +
        ggplot2::geom_line(data=plot.dt[variable == 'ci.high'], ggplot2::aes(y=value, lty=factor(mylty))) +
        ggplot2::scale_linetype_manual(name='',
                                       labels=c('Observed diff.', '95% CI', 'Mean permutation diff.'),
                                       values=1:3) +
        ggplot2::theme(legend.position='bottom', legend.spacing.x=ggplot2::unit(9, 'pt'),
                       legend.key=ggplot2::element_rect(fill='white'),
                       legend.background=ggplot2::element_rect(fill='gray79'))

      ylabel1 <- 'Observed values'
      plot1 <- plot1 +
        ggplot2::labs(title=ptitle, y=ylabel1, x='Density') +
        ggplot2::theme(plot.title=ggplot2::element_text(hjust=0.5, face='bold'))
      plot2 <- plot2 +
        ggplot2::labs(title=ptitle, y=ylabel2, x='Density') +
        ggplot2::theme(plot.title=ggplot2::element_text(hjust=0.5, face='bold'))
      return(list(plot1, plot2))
    }

  # VERTEX LEVEL
  #-------------------------------------
  } else {
    plot.dt <- droplevels(sum.dt[get(p.sig) < alpha])
    if (dim(plot.dt)[1L] == 0L) stop('No significant results!')

    # 'base' plotting
    if (!requireNamespace('ggplot2', quietly=TRUE)) {
      ymin <- plot.dt[, min(c(obs.diff, ci.low))]
      ymax <- plot.dt[, max(c(obs.diff, ci.high))]
      plot.dt[, reg.num2 := as.numeric(as.factor(region))]
      dotplot(perm.diff ~ region | as.factor(densities),
              xlab='Region', ylab=ylabel2, main=perm.sum$meas.full,
              data=plot.dt, ylim=extendrange(c(ymin, ymax)),
              x0=plot.dt$reg.num2, y0=plot.dt$ci.low, y1=plot.dt$ci.high, yobs=plot.dt$obs.diff,
              scales=list(x=list(relation='free')),
              panel=function(x, y, x0, y0, y1, yobs, subscripts) {
                lpoints(x, y, pch=19)
                panel.abline(h=0, lty=3)
                larrows(x0=x0[subscripts], y0=y0[subscripts], x1=x0[subscripts], y1=y1[subscripts],
                        code=3, angle=90, length=0.1)
                lsegments(x0[subscripts] - 0.1, yobs[subscripts],
                          x0[subscripts] + 0.1, yobs[subscripts],
                          col='red', lwd=3)
              })

    # 'ggplot2' plotting
    } else {
      plot.dt[, reg.num := seq_len(.N), by=densities]
      plot1 <- ggplot2::ggplot(plot.dt, ggplot2::aes(x=region)) +
        ggplot2::geom_point(ggplot2::aes(y=perm.diff)) +
        ggplot2::geom_errorbar(ggplot2::aes(ymin=ci.low, ymax=ci.high)) +
        ggplot2::geom_segment(ggplot2::aes(x=reg.num - .25, xend=reg.num + .25, y=obs.diff, yend=obs.diff),
                              col='red', size=1.25) +
        ggplot2::facet_wrap(~ densities, scales='free') +
        ggplot2::labs(title=ptitle, y=ylabel2, x='Region') +
        ggplot2::theme(plot.title=ggplot2::element_text(hjust=0.5, face='bold'))
      return(list(plot1, plot2))
    }
  }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create_graphs.R
\name{Creating_Graphs}
\alias{Creating_Graphs}
\alias{make_brainGraph}
\alias{make_brainGraph.igraph}
\alias{make_brainGraph.matrix}
\alias{make_brainGraph.bg_mediate}
\alias{is.brainGraph}
\alias{summary.brainGraph}
\alias{make_empty_brainGraph}
\title{Create a brainGraph object}
\usage{
make_brainGraph(
  x,
  atlas,
  type = c("observed", "random"),
  level = c("subject", "group", "contrast"),
  set.attrs = TRUE,
  modality = NULL,
  weighting = NULL,
  threshold = NULL,
  ...
)

\method{make_brainGraph}{igraph}(
  x,
  atlas,
  type = c("observed", "random"),
  level = c("subject", "group", "contrast"),
  set.attrs = TRUE,
  modality = NULL,
  weighting = NULL,
  threshold = NULL,
  name = NULL,
  Group = NULL,
  subnet = NULL,
  ...
)

\method{make_brainGraph}{matrix}(
  x,
  atlas,
  type = c("observed", "random"),
  level = c("subject", "group", "contrast"),
  set.attrs = TRUE,
  modality = NULL,
  weighting = NULL,
  threshold = NULL,
  name = NULL,
  Group = NULL,
  subnet = NULL,
  mode = "undirected",
  weighted = NULL,
  diag = FALSE,
  ...
)

\method{make_brainGraph}{bg_mediate}(
  x,
  atlas = x$atlas,
  type = "observed",
  level = "contrast",
  set.attrs = FALSE,
  modality = NULL,
  weighting = NULL,
  threshold = NULL,
  ...
)

is.brainGraph(x)

\method{summary}{brainGraph}(object, print.attrs = c("all", "graph", "vertex", "edge", "none"), ...)

make_empty_brainGraph(
  atlas,
  type = c("observed", "random"),
  level = c("subject", "group", "contrast"),
  modality = NULL,
  weighting = NULL,
  threshold = NULL,
  name = NULL,
  Group = NULL,
  ...
)
}
\arguments{
\item{x}{An \code{igraph} graph object, numeric matrix, or \code{bg_mediate}
object}

\item{atlas}{Character string specifying the brain atlas}

\item{type}{Character string indicating the type of graphs. Default:
\code{observed}}

\item{level}{Character string indicating whether the graphs are subject-,
group-, or contrast-specific. Default: \code{'subject'}}

\item{set.attrs}{Logical indicating whether to assign all graph-, vertex-,
and edge-level attributes (via \code{\link{set_brainGraph_attr}}). Default:
\code{TRUE}}

\item{modality}{Character string indicating imaging modality (e.g. 'dti').
Default: \code{NULL}}

\item{weighting}{Character string indicating how the edges are weighted
(e.g., 'fa', 'pearson', etc.). Default: \code{NULL}}

\item{threshold}{Integer or number indicating the threshold used when
\dQuote{sparsifying} the connectivity matrix (if any). Default: \code{NULL}}

\item{...}{Arguments passed to \code{\link{set_brainGraph_attr}}}

\item{name}{Character string indicating subject ID or group/contrast name,
depending on the \code{level}. Default: \code{NULL}}

\item{Group}{Character string indicating group membership. Default:
\code{NULL}}

\item{subnet}{Integer or character vector indicating the vertices to keep, if
you are interested in working with a subset of an atlas. By default, all
vertices are used.}

\item{mode}{Character string defining how the matrix should be interpreted.
Default: \code{'undirected'}}

\item{weighted}{Logical specifying whether to create a weighted network}

\item{diag}{Logical indicating whether to include the diagonal of the
connectivity matrix. Default: \code{FALSE}}

\item{object}{A \code{brainGraph} object}

\item{print.attrs}{Character string indicating whether or not to list the
object's attributes (default: \code{all})}
}
\value{
A \code{brainGraph} graph object with additional graph-, vertex-, and
  edge-level attributes (see below).

The method for \code{bg_mediate} returns a \code{brainGraph_mediate}
  object, which has extra attributes:
  \item{Graph}{\emph{mediator}, \emph{treat}, \emph{outcome}, \emph{nobs}}
  \item{Vertex}{\emph{b?.acme, p?.acme}, \emph{b?.ade, p?.ade},
    \emph{b?.prop, p?.prop}, \emph{b.tot, p.tot}}

\code{make_empty_brainGraph} -- An empty \code{brainGraph} graph
  object
}
\description{
\code{make_brainGraph} is the main creation function for creating a
\code{brainGraph} graph object. This is simply an \code{igraph} graph
object with additional attributes (at all levels). Several of the graph-level
attributes serve the purpose of providing metadata on how the connectivity
matrices/networks were created.

\code{make_brainGraph.bg_mediate} creates a graph only for
\emph{vertex}-level analyses.

\code{make_empty_brainGraph} creates an empty undirected \code{brainGraph}
object with vertex count equal to the atlas specified; i.e., it creates a
graph with 0 edges. Typically used to present results from an analysis in
which edges don't make sense (e.g., GLM comparing differences in a
vertex-level attribute).
}
\section{Graph-level attributes}{

Graph-level attributes added are:
\describe{
  \item{version}{The R, \code{brainGraph}, and \code{igraph} package versions
    used to create the graph}
  \item{date}{The creation date, from \code{\link{as.POSIXct}}}
  \item{atlas}{Character string denoting the brain atlas used}
  \item{type}{Character string specifying whether this is an \emph{observed}
    or \emph{random} graph}
  \item{modality}{The imaging modality; you can choose anything you like,
    but the \code{summary.brainGraph} knows about \code{dti}, \code{fmri},
    \code{thickness}, \code{area}, and \code{volume}}
  \item{weighting}{What edge weights represent; you can choose anything you
    like, but \code{summary.brainGraph} knows about \code{fa}, \code{sld}
    (streamline density, tractography), \code{pearson}, \code{spearman},
    \code{kendall}, and \code{partial} (partial correlation coefficient)}
  \item{threshold}{Numeric indicating the threshold used to create the final
    connectivity matrix (if any)}
  \item{name}{Character string specifying the study ID or group/contrast
    name, depending on the \code{level} argument}
  \item{Group}{Character string specifying the experimental group that the
    given subject belongs to, or if it is a group-level graph}
  \item{subnet}{Integer vector, if \code{subnet} was specified in the call}
}
}

\section{Vertex attributes}{

Vertex-level attributes added are:
\describe{
  \item{name}{The names of the brain regions in the network}
  \item{lobe}{The names of the major brain lobes for each vertex}
  \item{hemi}{The names of the hemisphere for each vertex (either \code{'L'},
    \code{'R'}, or \code{'B'})}
  \item{lobe.hemi}{The lobe-hemisphere combination (represented as an
    \emph{integer} vector)}
  \item{class}{The tissue class (if applicable)}
  \item{network}{The network (if the atlas is \code{dosenbach160})}
  \item{x,y,z}{The spatial coordinates of the (centers-of-mass) brain regions
    in MNI space}
  \item{x.mni,y.mni,z.mni}{Same as above}
  \item{color.lobe,color.class,color.network}{Colors for vertices of their
    respective membership}
  \item{circle.layout}{Integer vector indicating the order (going
    counter-clockwise from the top) for circular layouts}
}
}

\section{Edge attributes}{

Edge-level attributes added are:
\describe{
  \item{color.lobe,color.class,color.network}{Correspond to the vertex
    attribute of the same name. Inter-group edges will be colored
    \emph{gray}}
}
}

\section{Specifying a subnetwork}{

You can create a graph for a subset of an atlas's regions with the
\code{subnet} argument. This can either be a numeric or character vector. If
the input object (either a matrix or an \code{igraph} graph) has fewer
rows/columns or vertices, respectively, than the atlas then the \code{subnet}
graph attribute will also be added to the return object. This may occur if,
for example, you use \code{\link{make_auc_brainGraph}} on graphs that were
initially created from subnetworks.
}

\examples{
\dontrun{
bg <- make_brainGraph(A, 'dkt', modality='dti', weighting='fa',
  mode='undirected', diag=FALSE, weighted=TRUE)
}
}
\seealso{
Other Graph creation functions: 
\code{\link{Creating_Graphs_GLM}},
\code{\link{brainGraphList}},
\code{\link{make_ego_brainGraph}()}
}
\concept{Graph creation functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rich_club.R
\name{Rich Club}
\alias{Rich Club}
\alias{rich_club_coeff}
\alias{rich_club_all}
\alias{rich_club_norm}
\alias{rich_core}
\title{Rich club calculations}
\usage{
rich_club_coeff(g, k = 1, weighted = FALSE, A = NULL)

rich_club_all(g, weighted = FALSE, A = NULL)

rich_club_norm(g, N = 100, rand = NULL, ...)

rich_core(g, weighted = FALSE, A = NULL)
}
\arguments{
\item{g}{An \code{igraph} graph object}

\item{k}{Integer; the minimum degree for including a vertex. Default: 1}

\item{weighted}{Logical indicating whether or not edge weights should be
used. Default: \code{FALSE}}

\item{A}{Numeric matrix; the adjacency matrix of the input graph. Default:
\code{NULL}}

\item{N}{Integer; the number of random graphs to generate. Default: 100}

\item{rand}{A list of \code{igraph} graph objects, if random graphs have
already been generated. Default: \code{NULL}}

\item{...}{Other parameters (passed to \code{\link{sim.rand.graph.par}})}
}
\value{
\code{\link{rich_club_coeff}} - a list with components:
  \item{phi}{The rich club coefficient, \eqn{\phi}.}
  \item{graph}{A subgraph containing only the rich club vertices.}
  \item{Nk,Ek}{The number of vertices/edges in the rich club graph.}

\code{\link{rich_club_all}} - a \code{data.table} with components:
  \item{k}{A vector of all vertex degrees present in the original graph}
  \item{phi}{The rich-club coefficient}
  \item{Nk,Ek}{The number of vertices/edges in the rich club for each
    successive \emph{k}}

\code{\link{rich_club_norm}} - a data table with columns:
  \item{k}{Sequence of degrees}
  \item{rand}{Rich-club coefficients for the random graphs}
  \item{orig}{Rich-club coefficients for the original graph.}
  \item{norm}{Normalized rich-club coefficients.}
  \item{p}{P-values based on the distribution of \code{rand}}
  \item{p.fdr}{The FDR-adjusted P-values}
  \item{density}{The observed graph's density}
  \item{threshold,Group,name}{}

\code{\link{rich_core}} - a data table with columns:
  \item{density}{The density of the graph.}
  \item{rank}{The rank of the boundary for the rich core.}
  \item{k.r}{The degree/strength of the vertex at the boundary.}
  \item{core.size}{The size of the core relative to the graph size.}
  \item{weighted}{Whether or not weights were used}
}
\description{
\code{rich_club_coeff} calculates the \emph{rich club} of a graph, returning
the rich-club coefficient, \eqn{\phi}, and the subgraph of rich club
vertices.

\code{rich_club_all} is a wrapper for \code{\link{rich_club_coeff}} that
calculates the rich-club coefficient for all degrees present in the graph. It
returns a \code{data.table} with the coefficients and vertex and edge counts
for each successive rich club.

\code{rich_club_norm} will (optionally) generate a number of random graphs,
calculate their rich club coefficients (\eqn{\phi}), and return
\eqn{\phi_{norm}} of the graph of interest, which is the observed rich-club
coefficient divided by the mean across the random graphs.

\code{rich_core} finds the boundary of the rich core of a graph, based on the
decreasing order of vertex degree. It also calculates the degree that
corresponds to that rank, and the core size relative to the total number of
vertices in the graph.
}
\details{
If random graphs have already been generated, you can supply a list as an
argument.

For weighted graphs, the degree is substituted by a normalized weight:
\deqn{ceiling(A / w_{min})}
where \eqn{w_{min}} is the minimum weight (that is greater than 0), and
\eqn{ceiling()} is the \emph{ceiling} function that rounds up to the nearest
integer.
}
\references{
Zhou, S. and Mondragon, R.J. (2004) The rich-club phenomenon
  in the internet topology. \emph{IEEE Comm Lett}, \bold{8}, 180--182.
  \doi{10.4018/978-1-59140-993-9.ch066}

Opsahl, T. and Colizza, V. and Panzarasa, P. and Ramasco, J.J.
  (2008) Prominence and control: the weighted rich-club effect. \emph{Physical
  Review Letters}, \bold{101.16}, 168702.
  \doi{10.1103/PhysRevLett.101.168702}

Colizza, V. and Flammini, A. and Serrano, M.A. and Vespignani, A.
  (2006) Detecting rich-club ordering in complex networks. \emph{Nature
  Physics}, \bold{2}, 110--115. \doi{10.1038/nphys209}

Ma, A and Mondragon, R.J. (2015) Rich-cores in networks.
  \emph{PLoS One}, \bold{10(3)}, e0119678.
  \doi{10.1371/journal.pone.0119678}
}
\seealso{
Other Rich-club functions: 
\code{\link{plot_rich_norm}()},
\code{\link{rich_club_attrs}()}

Other Random graph functions: 
\code{\link{Random Graphs}}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}
\concept{Random graph functions}
\concept{Rich-club functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_brainGraph_multi.R
\name{plot_brainGraph_multi}
\alias{plot_brainGraph_multi}
\alias{slicer}
\title{Save PNG of one or three views for all graphs in a brainGraphList}
\usage{
plot_brainGraph_multi(
  g.list,
  filename = "orthoview.png",
  subgraph = NULL,
  main = NULL,
  label = NULL,
  cex.main = 1,
  ...
)

slicer(
  g.list,
  nrows,
  ncols,
  plane = "axial",
  hemi = "both",
  filename = "all.png",
  main = NULL,
  cex.main = 1,
  ...
)
}
\arguments{
\item{g.list}{A \code{brainGraphList} or a list of \code{brainGraph} objects}

\item{filename}{Character string of the filename of the PNG to be written.}

\item{subgraph}{A vector or list of character strings to (optionally) subset
the graph(s), possibly by multiple conditions}

\item{main}{A vector or list of character strings to be placed in the main
title of the center (axial) plot for each graph}

\item{label}{A vector or list of character strings to be placed in one of the
corners of the left plot (sagittal) in each row}

\item{cex.main}{Numeric specifying the level of character expansion for the
plot titles. Default: \code{1} (no expansion)}

\item{...}{Other arguments passed to \code{\link{plot.brainGraph}}}

\item{nrows}{Integer; the number of rows in the figure}

\item{ncols}{Integer; the number of columns in the figure}

\item{plane}{Character string indicating which orientation to plot.
Default: \code{'axial'}}

\item{hemi}{Character string indicating which hemisphere to plot. Default:
\code{'both'}}
}
\description{
\code{plot_brainGraph_multi} writes a PNG file to disk containing three views
(columns) of 1 or more \code{brainGraph} objects (from left-to-right): left
sagittal, axial, and right sagittal. The number of rows in the figure will
equal the number of graphs to plot.

\code{slicer} writes a PNG file to disk containing a single view (i.e.,
either sagittal, axial, or circular) of all \code{brainGraph} objects in the
input list/\code{brainGraphList}.
}
\details{
Whether the first input is a \code{brainGraphList} object or a \code{list} of
\code{brainGraph} objects, \emph{all} graphs in the object will be displayed
in the figure. For \code{plot_brainGraph_multi}, this may be undesirable if
you have more than 4 or 5 graphs in one object. You can choose fewer by using
simple subsetting operations (see Examples below).
}
\note{
All other arguments (passed to \code{\link{plot.brainGraph}}) will be
applied to \emph{all} graphs. For example, if you include
\code{vertex.label=NA} in the function call, vertex labels will be omitted
for all graphs.
}
\section{Using subgraphs, titles, and labels}{

There are three arguments that can differ for each graph to be displayed.
Each follows the same \dQuote{rules}. If you would like the same value
applied to all graphs, you can specify a \code{character} string. If you
would like a different value for each group, you must supply a \code{vector}
or \code{list} with length equal to the number of graphs. If its length is
less than the number of graphs, values will be recycled. To \dQuote{skip}
applying a value to one (or more) graph(s), you can use the \code{NULL} value
only within a list (see the Examples below).
\describe{
  \item{subgraph}{Can be used to apply one or more conditions for subsetting
    the graph(s).}
  \item{main}{Controls the main plot title, which appears in the \emph{axial}
    view along with each graph's \code{name} attribute. Depending on the
    \code{level} of the \code{brainGraphList}, this will either be a Study
    ID, Group name, or contrast name.}
  \item{label}{Can be used to print a text label in a corner for each
    group/graph. For example, you can print a letter if you will refer to,
    e.g., \dQuote{Figure 1A}, \dQuote{Figure 1B}, etc.}
}
}

\examples{
\dontrun{
## "g.hubs" contains 2 groups; apply same subset to both
plot_brainGraph_multi(g.hubs, filename='Figure01_hubs.png',
  subgraph='N > 0', vertex.color='color.lobe', vertex.size=15,
  show.legend=TRUE, vertex.label.cex=1.5)

## Single group, different subgraphs for both plots
## "g" is a "brainGraphList" object
gg <- g[rep(1, 3), drop=FALSE]
plot_brainGraph_multi(gg, filename='group1_5-6-7core.png',
  vertex.color='color.lobe', edge.color='color.lobe', vertex.label=NA,
  subgraph=as.list(paste('coreness >', 5:7)),
  main=as.list(paste('k-core', 5:7)))

## Apply different subset for groups 1 & 3; no subset for group 2
plot_brainGraph_multi(g, groups=1:3, vertex.label=NA,
  subgraph=list('degree > 5', NULL, 'degree > 4'))
}
}
\seealso{
Other Plotting functions: 
\code{\link{Plotting GLM graphs}},
\code{\link{plot.brainGraph}()},
\code{\link{plot.brainGraphList}()}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}
\concept{Plotting functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glm_methods.R
\name{GLM basic info}
\alias{GLM basic info}
\alias{nobs.bg_GLM}
\alias{terms.bg_GLM}
\alias{formula.bg_GLM}
\alias{labels.bg_GLM}
\alias{case.names.bg_GLM}
\alias{variable.names.bg_GLM}
\alias{region.names.bg_GLM}
\alias{nregions.bg_GLM}
\title{Extract basic information from a bg_GLM object}
\usage{
\method{nobs}{bg_GLM}(object, ...)

\method{terms}{bg_GLM}(x, ...)

\method{formula}{bg_GLM}(x, ...)

\method{labels}{bg_GLM}(object, ...)

\method{case.names}{bg_GLM}(object, ...)

\method{variable.names}{bg_GLM}(object, ...)

\method{region.names}{bg_GLM}(object)

\method{nregions}{bg_GLM}(object)
}
\arguments{
\item{...}{Unused}

\item{x, object}{A \code{bg_GLM} object}
}
\value{
\code{terms} returns a named integer list in which the names are the
  term labels and the list elements are the column(s) of the design matrix
  for each term. \code{nobs} returns an integer. The other functions return
  character vectors.
}
\description{
These functions return the \code{terms}, \emph{term labels}, \emph{model
formula}, \dQuote{case names}, \dQuote{variable names}, \emph{region names},
and number of observations for a \code{bg_GLM} object. The term labels are
used for ANOVA tables.
}
\note{
\code{formula} returns a character string, not a \code{formula}
object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glm_stats.R
\name{GLM statistics}
\alias{GLM statistics}
\alias{coef.bg_GLM}
\alias{confint.bg_GLM}
\alias{fitted.bg_GLM}
\alias{residuals.bg_GLM}
\alias{deviance.bg_GLM}
\alias{coeff_determ}
\alias{df.residual.bg_GLM}
\alias{sigma.bg_GLM}
\alias{vcov.bg_GLM}
\alias{coeff_table}
\alias{anova.bg_GLM}
\title{Extract model fit statistics from a bg_GLM object}
\usage{
\method{coef}{bg_GLM}(object, ...)

\method{confint}{bg_GLM}(object, parm, level = 0.95, ...)

\method{fitted}{bg_GLM}(object, ...)

\method{residuals}{bg_GLM}(object, type = c("response", "partial"), ...)

\method{deviance}{bg_GLM}(object, ...)

coeff_determ(object, adjusted = FALSE)

\method{df.residual}{bg_GLM}(object, ...)

\method{sigma}{bg_GLM}(object, ...)

\method{vcov}{bg_GLM}(object, ...)

coeff_table(object, CI = FALSE, level = 0.95)

\method{anova}{bg_GLM}(object, region = NULL, ...)
}
\arguments{
\item{object}{A \code{bg_GLM} object}

\item{...}{Unused}

\item{parm}{Vector of parameters to calculate confidence intervals for.
Default is to use all parameters}

\item{level}{The confidence level. Default: \code{0.95}}

\item{type}{Character string specifying the type of residuals to return.
Default: \code{'response'}}

\item{adjusted}{Logical indicating whether to calculate the adjusted
R-squared. Default: \code{FALSE}}

\item{CI}{Logical indicating whether to include confidence intervals of
parameter estimates in the coefficient summary table. Default: \code{FALSE}}

\item{region}{Character vector indicating the region(s) to calculate ANOVA
statistics for. Default: \code{NULL} (use all regions)}
}
\value{
A named numeric vector, matrix, or array, depending on the function:
  \item{coef}{Matrix in which rownames are parameter names and column names
    are regions}
  \item{fitted,residuals}{Matrix in which rownames are Study ID's and column
    names are regions. If \code{type='partial'}, an array is returned in
    which columns are \emph{terms} and the 3rd dimension are regions}
  \item{deviance,coeff_determ,sigma}{Numeric vector with elements for each
    region}
  \item{df.residual}{Single integer; the degrees of freedom}
  \item{confint,vcov,coeff_table}{Numeric array; the extent of the third
    dimension equals the number of regions}

\code{anova} returns a \emph{list} of tables of class \code{anova}
}
\description{
These functions extract or calculate model fit statistics of a
\code{bg_GLM} object. These can be found in the output from
\code{\link[stats]{summary.lm}}.
}
\details{
These mimic the same functions that operate on \code{\link{lm}} objects, and
include:
\describe{
  \item{coef}{Regression coefficients (parameter estimates)}
  \item{confint}{Confidence intervals (by default, 95\%) for parameter
    estimates}
  \item{fitted}{Fitted (mean) values; i.e., the design matrix multiplied by
    the parameter estimates, \eqn{X \hat{\beta}}}
  \item{residuals}{Model residuals; i.e., the response/outcome variable minus
    the \emph{fitted} values. Partial residuals can also be calculated}
  \item{deviance}{Model deviance, or the \emph{residual sum of squares}}
  \item{coeff_determ}{Calculate the \emph{coefficient of determination} (or
    \eqn{R^2}), adjusted or unadjusted}
  \item{df.residual}{Residual degrees of freedom}
  \item{sigma}{Residual standard deviation, sometimes called the \emph{root
    mean squared error (RMSE)}}
  \item{vcov}{Variance-covariance matrix of the model parameters}
}

\code{coeff_table} returns model coefficients, standard errors, T-statistics,
and P-values for all model terms and regions in a \code{bg_GLM} object. This
is the same as running \code{summary(x)$coefficients} for a \code{lm} object.
}
\note{
\code{sigma} -- The denominator is \emph{not} the number of
  observations, but rather the model's \emph{residual degrees of freedom}.

When calculating \emph{partial residuals}, the parameter estimates are
  \emph{not} re-calculated after removing one of the model terms.
}
\section{ANOVA tables}{

The \code{anova} method calculates the so-called \emph{Type III} test
statistics for a \code{bg_GLM} object. These standard ANOVA statistics
include: sum of squares, mean squares, degrees of freedom, F statistics, and
P-values. Additional statistics calculated are: \eqn{\eta^2}, partial
\eqn{\eta^2}, \eqn{\omega^2}, and partial \eqn{\omega^2} as measures of
\emph{effect size}.
}

\seealso{
\code{\link{GLM}}, \code{\link[car]{Anova}}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glm_stats.R
\name{vif.bg_GLM}
\alias{vif.bg_GLM}
\title{Variance inflation factors for \code{bg_GLM} objects}
\usage{
vif.bg_GLM(mod, ...)
}
\arguments{
\item{mod}{A \code{bg_GLM} object}

\item{...}{Unused}
}
\value{
A named array of VIFs; names of the 3rd dimension are regions
}
\description{
Variance inflation factors for \code{bg_GLM} objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/permute_group.R
\name{brainGraph_permute}
\alias{brainGraph_permute}
\alias{summary.brainGraph_permute}
\alias{plot.brainGraph_permute}
\title{Permutation test for group difference of graph measures}
\usage{
brainGraph_permute(
  densities,
  resids,
  N = 5000,
  perms = NULL,
  auc = FALSE,
  level = c("graph", "vertex", "other"),
  measure = c("btwn.cent", "coreness", "degree", "eccentricity", "clo.cent",
    "communicability", "ev.cent", "lev.cent", "pagerank", "subg.cent", "E.local",
    "E.nodal", "knn", "Lp", "transitivity", "vulnerability"),
  .function = NULL
)

\method{summary}{brainGraph_permute}(
  object,
  measure = object$measure,
  alternative = c("two.sided", "less", "greater"),
  alpha = 0.05,
  p.sig = c("p", "p.fdr"),
  ...
)

\method{plot}{brainGraph_permute}(
  x,
  measure = x$measure,
  alternative = c("two.sided", "less", "greater"),
  alpha = 0.05,
  p.sig = c("p", "p.fdr"),
  ptitle = NULL,
  ...
)
}
\arguments{
\item{densities}{Numeric vector of graph densities}

\item{resids}{An object of class \code{brainGraph_resids} (the output from
\code{\link{get.resid}})}

\item{N}{Integer; the number of permutations (default: 5e3)}

\item{perms}{Numeric matrix of permutations, if you would like to provide
your own (default: \code{NULL})}

\item{auc}{Logical indicating whether or not to calculate differences in the
area-under-the-curve of metrics (default: \code{FALSE})}

\item{level}{A character string for the attribute \dQuote{level} to calculate
differences (default: \code{graph})}

\item{measure}{A character string specifying the vertex-level metric to
calculate, only used if \code{level='vertex'} (default: \code{btwn.cent}).
For the \code{summary} method, this is to focus on a single
\emph{graph-level} measure (since multiple are calculated at once).}

\item{.function}{A custom function you can pass if \code{level='other'}}

\item{object, x}{A \code{brainGraph_permute} object (output by
\code{\link{brainGraph_permute}}).}

\item{alternative}{Character string, whether to do a two- or one-sided test.
Default: \code{'two.sided'}}

\item{alpha}{Numeric; the significance level. Default: 0.05}

\item{p.sig}{Character string specifying which p-value to use for displaying
significant results (default: \code{p})}

\item{...}{Unused}

\item{ptitle}{Character string specifying a title for the plot (default:
\code{NULL})}
}
\value{
An object of class \code{brainGraph_permute} with input arguments in
  addition to:
  \item{DT}{A data table with permutation statistics}
  \item{obs.diff}{A data table of the observed group differences}
  \item{Group}{Group names}

The \code{plot} method returns a \emph{list} of \code{ggplot} objects
}
\description{
\code{brainGraph_permute} draws permutations from linear model residuals to
determine the significance of between-group differences of a global or
vertex-wise graph measure. It is intended for structural covariance networks
(in which there is only one graph per group), but can be extended to other
types of data.
}
\details{
If you would like to calculate differences in the area-under-the-curve (AUC)
across densities, then specify \code{auc=TRUE}.

There are three possible \dQuote{levels}:
\enumerate{
  \item \emph{graph} Calculate modularity (Louvain algorithm), clustering
  coefficient, characteristic path length, degree assortativity, and global
  efficiency.
  \item \emph{vertex} Choose one of: centrality metrics (betweenness,
  closeness, communicability, eigenvector, leverage, pagerank, subgraph);
  k-core; degree; eccentricity; nodal or local efficiency; k-nearest neighbor
  degree; shortest path length; transitivity; or vulnerability.
  \item \emph{other} Supply your own function. This is useful if you want to
  calculate something that I haven't hard-coded. It must take as its own
  arguments: \code{g} (a list of lists of \code{igraph} graph objects); and
  \code{densities} (numeric vector).
}
}
\examples{
\dontrun{
myResids <- get.resid(lhrh, covars)
myPerms <- shuffleSet(n=nrow(myResids$resids.all), nset=1e3)
out <- brainGraph_permute(densities, m, perms=myPerms)
out <- brainGraph_permute(densities, m, perms=myPerms, level='vertex')
out <- brainGraph_permute(densities, m, perms=myPerms,
  level='other', .function=myFun)
}
}
\seealso{
Other Group analysis functions: 
\code{\link{Bootstrapping}},
\code{\link{GLM}},
\code{\link{Mediation}},
\code{\link{NBS}()},
\code{\link{mtpc}()}

Other Structural covariance network functions: 
\code{\link{Bootstrapping}},
\code{\link{IndividualContributions}},
\code{\link{Residuals}},
\code{\link{corr.matrix}()},
\code{\link{import_scn}()},
\code{\link{plot_volumetric}()}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}
\concept{Group analysis functions}
\concept{Structural covariance network functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/analysis_random_graphs.R, R/random_graphs.R
\name{Random Graphs}
\alias{Random Graphs}
\alias{analysis_random_graphs}
\alias{sim.rand.graph.par}
\alias{sim.rand.graph.clust}
\alias{sim.rand.graph.hqs}
\title{Perform an analysis with random graphs for brain MRI data}
\usage{
analysis_random_graphs(
  g.list,
  level = g.list[[1L]]$level,
  N = 100L,
  savedir = ".",
  ...
)

sim.rand.graph.par(
  g,
  level = c("subject", "group"),
  N = 100L,
  clustering = FALSE,
  rewire.iters = max(10 * ecount(g), 10000L),
  cl = g$transitivity,
  max.iters = 100L,
  ...
)

sim.rand.graph.clust(
  g,
  rewire.iters = 10000,
  cl = g$transitivity,
  max.iters = 100
)

sim.rand.graph.hqs(
  resids,
  level = c("subject", "group"),
  N = 100L,
  weighted = TRUE,
  r.thresh = NULL,
  ...
)
}
\arguments{
\item{g.list}{List of \code{brainGraphList} objects; the length of this list
should equal the number of thresholds/densities in the study}

\item{level}{Character string indicating whether the graphs are subject-,
group-, or contrast-specific. Default: \code{'subject'}}

\item{N}{Integer; the number of random graphs to simulate. Default: 100}

\item{savedir}{Character string specifying the directory in which to save the
generated graphs. Default: current working directory}

\item{...}{Other arguments passed to \code{\link{make_brainGraph}}}

\item{g}{A graph object}

\item{clustering}{Logical; whether or not to control for clustering. Default:
\code{FALSE}}

\item{rewire.iters}{Integer; number of rewiring iterations for the initial
graph randomization. Default: 1e4}

\item{cl}{The clustering measure. Default: \emph{transitivity}}

\item{max.iters}{The maximum number of iterations to perform; choosing a
lower number may result in clustering that is further away from the
observed graph's. Default: 100}

\item{resids}{A \code{brainGraph_resids} object, a \code{data.table} of
residuals, or a numeric matrix}

\item{weighted}{Logical indicating whether to create weighted graphs. If
true, a threshold must be provided.}

\item{r.thresh}{Numeric value for the correlation threshold, if
\code{weighted=FALSE}.}
}
\value{
\code{analysis_random_graphs} returns a \emph{list} containing:
\item{rich}{A data table containing normalized rich-club coefficients and
  p-values}
\item{small}{A data table with small-world parameters}
\item{rand}{A data table with some global graph measures for all random
  graphs generated}

\code{sim.rand.graph.par} - a \emph{list} of \emph{N} random graphs
  with some additional vertex and graph attributes

\code{sim.rand.graph.clust} - A single \code{igraph} graph object

\code{sim.rand.graph.hqs} - A list of random graphs from the null
  covariance matrices
}
\description{
\code{analysis_random_graphs} performs the steps needed for doing typical
graph theory analyses with brain MRI data if you need to generate equivalent
random graphs. This includes calculating \emph{small world} parameters and
normalized \emph{rich club} coefficients.

\code{sim.rand.graph.par} simulates \code{N} simple random graphs with the
same clustering (optional) and degree sequence as the input. Essentially a
wrapper for \code{\link[igraph]{sample_degseq}} (or, if you want to match by
clustering, \code{\link{sim.rand.graph.clust}}) and
\code{\link{make_brainGraph}}. It uses \code{\link[foreach]{foreach}} for
parallel processing.

\code{sim.rand.graph.clust} simulates a random graph with a given degree
sequence \emph{and} clustering coefficient. Increasing the \code{max.iters}
value will result in a closer match of clustering with the observed graph.

\code{sim.rand.graph.hqs} generates a number of random covariance matrices
using the Hirschberger-Qi-Steuer (HQS) algorithm, and create graphs from
those matrices.
}
\details{
\code{analysis_random_graphs} does the following:
\enumerate{
  \item Generate \code{N} random graphs for each graph and density/threshold
  \item Write graphs to disk in \code{savedir}. Read them back into \code{R}
    and combine into lists; then write these lists to disk. You can later
    delete the individual \code{.rds} files afterwards.
  \item Calculate \emph{small world} parameters, along with values for a few
    global graph measures that may be of interest.
  \item Calculate \emph{normalized rich club coefficients} and associated
    p-values.
}

If you do not want to match by clustering, then simple rewiring of the input
graph is performed (the number of rewires equaling the larger of \code{1e4}
and \eqn{10 \times m}, where \eqn{m} is the graph's edge count).

\code{sim.rand.graph.hqs} - The first step is to create the observed
covariance of residuals (or whatever matrix/data.table is provided). Then
random covariance matrices are created with the same distributional
properties as the observed matrix, they are converted to correlation
matrices, and finally graphs from these matrices. By default, weighted graphs
will be created in which the edge weights represent correlation values. If
you want binary matrices, you must provide a correlation threshold.
}
\examples{
\dontrun{
rand_all <- analysis_random_graphs(g.norm, 1e2,
  savedir='/home/cwatson/dti/rand', clustering=F)
}
\dontrun{
rand1 <- sim.rand.graph.par(g[[1]][[N]], N=1e3)
rand1.cl <- sim.rand.graph.par(g[[1]][[N]], N=1e2,
  clustering=T, max.iters=1e3)
}
}
\references{
Bansal, S. and Khandelwal, S. and Meyers, L.A. (2009) Exploring
  biological network structure with clustered random networks. \emph{BMC
  Bioinformatics}, \bold{10}, 405--421.
  \doi{10.1186/1471-2105-10-405}

Hirschberger M., Qi Y., Steuer R.E. (2007) Randomly generating
  portfolio-selection covariance matrices with specified distributional
  characteristics. \emph{European Journal of Operational Research}.
  \bold{177}, 1610--1625. \doi{10.1016/j.ejor.2005.10.014}
}
\seealso{
\code{\link{small.world}}

\code{\link[igraph]{rewire}, \link[igraph]{sample_degseq},
  \link[igraph]{keeping_degseq}}

\code{\link[igraph]{transitivity}}

Other Random graph functions: 
\code{\link{Rich Club}}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}
\concept{Random graph functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/brainGraph_mediate.R
\name{Mediation}
\alias{Mediation}
\alias{brainGraph_mediate}
\alias{summary.bg_mediate}
\alias{bg_to_mediate}
\title{Mediation analysis with brain graph measures as mediator variables}
\usage{
brainGraph_mediate(
  g.list,
  covars,
  mediator,
  treat,
  outcome,
  covar.names,
  level = c("graph", "vertex"),
  control.value = 0,
  treat.value = 1,
  int = FALSE,
  boot = TRUE,
  boot.ci.type = c("perc", "bca"),
  N = 1000,
  conf.level = 0.95,
  long = FALSE,
  ...
)

\method{summary}{bg_mediate}(
  object,
  mediate = FALSE,
  region = NULL,
  digits = max(3L, getOption("digits") - 2L),
  ...
)

bg_to_mediate(x, region = NULL)
}
\arguments{
\item{g.list}{A \code{brainGraphList} object}

\item{covars}{A data table containing covariates of interest. It must include
columns for \code{getOption('bg.subject_id')}, \code{treat},
\code{outcome}, and \code{covar.names}.}

\item{mediator}{Character string; the name of the graph measure acting as
the \emph{mediating} variable}

\item{treat}{Character string; the \emph{treatment} variable (e.g.,
\emph{Group})}

\item{outcome}{Character string; the name of the outcome variable of interest}

\item{covar.names}{Character vector of the column name(s) in \code{covars} to
include in the models as pre-treatment covariate(s).}

\item{level}{Character string; either \code{vertex} (default) or
\code{graph}}

\item{control.value}{Value of \code{treat} to be used as the control
condition. Default: \code{0}}

\item{treat.value}{Value of \code{treat} to be used as the treatment
condition. Default: \code{1}}

\item{int}{Logical indicating whether or not to include an interaction of the
mediator and treatment. Default: \code{FALSE}}

\item{boot}{Logical indicating whether or not to perform bootstrapping. This
should always be done. Default: \code{TRUE}}

\item{boot.ci.type}{Character string; which type of CI's to calculate.
Default: \code{perc}}

\item{N}{Integer; the number of bootstrap samples to run. Default:
\code{1e3}}

\item{conf.level}{Numeric between 0 and 1; the level of the CI's to
calculate. Default: \code{0.95} for the 2.5 and 97.5 percentiles)}

\item{long}{Logical indicating whether or not to return all bootstrap
samples. Default: \code{FALSE}}

\item{...}{Other arguments passed to \code{\link{brainGraph_GLM_design}}
(e.g., \code{binarize}) (unused in the \code{summary} method)}

\item{object}{A \code{bg_mediate} object}

\item{mediate}{Logical indicating whether or not to use the \code{summary}
method from \code{\link[mediation]{mediate}} (default: \code{FALSE}). If
\code{TRUE}, only a single region can be printed.}

\item{region}{Character string specifying which region's results to
summarize; only relevant if \code{level='vertex'} (default: \code{NULL})}

\item{digits}{Integer specifying the number of digits to display for P-values}

\item{x}{Object output from \code{\link{brainGraph_mediate}}}
}
\value{
An object of class \code{bg_mediate} with elements:
  \item{level}{Either \code{graph} or \code{vertex}.}
  \item{removed.subs}{A character vector of Study.ID's removed due to
    incomplete data}
  \item{X.m, X.y}{Design matrix and numeric array for the model with the
    mediator as the outcome variable (\code{X.m}) and for the model with the
    mediator as an additional predictor (\code{X.y}), respectively}
  \item{y.m, y.y}{Outcome variables for the associated design matrices above.
    \code{y.m} will be a matrix of size \emph{# subj. X # regions}}
  \item{res.obs}{A \code{data.table} of the observed values of the point
    estimates.}
  \item{res.ci}{A \code{data.table} of the confidence intervals for the
    effect estimates.}
  \item{res.p}{A \code{data.table} of the two-sided p-values for the effect
    estimates}
  \item{boot}{Logical, the \code{boot} argument.}
  \item{boot.ci.type}{Character string indicating which type of bootstrap
    confidence intervals were calculated.}
  \item{res.boot}{A \code{data.table} with \code{N} rows of the bootstrap
    results for all effects.}
  \item{treat}{Character string of the treatment variable.}
  \item{mediator}{Character string of the mediator variable.}
  \item{outcome}{Character string of the outcome variable.}
  \item{covariates}{Returns \code{NULL}; not used in this package.}
  \item{INT}{Logical indicating whether the models included an interaction
    between treatment and mediator.}
  \item{conf.level}{The confidence level.}
  \item{control.value}{The value of the treatment variable used as the
    control condition.}
  \item{treat.value}{The value of the treatment variable used as the
    treatment condition.}
  \item{nobs}{Integer; the number of observations in the models.}
  \item{sims}{Integer; the number of bootstrap replications.}
  \item{covar.names}{The pre-treatment covariate names.}

\code{bg_to_mediate} returns an object of class \code{mediate}
}
\description{
\code{brainGraph_mediate} performs simple mediation analyses in which a given
graph- or vertex-level measure (e.g., \emph{weighted global efficiency}) is
the mediator \emph{M}. The outcome (or dependent/response) variable \emph{Y}
can be a neuropsychological measure (e.g., \emph{IQ}) or can be a
disease-specific metric (e.g., recovery time).

\code{\link{bg_to_mediate}} converts the results into an object of class
\code{\link[mediation]{mediate}}. In \code{brainGraph}, it is only used for
the \code{\link[mediation]{summary.mediate}} method, but you can similarly
use its output for the \code{\link[mediation]{plot.mediate}} method.
}
\details{
This code was adapted closely from \code{\link[mediation]{mediate}} in the
\code{mediation} package, and the procedure is exactly the same as theirs
(see the references listed below). If you use this function, please cite
their work.
}
\note{
As of \code{brainGraph v2.0.0}, this function has been tested only for
a treatment (independent) variable \emph{X} being a \emph{factor} (e.g.,
disease group, old vs. young, etc.). If your treatment variable has more
than 2 levels, then you must explicitly specify the levels you would like to
compare; otherwise, the baseline and first levels are taken to be the
control and treatment values, respectively. Be aware that these are \emph{0}
indexed; that is, if you have 3 groups and you would like the treatment
group to be the 3rd, you should specify as either the group's character
string or as \code{treat.value=2}.

Allowing for treatment-mediator interaction (setting \code{int=TRUE})
currently will only work properly if the mediator is a continuous variable;
since the mediator is always a graph metric, this should always be the case.
}
\examples{
\dontrun{
med.EglobWt.FSIQ <- brainGraph_mediate(g[[5]], covars.med, 'E.global.wt',
  'Group', 'FSIQ', covar.names=c('age', 'gender'), N=1e4)
med.strength.FSIQ <- brainGraph_mediate(g[[5]], covars.med, 'strength',
  'Group', 'FSIQ', covar.names=c('age', 'gender'), level='vertex')
}
}
\references{
Tingley, D. and Yamamoto, T. and Hirose, K. and Keele, L. and
  Imai, K. (2014) mediation: R package for causal mediation analysis.
  \emph{Journal of Statistical Software}, \bold{59(5)}, 1--38.
  \doi{10.18637/jss.v059.i05}

Imai, K. and Keele, L. and Yamamoto, T. (2010) Identification
  inference, and sensitivity analysis for causal mediation effects.
  \emph{Statistical Science}, \bold{25(1)}, 51--71.
  \doi{10.1214/10-STS321}

Imai, K. and Keele, L. and Tingley, D. (2010) A general approach
  to causal mediation analysis. \emph{Psychological Methods}, \bold{15(4)},
  309--334. \doi{10.1037/a0020761}

Imai, K. and Keele, L. and Tingley, D. and Yamamoto, T. (2011)
  Unpacking the black box of causality: learning about causal mechanisms from
  experimental and observational studies. \emph{American Political Science
  Review}, \bold{105(4)}, 765--789.
  \doi{10.1017/S0003055411000414}

Imai, K. and Yamamoto, T. (2013) Identification and sensitivity
  analysis for multiple causal mechanisms: revisiting evidence from framing
  experiments. \emph{Political Analysis}, \bold{21(2)}, 141--171.
  \doi{10.1093/pan/mps040}
}
\seealso{
\code{\link[mediation]{mediate}}

Other Group analysis functions: 
\code{\link{Bootstrapping}},
\code{\link{GLM}},
\code{\link{NBS}()},
\code{\link{brainGraph_permute}()},
\code{\link{mtpc}()}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}
\concept{Group analysis functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/s_core.R
\name{s_core}
\alias{s_core}
\title{Calculate the s-core of a network}
\usage{
s_core(g, W = NULL)
}
\arguments{
\item{g}{An \code{igraph} graph object}

\item{W}{Numeric matrix of edge weights (default: \code{NULL})}
}
\value{
Integer vector of the vertices' \emph{s-core} membership
}
\description{
Calculates the \emph{s-core} decomposition of a network. This is analogous to
the \emph{k-core} decomposition, but takes into account the \emph{strength}
of vertices (i.e., in weighted networks). If an unweighted network is
supplied, then the output of the function \code{\link[igraph]{coreness}} is
returned.
}
\details{
The \emph{s-core} consists of all vertices \eqn{i} with \eqn{s_i > s}, where
\eqn{s} is some threshold value. The \eqn{s_0} core is the entire network,
and the threshold value of the \eqn{s_{n}} core is
\deqn{s_{n-1} = min_i s_i}
for all vertices \eqn{i} in the \eqn{s_{n-1}} core.

Note that in networks with a wide distribution of vertex strengths, in which
there are almost as many unique values as there are vertices, then several
separate cores will have a single vertex. See the reference provided below.
}
\references{
Eidsaa, M and Almaas, E. (2013) s-core network decomposition: a
  generalization of k-core analysis to weighted networks. \emph{Physical
  Review E}, \bold{88}, 062819.
  \doi{10.1103/PhysRevE.88.062819}
}
\seealso{
\code{\link[igraph]{coreness}}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/analysis_random_graphs.R
\name{get_rand_attrs}
\alias{get_rand_attrs}
\title{Convenience function to get attributes for lists of random graphs}
\usage{
get_rand_attrs(bg.list, level)
}
\arguments{
\item{bg.list}{A \code{brainGraphList} object, created in
\code{\link{analysis_random_graphs}}}
}
\value{
A \code{data.table}
}
\description{
Convenience function to get attributes for lists of random graphs
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hubs.R
\name{hubness}
\alias{hubness}
\title{Calculate vertex hubness}
\usage{
hubness(g, xfm.type = g$xfm.type, weights = NULL, prop.keep = 0.2)
}
\arguments{
\item{g}{An \code{igraph} graph object}

\item{xfm.type}{Character string specifying how to transform the weights.
Default: \code{1/w}}

\item{weights}{Numeric vector of edge weights; if \code{NULL} (the default),
and if the graph has edge attribute \code{weight}, then that will be used.
To avoid using weights, this should be \code{NA}.}

\item{prop.keep}{Numeric (between 0 and 1) indicating the proportion of
vertices to consider as having a high score. Default: 0.2 (20\%)}
}
\value{
A numeric vector with the vertices' hubness score
}
\description{
\code{hubness} calculates the \dQuote{hubness} (see reference) of the
vertices in a graph. These are vertices which meet at least two of the
following four criteria:
\enumerate{
  \item Have high degree/strength
  \item Have high betweenness centrality
  \item Have low clustering coefficient
  \item Have low average path length
}
For each criterion, \dQuote{high} or \dQuote{low} means \dQuote{in the top
20\%} across all vertices. Vertices meeting any of the criteria get a value
of 1 for that metric; these are summed to yield the hubness score which
ranges from 0-4. As in the reference article, vertices with a score of 2 or
higher are to be considered hubs, although that determination isn't made in
this function.
}
\references{
van den Heuvel, M.P. and Mandl, R.C.W. and Stam, C.J. and Kahn,
  R.S. and Pol, H.E.H. (2010) Aberrant frontal and temporal complex network
  structure in schizophrenia: a graph theoretical analysis. \emph{The Journal
  of Neuroscience}, \bold{30(47)}, 15915--15926.
  \doi{10.1523/JNEUROSCI.2874-10.2010}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/set_brainGraph_attributes.R
\name{delete_all_attr}
\alias{delete_all_attr}
\title{Delete all attributes of a graph}
\usage{
delete_all_attr(g, keep.names = FALSE)
}
\arguments{
\item{g}{An \code{igraph} graph object}

\item{keep.names}{Logical indicating whether to keep the \code{name} vertex
attribute (default: \code{FALSE})}
}
\value{
An \code{igraph} graph object
}
\description{
Deletes all graph-, vertex-, and edge-level attributes of an \code{igraph}
graph object.
}
\seealso{
\code{\link[igraph]{delete_graph_attr},
  \link[igraph]{delete_vertex_attr}, \link[igraph]{delete_edge_attr}}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/communicability.R
\name{centr_betw_comm}
\alias{centr_betw_comm}
\title{Calculate communicability betweenness centrality}
\usage{
centr_betw_comm(g, A = NULL)
}
\arguments{
\item{g}{An \code{igraph} graph object}

\item{A}{Numeric matrix; the adjacency matrix of the input graph. Default:
\code{NULL}}
}
\value{
A numeric vector of the centrality for each vertex
}
\description{
\code{centr_betw_comm} calculates the \emph{communicability betweenness} of
the vertices of a graph. The centrality for vertex \code{r} is
\deqn{\omega_r = \frac{1}{C} \sum_p \sum_q \frac{(e^{\mathbf{A}})_{pq} -
(e^{\mathbf{A} + \mathbf{E}(r)})_{pq}}{(e^{\mathbf{A}})_{pq}}}
where \eqn{C = (n - 1)^2 - (n - 1)} is a normalization factor.
}
\references{
Estrada, E. and Higham, D.J. and Hatano N. (2009) Communicability
  betweenness in complex networks. \emph{Physica A}, \bold{388}, 764--774.
  \doi{10.1016/j.physa.2008.11.011}
}
\seealso{
Other Centrality functions: 
\code{\link{centr_lev}()}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}
\concept{Centrality functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vertex_roles.R
\name{Vertex Roles}
\alias{Vertex Roles}
\alias{gateway_coeff}
\alias{part_coeff}
\alias{within_module_deg_z_score}
\title{Gateway coefficient, participation coefficient, and within-mod degree z-score}
\usage{
gateway_coeff(
  g,
  memb,
  centr = c("btwn.cent", "degree", "strength"),
  A = NULL,
  weighted = FALSE
)

part_coeff(g, memb, A = NULL, weighted = FALSE)

within_module_deg_z_score(g, memb, A = NULL, weighted = FALSE)
}
\arguments{
\item{g}{An \code{igraph} graph object}

\item{memb}{A numeric vector of membership indices of each vertex}

\item{centr}{Character string; the type of centrality to use in calculating
GC. Default: \code{btwn.cent}}

\item{A}{Numeric matrix; the adjacency matrix of the input graph. Default:
\code{NULL}}

\item{weighted}{Logical indicating whether to calculate metrics using edge
weights. Default: \code{FALSE}}
}
\value{
A vector of the participation coefficients, within-module degree
  z-scores, or gateway coefficients for each vertex of the graph.
}
\description{
\code{gateway_coeff} calculates the gateway coefficient of each vertex,
based on community membership.

\code{part_coeff} calculates the participation coefficient of each vertex,
based on community membership.

\code{within_module_deg_z_score} is a measure of the connectivity from a
given vertex to other vertices in its module/community.
}
\details{
The gateway coefficient \eqn{G_i} of vertex \emph{i} is:
\deqn{G_i = 1 - \sum_{S=1}^{N_M} \left ( \frac{\kappa_{iS}}{\kappa_i} \right
)^2 (g_{iS})^2}
where \eqn{\kappa_{iS}} is the number of edges from vertex \emph{i} to
vertices in module \emph{S}, and \eqn{\kappa_i} is the degree of vertex
\emph{i}. \eqn{N_M} equals the number of modules. \eqn{g_{ii}} is a weight,
defined as:
\deqn{g_{iS} = 1 - \bar{\kappa_{iS}} \bar{c_{iS}}}
where
\deqn{\bar{\kappa_{iS}} = \frac{\kappa_{iS}}{\sum_j \kappa_{jS}}}
for all nodes \eqn{j} in node \eqn{i}'s module, and
\deqn{\bar{c_{iS}} = c_{iS} / max(c_n)}

The participation coefficient \eqn{P_i} of vertex \emph{i} is:
\deqn{P_i = 1 - \sum_{s=1}^{N_M} \left ( \frac{\kappa_{is}}{\kappa_i} \right )^2}
where \eqn{\kappa_{is}} is the number of edges from vertex \emph{i} to
vertices in module \emph{s}, and \eqn{\kappa_s} is the degree of vertex
\emph{i}. \eqn{N_M} equals the number of modules.

As discussed in Guimera et al., \eqn{P_i = 0} if vertex \emph{i} is connected
only to vertices in the same module, and \eqn{P_i = 1} if vertex \emph{i} is
equally connected to all other modules.

The within-module degree z-score is:
\deqn{z_i = \frac{\kappa_i - \bar{\kappa}_{s_i}}{\sigma_{\kappa_{s_i}}}}
where \eqn{\kappa_i} is the number of edges from vertex \emph{i} to vertices
in the same module \eqn{s_i}, \eqn{\bar{\kappa}_{s_i}} is the average of
\eqn{\kappa} over all vertices in \eqn{s_i}, and \eqn{\sigma_{\kappa_{s_i}}}
is the standard deviation.
}
\references{
Vargas, E.R. and Wahl, L.M. (2014) The gateway coefficient: a
  novel metric for identifying critical connections in modular networks.
  \emph{Eur Phys J B}, \bold{87}, 161--170.
  \doi{10.1140/epjb/e2014-40800-7}

Guimera, R. and Amaral, L.A.N. (2005) Cartography of complex
  networks: modules and universal roles. \emph{Journal of Statistical
  Mechanics: Theory and Experiment}, \bold{02}, P02001.
  \doi{10.1088/1742-5468/2005/02/P02001}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_global.R
\name{rename_levels}
\alias{rename_levels}
\title{Rename the levels of global metrics in a data.table}
\usage{
rename_levels(DT)
}
\arguments{
\item{DT}{A \code{data.table}}
}
\value{
The levels of the \code{variable} column after being changed
}
\description{
Rename the levels of global metrics in a data.table
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/corr_matrix.R, R/methods.R
\name{corr.matrix}
\alias{corr.matrix}
\alias{Extract.corr_mats}
\alias{[.corr_mats}
\alias{plot.corr_mats}
\alias{region.names.corr_mats}
\alias{nregions.corr_mats}
\title{Calculate correlation matrix and threshold}
\usage{
corr.matrix(
  resids,
  densities,
  thresholds = NULL,
  what = c("resids", "raw"),
  exclude.reg = NULL,
  type = c("pearson", "spearman"),
  rand = FALSE
)

\method{[}{corr_mats}(x, i, g = NULL)

\method{plot}{corr_mats}(
  x,
  mat.type = c("thresholded", "raw"),
  thresh.num = 1L,
  ordered = TRUE,
  order.by = "lobe",
  graphs = NULL,
  grp.names = NULL,
  legend.title = NULL,
  ...
)

\method{region.names}{corr_mats}(object)

\method{nregions}{corr_mats}(object)
}
\arguments{
\item{resids}{An object of class \code{brainGraph_resids} (the output from
\code{\link{get.resid}})}

\item{densities}{Numeric vector indicating the resultant network
densities; keeps the top \emph{X}\% of correlations}

\item{thresholds}{Numeric; absolute correlation value to threshold by
(default: \code{NULL})}

\item{what}{Character string indicating whether to correlate the residuals or
the raw structural MRI values (default: \code{'resids'})}

\item{exclude.reg}{Character vector of regions to exclude (default:
\code{NULL})}

\item{type}{Character string indicating which type of correlation coefficient
to calculate (default: \code{'pearson'})}

\item{rand}{Logical indicating whether the function is being called for
permutation testing; not intended for general use (default: \code{FALSE})}

\item{x, object}{A \code{corr_mats} object}

\item{i}{Integer for subsetting by density/threshold}

\item{g}{Integer, character, or logical for subsetting by group}

\item{mat.type}{Character string indicating whether to plot raw or thresholded
(binarized) matrices. Default: \code{'raw'}}

\item{thresh.num}{Integer specifying which threshold to plot (if
\code{mat.type='thresholded'}). Default: \code{1L}}

\item{ordered}{Logical indicating whether to order the vertices by some
grouping. Default: \code{TRUE}}

\item{order.by}{Character string indicating how to group vertices. Default:
\code{'lobe'}}

\item{graphs}{A \code{brainGraphList} object containing graphs with the
vertex-level attribute of interest. Default: \code{NULL}}

\item{grp.names}{Character vector specifying the names of each group of
vertices. Default: \code{NULL}}

\item{legend.title}{Character string for the legend title. Default is to
leave blank}

\item{...}{Unused}
}
\value{
A \code{corr_mats} object containing the following components:
  \item{R,P}{Numeric arrays of correlation coefficients and P-values. The
    length of the 3rd dimension equals the number of groups}
  \item{r.thresh}{A list of 3-d binary arrays indicating correlations that
    are above a certain threshold. The length of the list equals the number
    of groups, and the length of the 3rd dimension equals the number of
    thresholds/densities.}
  \item{thresholds}{Numeric matrix of the thresholds supplied. The number of
    columns equals the number of groups.}
  \item{what}{Residuals or raw values}
  \item{exclude.reg}{Excluded regions (if any)}
  \item{type}{Pearson or Spearman}
  \item{atlas}{The brain atlas used}
  \item{densities}{Numeric vector; the densities of the resulting graphs, if
    you chose to threshold each group to have equal densities.}
}
\description{
\code{corr.matrix} calculates the correlation between all column pairs of a
given data frame, and thresholds the resultant correlation matrix based on a
given density (e.g., \code{0.1} if you want to keep only the 10\% strongest
correlations). If you want to threshold by a specific correlation coefficient
(via the \code{thresholds} argument), then the \code{densities} argument is
ignored.

The \code{plot} method will plot \dQuote{heat maps} of the correlation
matrices.
}
\details{
If you wish to exclude regions from your analysis, you can give the indices
of their columns with the \code{exclude.reg} argument.

By default, the Pearson correlation coefficients are calculated, but you can
return Spearman by changing the \code{type} argument.
}
\section{Plotting correlation matrices}{

There are several ways to control the plot appearance. First, you may plot
the \dQuote{raw} correlations, or only those of the thresholded (binarized)
matrices. Second, you may order the vertices by a given vertex attribute; by
default, they will be ordered by \emph{lobe}, but you may also choose to
order by, e.g., \emph{network} (for the \code{dosenbach160} atlas) or by
\emph{community membership}. In the latter case, you need to pass a
\code{brainGraphList} object to the \code{graphs} argument; each graph in the
object must have a vertex attribute specified in \code{order.by}. Finally,
you can control the legend text with \code{grp.names}.
}

\examples{
\dontrun{
myResids <- get.resid(lhrh, covars)
corrs <- corr.matrix(myResids, densities=densities)))
}
\dontrun{
corrs <- corr.matrix(myResids, densities)
plot(corrs, order.by='comm', graphs=g.list, grp.names='Community')
}
}
\seealso{
\code{\link[Hmisc]{rcorr}}

Other Structural covariance network functions: 
\code{\link{Bootstrapping}},
\code{\link{IndividualContributions}},
\code{\link{Residuals}},
\code{\link{brainGraph_permute}()},
\code{\link{import_scn}()},
\code{\link{plot_volumetric}()}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}
\concept{Structural covariance network functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_tables.R
\name{Graph Data Tables}
\alias{Graph Data Tables}
\alias{graph_attr_dt}
\alias{vertex_attr_dt}
\title{Create a data table with graph global and vertex measures}
\usage{
graph_attr_dt(bg.list)

vertex_attr_dt(bg.list)
}
\arguments{
\item{bg.list}{A \code{brainGraphList} object, or a list of graph objects}
}
\value{
A \code{data.table}
}
\description{
\code{graph_attr_dt} is a helper function that takes a \code{brainGraphList}
or a list of graphs and creates a \code{data.table} of global measures for
each graph. Each row will be for a different graph.

\code{vertex_attr_dt} is a helper function that creates a \code{data.table}
in which each row is a vertex and each column is a different network measure
(degree, centrality, etc.).
}
\seealso{
\code{\link[igraph]{graph_attr}, \link[igraph]{graph_attr_names}}

\code{\link[igraph]{vertex_attr}, \link[igraph]{vertex_attr_names},
\link[igraph]{graph_from_data_frame}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/edge_asymmetry.R
\name{edge_asymmetry}
\alias{edge_asymmetry}
\title{Calculate an asymmetry index based on edge counts}
\usage{
edge_asymmetry(g, level = c("hemi", "vertex"), A = NULL)
}
\arguments{
\item{g}{An \code{igraph} graph object}

\item{level}{Character string indicating whether to calculate asymmetry for
each region, or the hemisphere as a whole (default: \code{'hemi'})}

\item{A}{Numeric matrix; the adjacency matrix of the input graph. Default:
\code{NULL}}
}
\value{
A data table with edge counts for both hemispheres and the asymmetry
  index; if \code{level} is \emph{vertex}, the data table will have
  \code{vcount(g)} rows.
}
\description{
Calculate an \emph{asymmetry index}, a ratio of intra-hemispheric edges in
the left to right hemisphere of a graph for brain MRI data.
}
\details{
The equation is:
\deqn{A = \frac{E_{lh} - E_{rh}}{0.5 \times (E_{lh} + E_{rh})}}
where \emph{lh} and \emph{rh} are left and right hemispheres, respectively.
The range of this measure is \eqn{[-2, 2]} (although the limits will only be
reached if all edges are in one hemisphere), with negative numbers
indicating more edges in the right hemisphere, and a value of 0 indicating
equal number of edges in each hemisphere.

The \code{level} argument specifies whether to calculate asymmetry for each
vertex, or for the whole hemisphere.
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/small_world.R
\name{small.world}
\alias{small.world}
\title{Calculate graph small-worldness}
\usage{
small.world(g.list, rand)
}
\arguments{
\item{g.list}{A \code{brainGraphList} object or list of graphs}

\item{rand}{List of (lists of) equivalent random graphs (output from
\code{\link{sim.rand.graph.par}})}
}
\value{
A \code{data.table} with the following components:
\item{density}{The range of density thresholds used.}
\item{N}{The number of random graphs that were generated.}
\item{Lp,Lp.rand,Lp.norm}{The observed, average random, and normalized
  characteristic path length.}
\item{Cp,Cp.rand,Cp.norm}{The observed, average random, and normalized
  clustering coefficient.}
\item{sigma}{The small-world measure of the graph.}
}
\description{
\code{small.world} calculates the normalized characteristic path length and
clustering coefficient based on observed and random graphs, used to calculate
the small-world coefficient \eqn{\sigma}.
}
\references{
Watts, D.J. and Strogatz S.H. (1998) Collective dynamics of
  "small-world" networks. \emph{Nature}, \bold{393}, 440--442.
  \doi{10.1038/30918}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_rich_norm.R
\name{plot_rich_norm}
\alias{plot_rich_norm}
\title{Plot normalized rich club coefficients against degree threshold}
\usage{
plot_rich_norm(
  rich.dt,
  facet.by = c("density", "threshold"),
  densities,
  alpha = 0.05,
  fdr = TRUE,
  g.list = NULL,
  smooth = TRUE
)
}
\arguments{
\item{rich.dt}{A \code{data.table} with rich-club coefficients}

\item{facet.by}{A character string indicating whether the variable of
interest is \dQuote{density} or \dQuote{threshold} (e.g. with DTI data)}

\item{densities}{A numeric vector of the densities to plot}

\item{alpha}{The significance level. Default: \code{0.05}}

\item{fdr}{A logical, indicating whether or not to use the FDR-adjusted
p-value for determining significance. Default: \code{TRUE}}

\item{g.list}{A list \code{brainGraphList} objects; required if you want to
plot a shaded region demarcating the \code{\link{rich_core}}}

\item{smooth}{Logical indicating whether or not to plot a smooth curve when
data from multiple subjects (per group) are present. Default: \code{TRUE}.
Ignored for group-level data.}
}
\value{
A \code{trellis} or \code{ggplot} object
}
\description{
Returns a line plot of the normalized rich club coefficient. Optionally, can
include a shaded region demarcating the \code{\link{rich_core}} cutoff (if
you supply a list of graph objects to the \code{g} argument).
}
\examples{
\dontrun{
plot_rich_norm(rich.dt, facet.by='density', densities[N:(N+1)], g=g)
}
}
\seealso{
Other Rich-club functions: 
\code{\link{Rich Club}},
\code{\link{rich_club_attrs}()}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}
\concept{Rich-club functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/centr_lev.R
\name{centr_lev}
\alias{centr_lev}
\title{Calculate a vertex's leverage centrality}
\usage{
centr_lev(g, A = NULL)
}
\arguments{
\item{g}{An \code{igraph} graph object}

\item{A}{Numeric matrix; the adjacency matrix of the input graph. Default:
\code{NULL}}
}
\value{
A vector of the leverage centrality for all vertices.
}
\description{
Calculates the leverage centrality of each vertex in a graph.
}
\details{
The leverage centrality relates a vertex's degree with the degree of its
neighbors. The equation is:
\deqn{l_i = \frac{1}{k_i} \sum_{j \in N_i} \frac{k_i - k_j}{k_i + k_j}}
where \eqn{k_i} is the degree of the \eqn{i^{th}} vertex and \eqn{N_i} is the
set of neighbors of \emph{i}. This function replaces \emph{NaN} with
\emph{NA} (for functions that have the argument \emph{na.rm}).
}
\references{
Joyce, K.E. and Laurienti P.J. and Burdette J.H. and Hayasaka S.
(2010) A new measure of centrality for brain networks. \emph{PLoS One},
\bold{5(8)}, e12200. \doi{10.1371/journal.pone.0012200}
}
\seealso{
Other Centrality functions: 
\code{\link{centr_betw_comm}()}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}
\concept{Centrality functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_group_means.R
\name{plot_volumetric}
\alias{plot_volumetric}
\title{Plot group distributions of volumetric measures for a given brain region}
\usage{
plot_volumetric(
  dat,
  regions,
  type = c("violin", "histogram"),
  all.vals = TRUE,
  modality = c("thickness", "volume", "lgi", "area")
)
}
\arguments{
\item{dat}{A data table of volumetric data; needs columns for 'Group',
'region', and 'value'}

\item{regions}{A vector of character strings or integers of the brain
region(s) to plot; if integer, the region(s) is/are chosen from the input
data table based on the index}

\item{type}{A character string indicating the plot type; either 'histogram'
or 'violin'}

\item{all.vals}{A logical indicating whether or not to plot horizontal lines
for all observations (only valid for 'violin' plots) (default: TRUE)}

\item{modality}{A character string indicating the type of volumetric measure
('thickness', 'volume', 'lgi', or 'area')}
}
\value{
A \code{trellis} or \code{ggplot} object
}
\description{
This function takes a \dQuote{tidied} dataset of cortical volumetric measures
(thickness, volume, LGI, etc.) and plots a histogram or violin plot for 1 or
more groups, and of 1 or more brain regions.
}
\seealso{
Other Structural covariance network functions: 
\code{\link{Bootstrapping}},
\code{\link{IndividualContributions}},
\code{\link{Residuals}},
\code{\link{brainGraph_permute}()},
\code{\link{corr.matrix}()},
\code{\link{import_scn}()}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}
\concept{Structural covariance network functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glm_stats.R
\name{GLM influence measures}
\alias{GLM influence measures}
\alias{rstandard.bg_GLM}
\alias{rstudent.bg_GLM}
\alias{hatvalues.bg_GLM}
\alias{cooks.distance.bg_GLM}
\alias{dffits_bg_GLM}
\alias{dfbeta.bg_GLM}
\alias{dfbetas.bg_GLM}
\alias{covratio.bg_GLM}
\alias{influence.bg_GLM}
\title{Influence measures for a bg_GLM object}
\usage{
\method{rstandard}{bg_GLM}(model, type = c("sd.1", "predictive"), ...)

\method{rstudent}{bg_GLM}(model, ...)

\method{hatvalues}{bg_GLM}(model, ...)

\method{cooks.distance}{bg_GLM}(model, ...)

dffits_bg_GLM(model)

\method{dfbeta}{bg_GLM}(model, ...)

\method{dfbetas}{bg_GLM}(model, ...)

covratio.bg_GLM(model)

\method{influence}{bg_GLM}(model, do.coef = TRUE, region = NULL, ...)
}
\arguments{
\item{model}{A \code{bg_GLM} object}

\item{type}{The type of standardized residuals. Default: \code{'sd.1'}}

\item{...}{Unused}

\item{do.coef}{Logical indicating whether to calculate \code{dfbeta}}

\item{region}{Character string of the region(s) to return results for.
Default is to calculate for all regions}
}
\value{
Most influence functions return a numeric matrix in which rownames
  are Study ID's and column names are regions. \code{dfbeta} and
  \code{dfbetas} return a numeric array in which each column is a parameter
  estimate and the 3rd dimension is for each region. \code{influence} returns
  a list with class \code{infl.bg_GLM} and elements:
  \item{infmat}{Numeric array (like \code{dfbeta}) with DFBETAs, DFFITs,
    covratios, Cook's distance, and hat values}
  \item{is.inf}{Logical array of the same data as \code{infmat}; values of
    \code{TRUE} indicate the subject-variable-region combination is an
    outlier value}
  \item{f}{The model \emph{formula}}
  \item{sigma}{The leave-one-out residual standard deviation}
  \item{wt.res}{Model residuals}
}
\description{
These functions compute common (leave-one-out) diagnostics for the models in
a \code{bg_GLM} object.
}
\details{
The \code{influence} method calculates all diagnostics present in
\code{\link[stats]{lm.influence}} and
\code{\link[stats]{influence.measures}}, consisting of the following
functions:
\describe{
  \item{rstandard}{Standardized residuals. Choosing \code{type='predictive'}
    returns leave-one-out cross validation residuals. The \dQuote{PRESS}
    statistic can be calculated as \code{colSums(resids.p^2)}}
  \item{rstudent}{Studentized residuals}
  \item{hatvalues}{The \emph{leverage}, or the diagonal of the
    \emph{hat/projection matrix}}
  \item{cooks.distance}{Cook's distance}
  \item{dffits_bg_GLM}{The change in fitted values when deleting
    observations}
  \item{dfbeta}{The change in parameter estimates (coefficients) when
    deleting observations}
  \item{dfbetas}{The \emph{scaled} change in parameter estimates}
  \item{covratio.bg_GLM}{The covariance ratios, or the change in the
    determinant of the covariance matrix of parameter estimates when deleting
    observations}
}
}
\section{Outlier values}{

Each variable has a different criterion for determining outliers. In the
following: \code{x} is the influence variable (for \code{DFBETA}, the
criterion applies to all DFBETAs); \code{k} is the number of columns of the
design matrix; \code{dfR} is the residual degrees of freedom; and \code{n} is
the number of observations.
\describe{
  \item{DFBETAs}{If \eqn{|x| > 1}}
  \item{DFFITs}{If \eqn{|x| > 3 \sqrt{k / dfR}}}
  \item{covratio}{If \eqn{|1 - x| > (3k / dfR)}}
  \item{cook}{If \eqn{F_{k, dfR}(x) > 0.5}}
  \item{hat}{If \eqn{x > 3k / n}}
}
The return object of \code{influence} has a \code{print} method which will
list the subjects/variables/regions for which an outlier was detected.
}

\seealso{
\code{\link{GLM}}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/write_brainnet.R
\name{write_brainnet}
\alias{write_brainnet}
\title{Write files to be used for visualization with BrainNet Viewer}
\usage{
write_brainnet(
  g,
  vcolor = "none",
  vsize = "constant",
  edge.wt = NULL,
  file.prefix = ""
)
}
\arguments{
\item{g}{The \code{igraph} graph object of interest}

\item{vcolor}{Character string indicating how to color the vertices (default:
\code{'none'})}

\item{vsize}{Character string indicating what size the vertices should be;
can be any vertex-level attribute (default: \code{'constant'})}

\item{edge.wt}{Character string indicating the edge attribute to use to
return a weighted adjacency matrix (default: \code{NULL})}

\item{file.prefix}{Character string for the basename of the \code{.node} and
\code{.edge} files that are written}
}
\description{
Write the \code{.node} and \code{.edge} files necessary for visualization
with the BrainNet Viewer software.
}
\details{
For the \code{.node} file, there are 6 columns:
\itemize{
  \item \emph{Columns 1-3}: Vertex x-, y-, and z-coordinates
  \item \emph{Column 4}: Vertex color
  \item \emph{Column 5}: Vertex size
  \item \emph{Column 6}: Vertex label
}
The \code{.edge} file is the graph's associated adjacency matrix; a weighted
adjacency matrix can be returned by using the \code{edge.wt} argument.
}
\examples{
\dontrun{
write_brainnet(g, vcolor='community', vsize='degree', edge.wt='t.stat')
}
}
\references{
Xia, M. and Wang, J. and He, Y. (2013). BrainNet Viewer: a
  network visualization tool for human brain connectomics. \emph{PLoS One},
  \bold{8(7)}, e68910. \doi{10.1371/journal.pone.0068910}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glm_stats.R
\name{GLM model selection}
\alias{GLM model selection}
\alias{logLik.bg_GLM}
\alias{extractAIC.bg_GLM}
\title{Model selection for bg_GLM objects}
\usage{
\method{logLik}{bg_GLM}(object, REML = FALSE, ...)

\method{extractAIC}{bg_GLM}(fit, scale = 0, k = 2, ...)
}
\arguments{
\item{object, fit}{A \code{bg_GLM} object}

\item{REML}{Logical indicating whether to return the \emph{restricted}
log-likelihood. Default: \code{FALSE}}

\item{...}{Unused}

\item{scale}{Should be left at its default}

\item{k}{Numeric; the weight of the equivalent degrees of freedom}
}
\value{
\code{logLik} returns an object of class \code{logLik} with several
  attributes. \code{extractAIC} returns a numeric vector in which the first
  element is the \emph{equivalent degrees of freedom} and the remaining are
  the AIC's for each region
}
\description{
These functions compute the log-likelihood and Akaike's \emph{An Information
Criterion (AIC)} of a \code{bg_GLM} object. See
\code{\link[stats]{logLik.lm}} and \code{\link[stats]{extractAIC}} for
details.
}
\details{
The functions \code{\link[stats]{AIC}} and \code{\link[stats]{BIC}} will also
work for \code{bg_GLM} objects because they each call \code{logLik}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/brainGraph_GLM.R, R/glm_methods.R
\name{GLM}
\alias{GLM}
\alias{brainGraph_GLM}
\alias{print.bg_GLM}
\alias{summary.bg_GLM}
\alias{plot.bg_GLM}
\alias{[.bg_GLM}
\title{Fit General Linear Models at each vertex of a graph}
\usage{
brainGraph_GLM(
  g.list,
  covars,
  measure,
  contrasts,
  con.type = c("t", "f"),
  outcome = NULL,
  X = NULL,
  con.name = NULL,
  alternative = c("two.sided", "less", "greater"),
  alpha = 0.05,
  level = c("vertex", "graph"),
  permute = FALSE,
  perm.method = c("freedmanLane", "terBraak", "smith", "draperStoneman", "manly",
    "stillWhite"),
  part.method = c("beckmann", "guttman", "ridgway"),
  N = 5000,
  perms = NULL,
  long = FALSE,
  ...
)

\method{print}{bg_GLM}(x, ...)

\method{summary}{bg_GLM}(
  object,
  p.sig = c("p", "p.fdr", "p.perm"),
  contrast = NULL,
  alpha = object$alpha,
  digits = max(3L, getOption("digits") - 2L),
  print.head = TRUE,
  ...
)

\method{plot}{bg_GLM}(x, region = NULL, which = c(1L:3L, 5L), ids = TRUE, ...)

\method{[}{bg_GLM}(x, i, j)
}
\arguments{
\item{g.list}{A \code{brainGraphList} object}

\item{covars}{A \code{data.table} of covariates}

\item{measure}{Character string of the graph measure of interest}

\item{contrasts}{Numeric matrix (for T statistics) or list of matrices (for F
statistics) specifying the contrast(s) of interest; if only one contrast is
desired, you can supply a vector (for T statistics)}

\item{con.type}{Character string; either \code{'t'} or \code{'f'} (for t or
F-statistics). Default: \code{'t'}}

\item{outcome}{Character string specifying the name of the outcome variable,
if it differs from the graph metric (\code{measure})}

\item{X}{Numeric matrix, if you wish to supply your own design matrix.
Ignored if \code{outcome != measure}.}

\item{con.name}{Character vector of the contrast name(s); if \code{contrasts}
has row/list names, those will be used for reporting results}

\item{alternative}{Character string, whether to do a two- or one-sided test.
Default: \code{'two.sided'}}

\item{alpha}{Numeric; the significance level. Default: 0.05}

\item{level}{Character string; either \code{vertex} (default) or
\code{graph}}

\item{permute}{Logical indicating whether or not to permute group labels.
Default: \code{FALSE}}

\item{perm.method}{Character string indicating the permutation method.
Default: \code{'freedmanLane'}}

\item{part.method}{Character string; the method of partitioning the design
matrix into covariates of interest and nuisance. Default: \code{'beckmann'}}

\item{N}{Integer; number of permutations to create. Default: \code{5e3}}

\item{perms}{Matrix of permutations, if you would like to provide your own.
Default: \code{NULL}}

\item{long}{Logical indicating whether or not to return all permutation
results. Default: \code{FALSE}}

\item{...}{Arguments passed to \code{\link{brainGraph_GLM_design}}}

\item{object, x}{A \code{bg_GLM} object}

\item{p.sig}{Character string specifying which P-value to use for displaying
significant results (default: \code{p})}

\item{contrast}{Integer specifying the contrast to plot/summarize; defaults
to showing results for all contrasts}

\item{digits}{Integer specifying the number of digits to display for P-values}

\item{print.head}{Logical indicating whether or not to print only the first
and last 5 rows of the statistics tables (default: \code{TRUE})}

\item{region}{Character string specifying which region's results to
plot; only relevant if \code{level='vertex'}. Default: \code{NULL}}

\item{which}{Integer vector indicating which of the 6 plots to print to the
plot device. Default: \code{c(1:3, 5)}}

\item{ids}{Logical indicating whether to plot subject ID's for outliers.
Otherwise plots the integer index}

\item{i}{Integer/character vector; the observation number(s) or row names to
select or remove}

\item{j}{Integer/character vector; the design matrix column number(s) or
names to select or remove}
}
\value{
An object of class \code{bg_GLM} containing some input-specific
  variables (\code{level}, \code{outcome}, \code{measure}, \code{con.type},
  \code{contrasts}, \code{con.name}, \code{alt}, \code{alpha},
  \code{permute}, \code{perm.method}, \code{part.method}, \code{N}) in
  addition to:
  \item{DT.Xy}{A data table from which the design matrices are created and
    the outcome variable, for all regions.}
  \item{X}{A named numeric matrix or a 3D array of the design matrix.
    Rownames are subject IDs, column names are predictor variables, and
    dimnames along the 3rd dimension are region names (if applicable). This
    is a 3D array only if \code{outcome != measure} and \code{level ==
    'vertex'}.}
  \item{y}{A named numeric matrix of the outcome variable. Rownames are Study
    IDs and column names are regions. There will be multiple columns only if
    \code{outcome == measure} and \code{level == 'vertex'}.}
  \item{DT}{A data table with an entry for each vertex (region) containing
    statistics of interest}
  \item{removed.subs}{A named integer vector in which the names are subject
    ID's of those removed due to incomplete data (if any). The integers
    correspond to the row number in the input \code{covars} table.}
  \item{runX}{If \code{outcome != measure} and \code{level == 'vertex'}, this
    will be a character vector of the regions for which the design matrix is
    invertible. Otherwise, it is \code{NULL}.}
  \item{runY}{Character vector of the regions for which the outcome variable
    has 0 variability. For example, if \code{level='vertex'} and
    \code{measure='degree'}, some regions may be disconnected or have the
    same degree for all subjects.}
  \item{atlas}{Character string of the atlas used (guessed based on the
    vertex count).}
  \item{perm}{A list containing: \emph{null.dist} (the null distribution of
    maximum statistics), \emph{thresh} (the statistic value corresponding
    to the \eqn{100 \times (1 - \alpha)}th\% percentile of the null
    distribution)}

The \code{plot} method returns a \emph{list} of \code{ggplot} objects
  (if installed) or writes the plots to a PDF in the current directory named
  \code{bg_GLM_diagnostics.pdf}

A \code{bg_GLM} object with the specified row(s) selected or removed
  from both \code{X} and \code{y}, and column(s) selected/removed from
  \code{X}
}
\description{
\code{brainGraph_GLM} specifies and fits a General Linear Model (GLM) at each
vertex for a given vertex measure (e.g. \emph{degree}) or at the graph-level
(e.g., \emph{global efficiency}). Given a contrast matrix or list of
contrast(s), and contrast type (for t- or F-contrast(s), respectively) it
will calculate the associated statistic(s) for the given contrast(s).

The \code{summary} method prints the results, only for which
\eqn{p < \alpha}, where \code{alpha} comes from the \code{bg_GLM} object.
\dQuote{Simple} P-values are used by default, but you may change this to the
FDR-adjusted or permutation P-values via the function argument \code{p.sig}.
You may also choose to subset by \emph{contrast}.

The \code{plot} method plots the GLM diagnostics (similar to that of
\code{\link[stats]{plot.lm}}). There are a total of 6 possible plots,
specified by the \code{which} argument; the behavior is the same as in
\code{\link[stats]{plot.lm}}. Please see the help for that function.

The \code{[} method allows you to select observations (i.e., rows of \code{X}
and \code{y}) and independent variables (i.e., columns of \code{X}) from a
\code{bg_GLM} object.
}
\details{
The \code{measure} argument will be the graph- or vertex-level measure of
interest. Often, this will serve as the model's \emph{outcome} (or dependent,
or response) variable; i.e., the variable typically denoted by \emph{y} in
GLMs. In other cases, you may wish to choose some other variable as the
outcome; e.g., IQ, age, etc. Then you could test for a direct association
between the network measure and outcome of interest, or test for another
association while adjusting for the network metric. For these applications,
you must provide the variable name via the \code{outcome} argument. This is
analogous to \code{-evperdat} in FSL's PALM and to \code{--pvr} in
FreeSurfer.
}
\note{
The \code{[} method is used when calculating \emph{studentized
residuals} and other \dQuote{leave-one-out} diagnostics, and typically should
not be called directly by the user.
}
\section{Design matrix}{

The GLM's \emph{design matrix} will often be identical to the \emph{model
matrix} associated with \code{lm} objects (if \dQuote{dummy} coding, the
default, is used) and is created from the input \code{data.table} and
arguments passed to \code{\link{brainGraph_GLM_design}}. The first column
should have the name of \code{getOption('bg.subject_id')} and its values
must match the \emph{name} graph-level attribute of the input graphs. The
covariates table must be supplied even if you provide your own design matrix
\code{X}.  If \code{level='vertex'} and \code{outcome == measure}, there will
be a single design for all regions but a separate model for each region
(since the graph measure varies by region). If \code{level='vertex'} and
\code{outcome != measure}, there will be a separate design (and, therefore, a
separate model) for each region even though the outcome is the same in all
models.
}

\section{Contrasts and statistics}{

Either t- or F-contrasts can be calculated (specified by \code{con.type}).
Multiple t-contrasts can be specified by passing a multi-row \emph{matrix} to
\code{contrasts}. Multiple F-contrasts can be specified by passing a
\emph{list} of matrices; all matrices must have the same number of columns.
All F-contrasts are necessarily \emph{two-sided}; t-contrasts can be any
direction, but only one can be chosen per function call.
If you choose \code{con.type="f"}, the calculated effect size is represented
by the \code{ESS} (\dQuote{extra sum of squares}), the additional variance
explained for by the model parameters of interest (as determined by the
contrast matrix). The standard error for F-contrasts is the sum of squared
errors of the \emph{full model}.
}

\section{Non-parametric permutation tests}{

You can calculate permutations of the data to build a null distribution of
the maximum statistic which corrects for multiple testing. To account for
complex designs, the design matrix must be \emph{partitioned} into covariates
of interest and nuisance; the default method is the \emph{Beckmann} method.
The default permutation strategy is that of Freedman & Lane (1983), and is
the same as that in FSL's \emph{randomise}. See \code{\link{randomise}}.
}

\examples{
\dontrun{
conmat <- matrix(c(0, 0, 0, 1), nrow=1)
rownames(conmat) <- 'Control > Patient'

res.lm <- brainGraph_GLM(g[[6]], covars=covars.all[tract == 1],
  measure='strength', contrasts=conmat, alt='greater', permute=TRUE, long=TRUE)
}
\dontrun{
## Save objects and then to multipage PDF
lmPlots <- plot(x)
ggsave('lmPlots.pdf', lmPlots)

## Save all the GLM sub-objects from MTPC analysis
res.mtpc <- mtpc(...)
glmPlots <- lapply(res.mtpc$res.glm, plot, which=1:6)
ml <- marrangeGrob(glmPlots, nrow=1, ncol=1)
ggsave('glmPlots.pdf', ml, width=8.5, height=11)
}
}
\seealso{
\code{\link[stats]{plot.lm}}

Other GLM functions: 
\code{\link{GLM design}},
\code{\link{GLM fits}},
\code{\link{mtpc}()}

Other Group analysis functions: 
\code{\link{Bootstrapping}},
\code{\link{Mediation}},
\code{\link{NBS}()},
\code{\link{brainGraph_permute}()},
\code{\link{mtpc}()}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}
\concept{GLM functions}
\concept{Group analysis functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R, R/utils_matrix.R
\name{Inverse}
\alias{Inverse}
\alias{inv}
\alias{inv.matrix}
\alias{inv.array}
\alias{inv.qr}
\alias{inv.list}
\alias{pinv}
\title{Calculate the inverse of the cross product of a design matrix}
\usage{
inv(x, ...)

\method{inv}{matrix}(x, y = NULL, transpose = FALSE, ...)

\method{inv}{array}(x, y = NULL, transpose = FALSE, ...)

\method{inv}{qr}(x, p = x$rank, ...)

\method{inv}{list}(
  x,
  p = x[[1L]]$rank,
  r = length(x),
  vnames = dimnames(x[[1L]]$qr)[[2L]],
  nms = names(x),
  ...
)

pinv(x)
}
\arguments{
\item{x}{A numeric matrix or array, a \code{qr} object, or a list of
\code{qr} objects}

\item{...}{Unused}

\item{y}{A numeric matrix or vector (for the \code{matrix} and \code{array}
methods). If supplied, this will be multiplied by \code{x} before the
inverse is calculated. Default: \code{NULL}}

\item{transpose}{Logical. If \code{FALSE} (the default), take the cross
product of the arguments. If \code{TRUE}, use \code{\link{tcrossprod}}}

\item{p}{The rank of the original matrix}

\item{r}{The number of design matrices; i.e., the length of the input list}

\item{vnames}{Character vector of the design matrix's variable names}

\item{nms}{The region names; i.e., the names of the input list}
}
\value{
A numeric matrix or array

\code{pinv} returns the input matrix's pseudoinverse
}
\description{
\code{inv} is a \code{S3} generic that calculates the inverse of the cross
product of a design matrix, also referred to as the \dQuote{unscaled
covariance matrix}.

\code{pinv} calculates \eqn{M^{+} = (M^T M)^{-1} M^T} for full (column) rank
matrices. However, it does not verify the matrix's rank.
}
\details{
If \code{x} is a matrix, the Cholesky decomposition of the cross product is
calculated (or using \code{\link{tcrossprod}} if \code{transpose=TRUE}), and
the inverse is calculated from that result. That is,
\deqn{inv(X) = (X^T X)^{-1}}
\deqn{inv(X, transpose=TRUE) = (X X^T)^{-1}}
\deqn{inv(X, y) = (X^T y)^{-1}}

If \code{x} is a 3-dimensional array, then the inverse will be calculated for
each matrix along the 3rd dimension, with the same input arguments for each.

Finally, there is a method for objects with class \code{qr}, and lists of QR
decomposition objects.
}
\note{
These methods should only be used on \emph{full-rank} matrices, as
there is no error checking being performed.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spatial_dist.R
\name{Graph Distances}
\alias{Graph Distances}
\alias{edge_spatial_dist}
\alias{vertex_spatial_dist}
\title{Calculate Euclidean distance of edges and vertices}
\usage{
edge_spatial_dist(g)

vertex_spatial_dist(g)
}
\arguments{
\item{g}{An \code{igraph} graph object}
}
\value{
\code{edge_spatial_dist} - a numeric vector with length equal to the
  edge count of the input graph, consisting of the Euclidean distance (in
  \emph{mm}) of each edge

\code{vertex_spatial_dist} - a named numeric vector with length equal
  to the number of vertices, consisting of the average distance (in
  \emph{mm}) for each vertex
}
\description{
\code{edge_spatial_dist} calculates the Euclidean distance of an
\code{igraph} graph object's edges. The distances are in \emph{mm} and based
on MNI space. These distances are \emph{NOT} along the cortical surface, so
can only be considered approximations, particularly concerning
inter-hemispheric connections. The input graph must have \emph{atlas} as a
graph-level attribute.

\code{vertex_spatial_dist} calculates, for each vertex of a graph, the
average Euclidean distance across all of that vertex's connections.
}
\references{
Alexander-Bloch, A.F. and Vertes, P.E. and Stidd, R. et al.
  (2013) The anatomical distance of functional connections predicts brain
  network topology in health and schizophrenia. \emph{Cerebral Cortex},
  \bold{23}, 127--138. \doi{10.1093/cercor/bhr388}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/list.R
\name{brainGraphList}
\alias{brainGraphList}
\alias{make_brainGraphList}
\alias{make_brainGraphList.array}
\alias{make_brainGraphList.corr_mats}
\alias{Extract.brainGraphList}
\alias{[.brainGraphList}
\alias{print.brainGraphList}
\alias{is.brainGraphList}
\alias{nobs.brainGraphList}
\alias{as_brainGraphList}
\title{Create a list of brainGraph graphs}
\usage{
make_brainGraphList(
  x,
  atlas,
  type = c("observed", "random"),
  level = c("subject", "group", "contrast"),
  set.attrs = TRUE,
  modality = NULL,
  weighting = NULL,
  threshold = NULL,
  gnames = NULL,
  ...
)

\method{make_brainGraphList}{array}(
  x,
  atlas,
  type = c("observed", "random"),
  level = c("subject", "group", "contrast"),
  set.attrs = TRUE,
  modality = NULL,
  weighting = NULL,
  threshold = NULL,
  gnames = NULL,
  grpNames = NULL,
  subnet = NULL,
  mode = "undirected",
  weighted = NULL,
  diag = FALSE,
  .progress = getOption("bg.progress"),
  ...
)

\method{make_brainGraphList}{corr_mats}(
  x,
  atlas = x$atlas,
  type = "observed",
  level = "group",
  set.attrs = TRUE,
  modality = NULL,
  weighting = NULL,
  threshold = x$densities,
  gnames = names(x$r.thresh),
  grpNames = gnames,
  mode = "undirected",
  weighted = NULL,
  diag = FALSE,
  .progress = getOption("bg.progress"),
  ...
)

\method{[}{brainGraphList}(x, i, g = NULL, drop = TRUE)

\method{print}{brainGraphList}(x, ...)

is.brainGraphList(x)

\method{nobs}{brainGraphList}(object, ...)

as_brainGraphList(
  g.list,
  type = c("observed", "random"),
  level = c("subject", "group", "contrast")
)
}
\arguments{
\item{x}{3-D numeric array of all subjects' connectivity matrices (for a
single threshold) or a \code{corr_mats} object}

\item{atlas}{Character string specifying the brain atlas}

\item{type}{Character string indicating the type of graphs. Default:
\code{observed}}

\item{level}{Character string indicating whether the graphs are subject-,
group-, or contrast-specific. Default: \code{'subject'}}

\item{set.attrs}{Logical indicating whether to assign all graph-, vertex-,
and edge-level attributes (via \code{\link{set_brainGraph_attr}}). Default:
\code{TRUE}}

\item{modality}{Character string indicating imaging modality (e.g. 'dti').
Default: \code{NULL}}

\item{weighting}{Character string indicating how the edges are weighted
(e.g., 'fa', 'pearson', etc.). Default: \code{NULL}}

\item{threshold}{Integer or number indicating the threshold used when
\dQuote{sparsifying} the connectivity matrix (if any). Default: \code{NULL}}

\item{gnames}{Character vector of graph names (e.g., study IDs if
\code{level='subject'}). Default: \code{NULL}}

\item{...}{Other arguments passed to \code{\link{set_brainGraph_attr}}}

\item{grpNames}{Character (or factor) vector of group names. If \code{level ==
'group'}, then you do not need to include this argument (the group names
will be the same as \code{gnames}). Default: \code{NULL})}

\item{subnet}{Integer or character vector indicating the vertices to keep, if
you are interested in working with a subset of an atlas. By default, all
vertices are used.}

\item{mode}{Character string defining how the matrix should be interpreted.
Default: \code{'undirected'}}

\item{weighted}{Logical specifying whether to create a weighted network}

\item{diag}{Logical indicating whether to include the diagonal of the
connectivity matrix. Default: \code{FALSE}}

\item{.progress}{Logical indicating whether to print a progress bar. Default:
\code{getOption('bg.progress')}}

\item{i}{Integer, character, or logical vector for subsetting by subject, or
by group (if \code{x$level='group'})}

\item{g}{Integer, character, or logical vector for subsetting by group (if
\code{x$level='subject'})}

\item{drop}{If \code{TRUE} (the default), then return only the list of
graphs; otherwise, subset the graphs and return the entire object}

\item{object}{A \code{brainGraphList} object}

\item{g.list}{List of graph objects}
}
\value{
\code{make_brainGraphList} returns an object of class
  \code{brainGraphList} with elements:
  \item{threshold}{The specified threshold/density}
  \item{version}{The versions of \code{R}, \code{igraph}, and
    \code{brainGraph} used when creating the graphs}
  \item{atlas}{The atlas common to all the graphs}
  \item{modality}{The imaging modality (if supplied)}
  \item{weighting}{A string indicating what edge weights represent (if
    applicable)}
  \item{graphs}{A \emph{named list} of \code{brainGraph} graphs; the names
    correspond to the individual graphs' Study IDs}

\code{[} -- A \code{brainGraphList} object (if \code{drop=FALSE}) or
  a list of graphs
}
\description{
\code{make_brainGraphList} creates a \code{brainGraphList} object, a list
containing a set of graphs for all subjects (or group-average graphs) in a
study at a specific threshold (or density), in addition to some graph-level
attributes common to those graphs.

The \code{[} method will let you subset/slice the graphs for individual
subjects and/or \emph{groups}.

\code{as_brainGraphList} coerces a list of graphs to a \code{brainGraphList}
object. It is assumed that certain metadata attributes -- threshold, package
version, atlas, imaging modality, edge weighting, and whether they are
random graphs -- are identical for all graphs in the list.
}
\details{
In addition to creating the initial \code{igraph} graphs from the
connectivity matrices, then attributes will be calculated and assigned for
each graph via \code{\link{set_brainGraph_attr}} if \code{set.attrs=TRUE}.
Other arguments can be passed to that function. You may display a progress
bar by setting \code{.progress=TRUE}.

This object can be considered comparable to a 4-D \emph{NIfTI} file,
particularly that returned by FSL's \emph{TBSS} \dQuote{prestats} step since
that file contains the FA volumes for all study subjects.

To convert an object with 3 \dQuote{levels} (i.e., subject-level lists from
an older \code{brainGraph} version), see the code in the Examples below.
}
\note{
If the input is a \code{corr_mats} object, and the extent of the 3-D
array is greater than 1, then only the first will be converted to a graph.
}
\section{Subsetting/extracting}{

The first index is for subsetting the individual graphs. The second index is
for subsetting by group membership and requires that the graphs have a
\code{Group} graph attribute. When both are included, the first index cannot
have length or numeric value greater than the number of \emph{remaining}
subjects \emph{after} subsetting by group.

If the indexing vector(s) is (are) \code{character}, the vector(s) must
contain one (or more) of the subject or group names. If \code{logical}, its
length must equal the number of subjects or groups.
}

\examples{
\dontrun{
# Create a list, one for each threshold
g <- vector('list', length(thresholds))
for (i in seq_along(thresholds)) {
  g[[i]] <- make_brainGraphList(A.norm.sub[[i]], thresholds[i], atlas,
      covars.dti$Study.ID, covars.dti$Group, modality='dti', weighting='fa')
}
}
\dontrun{
# Subset the first 10 subjects, irrespective of group
my.bgl[1:10]

# Return object for only 'Control' subjects
my.bgl[, 'Control']

# Return object with graphs from groups 1 and 3
my.bgl[g=c(1, 3), drop=FALSE]

# Subset the first 10 subjects of group 2
my.bgl[1:10, 2]
}
\dontrun{
## Convert old version single-subject graph lists
## g[[1]] is group 1, g[[1]][[1]] is threshold 1, g[[1]][[1]][[1]] is subj. 1
kNumThresholds <- length(g[[1]])
g.l <- vector('list', kNumThresholds)
for (i in seq_len(kNumThresholds)) {
  g.l[[i]] <- as_brainGraphList(do.call(Map, c(c, g))[[i]])
}
}
}
\seealso{
Other Graph creation functions: 
\code{\link{Creating_Graphs}},
\code{\link{Creating_Graphs_GLM}},
\code{\link{make_ego_brainGraph}()}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}
\concept{Graph creation functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{Utility functions}
\alias{Utility functions}
\alias{get_metadata}
\alias{outer_vec}
\alias{split_string}
\alias{vec.transform}
\title{Utility functions}
\usage{
get_metadata(object)

outer_vec(x, y)

split_string(x, max_len = 80L, delim = "\\\\+", sep = "\\n")

vec.transform(x, min.val = 0, max.val = 1)
}
\arguments{
\item{object}{A list-like object}

\item{x}{A character string}

\item{max_len}{Integer; the max length of one line. Default: 80}

\item{delim}{Character specifying where to end a line if it is longer than
\code{max_len}. Default: \code{'+'}}

\item{sep}{Character specifying what to split by. Default: \code{'\n'} (a
newline character)}

\item{min.val}{the minimum value of the new range}

\item{max.val}{the maximum value of the new range}
}
\value{
\code{get_metadata} - the same object with version, system, and date
  information added

A vector of the transformed input.
}
\description{
\code{get_metadata} adds metadata to a list-like object.

\code{outer_vec} simply performs the cross-product, specifically \code{x %*%
t(y)}, and assigns dimnames to the resulting matrix.

\code{split_string} inserts separator characters in a character string to
truncate strings for printing.

\code{vec.transform} takes a vector and transforms it to have a new range,
given the input, or the default values of [0, 1].
}
\details{
If the object is a graph, graph-level attributes will be added. The
elements added are:
\describe{
  \item{version}{A list with R, brainGraph, and igraph versions}
  \item{sys}{Character vector of system information}
  \item{date}{The date and time of creation}
}

The \code{delim} argument determines \emph{where} to insert the separator.
For example, given the default, it will attempt to insert newline characters
only following a plus sign.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create_graphs.R
\name{make_ego_brainGraph}
\alias{make_ego_brainGraph}
\title{Create a graph of the union of multiple vertex neighborhoods}
\usage{
make_ego_brainGraph(g, vs)
}
\arguments{
\item{g}{An \code{igraph} graph object}

\item{vs}{Either a character or integer vector (vertex names or indices,
respectively) for the vertices of interest}
}
\value{
An \code{igraph} graph object containing the union of all edges and
  vertices in the neighborhoods of the input vertices; only the vertex
  attribute \emph{name} will be present
}
\description{
This function accepts multiple vertices, creates graphs of their
neighborhoods (of order 1), and returns the union of those graphs.
}
\examples{
\dontrun{
subg <- make_ego_brainGraph(g1[[N]], c(24, 58))
subg <- make_ego_brainGraph(g1[[N]], c('lPCUN', 'rPCUN'))
}
}
\seealso{
\code{\link[igraph]{ego}}

Other Graph creation functions: 
\code{\link{Creating_Graphs}},
\code{\link{Creating_Graphs_GLM}},
\code{\link{brainGraphList}}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}
\concept{Graph creation functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{check_weights}
\alias{check_weights}
\alias{check_degree}
\alias{check_strength}
\title{Check for vertex or edge attributes}
\usage{
check_weights(g, weights)

check_degree(g)

check_strength(g)
}
\value{
\code{check_weights} - If \code{weights=NULL} and the graph has a
  \code{'weight'} attribute, then \code{NULL} will be returned. If
  \code{weights=NA}, then \code{NA} is returned.
}
\description{
\code{check_weights} is a helper function for dealing with edge weights that
get passed to different \code{igraph} functions.

\code{check_degree} is a helper function to check if \code{degree} is a
vertex attribute of the input graph. Returns a numeric vector of the degree
values.

\code{check_strength} is a helper function to check if \code{strength} is a
vertex attribute of the input graph. Returns a numeric vector of the strength
values.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glm_fit.R
\name{GLM fits}
\alias{GLM fits}
\alias{fastLmBG}
\alias{fastLmBG_3d}
\alias{fastLmBG_3dY}
\alias{fastLmBG_3dY_1p}
\alias{fastLmBG_t}
\alias{fastLmBG_f}
\title{Fit design matrices to one or multiple outcomes}
\usage{
fastLmBG(
  X,
  Y,
  QR = qr.default(X),
  Q = qr_Q2(QR, n = n, p = p),
  R = qr_R2(QR, p),
  n = dim(X)[1L],
  p = QR$rank,
  ny = dim(Y)[2L],
  dfR = n - p,
  XtXinv = inv(QR)
)

fastLmBG_3d(
  X,
  Y,
  runX,
  QR = qr(X[, , runX, drop = FALSE]),
  Q = lapply(QR, qr_Q2, n = n, p = p),
  R = lapply(QR, qr_R2, p),
  n = dim(X)[1L],
  p = QR[[1L]]$rank,
  ny = length(runX),
  dfR = n - p,
  XtXinv = inv(QR)
)

fastLmBG_3dY(
  X,
  Y,
  runX,
  QR = qr(X[, , runX, drop = FALSE]),
  Q = lapply(QR, qr_Q2, n = n, p = p),
  R = lapply(QR, qr_R2, p),
  n = dim(X)[1L],
  p = QR[[1L]]$rank,
  ny = length(runX),
  dfR = n - p,
  XtXinv = inv(QR)
)

fastLmBG_3dY_1p(
  X,
  Y,
  runX,
  QR = qr(X[, , runX, drop = FALSE]),
  Q = lapply(QR, qr_Q2, diag(1L, n, 1L), n, 1L),
  R = lapply(QR, function(r) r$qr[1L]),
  n = dim(X)[1L],
  p = 1L,
  ny = length(runX),
  dfR = n - 1L,
  XtXinv = inv(QR)
)

fastLmBG_t(
  fits,
  contrasts,
  alternative = c("two.sided", "less", "greater"),
  alpha = NULL
)

fastLmBG_f(fits, contrasts, rkC = NULL, nC = length(contrasts))
}
\arguments{
\item{X}{Design matrix or 3D array of design matrices}

\item{Y}{Numeric matrix; there should be 1 column for each outcome variable
(so that in a graph-level analysis, this is a column matrix)}

\item{QR, Q, R}{The QR decomposition(s) and Q and R matrix(es) of the design
matrix(es). If \code{X} is a 3D array, these should be \emph{lists}}

\item{n, p, ny, dfR}{Integers; the number of observations, model \emph{rank},
number of regions/outcome variables, and residual degrees of freedom}

\item{XtXinv}{Numeric matrix or array; the inverse of the cross-product of
the design matrix(es)}

\item{runX}{Character vector of the regions for which the design matrix is
not singular}

\item{fits}{List object output by one of the model fitting functions (e.g.,
\code{fastLmBG})}

\item{contrasts}{Numeric matrix (for T statistics) or list of matrices (for F
statistics) specifying the contrast(s) of interest; if only one contrast is
desired, you can supply a vector (for T statistics)}

\item{alternative}{Character string, whether to do a two- or one-sided test.
Default: \code{'two.sided'}}

\item{alpha}{Numeric; the significance level. Default: 0.05}

\item{rkC, nC}{Integers; the rank of the contrast matrix and number of
contrasts, respectively (for F contrasts)}
}
\value{
A list with elements
  \item{coefficients}{Parameter estimates}
  \item{rank}{Model rank}
  \item{df.residual}{Residual degrees of freedom}
  \item{residuals}{Model residuals}
  \item{sigma}{The residual standard deviation, or \emph{root mean square
    error (RMSE)}}
  \item{fitted.values}{Model fitted values}
  \item{qr}{The design matrix QR decomposition(s)}
  \item{cov.unscaled}{The \dQuote{unscaled covariance matrix}}

\code{fastLmBG_t} -- A multidimensional array with the third
  dimension equaling the number of contrasts; each matrix contains the
  contrast of parameter estimates, standard error of the contrast,
  T-statistics, P-values, FDR-adjusted P-values, and confidence intervals (if
  \code{alpha} is given)

\code{fastLmBG_f} -- A numeric matrix with columns for the effect
  size, standard error, F statistic, P-values, and FDR-adjusted P-values
}
\description{
These are the \dQuote{base} model-fitting functions that solve the
\emph{least squares problem} to estimate model coefficients, residuals, etc.
for brain network data.

\code{fastLmBG_t} and \code{fastLmBG_f} calculate contrast-based statistics
for T or F contrasts, respectively. It accepts any number of \emph{contrasts}
(i.e., a multi-row contrast matrix).
}
\section{Parameter estimation}{

These functions use the \emph{QR} decomposition to calculate the least
squares solution which is the same as the base \code{\link[stats]{lm}}
function. If we substitute \eqn{X = QR} in the standard normal equations, the
equation to be solved reduces to
\deqn{X^T X \hat{\beta} = X^T y \Rightarrow R \hat{\beta} = Q^T y}

Since \code{R} is an \emph{upper-triangular} matrix, we can use the
\code{\link{backsolve}} function which is a bit faster than
\code{\link{solve}}. In some cases, the \code{fastLmBG*} functions are about
as fast or faster (particularly when \code{X} is not permuted) as one in
which the normal equations are solved directly; additionally, using the
\emph{QR} method affords greater numerical stability.
}

\section{Different scenarios}{

There are a few different scenarios for fitting models of the data, with a
separate function for each:
\describe{
  \item{fastLmBG}{The main function for when there is a single design matrix
    \eqn{X} and any number of outcome variables \eqn{Y}.}
  \item{fastLmBG_3d}{Fits models when there is a different design matrix
    \eqn{X} for each region and a single outcome variable \eqn{Y}, which in
    this case will be a column matrix.}
  \item{fastLmBG_3dY}{Fits models when there is both a different design
    matrix \eqn{X} and outcome variable \eqn{Y} for each region. Occurs under
    permutation for the Freedman-Lane, ter Braak, and Still-White methods.}
  \item{fastLmBG_3dY_1p}{Fits models when there is both a different design
    and outcome variable for each region, and also when \eqn{X} is a rank-1
    matrix (i.e., it has 1 column). Only occurs under permutation with the
    Still-White method if there is a single regressor of interest.}
}

In the last case above, model coefficients are calculated by simple (i.e.,
non-matrix) algebra.
}

\section{Improving speed/efficiency}{

Speed/efficiency gains will be vast for analyses in which there is a single
design matrix \eqn{X} for all regions, there are multiple outcome variables
(i.e., vertex-level analysis), and the permutation method chosen does
not permute \eqn{X}. Specifically, these are \emph{Freedman-Lane}, \emph{ter
Braak}, and \emph{Manly} methods. Therefore, the QR decomposition, the
\eqn{Q} and \eqn{R} matrices, and the \dQuote{unscaled covariance matrix}
(which is \eqn{(X^T X)^{-1}}) only need to be calculated once for the entire
analysis. Other functions (e.g., \code{lm.fit}) would recalculate these for
each permutation.

Furthermore, this (and the other model fitting functions in the package) will
likely only work in models with full rank. I sacrifice proper error checking
in favor of speed, but hopefully any issues with the model will be identified
prior to the permutation step. Finally, the number of observations, model
rank, number of outcome variables, and degrees of freedom will not change and
therefore do not need to be recalculated (although these probably amount to a
negligible speed boost).

In case there are multiple design matrices, or the permutation method
permutes the design, then the QR decomposition will need to be calculated
each time anyway. For these cases, I use more simplified functions
\code{qr_Q2} and \code{qr_R2} to calculate the \eqn{Q} and \eqn{R} matrices,
and then the fitted values, residuals, and residual standard deviation are
calculated at the same time (whereas \code{lm.fit} and others would calculate
these each time).
}

\section{Contrast-based statistics}{

The \emph{contrast of parameter estimates}, \eqn{\gamma}, for T contrasts is
\deqn{\gamma = C \hat{\beta}}
where \eqn{C} is the contrast matrix with size \eqn{k \times p} (where
\eqn{k} is the number of contrasts) and \eqn{\hat{\beta}} is the matrix of
parameter estimates with size \eqn{p \times r} (where \eqn{r} is the number
of regions). For F contrasts, the effect size is the \emph{extra sum of
squares} and is calculated as
\deqn{\gamma (C (X^T X)^{-1} C^T)^{-1} \gamma^T}
The \emph{standard error} of a T contrast is
\deqn{\sqrt{\hat{\sigma} (X^T X)^{-1}}}
where \eqn{\hat{\sigma}} is the \emph{residual standard deviation} of the
model and the second term is the unscaled covariance matrix. The standard
error for F contrasts is simply the \emph{residual sum of squares}. P-values
and FDR-adjusted P-values (across regions) are also calculated. Finally, if
\eqn{\alpha} is provided for T contrasts, confidence limits are calculated.
}

\seealso{
randomise

Other GLM functions: 
\code{\link{GLM}},
\code{\link{GLM design}},
\code{\link{mtpc}()}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}
\concept{GLM functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import.R
\name{import_scn}
\alias{import_scn}
\title{Import data for structural connectivity analysis}
\usage{
import_scn(
  datadir,
  atlas,
  modality = "thickness",
  exclude.subs = NULL,
  custom.atlas = NULL
)
}
\arguments{
\item{datadir}{The path name of the directory containing the data files}

\item{atlas}{Character string specifying the atlas in use. For a custom
atlas, please specify \code{'custom'}, and provide the name to the
\code{custom.atlas} argument}

\item{modality}{The structural imaging measure (default: \code{'thickness'})}

\item{exclude.subs}{Vector indicating the subjects to exclude, if any
(default: \code{NULL})}

\item{custom.atlas}{Character string specifying the name of the R object for
the atlas in use, if \code{atlas='custom'} was also supplied (default:
\code{NULL})}
}
\value{
A list containing:
  \item{atlas}{Character string}
  \item{modality}{Character string}
  \item{lhrh}{A \code{data.table} of structural MRI measures for both
    hemispheres}
  \item{aseg}{A \code{data.table} of structural MRI measures for subcortical
    gray matter, if applicable}
  \item{subs.excluded}{Vector of subject ID's that were excluded}
  \item{subs.missing}{Vector of subject ID's that are not present in \emph{both}
    the cortical and subcortical tables (if applicable)}
}
\description{
Given a directory, atlas name, and imaging modality/structural metric, this
function imports data for structural connectivity analysis. It expects files
containing a table of region-wise structural MRI measures (e.g., mean
cortical thickness), with one file for each hemisphere. The first column of
all files should contain the \emph{subject ID}; the column name will be
changed to the value of \code{getOption('bg.subject_id')}.
}
\details{
The files should have specific names; the second in the following list is
only required for atlases/parcellations that include \emph{subcortical gray
matter} (e.g., \code{dk.scgm}).
\itemize{
  \item \code{${parcellation}_${hemi}_${modality}.csv} for cortical volume,
    thickness, surface area, or local gyrification index (LGI). Here,
    \code{${parcellation}} can be \code{aparc}, \code{aparc.DKTatlas40},
    or \code{aparc.a2009s}. For example, for cortical thickness with the
    \emph{Desikan-Killiany} atlas, the filename should be
    \code{aparc_lh_thickness.csv}. If you are using a custom atlas, see the
    \emph{Note} below. The \code{${hemi}} variable is either \code{lh} or
    \code{rh}. Finally, \code{${modality}} should be either \code{volume},
    \code{thickness}, \code{area}, or \code{lgi}.
  \item \code{asegstats.csv} for SCGM volume
}
}
\note{
When using a custom atlas, the name of the atlas's data.table should
  match the \code{${parcellation}} portion of the filename (specification
  shown above). Furthermore, it must conform to the output of Freesurfer's
  \code{aparcstats2table} (and \code{asegstats2table}, if applicable).
  Otherwise, please contact me for inclusion of a different data type.

The subject ID column will be zero-padded (to the left) to avoid issues
  when the variable is numeric; this ensures that all ID's will have the same
  number of characters and sorting will be done properly.
}
\examples{
\dontrun{
  raw_data <- import_scn('/home/cwatson/data', atlas='dkt',
                         exclude.subs=c('con07', 'con23', 'pat15'))
}
}
\seealso{
Other Structural covariance network functions: 
\code{\link{Bootstrapping}},
\code{\link{IndividualContributions}},
\code{\link{Residuals}},
\code{\link{brainGraph_permute}()},
\code{\link{corr.matrix}()},
\code{\link{plot_volumetric}()}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}
\concept{Structural covariance network functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/brainGraph_GLM.R, R/glm_fit.R, R/utils.R
\name{GLM helpers}
\alias{GLM helpers}
\alias{setup_glm}
\alias{contrast_names}
\alias{check_if_singular}
\alias{add_nans}
\alias{cxtxfun_3d}
\alias{glm_data_table}
\alias{matrix2list}
\alias{maxfun}
\alias{sortfun}
\title{Helper functions to set-up for GLM analyses}
\usage{
setup_glm(
  g.list,
  level,
  covars,
  X,
  contrasts,
  con.type,
  con.name,
  measure,
  outcome,
  ...
)

contrast_names(contrasts, con.type, con.name, X)

check_if_singular(QR)

add_nans(fits, dimX, namesX, runX = names(fits$qr))

cxtxfun_3d(con.type, transpose = TRUE)

glm_data_table(g.list, level, measure)

matrix2list(mat)

maxfun(alternative)

sortfun(alternative)
}
\arguments{
\item{g.list}{A \code{brainGraphList} object}

\item{level}{Character string; either \code{vertex} (default) or
\code{graph}}

\item{covars}{A \code{data.table} of covariates}

\item{X}{Numeric matrix, if you wish to supply your own design matrix.
Ignored if \code{outcome != measure}.}

\item{contrasts}{Numeric matrix (for T statistics) or list of matrices (for F
statistics) specifying the contrast(s) of interest; if only one contrast is
desired, you can supply a vector (for T statistics)}

\item{con.type}{Either \code{'t'} or \code{'f'}}

\item{con.name}{Character vector of the contrast name(s); if \code{contrasts}
has row/list names, those will be used for reporting results}

\item{measure}{Character string of the graph measure of interest}

\item{outcome}{Character string specifying the name of the outcome variable,
if it differs from the graph metric (\code{measure})}

\item{...}{Arguments passed to \code{\link{brainGraph_GLM_design}}}

\item{QR}{List of QR decompositions for each design matrix}

\item{fits}{List object output by one of the model fitting functions (e.g.,
\code{\link{fastLmBG}})}

\item{dimX}{Integer vector containing the dimensions of the original design
matrix/array (including singular designs)}

\item{namesX}{List of character vectors containing the dimension names from
the original design matrix/array}

\item{runX}{Character vector of regions for which models were fit}

\item{transpose}{Logical indicating whether to transpose the output of the
selected function. Ignored for F-contrasts. Should be \code{FALSE} for
T-contrasts when the \dQuote{Guttman} partition method is used. Default:
\code{TRUE}}

\item{mat}{Numeric matrix in which each row is a single contrast vector}

\item{alternative}{Character string, whether to do a two- or one-sided test.
Default: \code{'two.sided'}}
}
\value{
\code{contrast_names} -- list containing the contrasts (matrix or
  list), contrast names, and number of contrasts

\code{add_nans} -- the original \code{fits} object with \code{NaN} or
  \code{NA} inserted

\code{cxtxfun_3d} -- A function with arguments for the contrast
  (numeric matrix), the \dQuote{unscaled covariance} array, the (row) rank of
  the contrast, and the number of regions in the analysis (only used for
  F-contrasts)

\code{glm_data_table} - A \code{data.table} with one column
  containing the subject ID's and 1 or more columns with the graph- or
  vertex-level measure of interest.

\code{matrix2list} -- A list with length equal to the number of rows
  of \code{C}
}
\description{
\code{setup_glm} is used to setup the data/objects for any function that uses
the main GLM functionality in \code{brainGraph}.

\code{contrast_names} checks the dimensions of contrasts, generates contrast
names, and sets column names for GLM functions. For F-contrasts, if a
\code{matrix} is given, it converts it to a \code{list} to simplify processing
later.

\code{add_nans} adds rows/columns (or higher dimensions) to model fit data
for regions which were skipped (due to having a singular design matrix,
usually).

\code{cxtxfun_3d} returns a function that calculates the \dQuote{CXtX}
matrix/array, used to calculate the standard error of a contrast. The
function signature will be \code{f(contrast, xtx, rkC, ny)}.

\code{glm_data_table} is used in \code{brainGraph_GLM} and
\code{brainGraph_mediate} to create a \code{data.table} with the
\emph{subject IDs} and column(s) for the graph- or vertex-level metric of
interest.

\code{matrix2list} makes working with different contrast types (i.e., t or F)
a little simpler.
}
\details{
This function: removes unused levels from \code{covars} and \code{DT.y.m},
removes subjects with incomplete data, creates a design matrix (if not
supplied), and supplies names to the contrast matrix.

The \dQuote{CXtX} matrix/array for T-contrasts is the diagonal of
\deqn{C (X^T X)^{-1} C^T}
For F-contrasts, it is the \emph{inverse} of the matrix:
\deqn{(C (X^T X)^{-1} C^T)^{-1}}
where in both cases, \eqn{C} is the contrast matrix and \eqn{X} is the design
matrix/array.

For T-contrasts, the function will return a numeric vector/matrix with
dimensions \eqn{k \times r} where \eqn{k} is the \emph{number} of contrasts
(i.e., the number of rows in the contrast matrix) and \eqn{r} is the number
of regions. For F-contrasts, the function would return a numeric array with
dimensions \eqn{k \times k \times r}, where \eqn{k} is the rank of the
contrast matrix and \eqn{r} is the number of regions. If there is a single
design for all regions, it will be a \eqn{k \times k} matrix.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{cor.diff.test}
\alias{cor.diff.test}
\title{Calculate the p-value for differences in correlation coefficients}
\usage{
cor.diff.test(r1, r2, n, alternative = c("two.sided", "less", "greater"))
}
\arguments{
\item{r1, r2}{Numeric (vector or matrix) of correlation coefficients for both
groups}

\item{n}{Integer vector; number of observations for both groups}

\item{alternative}{Character string, whether to do a two- or one-sided test.
Default: \code{'two.sided'}}
}
\value{
A list with elements \code{p} and \code{z}, the p-values
  and z-scores for the difference in correlations.
}
\description{
Given two sets of correlation coefficients and sample sizes, this function
calculates and returns the \emph{z-scores} and \emph{p-values} associated
with the difference between correlation coefficients.
}
\examples{
\dontrun{
kNumSubjs <- summary(covars$Group)
corr.diffs <- cor.diff.test(corrs$R[, , 1], corrs$R[, , 2], kNumSubjs)
edge.diffs <- t(sapply(which(corr.diffs$p < .05), function(x)
                       mapply('[[',
                              dimnames(corr.diffs$p),
                              arrayInd(x, dim(corr.diffs$p)))
                              ))
}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/set_brainGraph_attributes.R
\name{set_graph_colors}
\alias{set_graph_colors}
\title{Color graph vertices and edges}
\usage{
set_graph_colors(g, name, memb)
}
\arguments{
\item{g}{An \code{igraph} graph object}

\item{name}{Character string of the name of the attribute to add}

\item{memb}{An integer vector representing membership of e.g. a community}
}
\value{
The same graph with additional vertex and edge attributes
}
\description{
\code{set_graph_colors} takes an integer vector representing membership of
some grouping (e.g., a community or connected component) and creates a
character vector of colors for each grouping. Isolated vertices will be
colored \emph{gray}. Edges are assigned the same color if connected to
vertices in the same group, and assigned \emph{gray} otherwise.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/individ_contrib.R
\name{IndividualContributions}
\alias{IndividualContributions}
\alias{loo}
\alias{aop}
\alias{summary.IC}
\alias{plot.IC}
\title{Approaches to estimate individual network contribution}
\usage{
loo(resids, corrs, level = c("global", "regional"))

aop(resids, corrs, level = c("global", "regional"), control.value = 1L)

\method{summary}{IC}(object, region = NULL, digits = max(3L, getOption("digits") - 2L), ...)

\method{plot}{IC}(
  x,
  plot.type = c("mean", "smooth", "boxplot"),
  region = NULL,
  ids = TRUE,
  ...
)
}
\arguments{
\item{resids}{An object of class \code{brainGraph_resids} (the output from
\code{\link{get.resid}})}

\item{corrs}{List of lists of correlation matrices (as output by
\code{\link{corr.matrix}}).}

\item{level}{Character string; the level at which you want to calculate
contributions (either \code{global} or \code{regional})}

\item{control.value}{Integer or character string specifying the control group
(default: 1L)}

\item{object, x}{A \code{IC} object}

\item{region}{Character vector specifying which regions' IC's to print. Only
relevant if \code{method='Leave one out'}}

\item{digits}{Integer specifying the number of digits to display for P-values}

\item{...}{Unused}

\item{plot.type}{Character string indicating the type of plot; the default is
to plot the mean (along with standard errors)}

\item{ids}{Logical indicating whether to plot Study ID's for outliers.
Otherwise plots the integer index}
}
\value{
A \code{data.table} with columns for
  \item{Study.ID}{Subject identifier}
  \item{Group}{Group membership}
  \item{region}{If \code{level='regional'}}
  \item{IC,RC}{The value of the individual/regional contributions}
}
\description{
\code{loo} calculates the individual contribution to group network data for
each subject in each group using a \dQuote{leave-one-out} approach. The
residuals of a single subject are excluded, and a correlation matrix is
created. This is compared to the original correlation matrix using the Mantel
test.

\code{aop} calculates the individual contribution using an
\dQuote{add-one-patient} approach. The residuals of a single patient are
added to those of a control group, and a correlation matrix is created. This
is repeated for all individual patients and each patient group.

The \code{summary} method prints the group/region-wise means and standard
deviations.

The \code{plot} method is only valid for \emph{regional} contribution
estimates, and plots the average regional contribution for each
vertex/region.
}
\note{
For \code{aop}, it is assumed by default that the control group is the
  first group.
}
\examples{
\dontrun{
IC <- loo(resids.all, corrs)
RC <- loo(resids.all, corrs, level='regional')
}
\dontrun{
IC <- aop(resids.all, corrs)
RC <- aop(resids.all, corrs, level='regional')
}
}
\references{
Saggar, M. and Hosseini, S.M.H. and Buno, J.L. and Quintin, E.
  and Raman, M.M. and Kesler, S.R. and Reiss, A.L. (2015) Estimating
  individual contributions from group-based structural correlations networks.
  \emph{NeuroImage}, \bold{120}, 274--284.
  \doi{10.1016/j.neuroimage.2015.07.006}
}
\seealso{
Other Structural covariance network functions: 
\code{\link{Bootstrapping}},
\code{\link{Residuals}},
\code{\link{brainGraph_permute}()},
\code{\link{corr.matrix}()},
\code{\link{import_scn}()},
\code{\link{plot_volumetric}()}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}
\concept{Structural covariance network functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/random_graphs.R
\name{choose.edges}
\alias{choose.edges}
\title{Select edges for re-wiring}
\usage{
choose.edges(A, degs.large)
}
\arguments{
\item{A}{Numeric (adjacency) matrix}

\item{degs.large}{Integer vector of vertex numbers with degree greater than
one}
}
\value{
A data frame with four elements; two edges \code{(y1, z1)} and
  \code{(y2, z2)} will be removed, and two edges \code{(y1, y2)} and
  \code{(z1, z2)} will be added between the four vertices.
}
\description{
\code{choose.edges} selects edges to be re-wired when simulating random
graphs while controlling for \emph{clustering}. It is based on the algorithm
by Bansal et al. (2009), BMC Bioinformatics.
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/communicability.R
\name{communicability}
\alias{communicability}
\title{Calculate communicability}
\usage{
communicability(g, weights = NULL)
}
\arguments{
\item{g}{An \code{igraph} graph object}

\item{weights}{Numeric vector of edge weights; if \code{NULL} (the default),
and if the graph has edge attribute \code{weight}, then that will be used.
To avoid using weights, this should be \code{NA}.}
}
\value{
A numeric matrix of the communicability
}
\description{
\code{communicability} calculates the communicability of a network, a measure
which takes into account all possible paths (including non-shortest paths)
between vertex pairs.
}
\details{
The communicability \eqn{G_{pq}} is a weighted sum of the number of walks
from vertex \emph{p} to \emph{q} and is calculated by taking the exponential
of the adjacency matrix \emph{A}:
\deqn{G_{pq} = \sum_{k=0}^{\infty} \frac{(\mathbf{A}^k)_{pq}}{k!} =
(e^{\mathbf{A}})_{pq}}
where \eqn{k} is \emph{walk} length.

For weighted graphs with \eqn{D = diag(d_i)} a diagonal matrix of vertex
strength,
\deqn{G_{pq} = (e^{\mathbf{D}^{-1/2} \mathbf{A} \mathbf{D}^{-1/2}})_{pq}}
}
\references{
Estrada, E. and Hatano, N. (2008) Communicability in complex
  networks. \emph{Physical Review E}. \bold{77}, 036111.
  \doi{10.1103/PhysRevE.77.036111}

Crofts, J.J. and Higham, D.J. (2009) A weighted communicability
  measure applied to complex brain networks. \emph{J. R. Soc. Interface}.
  \bold{6}, 411--414. \doi{10.1098/rsif.2008.0484}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graph_efficiency.R
\name{efficiency}
\alias{efficiency}
\title{Calculate graph global, local, or nodal efficiency}
\usage{
efficiency(
  g,
  type = c("local", "nodal", "global"),
  weights = NULL,
  xfm = FALSE,
  xfm.type = NULL,
  use.parallel = TRUE,
  A = NULL,
  D = NULL
)
}
\arguments{
\item{g}{An \code{igraph} graph object}

\item{type}{Character string; either \code{local}, \code{nodal}, or
\code{global}. Default: \code{local}}

\item{weights}{Numeric vector of edge weights; if \code{NULL} (the default),
and if the graph has edge attribute \code{weight}, then that will be used.
To avoid using weights, this should be \code{NA}.}

\item{xfm}{Logical indicating whether to transform the edge weights. Default:
\code{FALSE}}

\item{xfm.type}{Character string specifying how to transform the weights.
Default: \code{1/w}}

\item{use.parallel}{Logical indicating whether or not to use \code{foreach}.
Default: \code{TRUE}}

\item{A}{Numeric matrix; the adjacency matrix of the input graph. Default:
\code{NULL}}

\item{D}{Numeric matrix; the graph's \dQuote{distance matrix}}
}
\value{
A numeric vector of the efficiencies for each vertex of the graph
  (if \emph{type} is \code{local|nodal}) or a single number (if \emph{type}
  is \code{global}).
}
\description{
This function calculates the global efficiency of a graph or the local or
nodal efficiency of each vertex of a graph.
}
\details{
Local efficiency for vertex \emph{i} is:
\deqn{E_{local}(i) = \frac{1}{N} \sum_{i \in G} E_{global}(G_i)}
where \eqn{G_i} is the subgraph of neighbors of \emph{i}, and \emph{N} is the
number of vertices in that subgraph.

Nodal efficiency for vertex \emph{i} is:
\deqn{E_{nodal}(i) = \frac{1}{N-1} \sum_{j \in G} \frac{1}{d_{ij}}}

Global efficiency for graph \emph{G} with \emph{N} vertices is:
\deqn{E_{global}(G) = \frac{1}{N(N-1)} \sum_{i \ne j \in G} \frac{1}{d_{ij}}}
where \eqn{d_{ij}} is the shortest path length between vertices \emph{i} and
\emph{j}. Alternatively, global efficiency is equal to the mean of all nodal
efficiencies.
}
\references{
Latora, V. and Marchiori, M. (2001) Efficient behavior of
  small-world networks. \emph{Phys Rev Lett}, \bold{87.19}, 198701.
  \doi{10.1103/PhysRevLett.87.198701}

Latora, V. and Marchiori, M. (2003) Economic small-world
  behavior in weighted networks. \emph{Eur Phys J B}, \bold{32}, 249--263.
  \doi{10.1140/epjb/e2003-00095-5}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_brainGraph.R
\name{Plotting GLM graphs}
\alias{Plotting GLM graphs}
\alias{plot.brainGraph_NBS}
\alias{plot.brainGraph_GLM}
\alias{plot.brainGraph_mtpc}
\alias{plot.brainGraph_mediate}
\title{Plot a graph with results from GLM-based analyses}
\usage{
\method{plot}{brainGraph_NBS}(
  x,
  alpha = 0.05,
  subgraph = paste("p.nbs >", 1 - alpha),
  vertex.label = NA,
  vertex.color = "color.comp",
  edge.color = "color.comp",
  subtitle = NULL,
  main = paste0("NBS: ", x$name),
  cex.main = 2,
  ...
)

\method{plot}{brainGraph_GLM}(
  x,
  p.sig = c("p", "p.fdr", "p.perm"),
  subgraph = NULL,
  main = paste0(x$outcome, ": ", x$name),
  subtitle = NULL,
  cex.main = 2,
  ...
)

\method{plot}{brainGraph_mtpc}(
  x,
  subgraph = "sig == 1",
  main = paste0(x$outcome, ": ", x$name),
  subtitle = NULL,
  cex.main = 2,
  ...
)

\method{plot}{brainGraph_mediate}(
  x,
  subgraph = "p.acme > 0.95",
  main = sprintf("Effect of \\"\%s\\" on\\n\\"\%s\\"\\nmediated by \\"\%s\\"", x$treat,
    x$outcome, x$mediator),
  subtitle = NULL,
  cex.main = 1,
  ...
)
}
\arguments{
\item{x}{A \code{brainGraph_GLM}, \code{brainGraph_mtpc},
\code{brainGraph_mediate}, or \code{brainGraph_NBS} object}

\item{alpha}{Numeric; the significance level. Default: \code{0.05}}

\item{subgraph}{Character string specifying the condition for subsetting the
graph.}

\item{vertex.label}{Character vector of the vertex labels to be displayed.}

\item{vertex.color}{Character string specifying the vertex attribute to color
the vertices by.}

\item{edge.color}{Character string specifying the edge attribute to color
the edges by.}

\item{subtitle}{Character string; the subtitle. Default: \code{'default'}}

\item{main}{Character string; the main title. Default: \code{NULL}}

\item{cex.main}{Numeric; the scaling factor for text size; see
\code{\link[graphics]{par}}}

\item{...}{Other arguments passed to \code{\link{plot.brainGraph}}}

\item{p.sig}{Character string indicating which p-value to use for determining
significance (default: \code{p})}
}
\description{
These methods are convenience functions for plotting a graph based on results
from GLM-based analyses (i.e., \code{\link{brainGraph_GLM},
\link{brainGraph_mediate}, \link{mtpc}, \link{NBS}}). There are several
default arguments which differ depending on the input object.
}
\details{
The default arguments are specified so that the user only needs to type
\code{plot(x)} at the console, if desired. For all methods, the plot's
\emph{subtitle} will be omitted.
}
\section{NBS}{

By default, a subgraph will be plotted consisting of only those vertices
which are part of a significant connected component. Vertex/edge colors will
correspond to connected component membership. Vertex names will be omitted.
Finally, the plot title will contain the contrast name.
}

\section{brainGraph_GLM}{

By default, a subgraph will be plotted consisting of only those vertices for
which \eqn{p < \alpha}. It will also include a plot title with the outcome
measure and contrast name.
}

\section{mtpc}{

By default, a subgraph will be plotted consisting of only those vertices for
which \eqn{A_{mtpc} > A_{crit}}. It will also include a plot title with the
outcome measure and contrast name.
}

\section{brainGraph_mediate}{

By default, a subgraph will be plotted consisting of only those vertices for
which \eqn{P_{acme} < \alpha}. It will also include a plot title with the
treatment, mediator, and outcome variable names.
}

\seealso{
Other Plotting functions: 
\code{\link{plot.brainGraph}()},
\code{\link{plot.brainGraphList}()},
\code{\link{plot_brainGraph_multi}()}
}
\concept{Plotting functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/contract_brainGraph.R
\name{contract_brainGraph}
\alias{contract_brainGraph}
\title{Contract graph vertices based on brain lobe and hemisphere}
\usage{
contract_brainGraph(g, vgroup = "lobe.hemi")
}
\arguments{
\item{g}{A \code{brainGraph} graph object}

\item{vgroup}{Character string; the name of the vertex attribute to use when
contracting the graph. Default: \code{'lobe.hemi'}}
}
\value{
A new \code{brainGraph} graph object with vertex-level attributes
  representing the mean spatial coordinates, and vertex- and edge-level
  attributes of color names
}
\description{
Create a new graph after merging vertices within specified groups.
By default, groups are brain \emph{lobe} and \emph{hemisphere} membership.
}
\details{
The \code{size} vertex-level attribute of the resultant graph is equal to the
number of vertices in each group. The x-, y-, and z-coordinates of the new
graph are equal to the mean coordinates of the vertices per group.
The new edge weights are equal to the number of inter-group connections of
the original graph.
}
\seealso{
\code{\link[igraph]{contract}}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import.R
\name{update_fs_names}
\alias{update_fs_names}
\title{Update column names in a Freesurfer table}
\usage{
update_fs_names(filename, modality, parcellation, hemi, exclude.subs)
}
\description{
Updates column names by abbreviating to match with the names in the
\code{brainGraph} atlases.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boot_global.R
\name{Bootstrapping}
\alias{Bootstrapping}
\alias{brainGraph_boot}
\alias{summary.brainGraph_boot}
\alias{plot.brainGraph_boot}
\title{Bootstrapping for global graph measures}
\usage{
brainGraph_boot(
  densities,
  resids,
  R = 1000,
  measure = c("mod", "E.global", "Cp", "Lp", "assortativity", "strength", "mod.wt",
    "E.global.wt"),
  conf = 0.95,
  .progress = getOption("bg.progress"),
  xfm.type = c("1/w", "-log(w)", "1-w", "-log10(w/max(w))", "-log10(w/max(w)+1)")
)

\method{summary}{brainGraph_boot}(object, ...)

\method{plot}{brainGraph_boot}(x, ..., alpha = 0.4)
}
\arguments{
\item{densities}{Numeric vector of graph densities to loop through}

\item{resids}{An object of class \code{brainGraph_resids} (the output from
\code{\link{get.resid}})}

\item{R}{Integer; the number of bootstrap replicates. Default: \code{1e3}}

\item{measure}{Character string of the measure to test. Default: \code{mod}}

\item{conf}{Numeric; the level for calculating confidence intervals. Default:
\code{0.95}}

\item{.progress}{Logical indicating whether or not to show a progress bar.
Default: \code{getOption('bg.progress')}}

\item{xfm.type}{Character string specifying how to transform the weights.
Default: \code{1/w}}

\item{object, x}{A \code{brainGraph_boot} object}

\item{...}{Unused}

\item{alpha}{A numeric indicating the opacity for the confidence bands}
}
\value{
\code{brainGraph_boot} -- an object of class \code{brainGraph_boot}
  containing some input variables, in addition to a list of
  \code{\link[boot]{boot}} objects (one for each group).

\code{plot} -- \emph{list} with the following elements:
  \item{se}{A ggplot object with ribbon representing standard error}
  \item{ci}{A ggplot object with ribbon representing confidence intervals}
}
\description{
Perform bootstrapping to obtain groupwise standard error estimates of a
global graph measure.

The \code{plot} method returns two \code{ggplot} objects: one with shaded
regions based on the standard error, and the other based on confidence
intervals (calculated using the normal approximation).
}
\details{
The confidence intervals are calculated using the \emph{normal approximation}
at the \eqn{100 \times conf}\% level (by default, 95\%).

For getting estimates of \emph{weighted global efficiency}, a method for
transforming edge weights must be provided. The default is to invert them.
See \code{\link{xfm.weights}}.
}
\examples{
\dontrun{
boot.E.global <- brainGraph_boot(densities, resids.all, 1e3, 'E.global')
}
}
\seealso{
\code{\link[boot]{boot}}, \code{\link[boot]{boot.ci}}

Other Group analysis functions: 
\code{\link{GLM}},
\code{\link{Mediation}},
\code{\link{NBS}()},
\code{\link{brainGraph_permute}()},
\code{\link{mtpc}()}

Other Structural covariance network functions: 
\code{\link{IndividualContributions}},
\code{\link{Residuals}},
\code{\link{brainGraph_permute}()},
\code{\link{corr.matrix}()},
\code{\link{import_scn}()},
\code{\link{plot_volumetric}()}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}
\concept{Group analysis functions}
\concept{Structural covariance network functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{check_sID}
\alias{check_sID}
\alias{pad_zeros}
\title{Test if an object is a character vector of numbers}
\usage{
check_sID(x)

pad_zeros(x)
}
\arguments{
\item{x}{\code{pad_zeros} accepts either a vector (numeric or character) or a
single integer. \code{check_sID} accepts a character, numeric, or factor
vector}
}
\value{
\code{check_sID} returns either the input vector or a character
  vector padded with \code{0}

A character vector with zero-padded values
}
\description{
\code{check_sID} is a convenience function to test if a vector (typically the
\emph{subject ID} column in a \code{data.table}) is a character vector of
numbers, a factor vector of numbers, or a numeric vector. If so, it will
zero-pad the variable to have equal width.

\code{pad_zeros} pads a vector with zeros to avoid issues with ordering a
column of integers or integers converted to \code{character}.
}
\details{
This function is meant to avoid issues that arise when sorting a vector of
numbers that have been converted to \code{character}. For example,
\code{\link{import_scn}} automatically reads in the first column (with
\emph{FreeSurfer} outputs this is the column of subject IDs) as a
\code{character} variable. If the subject IDs had been all numbers/integers,
then sorting (i.e., setting the \code{key} in a \code{data.table}) would be
incorrect: e.g., it might be \code{'1', '10', '2', ...}.

If \dQuote{x} is a numeric vector, then the resultant string width will be
determined by \code{max(x)} or \code{x} itself if the input is a single
integer. For example, if \code{x=10}, it will return \code{'01', '02', ...,
'10'}. If \dQuote{x} is a character vector, then the output's string width
will be \code{max(nchar(x))}. For example, if \code{x} includes both
\code{'1'} and \code{'1000'}, it will return \code{'0001'}, etc.
}
\examples{
pad_zeros(10)  # '01' '02' ... '10'
x <- c(1, 10, 100)
pad_zeros(x)   # '001' '010' '100'
x <- as.character(x)
pad_zeros(x)   # '001' '010' '100'
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/robustness.R
\name{robustness}
\alias{robustness}
\title{Analysis of network robustness}
\usage{
robustness(
  g,
  type = c("vertex", "edge"),
  measure = c("btwn.cent", "degree", "random"),
  N = 1000
)
}
\arguments{
\item{g}{An \code{igraph} graph object}

\item{type}{Character string; either \code{'vertex'} or \code{'edge'}
removals. Default: \code{vertex}}

\item{measure}{Character string; sort by either \code{'btwn.cent'} or
\code{'degree'}, or choose \code{'random'}. Default: \code{'btwn.cent'}}

\item{N}{Integer; the number of iterations if \code{'random'} is chosen.
Default: \code{1e3}}
}
\value{
Data table with elements:
  \item{type}{Character string describing the type of analysis performed}
  \item{measure}{The input argument \code{measure}}
  \item{comp.size}{The size of the largest component after edge/vertex
    removal}
  \item{comp.pct}{Numeric vector of the ratio of maximal component size after
    each removal to the observed graph's maximal component size}
  \item{removed.pct}{Numeric vector of the ratio of vertices/edges removed}
  \item{Group}{Character string indicating the subject group, if applicable}
}
\description{
This function performs a \dQuote{targeted attack} of a graph or a
\dQuote{random failure} analysis, calculating the size of the largest
component after edge or vertex removal.
}
\details{
In a targeted attack, it will sort the vertices by either degree or
betweenness centrality (or sort edges by betweenness), and successively
remove the top vertices/edges. Then it calculates the size of the largest
component.

In a random failure analysis, vertices/edges are removed in a random order.
}
\references{
Albert, R. and Jeong, H. and Barabasi, A. (2000) Error and attack
  tolerance of complex networks. \emph{Nature}, \bold{406}, 378--381.
  \doi{10.1038/35019019}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_vertex_measures.R
\name{plot_vertex_measures}
\alias{plot_vertex_measures}
\title{Plot vertex-level graph measures at a single density or threshold}
\usage{
plot_vertex_measures(
  g.list,
  measure,
  facet.by = NULL,
  group.by = getOption("bg.group"),
  type = c("violin", "boxplot"),
  show.points = FALSE,
  ylabel = measure,
  ...
)
}
\arguments{
\item{g.list}{A \code{brainGraphList} or a list of \code{brainGraph} objects}

\item{measure}{A character string of the graph measure to plot}

\item{facet.by}{Character string indicating the variable to facet by (if
any). Default: \code{NULL}}

\item{group.by}{Character string indicating which variable to group the data
by. Default: \code{getOption('bg.group')}}

\item{type}{Character string indicating the plot type. Default:
\code{'violin'}}

\item{show.points}{Logical indicating whether or not to show individual data
points (default: FALSE)}

\item{ylabel}{A character string for the y-axis label}

\item{...}{Arguments passed to \code{geom_boxplot} or \code{geom_violin}}
}
\value{
A \code{trellis} or \code{ggplot} object
}
\description{
Creates boxplots of a single vertex-level graph measure at a single density
or threshold, grouped by the variable specified by \code{group.by} and
optionally faceted by another variable (e.g., \emph{lobe} or \emph{network}).
}
\examples{
\dontrun{
p.deg <- plot_vertex_measures(g[[1]], facet.by='network', measure='degree')
}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mtpc.R, R/methods.R
\name{mtpc}
\alias{mtpc}
\alias{summary.mtpc}
\alias{plot.mtpc}
\alias{nobs.mtpc}
\alias{terms.mtpc}
\alias{formula.mtpc}
\alias{labels.mtpc}
\alias{case.names.mtpc}
\alias{variable.names.mtpc}
\alias{df.residual.mtpc}
\alias{region.names.mtpc}
\alias{nregions.mtpc}
\title{Multi-threshold permutation correction}
\usage{
mtpc(
  g.list,
  thresholds,
  covars,
  measure,
  contrasts,
  con.type = c("t", "f"),
  outcome = NULL,
  con.name = NULL,
  level = c("vertex", "graph"),
  clust.size = 3L,
  perm.method = c("freedmanLane", "terBraak", "smith", "draperStoneman", "manly",
    "stillWhite"),
  part.method = c("beckmann", "guttman", "ridgway"),
  N = 500L,
  perms = NULL,
  alpha = 0.05,
  res.glm = NULL,
  long = TRUE,
  ...
)

\method{summary}{mtpc}(
  object,
  contrast = NULL,
  digits = max(3L, getOption("digits") - 2L),
  print.head = TRUE,
  ...
)

\method{plot}{mtpc}(
  x,
  contrast = 1L,
  region = NULL,
  only.sig.regions = TRUE,
  show.null = TRUE,
  caption.stats = FALSE,
  ...
)

\method{nobs}{mtpc}(object, ...)

\method{terms}{mtpc}(x, ...)

\method{formula}{mtpc}(x, ...)

\method{labels}{mtpc}(object, ...)

\method{case.names}{mtpc}(object, ...)

\method{variable.names}{mtpc}(object, ...)

\method{df.residual}{mtpc}(object, ...)

\method{region.names}{mtpc}(object)

\method{nregions}{mtpc}(object)
}
\arguments{
\item{g.list}{A list of \code{brainGraphList} objects for all thresholds}

\item{thresholds}{Numeric vector of the thresholds applied to the raw
connectivity matrices.}

\item{covars}{A \code{data.table} of covariates}

\item{measure}{Character string of the graph measure of interest}

\item{contrasts}{Numeric matrix (for T statistics) or list of matrices (for F
statistics) specifying the contrast(s) of interest; if only one contrast is
desired, you can supply a vector (for T statistics)}

\item{con.type}{Character string; either \code{'t'} or \code{'f'} (for t or
F-statistics). Default: \code{'t'}}

\item{outcome}{Character string specifying the name of the outcome variable,
if it differs from the graph metric (\code{measure})}

\item{con.name}{Character vector of the contrast name(s); if \code{contrasts}
has row/list names, those will be used for reporting results}

\item{level}{Character string; either \code{vertex} (default) or
\code{graph}}

\item{clust.size}{Integer indicating the size of \dQuote{clusters} (i.e.,
consecutive thresholds for which the observed statistic exceeds the null)
(default: \code{3L})}

\item{perm.method}{Character string indicating the permutation method.
Default: \code{'freedmanLane'}}

\item{part.method}{Character string; the method of partitioning the design
matrix into covariates of interest and nuisance. Default: \code{'beckmann'}}

\item{N}{Integer; number of permutations to create. Default: \code{5e3}}

\item{perms}{Matrix of permutations, if you would like to provide your own.
Default: \code{NULL}}

\item{alpha}{Numeric; the significance level. Default: 0.05}

\item{res.glm}{A list of \code{bg_GLM} objects, as output by a previous run
of \code{mtpc}. Useful if you want to change the cluster size without
re-running all of the GLM's and permutations (default: \code{NULL})}

\item{long}{Logical indicating whether or not to return all permutation
results. Default: \code{FALSE}}

\item{...}{Other arguments passed to \code{\link{brainGraph_GLM}} and/or
\code{\link{brainGraph_GLM_design}}}

\item{object, x}{A \code{mtpc} object}

\item{contrast}{Integer specifying the contrast to plot/summarize; defaults
to showing results for all contrasts}

\item{digits}{Integer specifying the number of digits to display for P-values}

\item{print.head}{Logical indicating whether or not to print only the first
and last 5 rows of the statistics tables (default: \code{TRUE})}

\item{region}{Character string specifying which region's results to
plot; only relevant if \code{level='vertex'}. Default: \code{NULL}}

\item{only.sig.regions}{Logical indicating whether to plot only significant
regions (default: \code{TRUE})}

\item{show.null}{Logical indicating whether to plot points of the maximum
null statistics (per permutation)}

\item{caption.stats}{Logical indicating whether to print the MTPC statistics
in the caption of the plot. Default: \code{FALSE}}
}
\value{
An object of class \code{mtpc} with some input arguments plus the
  following elements:
  \item{X,qr,cov.unscaled}{Design matrix, QR decomposition, and unscaled
    covariance matrix, if the design is the same across thresholds}
  \item{contrasts}{The contrast matrix or list of matrices}
  \item{con.name}{Contrast names}
  \item{removed.subs}{Named integer vector of subjects with incomplete data}
  \item{atlas}{The atlas of the input graphs}
  \item{rank,df.residual}{The model rank and residual degrees of freedom}
  \item{res.glm}{List with length equal to the number of thresholds; each
    list element is the output from \code{\link{brainGraph_GLM}}}
  \item{DT}{A \code{data.table} for all thresholds, combined from the outputs
    of \code{\link{brainGraph_GLM}}}
  \item{stats}{A data.table containing \code{S.mtpc} (the max. observed
    statistic), \code{tau.mtpc} (the threshold of the max. observed
    statistic), \code{S.crit} (the critical statistic value), and
    \code{A.crit} (the critical AUC)}
  \item{null.dist}{Numeric array with \code{N} columns and number of rows
    equal to the number of thresholds. The 3rd dimension is for each
    contrast. Each element of the array is the maximum statistic
    for that permutation, threshold, and contrast combination.}
  \item{perm.order}{Numeric matrix; the permutation set applied for all
    thresholds (each row is a separate permutation)}

The \code{plot} method returns a \code{trellis} object or a list of
  \code{ggplot} objects
}
\description{
Applies the \emph{multi-threshold permutation correction (MTPC)} method to
perform inference in graph theory analyses of brain MRI data.

Plot the statistics from an MTPC analysis, along with the maximum permuted
statistics. The output is similar to Figure 11 in Drakesmith et al. (2015).
}
\details{
This is a multi-step procedure: (steps 3-4 are the time-consuming steps)
\enumerate{
  \item Apply thresholds \eqn{\tau} to the networks, and compute network
    metrics for all networks and thresholds. (already done beforehand)
  \item Compute test statistics \eqn{S_{obs}} for each threshold. (done by
    \code{\link{brainGraph_GLM}})
  \item Permute group assignments and compute test statistics for each
    permutation and threshold. (done by \code{\link{brainGraph_GLM}})
  \item Build a null distribution of the maximum statistic across thresholds
    (and across brain regions) for each permutation. (done by
    \code{\link{brainGraph_GLM}})
  \item Determine the critical value, \eqn{S_{crit}} from the null
    distribution of maximum statistics.
  \item Identify clusters where \eqn{S_{obs} > S_{crit}} and compute the AUC
    for these clusters (denoted \eqn{A_{MTPC}}).
  \item Compute a critical AUC (\eqn{A_{crit}}) from the mean of the
    supra-critical AUC's for the permuted tests.
  \item Reject \eqn{H_0} if \eqn{A_{MTPC} > A_{crit}}.
}
}
\examples{
\dontrun{
diffs.mtpc <- mtpc(g.list=g.norm, thresholds=thresholds, N=N,
     covars=covars.dti, measure='E.nodal.wt', coding='effects',
     contrasts=c(0, 0, 0, 0, -2), alt='greater',
     binarize=c('Sex', 'Scanner'), con.name='Group 1 > Group 2')
sig.regions <- diffs.mtpc$DT[A.mtpc > A.crit]
}
\dontrun{
mtpcPlots <- plot(mtpc.diffs)

## Arrange plots into 3x3 grids
ml <- marrangeGrob(mtpcPlots, nrow=3, ncol=3)
ggsave('mtpc.pdf', ml)
}
}
\references{
Drakesmith, M. and Caeyenberghs, K. and Dutt, A. and Lewis, G. and
  David, A.S. and Jones, D.K. (2015) Overcoming the effects of false
  positives and threshold bias in graph theoretical analyses of neuroimaging
  data. \emph{NeuroImage}, \bold{118}, 313--333.
  \doi{10.1016/j.neuroimage.2015.05.011}
}
\seealso{
Other Group analysis functions: 
\code{\link{Bootstrapping}},
\code{\link{GLM}},
\code{\link{Mediation}},
\code{\link{NBS}()},
\code{\link{brainGraph_permute}()}

Other GLM functions: 
\code{\link{GLM}},
\code{\link{GLM design}},
\code{\link{GLM fits}}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}
\concept{GLM functions}
\concept{Group analysis functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/auc.R
\name{make_auc_brainGraph}
\alias{make_auc_brainGraph}
\title{Calculate the AUC across densities of given attributes}
\usage{
make_auc_brainGraph(g.list, g.attr = NULL, v.attr = NULL, norm = FALSE)
}
\arguments{
\item{g.list}{A list of \code{brainGraphList} objects}

\item{g.attr}{A character vector of graph attribute name(s). Default:
\code{NULL}}

\item{v.attr}{A character vector of vertex attribute name(s). Default:
\code{NULL}}

\item{norm}{Logical indicating whether to normalize threshold values to be
between 0 and 1 (inclusive). Default: \code{FALSE}}
}
\value{
A \code{brainGraphList} object with one graph for each subject
}
\description{
Given a list of \code{brainGraphList} objects, this function will calculate
the area under the curve (AUC) across all thresholds/densities for each
subject or group.
}
\details{
If the elements of the input list do not have a \code{threshold} element (or
if it is \code{NULL}) then the AUC will be calculated on the interval
\eqn{[0, 1]} (inclusive). This has the same effect as specifying
\code{norm=TRUE} in the function call.
}
\examples{
\dontrun{
g.auc <- make_auc_brainGraph(g.fa, g.attr='E.global.wt')
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create_mats.R
\name{apply_thresholds}
\alias{apply_thresholds}
\title{Threshold additional set of matrices}
\usage{
apply_thresholds(sub.mats, group.mats, W.files, inds)
}
\arguments{
\item{sub.mats}{List (length equal to number of thresholds) of numeric arrays
(3-dim) for all subjects}

\item{group.mats}{List (length equal to number of thresholds) of numeric
arrays (3-dim) for group-level data}

\item{W.files}{Character vector of the filenames of the files with
connectivity matrices}

\item{inds}{List (length equal to number of groups) of integers; each list
element should be a vector of length equal to the group sizes}
}
\value{
List containing:
\item{W}{A 3-d array of the raw connection matrices}
\item{W.norm.sub}{List of 3-d arrays of the normalized connection matrices
  for all given thresholds}
\item{W.norm.mean}{List of 3-d arrays of the normalized connection matrices
  averaged for each group}
}
\description{
\code{apply_thresholds} thresholds an additional set of matrices (e.g.,
FA-weighted matrices for DTI tractography) based on the matrices that have
been returned from \code{\link{create_mats}}. This ensures that the same
connections are present in both sets of matrices.
}
\details{
The argument \code{W.files} accepts the same formats as \code{A.files}; see
\code{\link{create_mats}} for details.
}
\examples{
\dontrun{
  W.mats <- apply_thresholds(A.norm.sub, A.norm.mean, f.W, inds)
}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glm_design.R
\name{GLM design}
\alias{GLM design}
\alias{brainGraph_GLM_design}
\title{Create a design matrix for linear model analysis}
\usage{
brainGraph_GLM_design(
  covars,
  coding = c("dummy", "effects", "cell.means"),
  factorize = TRUE,
  binarize = NULL,
  int = NULL,
  mean.center = FALSE,
  center.how = c("all", "within-groups"),
  center.by = getOption("bg.group")
)
}
\arguments{
\item{covars}{A \code{data.table} of covariates}

\item{coding}{Character string indicating how factor variables will be coded.
Default: \code{'dummy'}}

\item{factorize}{Logical indicating whether to convert \emph{character}
columns into \emph{factor}. Default: \code{TRUE}}

\item{binarize}{Character vector specifying the column name(s) of the
covariate(s) to be converted from type \code{factor} to \code{numeric}.
Default: \code{NULL}}

\item{int}{Character vector specifying the column name(s) of the
covariate(s) to test for an interaction. Default: \code{NULL}}

\item{mean.center}{Logical indicating whether to mean center non-factor
variables. Default: \code{FALSE}}

\item{center.how}{Character string indicating whether to use the grand mean
or groupwise means. Default: \code{'all'}}

\item{center.by}{Character string indicating which grouping variable to use
for calculating means (if applicable). Default: \code{'Group'}}
}
\value{
A numeric matrix. Rownames are subject ID's and column names are the
  variable names. There will be additional attributes recording the
  \code{coding}, \code{factorize}, and \code{mean.center} function arguments.
  There will also be attributes for \code{binarize} and \code{int} if they
  are not \code{NULL}, and \code{center.how} and \code{center.by} if
  \code{mean.center=TRUE}.
}
\description{
\code{brainGraph_GLM_design} takes a \code{data.table} of covariates and
returns a \emph{design matrix} to be used in linear model analysis.
}
\details{
There are three different ways to code factors: \emph{dummy}, \emph{effects},
or \emph{cell-means} (chosen by the argument \code{coding}). \emph{Effects}
coding is sometimes referred to as \emph{deviation} coding. \emph{Dummy}
coding is the default when calling \code{lm}. To understand the difference
between these, see Chapter 8 of the User Guide.
}
\section{Character variables}{

The default behavior is to convert all character columns (excluding the Study
ID column and any that you list in the \code{binarize} argument) to factor
variables. To change this, set \code{factorize=FALSE}. So, if your covariates
include multiple character columns, but you want to convert \emph{Scanner} to
binary instead of a factor, you may still specify \code{binarize='Scanner'}
and get the expected result. \code{binarize} will convert the given factor
variable(s) into numeric variable(s), which is performed \emph{before}
centering (if applicable).
}

\section{Centering}{

The argument \code{mean.center} will mean-center (i.e., subtract the mean of
from each variable) any non-factor variables (including any dummy/indicator
covariates). This is done \emph{after} \dQuote{factorizing} and
\dQuote{binarizing}. If \code{center.how='all'}, then the \dQuote{grand mean}
will be used; otherwise, the groupwise means will be used. The grouping
variable is determined by \code{center.by} and is by default \code{'Group'}.
}

\section{Interactions}{

\code{int} specifies which variables should interact with one another. This
argument accepts both numeric/continuous (e.g., \emph{Age}) and factor
variables (e.g., \emph{Sex}). All interaction combinations will be generated:
if you supply 3 variables, all two-way and the single three-way interaction
will be generated. This variable \emph{must} have at least two elements; it
is otherwise ignored. It is generally recommended that centering be performed
when including interaction terms.
}

\examples{
\dontrun{
# Recreate design matrix when "outcome == measure"
DT <- res.glm$DT.Xy[region == levels(region)[1L],
                    !c('region', res.glm$outcome),
                    with=FALSE]
X <- do.call(brainGraph_GLM_design, c(list(covars=DT),
                                      attributes(res.glm$X)[-c(1L, 2L)]))
all.equal(X, res.glm$X)
}
}
\seealso{
Other GLM functions: 
\code{\link{GLM}},
\code{\link{GLM fits}},
\code{\link{mtpc}()}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}
\concept{GLM functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create_mats.R
\name{dir2files}
\alias{dir2files}
\title{Return a vector of filenames based on a directory name or options list}
\usage{
dir2files(x)
}
\arguments{
\item{x}{Either a character string specifying a directory or a list of
arguments to be passed to \code{\link{list.files}}}
}
\description{
Return a vector of filenames based on a directory name or options list
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{brainGraph-methods}
\alias{brainGraph-methods}
\alias{groups.brainGraphList}
\alias{groups.corr_mats}
\alias{region.names}
\alias{region.names.data.table}
\alias{nregions}
\title{brainGraph generic methods}
\usage{
\method{groups}{brainGraphList}(x)

\method{groups}{corr_mats}(x)

region.names(object)

\method{region.names}{data.table}(object)

nregions(object)
}
\arguments{
\item{x, object}{An object}
}
\description{
These functions are S3 \emph{generics} for various \code{brainGraph}-defined
objects.

\code{groups} returns the \dQuote{Group} graph attribute for each graph or
observation in the object.

\code{region.names} is a generic method for extracting region names from
various \code{brainGraph} objects. These are generally convenience functions.

\code{nregions} is a generic method for extracting the number of regions from
various \code{brainGraph} objects.
}
\details{
For a \code{data.table}, \code{region.names} assumes that it contains a
\emph{factor} column named \code{region}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create_mats.R
\name{create_mats}
\alias{create_mats}
\title{Create connection matrices for tractography or fMRI data}
\usage{
create_mats(
  A.files,
  modality = c("dti", "fmri"),
  divisor = c("none", "waytotal", "size", "rowSums"),
  div.files = NULL,
  threshold.by = c("consensus", "density", "mean", "consistency", "raw"),
  mat.thresh = 0,
  sub.thresh = 0.5,
  inds = list(seq_along(A.files)),
  algo = c("probabilistic", "deterministic"),
  P = 5000,
  ...
)
}
\arguments{
\item{A.files}{Character vector of the filenames with connection matrices}

\item{modality}{Character string indicating data modality (default:
\code{dti})}

\item{divisor}{Character string indicating how to normalize the connection
matrices; either 'none' (default), 'waytotal', 'size', or 'rowSums'
(ignored if \code{modality} equals \code{fmri})}

\item{div.files}{Character vector of the filenames with the data to
normalize by (e.g. a list of \emph{waytotal} files) (default: \code{NULL})}

\item{threshold.by}{Character string indicating how to threshold the data;
choose \code{density}, \code{mean}, or \code{consistency} if you want all
resulting matrices to have the same densities (default: \code{consensus})}

\item{mat.thresh}{Numeric (vector) for thresholding connection matrices
(default: 0)}

\item{sub.thresh}{Numeric (between 0 and 1) for thresholding by subject
numbers (default: 0.5)}

\item{inds}{List (length equal to number of groups) of integers; each list
element should be a vector of length equal to the group sizes}

\item{algo}{Character string of the tractography algorithm used (default:
\code{'probabilistic'}). Ignored if \emph{modality} is \code{fmri}.}

\item{P}{Integer; number of samples per seed voxel (default: 5000)}

\item{...}{Arguments passed to \code{\link{symmetrize}}}
}
\value{
A list containing:
\item{A}{A 3-d array of the raw connection matrices}
\item{A.norm}{A 3-d array of the normalized connection matrices}
\item{A.bin}{A list of 3-d arrays of binarized connection matrices, one array
  for each threshold}
\item{A.bin.sums}{A list of 3-d arrays of connection matrices, with each
  entry signifying the number of subjects with a connection present; the
  number of list elements equals the length of \code{mat.thresh}, and the
  extent of the arrays equals the number of groups}
\item{A.inds}{A list of arrays of binarized connection matrices, containing 1
  if that entry is to be included}
\item{A.norm.sub}{List of 3-d arrays of the normalized connection matrices
  for all given thresholds}
\item{A.norm.mean}{List of 3-d arrays of connection matrices averaged for
  each group}
}
\description{
\code{create_mats} creates arrays from connection matrices (e.g.,
\emph{fdt_network_matrix} from FSL or \emph{ROICorrelation.txt} from DPABI).
You may choose to normalize these matrices by the \emph{waytotal} or
\emph{region size} (tractography), or not at all.
}
\section{Connection matrix files}{

The \code{A.files} argument is mandatory and may be specified in a few ways:
\enumerate{
  \item A character vector of the filenames (preferably with full path).
  \item A single character string specifying the \emph{directory} in which
    all connectivity matrices are located. This will load \emph{all} files in
    the directory.
  \item A \emph{named list} in which the names match the arguments to
    \code{\link{list.files}}. This will load \emph{all} files in \code{path}
    that match the \code{pattern} argument, if present, and will load
    \emph{all} files in child directories if \code{recursive=TRUE}. See
    examples below.
}
The same options apply to \code{div.files} as well.
}

\section{Thresholding methods}{

The argument \code{threshold.by} has 5 options:
\enumerate{
  \item \code{consensus} Threshold based on the raw (normalized, if selected)
    values in the matrices. If this is selected, it uses the
    \code{sub.thresh} value to perform \dQuote{consensus} thresholding.
  \item \code{density} Threshold the matrices to yield a specific graph
    density (given by the \code{mat.thresh} argument).
  \item \code{mean} Keep only connections for which the cross-subject mean is
    at least 2 standard deviations higher than the threshold (specified by
    \code{mat.thresh})
  \item \code{consistency} Threshold based on the coefficient of variation to
    yield a graph with a specific density (given by \code{mat.thresh}). The
    edge weights will still represent those of the input matrices. See
    Roberts et al. (2017) for more on \dQuote{consistency-based}
    thresholding.
  \item \code{raw} Threshold each subject's matrix \emph{individually},
    irrespective of group membership. Ignores \code{sub.thresh}.
}

The argument \code{mat.thresh} allows you to choose a numeric threshold,
below which the connections will be replaced with 0; this argument will also
accept a numeric vector. The argument \code{sub.thresh} will keep only those
connections for which at least \emph{X}\% of subjects have a positive entry
(the default is 0.5, or 50\%).
}

\examples{
\dontrun{
thresholds <- seq(from=0.001, to=0.01, by=0.001)
fmri.mats <- create_mats(f.A, modality='fmri', threshold.by='consensus',
  mat.thresh=thresholds, sub.thresh=0.5, inds=inds)
dti.mats <- create_mats(f.A, divisor='waytotal', div.files=f.way,
  mat.thresh=thresholds, sub.thresh=0.5, inds=inds)

# Specify a directory and filename pattern
conn_files <- list(path='~/data', pattern='.*fdt_network_matrix')
dti.mats <- create_mats(conn_files, ...)
}
}
\references{
Roberts, JA and Perry, A and Roberts, G and Mitchell, PB and
  Breakspear, M (2017) Consistency-based thresholding of the human
  connectome. \emph{NeuroImage}. \bold{145}, 118--129.
  \doi{10.1016/j.neuroimage.2016.09.053}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz.R
\docType{package}
\name{brainGraph}
\alias{brainGraph-package}
\alias{brainGraph}
\alias{brainGraph-options}
\title{Default options for brainGraph}
\description{
brainGraph is a package for performing \emph{graph theory analysis} of brain
MRI data.
}
\section{Package options}{


brainGraph uses the following \code{\link{options}} to configure behavior:
\itemize{
  \item \code{bg.subject_id}: character string specifying the name your
    project/study uses as a subject identifier. All imported data (e.g.,
    covariates tables) \emph{MUST} have a column matching this. One possible
    alternative is \code{'participant_id'}, recommended by BIDS. Default:
    \code{'Study.ID'}
  \item \code{bg.group}: character string specifying the name your
    project/study uses as a group identifier. All imported data (e.g.,
    covariates tables) \emph{MUST} have a column matching this. One possible
    alternative is \code{'group'}, recommended by BIDS. Default:
    \code{'Group'}
  \item \code{bg.session}: character string specifying the name your
    project/study uses as a \dQuote{time} or session identifier, in the case
    of longitudinal studies. All imported data (e.g., covariates tables)
    \emph{MUST} have a column matching this. One possible alternative is
    \code{'session_id'}, recommended by BIDS. Default: \code{'Time'}
  \item \code{bg.progress}: logical indicating whether to show progress bars
    for functions that provide the option. Default: \code{TRUE}
  \item \code{bg.ncpus}: integer indicating the number of cores to use for
    parallel operations. Only used if you have not already registered a
    parallel backend (see Chapter 5 of the User Guide or
    \url{https://github.com/cwatson/brainGraph/blob/master/README.md} for
    examples). Default: \code{2L}
}
}

\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/cwatson/brainGraph}
  \item Report bugs at \url{https://groups.google.com/forum/?hl=en#!forum/brainGraph-help}
}

}
\author{
\strong{Maintainer}: Christopher G. Watson \email{cgwatson@bu.edu} (\href{https://orcid.org/0000-0002-7082-7631}{ORCID})

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_resid.R, R/methods.R
\name{Residuals}
\alias{Residuals}
\alias{get.resid}
\alias{Extract.brainGraph_resids}
\alias{[.brainGraph_resids}
\alias{summary.brainGraph_resids}
\alias{plot.brainGraph_resids}
\alias{nobs.brainGraph_resids}
\alias{case.names.brainGraph_resids}
\alias{groups.brainGraph_resids}
\alias{region.names.brainGraph_resids}
\alias{nregions.brainGraph_resids}
\title{Linear model residuals in structural covariance networks}
\usage{
get.resid(
  dt.vol,
  covars,
  method = c("comb.groups", "sep.groups"),
  use.mean = FALSE,
  exclude.cov = NULL,
  atlas = NULL,
  ...
)

\method{[}{brainGraph_resids}(x, i, g = NULL)

\method{summary}{brainGraph_resids}(object, region = NULL, outlier.thresh = 2, ...)

\method{plot}{brainGraph_resids}(x, region = NULL, outlier.thresh = 2, cols = FALSE, ids = TRUE, ...)

\method{nobs}{brainGraph_resids}(object, ...)

\method{case.names}{brainGraph_resids}(object, ...)

\method{groups}{brainGraph_resids}(x)

\method{region.names}{brainGraph_resids}(object)

\method{nregions}{brainGraph_resids}(object)
}
\arguments{
\item{dt.vol}{A \code{data.table} containing all the volumetric measure of
interest (i.e., the object \code{lhrh} as output by
\code{\link{import_scn}})}

\item{covars}{A \code{data.table} of the covariates of interest}

\item{method}{Character string indicating whether to test models for subject
groups separately or combined. Default: \code{comb.groups}}

\item{use.mean}{Logical should we control for the mean hemispheric brain
value (e.g. mean LH/RH cortical thickness). Default: \code{FALSE}}

\item{exclude.cov}{Character vector of covariates to exclude. Default:
\code{NULL}}

\item{atlas}{Character string indicating the brain atlas}

\item{...}{Arguments passed to \code{\link{brainGraph_GLM_design}} (optional)}

\item{x, object}{A \code{brainGraph_resids} object}

\item{i}{Numeric vector of the indices}

\item{g}{Character string indicating the group. Default: \code{NULL}}

\item{region}{Character vector of region(s) to focus on; default behavior is
to show summary for all regions}

\item{outlier.thresh}{Number indicating how many standard deviations
above/below the mean indicate an outlier. Default: \code{2}}

\item{cols}{Logical indicating whether to color by group. Default:
\code{FALSE}}

\item{ids}{Logical indicating whether to plot subject ID's for outliers.
Otherwise plots the integer index}
}
\value{
\code{get.resid} - an object of class \code{brainGraph_resids} with
  elements:
  \item{data}{A data.table with the input volume/thickness/etc. data as well
    as the covariates used in creating the design matrix.}
  \item{X}{The \emph{design matrix}, if using default arguments. If
  \code{use.mean=TRUE} then it will be a \emph{named list} with a separate
  matrix for the left and right hemispheres. If \code{method='sep.groups'}, a
  nested named list for each group and hemisphere.}
  \item{method}{The input argument \code{method}}
  \item{use.mean}{The input argument \code{use.mean}}
  \item{resids.all}{The \dQuote{wide} \code{data.table} of residuals}
  \item{Group}{Group names}
  \item{atlas}{The atlas name}

\code{\link{summary.brainGraph_resids}} returns a list with two
  data tables, one of the residuals, and one of only the outlier regions

The \code{plot} method returns a \code{trellis} object or a list of
  \code{ggplot} objects
}
\description{
\code{get.resid} runs linear models across brain regions listed in a
\code{data.table} (e.g., cortical thickness), adjusting for variables in
\code{covars} (e.g. age, sex, etc.), and calculates the
\emph{externally Studentized} (or \emph{leave-one-out}) residuals.

The \code{[} method reorders or subsets residuals based on a given
numeric vector. However, this is used in bootstrap and permutation analysis
and should generally not be called directly by the user.

The \code{summary} method prints the number of outliers per region, and the
number of times a given subject was an outlier (i.e., across regions).

The \code{plot} method lets you check the model residuals for each brain
region in a structural covariance analysis. It shows a \emph{qqplot} of the
studentized residuals, as output from \code{\link{get.resid}}.
}
\details{
You can choose to run models for each of your subject groups separately or
combined (the default) via the \code{method} argument. You may also choose
whether to include the mean, per-hemisphere structural measure in the
models. Finally, you can specify variables that are present in \code{covars}
which you would like to exclude from the models. Optional arguments can be
provided that get passed to \code{\link{brainGraph_GLM_design}}.

If you do not explicitly specify the atlas name, then it will be guessed from
the size of your data. This could cause problems if you are using a custom
atlas, with or without the same number of regions as a dataset in the
package.
}
\note{
It is assumed that \code{dt.vol} was created using
\code{\link{import_scn}}. In older versions, there were issues when the Study
ID was specified as an integer and was not \dQuote{zero-padded}. This is done
automatically by \code{\link{import_scn}}, so if you are using an external
program, please be sure that the Study ID column is matched in both
\code{dt.vol} and \code{covars}.
}
\examples{
\dontrun{
myresids <- get.resids(lhrh, covars)
residPlots <- plot(myresids, cols=TRUE)

## Save as a multi-page PDF
ml <- marrangeGrob(residPlots, nrow=3, ncol=3)
ggsave('residuals.pdf', ml)
}
}
\seealso{
\code{\link[stats]{influence.measures}}, \code{\link[stats]{qqnorm}}

Other Structural covariance network functions: 
\code{\link{Bootstrapping}},
\code{\link{IndividualContributions}},
\code{\link{brainGraph_permute}()},
\code{\link{corr.matrix}()},
\code{\link{import_scn}()},
\code{\link{plot_volumetric}()}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}
\concept{Structural covariance network functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vulnerability.R
\name{vulnerability}
\alias{vulnerability}
\title{Calculate graph vulnerability}
\usage{
vulnerability(g, use.parallel = TRUE, weighted = FALSE)
}
\arguments{
\item{g}{An \code{igraph} graph object}

\item{use.parallel}{Logical indicating whether or not to use \emph{foreach}
(default: \code{TRUE})}

\item{weighted}{Logical indicating whether weighted efficiency should be
calculated (default: \code{FALSE})}
}
\value{
A numeric vector of length equal to the vertex count of \emph{g}
}
\description{
This function calculates the \emph{vulnerability} of the vertices of a graph.
Here, vulnerability is considered to be the proportional drop in global
efficiency when a given vertex is removed from the graph. The vulnerability
of the graph is considered the maximum across all vertices.
}
\references{
Latora, V. and Marchiori, M. (2005) Variability and protection of
  infrastructure networks. \emph{Physical Review E}, \bold{71}, 015103.
  \doi{10.1103/physreve.71.015103}
}
\seealso{
\code{\link{efficiency}}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distances.R
\name{mean_distance_wt}
\alias{mean_distance_wt}
\title{Calculate weighted shortest path lengths}
\usage{
mean_distance_wt(
  g,
  level = c("graph", "vertex"),
  weights = NULL,
  xfm = FALSE,
  xfm.type = NULL,
  D = NULL
)
}
\arguments{
\item{g}{An \code{igraph} graph object}

\item{level}{Character string indicating whether to calculate vertex- or
graph-level shortest path length. Default: \code{'graph'}}

\item{weights}{Numeric vector of edge weights; if \code{NULL} (the default),
and if the graph has edge attribute \code{weight}, then that will be used.
To avoid using weights, this should be \code{NA}.}

\item{xfm}{Logical indicating whether to transform the edge weights. Default:
\code{FALSE}}

\item{xfm.type}{Character string specifying how to transform the weights.
Default: \code{1/w}}

\item{D}{Numeric matrix; the graph's \dQuote{distance matrix}}
}
\value{
Numeric vector (if \code{level='vertex'}) of each vertex's shortest
  path length, or a single number for the graph-level average
}
\description{
Calculate graph or vertex average shortest path lengths. For vertices, this
is just the row means of the distance matrix. For the graph-level, it is the
overall mean of the distance matrix.
}
\details{
By default, edge weights are not transformed (e.g., inverted). However, if
set to \code{TRUE}, then the input graph must have a graph-level attribute
called \code{'xfm.type'} or you must supply a value in the function call. If
you supply a distance matrix (the \code{D} argument), it is not necessary to
transform edge weights, as it is assumed the the distance matrix was
calculated from a graph with transformed edge weights already.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{subset_graph}
\alias{subset_graph}
\title{Subset graphs based on a given logical condition}
\usage{
subset_graph(g, condition)
}
\arguments{
\item{g}{A graph object}

\item{condition}{Character string specifying an equation for which vertices to
keep}
}
\value{
A graph object
}
\description{
\code{subset_graph} will subset a given graph based on the given logical
condition(s). This can be a \dQuote{simple} logical equation, or can include
combinations of \emph{AND} and \emph{OR} (i.e., \code{&} and \code{|}).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rich_club.R
\name{rich_club_attrs}
\alias{rich_club_attrs}
\title{Assign graph attributes based on rich-club analysis}
\usage{
rich_club_attrs(g, deg.range = NULL, adj.vsize = FALSE)
}
\arguments{
\item{g}{An \code{igraph} graph object}

\item{deg.range}{Numeric vector of the range of degrees indicating
inclusion in the rich-club; if the default \emph{NULL}, it will be from 1
to the maximum degree in the graph}

\item{adj.vsize}{Logical indicating whether to adjust vertex size
proportional to degree. Default: \code{FALSE}}
}
\value{
An \code{igraph} graph object with additional attributes:
  \item{rich}{Binary indicating membership in the rich-club}
  \item{type.rich}{Edge attribute indicating the type of connection}
  \item{color.rich}{Edge and vertex attributes}
  \item{size.rich}{Edge and vertex attributes}
}
\description{
Assigns vertex- and edge-level attributes based on the results of a
\emph{rich-club} analysis, based on a range of vertex degrees in which the
rich-club coefficient was determined to be significantly greater than that of
a set of random graphs (see \code{\link{rich_club_norm}}).
}
\details{
Vertices which are in the rich club will be assigned an attribute
\code{rich}, taking on a binary value. Their colors (attribute
\code{color.rich}) will be either \emph{red} or \emph{gray}. Their sizes
(attribute \code{size.rich}) will either be 10 or will be proportional to
their degree.

Edge attribute \code{type.rich} takes on three values: \emph{rich-club} (if
it connects two rich-club vertices), \emph{feeder} (if it connects a rich- to
a non-rich-club vertex), and \emph{local} (if it connects two non-rich-club
vertices). The \code{color.rich} attribute is \emph{red}, \emph{orange}, or
\emph{green}. Edge sizes (\code{size.rich}) will be largest for
\emph{rich-club} connections, then smaller for \emph{feeder}, and smallest
for \emph{local}.
}
\examples{
\dontrun{
g <- rich_club_attrs(g, rich.dt[density == densities[N] & p.fdr < .01,
                                range(k)])
}
}
\seealso{
Other Rich-club functions: 
\code{\link{Rich Club}},
\code{\link{plot_rich_norm}()}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}
\concept{Rich-club functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glm_randomise.R
\name{randomise}
\alias{randomise}
\alias{partition}
\alias{randomise_3d}
\title{GLM non-parametric permutation testing}
\usage{
partition(M, contrast, part.method = c("beckmann", "guttman", "ridgway"))

randomise(
  perm.method,
  part.method,
  N,
  perms,
  X,
  y,
  contrasts,
  ctype,
  nC,
  skip = NULL,
  n = dim(X)[1L],
  p = qr.default(X)$rank,
  ny = dim(y)[2L],
  dfR = n - p
)

randomise_3d(
  perm.method,
  part.method,
  N,
  perms,
  X,
  y,
  contrasts,
  ctype,
  nC,
  runX = dimnames(X)[[3L]],
  n = dim(X)[1L],
  p = qr.default(X[, , 1L])$rank,
  ny = length(runX),
  dfR = n - p
)
}
\arguments{
\item{M}{Numeric matrix or array of the full design matrix(es)}

\item{contrast}{For \code{partition}, a numeric matrix with 1 or more rows
(for T and F contrasts, respectively) representing a \emph{single
contrast}.}

\item{part.method}{Character string; the method of partitioning the design
matrix into covariates of interest and nuisance. Default: \code{'beckmann'}}

\item{perm.method}{Character string indicating the permutation method.
Default: \code{'freedmanLane'}}

\item{N}{Integer; number of permutations to create. Default: \code{5e3}}

\item{perms}{Matrix of permutations, if you would like to provide your own.
Default: \code{NULL}}

\item{X}{Numeric matrix or 3D array of the design matrix(es)}

\item{y}{Numeric matrix of outcome variables, with 1 column per region, or a
single column if there is a different design matrix per region}

\item{contrasts}{Numeric matrix (for T statistics) or list of matrices (for F
statistics) specifying the contrast(s) of interest; if only one contrast is
desired, you can supply a vector (for T statistics)}

\item{ctype}{The contrast type}

\item{nC}{Integer; the number of contrasts}

\item{skip}{Integer vector indicating which (if any) contrasts to skip. Only
used by \code{\link{NBS}}.}

\item{n, p, ny, dfR}{Integers for the number of observations, design matrix
columns (its rank), number of regions/outcome variables, and residual
degrees of freedom, respectively}

\item{runX}{Character vector of regions with non-singular designs}
}
\value{
\code{partition} returns a list containing:
  \item{Mp}{Numeric array; the combined partitioned arrays}
  \item{X}{Numeric array for the covariates of interest}
  \item{Z}{Numeric array for the nuisance covariates}
  \item{eCm}{The \emph{effective contrast}, equivalent to the original, for
    the partitioned model \code{[X, Z]} and considering all covariates}
  \item{eCx}{Same as \code{eCm}, but considering only \code{X}}

A numeric array with dimensions \eqn{n_y \times N \times n_c};
  the number of rows equals number of regions/outcome variables, number of
  columns equals \code{N}, and the 3rd dimension is the number of contrasts
}
\description{
\code{randomise} and \code{randomise_3d} perform non-parametric permutation
testing for analyses in which there is a single or multiple design matrix per
region, respectively. In the latter case, \code{X} should be a 3D array.

\code{partition} partitions a full design matrix into separate matrices of
covariates of interest and nuisance covariates based on a given contrast and
partition method.
}
\section{Model partitioning}{

Consider the matrix formulation of the \emph{general linear model}:
\deqn{\mathbf{Y} = \mathbf{M} \psi + \in}
where \eqn{Y} is the vector of outcomes, \eqn{M} is the full design matrix
(including nuisance covariates), \eqn{\psi} is the vector of parameter
estimates, and \eqn{\in} is the vector of error terms. In a permutation
framework, algorithms are applied differently depending on the
presence/absence of nuisance covariates; thus the model is separated
depending on the contrast of interest:
\deqn{\mathbf{Y} = \mathbf{X}\beta + \mathbf{Z}\gamma + \in}
where \eqn{\mathbf{X}} contains covariates of interest, \eqn{\mathbf{Z}}
contains nuisance covariates, and \eqn{\beta} and \eqn{\gamma} are the
associated parameter estimates.

The manner of partitioning depends on the method. For example, for the
\code{guttman} method, \code{X} is formed from the columns of \code{contrast}
that have non-zero entries.
}

\section{Permutation methods}{

The permutation methods can be split into 2 groups, depending on which part
of the model they permute. For full details, see \emph{Winkler et al., 2014}.
\describe{
  \item{Permute Y}{Freedman-Lane, Manly, and ter Braak}
  \item{Permute X}{Smith, Draper-Stoneman, and Still-White}
}

Depending on the size of the data, it may be faster to use a method that
permutes \code{Y} instead of \code{X}. For example, in \code{\link{NBS}} with
dense matrices (more than 400-500 edges), it will be somewhat faster to use
the \dQuote{Smith} method compared to \dQuote{Freedman-Lane}. If using
\code{\link{brainGraph_GLM}}, the number of vertices follows the same
relationship.

Furthermore, all methods except Still-White include the \code{Z} (nuisance
covariate) matrix when calculating the permuted statistics.
}

\references{
Beckmann, C.F. and Jenkinson, M. and Smith, S.M. (2001) General
  multi-level linear modelling for group analysis in FMRI. Tech Rep.
  University of Oxford, Oxford.

Guttman, I. (1982) \emph{Linear Models: An Introduction}. Wiley,
  New York.

Ridgway, G.R. (2009) Statistical analysis for longitudinal MR
  imaging of dementia. PhD thesis.

Draper, N.R. and Stoneman, D.M. (1966) Testing for the inclusion
  of variables in linear regression by a randomisation technique.
  \emph{Technometrics}. \bold{8(4)}, 695--699.

Freedman, D. and Lane, D. (1983) A nonstochastic interpretation
  of reported significance levels. \emph{J Bus Econ Stat}, \bold{1(4)},
  292--298. \doi{10.1080/07350015.1983.10509354}

Manly B.F.J. (1986) Randomization and regression methods for
  testing for associations with geographical, environmental, and biological
  distances between populations. \emph{Res Popul Ecol}. \bold{28(2)},
  201--218.

Nichols, T.E. and Holmes, A.P. (2001) Nonparametric permutation
  tests for functional neuroimaging: A primer with examples. \emph{Human
  Brain Mapping}. \bold{15(1)}, 1--25.
  \doi{10.1002/hbm.1058}

Smith, S.M. and Jenkinson, M. and Beckmann, C. and Miller, K. and
  Woolrich, M. (2007) Meaningful design and contrast estimability in fMRI.
  \emph{NeuroImage}. \bold{34(1)}, 127--36.
  \doi{10.1016/j.neuroimage.2006.09.019}

Still, A.W. and White, A.P. (1981) The approximate randomization
  test as an alternative to the F test in analysis of variance. \emph{Br J
  Math Stat Psychol}. \bold{34(2)}, 243--252.

ter Braak, C.J.F. 1992. Permutation versus bootstrap significance
  tests in multiple regression and ANOVA. \emph{Bootstrapping and related
  techniques}. Springer, Berlin, Heidelberg. 79--85.

Winkler, A.M. and Ridgway, G.R. and Webster, M.A. and Smith, S.M.
  and Nichols, T.E. (2014) Permutation inference for the general linear
  model. \emph{NeuroImage}. \bold{92}, 381--397.
  \doi{10.1016/j.neuroimage.2014.01.060}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/NBS.R, R/methods.R
\name{NBS}
\alias{NBS}
\alias{summary.NBS}
\alias{nobs.NBS}
\alias{terms.NBS}
\alias{formula.NBS}
\alias{labels.NBS}
\alias{case.names.NBS}
\alias{variable.names.NBS}
\alias{df.residual.NBS}
\alias{nregions.NBS}
\title{Network-based statistic for brain MRI data}
\usage{
NBS(
  A,
  covars,
  contrasts,
  con.type = c("t", "f"),
  X = NULL,
  con.name = NULL,
  p.init = 0.001,
  perm.method = c("freedmanLane", "terBraak", "smith", "draperStoneman", "manly",
    "stillWhite"),
  part.method = c("beckmann", "guttman", "ridgway"),
  N = 1000,
  perms = NULL,
  symm.by = c("max", "min", "avg"),
  alternative = c("two.sided", "less", "greater"),
  long = FALSE,
  ...
)

\method{summary}{NBS}(
  object,
  contrast = NULL,
  digits = max(3L, getOption("digits") - 2L),
  ...
)

\method{nobs}{NBS}(object, ...)

\method{terms}{NBS}(x, ...)

\method{formula}{NBS}(x, ...)

\method{labels}{NBS}(object, ...)

\method{case.names}{NBS}(object, ...)

\method{variable.names}{NBS}(object, ...)

\method{df.residual}{NBS}(object, ...)

\method{nregions}{NBS}(object)
}
\arguments{
\item{A}{Three-dimensional array of all subjects' connectivity matrices}

\item{covars}{A \code{data.table} of covariates}

\item{contrasts}{Numeric matrix (for T statistics) or list of matrices (for F
statistics) specifying the contrast(s) of interest; if only one contrast is
desired, you can supply a vector (for T statistics)}

\item{con.type}{Character string; either \code{'t'} or \code{'f'} (for t or
F-statistics). Default: \code{'t'}}

\item{X}{Numeric matrix, if you wish to supply your own design matrix.
Ignored if \code{outcome != measure}.}

\item{con.name}{Character vector of the contrast name(s); if \code{contrasts}
has row/list names, those will be used for reporting results}

\item{p.init}{Numeric; the initial p-value threshold (default: \code{0.001})}

\item{perm.method}{Character string indicating the permutation method.
Default: \code{'freedmanLane'}}

\item{part.method}{Character string; the method of partitioning the design
matrix into covariates of interest and nuisance. Default: \code{'beckmann'}}

\item{N}{Integer; number of permutations to create. Default: \code{5e3}}

\item{perms}{Matrix of permutations, if you would like to provide your own.
Default: \code{NULL}}

\item{symm.by}{Character string; how to create symmetric off-diagonal
elements. Default: \code{max}}

\item{alternative}{Character string, whether to do a two- or one-sided test.
Default: \code{'two.sided'}}

\item{long}{Logical indicating whether or not to return all permutation
results. Default: \code{FALSE}}

\item{...}{Arguments passed to \code{\link{brainGraph_GLM_design}}}

\item{object, x}{A \code{NBS} object}

\item{contrast}{Integer specifying the contrast to plot/summarize; defaults
to showing results for all contrasts}

\item{digits}{Integer specifying the number of digits to display for P-values}
}
\value{
An object of class \code{NBS} with some input arguments in addition
  to:
  \item{X}{The design matrix}
  \item{removed.subs}{Character vector of subject ID's removed due to
    incomplete data (if any)}
  \item{T.mat}{3-d array of (symmetric) numeric matrices containing the
    statistics for each edge}
  \item{p.mat}{3-d array of (symmetric) numeric matrices containing the
    P-values}
  \item{components}{List containing data tables of the observed and permuted
    connected component sizes and P-values}
  \item{rank,df.residual,qr,cov.unscaled}{The rank, residual degrees of
    freedom, QR decomposition, and unscaled covariance matrix of the design
    matrix}
}
\description{
Calculates the \emph{network-based statistic (NBS)}, which allows for
family-wise error (FWE) control over network data, introduced for brain MRI
data by Zalesky et al. Requires a three-dimensional array of all subjects'
connectivity matrices and a \code{data.table} of covariates, in addition to a
contrast matrix or list. A null distribution of the largest connected
component size is created by fitting a GLM to permuted data. For details, see
\code{\link{GLM}}.
}
\details{
When printing a \code{summary}, you can include arguments to
\code{\link[stats]{printCoefmat}}.
}
\note{
It is assumed that the order of the subjects in \code{covars} matches
that of the input array \code{A}. You will need to ensure that this is the
case. Prior to \code{v3.0.0}, the \code{covars} table was sorted by
\code{Study.ID} before creating the design matrix.
}
\examples{
\dontrun{
max.comp.nbs <- NBS(A.norm.sub[[1]], covars.dti, N=5e3)
}
}
\references{
Zalesky, A. and Fornito,  A. and Bullmore, E.T. (2010)
  Network-based statistic: identifying differences in brain networks.
  \emph{NeuroImage}, \bold{53(4)}, 1197--1207.
  \doi{10.1016/j.neuroimage.2010.06.041}
}
\seealso{
Other Group analysis functions: 
\code{\link{Bootstrapping}},
\code{\link{GLM}},
\code{\link{Mediation}},
\code{\link{brainGraph_permute}()},
\code{\link{mtpc}()}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}
\concept{Group analysis functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create_graphs.R
\name{make_intersection_brainGraph}
\alias{make_intersection_brainGraph}
\title{Create the intersection of graphs based on a logical condition}
\usage{
make_intersection_brainGraph(..., subgraph, keep.all.vertices = FALSE)
}
\arguments{
\item{...}{Graph objects or lists of graph objects}

\item{subgraph}{Character string specifying an equation (logical condition)
for the vertices to subset}

\item{keep.all.vertices}{Logical indicating whether to keep all vertices that
meet the criteria in at least 1 input graph. Default: \code{FALSE}}
}
\value{
An \code{igraph} graph object
}
\description{
Returns a graph object with vertices that meet certain criteria. By default,
only vertices that meet these criteria for \emph{all} input graphs will be
retained.
}
\details{
If no vertices meet criteria for all input graphs, then an \code{igraph}
graph object with 0 vertices is returned. If \code{keep.all.vertices=TRUE},
this is essentially performing a \emph{union} of vertex sets that meet the
criteria. In any case, the return graph will have 0 edges.
}
\examples{
\dontrun{
res.mtpc <- mtpc(g, covars, ...)
g.mtpc <- make_glm_brainGraph(res.mtpc, atlas)

## All vertices with a significant MTPC result for all contrasts:
g.mtpc.int <- make_intersection_brainGraph(g.mtpc, subgraph='sig == 1')

## Return graphs with vertices with degree > 0 for each group separately
tapply(g.list, groups(g.list), make_intersection_brainGraph,
       subgraph='degree > 0')
}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_global.R
\name{plot_global}
\alias{plot_global}
\title{Plot global graph measures across densities}
\usage{
plot_global(
  g.list,
  xvar = c("density", "threshold"),
  vline = NULL,
  level.names = "default",
  exclude = NULL,
  perms = NULL,
  alt = "two.sided"
)
}
\arguments{
\item{g.list}{List of \code{brainGraphList} objects; the length of this list
should equal the number of thresholds/densities in the study}

\item{xvar}{A character string indicating whether the variable of
interest is \dQuote{density} or \dQuote{threshold} (e.g. with DTI data)}

\item{vline}{Numeric of length 1 specifying the x-intercept if you would like
to plot a vertical dashed line (e.g., if there is a particular density of
interest). Default: \code{NULL}}

\item{level.names}{Character vector of variable names, which are displayed as
facet labels. If you do not want to change them, specify \code{NULL}. By
default, they are changed to pre-set values.}

\item{exclude}{Character vector of variables to exclude. Default: \code{NULL}}

\item{perms}{A \code{\link[data.table]{data.table}} of permutation group differences}

\item{alt}{Character vector of alternative hypotheses; required if
\emph{perms} is provided, but defaults to \dQuote{two.sided} for all
variables}
}
\value{
Either a \code{trellis} or \code{ggplot} object
}
\description{
Create a faceted line plot of global graph measures across a range of graph
densities, calculated from a list of \code{brainGraphList} objects. This
requires that the variables of interest are graph-level attributes of the
input graphs.
}
\details{
You can choose to insert a dashed vertical line at a specific
density/threshold of interest, rename the variable levels (which become the
facet titles), exclude variables, and include a \code{brainGraph_permute}
object of permutation data to add asterisks indicating significant group
differences.
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_matrix.R
\name{Matrix utilities}
\alias{Matrix utilities}
\alias{colMax}
\alias{colMaxAbs}
\alias{colMin}
\alias{diag_sq}
\alias{get_thresholds}
\alias{is_binary}
\alias{qr.array}
\alias{qr_Q2}
\alias{qr_R2}
\alias{symm_mean}
\alias{symmetrize}
\alias{symmetrize.matrix}
\alias{symmetrize.array}
\title{Matrix/array utility functions}
\usage{
colMax(x, n = dim(x)[1L])

colMaxAbs(x, n = dim(x)[1L])

colMin(x, n = dim(x)[1L])

diag_sq(x, n = dim(x)[1L], inds = 1L + 0L:(n - 1L) * (n + 1L))

get_thresholds(x, densities, emax = dim(x)[1L] * (dim(x)[1L] - 1L)/2, ...)

is_binary(x)

\method{qr}{array}(x, ...)

qr_Q2(QR, y = diag(1, n, p), n = dim(QR$qr)[1L], p = QR$rank)

qr_R2(QR, p = QR$rank)

symm_mean(x)

symmetrize(x, ...)

\method{symmetrize}{matrix}(x, symm.by = c("max", "min", "avg"), ...)

\method{symmetrize}{array}(x, symm.by = c("max", "min", "avg"), ...)
}
\arguments{
\item{x}{Numeric matrix or array (the latter, for \code{qr.array} and
\code{symmetrize.array})}

\item{n, p}{Integer; the number of rows or rank (respectively) of
the input matrix or QR decomposition}

\item{inds}{Vector-based indices of the diagonal}

\item{densities}{Numeric vector of densities}

\item{emax}{Integer; the maximum number of edges}

\item{...}{Arguments passed to either \code{\link{sort}} (for
\code{get_thresholds}) or \code{\link{qr.default}} (for \code{qr.array}).
For the former, this will typically only be \code{decreasing=TRUE}, if that
is the desired behavior}

\item{QR}{A \code{qr} object}

\item{y}{A numeric matrix with \code{1} along the diagonal, of the same size
as the input matrix (i.e., \code{QR$qr})}

\item{symm.by}{Character string; how to create symmetric off-diagonal
elements. Default: \code{max}}
}
\value{
\code{diag_sq} returns an unnamed numeric vector with the values
  along the diagonal of the input matrix

\code{get_thresholds} returns a numeric vector of the thresholds

\code{is_binary} returns a logical of length 1

\code{qr.array} returns a \emph{list} in which each element is the QR
  decomposition of each matrix along \code{x}'s 3rd dimension
}
\description{
These functions are utility/helper functions when working with matrices or
arrays.

\code{diag_sq} is a pared-down version of \code{\link{diag}} for square
matrices. It does not return any dimnames, does not check if \code{x} is a
square matrix, and it cannot be used to \emph{create} a matrix with a given
value along the diagonal. Meant to be used in code that is called repeatedly
(thousands of times).

\code{get_thresholds} calculates the threshold values that would result in a
specific graph density. These depend, necessarily on the values in the matrix
themselves.

\code{qr.array} will calculate the QR decomposition for each matrix in a 3D
array.

\code{qr_Q2} and \code{qr_R2} are simplified versions of \code{\link{qr.Q}}
and \code{\link{qr.R}}.

\code{symm_mean} returns a symmetric matrix in which the off-diagonal
elements \eqn{A[i, j]} and \eqn{A[j, i]} are set to the mean of the values
in the input matrix.

\code{symmetrize} will symmetrize a numeric matrix (or each matrix in an
array) by assigning to the off-diagonal elements either the \code{max}
(default), \code{min}, or \code{average} of \eqn{\{A(i, j), A(j, i)\}}.
}
\details{
Given a vector of densities, \code{get_thresholds} returns the numeric values
that will result in graphs of the given densities after thresholding by those
values. In the \emph{Examples} section, the thresholds should result in
graphs with densities of \eqn{5, 15, \dots, 55} percent.
}
\examples{
x <- matrix(runif(25 * 25), 25, 25)
x <- symmetrize(x)
diag(x) <- 0
densities <- seq(0.05, 0.55, by=0.1)
threshes <- get_thresholds(x, densities)
## Verify that the densities are correct
graphs <- lapply(threshes, function(th) {
  graph_from_adjacency_matrix(x * (x > th), mode='undirected',
                              diag=FALSE, weighted=TRUE)
  })
sapply(graphs, graph.density)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_brainGraph.R
\name{plot.brainGraph}
\alias{plot.brainGraph}
\title{Plot a brain graph with a specific spatial layout}
\usage{
\method{plot}{brainGraph}(
  x,
  plane = c("axial", "sagittal", "circular"),
  hemi = c("both", "L", "R"),
  mni = TRUE,
  subgraph = NULL,
  main = NULL,
  subtitle = "default",
  label = NULL,
  side = 1,
  line = -2,
  adj = 0.025,
  cex = 2.5,
  col = "white",
  font = 2,
  show.legend = FALSE,
  rescale = FALSE,
  asp = 0,
  ...
)
}
\arguments{
\item{x}{A \code{brainGraph} graph object}

\item{plane}{Character string indicating which orientation to plot.
Default: \code{'axial'}}

\item{hemi}{Character string indicating which hemisphere to plot. Default:
\code{'both'}}

\item{mni}{Logical indicating whether or not to plot over a slice of the
brain. Default: \code{TRUE}}

\item{subgraph}{Character string specifying a logical condition for vertices
to plot. Default: \code{NULL}}

\item{main}{Character string; the main title. Default: \code{NULL}}

\item{subtitle}{Character string; the subtitle. Default: \code{'default'}}

\item{label}{Character string specifying text to display in one corner of the
plot (e.g., \code{'A.'}). Default: NULL}

\item{side}{Label placement. Default: \code{1} (bottom)}

\item{line}{Which margin line to place the text.}

\item{adj}{If \code{side=1}, a value closer to 0 places the text closer to
the left margin. Default: \code{0.025}}

\item{cex}{Amount of character expansion of the label text. Default:
\code{2.5}}

\item{col}{Label font color. Default: \code{'white'}}

\item{font}{Integer specifying the font type. Default: \code{2} (bold face)}

\item{show.legend}{Logical indicating whether or not to show a legend.
Default: \code{FALSE}}

\item{rescale}{Logical, whether to rescale the coordinates. Default:
\code{FALSE}}

\item{asp}{Numeric constant; the aspect ratio. Default: 0}

\item{...}{Other parameters (passed to \code{\link[igraph]{plot.igraph}}).
See \code{\link[igraph]{plot.common}} for details.}
}
\description{
\code{plot.brainGraph} plots a graph in which the spatial layout of the nodes
is important. The network itself is plotted over a brain MRI slice from the
MNI152 template by default (when \code{mni=TRUE}).
}
\section{Selecting specific vertices to display}{

With the argument \code{subgraph}, you can supply a simple logical expression
specifying which vertices to show. For example, \emph{'degree > 10'} will
plot only vertices with a \emph{degree} greater than 10. Combinations of
\emph{AND} (i.e., \code{&}) and \emph{OR} (i.e., \code{|}) are allowed. This
requires that any vertex attribute in the expression must be present in the
graph; e.g., \code{V(g)$degree} must exist.
}

\section{Title, subtitle, and label}{

By default, a \emph{title} (i.e., text displayed at the top of the figure) is
not included. You can include one by passing a character string to
\code{main}, and control the size with \code{cex.main}. A \emph{subtitle}
(i.e., text at the bottom), is included by default and displays the number of
vertices and edges along with the graph density. To exclude this, specify
\code{subtitle=NULL}. A \dQuote{label} can be included in one corner of the
figure (for publications). For example, you can choose \code{label='A.'} or
\code{label='a)'}. Arguments controlling the location and appearance can be
changed, but the default values are optimal for bottom-left placement. See
\code{\link[graphics]{mtext}} for more details. The label-specific arguments
are:
\describe{
  \item{side}{The location. \code{1} is for bottom placement.}
  \item{line}{If \code{side=1} (bottom), a negative number places the text
    \emph{above} the bottom of the figure; a higher number could result in
    the bottom part of the text to be missing. This can differ if
    \code{plane='circular'}, in which case you may want to specify a positive
    number.}
  \item{adj}{Seems to be the percentage away from the margin. So, for
    example, \code{adj=0.1} would place the text closer to the center than
    the default value, and \code{adj=0.5} places it in the center.}
  \item{cex}{The degree of \dQuote{character expansion}. A value of 1 would
    not increase the text size.}
  \item{col}{The text color.}
  \item{font}{The font type. The default \code{font=2} is bold face. See
    \code{\link[graphics]{par}} for details.}
}
}

\examples{
\dontrun{
plot(g[[1]], hemi='R')
plot(g[[1]], subgraph='degree > 10 | btwn.cent > 50')

## Place label in upper-left
plot(g.ex, label='A)', side=3, line=-2.5)
}
}
\seealso{
Other Plotting functions: 
\code{\link{Plotting GLM graphs}},
\code{\link{plot.brainGraphList}()},
\code{\link{plot_brainGraph_multi}()}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}
\concept{Plotting functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/auc.R
\name{auc_diff}
\alias{auc_diff}
\title{Difference in the area-under-the-curve of two vectors}
\usage{
auc_diff(x, y)
}
\arguments{
\item{x}{Numeric vector of the x-values}

\item{y}{A numeric vector or matrix}
}
\value{
A numeric value of the difference between two groups, or a numeric
  vector of the AUC across vertices
}
\description{
This function takes two vectors, calculates the area-under-the-curve (AUC),
and calculates the difference between the two (if applicable).
}
\details{
There are 4 different behaviors for this function:
\enumerate{
  \item If \code{x} is a single numeric value, then \code{y} should be a
    vector of 2 values and the difference is returned. This generally should
    not occur and may be removed in the future.
  \item If \code{y} has 1 column (or is a vector), then the AUC of \code{y}
    is returned.
  \item If \code{y} has exactly 2 columns, then each column should contain
    the values of interest for each subject group, and the difference in
    AUC's for each group is returned.
  \item If \code{y} has multiple columns (e.g., equal to the number of
    vertices of a graph), it will calculate the AUC for each column.
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/list.R
\name{plot.brainGraphList}
\alias{plot.brainGraphList}
\title{Plot a brainGraphList and write to PDF}
\usage{
\method{plot}{brainGraphList}(x, plane, hemi, filename.base, diffs = FALSE, ...)
}
\arguments{
\item{x}{A \code{brainGraphList} object}

\item{plane}{Character string indicating which orientation to plot.
Default: \code{'axial'}}

\item{hemi}{Character string indicating which hemisphere to plot. Default:
\code{'both'}}

\item{filename.base}{Character string specifying the base of the filename}

\item{diffs}{Logical, indicating whether edge differences should be
highlighted. Default: \code{FALSE}}

\item{...}{Other parameters (passed to \code{\link{plot.brainGraph}})}
}
\description{
The \code{plot} method will write a PDF file containing plots for all graphs
in the given object.
}
\details{
You can choose to highlight edge differences between subsequent list
elements; in this case, new/different edges are colored pink. This is useful
mostly for a list of group-level graphs.
}
\seealso{
Other Plotting functions: 
\code{\link{Plotting GLM graphs}},
\code{\link{plot.brainGraph}()},
\code{\link{plot_brainGraph_multi}()}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}
\concept{Plotting functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/list.R
\name{Creating_Graphs_GLM}
\alias{Creating_Graphs_GLM}
\alias{make_brainGraphList.bg_GLM}
\alias{make_brainGraphList.mtpc}
\alias{make_brainGraphList.NBS}
\title{Create a graph list with GLM-specific attributes}
\usage{
\method{make_brainGraphList}{bg_GLM}(
  x,
  atlas = x$atlas,
  type = "observed",
  level = "contrast",
  set.attrs = FALSE,
  modality = NULL,
  weighting = NULL,
  threshold = NULL,
  gnames = x$con.name,
  ...
)

\method{make_brainGraphList}{mtpc}(
  x,
  atlas = x$atlas,
  type = "observed",
  level = "contrast",
  set.attrs = FALSE,
  modality = NULL,
  weighting = NULL,
  threshold = NULL,
  gnames = x$con.name,
  ...
)

\method{make_brainGraphList}{NBS}(
  x,
  atlas,
  type = "observed",
  level = "contrast",
  set.attrs = TRUE,
  modality = NULL,
  weighting = NULL,
  threshold = NULL,
  gnames = x$con.name,
  mode = "undirected",
  weighted = TRUE,
  diag = FALSE,
  ...
)
}
\arguments{
\item{x}{A \code{bg_GLM}, \code{mtpc}, or \code{NBS} object}

\item{atlas}{Character string specifying the brain atlas to use}

\item{type}{Character string indicating the type of graphs. Default:
\code{observed}}

\item{level}{Character string indicating whether the graphs are subject-,
group-, or contrast-specific. Default: \code{'subject'}}

\item{set.attrs}{Logical indicating whether to assign all graph-, vertex-,
and edge-level attributes (via \code{\link{set_brainGraph_attr}}). Default:
\code{TRUE}}

\item{modality}{Character string indicating imaging modality (e.g. 'dti').
Default: \code{NULL}}

\item{weighting}{Character string indicating how the edges are weighted
(e.g., 'fa', 'pearson', etc.). Default: \code{NULL}}

\item{threshold}{Integer or number indicating the threshold used when
\dQuote{sparsifying} the connectivity matrix (if any). Default: \code{NULL}}

\item{gnames}{Character vector of graph names (e.g., study IDs if
\code{level='subject'}). Default: \code{NULL}}

\item{...}{Other arguments passed to \code{\link{set_brainGraph_attr}}}

\item{mode}{Character string defining how the matrix should be interpreted.
Default: \code{'undirected'}}

\item{weighted}{Logical specifying whether to create a weighted network}

\item{diag}{Logical indicating whether to include the diagonal of the
connectivity matrix. Default: \code{FALSE}}
}
\value{
A \code{brainGraphList} object, with a graph object for each contrast
  with additional attributes:
  \item{Graph}{\emph{name} (contrast name), \emph{outcome} (the outcome
    variable), \emph{alpha} (the significance level); for MTPC:
    \emph{tau.mtpc}, \emph{S.mtpc}, \emph{S.crit}, \emph{A.crit}}
  \item{Vertex}{\emph{size2} (t-statistic); \emph{size} (the t-stat
    transformed for visualization purposes); \emph{p} (equal to \eqn{1-p});
    \emph{p.fdr} (equal to \eqn{1-p_{FDR}}, the FDR-adjusted p-value);
    \emph{effect.size} (the contrast of parameter estimates for t-contrasts;
    the extra sum of squares for F-contrasts); \emph{se} (the
    standard error of \emph{gamma}); \emph{A.mtpc}, \emph{sig} (binary
    indicating whether \code{A.mtpc > A.crit}) (for MTPC)}

\code{make_brainGraphList.NBS} returns graphs with additional
  attributes:
  \item{Vertex}{\emph{comp} (integer vector indicating connected component
    membership), \emph{p.nbs} (P-value for each component)}
  \item{Edge}{\emph{stat} (the test statistic for each connection), \emph{p}
    (the P-value)}
}
\description{
These methods create a \code{brainGraphList} with attributes specific to the
results of \code{\link{brainGraph_GLM}}, \code{\link{mtpc}}, or
\code{\link{NBS}}. The \code{graphs} element of the returned object will
contain one graph for each contrast.
}
\note{
Only valid for \emph{vertex}-level and \emph{NBS} analyses.
}
\seealso{
\code{\link{brainGraph_GLM}, \link{mtpc}, \link{NBS}}

Other Graph creation functions: 
\code{\link{Creating_Graphs}},
\code{\link{brainGraphList}},
\code{\link{make_ego_brainGraph}()}
}
\concept{Graph creation functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/count_edges.R
\name{Count Edges}
\alias{Count Edges}
\alias{count_homologous}
\alias{count_inter}
\title{Count number of edges of a brain graph}
\usage{
count_homologous(g)

count_inter(
  g,
  group = c("lobe", "hemi", "network", "class", "gyrus", "Yeo_7network", "Yeo_17network",
    "area", "Brodmann")
)
}
\arguments{
\item{g}{A \code{brainGraph} graph object}

\item{group}{Character string specifying which grouping to calculate edge
counts for. Default: \code{'lobe'}}
}
\value{
\code{count_homologous} - a named vector of the edge ID's connecting
  homologous regions

\code{count_inter} - a \code{data.table} of total, intra-, and
  inter-group edge counts
}
\description{
\code{count_homologous} counts the number of edges between homologous regions
in a brain graph (e.g. between L and R superior frontal).

\code{count_inter} counts the number of edges between and within all vertices
in one group (e.g. \emph{lobe}, \emph{hemi}, \emph{network}, etc.).
}
\examples{
\dontrun{
g1.lobecounts <- count_inter(g[[1]][[N]], 'lobe')
}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{coeff_var}
\alias{coeff_var}
\alias{coeff_var.default}
\title{Calculate coefficient of variation}
\usage{
coeff_var(x, na.rm = FALSE, ...)

\method{coeff_var}{default}(x, na.rm = FALSE, ...)
}
\arguments{
\item{x}{Numeric vector, matrix, or array}

\item{na.rm}{Logical indicating whether \code{NA} values should be stripped
when calculating sums. Default: \code{FALSE}}

\item{...}{Unused}
}
\value{
A numeric vector or matrix
}
\description{
\code{coeff_var} is a S3 generic that calculates the \emph{coefficient of
variation}, defined as
\deqn{CV(x) = \frac{sd(x)}{mean(x)}}
}
\details{
If \code{x} is a matrix, it will calculate the CV for each \emph{column}. If
\code{x} is a 3D array, it will calculate the coefficient of variation for
each \emph{row-column} combination. If the input dimensions are \eqn{n \times
n \times r}, a matrix with size \eqn{n \times n} will be returned.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/atlas.R
\name{Atlas Helpers}
\alias{Atlas Helpers}
\alias{guess_atlas}
\alias{as_atlas}
\alias{create_atlas}
\title{Atlas helper functions}
\usage{
guess_atlas(x)

as_atlas(object)

create_atlas(regions, coords, lobes, hemis, other = NULL)
}
\arguments{
\item{x, object}{An object to test or convert to an atlas data.table}

\item{regions}{Character vector of region names}

\item{coords}{Numeric matrix of spatial coordinates; must have 3 columns}

\item{lobes}{Character or factor vector of lobe membership}

\item{hemis}{Character or factor vector of hemisphere membership. There
should probably not be more than 3 unique elements (for left, right, and
bi-hemispheric regions)}

\item{other}{A \emph{named list} of vectors with other data. The names of the
list will become column names in the return object.}
}
\value{
\code{guess_atlas} - Character string; either the matched atlas or
  \code{NA}

\code{as_atlas} and \code{create_atlas} return a \code{data.table}
  that conforms to other atlases in the package, or exits with an error.
}
\description{
\code{guess_atlas} tries to determine which atlas is being used based on the
data; i.e., the number of vertices/regions.

\code{as_atlas} and \code{create_atlas} converts/coerces an object to a
a \code{data.table}, or creates one, that is compatible with
\code{brainGraph}.
}
\section{Guessing the atlas from an object}{

There are several valid inputs to \code{guess_atlas}:
\describe{
  \item{data.table}{The atlas will be guessed based on the number of columns
    (subtracting by 1 if a \dQuote{Study ID} column is present). This is the
    same behavior as for \code{data.frame} objects, as well.}
  \item{igraph}{The vertex count}
  \item{brainGraph}{If there is a \code{atlas} graph-level attribute, it will
    return that. Otherwise, the vertex count.}
  \item{matrix,array}{The number of rows, which should equal the number of
    columns if the input is a connectivity matrix.}
}

Note that this will only work properly for atlases that are currently in the
package. If you are using a custom atlas and you receive errors, please open
an issue on \emph{GitHub}.
}

\section{Coercing to an atlas}{

There are several things \code{as_atlas} tries to do to make it work without
error:
\itemize{
  \item Coerce the object to \code{data.table}
  \item Add a column of integers named \code{index}
  \item Change columns named \code{'x'}, \code{'y'}, or \code{'z'} to have
    \code{.mni} at the end
  \item Convert the \code{lobe} and \code{hemi} columns to be \emph{factors}
}
}

\examples{
my_atlas <- data.frame(name=paste('Region', 1:10), x.mni=rnorm(10),
  y.mni=rnorm(10), z.mni=rnorm(10),
  lobe=rep(c('Frontal', 'Parietal', 'Temporal', 'Occipital', 'Limbic'), 2),
  hemi=c(rep('L', 5), rep('R', 5)))
my_atlas2 <- as_atlas(my_atlas)
str(my_atlas)
str(my_atlas2)
regions <- paste('Region', 1:10)
xyz <- matrix(rnorm(30), nrow=10, ncol=3)
lobe <- rep(c('Frontal', 'Parietal', 'Temporal', 'Occipital', 'Limbic'), 2)
hemi <- c(rep('L', 5), rep('R', 5))
other <- list(network=rep(c('Default mode', 'Task positive'), 5))
my_atlas <- create_atlas(regions, xyz, lobe, hemi, other)
str(my_atlas)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/set_brainGraph_attributes.R
\name{Attributes}
\alias{Attributes}
\alias{set_brainGraph_attr}
\alias{xfm.weights}
\title{Set graph, vertex, and edge attributes common in MRI analyses}
\usage{
set_brainGraph_attr(
  g,
  type = c("observed", "random"),
  use.parallel = TRUE,
  A = NULL,
  xfm.type = c("1/w", "-log(w)", "1-w", "-log10(w/max(w))", "-log10(w/max(w)+1)"),
  clust.method = "louvain"
)

xfm.weights(
  g,
  xfm.type = c("1/w", "-log(w)", "1-w", "-log10(w/max(w))", "-log10(w/max(w)+1)"),
  invert = FALSE
)
}
\arguments{
\item{g}{A graph object}

\item{type}{Character string indicating the type of graphs. Default:
\code{observed}}

\item{use.parallel}{Logical indicating whether to use \emph{foreach}.
Default: \code{TRUE}}

\item{A}{Numeric matrix; the (weighted) adjacency matrix, which can be used
for faster calculation of local efficiency. Default: \code{NULL}}

\item{xfm.type}{Character string specifying how to transform the weights.
Default: \code{1/w}}

\item{clust.method}{Character string indicating which method to use for
community detection. Default: \code{'louvain'}}

\item{invert}{Logical indicating whether or not to invert the transformation.
Default: \code{FALSE}}
}
\value{
A graph object with the following attributes:
  \item{Graph-level}{Density, connected component sizes, diameter, # of
    triangles, transitivity, average path length, assortativity, global &
    local efficiency, modularity, vulnerability, hub score, rich-club
    coefficient, # of hubs, edge asymmetry}
  \item{Vertex-level}{Degree, strength; betweenness, eigenvector, and
    leverage centralities; hubs; transitivity (local); k-core, s-core; local
    & nodal efficiency; color (community, lobe, component); membership
    (community, lobe, component); gateway and participation coefficients,
    within-module degree z-score; vulnerability; and coordinates (x, y, and
    z)}
  \item{Edge-level}{Color (community, lobe, component), edge betweenness,
    Euclidean distance (in mm), weight (if weighted)}

\code{xfm.weights} returns the same graph object, with transformed
  edge weights plus a graph attribute (\code{xfm.type}) recording the method
  of transformation
}
\description{
\code{set_brainGraph_attr} is a convenience function that sets a number of
graph, vertex, and edge attributes for a given graph object. Specifically, it
calculates measures that are common in MRI analyses of brain networks.
}
\details{
Including \code{type='random'} in the function call will reduce the number of
attributes calculated. It will only add graph-level attributes for:
clustering coefficient, characteristic path length, rich club coefficient,
global efficiency, and modularity.
}
\section{Negative edge weights}{

If there are any negative edge weights in the graph, several of the
distance-based metrics will \emph{not} be calculated, because they can throw
errors which is undesirable when processing a large dataset. The metrics are:
local and nodal efficiency, diameter, characteristic path length, and
hubness.
}

\section{Transforming edge weights}{

For distance-based measures, it is important to transform the edge weights so
that the \emph{strongest} connections are re-mapped to having the
\emph{lowest} weights. Then you may calculate e.g., the \emph{shortest path
length} which will include the strongest connections.

\code{xfm.type} allows you to choose from 5 options for transforming edge
weights when calculating distance-based metrics (e.g., shortest paths). There
is no \dQuote{best-practice} for choosing one over the other, but the reciprocal is
probably most common.
\describe{
  \item{\code{1/w}}{reciprocal (default)}
  \item{\code{-log(w)}}{the negative (natural) logarithm}
  \item{\code{1-w}}{subtract weights from 1}
  \item{\code{-log10(w/max(w))}}{negative (base-10) log of normalized
  weights}
  \item{\code{-log10(w/max(w)+1)}}{same as above, but add 1 before taking
    the log}
}

To transform the weights back to original values, specify \code{invert=TRUE}.
}

\section{Community detection}{

\code{clust.method} allows you to choose from any of the clustering
(community detection) functions available in \code{igraph}. These functions
begin with \code{cluster_}; the function argument should not include this
leading character string. There are a few possibilities, depending on the
value and the type of input graph:
\enumerate{
  \item By default, \code{louvain} is used, calling
    \code{\link[igraph]{cluster_louvain}}
  \item Uses \code{spinglass} if there are any negative edges and/or the
    selected method is \code{spinglass}
  \item Uses \code{walktrap} if there are any negative edge weights and any
    other method (besides \code{spinglass}) is selected
  \item Automatically transforms the edge weights if \code{edge_betweenness}
    is selected and the graph is weighted, because the algorithm considers
    edges as \emph{distances}
}
}

\seealso{
\code{\link[igraph]{components}}, \code{\link[igraph]{diameter}},
\code{\link[igraph]{centr_betw}}, \code{\link[igraph]{betweenness}},
\code{\link[igraph]{centr_eigen}}, \code{\link[igraph]{transitivity}},
\code{\link[igraph]{distances}}, \code{\link[igraph]{assortativity}},
\code{\link[igraph]{coreness}}, \code{\link[igraph]{communities}},
\code{\link[igraph]{knn}}
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Brain Atlases}
\alias{Brain Atlases}
\alias{aal116}
\alias{aal90}
\alias{aal2.120}
\alias{aal2.94}
\alias{destrieux}
\alias{destrieux.scgm}
\alias{dk}
\alias{dk.scgm}
\alias{dkt}
\alias{dkt.scgm}
\alias{brainsuite}
\alias{craddock200}
\alias{dosenbach160}
\alias{hoa112}
\alias{lpba40}
\alias{hcp_mmp1.0}
\alias{power264}
\alias{brainnetome}
\alias{gordon333}
\title{Coordinates for data from brain atlases}
\format{
A data frame with 90 or 116 (for the original AAL atlases), 94 or 120
(for the newer AAL2 atlases), 148 or 162 (for Destrieux), 68 or 82 (for DK),
62 or 76 (for DKT), 74 (Brainsuite), 200 (Craddock), 160 (Dosenbach), 112
(Harvard-Oxford), 40 (LONI), 246 (Brainnetome), 360 (HCP), 264 (Power), or
333 (Gordon) observations on (some of) the following 19 variables:
\describe{
  \item{\code{name}}{a character vector of region names}
  \item{\code{x.mni}}{a numeric vector of x-coordinates (in MNI space)}
  \item{\code{y.mni}}{a numeric vector of y-coordinates (in MNI space)}
  \item{\code{z.mni}}{a numeric vector of z-coordinates (in MNI space)}
  \item{\code{lobe}}{a factor with some of levels \code{Frontal}
    \code{Parietal} \code{Temporal} \code{Occipital} \code{Insula}
    \code{Limbic} \code{Cingulate} \code{SCGM} \code{Cerebellum} (for
    \code{aal116} and \code{aal2.120}) and \code{Brainstem} (for
    \code{craddock200})}
  \item{\code{hemi}}{a factor with levels \code{L} \code{R} and \code{B} (for
    \code{dosenbach160})}
  \item{\code{index}}{a numeric vector}
  \item{\code{name.full}}{a character vector of full region names, for the DK
    and DKT atlases}
  \item{\code{class}}{a factor with levels \code{G} \code{G_and_S} \code{S},
    for the Destrieux atlases}
  \item{\code{network}}{(dosenbach160) a factor with levels \code{default}
    \code{fronto-parietal} \code{cingulo-opercular} \code{sensorimotor}
    \code{cerebellum} \code{occipital}}
  \item{\code{gyrus}}{(brainnetome) Abbreviated names of gyri/regions
    (including subcortical), with 24 unique values}
  \item{\code{gyrus.full}}{(brainnetome) Full names of \code{gyrus}}
  \item{\code{subregion}}{(brainnetome) Abbreviated names of subregions
    (including subdivisions of subcortical gray matter)}
  \item{\code{subregion.full}}{(brainnetome) Full names of \code{subregion}}
  \item{\code{Yeo_7network}}{(brainnetome) Factor with 8 levels consisting of
    SCGM plus the 7 networks from Yeo et al.}
  \item{\code{Yeo_17network}}{(brainnetome) Factor with 18 levels consisting
    of SCGM plus the 17 networks from Yeo et al.}
  \item{\code{area}}{(HCP) a factor with 23 cortical areas}
  \item{\code{Anatomy}}{(power264) Full region/gyrus names for the Power
    atlas; contains 53 unique regions}
  \item{\code{Brodmann}}{(power264) Integer values for Brodmann areas}
}

An object of class \code{data.table} (inherits from \code{data.frame}) with 90 rows and 7 columns.

An object of class \code{data.table} (inherits from \code{data.frame}) with 120 rows and 7 columns.

An object of class \code{data.table} (inherits from \code{data.frame}) with 94 rows and 7 columns.

An object of class \code{data.table} (inherits from \code{data.frame}) with 148 rows and 8 columns.

An object of class \code{data.table} (inherits from \code{data.frame}) with 162 rows and 8 columns.

An object of class \code{data.table} (inherits from \code{data.frame}) with 68 rows and 8 columns.

An object of class \code{data.table} (inherits from \code{data.frame}) with 82 rows and 8 columns.

An object of class \code{data.table} (inherits from \code{data.frame}) with 62 rows and 8 columns.

An object of class \code{data.table} (inherits from \code{data.frame}) with 76 rows and 8 columns.

An object of class \code{data.table} (inherits from \code{data.frame}) with 74 rows and 7 columns.

An object of class \code{data.table} (inherits from \code{data.frame}) with 200 rows and 8 columns.

An object of class \code{data.table} (inherits from \code{data.frame}) with 160 rows and 8 columns.

An object of class \code{data.table} (inherits from \code{data.frame}) with 112 rows and 7 columns.

An object of class \code{data.table} (inherits from \code{data.frame}) with 56 rows and 7 columns.

An object of class \code{data.table} (inherits from \code{data.frame}) with 360 rows and 9 columns.

An object of class \code{data.table} (inherits from \code{data.frame}) with 264 rows and 10 columns.

An object of class \code{data.table} (inherits from \code{data.frame}) with 246 rows and 13 columns.

An object of class \code{data.table} (inherits from \code{data.frame}) with 333 rows and 9 columns.
}
\source{
\url{https://neuroimaging-core-docs.readthedocs.io/en/latest/pages/atlases.html}
}
\usage{
aal116

aal90

aal2.120

aal2.94

destrieux

destrieux.scgm

dk

dk.scgm

dkt

dkt.scgm

brainsuite

craddock200

dosenbach160

hoa112

lpba40

hcp_mmp1.0

power264

brainnetome

gordon333
}
\description{
Datasets containing spatial coordinates for: the original AAL atlases, the
newer AAL2 atlases, Freesurfer atlases, Brainsuite, Craddock200,
Dosenbach160, Harvard-Oxford, and LONI probabilistic brain atlas. In addition
to coordinates, there are indices for the major lobes and hemispheres of the
brain, the \emph{class} variable (for Destrieux atlases), functional networks
(for Dosenbach, Power, and Gordon atlases; plus the Yeo network labels for
the Brainnetome atlas).
}
\note{
Use of the HCP parcellation is subject to the terms at
  \url{https://balsa.wustl.edu/WN56}. In particular: "I will acknowledge the
  use of WU-Minn HCP data and data derived from WU-Minn HCP data when
  publicly presenting any results or algorithms that benefitted from their
  use."

Region names in the \code{gordon333} atlas were chosen to match those
  of the \code{hcp_mmp1.0} atlas. Many were determined from the coordinates
  (using FSL's \code{atlasquery}), while the rest were entered manually by
  me. The \code{lobe} values were matched to the HCP atlas, as well.
}
\references{
Tzourio-Mazoyer, N. and Landeau, B. and Papathanassiou, D. and
  Crivello, F. and Etard, O. and Delcroix, N. and Mazoyer, B. and Joliot, M.
  (2002) Automated anatomical labeling of activations in SPM using a
  macroscopic anatomical parcellation of the MNI MRI single-subject brain.
  \emph{NeuroImage}, \bold{15(1)}, 273--289.
  \doi{10.1006/nimg.2001.0978}

Rolls, E.T. and Joliot, M. and Tzourio-Mazoyer, N. (2015)
  Implementation of a new parcellation of the orbitofrontal cortex in the
  automated anatomical labelling atlas. \emph{NeuroImage}, \bold{122}, 1--5.
  \doi{10.1016/j.neuroimage.2015.07.075}

Destrieux, C. and Fischl, B. and Dale, A. and Halgren E. (2010)
  Automatic parcellation of human cortical gyri and sulci using standard
  anatomic nomenclature. \emph{NeuroImage}, \bold{53(1)}, 1--15.
  \doi{10.1016/j.neuroimage.2010.06.010}

Desikan, R.S. and Segonne, F. and Fischl, B. et al. (2006) An
  automated labeling system for subdividing the human cerebral cortex on MRI
  scans into gyral based regions of interest. \emph{NeuroImage}, \bold{31},
  968--980. \doi{10.1016/j.neuroimage.2006.01.021}

Klein, A. and Tourville, J. (2012) 101 labeled brain images
  and a consistent human cortical labeling protocol. \emph{Front Neurosci},
  6. \doi{10.3389/fnins.2012.00171}

Shattuck, D.W. and Leahy, R.M. (2002) BrainSuite: an automated
  cortical surface identification tool. \emph{Medical Image Analysis},
  \bold{8(2)}, 129--142.

Pantazis, D. and Joshi, A.A. and Jintao, J. and Shattuck, D.W.
  and Bernstein, L.E. and Damasio, H. and Leahy, R.M. (2009) Comparison of
  landmark-based and automatic methods for cortical surface registration.
  \emph{NeuroImage}, \bold{49(3)}, 2479--2493.

Craddock, R.C. and James, G.A. and Holtzheimer, P.E. and Hu, X.P.
  and Mayberg, H.S. (2012) A whole brain fMRI atlas generated via spatially
  constrained spectral clustering. \emph{Human Brain Mapping}, \bold{33},
  1914--1928. \doi{10.1002/hbm.21333}

Dosenbach, N.U. and Nardos, B. and Cohen, A.L. and Fair, D.A. and
  Power, J.D. and Church, J.A. and Nelson, S.M. and Wig, G.S. and Vogel, A.C.
  and Lessov-Schlaggar, C.N. and Barnes, K.A. (2010) Prediction of individual
  brain maturity using fMRI. \emph{Science}, \bold{329(5997)}, 1358--1361.

Makris, N. and Goldstein, J.M. and Kennedy, D. et al. (2006)
  Decreased volume of left and total anterior insular lobule in
  schizophrenia. \emph{Schizophr Res}, \bold{83(2-3)}, 155--171.

Shattuck, D.W. and Mirza, M. and Adisetiyo, V. and Hojatkashani,
  C. and Salamon, G. and Narr, K.L. and Poldrack, R.A. and Bilder, R.M. and
  Toga, A.W. (2008) Construction of a 3D probabilistic atlas of human
  cortical structures. \emph{NeuroImage}, \bold{39(3)}, 1064--1080.
  \doi{10.1016/j.neuroimage.2007.09.031}

Glasser, M.F. and Coalson, T.S. and Robinson, E.C. and Hacker,
  C.D. and Harwell, J. and Yacoub, E. and Ugurbil, K. and Andersson, J. and
  Beckmann, C.F. and Jenkinson, M. and Smith, S.M. and van Essen, D.C.
  (2016) A multi-modal parcellation of human cerebral cortex. \emph{Nature},
  \bold{536}, 171--178. \doi{10.1038/nature18933}. PMID:
  27437579.

Power, J.D. and Cohen, A.L. and Nelson, S.M. and Wig, G.S. and
  Barnes, K.A. and Church, J.A. and Vogel, A.C. and Laumann, T.O. and Miezin,
  F.M. and Schlaggar, B.L. and Petersen, S.E. (2011) Functional network
  organization of the human brain. \emph{Neuron}, \bold{72(4)}, 665--678.
  \doi{10.1016/j.neuron.2011.09.006}

Fan, L. and Li, H. and Zhuo, J. and Zhang, Y. and Wang, J. and
  Chen, L. and Yang, Z. and Chu, C. and Xie, S. and Laird, A.R. and Fox,
  P.T. and Eickhoff, S.B. and Yu, C. and Jiang, T (2016) The Human
  Brainnetome Atlas: A New Brain Atlas Based on Connectional Architecture.
  \emph{Cerebral Cortex}, \bold{26(8)}, 3508--3526.
  \doi{10.1093/cercor/bhw157}

Gordon, E.M. and Laumann, T.O. and Adeyemo, B. and Huckins, J.F.
  and Kelley, W.M. and Petersen, S.E. (2014) Generation and Evaluation of a
  Cortical Area Parcellation from Resting-State Correlations. \emph{Cerebral
  Cortex}, \bold{26(1)}, 288--303.
  \doi{10.1093/cercor/bhu239}
}
\keyword{datasets}
